# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import datetime
import time
import re
from report import report_sxw
from itertools import groupby
from operator import itemgetter
from tools.translate import _
#
# Use period and Journal for selection or resources
#
class report_assert_account(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(report_assert_account, self).__init__(cr, uid, name, context=context)
        self.localcontext.update( {
            'time': time,
            'datetime': datetime,
            'execute_code': self.execute_code,
        })

    def execute_code(self, code_exec):
        def group(lst, col):
            return dict((k, [v for v in itr]) for k, itr in groupby(sorted(lst, key=lambda x: x[col]), itemgetter(col)))

        def sort_by_intified_num(a, b):
            if a is None:
                return -1
            elif b is None:
                return 1
            else:
                #if a is not None and b is not None:
                return cmp(int(a), int(b))

        def getdict(item, sequence, num_pat, journal):
            if not item.get('error'):
                match = num_pat.match(str(item['number']))
            else:
                match = None
            if match:
                res = match.groupdict()
                res.update({
                    'complete_num': item['number'],
                    'error': None,
                    'id': item['id'],
                    'state': item['state'],
                    })
                if not 'year'in res:
                    res['year'] = None
            else:
                msg = "Number does not match defined pattern: %s" % (sequence['prefix'], )
                res = {
                    'error': msg,
                    'id': item['id'],
                    'state': item['state'],
                    'complete_num': item['number'],
                    'year': None,
                    'num': None,
                }
            res['journal_code'] = journal['code']
            return res

        def check_numbers(table, list_bad_pattern=False):
            # XXX suffix is not handled. Only prefix is handled up to now

            self.cr.execute("select * from ir_sequence")
            sequences = self.cr.dictfetchall()

            self.cr.execute("select * from account_journal")
            journals = self.cr.dictfetchall()

            result = []

            for journal in journals:
                if table == 'account_invoice':
                    sql = "select distinct type from account_invoice where journal_id = %s"
                    self.cr.execute(sql, [journal['id']])
                    inv_types = [rec[0] for rec in self.cr.fetchall()]
                    for inv_type in inv_types:
                        sql = "select number, id, type, state from account_invoice where journal_id = %s and type = %s and state <> 'draft' order by id"
                        self.cr.execute(sql, [journal['id'], inv_type])
                        items = self.cr.dictfetchall()
                        partial_result = get_partial_result(items, sequences, journal, list_bad_pattern)
                        result.extend(partial_result)
                elif table == 'account_bank_statement':
                    sql = "select name as number, id, state from account_bank_statement where journal_id = %s order by id"
                    self.cr.execute(sql, [journal['id']])
                    items = self.cr.dictfetchall()
                    partial_result = get_partial_result(items, sequences, journal, list_bad_pattern)
                    result.extend(partial_result)
                elif table == 'account_move':
                    sql = "select name as number, id, state from account_move where journal_id = %s order by id"
                    self.cr.execute(sql, [journal['id']])
                    items = self.cr.dictfetchall()
                    partial_result = get_partial_result(items, sequences, journal, list_bad_pattern)
                    result.extend(partial_result)
                else:
                    raise Exception("Checking numbers for table '%s' is not implemented." % (table, ))

            return result

        def get_partial_result(items, sequences, journal, list_bad_pattern):
            if not items:
                return []

            result = []

            sequence = [seq for seq in sequences if seq['id'] == journal['sequence_id']][-1]

            if not sequence['prefix']:
                num_pat_str = r'(?P<num>[\d]*)'
                num_pat = re.compile(num_pat_str)
            elif re.search('%\(year\)s', sequence['prefix']):
                num_pat_str = sequence['prefix'].replace('%(year)s', '20[\d]{2}')
                num_pat = re.compile(num_pat_str)
            elif re.search('%\(y\)s', sequence['prefix']):
                start, end = re.search('%\(y\)s', sequence['prefix']).span()
                num_pat_str = r'%s(?P<year>[\d]{2})%s(?P<num>[\d]*)' % (sequence['prefix'][:start], sequence['prefix'][end:])
                num_pat = re.compile(num_pat_str)
            elif re.search('20\d\d', sequence['prefix']):
                # hardcoded year
                start, end = re.search('20\d\d', sequence['prefix']).span()
                num_pat_str = r'%s(?P<year>20[\d]{2})%s(?P<num>[\d]*)' % (sequence['prefix'][:start], sequence['prefix'][end:])
                num_pat = re.compile(num_pat_str)
            else:
                num_pat_str = r'%s(?P<num>[\d]*)' % (sequence['prefix'], )
                num_pat = re.compile(num_pat_str)

            # update list of dicts:
            item_numbers = [getdict(item, sequence, num_pat, journal) for item in items]

            # ITEMS IN ERRORS:
            wrong_items = [item for item in item_numbers if item['error']]

            if list_bad_pattern:
                for item in wrong_items:
                    result.append({
                        'number': item['complete_num'],
                        'error': item['error'],
                        'id': item['id'],
                        'state': item['state'],
                        'journal_code': item['journal_code'],
                    })
            else:
                msg = {'warning': "Number of items in journal '%s' that were not checked because the sequence pattern does not match the item number: %s" % (journal['code'], len(wrong_items), )}
                result.insert(0, msg)

            unique_years = set([item['year'] for item in item_numbers if item['year']])
            for year in unique_years:
                items_by_year_and_sorted_by_num = sorted(group(item_numbers, 'year')[year], key=itemgetter('num'), cmp=sort_by_intified_num)

                for i, item in enumerate(items_by_year_and_sorted_by_num):
                    if i == 0:
                        continue # no need to check first item
                    previous_item = items_by_year_and_sorted_by_num[i-1]

                    if int(item['num']) != int(previous_item['num'])+1:
                        result.append({
                            'wrong number': item['num'],
                            'previous number': items_by_year_and_sorted_by_num[i-1]['num'],
                            'id': item['id'],
                            'previous id': items_by_year_and_sorted_by_num[i-1]['id'],
                            'journal_code': item['journal_code'],
                        })

            return result

        def get_parent(acc_id):
            acc_an_id = self.pool.get('account.analytic.account').browse(self.cr, self.uid, acc_id).parent_id
            while acc_an_id.parent_id:
                acc_an_id = acc_an_id.parent_id
            return acc_an_id.id

        def reconciled_inv():
            reconciled_inv_ids = self.pool.get('account.invoice').search(self.cr, self.uid, [('reconciled','=',True)])
            return reconciled_inv_ids

        def order_columns(item, cols=None):
            if cols is None:
                cols = item.keys()
            return [(col, item.get(col)) for col in cols if col in item.keys()]

        localdict = {
            'cr': self.cr,
            '_': _,
            'group' : group,
            'reconciled_inv' : reconciled_inv,
            'get_parent': get_parent,
            'check_numbers': check_numbers,
            'now': datetime.datetime.now(),
            'result': None,
            'column_order': None,
        }

        exec code_exec in localdict

        result = localdict['result']
        column_order = localdict.get('column_order', None)

        if not isinstance(result, (tuple, list, set)):
            result = [result]

        if not result:
            result = [_('The test was passed successfully')]
        elif all([isinstance(x, dict) for x in result]):
            result = [', '.join(["%s: %s" % (k, v) for k, v in order_columns(rec, column_order)]) for rec in result]
        else:
            def _format(a):
                if isinstance(a, dict):
                    return ', '.join(["%s: %s" % (tup[0], tup[1]) for tup in order_columns(a, column_order)])
                else:
                    return a
            result = [_format(rec) for rec in result]

        return result

report_sxw.report_sxw('report.account.test.assert.cci', 'accounting.assert.test', 'addons/account_test/report/account_test.rml', parser=report_assert_account, header=False)

