# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from osv.orm import except_orm
import netsvc
import time
import pickle
from tools.translate import _
from tools import config


class account_move(osv.osv):
    _inherit = "account.move"
    _description = "Account Entry"

    def write(self, cr, uid, ids, vals, context=None):
        line_ids = []
        if vals.get('period_id', False):
            lines = self.browse(cr, uid, ids, context=context)[0].line_id
            map(lambda x: line_ids.append(x.id),lines)
            self.pool.get('account.move.line').write(cr, uid, line_ids, {'period_id': vals['period_id']}, context=context)
        result = super(osv.osv, self).write(cr, uid, ids, vals, context)
        return result

    #
    # Validate a balanced move. If it is a centralised journal, create a move.
    #
    def validate(self, cr, uid, ids, context={}):
        if context and ('__last_update' in context):
            del context['__last_update']

        valid_moves = [] #Maintains a list of moves which can be responsible to create analytic entries

        for move in self.browse(cr, uid, ids, context):
            # Unlink old analytic lines on move_lines
            for obj_line in move.line_id:
                for obj in obj_line.analytic_lines:
                    self.pool.get('account.analytic.line').unlink(cr,uid,obj.id)

            journal = move.journal_id

            cr.execute('select sum(debit - credit) from account_move_line where move_id = %s', (move.id,))
            amount = cr.fetchone()[0] or 0.0
            line_ids = [l.id for l in move.line_id]
            #TODO: bellow code is commented and replaced by the 3 lines above, to speed up the process of validating.
            #     Hence, we didn't put back the checks on comapny_id and curerncy_id. 
            #     I guess we should put that in psql_contraint, instead.
            #for line in move.line_id:
                #amount += line.debit - line.credit
                #line_ids.append(line.id)
                #if line.state == 'draft':
                #    line_draft_ids.append(line.id)

                #if not company_id:
                #    company_id = line.account_id.company_id.id
                #if not company_id == line.account_id.company_id.id:
                #    raise osv.except_osv(_('Error'), _("Couldn't create move between different companies"))

                #if line.account_id.currency_id:
                #    if line.account_id.currency_id.id != line.currency_id.id and (line.account_id.currency_id.id != line.account_id.company_id.currency_id.id or line.currency_id):
                #        raise osv.except_osv(_('Error'), _("Couldn't create move with currency different from the secondary currency of the account '%s - %s'. Clear the secondary currency field of the account definition if you want to accept all currencies.") % (line.account_id.code, line.account_id.name))

            # Check that the move balances, the tolerance for debit/credit must
            # be smaller than the smallest value according to price accuracy
            # (hence the +1 below)
            # Example:
            #    difference == 0.01 is OK iff price_accuracy <= 1!
            #    difference == 0.0001 is OK iff price_accuracy <= 3!
            if abs(amount) < 10 ** -(int(config['price_accuracy'])+1):
                # If the move is balanced
                # Add to the list of valid moves
                # (analytic lines will be created later for valid moves)
                valid_moves.append(move)

                # Check whether the move lines are confirmed
                #if not len(line_draft_ids):
                #    continue

                # Update the move lines (set them as valid)
                self.pool.get('account.move.line').write(cr, uid, line_ids, {
                    'journal_id': move.journal_id.id,
                    'period_id': move.period_id.id,
                    'state': 'valid'
                }, context, check=False, update_check=False)

                account = {}
                account2 = {}

                if journal.type in ('purchase','sale'):

                    for line in move.line_id:
                        code = amount = 0
                        key = (line.account_id.id, line.tax_code_id.id)
                        if key in account2:
                            code = account2[key][0]
                            amount = account2[key][1] * (line.debit + line.credit)
                        elif line.account_id.id in account:
                            code = account[line.account_id.id][0]
                            amount = account[line.account_id.id][1] * (line.debit + line.credit)
                        if (code or amount) and not (line.tax_code_id or line.tax_amount):
                            self.pool.get('account.move.line').write(cr, uid, [line.id], {
                                'tax_code_id': code,
                                'tax_amount': amount
                            }, context, check=False)

            elif journal.centralisation:
                # If the move is not balanced, it must be centralised...

                # Add to the list of valid moves
                # (analytic lines will be created later for valid moves)
                #we don't need analytic on centralized move, do we? it would be better to remove above statement
                #valid_moves.append(move)

                # Update the move lines (set them as valid)
                self._centralise(cr, uid, move, 'debit', context=context)
                self._centralise(cr, uid, move, 'credit', context=context)
                self.pool.get('account.move.line').write(cr, uid, line_ids, {
                    'state': 'valid'
                }, context, check=False)
            else:
                # We can't validate it (it's unbalanced)
                # Setting the lines as draft
                self.pool.get('account.move.line').write(cr, uid, line_ids, {
                    'journal_id': move.journal_id.id,
                    'period_id': move.period_id.id,
                    'state': 'draft'
                }, context, check=False)

        # Create analytic lines for the valid moves
        for record in valid_moves:
            self.pool.get('account.move.line').create_analytic_lines(cr, uid, line_ids, context)
        return len(valid_moves) > 0

account_move()


class account_invoice_action_methods_inherit(osv.osv):
    _inherit = "account.invoice"

#    def write(self, cr, uid, ids, vals, context=None):
#        for inv in self.browse(cr, uid, ids, context=context):
#            if inv.tax_line and inv.type in ('in_invoice', 'in_refund') and abs(inv.check_total - inv.amount_total) >= (inv.currency_id.rounding/2.0):
#                raise osv.except_osv(_('Bad total !'), _('Please verify the price of the invoice !\nThe real total does not match the computed total.'))
#        result = super(osv.osv, self).write(cr, uid, ids, vals, context)
#        return result

    def name_get(self, cr, uid, ids, context=None):
        if not len(ids):
            return []
        types = {
                'out_invoice': 'CI: ',
                'in_invoice': 'SI: ',
                'out_refund': 'OR: ',
                'in_refund': 'SR: ',
                }

        if ids and isinstance(ids[0], tuple):
            ids = [t[0] for t in ids]

        return [(r['id'], types[r['type']]+(r['number'] or '')+' '+(r['name'] or '')) for r in self.read(cr, uid, ids, ['type', 'number', 'name'], context, load='_classic_write')]

    def action_move_create(self, cr, uid, ids, *args):
        ait_obj = self.pool.get('account.invoice.tax')
        cur_obj = self.pool.get('res.currency')
        context = {}
        for inv in self.browse(cr, uid, ids):
            tax_cal = 0
            if inv.move_id:
                continue

            if not inv.date_invoice:
                self.write(cr, uid, [inv.id], {'date_invoice':time.strftime('%Y-%m-%d')})
            company_currency = inv.company_id.currency_id.id
            # create the analytical lines
            line_ids = self.read(cr, uid, [inv.id], ['invoice_line'])[0]['invoice_line']
            # one move line per invoice line
            iml = self._get_analytic_lines(cr, uid, inv.id)
            # check if taxes are all computed
            ctx = context.copy()
            ctx.update({'lang': inv.partner_id.lang})
            compute_taxes = ait_obj.compute(cr, uid, inv.id, context=ctx)
            if not inv.tax_line:
                for tax in compute_taxes.values():
                    tax_cal += tax['tax_amount']
                    ait_obj.create(cr, uid, tax)
            else:
                tax_key = []
                for tax in inv.tax_line:
                    if tax.manual:
                        continue
                    key = (tax.tax_code_id.id, tax.base_code_id.id, tax.account_id.id)
                    tax_key.append(key)
                    if not key in compute_taxes:
                        raise osv.except_osv(_('Warning !'), _('Global taxes defined, but are not in invoice lines !'))
                    base = compute_taxes[key]['base']
                    if abs(base - tax.base) > inv.company_id.currency_id.rounding:
                        raise osv.except_osv(_('Warning !'), _('Tax base different !\nClick on compute to update tax base'))
                for key in compute_taxes:
                    if not key in tax_key:
                        raise osv.except_osv(_('Warning !'), _('Taxes missing !'))
            if inv.type in ('in_invoice', 'in_refund') and abs(inv.check_total - (inv.amount_total + tax_cal)) <> 0.0:
                raise osv.except_osv(_('Bad total !'), _('Please verify the price of the invoice !\nThe real total does not match the computed total. \n You maybe need to press the calculate taxes button.'))

            # one move line per tax line
            iml += ait_obj.move_line_get(cr, uid, inv.id)

            if inv.type in ('in_invoice', 'in_refund'):
                ref = inv.reference
            else:
                ref = self._convert_ref(cr, uid, inv.number)

            diff_currency_p = inv.currency_id.id <> company_currency
            # create one move line for the total and possibly adjust the other lines amount
            total = 0
            total_currency = 0
            for i in iml:
                if inv.currency_id.id != company_currency:
                    i['currency_id'] = inv.currency_id.id
                    i['amount_currency'] = i['price']
                    i['price'] = cur_obj.compute(cr, uid, inv.currency_id.id,
                            company_currency, i['price'],
                            context={'date': inv.date_invoice or time.strftime('%Y-%m-%d')})
                else:
                    i['amount_currency'] = False
                    i['currency_id'] = False
                i['ref'] = ref
                if inv.type in ('out_invoice','in_refund'):
                    total += i['price']
                    total_currency += i['amount_currency'] or i['price']
                    i['price'] = - i['price']
                else:
                    total -= i['price']
                    total_currency -= i['amount_currency'] or i['price']
            acc_id = inv.account_id.id

            name = inv['name'] or '/'
            totlines = False
            if inv.payment_term:
                totlines = self.pool.get('account.payment.term').compute(cr,
                        uid, inv.payment_term.id, total, inv.date_invoice or False)
            if totlines:
                res_amount_currency = total_currency
                i = 0
                for t in totlines:
                    if inv.currency_id.id != company_currency:
                        amount_currency = cur_obj.compute(cr, uid,
                                company_currency, inv.currency_id.id, t[1])
                    else:
                        amount_currency = False

                    # last line add the diff
                    res_amount_currency -= amount_currency or 0
                    i += 1
                    if i == len(totlines):
                        amount_currency += res_amount_currency

                    iml.append({
                        'type': 'dest',
                        'name': name,
                        'price': t[1],
                        'account_id': acc_id,
                        'date_maturity': t[0],
                        'amount_currency': diff_currency_p \
                                and  amount_currency or False,
                        'currency_id': diff_currency_p \
                                and inv.currency_id.id or False,
                        'ref': ref,
                    })
            else:
                iml.append({
                    'type': 'dest',
                    'name': name,
                    'price': total,
                    'account_id': acc_id,
                    'date_maturity' : inv.date_due or False,
                    'amount_currency': diff_currency_p \
                            and total_currency or False,
                    'currency_id': diff_currency_p \
                            and inv.currency_id.id or False,
                    'ref': ref
            })

            date = inv.date_invoice or time.strftime('%Y-%m-%d')
            part = inv.partner_id.id

            line = map(lambda x:(0,0,self.line_get_convert(cr, uid, x, part, date, context={})) ,iml)

            if inv.journal_id.group_invoice_lines:
                line2 = {}
                for x, y, l in line:
                    tmp = str(l['account_id'])
                    tmp += '-'+str(l.get('tax_code_id',"False"))
                    tmp += '-'+str(l.get('product_id',"False"))
                    tmp += '-'+str(l.get('analytic_account_id',"False"))
                    tmp += '-'+str(l.get('date_maturity',"False"))
                    
                    if tmp in line2:
                        am = line2[tmp]['debit'] - line2[tmp]['credit'] + (l['debit'] - l['credit'])
                        line2[tmp]['debit'] = (am > 0) and am or 0.0
                        line2[tmp]['credit'] = (am < 0) and -am or 0.0
                        line2[tmp]['tax_amount'] += l['tax_amount']
                        line2[tmp]['analytic_lines'] += l['analytic_lines']
                    else:
                        line2[tmp] = l
                line = []
                for key, val in line2.items():
                    line.append((0,0,val))

            journal_id = inv.journal_id.id #self._get_journal(cr, uid, {'type': inv['type']})
            journal = self.pool.get('account.journal').browse(cr, uid, journal_id)
            if journal.centralisation:
                raise osv.except_osv(_('UserError'),
                        _('Cannot create invoice move on centralised journal'))

            line = self.finalize_invoice_move_lines(cr, uid, inv, line)

            move = {'ref': inv.number, 'line_id': line, 'journal_id': journal_id, 'date': date}
            period_id=inv.period_id and inv.period_id.id or False
            if not period_id:
                period_ids= self.pool.get('account.period').search(cr,uid,[('date_start','<=',inv.date_invoice or time.strftime('%Y-%m-%d')),('date_stop','>=',inv.date_invoice or time.strftime('%Y-%m-%d'))])
                if len(period_ids):
                    period_id=period_ids[0]
            if period_id:
                move['period_id'] = period_id
                for i in line:
                    i[2]['period_id'] = period_id

            move_id = self.pool.get('account.move').create(cr, uid, move, context = {'invoice':inv})
            # make the invoice point to that move
            self.write(cr, uid, [inv.id], {'move_id': move_id,'period_id':period_id})
            # Pass invoice in context in method post: used if you want to get the same
            # account move reference when creating the same invoice after a cancelled one:
            self.pool.get('account.move').post(cr, uid, [move_id], context={'invoice':inv})
        self._log_event(cr, uid, ids)
        return True

    def action_number(self, cr, uid, ids, *args):
        cr.execute('SELECT id, type, number, move_id, reference ' \
                   'FROM account_invoice ' \
                   'WHERE id IN %s',
                   (tuple(ids),))
        obj_inv = self.browse(cr, uid, ids)[0]
        for (id, invtype, number, move_id, reference) in cr.fetchall():
            if not number:
                if obj_inv.journal_id.invoice_sequence_id:
                    sid = obj_inv.journal_id.invoice_sequence_id.id
                    number = self.pool.get('ir.sequence').get_id(cr, uid, sid, 'id=%s', {'fiscalyear_id': obj_inv.period_id.fiscalyear_id.id})
                else:
                    number = self.pool.get('ir.sequence').get(cr, uid,
                            'account.invoice.' + invtype)
                if invtype in ('in_invoice', 'in_refund'):
                    ref = reference
                else:
                    ref = self._convert_ref(cr, uid, number)
                cr.execute('UPDATE account_invoice SET number=%s ' \
                        'WHERE id=%s', (number, id))
                cr.execute('UPDATE account_move SET ref=%s ' \
                        'WHERE id=%s AND (ref is null OR ref = \'\')',
                        (ref, move_id))
                cr.execute('UPDATE account_move_line SET ref=%s ' \
                        'WHERE move_id=%s AND (ref is null OR ref = \'\')',
                        (ref, move_id))
                cr.execute('UPDATE account_analytic_line SET ref=%s ' \
                        'FROM account_move_line ' \
                        'WHERE account_move_line.move_id = %s ' \
                            'AND account_analytic_line.move_id = account_move_line.id',
                            (ref, move_id))
                cr.execute('UPDATE account_move SET name=%s ' \
                        'WHERE id=%s ',
                        (number, move_id))
        return True

    def create(self, cr, uid, vals, context={}):
        try:
            res = super(account_invoice_action_methods_inherit, self).create(cr, uid, vals, context)
            return res
        except Exception,e:
            if not isinstance(e, KeyError) and '"journal_id" viol' in e.args[0]:
                raise except_orm(_('Configuration Error!'),
                     _('There is no Accounting Journal of type Sale/Purchase defined!'))
            else:
                raise except_orm(_('UnknownError'), str(e))

account_invoice_action_methods_inherit()


class account_invoice_onchange_partner_id_inherit(osv.osv):
    _inherit = "account.invoice"

    def onchange_partner_id(self, cr, uid, ids, type, partner_id,
            date_invoice=False, payment_term=False, partner_bank=False):
        invoice_addr_id = False
        contact_addr_id = False
        partner_payment_term = False
        acc_id = False
        bank_id = False
        fiscal_position = False

        opt = [('uid', str(uid))]
        p = False
        if partner_id:

            opt.insert(0, ('id', partner_id))
            res = self.pool.get('res.partner').address_get(cr, uid, [partner_id], ['contact', 'invoice'])
            contact_addr_id = res['contact']
            invoice_addr_id = res['invoice']
            p = self.pool.get('res.partner').browse(cr, uid, partner_id)
            if type in ('out_invoice', 'out_refund'):
                acc_id = p.property_account_receivable.id
            else:
                acc_id = p.property_account_payable.id
            fiscal_position = p.property_account_position and p.property_account_position.id or False
            partner_payment_term = p.property_payment_term and p.property_payment_term.id or False
            if p.bank_ids:
                bank_id = p.bank_ids[0].id

        result = {'value': {
            'address_contact_id': contact_addr_id,
            'address_invoice_id': invoice_addr_id,
            'account_id': acc_id,
            'payment_term': partner_payment_term,
            'vat_num': p and p.vat or None,
            'fiscal_position': fiscal_position
            }
        }

        if type in ('in_invoice', 'in_refund'):
            result['value']['partner_bank'] = bank_id

        if partner_bank != bank_id:
            to_update = self.onchange_partner_bank(cr, uid, ids, bank_id)
            result['value'].update(to_update['value'])
        return result

account_invoice_onchange_partner_id_inherit()


class account_invoice(osv.osv):
    _inherit = "account.invoice"
    _order = "date_invoice desc"

    def _onchange_user_id(self, cr, uid, ids, user_id, status):
        result = {'value': {'user_id': user_id}}
        data_pool = self.pool.get('ir.model.data')
        user_obj =  self.pool.get('res.users')
        user_group_ids = user_obj.browse(cr, uid, uid).groups_id
        group_ch_user_id = data_pool._get_id(cr, uid, 'cci_account', 'group_inv_change_user')
        if group_ch_user_id:
            group_id = data_pool.browse(cr, uid, group_ch_user_id).res_id
        groups_user = [x.id for x in user_group_ids] 
        if (status not in ('open')) or ((status == 'open') and (group_id in groups_user)):
            result = {'value': {'user_id': user_id}}
        else:
            raise osv.except_osv('Error!','You don\'t have enough access to change the user on confirmed invoice.')
        return result

    _columns = {
        'name': fields.char('Description', size=64, select=True),
        'ref_move': fields.char('Ref Move', size=64, select=True),
        'dept':fields.many2one('hr.department','Department'),
        'invoice_special':fields.boolean('Special Invoice'),
        'internal_note': fields.text('Internal Note'),
        'vat_num' : fields.related('partner_id', 'vat',  type='char', string="VAT"),
        'create_uid': fields.many2one('res.users', 'Creation User', readonly=True),
        'user_id': fields.many2one('res.users', 'User'),
        'date_due': fields.date('Due Date', states={'close':[('readonly',True)]},
            help="If you use payment terms, the due date will be computed automatically at the generation "\
                "of accounting entries. If you keep the payment term and the due date empty, it means direct payment. The payment term may compute several due dates, for example 50% now, 50% in one month."),
        'type': fields.selection([
            ('out_invoice','Customer Invoice'),
            ('in_invoice','Supplier Invoice'),
            ('out_refund','Customer Refund'),
            ('in_refund','Supplier Refund'),
            ],'Type', readonly=True, select=True, change_default=True),
    }

    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}

        data_pool = self.pool.get('ir.model.data')
        user_obj =  self.pool.get('res.users')
        if context.get('type') == 'in_invoice' and not context.get('from_purchase'):
            user_group_ids = user_obj.browse(cr, uid, uid, context).groups_id
            group_inv_id = data_pool._get_id(cr, uid, 'cci_account','group_invoice_supplier')
            if group_inv_id:
                group_id = data_pool.browse(cr, uid, group_inv_id, context=context).res_id
                if group_id not in [x.id for x in user_group_ids]:
                    raise osv.except_osv('Error!','You don\'t have enough access to create a supplier invoice.')
        if vals.has_key('partner_id') and vals['partner_id']:
            partner = self.pool.get('res.partner').browse(cr, uid, vals['partner_id'], context=context)
            vals.update({'invoice_special':partner.invoice_special,'domiciled':partner.domiciliation})
            values_obj = self.pool.get('ir.values')
            term_id = []
            term_ids = values_obj.search(cr, uid, [('key','=','default'),('name','=','payment_term'),('model','=','account.invoice'),('key2','=',vals['partner_id'])])
            if not term_ids:
                term_ids = self.pool.get('ir.values').search(cr, uid, [('key','=','default'),('name','=','payment_term'),('model','=','account.invoice')])
            if term_ids:
                vals.update({'payment_term':pickle.loads(str(values_obj.browse(cr, uid, term_ids)[0].value))})

        return super(account_invoice, self).create(cr, uid, vals, context=context)

    def action_move_create(self, cr, uid, ids, context=None):
        flag = membership_flag = False
        product_ids = []
        move_obj = self.pool.get('account.move')
        move_line_obj = self.pool.get('account.move.line')
        data_invoice = self.browse(cr,uid,ids[0])
        #raise an error if one of the account_invoice_line doesn't have an analytic entry
        for line in data_invoice.invoice_line:
            if not line.analytics_id:
                flag = True
        if flag:
            raise osv.except_osv('Error!','Invoice line should have Analytic Distribution to create Analytic Entries.')
        super(account_invoice, self).action_move_create(cr, uid, ids, context)
        for invoice_rec in self.browse(cr, uid, ids): 
            if invoice_rec.move_id and not invoice_rec.ref_move:
                self.write(cr, uid, [invoice_rec.id], {'ref_move':invoice_rec.move_id.name})
        for lines in data_invoice.abstract_line_ids:
            if lines.product_id:
                product_ids.append(lines.product_id.id)
        if product_ids:
            data_product = self.pool.get('product.product').browse(cr,uid,product_ids)
            for product in data_product:
                if product.membership:
                    membership_flag = True
        if data_invoice.partner_id.alert_membership and membership_flag:
            raise osv.except_osv('Error!',data_invoice.partner_id.alert_explanation or 'Partner is not valid')
        #create other move lines if the invoice_line is related to a check payment or an AWEX credence
        for inv in self.browse(cr, uid, ids):
            for item in self.pool.get('account.invoice.line').search(cr, uid, [('invoice_id','=',inv.id)]):
                line = self.pool.get('account.invoice.line').browse(cr,uid, [item])[0]
                if line.cci_special_reference:
                    iml = []
                    if inv.type in ('in_invoice', 'in_refund'):
                        ref = inv.reference
                    else:
                        ref = self._convert_ref(cr, uid, inv.number)
                    temp = line.cci_special_reference.split('*')
                    obj = temp[0]
                    obj_id = int(temp[1])
                    obj_ref = self.pool.get(obj).browse(cr, uid, [obj_id])[0]
                    linename = False
                    if obj == "event.registration":
                        #acc_id = self.pool.get('account.account').search(cr, uid, [('name','=','Creances AWEX - Cheques Formations et Cheques Langues')])[0]
                        journal_id = self.pool.get('account.journal').search(cr, uid, [('name','=','CFL Journal')])[0]
                        amount = obj_ref.check_amount
                        linename = obj_ref.invoice_label
                    else:
                        journal_id = self.pool.get('account.journal').search(cr, uid, [('name','=','AWEX Journal')])[0]
                        #acc_id = self.pool.get('account.account').search(cr, uid, [('name','=','Creances AWEX - Cheques Formations et Cheques Langues')])[0]
                        amount = obj_ref.awex_amount
                        linename = obj_ref.order_desc
                    acc_id = self.pool.get('account.journal').browse(cr, uid, [journal_id])[0].default_debit_account_id.id
                    iml.append({
                        'type': 'dest',
                        'name': linename or '/',
                        'price': amount,
                        'account_id': acc_id,
                        'date_maturity': inv.date_due or False,
                        'amount_currency': False,
                        'currency_id': inv.currency_id.id or False,
                        'ref': ref,
                    })
                    iml.append({
                        'type': 'dest',
                        'name': linename or '/',
                        'price': -(amount),
                        'account_id': inv.account_id.id,
                        'date_maturity': inv.date_due or False,
                        'amount_currency': False,
                        'currency_id': inv.currency_id.id or False,
                        'ref': ref,
                    })
                    date = inv.date_invoice
                    part = inv.partner_id.id
                    new_lines = map(lambda x:(0,0,self.line_get_convert(cr, uid, x, part, date, context={})) ,iml)
                    for item in new_lines:
                        if item[2]['credit']:
                            id1 = item[2]['credit']

                    journal = self.pool.get('account.journal').browse(cr, uid, journal_id)
                    if inv.move_id:
                        name = inv.move_id.name
                    elif journal.sequence_id:
                        name = self.pool.get('ir.sequence').get_id(cr, uid, journal.sequence_id.id)
                    move = {'name': name, 'line_id': new_lines, 'journal_id': journal_id}
                    self.write(cr, uid, [inv.id], {'ref_move': inv.move_id.name})
                    if inv.period_id:
                        move['period_id'] = inv.period_id.id
                        #for i in line:
                        #    i[2]['period_id'] = inv.period_id.id
                    move_id = move_obj.create(cr, uid, move)
                    move_obj.post(cr, uid, [move_id])

                #this function could be improved in order to enable having more than one translation line per invoice
                    id1 = move_line_obj.search(cr, uid, [('move_id','=',move_id),('credit','<>',False)])[0]
                    id2 = move_line_obj.search(cr, uid, [('invoice','=',inv.id),('debit','<>',0)])[0]
                    move_line_obj.reconcile_partial(cr, uid, [id2,id1], 'manual', context=context)

        return True

    def action_number(self, cr, uid, ids, *args):
        res = super(account_invoice,self).action_number(cr, uid, ids, args)
        for inv in self.browse(cr, uid, ids):
            vcs =''
            if inv.number and not inv.name:
                vcs3 = inv.number.split('/')[1]
                vcs1 = '0'+ str(inv.date_invoice[2:4])
                if len(vcs3) >= 6:
                    vcs2= vcs3[0:6]
                else:
                    vcs2 = vcs3.rjust(6,'0')

                vcs4= vcs1 + vcs2 + '0'

                vcs5 = int(vcs4)
                check_digit = vcs5 % 97

                if check_digit == 0:
                    check_digit = '97'
                if check_digit <= 9:
                    check_digit = '0' + str(check_digit)
                vcs = vcs1 + '/' + vcs2[0:4] + '/' + vcs2[4:6] + '0' + str(check_digit)

                self.write(cr, uid, [inv.id], {'name':vcs})
                self.pool.get('account.move').write(cr, uid, [inv.move_id.id], {'name' : inv.number})

                # move line is already posted, so we can only update it in sql:
                ids = self.pool.get('account.move.line').search(cr, uid, [('move_id', '=', inv.move_id.id)])
                if ids:
                    cr.execute("update account_move_line set name = %s where id in %s", [vcs, tuple(ids)])
        return res

    #raise an error if the partner has the warning 'alert_others' when we choose him in the account_invoice form
    def onchange_partner_id(self, cr, uid, ids, type, partner_id, date_invoice=False, payment_term=False, partner_bank=False):
        warning = False
        inv_special=False
        domiciled = False
        user_id = False
        if partner_id:
            data_partner = self.pool.get('res.partner').browse(cr,uid,partner_id)
            domiciled = bool(data_partner.domiciliation)
            user_id = data_partner.user_id.id
            inv_special=data_partner.invoice_special
            if data_partner.alert_others:
                warning = {
                    'title': "Warning:",
                    'message': data_partner.alert_explanation or 'Partner is not valid'
                        }
        data=super(account_invoice,self).onchange_partner_id( cr, uid, ids, type, partner_id, date_invoice, payment_term, partner_bank)
        data['value']['domiciled'] = domiciled
        data['value']['user_id'] = user_id
        data['value']['invoice_special']=inv_special
        if warning:
            data['warning'] = warning
        return data

account_invoice()


class sale_order(osv.osv):
    _inherit = "sale.order"
    _columns = {
        'dept' :  fields.many2one('hr.department','Department'),
    }

sale_order()


class account_invoice_line(osv.osv):
    _inherit = "account.invoice.line"
    _columns = {
        'cci_special_reference' : fields.char('Special Reference', size=64),
        'sequence': fields.integer('Sequence'),
    }
    _defaults = {
        'cci_special_reference': lambda *a : False,
    }
account_invoice_line()


class account_bank_statement_button_confirm(osv.osv):
    _inherit = 'account.bank.statement'

    def button_confirm(self, cr, uid, ids, context={}):
        done = []
        res_currency_obj = self.pool.get('res.currency')
        res_users_obj = self.pool.get('res.users')
        account_move_obj = self.pool.get('account.move')
        account_move_line_obj = self.pool.get('account.move.line')
        account_bank_statement_line_obj = self.pool.get('account.bank.statement.line')

        company_currency_id = res_users_obj.browse(cr, uid, uid,
                context=context).company_id.currency_id.id

        for st in self.browse(cr, uid, ids, context):
            if not st.state=='draft':
                continue

            if not (abs(st.balance_end - st.balance_end_real) < 0.0001):
                raise osv.except_osv(_('Error !'),
                        _('The statement balance is incorrect !\n') +
                        _('The expected balance (%.2f) is different than the computed one. (%.2f)') % (st.balance_end_real, st.balance_end))
            if (not st.journal_id.default_credit_account_id) \
                    or (not st.journal_id.default_debit_account_id):
                raise osv.except_osv(_('Configuration Error !'),
                        _('Please verify that an account is defined in the journal.'))

            for line in st.move_line_ids:
                if line.state <> 'valid':
                    raise osv.except_osv(_('Error !'),
                            _('The account entries lines are not in valid state.'))
            # for bank.statement.lines
            # In line we get reconcile_id on bank.ste.rec.
            # in bank stat.rec we get line_new_ids on bank.stat.rec.line
            for move in st.line_ids:
                context.update({'date':move.date})
                move_id = account_move_obj.create(cr, uid, {
                    'journal_id': st.journal_id.id,
                    'name':st.name,
                    'period_id': st.period_id.id,
                    'date': move.date,
                }, context=context)
                account_bank_statement_line_obj.write(cr, uid, [move.id], {
                    'move_ids': [(4,move_id, False)]
                })
                if not move.amount:
                    continue

                torec = []
                if move.amount >= 0:
                    account_id = st.journal_id.default_credit_account_id.id
                else:
                    account_id = st.journal_id.default_debit_account_id.id
                acc_cur = ((move.amount<=0) and st.journal_id.default_debit_account_id) or move.account_id
                amount = res_currency_obj.compute(cr, uid, st.currency.id,
                        company_currency_id, move.amount, context=context,
                        account=acc_cur)
                if move.reconcile_id and move.reconcile_id.line_new_ids:
                    for newline in move.reconcile_id.line_new_ids:
                        amount += newline.amount

                val = {
                    'name': move.name,
                    'date': move.date,
                    'ref': move.ref,
                    'move_id': move_id,
                    'partner_id': ((move.partner_id) and move.partner_id.id) or False,
                    'account_id': (move.account_id) and move.account_id.id,
                    'credit': ((amount>0) and amount) or 0.0,
                    'debit': ((amount<0) and -amount) or 0.0,
                    'statement_id': st.id,
                    'journal_id': st.journal_id.id,
                    'period_id': st.period_id.id,
                    'currency_id': st.currency.id,
                }

                amount = res_currency_obj.compute(cr, uid, st.currency.id,
                        company_currency_id, move.amount, context=context,
                        account=acc_cur)
                if st.currency.id <> company_currency_id:
                    amount_cur = res_currency_obj.compute(cr, uid, company_currency_id,
                                st.currency.id, amount, context=context,
                                account=acc_cur)
                    val['amount_currency'] = -amount_cur

                if move.account_id and move.account_id.currency_id and move.account_id.currency_id.id <> company_currency_id:
                    val['currency_id'] = move.account_id.currency_id.id
                    if company_currency_id==move.account_id.currency_id.id:
                        amount_cur = move.amount
                    else:
                        amount_cur = res_currency_obj.compute(cr, uid, company_currency_id,
                                move.account_id.currency_id.id, amount, context=context,
                                account=acc_cur)
                    val['amount_currency'] = amount_cur

                torec.append(account_move_line_obj.create(cr, uid, val , context=context))

                if move.reconcile_id and move.reconcile_id.line_new_ids:
                    for newline in move.reconcile_id.line_new_ids:
                        account_move_line_obj.create(cr, uid, {
                            'name': newline.name or move.name,
                            'date': move.date,
                            'ref': move.ref,
                            'move_id': move_id,
                            'partner_id': ((move.partner_id) and move.partner_id.id) or False,
                            'account_id': (newline.account_id) and newline.account_id.id,
                            'debit': newline.amount>0 and newline.amount or 0.0,
                            'credit': newline.amount<0 and -newline.amount or 0.0,
                            'statement_id': st.id,
                            'journal_id': st.journal_id.id,
                            'period_id': st.period_id.id,

                        }, context=context)

                # Fill the secondary amount/currency
                # if currency is not the same than the company
                amount_currency = False
                currency_id = False
                if st.currency.id <> company_currency_id:
                    amount_currency = move.amount
                    currency_id = st.currency.id
                account_move_line_obj.create(cr, uid, {
                    'name': move.name,
                    'date': move.date,
                    'ref': move.ref,
                    'move_id': move_id,
                    'partner_id': ((move.partner_id) and move.partner_id.id) or False,
                    'account_id': account_id,
                    'credit': ((amount < 0) and -amount) or 0.0,
                    'debit': ((amount > 0) and amount) or 0.0,
                    'statement_id': st.id,
                    'journal_id': st.journal_id.id,
                    'period_id': st.period_id.id,
                    'amount_currency': amount_currency,
                    'currency_id': currency_id,
                }, context=context)

                lst = []
                lst_move = []
                # re-read browse record after changes:
                move = account_bank_statement_line_obj.browse(cr, uid, move.id)
                for move_l in move.statement_id.move_line_ids:
                    if move_l.account_id in [item.account_id for item in move.statement_id.line_ids if item.analytics_id] and move.analytics_id and move.id not in lst_move:
                        lst_move.append(move.id)
                        account_move_line_obj.write(cr, uid, [move_l.id], {'analytics_id': move.analytics_id.id, 'quantity': move.amount or 1})
                        lst.append(move_l.id)
                if lst:
                    try:
                        account_move_line_obj.create_analytic_lines(cr, uid, lst, context)
                    except Exception, e:
                        raise osv.except_osv('Error !', 'Cannot create all analytic lines: "%s".' % (e, ))

                for line in account_move_line_obj.browse(cr, uid, [x.id for x in
                        account_move_obj.browse(cr, uid, move_id,
                            context=context).line_id],
                        context=context):
                    if line.state <> 'valid':
                        raise osv.except_osv(_('Error !'),
                                _('Account move line "%s" is not valid') % line.name)

                if move.reconcile_id and move.reconcile_id.line_ids:
                    ## Search if move has already a partial reconciliation
                    previous_partial = False
                    for line_reconcile_move in move.reconcile_id.line_ids:
                        if line_reconcile_move.reconcile_partial_id:
                            previous_partial = True
                            break
                    ##
                    torec += map(lambda x: x.id, move.reconcile_id.line_ids)
                    #try:
                    if abs(move.reconcile_amount-move.amount)<0.0001:

                        writeoff_acc_id = False
                        #There should only be one write-off account!
                        for entry in move.reconcile_id.line_new_ids:
                            writeoff_acc_id = entry.account_id.id
                            break
                        ## If we have already a partial reconciliation
                        ## We need to make a partial reconciliation
                        ## To add this amount to previous paid amount
                        if previous_partial:
                            account_move_line_obj.reconcile_partial(cr, uid, torec, 'statement', context)
                        ## If it's the first reconciliation, we do a full reconciliation as regular
                        else:
                            account_move_line_obj.reconcile(cr, uid, torec, 'statement', writeoff_acc_id=writeoff_acc_id, writeoff_period_id=st.period_id.id, writeoff_journal_id=st.journal_id.id, context=context)

                    else:
                        account_move_line_obj.reconcile_partial(cr, uid, torec, 'statement', context)
                    #except:
                    #    raise osv.except_osv(_('Error !'), _('Unable to reconcile entry "%s": %.2f') % (move.name, move.amount))

                if st.journal_id.entry_posted:
                    account_move_obj.write(cr, uid, [move_id], {'state':'posted'})
            done.append(st.id)
        self.write(cr, uid, done, {'state':'confirm'}, context=context)

        return True

account_bank_statement_button_confirm()


class account_bank_statement(osv.osv):
    _inherit = 'account.bank.statement'

    def _check_st(self, cr, uid, ids):
        st_ids = self.search(cr, uid, [('state','=','draft')])
        if len(st_ids)>1:
            return False
        return True

    def onchange_journal_id(self, cursor, user, statement_id, journal_id, name=None, context=None):
        if not journal_id:
            return {'value': {'currency': False}}

        account_journal_obj = self.pool.get('account.journal')
        res_users_obj = self.pool.get('res.users')
        res_currency_obj = self.pool.get('res.currency')
        journal_rec = account_journal_obj.browse(cursor, user, journal_id, context=context)
        cursor.execute('SELECT balance_end_real \
                FROM account_bank_statement \
                WHERE journal_id = %s \
                ORDER BY date DESC,id DESC LIMIT 1', (journal_id,))
        res = cursor.fetchone()
        balance_start = res and res[0] or 0.0

        currency_id = journal_rec.currency.id
        if not currency_id:
            currency_id = res_users_obj.browse(cursor, user, user,
                    context=context).company_id.currency_id.id
        currency = res_currency_obj.name_get(cursor, user, [currency_id],
                context=context)[0]
        if journal_rec.sequence_id and not name:
            jid = journal_rec.sequence_id.id
            number = self.pool.get('ir.sequence').get_id(cursor, user, jid, 'id=%s')
        elif name:
            number = name
        else:
            number = self.pool.get('ir.sequence').get(cursor, user, 'account.statement')

        return {'value': {'balance_start': balance_start, 'currency': currency, 'name':number}}

    _defaults = {
        'name': lambda *a: '',
    }

  #  _constraints = [(_check_st, u"Can not open more that one statement", ('name',))]

account_bank_statement()

class account_bank_statement_line(osv.osv):
    _inherit = 'account.bank.statement.line'

    _columns = {
        'analytics_id':fields.many2one('account.analytic.plan.instance','Analytic Distribution'),
    }

    def _check_account(self, cr, uid, ids):
        for line in self.browse(cr, uid, ids):
            if line.account_id.code in ('400000', '440000') and not line.partner_id:
                return False
        return True

 #   _constraints = [(_check_account, u"Can not register this line without partner", ('name',))]

account_bank_statement_line()

class account_move_post_override(osv.osv):
    _inherit = "account.move"

    _columns = {
        'line_id': fields.one2many('account.move.line', 'move_id', 'Entries'),
}

    def post(self, cr, uid, ids, context=None):
        move_ref = ''
        if context and context.get('invoice', None) and context['invoice']:
            move_ref = context['invoice'].number
        if self.validate(cr, uid, ids, context) and len(ids):
            for move in self.browse(cr, uid, ids):
                if move.name =='/':
                    new_name = False
                    journal = move.journal_id
                    if move_ref:
                        new_name = move_ref
                    elif journal.sequence_id:
                        c = {'fiscalyear_id': move.period_id.fiscalyear_id.id}
                        new_name = self.pool.get('ir.sequence').get_id(cr, uid, journal.sequence_id.id, context=c)
                    else:
                        raise osv.except_osv(_('Error'), _('No sequence defined in the journal !'))
                    if new_name:
                        self.write(cr, uid, [move.id], {'name':new_name})

            cr.execute('UPDATE account_move '\
                       'SET state=%s '\
                       'WHERE id IN %s',
                       ('posted', tuple(ids)))
        else:
            raise osv.except_osv(_('Integrity Error !'), _('You can not validate a non-balanced entry !\nMake sure you have configured Payment Term properly !\nIt should contain atleast one Payment Term Line with type "Balance" !'))
        return True

account_move_post_override()


class account_move_line(osv.osv):
    _inherit = "account.move.line"

    _columns = {
        'reconcile_id': fields.many2one('account.move.reconcile', 'Reconcile', ondelete='set null', select=2),
    }

    def reconcile_partial(self, cr, uid, ids, type='auto', context={}):
        merges = []
        unmerge = []
        total = 0.0
        merges_rec = []
        for line in self.browse(cr, uid, ids, context):
            if line.reconcile_id:
                err_mess = 'Verifiez la ligne pour le partenaire '
                err_mess += line.partner_id and line.partner_id.name or '/'
                raise osv.except_osv(_('Already Reconciled'), err_mess)
                #raise osv.except_osv(_('Already Reconciled'), _('Already Reconciled'))

            if line.reconcile_partial_id:
                for line2 in line.reconcile_partial_id.line_partial_ids:
                    if not line2.reconcile_id:
                        if line2.id not in merges:
                            merges.append(line2.id)
                        total += (line2.debit or 0.0) - (line2.credit or 0.0)
                merges_rec.append(line.reconcile_partial_id.id)
            else:
                unmerge.append(line.id)
                total += (line.debit or 0.0) - (line.credit or 0.0)

        if not total:
            res = self.reconcile(cr, uid, merges+unmerge, context=context)
            return res
        r_id = self.pool.get('account.move.reconcile').create(cr, uid, {
            'type': type,
            'line_partial_ids': map(lambda x: (4,x,False), merges+unmerge)
        })
        self.pool.get('account.move.reconcile').reconcile_partial_check(cr, uid, [r_id] + merges_rec, context=context)
        return True

    def reconcile(self, cr, uid, ids, type='auto', writeoff_acc_id=False, writeoff_period_id=False, writeoff_journal_id=False, context={}):
        id_set = ','.join(map(str, ids))
        
        lines = self.browse(cr, uid, ids, context=context)
        unrec_lines = filter(lambda x: not x['reconcile_id'], lines)
        credit = debit = 0.0
        currency = 0.0
        account_id = False
        partner_id = False
        invoice = False
        for line in unrec_lines:
            if line.invoice:
                invoice = line.invoice
            if line.state <> 'valid':
                raise osv.except_osv(_('Error'),
                        _('Entry "%s" is not valid !') % line.name)
            credit += line['credit']
            debit += line['debit']
            currency += line['amount_currency'] or 0.0
            account_id = line['account_id']['id']
            account_type = line['account_id']['type']
            partner_id = (line['partner_id'] and line['partner_id']['id']) or False
        writeoff = debit - credit
        # Ifdate_p in context => take this date
        if context.has_key('date_p') and context['date_p']:
            date=context['date_p']
        else:
            date = time.strftime('%Y-%m-%d')

        cr.execute('SELECT account_id, reconcile_id '\
                   'FROM account_move_line '\
                   'WHERE id IN %s '\
                   'GROUP BY account_id,reconcile_id',
                   (tuple(ids),))
        r = cr.fetchall()
#TODO: move this check to a constraint in the account_move_reconcile object
        if (len(r) != 1) and not context.get('fy_closing', False):
            cr.execute('SELECT p.name \
                 FROM account_move_line l\
                 LEFT JOIN res_partner p ON (l.partner_id = p.id) \
                 WHERE l.id IN ('+id_set+') \
                 ')
            r = cr.fetchall()
            raise osv.except_osv(_('Error'), ("L'entree du partenaire "+r[0][0]+" est deja reconciliee"))
            #raise osv.except_osv(_('Error'), _('Entries are not of the same account or already reconciled ! '))

        if not unrec_lines:
            raise osv.except_osv(_('Error'), _('Entry is already reconciled'))
        account = self.pool.get('account.account').browse(cr, uid, account_id, context=context)
        if not context.get('fy_closing', False) and not account.reconcile:
            raise osv.except_osv(_('Error'), _('The account is not defined to be reconciled !'))
        if r[0][1] != None:
            raise osv.except_osv(_('Error'), _('Some entries are already reconciled !'))
        writeoff_move_id = 0
        if (not self.pool.get('res.currency').is_zero(cr, uid, account.company_id.currency_id, writeoff)) or \
           (account.currency_id and (not self.pool.get('res.currency').is_zero(cr, uid, account.currency_id, currency))):
            if not writeoff_acc_id:
                raise osv.except_osv(_('Warning'), _('You have to provide an account for the write off entry !'))
            if writeoff > 0:
                debit = writeoff
                credit = 0.0
                self_credit = writeoff
                self_debit = 0.0
            else:
                debit = 0.0
                credit = -writeoff
                self_credit = 0.0
                self_debit = -writeoff

            # If comment exist in context, take it
            if 'comment' in context and context['comment']:
                libelle=context['comment']
            else:
                libelle= _('Write-Off')

            cur_obj = self.pool.get('res.currency')
            cur_id = False
            amount_currency_writeoff = 0.0
            if context.get('company_currency_id',False) != context.get('currency_id',False):
                cur_id = context.get('currency_id',False)
                for line in unrec_lines:
                    #tmp_amount = cur_obj.compute(cr, uid, invoice.company_id.currency_id.id, invoice.currency_id.id, abs(line.debit-line.credit), context={'date': line.date})
                    if line.currency_id and line.currency_id.id == context.get('currency_id',False):
                        amount_currency_writeoff += line.amount_currency
                    else:
                        tmp_amount = cur_obj.compute(cr, uid, line.account_id.company_id.currency_id.id, context.get('currency_id',False), abs(line.debit-line.credit), context={'date': line.date})
                        amount_currency_writeoff += (line.debit > 0) and tmp_amount or -tmp_amount
            writeoff_lines = [
                (0, 0, {
                    'name':libelle,
                    'debit':self_debit,
                    'credit':self_credit,
                    'account_id':account_id,
                    'date':date,
                    'partner_id':partner_id,
                    'currency_id': cur_id or (account.currency_id.id or False),
                    'amount_currency': amount_currency_writeoff and -1 * amount_currency_writeoff or (account.currency_id.id and -1 * currency or 0.0)
                }),
                (0, 0, {
                    'name':libelle,
                    'debit':debit,
                    'credit':credit,
                    'account_id':writeoff_acc_id,
                    'analytics_id': context.get('writeoff_analytics_id', False),
                    'date':date,
                    'partner_id':partner_id,
                    'currency_id': cur_id or (account.currency_id.id or False),
                    'amount_currency': amount_currency_writeoff and amount_currency_writeoff or (account.currency_id.id and currency or 0.0)
                })
            ]
            writeoff_move_id = self.pool.get('account.move').create(cr, uid, {
                'period_id': writeoff_period_id,
                'journal_id': writeoff_journal_id,
                'date':date,
                'state': 'draft',
                'line_id': writeoff_lines
            })
            writeoff_line_ids = self.search(cr, uid, [('move_id', '=', writeoff_move_id), ('account_id', '=', account_id)])
            if account_id == writeoff_acc_id:
                writeoff_line_ids = [writeoff_line_ids[1]]
            ids += writeoff_line_ids

        r_id = self.pool.get('account.move.reconcile').create(cr, uid, {
            #'name': date,
            'type': type,
            'line_id': map(lambda x: (4,x,False), ids),
            'line_partial_ids': map(lambda x: (3,x,False), ids)
        })
        wf_service = netsvc.LocalService("workflow")
        # the id of the move.reconcile is written in the move.line (self) by the create method above
        # because of the way the line_id are defined: (4, x, False)
        for id in ids:
            wf_service.trg_trigger(uid, 'account.move.line', id, cr)
        if writeoff_move_id:
            self.pool.get('account.move').post(cr,uid,[writeoff_move_id],context)  ## always confirm the JOD for write off
        return r_id

account_move_line()

class res_partner_address(osv.osv):
    _inherit = "res.partner.address"
    _description = 'res.partner.address'
    _columns = {
        'special_ref_on_invoice' : fields.char('Special Ref On invoices', size=60, help="Message that appears on all invoices sended to this address"),
    }
res_partner_address()


