# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import datetime
import pooler
import time
import base64
from pyExcelerator import *
import tools
from osv import fields, osv
from tools.translate import _

_form = """<?xml version="1.0"?>
<form string="Extract 281.50 records">
    <label string="This wizard extract data from bookkeeping to create official belgian 281.50 records.\nYou'll be abel to correct and select the final datas to print, but the best,\nis to correct financial datas and re-extract correct data." colspan="4" />
    <label string="Be CAREFULL : this new extraction will REPLACE the old one, deleting all data you have manually corrected." colspan="4" />
    <field name="year_id"/>
    <field name="tag_id"/>
    <field name="limit"/>
    <field name="min_natnum_percent"/>
    <newline/>
    <field name="account_starts_list" colspan="4" width="750"/>
</form>"""

_fields = {
    'year_id': {'string': 'Concerned Year', 'type': 'many2one', 'relation':'account.fiscalyear','required': True, 'help':"The year of the extracted purchases"},
    'tag_id': {'string':'Tag associated','type':'many2one','relation':'crm_profiling.question','domain':"[('target','=','res.partner')]",'required':True},
    'account_starts_list': {'string': 'Accounts list', 'type':'char', 'size':250, 'required': True, 'help':'List of accounts beginnings on wich extract purchases; i.e. "603,613,612950"'},
    'limit': {'string':'Limit','type':'float','required':True,'default':125.0,'help':'Limit under witch no 281.50 are emitted.'},
    'min_natnum_percent':{'string':'Percent minimum Nat.Number','type':'integer','required':True,'default':85,'help':'Minimum percent of national number'},
}

class extract281_50(wizard.interface):
    def _init_data(self ,cr, uid, data, context):
        past_year = int(datetime.datetime.now().strftime('%Y'))-1
        pool = pooler.get_pool(cr.dbname)
        prec_fy_id = pool.get('account.fiscalyear').search(cr,uid,[('date_start','=',str(past_year)+'-01-01')])
        if prec_fy_id:
            data['form']['year_id'] = prec_fy_id[0]
        return data
    def _split_streets_and_number(self,street1,street2):
        final_street = street1 or ''
        number = ''
        if ',' in final_street:
            pos = final_street.find(',')
            number = final_street[pos+1:].strip()
            final_street = final_street[0:pos].strip()
        if street2:
            final_street += ( ' - ' + street2 )
        return (final_street,number)
    def _create_records(self ,cr, uid, data, context):
        # erase the records of the same year to create new ones
        pool = pooler.get_pool(cr.dbname)
        sel_year = int(pool.get('account.fiscalyear').read(cr,uid,[data['form']['year_id']],['date_start'])[0]['date_start'][0:4])
        record_obj = pool.get('account.partner281_50')
        old_record_ids = record_obj.search(cr,uid,[('year','=',sel_year)])
        if old_record_ids:
            record_obj.unlink(cr,uid,old_record_ids)
        # extraction of purchases invoices with at least a line in the selected accounts
        # for each of them, we control if the invoice OR the payments are in the concerned year
        # If concerned, we get it

        # construction of the account_ids list
        account_starts_list = data['form']['account_starts_list']
        account_starts = account_starts_list.split(',')
        account_starts = [x.strip() for x in account_starts]
        account_ids = []
        for start in account_starts:
            start_ids = pool.get('account.account').search(cr,uid,[('code','ilike',start+'%'),('type','<>','view')])
            account_ids.extend(start_ids)

        # creation of the table to convert periods to year in string
        all_period_ids = pool.get('account.period').search(cr,uid,[])
        all_periods = pool.get('account.period').read(cr,uid,all_period_ids,['date_start'])
        dPeriod2Year = {}
        for per in all_periods:
            dPeriod2Year[ per['id'] ] = int(per['date_start'][0:4])

        # selection of tagged partners
        selection1 = """SELECT partner
                                           FROM partner_question_rel as pqr, crm_profiling_answer as cpa
                                           WHERE cpa.question_id = %s AND pqr.answer = cpa.id 
                     """ % data['form']['tag_id']
        cr.execute(selection1)
        res1 = cr.fetchall()
        partner_ids = [str(x[0]) for x in res1]
        
        # selection of concerned invoices
        selection = """SELECT id
                           FROM account_invoice 
                           WHERE type in ('in_invoice','in_refund') AND
                                 state in ('open','paid') AND 
                                 partner_id in (%s) AND
                                 id in ( SELECT invoice_id
                                             FROM account_invoice_line 
                                             WHERE account_id in (%s) )
                    """ % (','.join(partner_ids),
                           ','.join([str(x) for x in account_ids])
                          )
        cr.execute(selection)
        res = cr.fetchall()
        dPartners = {}
        invoice_ids = [x[0] for x in res]
        invoices = pool.get('account.invoice').browse(cr,uid,invoice_ids)
        for invoice in invoices:
            lInvoiceToTake = False
            # if this invoice is linked to payment(s), we extract the list of payment(s) name and payment(s) year(s)
            pay_years = []
            payments = []
            if invoice.payment_ids:
                for payment in invoice.payment_ids:
                    payment_year = dPeriod2Year[payment.period_id.id]
                    payment_name = payment.move_id.name and payment.move_id.name or payment.journal_id.name
                    if payment_year not in pay_years:
                        pay_years.append(payment_year)
                    payments.append(payment_name)
                # we keep the invoice if the invoice year or one of the payment(s) year is the concerned year
                if sel_year in pay_years:
                    lInvoiceToTake = True
            if lInvoiceToTake:
                # for this invoice, we'll check each line to check the account
                # and we'll cumulate these lines by partners
                for iline in invoice.invoice_line:
                    if iline.account_id.id in account_ids:
                        if iline.invoice_id.partner_id.id not in dPartners.keys():
                            dPartners[iline.invoice_id.partner_id.id] = 0.0
                        if invoice.type == 'in_invoice':
                            dPartners[iline.invoice_id.partner_id.id] += iline.price_subtotal
                        else:
                            dPartners[iline.invoice_id.partner_id.id] -= iline.price_subtotal
        partners = pool.get('res.partner').browse(cr,uid,dPartners.keys())
        new_ids = []
        for partner in partners:
            if dPartners[partner.id] > data['form']['limit']:
                new_record = {}
                new_record['partner_id'] = partner.id
                new_record['year'] = sel_year
                new_record['calc_sum_b'] = dPartners[partner.id]
                new_record['name'] = partner.name
                if partner.address:
                    new_record['street1'] = partner.address[0].street or ''
                    new_record['street2'] = partner.address[0].street2 or ''
                    if partner.address[0].zip_id:
                        new_record['zip_code'] = partner.address[0].zip_id.name or ''
                        new_record['city'] = partner.address[0].zip_id.city or ''
                    if partner.vat:
                        new_record['company_number'] = partner.vat
                    else:
                        # search of national number of the first contact_person
                        if partner.address[0].job_ids:
                            if partner.address[0].job_ids[0].contact_id:
                                new_record['national_number'] = partner.address[0].job_ids[0].contact_id.national_number or ''
                if partner.answers_ids:
                    for answer in partner.answers_ids:
                        if answer.question_id.id == data['form']['tag_id']:
                            new_record['profession'] = answer.text.replace('\n','').strip()
                new_record['calc_sum_b'] = dPartners[partner.id]
                new_id = record_obj.create(cr,uid,new_record)
                new_ids.append(new_id)
        result = {
            'domain': [('id', 'in', new_ids)],
            'name': _('Final 281.50 records'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'account.partner281_50',
            'context': {},
            'type': 'ir.actions.act_window'
        }
        return result
    states = {
        'init': {
            'actions': [_init_data],
            'result': {'type':'form', 'arch':_form, 'fields':_fields, 'state':[('end','Cancel'),('extract','Extract')]},
        },
        'extract': {
            'actions': [],
            'result': {'type':'action', 'action':_create_records,'state':'end'}
        },
    }

extract281_50('extract281_50')

