# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields
from osv import osv
import datetime
import netsvc
from tools.translate import _
from pyExcelerator import *
from tools.translate import _

class account_analytic_account(osv.osv):

    _inherit = 'account.analytic.account'
    _description = 'Analytic Accounts'

    def _get_periods_range(self,cr,uid,period_beg,period_end):
        per_obj = self.pool.get('account.period')
        per_beg = per_obj.browse(cr, uid, period_beg)
        per_end = per_obj.browse(cr, uid, period_end)
        if per_beg.date_start > per_end.date_start:
            per_beg,per_end = per_end,per_beg
        pers_start = per_beg.date_start
        pers_end = per_end.date_stop
        periods = per_obj.search(cr,uid, [('special','=',False),('date_start','>=',pers_start),('date_stop','<=',pers_end)])
        str_per_ids = ','.join([str(x) for x in periods])
        return str_per_ids

    def _credit_calc(self, cr, uid, ids, name, arg, context={}):
        if not ids: return {}

        if context.get('from_period') and context.get('to_period') and not context.get('from_date') and not context.get('to_date'):
            where_clause = self._get_periods_range(cr,uid,context.get('from_period'),context.get('to_period'))
            list_ids = ','.join([str(x) for x in ids])
            cr.execute("SELECT a.id, COALESCE(SUM(l.amount), 0) \
                          FROM account_analytic_account a \
                          LEFT JOIN ( SELECT aal.amount as amount, aal.account_id as account_id \
                                		FROM account_analytic_line as aal, account_move_line as aml \
                                		WHERE aal.move_id = aml.id and aml.period_id in (%s) \
                                    ) as l ON (a.id=l.account_id) \
                          WHERE l.amount < 0 AND a.id IN (%s) \
                          GROUP BY a.id" % (where_clause, list_ids) )
        else:
            where_date = ''
            if context.get('from_date'):
                where_date += " AND l.date >= %(from_date)s"
            if context.get('to_date'):
                where_date += " AND l.date <= %(to_date)s"

            cr.execute("SELECT a.id, COALESCE(SUM(l.amount), 0) "
                       "FROM account_analytic_account a "
                       "LEFT JOIN account_analytic_line l ON (a.id=l.account_id %s)"
                       "WHERE l.amount < 0 AND a.id IN %%(ids)s "
                       "GROUP BY a.id" % (where_date),
                       dict(context, ids=tuple(ids)))
        r = dict(cr.fetchall())
        self.logger.notifyChannel('addons.'+self._name, netsvc.LOG_DEBUG,
                                  '_credit_calc results: %s'%r)
        for i in ids:
            r.setdefault(i,0.0)
        return r

    def _debit_calc(self, cr, uid, ids, name, arg, context={}):
        self.logger.notifyChannel('addons.'+self._name, netsvc.LOG_DEBUG,
                                  'Entering _debit_calc; ids:%s'%ids)
        if not ids: return {}

        if context.get('from_period') and context.get('to_period') and not context.get('from_date') and not context.get('to_date'):
            where_clause = self._get_periods_range(cr,uid,context.get('from_period'),context.get('to_period'))
            list_ids = ','.join([str(x) for x in ids])
            cr.execute("SELECT a.id, COALESCE(SUM(l.amount), 0) \
                          FROM account_analytic_account a \
                          LEFT JOIN ( SELECT aal.amount as amount, aal.account_id as account_id \
                                		FROM account_analytic_line as aal, account_move_line as aml \
                                		WHERE aal.move_id = aml.id and aml.period_id in (%s) \
                                    ) as l ON (a.id=l.account_id) \
                          WHERE l.amount > 0 AND a.id IN (%s) \
                          GROUP BY a.id" % (where_clause, list_ids) )
        else:
            where_date = ''
            if context.get('from_date'):
                where_date += " AND l.date >= %(from_date)s"
            if context.get('to_date'):
                where_date += " AND l.date <= %(to_date)s"

            cr.execute("SELECT a.id, COALESCE(SUM(l.amount), 0) "
                       "FROM account_analytic_account a "
                       "LEFT JOIN account_analytic_line l ON (a.id=l.account_id %s)"
                       " WHERE l.amount > 0 AND a.id IN %%(ids)s "
                       "GROUP BY a.id" % (where_date),
                       dict(context, ids=tuple(ids)))
        r = dict(cr.fetchall())
        self.logger.notifyChannel('addons.'+self._name, netsvc.LOG_DEBUG,
                                  '_debut_calc results: %s'%r)
        for i in ids:
            r.setdefault(i,0.0)
        return r

    def _balance_calc(self, cr, uid, ids, name, arg, context={}):
        res = {}
        ids2 = self.search(cr, uid, [('parent_id', 'child_of', ids)])
        self.logger.notifyChannel('addons.'+self._name, netsvc.LOG_DEBUG,
                                  'Entering _balance_calc; ids:%s; ids2:%s'%(
                            ids, ids2))

        for i in ids:
            res.setdefault(i,0.0)

        if not ids2:
            return res

        if context.get('from_period') and context.get('to_period') and not context.get('from_date') and not context.get('to_date'):
            where_clause = self._get_periods_range(cr,uid,context.get('from_period'),context.get('to_period'))
            list_ids2 = ','.join([str(x) for x in ids])
            cr.execute("SELECT a.id, COALESCE(SUM(l.amount), 0) \
                          FROM account_analytic_account a \
                          LEFT JOIN ( SELECT aal.amount as amount, aal.account_id as account_id \
                                		FROM account_analytic_line as aal, account_move_line as aml \
                                		WHERE aal.move_id = aml.id and aml.period_id in (%s) \
                                    ) as l ON (a.id=l.account_id) \
                          WHERE a.id IN (%s) \
                          GROUP BY a.id" % (where_clause, list_ids2) )
        else:
            where_date = ''
            if context.get('from_date'):
                where_date += " AND l.date >= %(from_date)s"
            if context.get('to_date'):
                where_date += " AND l.date <= %(to_date)s"

            cr.execute("SELECT a.id, COALESCE(SUM(l.amount),0) "
                       "FROM account_analytic_account a "
                       "LEFT JOIN account_analytic_line l ON (a.id=l.account_id %s)"
                       " WHERE a.id IN %%(ids)s "
                       "GROUP BY a.id" % (where_date),
                       dict(context, ids=tuple(ids2)))

        for account_id, sum in cr.fetchall():
            res[account_id] = sum
        self.logger.notifyChannel('addons.'+self._name, netsvc.LOG_DEBUG,
                                  '_balance_calc, (id, sum): %s'%res)

        cr.execute("SELECT a.id, r.currency_id "
                   "FROM account_analytic_account a "
                   "INNER JOIN res_company r ON (a.company_id = r.id) "
                   "WHERE a.id in %s", (tuple(ids2),))

        currency = dict(cr.fetchall())
        self.logger.notifyChannel('addons.'+self._name, netsvc.LOG_DEBUG,
                                  '_balance_calc currency: %s'%currency)

        res_currency= self.pool.get('res.currency')
        for id in ids:
            if id not in ids2:
                continue
            for child in self.search(cr, uid, [('parent_id', 'child_of', [id])]):
                if child != id:
                    res.setdefault(id, 0.0)
                    if  currency[child]<>currency[id]:
                        res[id] += res_currency.compute(cr, uid, currency[child], currency[id], res.get(child, 0.0), context=context)
                    else:
                        res[id] += res.get(child, 0.0)

        cur_obj = res_currency.browse(cr,uid,currency.values(),context)
        cur_obj = dict([(o.id, o) for o in cur_obj])
        for id in ids:
            if id in ids2:
                res[id] = res_currency.round(cr,uid,cur_obj[currency[id]],res.get(id,0.0))

        return dict([(i, res[i]) for i in ids ])
    _columns = {
        'balance' : fields.function(_balance_calc, method=True, type='float', string='Balance'),
        'debit' : fields.function(_debit_calc, method=True, type='float', string='Debit'),
        'credit' : fields.function(_credit_calc, method=True, type='float', string='Credit'),
    }

account_analytic_account()

class account_analytic_line(osv.osv):
    _inherit = 'account.analytic.line'
    _columns = {
        'partner_id' : fields.related('move_id', 'partner_id',  type='many2one', relation='res.partner', string="Partner",store=False),
        'period_id' : fields.related('move_id', 'period_id',  type='many2one', relation='account.period', string="Period",store=False),
        'account_journal_id' : fields.related('move_id', 'journal_id',  type='many2one', relation='account.journal', string="Account Journal",store=False),
        'move_name' : fields.related('move_id', 'move_id',  type='many2one', relation='account.move', string="Move name",store=False),
    }

    def _get_periods_list(self,cr,uid,period_beg,period_end):
        per_obj = self.pool.get('account.period')
        per_beg = per_obj.browse(cr, uid, period_beg)
        per_end = per_obj.browse(cr, uid, period_end)
        if per_beg.date_start > per_end.date_start:
            per_beg,per_end = per_end,per_beg
        pers_start = per_beg.date_start
        pers_end = per_end.date_stop
        periods = per_obj.search(cr,uid, [('special','=',False),('date_start','>=',pers_start),('date_stop','<=',pers_end)])
        return periods

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):

        if context and context.get('from_period',False):
            args.append(['period_id', 'in',self._get_periods_list(cr,uid,context['from_period'],context['to_period'])])
            
        return super(account_analytic_line, self).search(cr, uid, args, offset, limit,
                order, context=context, count=count)

account_analytic_line()

class account_invoice(osv.osv):
    _inherit = 'account.invoice'
    _description = 'Account Invoice'

    def on_change_partner_id(self, cr, uid, ids, type, partner_id,date_invoice=False, payment_term=False):
        # we surcharge this method created in account_l10nbe_domiciliation/l10nbe_domiciliation.py
        # because we don't use the domiciliation_bool for the same usage : fur us, it serve to say
        # if we can use the domiciliation on membership products
        data=super(account_invoice,self).onchange_partner_id( cr, uid, ids, type, partner_id,date_invoice, payment_term)
        if not partner_id:
            return data['value'].update({'domiciled' : False})
        partner_obj = self.pool.get('res.partner').browse(cr, uid, partner_id)
        domiciled = partner_obj.domiciliation
        data['value']['domiciled'] = domiciled
        warning = False
        if partner_id:
            if partner_obj.alert_others:
                warning = {
                    'title': "Warning:",
                    'message': partner_obj.alert_explanation or 'Partner is not valid'
                        }
        if warning:
            data['warning'] =  warning
        return data
    def unlink(self, cr, uid, ids, context=None):
        # we change the way the unlink is prohibited
        invoices = self.read(cr, uid, ids, ['state','number'])
        unlink_ids = []
        for t in invoices:
            if t['state'] in ('draft', 'cancel'):
                if not t['number']:
                    unlink_ids.append(t['id'])
                else:
                    raise osv.except_osv(_('Invalid action !'), _('Cannot delete invoice(s) that have a number !'))
            else:
                raise osv.except_osv(_('Invalid action !'), _('Cannot delete invoice(s) that are already opened or paid !'))
        osv.osv.unlink(self, cr, uid, unlink_ids, context=context)
        return True
account_invoice()

class account_move(osv.osv):
    _inherit = 'account.move'
    def _correct_ref_purchases(self,cr,uid):
        # extract all purchases journal
        journal_ids = self.pool.get('account.journal').search(cr,uid,[('type','=','purchase')])
        journal_list = ','.join(map(str,journal_ids))
        # extract all names from account_move for lines without ref
        selection = """SELECT id, name
                        	FROM account_move where id in 
                        	    (SELECT distinct(aline.move_id) as id
                        	        FROM account_move_line as aline 
                                	WHERE aline.journal_id in (%s) and aline.ref is null)
                    """ % journal_list
        cr.execute(selection)
        result = cr.fetchall()

        # correct all lines of all these incomplete moves
        corrected = 0
        for move in result:
            if move[1]:
                corrected += 1
                correction = "UPDATE account_move_line set ref = '%s' where move_id = %s and ref is null" % (move[1].replace('/',''),move[0])
                cr.execute(correction)

        # recording of the asked changes in the CCI Logs
        log_obj = self.pool.get('cci_logs.line')
        log_obj.create(cr,uid,{'name':'Correct Refs of Purchase',
                               'datetime':datetime.datetime.now(),
                               'user_id':uid,
                               'message':'Purchase Moves corrected : %s' % str(corrected),
                              })
        return corrected
account_move()

class account_analytic_plan_instance(osv.osv):
    _inherit='account.analytic.plan.instance'
    _columns={
        'active':fields.boolean('Active'),
        'date_start':fields.date('Start Use',help='For comment only'),
        'date_stop': fields.date('End Use',help='For comment only, make it inactivce if you don\'t want to use it from now'),
    }
    _defaults={
        'active': lambda *args: True,
        'date_start': lambda *args: datetime.datetime.today().strftime("%Y-%m-%d"), 
    }
    def write(self, cr, uid, ids, vals, context={}, check=True, update_check=True):
        this = self.browse(cr,uid,ids[0])
        vals.update({'plan_id':this.plan_id.id})
        # we do this to stop the wrong behavior caused by lack of 'vals['plan_id']' if the plan is not changed (99% cases)
        return super(account_analytic_plan_instance, self).write(cr, uid, ids, vals, context)
account_analytic_plan_instance()

class account_period(osv.osv):
    _inherit = "account.period"
    _columns = {
        'vat_last_time': fields.datetime('Last Compute of VAT'),
        'vat_last_value': fields.float('Last Computed VAT',digits=(16,2),help='If positive, VAT due to company [72]; if negative, VAT due to government [71]'),
        'smoothed': fields.boolean('Already smoothed'),
    }
    _default = {
        'smoothed': lambda *args: False,
    }
    def preceding(self,cr,uid,ids):
        period_obj = self.pool.get('account.period')
        result = {}
        for period_id in ids:
            period = period_obj.read(cr,uid,period_id,['date_start'])
            starting_date = datetime.datetime.strptime(period['date_start'], "%Y-%m-%d").date()
            new_month = starting_date.month-1
            new_year = starting_date.year
            while new_month < 1:
                new_month += 12
                new_year -= 1
            new_starting_date = datetime.date(new_year,new_month,starting_date.day).strftime('%Y-%m-%d')
            preceding_id = period_obj.search(cr,uid,[('date_start','=',new_starting_date),('special','=',False)])
            result[period_id] = preceding_id and preceding_id[0] or 0
        return result  ## {'period_id':'preceding_period_id',...}
    def precedings(self,cr,uid,ids,count=1):
        result = {}
        for period_id in ids:
            current_id = period_id
            preceding_ids = []
            current_count = 1
            while current_count <= count:
                prec = self.preceding(cr,uid,[current_id,])
                if prec:
                    current_id = prec[current_id]
                    preceding_ids.append(current_id)
                current_count += 1
            result[period_id] = preceding_ids
        return result  ## {'period_id':['period_id month - 1','period_id month - 2', ...],...}
    def smooth(self,cr,uid, ids,context={}):
        #
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['SpecialMovesInSmoothingIDS'])
        if param_values.has_key('SpecialMovesInSmoothingIDS'):
            special_move_ids = param_values['SpecialMovesInSmoothingIDS']
        else:
            special_move_ids = []
        #
        period_obj = self.pool.get('account.period')
        already_smoothed_period_ids = period_obj.search(cr,uid,[('id','in',ids),('smoothed','=',True)])
        if already_smoothed_period_ids:
            periods = period_obj.read(cr,uid,already_smoothed_period_ids,['name'])
            result = {'status':'KO',
                      'error_num':1,
                      'error_details':_('Already smoothed periods : %s\n\nYou can unmark them AFTER removing the smoothing movements already created in theses periodes.') % ','.join(x['name'] for x in periods),
                     }
        else:
            closed_period_ids = period_obj.search(cr,uid,[('id','in',ids),('state','<>','draft')])
            if closed_period_ids:
                periods = period_obj.read(cr,uid,closed_period_ids,['name'])
                result = {'status':'KO',
                          'error_num':2,
                          'error_details':_('Closed periods : %s\n\nWe can\'t create smmothing moves in a closed period.') % ','.join(x['name'] for x in periods),
                         }
            else:
                account_obj = self.pool.get('account.account')
                amove_obj = self.pool.get('account.move')
                amline_obj = self.pool.get('account.move.line')
                journal_obj = self.pool.get('account.journal')
                journal_ids = self.pool.get('account.journal').search(cr,uid,[('type','=','sale')])
                if journal_ids:
                    filename = 'smoothing_%s.xls' % datetime.datetime.now().strftime('%Y_%m_%d_%H_%M_%S')
                    wb = Workbook()
                    ws1 = wb.add_sheet(_('Smoothings'))
                    ws1.write(0,0,_('Origin Period'))
                    ws1.write(0,1,_('Origin Account'))
                    ws1.write(0,2,_('Target Account'))
                    ws1.write(0,3,_('Target Period'))
                    ws1.write(0,4,_('Amount'))
                    xls_line = 1
                    res_ids = []
                    total_amount = 0.0
                    for period_id in ids:
                        target_period = period_obj.browse(cr,uid,period_id)
                        concerned_account_ids = account_obj.search(cr,uid,[('smoothing_type','!=','none'),('smoothing_account_id','!=',False)])
                        concerned_accounts = account_obj.read(cr,uid,concerned_account_ids,['code','name','smoothing_type','smoothing_account_id','smoothing_analytics_id','smoothing_journal_id','smoothing_precyear_id'])
                        for account in concerned_accounts:
                            orig_account_name = account['code']
                            full_orig_account_name = account['code'] + ' ' + account['name']
                            target_account = account_obj.browse(cr,uid,account['smoothing_account_id'][0])
                            target_account_name = target_account.code
                            full_target_account_name = target_account.code + ' ' + target_account.name

                            #####################
                            # smooth on 12 months
                            #####################
                            if account['smoothing_type'] == 'months12':
                                preceding_ids = [period_id,] + self.precedings(cr,uid,[period_id,],11)[period_id]
                                lines_to_add = []
                                total_reported_amount = 0.0
                                for preceding_id in preceding_ids:
                                    amline_ids = amline_obj.search(cr,uid,[('period_id','=',preceding_id),('account_id','=',account['id']),('journal_id','in',journal_ids)])
                                    # possible special moves to take into account event not on a sales journal
                                    if special_move_ids:
                                        special_amline_ids = amline_obj.search(cr,uid,[('period_id','=',prec_period.id),('account_id','=',account['id']),('move_id','in',special_move_ids)])
                                        amline_ids.extend(special_amline_ids)
                                    # possible pre-year invoicing transfered to this year
                                    if int(prec_period.date_start[5:7]) == 1 and account['smoothing_precyear_id']:
                                        ouv_period_id = period_obj.search(cr,uid,[('date_start','=',period.date_start[0:4]+'-01-01'),('special','=',True)])
                                        ouv_amline_ids = amline_obj.search(cr,uid,[('period_id','in',ouv_period_id),('account_id','=',account['id'])])
                                        if ouv_amline_ids:
                                            amline_ids.extend(ouv_amline_ids)
                                    if amline_ids:
                                        amlines = amline_obj.browse(cr,uid,amline_ids)
                                        period_name = amlines[0].period_id.name
                                        sum_debit = 0.0
                                        sum_credit = 0.0
                                        for amline in amlines:
                                            sum_debit += amline.debit or 0.0
                                            sum_credit += amline.credit or 0.0
                                        total_amount_period = ( sum_credit - sum_debit )
                                        reported_amount_period = round(total_amount_period/12,2)
                                        if reported_amount_period >= 0.01 or reported_amount_period <= -0.01:
                                            lines_to_add.append({'account_id':account['id'],
                                                                 'analytics_id':account['smoothing_analytics_id'][0], # probably not after correction the obligation of analytics_id on invoices
                                                                 'journal_id':account['smoothing_journal_id'][0],
                                                                 'blocked':False,
                                                                 'credit':reported_amount_period <= -0.01 and abs(reported_amount_period) or 0.0,
                                                                 'debit':reported_amount_period >= 0.01 and reported_amount_period or 0.0,
                                                                 'date':target_period.date_stop,
                                                                 'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                                 'name':u'Récup de %s depuis pér %s vers %s - Type 1/12' % (orig_account_name,period_name,target_account_name),
                                                                 'period_id':period_id,
                                                                 'quantity':1,
                                                                })
                                            ws1.write(xls_line,0,period_name)
                                            ws1.write(xls_line,1,full_orig_account_name)
                                            ws1.write(xls_line,2,full_target_account_name)
                                            ws1.write(xls_line,3,target_period.name)
                                            ws1.write(xls_line,4,reported_amount_period)
                                            xls_line += 1
                                        total_reported_amount += reported_amount_period
                                if lines_to_add:
                                    lines_to_add.append({'account_id':target_account.id,
                                                         'analytics_id':account['smoothing_analytics_id'][0],
                                                         'journal_id':account['smoothing_journal_id'][0],
                                                         'blocked':False,
                                                         'credit':total_reported_amount >= 0.01 and total_reported_amount or 0.0,
                                                         'debit':total_reported_amount <= -0.01 and abs(total_reported_amount) or 0.0,
                                                         'date':target_period.date_stop,
                                                         'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                         'name':u'Récup de %s vers %s dans pér %s - Type 1/12' % (orig_account_name,target_account_name,target_period.name),
                                                         'period_id':period_id,
                                                         'quantity':1,
                                                        })
                                    vals = {
                                        'name': '/',
                                        'ref': u'Récup lissage de %s vers %s' % (orig_account_name,target_account_name),
                                        'period_id': period_id,
                                        'journal_id': account['smoothing_journal_id'][0],
                                        'date':target_period.date_stop,
                                    }
                                    move_id = amove_obj.create(cr, uid, vals, context=context,)
                                    res_ids.append(move_id)
                                    for aline in lines_to_add:
                                        aline['move_id'] = move_id
                                        amline_obj.create(cr,uid,aline)
                                    move = amove_obj.post(cr, uid, [move_id,])

                            #####################
                            # smooth on 10 months
                            #####################
                            elif account['smoothing_type'] == 'months10':
                                period = period_obj.browse(cr,uid,period_id)
                                if period.date_start[5:7] not in ('07','08'):
                                    # same treatment as months12 but /10 in place of /12 for periods not 08 and 07
                                    preceding_ids = [period_id,] + self.precedings(cr,uid,[period_id,],11)[period_id]
                                    lines_to_add = []
                                    total_reported_amount = 0.0
                                    for preceding_id in preceding_ids:
                                        current_period = period_obj.browse(cr,uid,preceding_id)
                                        period_end_date = current_period.date_stop
                                        period_name = current_period.name
                                        if int(period_end_date[5:7]) < 7 or int(period_end_date[5:7]) > 8:
                                            amline_ids = amline_obj.search(cr,uid,[('period_id','=',preceding_id),('account_id','=',account['id']),('journal_id','in',journal_ids)])
                                            # possible special moves to take into account event not on a sales journal
                                            if special_move_ids:
                                                special_amline_ids = amline_obj.search(cr,uid,[('period_id','=',prec_period.id),('account_id','=',account['id']),('move_id','in',special_move_ids)])
                                                amline_ids.extend(special_amline_ids)
                                            # possible pre-year invoicing transfered to this year
                                            if int(prec_period.date_start[5:7]) == 1 and account['smoothing_precyear_id']:
                                                ouv_period_id = period_obj.search(cr,uid,[('date_start','=',period.date_start[0:4]+'-01-01'),('special','=',True)])
                                                ouv_amline_ids = amline_obj.search(cr,uid,[('period_id','in',ouv_period_id),('account_id','=',account['id'])])
                                                if ouv_amline_ids:
                                                    amline_ids.extend(ouv_amline_ids)
                                            if amline_ids:
                                                amlines = amline_obj.browse(cr,uid,amline_ids)
                                                sum_debit = 0.0
                                                sum_credit = 0.0
                                                for amline in amlines:
                                                    sum_debit += amline.debit or 0.0
                                                    sum_credit += amline.credit or 0.0
                                                total_amount_period = ( sum_credit - sum_debit )
                                                reported_amount_period = round(total_amount_period/10,2)
                                                if reported_amount_period >= 0.01 or reported_amount_period <= -0.01:
                                                    lines_to_add.append({'account_id':account['id'],
                                                                         'analytics_id':account['smoothing_analytics_id'][0], # probably not after correction the obligation of analytics_id on invoices
                                                                         'journal_id':account['smoothing_journal_id'][0],
                                                                         'blocked':False,
                                                                         'credit':reported_amount_period <= -0.01 and abs(reported_amount_period) or 0.0,
                                                                         'debit':reported_amount_period >= 0.01 and reported_amount_period or 0.0,
                                                                         'date':target_period.date_stop,
                                                                         'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                                         'name':u'Récup de %s depuis pér %s vers %s - Type 1/10' % (orig_account_name,period_name,target_account_name),
                                                                         'period_id':period_id,
                                                                         'quantity':1,
                                                                        })
                                                    ws1.write(xls_line,0,period_name)
                                                    ws1.write(xls_line,1,full_orig_account_name)
                                                    ws1.write(xls_line,2,full_target_account_name)
                                                    ws1.write(xls_line,3,target_period.name)
                                                    ws1.write(xls_line,4,reported_amount_period)
                                                    xls_line += 1
                                                total_reported_amount += reported_amount_period
                                    if lines_to_add:
                                        lines_to_add.append({'account_id':target_account.id,
                                                             'analytics_id':account['smoothing_analytics_id'][0],
                                                             'journal_id':account['smoothing_journal_id'][0],
                                                             'blocked':False,
                                                             'credit':total_reported_amount >= 0.01 and total_reported_amount or 0.0,
                                                             'debit':total_reported_amount <= -0.01 and abs(total_reported_amount) or 0.0,
                                                             'date':target_period.date_stop,
                                                             'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                             'name':u'Récup de %s vers %s dans pér %s - Type 1/10' % (orig_account_name,target_account_name,target_period.name),
                                                             'period_id':period_id,
                                                             'quantity':1,
                                                            })
                                        vals = {
                                            'name': '/',
                                            'ref': u'Récup lissage de %s vers %s' % (orig_account_name,target_account_name),
                                            'period_id': period_id,
                                            'journal_id': account['smoothing_journal_id'][0],
                                            'date':target_period.date_stop,
                                        }
                                        move_id = amove_obj.create(cr, uid, vals, context=context,)
                                        res_ids.append(move_id)
                                        for aline in lines_to_add:
                                            aline['move_id'] = move_id
                                            amline_obj.create(cr,uid,aline)
                                        move = amove_obj.post(cr, uid, [move_id,])
                                else:
                                    # july or august => smooth all this period and only this period
                                    lines_to_add = []
                                    total_reported_amount = 0.0
                                    amline_ids = amline_obj.search(cr,uid,[('period_id','=',period_id),('account_id','=',account['id']),('journal_id','in',journal_ids)])
                                    if amline_ids:
                                        amlines = amline_obj.browse(cr,uid,amline_ids)
                                        period_name = period.name
                                        period_end_date = period.date_stop
                                        sum_debit = 0.0
                                        sum_credit = 0.0
                                        for amline in amlines:
                                            sum_debit += amline.debit or 0.0
                                            sum_credit += amline.credit or 0.0
                                        total_amount_period = ( sum_credit - sum_debit )
                                        reported_amount_period = round(total_amount_period,2)
                                        if reported_amount_period >= 0.01 or reported_amount_period <= -0.01:
                                            lines_to_add.append({'account_id':account['id'],
                                                                 'analytics_id':account['smoothing_analytics_id'][0], # probably not after correction the obligation of analytics_id on invoices
                                                                 'journal_id':account['smoothing_journal_id'][0],
                                                                 'blocked':False,
                                                                 'credit':reported_amount_period <= -0.01 and abs(reported_amount_period) or 0.0,
                                                                 'debit':reported_amount_period >= 0.01 and reported_amount_period or 0.0,
                                                                 'date':period.date_stop,
                                                                 'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                                 'name':u'Récup de %s depuis pér %s vers %s - Type 10 mois exceptés Juillet-Août' % (orig_account_name,period_name,target_account_name),
                                                                 'period_id':period_id,
                                                                 'quantity':1,
                                                                })
                                            ws1.write(xls_line,0,period_name)
                                            ws1.write(xls_line,1,full_orig_account_name)
                                            ws1.write(xls_line,2,full_target_account_name)
                                            ws1.write(xls_line,3,target_period.name)
                                            ws1.write(xls_line,4,reported_amount_period)
                                            xls_line += 1
                                    if lines_to_add:
                                        lines_to_add.append({'account_id':target_account.id,
                                                             'analytics_id':account['smoothing_analytics_id'][0],
                                                             'journal_id':account['smoothing_journal_id'][0],
                                                             'blocked':False,
                                                             'credit':reported_amount_period >= 0.01 and reported_amount_period or 0.0,
                                                             'debit':reported_amount_period <= -0.01 and abs(reported_amount_period) or 0.0,
                                                             'date':period.date_stop,
                                                             'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                             'name':u'Récup de %s vers %s dans pér %s - Type 10 mois exceptés Juillet-Août' % (orig_account_name,target_account_name,target_period.name),
                                                             'period_id':period_id,
                                                             'quantity':1,
                                                            })
                                        vals = {
                                            'name': '/',
                                            'ref': u'Récup lissage de %s vers %s' % (orig_account_name,target_account_name),
                                            'period_id': period_id,
                                            'journal_id': account['smoothing_journal_id'][0],
                                            'date':period.date_stop,
                                        }
                                        move_id = amove_obj.create(cr, uid, vals, context=context,)
                                        res_ids.append(move_id)
                                        for aline in lines_to_add:
                                            aline['move_id'] = move_id
                                            amline_obj.create(cr,uid,aline)
                                        move = amove_obj.post(cr, uid, [move_id,])

                            ##################################
                            # smooth on 6 months except July #
                            ##################################
                            elif account['smoothing_type'] == 'months6':
                                period = period_obj.browse(cr,uid,period_id)
                                if period.date_start[5:7] <> '07':
                                    # same treatment as months12 but /6 in place of /12 for periods not 07
                                    preceding_ids = [period_id,] + self.precedings(cr,uid,[period_id,],6)[period_id]
                                    lines_to_add = []
                                    total_reported_amount = 0.0
                                    treated_periods = 0
                                    for preceding_id in preceding_ids:
                                        current_period = period_obj.browse(cr,uid,preceding_id)
                                        period_end_date = current_period.date_stop
                                        period_name = current_period.name
                                        if int(period_end_date[5:7]) <> 7:
                                            treated_periods += 1
                                            if treated_periods > 6: # we treat just 6 periods, not 7, but we consider 7 periods because of july
                                                continue
                                            amline_ids = amline_obj.search(cr,uid,[('period_id','=',preceding_id),('account_id','=',account['id']),('journal_id','in',journal_ids)])
                                            # possible special moves to take into account event not on a sales journal
                                            if special_move_ids:
                                                special_amline_ids = amline_obj.search(cr,uid,[('period_id','=',prec_period.id),('account_id','=',account['id']),('move_id','in',special_move_ids)])
                                                amline_ids.extend(special_amline_ids)
                                            # possible pre-year invoicing transfered to this year
                                            if int(prec_period.date_start[5:7]) == 1 and account['smoothing_precyear_id']:
                                                ouv_period_id = period_obj.search(cr,uid,[('date_start','=',period.date_start[0:4]+'-01-01'),('special','=',True)])
                                                ouv_amline_ids = amline_obj.search(cr,uid,[('period_id','in',ouv_period_id),('account_id','=',account['id'])])
                                                if ouv_amline_ids:
                                                    amline_ids.extend(ouv_amline_ids)
                                            if amline_ids:
                                                amlines = amline_obj.browse(cr,uid,amline_ids)
                                                sum_debit = 0.0
                                                sum_credit = 0.0
                                                for amline in amlines:
                                                    sum_debit += amline.debit or 0.0
                                                    sum_credit += amline.credit or 0.0
                                                total_amount_period = ( sum_credit - sum_debit )
                                                reported_amount_period = round(total_amount_period/6,2)
                                                if reported_amount_period >= 0.01 or reported_amount_period <= -0.01:
                                                    lines_to_add.append({'account_id':account['id'],
                                                                         'analytics_id':account['smoothing_analytics_id'][0], # probably not after correction the obligation of analytics_id on invoices
                                                                         'journal_id':account['smoothing_journal_id'][0],
                                                                         'blocked':False,
                                                                         'credit':reported_amount_period <= -0.01 and abs(reported_amount_period) or 0.0,
                                                                         'debit':reported_amount_period >= 0.01 and reported_amount_period or 0.0,
                                                                         'date':target_period.date_stop,
                                                                         'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                                         'name':u'Récup de %s depuis pér %s vers %s - Type 1/6 sauf Juillet' % (orig_account_name,period_name,target_account_name),
                                                                         'period_id':period_id,
                                                                         'quantity':1,
                                                                        })
                                                    ws1.write(xls_line,0,period_name)
                                                    ws1.write(xls_line,1,full_orig_account_name)
                                                    ws1.write(xls_line,2,full_target_account_name)
                                                    ws1.write(xls_line,3,target_period.name)
                                                    ws1.write(xls_line,4,reported_amount_period)
                                                    xls_line += 1
                                                total_reported_amount += reported_amount_period
                                    if lines_to_add:
                                        lines_to_add.append({'account_id':target_account.id,
                                                             'analytics_id':account['smoothing_analytics_id'][0],
                                                             'journal_id':account['smoothing_journal_id'][0],
                                                             'blocked':False,
                                                             'credit':total_reported_amount >= 0.01 and total_reported_amount or 0.0,
                                                             'debit':total_reported_amount <= -0.01 and abs(total_reported_amount) or 0.0,
                                                             'date':target_period.date_stop,
                                                             'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                             'name':u'Récup de %s vers %s dans pér %s - Type 1/6 sauf Juillet' % (orig_account_name,target_account_name,target_period.name),
                                                             'period_id':period_id,
                                                             'quantity':1,
                                                            })
                                        vals = {
                                            'name': '/',
                                            'ref': u'Récup lissage de %s vers %s' % (orig_account_name,target_account_name),
                                            'period_id': period_id,
                                            'journal_id': account['smoothing_journal_id'][0],
                                            'date':target_period.date_stop,
                                        }
                                        move_id = amove_obj.create(cr, uid, vals, context=context,)
                                        res_ids.append(move_id)
                                        for aline in lines_to_add:
                                            aline['move_id'] = move_id
                                            amline_obj.create(cr,uid,aline)
                                        move = amove_obj.post(cr, uid, [move_id,])
                                else:
                                    # july => smooth all this period and only this period
                                    lines_to_add = []
                                    total_reported_amount = 0.0
                                    amline_ids = amline_obj.search(cr,uid,[('period_id','=',period_id),('account_id','=',account['id']),('journal_id','in',journal_ids)])
                                    if amline_ids:
                                        amlines = amline_obj.browse(cr,uid,amline_ids)
                                        period_name = period.name
                                        period_end_date = period.date_stop
                                        sum_debit = 0.0
                                        sum_credit = 0.0
                                        for amline in amlines:
                                            sum_debit += amline.debit or 0.0
                                            sum_credit += amline.credit or 0.0
                                        total_amount_period = ( sum_credit - sum_debit )
                                        reported_amount_period = round(total_amount_period,2)
                                        if reported_amount_period >= 0.01 or reported_amount_period <= -0.01:
                                            lines_to_add.append({'account_id':account['id'],
                                                                 'analytics_id':account['smoothing_analytics_id'][0], # probably not after correction the obligation of analytics_id on invoices
                                                                 'journal_id':account['smoothing_journal_id'][0],
                                                                 'blocked':False,
                                                                 'credit':reported_amount_period <= -0.01 and abs(reported_amount_period) or 0.0,
                                                                 'debit':reported_amount_period >= 0.01 and reported_amount_period or 0.0,
                                                                 'date':period.date_stop,
                                                                 'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                                 'name':u'Récup de %s depuis pér %s vers %s - Type 6 mois sauf Juillet' % (orig_account_name,period_name,target_account_name),
                                                                 'period_id':period_id,
                                                                 'quantity':1,
                                                                })
                                            ws1.write(xls_line,0,period_name)
                                            ws1.write(xls_line,1,full_orig_account_name)
                                            ws1.write(xls_line,2,full_target_account_name)
                                            ws1.write(xls_line,3,target_period.name)
                                            ws1.write(xls_line,4,reported_amount_period)
                                            xls_line += 1
                                    if lines_to_add:
                                        lines_to_add.append({'account_id':target_account.id,
                                                             'analytics_id':account['smoothing_analytics_id'][0],
                                                             'journal_id':account['smoothing_journal_id'][0],
                                                             'blocked':False,
                                                             'credit':reported_amount_period >= 0.01 and reported_amount_period or 0.0,
                                                             'debit':reported_amount_period <= -0.01 and abs(reported_amount_period) or 0.0,
                                                             'date':period.date_stop,
                                                             'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                             'name':u'Récup de %s vers %s dans pér %s - Type 6 mois sauf Juillet' % (orig_account_name,target_account_name,target_period.name),
                                                             'period_id':period_id,
                                                             'quantity':1,
                                                            })
                                        vals = {
                                            'name': '/',
                                            'ref':u'Récup lissage de %s vers %s' % (orig_account_name,target_account_name),
                                            'period_id': period_id,
                                            'journal_id': account['smoothing_journal_id'][0],
                                            'date':period.date_stop,
                                        }
                                        move_id = amove_obj.create(cr, uid, vals, context=context,)
                                        res_ids.append(move_id)
                                        for aline in lines_to_add:
                                            aline['move_id'] = move_id
                                            amline_obj.create(cr,uid,aline)
                                        move = amove_obj.post(cr, uid, [move_id,])

                            ###############################################
                            # smooth on 6 months except July and Augustus #
                            ###############################################
                            elif account['smoothing_type'] == 'months6ja':
                                period = period_obj.browse(cr,uid,period_id)
                                if period.date_start[5:7] not in ['07','08']:
                                    # same treatment as months12 but /6 in place of /12 for periods not 07 and 08
                                    preceding_ids = [period_id,] + self.precedings(cr,uid,[period_id,],7)[period_id]
                                    lines_to_add = []
                                    total_reported_amount = 0.0
                                    treated_periods = 0
                                    for preceding_id in preceding_ids:
                                        current_period = period_obj.browse(cr,uid,preceding_id)
                                        period_end_date = current_period.date_stop
                                        period_name = current_period.name
                                        if int(period_end_date[5:7]) not in [7,8]:
                                            treated_periods += 1
                                            if treated_periods > 6: # we treat just 6 periods, not 7, but we consider 8 periods because of july and augustus
                                                continue
                                            amline_ids = amline_obj.search(cr,uid,[('period_id','=',preceding_id),('account_id','=',account['id']),('journal_id','in',journal_ids)])
                                            # possible special moves to take into account event not on a sales journal
                                            if special_move_ids:
                                                special_amline_ids = amline_obj.search(cr,uid,[('period_id','=',prec_period.id),('account_id','=',account['id']),('move_id','in',special_move_ids)])
                                                amline_ids.extend(special_amline_ids)
                                            # possible pre-year invoicing transfered to this year
                                            if int(prec_period.date_start[5:7]) == 1 and account['smoothing_precyear_id']:
                                                ouv_period_id = period_obj.search(cr,uid,[('date_start','=',period.date_start[0:4]+'-01-01'),('special','=',True)])
                                                ouv_amline_ids = amline_obj.search(cr,uid,[('period_id','in',ouv_period_id),('account_id','=',account['id'])])
                                                if ouv_amline_ids:
                                                    amline_ids.extend(ouv_amline_ids)
                                            if amline_ids:
                                                amlines = amline_obj.browse(cr,uid,amline_ids)
                                                sum_debit = 0.0
                                                sum_credit = 0.0
                                                for amline in amlines:
                                                    sum_debit += amline.debit or 0.0
                                                    sum_credit += amline.credit or 0.0
                                                total_amount_period = ( sum_credit - sum_debit )
                                                reported_amount_period = round(total_amount_period/6,2)
                                                if reported_amount_period >= 0.01 or reported_amount_period <= -0.01:
                                                    lines_to_add.append({'account_id':account['id'],
                                                                         'analytics_id':account['smoothing_analytics_id'][0], # probably not after correction the obligation of analytics_id on invoices
                                                                         'journal_id':account['smoothing_journal_id'][0],
                                                                         'blocked':False,
                                                                         'credit':reported_amount_period <= -0.01 and abs(reported_amount_period) or 0.0,
                                                                         'debit':reported_amount_period >= 0.01 and reported_amount_period or 0.0,
                                                                         'date':target_period.date_stop,
                                                                         'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                                         'name':u'Récup de %s depuis pér %s vers %s - Type 1/6 sauf Juillet et Août' % (orig_account_name,period_name,target_account_name),
                                                                         'period_id':period_id,
                                                                         'quantity':1,
                                                                        })
                                                    ws1.write(xls_line,0,period_name)
                                                    ws1.write(xls_line,1,full_orig_account_name)
                                                    ws1.write(xls_line,2,full_target_account_name)
                                                    ws1.write(xls_line,3,target_period.name)
                                                    ws1.write(xls_line,4,reported_amount_period)
                                                    xls_line += 1
                                                total_reported_amount += reported_amount_period
                                    if lines_to_add:
                                        lines_to_add.append({'account_id':target_account.id,
                                                             'analytics_id':account['smoothing_analytics_id'][0],
                                                             'journal_id':account['smoothing_journal_id'][0],
                                                             'blocked':False,
                                                             'credit':total_reported_amount >= 0.01 and total_reported_amount or 0.0,
                                                             'debit':total_reported_amount <= -0.01 and abs(total_reported_amount) or 0.0,
                                                             'date':target_period.date_stop,
                                                             'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                             'name':u'Récup de %s vers %s dans pér %s - Type 1/6 sauf Juillet et Août' % (orig_account_name,target_account_name,target_period.name),
                                                             'period_id':period_id,
                                                             'quantity':1,
                                                            })
                                        vals = {
                                            'name': '/',
                                            'ref': u'Récup lissage de %s vers %s' % (orig_account_name,target_account_name),
                                            'period_id': period_id,
                                            'journal_id': account['smoothing_journal_id'][0],
                                            'date':target_period.date_stop,
                                        }
                                        move_id = amove_obj.create(cr, uid, vals, context=context,)
                                        res_ids.append(move_id)
                                        for aline in lines_to_add:
                                            aline['move_id'] = move_id
                                            amline_obj.create(cr,uid,aline)
                                        move = amove_obj.post(cr, uid, [move_id,])
                                else:
                                    # july or augustus=> smooth all this period and only this period
                                    lines_to_add = []
                                    total_reported_amount = 0.0
                                    amline_ids = amline_obj.search(cr,uid,[('period_id','=',period_id),('account_id','=',account['id']),('journal_id','in',journal_ids)])
                                    if amline_ids:
                                        amlines = amline_obj.browse(cr,uid,amline_ids)
                                        period_name = period.name
                                        period_end_date = period.date_stop
                                        sum_debit = 0.0
                                        sum_credit = 0.0
                                        for amline in amlines:
                                            sum_debit += amline.debit or 0.0
                                            sum_credit += amline.credit or 0.0
                                        total_amount_period = ( sum_credit - sum_debit )
                                        reported_amount_period = round(total_amount_period,2)
                                        if reported_amount_period >= 0.01 or reported_amount_period <= -0.01:
                                            lines_to_add.append({'account_id':account['id'],
                                                                 'analytics_id':account['smoothing_analytics_id'][0], # probably not after correction the obligation of analytics_id on invoices
                                                                 'journal_id':account['smoothing_journal_id'][0],
                                                                 'blocked':False,
                                                                 'credit':reported_amount_period <= -0.01 and abs(reported_amount_period) or 0.0,
                                                                 'debit':reported_amount_period >= 0.01 and reported_amount_period or 0.0,
                                                                 'date':period.date_stop,
                                                                 'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                                 'name':u'Récup de %s depuis pér %s vers %s - Type 6 mois sauf Juillet et Août' % (orig_account_name,period_name,target_account_name),
                                                                 'period_id':period_id,
                                                                 'quantity':1,
                                                                })
                                            ws1.write(xls_line,0,period_name)
                                            ws1.write(xls_line,1,full_orig_account_name)
                                            ws1.write(xls_line,2,full_target_account_name)
                                            ws1.write(xls_line,3,target_period.name)
                                            ws1.write(xls_line,4,reported_amount_period)
                                            xls_line += 1
                                    if lines_to_add:
                                        lines_to_add.append({'account_id':target_account.id,
                                                             'analytics_id':account['smoothing_analytics_id'][0],
                                                             'journal_id':account['smoothing_journal_id'][0],
                                                             'blocked':False,
                                                             'credit':reported_amount_period >= 0.01 and reported_amount_period or 0.0,
                                                             'debit':reported_amount_period <= -0.01 and abs(reported_amount_period) or 0.0,
                                                             'date':period.date_stop,
                                                             'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                             'name':u'Récup de %s vers %s dans pér %s - Type 6 mois sauf Juillet et Août' % (orig_account_name,target_account_name,target_period.name),
                                                             'period_id':period_id,
                                                             'quantity':1,
                                                            })
                                        vals = {
                                            'name': '/',
                                            'ref':u'Récup lissage de %s vers %s' % (orig_account_name,target_account_name),
                                            'period_id': period_id,
                                            'journal_id': account['smoothing_journal_id'][0],
                                            'date':period.date_stop,
                                        }
                                        move_id = amove_obj.create(cr, uid, vals, context=context,)
                                        res_ids.append(move_id)
                                        for aline in lines_to_add:
                                            aline['move_id'] = move_id
                                            amline_obj.create(cr,uid,aline)
                                        move = amove_obj.post(cr, uid, [move_id,])

                            #####################
                            # smooth on 11 months
                            #####################
                            elif account['smoothing_type'] == 'months11y':
                                period = period_obj.browse(cr,uid,period_id)
                                if period.date_start[5:7] <> '07':
                                    # same treatment as months12 but /11 in place of /12 for periods not 07
                                    preceding_ids = [period_id,] + self.precedings(cr,uid,[period_id,],11)[period_id]
                                    lines_to_add = []
                                    total_reported_amount = 0.0
                                    treated_periods = 0
                                    for preceding_id in preceding_ids:
                                        current_period = period_obj.browse(cr,uid,preceding_id)
                                        period_end_date = current_period.date_stop
                                        period_name = current_period.name
                                        if int(period_end_date[5:7]) <> 7:
                                            treated_periods += 1
                                            if treated_periods > 11: # we treat just 11 periods, not 7, but we consider 12 periods because of july
                                                continue
                                            amline_ids = amline_obj.search(cr,uid,[('period_id','=',preceding_id),('account_id','=',account['id']),('journal_id','in',journal_ids)])
                                            # possible special moves to take into account event not on a sales journal
                                            if special_move_ids:
                                                special_amline_ids = amline_obj.search(cr,uid,[('period_id','=',prec_period.id),('account_id','=',account['id']),('move_id','in',special_move_ids)])
                                                amline_ids.extend(special_amline_ids)
                                            # possible pre-year invoicing transfered to this year
                                            if int(prec_period.date_start[5:7]) == 1 and account['smoothing_precyear_id']:
                                                ouv_period_id = period_obj.search(cr,uid,[('date_start','=',period.date_start[0:4]+'-01-01'),('special','=',True)])
                                                ouv_amline_ids = amline_obj.search(cr,uid,[('period_id','in',ouv_period_id),('account_id','=',account['id'])])
                                                if ouv_amline_ids:
                                                    amline_ids.extend(ouv_amline_ids)
                                            if amline_ids:
                                                amlines = amline_obj.browse(cr,uid,amline_ids)
                                                sum_debit = 0.0
                                                sum_credit = 0.0
                                                for amline in amlines:
                                                    sum_debit += amline.debit or 0.0
                                                    sum_credit += amline.credit or 0.0
                                                total_amount_period = ( sum_credit - sum_debit )
                                                reported_amount_period = round(total_amount_period/11,2)
                                                if reported_amount_period >= 0.01 or reported_amount_period <= -0.01:
                                                    lines_to_add.append({'account_id':account['id'],
                                                                         'analytics_id':account['smoothing_analytics_id'][0], # probably not after correction the obligation of analytics_id on invoices
                                                                         'journal_id':account['smoothing_journal_id'][0],
                                                                         'blocked':False,
                                                                         'credit':reported_amount_period <= -0.01 and abs(reported_amount_period) or 0.0,
                                                                         'debit':reported_amount_period >= 0.01 and reported_amount_period or 0.0,
                                                                         'date':target_period.date_stop,
                                                                         'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                                         'name':u'Récup de %s depuis pér %s vers %s - Type 1/11 sauf Juillet' % (orig_account_name,period_name,target_account_name),
                                                                         'period_id':period_id,
                                                                         'quantity':1,
                                                                        })
                                                    ws1.write(xls_line,0,period_name)
                                                    ws1.write(xls_line,1,full_orig_account_name)
                                                    ws1.write(xls_line,2,full_target_account_name)
                                                    ws1.write(xls_line,3,target_period.name)
                                                    ws1.write(xls_line,4,reported_amount_period)
                                                    xls_line += 1
                                                total_reported_amount += reported_amount_period
                                    if lines_to_add:
                                        lines_to_add.append({'account_id':target_account.id,
                                                             'analytics_id':account['smoothing_analytics_id'][0],
                                                             'journal_id':account['smoothing_journal_id'][0],
                                                             'blocked':False,
                                                             'credit':total_reported_amount >= 0.01 and total_reported_amount or 0.0,
                                                             'debit':total_reported_amount <= -0.01 and abs(total_reported_amount) or 0.0,
                                                             'date':target_period.date_stop,
                                                             'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                             'name':u'Récup de %s vers %s dans pér %s - Type 1/11 sauf Juillet' % (orig_account_name,target_account_name,target_period.name),
                                                             'period_id':period_id,
                                                             'quantity':1,
                                                            })
                                        vals = {
                                            'name': '/',
                                            'ref': u'Récup lissage de %s vers %s' % (orig_account_name,target_account_name),
                                            'period_id': period_id,
                                            'journal_id': account['smoothing_journal_id'][0],
                                            'date':target_period.date_stop,
                                        }
                                        move_id = amove_obj.create(cr, uid, vals, context=context,)
                                        res_ids.append(move_id)
                                        for aline in lines_to_add:
                                            aline['move_id'] = move_id
                                            amline_obj.create(cr,uid,aline)
                                        move = amove_obj.post(cr, uid, [move_id,])
                                else:
                                    # july => smooth all this period and only this period
                                    lines_to_add = []
                                    total_reported_amount = 0.0
                                    amline_ids = amline_obj.search(cr,uid,[('period_id','=',period_id),('account_id','=',account['id']),('journal_id','in',journal_ids)])
                                    if amline_ids:
                                        amlines = amline_obj.browse(cr,uid,amline_ids)
                                        period_name = period.name
                                        period_end_date = period.date_stop
                                        sum_debit = 0.0
                                        sum_credit = 0.0
                                        for amline in amlines:
                                            sum_debit += amline.debit or 0.0
                                            sum_credit += amline.credit or 0.0
                                        total_amount_period = ( sum_credit - sum_debit )
                                        reported_amount_period = round(total_amount_period,2)
                                        if reported_amount_period >= 0.01 or reported_amount_period <= -0.01:
                                            lines_to_add.append({'account_id':account['id'],
                                                                 'analytics_id':account['smoothing_analytics_id'][0], # probably not after correction the obligation of analytics_id on invoices
                                                                 'journal_id':account['smoothing_journal_id'][0],
                                                                 'blocked':False,
                                                                 'credit':reported_amount_period <= -0.01 and abs(reported_amount_period) or 0.0,
                                                                 'debit':reported_amount_period >= 0.01 and reported_amount_period or 0.0,
                                                                 'date':period.date_stop,
                                                                 'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                                 'name':u'Récup de %s depuis pér %s vers %s - Type 11 mois sauf Juillet' % (orig_account_name,period_name,target_account_name),
                                                                 'period_id':period_id,
                                                                 'quantity':1,
                                                                })
                                            ws1.write(xls_line,0,period_name)
                                            ws1.write(xls_line,1,full_orig_account_name)
                                            ws1.write(xls_line,2,full_target_account_name)
                                            ws1.write(xls_line,3,target_period.name)
                                            ws1.write(xls_line,4,reported_amount_period)
                                            xls_line += 1
                                    if lines_to_add:
                                        lines_to_add.append({'account_id':target_account.id,
                                                             'analytics_id':account['smoothing_analytics_id'][0],
                                                             'journal_id':account['smoothing_journal_id'][0],
                                                             'blocked':False,
                                                             'credit':reported_amount_period >= 0.01 and reported_amount_period or 0.0,
                                                             'debit':reported_amount_period <= -0.01 and abs(reported_amount_period) or 0.0,
                                                             'date':period.date_stop,
                                                             'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                             'name':u'Récup de %s vers %s dans pér %s - Type 11 mois sauf Juillet' % (orig_account_name,target_account_name,target_period.name),
                                                             'period_id':period_id,
                                                             'quantity':1,
                                                            })
                                        vals = {
                                            'name': '/',
                                            'ref':u'Récup lissage de %s vers %s' % (orig_account_name,target_account_name),
                                            'period_id': period_id,
                                            'journal_id': account['smoothing_journal_id'][0],
                                            'date':period.date_stop,
                                        }
                                        move_id = amove_obj.create(cr, uid, vals, context=context,)
                                        res_ids.append(move_id)
                                        for aline in lines_to_add:
                                            aline['move_id'] = move_id
                                            amline_obj.create(cr,uid,aline)
                                        move = amove_obj.post(cr, uid, [move_id,])

                            ################################
                            # Smooth on the Rest of the Year
                            ################################
                            elif account['smoothing_type'] == 'rest_year':
                                period = period_obj.browse(cr,uid,period_id)
                                current_year = period.date_start[0:4]
                                current_month = int(period.date_start[5:7])
                                if current_month == 1:
                                    preceding_ids = [period_id,]
                                else:
                                    preceding_ids = [period_id,] + self.precedings(cr,uid,[period_id,],current_month-1)[period_id]
                                lines_to_add = []
                                total_reported_amount = 0.0
                                for preceding_id in preceding_ids:
                                    prec_period = period_obj.browse(cr,uid,preceding_id)
                                    period_name = prec_period.name
                                    period_end_date = prec_period.date_stop
                                    amline_ids = amline_obj.search(cr,uid,[('period_id','=',prec_period.id),('account_id','=',account['id']),('journal_id','in',journal_ids)])
                                    # possible special moves to take into account event not on a sales journal
                                    if special_move_ids:
                                        special_amline_ids = amline_obj.search(cr,uid,[('period_id','=',prec_period.id),('account_id','=',account['id']),('move_id','in',special_move_ids)])
                                        amline_ids.extend(special_amline_ids)
                                    # possible pre-year invoicing transfered to this year
                                    if int(prec_period.date_start[5:7]) == 1 and account['smoothing_precyear_id']:
                                        ouv_period_id = period_obj.search(cr,uid,[('date_start','=',period.date_start[0:4]+'-01-01'),('special','=',True)])
                                        ouv_amline_ids = amline_obj.search(cr,uid,[('period_id','in',ouv_period_id),('account_id','=',account['id'])])
                                        if ouv_amline_ids:
                                            amline_ids.extend(ouv_amline_ids)
                                    if amline_ids:
                                        amlines = amline_obj.browse(cr,uid,amline_ids)
                                        sum_debit = 0.0
                                        sum_credit = 0.0
                                        for amline in amlines:
                                            sum_debit += amline.debit or 0.0
                                            sum_credit += amline.credit or 0.0
                                        total_amount_period = ( sum_credit - sum_debit )
                                        divider = 13-int(prec_period.date_start[5:7])
                                        reported_amount_period = round(total_amount_period/divider,2)
                                        if reported_amount_period >= 0.01 or reported_amount_period <= -0.01:
                                            lines_to_add.append({'account_id':account['id'],
                                                                 'analytics_id':account['smoothing_analytics_id'][0], # probably not after correction the obligation of analytics_id on invoices
                                                                 'journal_id':account['smoothing_journal_id'][0],
                                                                 'blocked':False,
                                                                 'credit':reported_amount_period <= -0.01 and abs(reported_amount_period) or 0.0,
                                                                 'debit':reported_amount_period >= 0.01 and reported_amount_period or 0.0,
                                                                 'date':target_period.date_stop,
                                                                 'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                                 'name':u'Récup de %s depuis pér %s vers %s - Type Reste de l\'Année' % (orig_account_name,period_name,target_account_name),
                                                                 'period_id':period_id,
                                                                 'quantity':1,
                                                                })
                                            ws1.write(xls_line,0,period_name)
                                            ws1.write(xls_line,1,full_orig_account_name)
                                            ws1.write(xls_line,2,full_target_account_name)
                                            ws1.write(xls_line,3,target_period.name)
                                            ws1.write(xls_line,4,reported_amount_period)
                                            xls_line += 1
                                        total_reported_amount += reported_amount_period
                                if lines_to_add:
                                    lines_to_add.append({'account_id':target_account.id,
                                                         'analytics_id':account['smoothing_analytics_id'][0],
                                                         'journal_id':account['smoothing_journal_id'][0],
                                                         'blocked':False,
                                                         'credit':total_reported_amount >= 0.01 and total_reported_amount or 0.0,
                                                         'debit':total_reported_amount <= -0.01 and abs(total_reported_amount) or 0.0,
                                                         'date':target_period.date_stop,
                                                         'date_created':datetime.date.today().strftime('%Y-%m-%d'),
                                                         'name':u'Récup de %s vers %s dans pér %s - Type Reste de l\'Année' % (orig_account_name,target_account_name,target_period.name),
                                                         'period_id':period_id,
                                                         'quantity':1,
                                                        })
                                    vals = {
                                        'name': '/',
                                        'ref': u'Récup lissage de %s vers %s' % (orig_account_name,target_account_name),
                                        'period_id': period_id,
                                        'journal_id': account['smoothing_journal_id'][0],
                                        'date':target_period.date_stop,
                                    }
                                    move_id = amove_obj.create(cr, uid, vals, context=context,)
                                    res_ids.append(move_id)
                                    #amove_obj.write(cr,uid,[move_id,],{'line_id':[(6,0,lines_to_add)]})
                                    for aline in lines_to_add:
                                        aline['move_id'] = move_id
                                        amline_obj.create(cr,uid,aline)
                                    move = amove_obj.post(cr, uid, [move_id,])
                            total_amount += total_reported_amount

                        ######################################
                        # ALL CASES
                        # mark the smoothed period as smoothed
                        ######################################
                        period_obj.write(cr,uid,[target_period.id,],{'smoothed':True})
                    result = {'status':'OK','sum':total_amount,'filename':filename,'move_ids':res_ids}
                    wb.save(filename)
                else:
                    result = {'status':'KO',
                              'error_num':3,
                              'error_details':_('No journal of type "sales".'),
                             }
        return result
account_period()

class product_product(osv.osv):
    _inherit = 'product.product'
    _columns = {
        'cci_temp_intra_code' : fields.selection([('S','Prestations de services intracommunautaires'),
                                                  ('L','Livraisons intracommunautaires de biens exemptées'),
                                                  ('T','Transports de biens dans le cadre d\'opérations triangulaires')], 'Code Intracommunautaire'),
    }
    _defaults={
        'cci_temp_intra_code': lambda *args: 'S',
    }
product_product()

class account_account(osv.osv):
    _inherit ="account.account"
    _columns={
        'cci_forced_analytic' : fields.boolean('Forced Analytic' ),
        'smoothing_type': fields.selection([('none','None'),('months12','On next 12 months'),('months10','On next 10 months w/o July-Augustus'),('rest_year','Rest of the Year'),('months6','On next 6 months except July'),('months11y','On next 11 months w/o July'),('months6ja','On next 6 months except July and Augustus')],'Smoothing Type'),
        'smoothing_account_id' : fields.many2one('account.account', 'Account target for smoothing' ),
        'smoothing_analytics_id': fields.many2one('account.analytic.plan.instance','Analytic Distribution for Smoothing'),
        'smoothing_journal_id': fields.many2one('account.journal','Journal for Smoothing'),
        'smoothing_precyear_id': fields.many2one('account.account', 'Preceding Year Account', help='If Rest of Year Smooting, give the possible account for pre-year invoicing' ),
    }
    _defaults = {
        'cci_forced_analytic': lambda *args: False,
        'smoothing_type': lambda *args: 'none',
    }
account_account()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

