# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import pooler
import time
from report import report_sxw

class cci_account_analytic_cost_ledger(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(cci_account_analytic_cost_ledger, self).__init__(cr, uid, name, context=context)
        self.localcontext.update( {
            'time': time,
            'lines_g': self._lines_g,
            'lines_a': self._lines_a,
            'account_sum_amount': self._account_sum_amount,
            'sum_amount': self._sum_amount,
            'period_name': self._period_name,
        })

    def _period_name(self,period_id):    
        per_obj = pooler.get_pool(self.cr.dbname).get('account.period').browse(self.cr, self.uid, period_id)
        return per_obj.name
    def _get_periods_range(self,period_beg,period_end):
        per_obj = pooler.get_pool(self.cr.dbname).get('account.period')
        per_beg = per_obj.browse(self.cr, self.uid, period_beg)
        per_end = per_obj.browse(self.cr, self.uid, period_end)
        if per_beg.date_start > per_end.date_start:
            per_beg,per_end = per_end,per_beg
        pers_start = per_beg.date_start
        pers_end = per_end.date_stop
        periods = per_obj.search(self.cr,self.uid, [('special','=',False),('date_start','>=',pers_start),('date_stop','<=',pers_end)])
        str_per_ids = ','.join([str(x) for x in periods])
        return str_per_ids
    def _lines_g(self, account_id, period_from, period_to):
        str_per_ids = self._get_periods_range(period_from,period_to)
        self.cr.execute("SELECT sum(aal.amount) AS amount, aa.code AS code, aa.name AS name, aa.id AS id \
                FROM account_account AS aa, account_analytic_line AS aal, account_move_line as aml \
                WHERE (aal.account_id=%s) AND (aml.period_id in (%s) ) AND (aal.move_id=aml.id AND aal.general_account_id=aa.id) AND aa.active \
                GROUP BY aa.code, aa.name, aa.id ORDER BY aa.code" % (account_id, str_per_ids))
        res = self.cr.dictfetchall()
        return res

    def _lines_a(self, general_account_id, account_id, period_from, period_to):
        str_per_ids = self._get_periods_range(period_from,period_to)
        self.cr.execute("SELECT f.name, f.code, f.amount, f.period, f.move_name, partner.name as pname \
                            FROM (SELECT aml.partner_id as amlpid, aal.name AS name, aal.code AS code, aal.amount AS amount, ap.name AS period, am.name as move_name \
                                    FROM account_analytic_line AS aal, account_move_line as aml, account_move as am, account_period AS ap \
                                    WHERE (aal.general_account_id=%s) AND (aal.account_id=%s) AND (aml.period_id in (%s) ) \
                                        AND (aal.move_id=aml.id AND aml.move_id=am.id AND aml.period_id=ap.id) \
                                    ORDER BY aal.date \
                                 ) as f \
                            LEFT OUTER JOIN res_partner as partner ON (f.amlpid=partner.id)" % (general_account_id, account_id, str_per_ids))
        res = self.cr.dictfetchall()
        print len(res)
        return res

    def _account_sum_amount(self, account_id, period_from, period_to):
        str_per_ids = self._get_periods_range(period_from,period_to)
        self.cr.execute("SELECT sum(aal.amount) \
                         FROM account_analytic_line as aal, account_move_line as aml \
                         WHERE aal.account_id=%s AND aml.period_id in (%s) AND aal.move_id=aml.id" % (account_id, str_per_ids))
        return self.cr.fetchone()[0] or 0.0

    def _sum_amount(self, accounts, period_from, period_to):
        str_per_ids = self._get_periods_range(period_from,period_to)
        ids = map(lambda x: x.id, accounts)
        if not len(ids):
            return 0.0
        str_ids = ','.join([str(x) for x in tuple(ids)])
        self.cr.execute("SELECT sum(aal.amount) \
                         FROM account_analytic_line as aal, account_move_line as aml \
                         WHERE aal.account_id IN (%s) AND aml.period_id in (%s) AND aal.move_id=aml.id" % (str_ids, str_per_ids))
        return self.cr.fetchone()[0] or 0.0

report_sxw.report_sxw('report.cci_analytic_cost_ledger',
                      'account.analytic.account',
                      'addons/cci_account_extend/report/analytic_cost_ledger.rml',
                      parser=cci_account_analytic_cost_ledger,
                      header=False)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

