# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
from report import report_sxw

class cci_account_lines_open_by_partner(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(cci_account_lines_open_by_partner, self).__init__(cr, uid, name, context=context)
        self.localcontext.update( {
            'time': time,
            'get_partners': self._get_partners,
            'get_lines': self._get_lines,
            'get_title': self._get_title,
        })
    def _get_partners(self, form, objects):
        final = []
        account_obj = self.pool.get('account.account')
        if form['type'] in ['receivable','payable']:
            account_ids = account_obj.search(self.cr,self.uid,[('type','=',form['type'])])
        else:
            account_ids = account_obj.search(self.cr,self.uid,[('type','in',['receivable','payable'])])
        if account_ids:
            sentence = '''SELECT id, name FROM res_partner WHERE id in (
                                SELECT partner_id
                                    FROM account_move_line
                                    WHERE reconcile_id is null AND state = 'valid' AND account_id in (%s)
                                    GROUP BY partner_id
                              ) ORDER BY name''' % ','.join([str(x) for x in account_ids])
            self.cr.execute(sentence)
            res = self.cr.fetchall()
            mline_obj = self.pool.get('account.move.line')
            for rline in res:
                mline_ids = mline_obj.search(self.cr,self.uid,[('account_id','in',account_ids),('partner_id','=',rline[0]),('state','=','valid'),('reconcile_id','=',False)])
                sum_debit = 0.0
                sum_credit = 0.0
                if mline_ids:
                    mlines = mline_obj.read(self.cr,self.uid,mline_ids,['debit','credit'])
                    for mline in mlines:
                        sum_debit += mline['debit'] or 0.0
                        sum_credit += mline['credit'] or 0.0
                final.append({'id':rline[0],'name':rline[1],'line_ids':mline_ids,'sum_debit':'%.2f' % sum_debit,'sum_credit':'%.2f' % sum_credit,'balance':'%.2f' % (sum_debit-sum_credit)})
        return final
    def _get_lines(self, line_ids):
        result = []
        mlines = self.pool.get('account.move.line').browse(self.cr,self.uid,line_ids)
        for mline in mlines:
            record = {}
            record['move_name'] = mline.move_id and mline.move_id.name or '-'
            record['journal'] = mline.journal_id and mline.journal_id.code or '-'
            record['date'] = mline.date[8:10]+'/'+mline.date[5:7]+'/'+mline.date[0:4]
            record['period'] = mline.period_id.name
            record['debit'] = '%.2f' % (mline.debit or 0.0)
            record['credit'] = '%.2f' % (mline.credit or 0.0)
            record['reconcile'] = mline.reconcile_id and mline.reconcile_id.name or ''
            result.append(record)
        return result
    def _get_title(self, form, partner_id, partner_name):
        result = 'Etat de compte ' + ( form['type'] == 'receivable' and 'Client' or (form['type'] == 'payable' and 'Fournisseur' or 'Client-Fournisseur') )
        result += ( '\n' + partner_name + ' [%s]' % str(partner_id) )
        return result
report_sxw.report_sxw('report.open_lines_by_partner_report',
                      'account.move.line',
                      'addons/cci_account_extend/report/open_lines_by_partner.rml',
                      parser=cci_account_lines_open_by_partner,
                      header=False)
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

