# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2008 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import netsvc
import pooler
from tools.translate import _
from osv import fields, osv
from pyExcelerator import *
import datetime
import base64

form = """<?xml version="1.0"?>
<form string="Customer/Supplier Account Status List">
    <field name="type" colspan="4" width="650"/>
    <field name="sort" colspan="4" width="650"/>
    <field name="only_mature"/>
</form>
"""
fields = {
    'type':{'string':'Type','type':'selection','selection':[('receivable','Customer - Sales'),('payable','Supplier - Purchases'),('both','Partner - Sales and purchases')],'required':True,'default':'receivable'},
    'sort':{'string':'Sort','type':'selection','selection':[('partner_name','Partner Name'),('delay_days','Delay Days'),('line_date','Date'),('amount','Amount')],'required':True,'default':'partner_name'},
    'only_mature':{'string':'Only Mature','type':'boolean','default':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Results">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="open_lines_by_partner.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'open_lines_by_partner.xls':{'string': 'Prepared file','type': 'binary','readonly': True,},
}
def _get_number_of_days(date_from, date_to):
    DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
    from_dt = datetime.datetime.strptime(date_from, DATETIME_FORMAT)
    to_dt = datetime.datetime.strptime(date_to, DATETIME_FORMAT)
    timedelta = to_dt - from_dt
    diff_day = timedelta.days + float(timedelta.seconds) / 86400
    return diff_day
def french_date(english_date):
    if english_date and len(english_date) == 10:
        return english_date[8:]+'/'+english_date[5:7]+'/'+english_date[0:4]
    else:
        return ''
        
class open_lines_by_partner_excel(wizard.interface):
    def _export_lines(self, cr, uid, data, context):
        # create the workbook
        wb = Workbook()
        ws1 = wb.add_sheet(_('Lines'))
        ws1.write(0,0,_('ID Line'))
        ws1.write(0,1,_('ID Partner'))
        ws1.write(0,2,_('Partner Name'))
        ws1.write(0,3,_('Move Name'))
        ws1.write(0,4,_('Journal'))
        ws1.write(0,5,_('Line Name'))
        ws1.write(0,6,_('Date'))
        ws1.write(0,7,_('Maturity Date'))
        ws1.write(0,8,_('Period'))
        ws1.write(0,9,_('Debit'))
        ws1.write(0,10,_('Credit'))
        ws1.write(0,11,_('CCI Amount'))
        ws1.write(0,12,_('Partial Reconcile'))
        ws1.write(0,13,_('Delay days'))
        linenum = 1
        account_obj = pooler.get_pool(cr.dbname).get('account.account')
        if data['form']['type'] in ['receivable','payable']:
            account_ids = account_obj.search(cr,uid,[('type','=',data['form']['type'])])
        else:
            account_ids = account_obj.search(cr,uid,[('type','in',['receivable','payable'])])
        if account_ids:
            amline_obj = pooler.get_pool(cr.dbname).get('account.move.line')
            amline_ids = amline_obj.search(cr,uid,[('reconcile_id','=',False),('state','=','valid'),('account_id','in',account_ids)])
            now = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            today = datetime.date.today().strftime('%Y-%m-%d')
            if amline_ids:  
                amlines = amline_obj.browse(cr,uid,amline_ids)
                result = []
                for line in amlines:
                    line = {'line_id':line.id,
                            'partner_id':line.partner_id and line.partner_id.id or 0,
                            'partner_name':line.partner_id and line.partner_id.name or '',
                            'move_name':line.move_id.name or '',
                            'line_name':line.name or '',
                            'journal_code':line.journal_id and (line.journal_id.code or line.journal_id.name) or '',
                            'line_date':french_date(line.date or ''),
                            'sort_line_date':line.date or '',
                            'line_maturity':french_date(line.date_maturity or line.date or ''),
                            'sort_line_maturity':line.date_maturity or line.date or '',
                            'period_name':line.period_id.name,
                            'debit':line.debit or 0.0,
                            'credit':line.credit or 0.0,
                            'amount':(line.debit or 0.0) - (line.credit or 0.0),
                            'partial_reconcile':line.reconcile_partial_id and line.reconcile_partial_id.name or '',
                            'delay_days':int(_get_number_of_days((line.date_maturity or line.date) + ' 10:00:00',now)),
                           }
                    if line['line_maturity'] <= today or (not data['form']['only_mature']):
                        result.append(line)
                if result:
                    lines = sorted(result, key=lambda k: k[data['form']['sort']]) 
                    for line in lines:
                        ws1.write(linenum,0,line['line_id'])
                        ws1.write(linenum,1,line['partner_id'])
                        ws1.write(linenum,2,line['partner_name'])
                        ws1.write(linenum,3,line['move_name'])
                        ws1.write(linenum,4,line['line_name'])
                        ws1.write(linenum,5,line['journal_code'])
                        ws1.write(linenum,6,line['line_date'])
                        ws1.write(linenum,7,line['line_maturity'])
                        ws1.write(linenum,9,line['debit'])
                        ws1.write(linenum,10,line['credit'])
                        ws1.write(linenum,11,line['amount'])
                        ws1.write(linenum,12,line['partial_reconcile'])
                        ws1.write(linenum,13,line['delay_days'])
                        linenum += 1
        wb.save('open_lines_by_partner.xls')
        result_file = open('open_lines_by_partner.xls','rb').read()
        # give the result tos the user
        data['form']['msg']="File created 'open_lines_by_partner.xls'"
        data['form']['open_lines_by_partner.xls']=base64.encodestring(result_file)
        return data['form']
    states = {
        'init' : {
           'actions' : [],
           'result': {'type': 'form', 'arch': form, 'fields': fields, 'state':[('end','Cancel'), ('export','Export in Excel')]}
        },
        'export': {
            'actions': [_export_lines],
            'result' : {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }
open_lines_by_partner_excel("cci_open_move_line_by_partner_excel")
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
