# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import datetime
from osv import fields,osv
import wizard
import pooler
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Search strange moves between two periods">
    <field name="period_from"/>
    <field name="period_to"/>
    <field name="only_vat"/>
    <field name="only_valid"/>
</form>"""

fields = {
    'period_from' : {'string':'From','type':'many2one','relation':'account.period','required':True},
    'period_to': {'string':'To','type':'many2one','relation':'account.period','required':True},
    'only_vat':{'string':'Only lines concerning VAT','type':'boolean','default':True},
    'only_valid':{'string':'Only valid lines and moves','type':'boolean','default':True},
}

class search_write_date_after_vat(wizard.interface):
    def _search_lines(self, cr, uid, data, context):
        pool_obj = pooler.get_pool(cr.dbname)
        # extract the concerned list of periods
        from_period = pool_obj.get('account.period').read(cr,uid,[data['form']['period_from']],['date_start','date_stop'])[0]
        to_period = pool_obj.get('account.period').read(cr,uid,[data['form']['period_to']],['date_start','date_stop'])[0]
        from_date = min(from_period['date_start'],to_period['date_start'])
        to_date = max(from_period['date_stop'],to_period['date_stop'])
        period_ids = pool_obj.get('account.period').search(cr,uid,[('date_start','>=',from_date),('date_stop','<=',to_date),('special','=',False)])
        if period_ids:
            periods = pool_obj.get('account.period').read(cr,uid,period_ids,['id','date_start','vat_last_time'])
            final_ids = []
            for period in periods:
                if not period['vat_last_time']:
                    # if no vat_last_time recorded, we assume it's last possible day
                    if int(period['date_start'][5:7]) == 12:
                        period['vat_last_time'] = str(int(period['date_start'])+1)+'-01-20 16:00:00'
                    else:
                        period['vat_last_time'] = period['date_start'][0:5] + str(int(period['date_start'][5:7])+1).rjust(2,'0')+'-20 16:00:00'
                if data['form']['only_vat']:
                    complement_sentence = 'AND aml.tax_code_id IS NOT NULL'
                else:
                    complement_sentence = ''
                if not data['form']['only_valid']:
                    # we search on write_date, so we must use the SQL directly
                    sentence = """SELECT id
                                      FROM account_move_line as aml
                                      WHERE period_id = %s AND ( write_date < '%s' OR write_date > '%s' ) %s"""
                    sentence = sentence % (str(period['id']),period['date_start']+' 00:00:01',period['vat_last_time'],complement_sentence)
                else:
                    # we search on write_date, so we must use the SQL directly
                    sentence = """SELECT aml.id
                                      FROM account_move_line as aml, account_move as am
                                      WHERE aml.move_id = am.id AND aml.state = 'valid' AND am.state = 'posted' AND
                                            aml.period_id = %s AND (aml.write_date < '%s' OR aml.write_date > '%s' ) %s"""
                    sentence = sentence % (str(period['id']),period['date_start']+' 00:00:01',period['vat_last_time'],complement_sentence)
                cr.execute(sentence)
                res = cr.fetchall()
                for line in res:
                    final_ids.append(line[0])
        # show the result
        model_data_ids = pool_obj.get('ir.model.data').search(cr,uid,[('model','=','ir.ui.view'),('name','=','view_move_line_tree_taxes')])
        resource_id = pool_obj.get('ir.model.data').read(cr,uid,model_data_ids,fields=['res_id'])[0]['res_id']
        value = {
            'domain': [('id','in',final_ids)],
            'name': _('Strange Move Lines'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'account.move.line',
            'context': {},
            'views': [(resource_id,'tree'),(False,'form')],
            'type': 'ir.actions.act_window',
            }
        return value
    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel','gtk-stop'),('search','Search','gtk-find',True)]},
        },
        'search': {
            'actions': [],
            'result': {'type':'action','action':_search_lines,'state':'end'}
        },
    }
search_write_date_after_vat("search_write_date_after_vat")
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
