# -*- coding: utf-8 -*-
##############################################################################
import wizard
import time
import datetime
import pooler
import base64
from osv import fields, osv
from tools.translate import _
from pyExcelerator import *

form = """<?xml version="1.0"?>
<form string="CCI VAT Concordance">
    <field name="period_from"/>
    <field name="period_to"/>
    <newline/>
    <field name="accounts_list" colspan="4"/>
    <field name="vat_cases_list" colspan="4"/>
    <field name="sales_journals" colspan="4"/>
</form>"""

fields = {
    'accounts_list': {'string':'Accounts','type':'char','size':250,'required':True,'default':'70,493010,493011,493012,493013,493014,493021,493022,493031,493032,493033,493034,493035,493100,469000','help':'List of accounts separated by commas and you can give only the beginning of all accounts codes'},
    'special_account_ids': {'string':'Special Accounts','type':'char','size':250,'required':True,'default':'612400,601100','help':'List of special accounts separated by commas and you can give only the beginning of all accounts codes. A special account is an account not concerned by sales vat cases.'},
    'vat_cases_list': {'string':'Taxes Codes','type':'char','size':250,'required':True,'default':'00,01,02,03,44,45,46,47,48,49','help':'List of taxes codes numbers separated by commas'},
    'sales_journals': {'string':'Sales Journals','type':'char','size':'250','required':True,'default':'VEN,NCV'},
    'period_from': {'string':'From Period','type':'many2one','relation':'account.period','required':True},
    'period_to': {'string':'To Period','type':'many2one','relation':'account.period','required':True},
    }

msg_form = """<?xml version="1.0"?>
<form string="Notification">
    <separator string="XLS File has been Created."  colspan="4"/>
    <field name="vat_concordance.xls" colspan="4"/>
</form>"""
msg_fields = {
  'vat_concordance.xls':{'string': 'Save XLS File','type': 'binary','readonly': True,},
}

def num_to_letter(line_number_in_pyexcel):
    # this method convert an integer to a line label in Excel
    # first line = 0 (beware on pyExcelerator, first line = 0)
    # 0 => 'A'
    # 26 => 'AA'
    # 27 => 'AB'
    line_number_in_pyexcel += 1
    if line_number_in_pyexcel < 26:
        result = chr(64+line_number_in_pyexcel)
    else:
        first = line_number_in_pyexcel_number_in_pyexcel // 26
        last = number % 26
        result = chr(64+first)+chr(64+last)
    return result
def full_cell(line_number_in_pyexcel,col_number_in_pyexcel,fixed='none'):
    # fixed = none, col, line, both
    result = ''
    if fixed in ('col','both'):
        result += '$'
    result += num_to_letter(col_number_in_pyexcel)
    if fixed in ('line','both'):
        result += '$'
    result += str(line_number_in_pyexcel+1)
    return result

class vat_concordance_yearly(wizard.interface):

    def _create_xls(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)

        period_obj = pool.get('account.period')
        account_obj = pool.get('account.account')
        tax_code_obj = pool.get('account.tax.code')
        journal_obj = pool.get('account.journal')
        account_move_obj = pool.get('account.move')
        account_move_line_obj = pool.get('account.move.line')
        #
        from_period = period_obj.browse(cr,uid,data['form']['period_from'])
        to_period = period_obj.browse(cr,uid,data['form']['period_to'])
        period_ids = period_obj.search(cr,uid,[('date_start','>=',from_period.date_start),('date_stop','<=',to_period.date_stop),('special','=',False)])
        periods = period_obj.read(cr,uid,period_ids,['name'])
        dPeriods = {}
        for per in periods:
            dPeriods[per['id']] = per
        #
        laccounts = data['form']['accounts_list'].split(',')
        laccounts = [x.strip() for x in laccounts]
        selected_account_ids = []
        for account_code_beginning in laccounts:
            account_ids = account_obj.search(cr,uid,[('code','ilike',account_code_beginning)])
            if account_ids:
                selected_account_ids.extend(account_ids)
        selected_accounts = account_obj.read(cr,uid,selected_account_ids,['code'])
        dAccount2Codes = {}
        dAccount2IDs = {}
        lAccounts = []
        for account in selected_accounts:
            if len(account['code'].strip()) >= 6:
                dAccount2Codes[account['id']] = account['code']
                dAccount2IDs[account['code']] = account['id']
                lAccounts.append(account['code'])
        sAccounts = sorted(lAccounts)
        sales_vat_account_ids = account_obj.search(cr,uid,[('code','=','451000')])
        #
        lcases = data['form']['vat_cases_list'].split(',')
        lcases = [x.strip() for x in lcases]
        selected_tax_code_ids = []
        for tax_case in lcases:
            tax_code_ids = tax_code_obj.search(cr,uid,[('code','=',tax_case)])
            if tax_code_ids:
                selected_tax_code_ids.extend(tax_code_ids)
        tax_codes = tax_code_obj.read(cr,uid,selected_tax_code_ids,['id','code','name','sign'])
        dTaxCodes = {}
        dTaxIDs = {}
        for tax_code in tax_codes:
            dTaxCodes[tax_code['id']] = tax_code
            dTaxIDs[tax_code['code']] = tax_code['id']
        #
        ljournals = data['form']['sales_journals'].split(',')
        ljournals = [x.strip() for x in ljournals]
        journal_ids = journal_obj.search(cr,uid,[('code','in',ljournals)])
        journals = journal_obj.read(cr,uid,journal_ids,['code','name','refund_journal'])
        dJournals = {}
        for jnl in journals:
            dJournals[jnl['id']] = jnl
        #
        wb = Workbook()
        ws1 = wb.add_sheet(_('Tableau principal'))
        ws1.write(0,0,_('Concordance TVA'))
        ws1.write(1,0,u'Date:')
        ws1.write(1,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws1.write(2,0,_(u'From'))
        ws1.write(2,1,from_period.name)
        ws1.write(3,0,_(u'To'))
        ws1.write(3,1,to_period.name)
        ws1.write(4,0,_(u'Accounts List'))
        ws1.write(4,1,data['form']['accounts_list'])
        ws1.write(5,0,_(u'Taxes Codes List'))
        ws1.write(5,1,data['form']['vat_cases_list'])
        ws1.write(6,0,_(u'Sales Journals'))
        ws1.write(6,1,data['form']['sales_journals'])

        ws1.write(8,0,u'Déclaration TVA')
        ws1.write(9,0,u'Case TVA')
        ws1.write(9,1,u'Recalculée')
        start_cases = 10
        line_cases = start_cases
        for case in lcases:
            ws1.write(line_cases,0,case)
            selection = '''SELECT sum(credit),sum(debit) FROM account_move_line WHERE period_id in (%s) and tax_code_id = %i and state = 'valid' and move_id in (SELECT id from account_move where period_id in (%s) and state = 'posted')''' % (','.join([str(x) for x in period_ids]),dTaxIDs[case],','.join([str(x) for x in period_ids]))
            cr.execute(selection)
            res = cr.fetchall()
            ws1.write(line_cases,1,(res[0][0] or 0.0)-(res[0][1] or 0.0))
            line_cases += 1
        ws1.write(line_cases,1,Formula("SUM(%s:%s)" % (full_cell(start_cases,1),full_cell(line_cases-1,1))))
        line_cases += 1
        
        start_sums = line_cases
        ws1.write(8,3,u'Comptes de résultat (ne tient pas compte de la case TVA)')
        ws1.write(9,3,u'Compte')
        ws1.write(9,4,u'Total')
        ws1.write(9,5,u'VEN avec TVA')
        ws1.write(9,6,u'NCV avec TVA')
        ws1.write(9,7,u'Lissages')
        ws1.write(9,8,u'Autres (caisses, JOD, ...)')
        ws1.write(9,9,u'Sous-total')
        line_accounts = start_cases
        for account_code in sAccounts:
            if account_code[0] in ['6','7']:
                ws1.write(line_accounts,3,account_code)
                #
                selection = '''SELECT sum(credit),sum(debit) FROM account_move_line WHERE period_id in (%s) AND account_id = %i AND state = 'valid' AND move_id in (SELECT id from account_move where period_id in (%s) and state = 'posted')''' % (','.join([str(x) for x in period_ids]),dAccount2IDs[account_code],','.join([str(x) for x in period_ids]))
                cr.execute(selection)
                res = cr.fetchall()
                ws1.write(line_accounts,4,(res[0][0] or 0.0)-(res[0][1] or 0.0))
                #
                selection = '''SELECT sum(credit),sum(debit) FROM account_move_line WHERE period_id in (%s) AND account_id = %i AND journal_id = 19 AND tax_code_id in (%s) AND state = 'valid' AND move_id in (SELECT id from account_move where period_id in (%s) AND state = 'posted')''' % (','.join([str(x) for x in period_ids]),dAccount2IDs[account_code],','.join([str(x) for x in selected_tax_code_ids]),','.join([str(x) for x in period_ids]))
                cr.execute(selection)
                res = cr.fetchall()
                ws1.write(line_accounts,5,(res[0][0] or 0.0)-(res[0][1] or 0.0))
                #
                selection = '''SELECT sum(credit),sum(debit) FROM account_move_line WHERE period_id in (%s) AND account_id = %i AND journal_id = 15 AND tax_code_id in (%s) AND state = 'valid' AND move_id in (SELECT id from account_move where period_id in (%s) AND state = 'posted')''' % (','.join([str(x) for x in period_ids]),dAccount2IDs[account_code],','.join([str(x) for x in selected_tax_code_ids]),','.join([str(x) for x in period_ids]))
                cr.execute(selection)
                res = cr.fetchall()
                ws1.write(line_accounts,6,(res[0][0] or 0.0)-(res[0][1] or 0.0))
                #
                selection = '''SELECT sum(credit),sum(debit) FROM account_move_line WHERE period_id in (%s) AND account_id = %i AND journal_id = 32 AND state = 'valid' AND move_id in (SELECT id from account_move where period_id in (%s) AND state = 'posted')''' % (','.join([str(x) for x in period_ids]),dAccount2IDs[account_code],','.join([str(x) for x in period_ids]))
                cr.execute(selection)
                res = cr.fetchall()
                ws1.write(line_accounts,7,(res[0][0] or 0.0)-(res[0][1] or 0.0))
                #
                selection = '''SELECT sum(credit),sum(debit) FROM account_move_line WHERE period_id in (%s) AND account_id = %i AND journal_id not in (15,19,32) AND state = 'valid' AND move_id in (SELECT id from account_move where period_id in (%s) AND state = 'posted')''' % (','.join([str(x) for x in period_ids]),dAccount2IDs[account_code],','.join([str(x) for x in period_ids]))
                cr.execute(selection)
                res = cr.fetchall()
                ws1.write(line_accounts,8,(res[0][0] or 0.0)-(res[0][1] or 0.0))
                ws1.write(line_accounts,9,Formula('SUM(%s:%s)' % (full_cell(line_accounts,5),full_cell(line_accounts,8)) ))
                line_accounts += 1
        if line_accounts > start_cases:
            ws1.write(line_accounts,4,Formula("SUM(%s:%s)" % (full_cell(start_cases,4),full_cell(line_accounts-1,4))))
            ws1.write(line_accounts,5,Formula("SUM(%s:%s)" % (full_cell(start_cases,5),full_cell(line_accounts-1,5))))
            ws1.write(line_accounts,6,Formula("SUM(%s:%s)" % (full_cell(start_cases,6),full_cell(line_accounts-1,6))))
            ws1.write(line_accounts,7,Formula("SUM(%s:%s)" % (full_cell(start_cases,7),full_cell(line_accounts-1,7))))
            ws1.write(line_accounts,8,Formula("SUM(%s:%s)" % (full_cell(start_cases,8),full_cell(line_accounts-1,8))))
            ws1.write(line_accounts,9,Formula("SUM(%s:%s)" % (full_cell(start_cases,9),full_cell(line_accounts-1,9))))
            line_accounts += 1
            ws1.write(line_accounts,8,Formula("SUM(%s:%s)" % (full_cell(line_accounts-1,5),full_cell(line_accounts-1,8))))
            line_accounts += 1
        start_sums = max(start_sums,line_accounts)

        ws1.write(8,11,u'Bilan (ne tient pas compte de la case TVA)')
        ws1.write(9,12,u'Compte')
        ws1.write(9,13,u'Total')
        ws1.write(9,14,u'VEN avec TVA')
        ws1.write(9,15,u'NCV avec TVA')
        ws1.write(9,16,u'Lissages')
        ws1.write(9,17,u'Autres (caisses, JOD, ...)')
        ws1.write(9,19,u'Sous-total')
        line_saccounts = start_cases
        for account_code in sAccounts:
            if account_code[0] not in ['6','7']:
                ws1.write(line_saccounts,12,account_code)
                #
                selection = '''SELECT sum(credit),sum(debit) FROM account_move_line WHERE period_id in (%s) AND account_id = %i AND state = 'valid' AND move_id in (SELECT id from account_move where period_id in (%s) and state = 'posted')''' % (','.join([str(x) for x in period_ids]),dAccount2IDs[account_code],','.join([str(x) for x in period_ids]))
                cr.execute(selection)
                res = cr.fetchall()
                ws1.write(line_saccounts,13,(res[0][0] or 0.0)-(res[0][1] or 0.0))
                #
                selection = '''SELECT sum(credit),sum(debit) FROM account_move_line WHERE period_id in (%s) AND account_id = %i AND journal_id = 19 AND tax_code_id in (%s) AND state = 'valid' AND move_id in (SELECT id from account_move where period_id in (%s) AND state = 'posted')''' % (','.join([str(x) for x in period_ids]),dAccount2IDs[account_code],','.join([str(x) for x in selected_tax_code_ids]),','.join([str(x) for x in period_ids]))
                cr.execute(selection)
                res = cr.fetchall()
                ws1.write(line_saccounts,14,(res[0][0] or 0.0)-(res[0][1] or 0.0))
                #
                selection = '''SELECT sum(credit),sum(debit) FROM account_move_line WHERE period_id in (%s) AND account_id = %i AND journal_id = 15 AND tax_code_id in (%s) AND state = 'valid' AND move_id in (SELECT id from account_move where period_id in (%s) AND state = 'posted')''' % (','.join([str(x) for x in period_ids]),dAccount2IDs[account_code],','.join([str(x) for x in selected_tax_code_ids]),','.join([str(x) for x in period_ids]))
                cr.execute(selection)
                res = cr.fetchall()
                ws1.write(line_saccounts,15,(res[0][0] or 0.0)-(res[0][1] or 0.0))
                #
                selection = '''SELECT sum(credit),sum(debit) FROM account_move_line WHERE period_id in (%s) AND account_id = %i AND journal_id = 32 AND state = 'valid' AND move_id in (SELECT id from account_move where period_id in (%s) AND state = 'posted')''' % (','.join([str(x) for x in period_ids]),dAccount2IDs[account_code],','.join([str(x) for x in period_ids]))
                cr.execute(selection)
                res = cr.fetchall()
                ws1.write(line_saccounts,16,(res[0][0] or 0.0)-(res[0][1] or 0.0))
                #
                selection = '''SELECT sum(credit),sum(debit) FROM account_move_line WHERE period_id in (%s) AND account_id = %i AND journal_id not in (15,19,32) AND state = 'valid' AND move_id in (SELECT id from account_move where period_id in (%s) AND state = 'posted')''' % (','.join([str(x) for x in period_ids]),dAccount2IDs[account_code],','.join([str(x) for x in period_ids]))
                cr.execute(selection)
                res = cr.fetchall()
                ws1.write(line_saccounts,17,(res[0][0] or 0.0)-(res[0][1] or 0.0))
                ws1.write(line_saccounts,18,Formula('SUM(%s:%s)' % (full_cell(line_accounts,5),full_cell(line_accounts,8)) ))
                line_saccounts += 1
        if line_saccounts > start_cases:
            ws1.write(line_saccounts,13,Formula("SUM(%s:%s)" % (full_cell(start_cases,13),full_cell(line_saccounts-1,13))))
            ws1.write(line_saccounts,14,Formula("SUM(%s:%s)" % (full_cell(start_cases,14),full_cell(line_saccounts-1,14))))
            ws1.write(line_saccounts,15,Formula("SUM(%s:%s)" % (full_cell(start_cases,15),full_cell(line_saccounts-1,15))))
            ws1.write(line_saccounts,16,Formula("SUM(%s:%s)" % (full_cell(start_cases,16),full_cell(line_saccounts-1,16))))
            ws1.write(line_saccounts,17,Formula("SUM(%s:%s)" % (full_cell(start_cases,17),full_cell(line_saccounts-1,17))))
            ws1.write(line_saccounts,18,Formula("SUM(%s:%s)" % (full_cell(start_cases,18),full_cell(line_saccounts-1,18))))
            line_saccounts += 1
            ws1.write(line_saccounts,17,Formula("SUM(%s:%s)" % (full_cell(line_saccounts-1,14),full_cell(line_saccounts-1,17))))
            line_saccounts += 1
        start_sums = max(start_sums,line_saccounts)

        start_sums += 1
        line_sums = start_sums
        ws1.write(line_sums,0,u'Sommes des lignes qui devraient être avec TVA')
        line_sums += 1
        ws1.write(line_sums,0,u'Directes (comptes de résultat)')
        ws1.write(line_sums,1,Formula("%s-%s-%s" % (full_cell(line_accounts-2,4),full_cell(line_accounts-2,7),full_cell(line_accounts-2,8))))
        line_sums += 1
        ws1.write(line_sums,0,u'Lissées (et autres comptes 4 normaux)')
        ws1.write(line_sums,1,Formula("%s+%s" % (full_cell(line_saccounts-2,14),full_cell(line_saccounts-2,15))))
        line_sums += 1
        ws1.write(line_sums,0,u'Total NORMAL')
        ws1.write(line_sums,1,Formula("%s+%s" % (full_cell(line_sums-2,1),full_cell(line_sums-1,1))))
        ws1.write(line_sums,2,Formula("%s-%s" % (full_cell(line_sums,1),full_cell(line_cases-1,1))))
        #
        ws2 = wb.add_sheet(_(u'Erreur 1'))
        ws2.write(0,0,_('Concordance TVA Erreurs 1 - VEN/NCV sans Case TVA'))
        ws2.write(1,0,'Date:')
        ws2.write(1,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws2.write(3,0,u'line ID')
        ws2.write(3,1,u'Case TVA')
        ws2.write(3,2,u'Compte')
        ws2.write(3,3,u'Journal')
        ws2.write(3,4,u'Ecriture')
        ws2.write(3,5,u'Montant')
        ws2.write(3,6,u'Description ligne')
        ws2_line = 4
        error1_sum = 0.0
        selection = '''SELECT id FROM account_move_line WHERE period_id in (%s) AND journal_id in (15,19) AND tax_code_id IS NULL AND account_id in (%s) AND state = 'valid' AND move_id in (SELECT id from account_move where period_id in (%s) AND state = 'posted')''' % (','.join([str(x) for x in period_ids]),','.join([str(x) for x in selected_account_ids]),','.join([str(x) for x in period_ids]))
        cr.execute(selection)
        res = cr.fetchall()
        if res:
            amline_ids = []
            for line in res:
                amline_ids.append(line[0])
            index = 0
            size = 500
            while amline_ids[index:index+size]:
                amlines = account_move_line_obj.browse(cr,uid,amline_ids[index:index+size])
                for amline in amlines:
                    ws2.write(ws2_line,0,amline.id)
                    ws2.write(ws2_line,1,amline.tax_code_id and amline.tax_code_id.code or '')
                    ws2.write(ws2_line,2,amline.account_id and amline.account_id.code or '')
                    ws2.write(ws2_line,3,amline.journal_id and amline.journal_id.code or '')
                    ws2.write(ws2_line,4,amline.move_id.ref or amline.move_id.name or '')
                    ws2.write(ws2_line,5,(amline.credit or 0.0)-(amline.debit or 0.0))
                    ws2.write(ws2_line,6,amline.name or '')
                    error1_sum += (amline.credit or 0.0)-(amline.debit or 0.0)
                    ws2_line += 1
                index += size
        line_sums += 1
        ws1.write(line_sums,0,u'Erreurs 1')
        ws1.write(line_sums,1,error1_sum)
        ws1.write(line_sums,2,u'VEN/NCV sans Case TVA')
        
        #
        ws3 = wb.add_sheet(_(u'Erreur 2'))
        ws3.write(0,0,_('Concordance TVA Erreurs 2 - VEN/NCV avec Case TVA non Ventes'))
        ws3.write(1,0,'Date:')
        ws3.write(1,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws3.write(3,0,u'line ID')
        ws3.write(3,1,u'Case TVA')
        ws3.write(3,2,u'Compte')
        ws3.write(3,3,u'Journal')
        ws3.write(3,4,u'Ecriture')
        ws3.write(3,5,u'Montant')
        ws3.write(3,6,u'Description ligne')
        ws3_line = 4
        error2_sum = 0.0
        selection = '''SELECT id FROM account_move_line WHERE period_id in (%s) AND journal_id in (15,19) AND tax_code_id NOT IN (%s) AND account_id not in (%s) AND state = 'valid' AND move_id in (SELECT id from account_move where period_id in (%s) AND state = 'posted')''' % (','.join([str(x) for x in period_ids]),','.join([str(x) for x in selected_tax_code_ids]),','.join([str(x) for x in sales_vat_account_ids]),','.join([str(x) for x in period_ids]))
        cr.execute(selection)
        res = cr.fetchall()
        if res:
            amline_ids = []
            for line in res:
                amline_ids.append(line[0])
            index = 0
            size = 500
            while amline_ids[index:index+size]:
                amlines = account_move_line_obj.browse(cr,uid,amline_ids[index:index+size])
                for amline in amlines:
                    ws3.write(ws3_line,0,amline.id)
                    ws3.write(ws3_line,1,amline.tax_code_id and amline.tax_code_id.code or '')
                    ws3.write(ws3_line,2,amline.account_id and amline.account_id.code or '')
                    ws3.write(ws3_line,3,amline.journal_id and amline.journal_id.code or '')
                    ws3.write(ws3_line,4,amline.move_id.ref or amline.move_id.name or '')
                    ws3.write(ws3_line,5,(amline.credit or 0.0)-(amline.debit or 0.0))
                    ws3.write(ws3_line,6,amline.name or '')
                    error2_sum += (amline.credit or 0.0)-(amline.debit or 0.0)
                    ws3_line += 1
                index += size
        line_sums += 1
        ws1.write(line_sums,0,u'Erreurs 2')
        ws1.write(line_sums,1,error2_sum)
        ws1.write(line_sums,2,u'VEN/NCV avec Case TVA non Ventes')
        #
        error3_sum = 0.0
        ws4 = wb.add_sheet(u'Erreur 3')
        ws4.write(0,0,u'Concordance TVA - Erreur 3 - Lignes sur Case TVA de Ventes mais pas journaux de ventes')
        ws4.write(1,0,'Date:')
        ws4.write(1,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws4.write(3,0,u'line ID')
        ws4.write(3,1,u'Case TVA')
        ws4.write(3,2,u'Compte')
        ws4.write(3,3,u'Journal')
        ws4.write(3,4,u'Ecriture')
        ws4.write(3,5,u'Montant')
        ws4.write(3,6,u'Description ligne')
        ws4_line = 4
        amline_ids = account_move_line_obj.search(cr,uid,[('period_id','in',period_ids),('tax_code_id','in',selected_tax_code_ids),('journal_id','not in',journal_ids)])
        if amline_ids:
            amlines = account_move_line_obj.browse(cr,uid,amline_ids)
            for amline in amlines:
                ws4.write(ws4_line,0,amline.id)
                ws4.write(ws4_line,1,amline.tax_code_id and amline.tax_code_id.code or '')
                ws4.write(ws4_line,2,amline.account_id and amline.account_id.code or '')
                ws4.write(ws4_line,3,amline.journal_id and amline.journal_id.code or '')
                ws4.write(ws4_line,4,amline.move_id.ref or amline.move_id.name or '')
                ws4.write(ws4_line,5,(amline.credit or 0.0)-(amline.debit or 0.0))
                ws4.write(ws4_line,6,amline.name or '')
                error3_sum += (amline.credit or 0.0)-(amline.debit or 0.0)
                ws4_line += 1
        line_sums += 1
        ws1.write(line_sums,0,u'Erreurs 3')
        ws1.write(line_sums,1,error3_sum)
        ws1.write(line_sums,2,u'Lignes sur Case TVA de Ventes mais pas journaux de ventes')
        #
        error4_sum = 0.0
        ws5 = wb.add_sheet(u'Erreur 4')
        ws5.write(0,0,u'Concordance TVA - Erreur 4 - Lignes d\'écriture ou Ecritures Brouillon')
        ws5.write(1,0,'Date:')
        ws5.write(1,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws5.write(3,0,u'line ID')
        ws5.write(3,1,u'Case TVA')
        ws5.write(3,2,u'Compte')
        ws5.write(3,3,u'Journal')
        ws5.write(3,4,u'Ecriture')
        ws5.write(3,5,u'Montant')
        ws5.write(3,6,u'Description ligne')
        ws5.write(3,7,u'Raison')
        ws5_line = 4
        amline_ids = account_move_line_obj.search(cr,uid,[('period_id','in',period_ids),('state','=','draft')])
        move_ids = account_move_obj.search(cr,uid,[('period_id','in',period_ids),('state','=','draft')])
        if move_ids:
            supp_amline_ids = account_move_line_obj.search(cr,uid,[('move_id','in',move_ids)])
            for amline_id in supp_amline_ids:
                if amline_id not in amline_ids:
                    amline_ids.append(amline_id)
        if amline_ids:
            amlines = account_move_line_obj.browse(cr,uid,amline_ids)
            for amline in amlines:
                ws5.write(ws5_line,0,amline.id)
                ws5.write(ws5_line,1,amline.tax_code_id and amline.tax_code_id.code or '')
                ws5.write(ws5_line,2,amline.account_id and amline.account_id.code or '')
                ws5.write(ws5_line,3,amline.journal_id and amline.journal_id.code or '')
                ws5.write(ws5_line,4,amline.move_id.ref or amline.move_id.name or '')
                ws5.write(ws5_line,5,(amline.credit or 0.0)-(amline.debit or 0.0))
                ws5.write(ws5_line,6,amline.name or '')
                reasons = []
                if amline.state == 'draft':
                    reasons.append(u'Ligne d\'écriture non validée')
                if amline.move_id and amline.move_id.state == 'draft':
                    reasons.append(u'Ecriture toujours en brouillon')
                ws5.write(ws5_line,7,','.join(reasons))
                error4_sum += (amline.credit or 0.0)-(amline.debit or 0.0)
                ws5_line += 1
        line_sums += 1
        ws1.write(line_sums,0,u'Erreurs 4')
        ws1.write(line_sums,1,error4_sum)
        ws1.write(line_sums,2,u'Lignes d\'écriture ou Ecritures Brouillon')        
        #
        error5_sum = 0.0
        ws6 = wb.add_sheet(u'Erreur 5')
        ws6.write(0,0,u'Concordance TVA - Erreur 5 - NCV mais sur des cases TVA hors N.C. OU VEN sur cases TVA de type N.C.')
        ws6.write(1,0,'Date:')
        ws6.write(1,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws6.write(3,0,u'line ID')
        ws6.write(3,1,u'Case TVA')
        ws6.write(3,2,u'Compte')
        ws6.write(3,3,u'Journal')
        ws6.write(3,4,u'Ecriture')
        ws6.write(3,5,u'Montant')
        ws6.write(3,6,u'Description ligne')
        ws6_line = 4
        # sum by period and case all lines in the periods and accounts
        amline_ids = account_move_line_obj.search(cr,uid,[('period_id','in',period_ids),('tax_code_id','in',selected_tax_code_ids),('journal_id','in',[15,19]),('account_id','in',selected_account_ids)])
        if amline_ids:
            index = 0
            size = 500
            while amline_ids[index:index+size]:
                amlines = account_move_line_obj.browse(cr,uid,amline_ids[index:index+size])
                for amline in amlines:
                    if (amline.journal_id.refund_journal and amline.tax_code_id.sign == 1) or ((not amline.journal_id.refund_journal) and (amline.tax_code_id.sign == -1)):
                        ws6.write(ws6_line,0,amline.id)
                        ws6.write(ws6_line,1,amline.tax_code_id and amline.tax_code_id.code or '')
                        ws6.write(ws6_line,2,amline.account_id and amline.account_id.code or '')
                        ws6.write(ws6_line,3,amline.journal_id and amline.journal_id.code or '')
                        ws6.write(ws6_line,4,amline.move_id.ref or amline.move_id.name or '')
                        ws6.write(ws6_line,5,(amline.credit or 0.0)-(amline.debit or 0.0))
                        ws6.write(ws6_line,6,amline.name or '')
                        ws6_line += 1
                        error5_sum += (amline.credit or 0.0)-(amline.debit or 0.0)
                index += size
        line_sums += 1
        ws1.write(line_sums,0,u'Erreurs 5')
        ws1.write(line_sums,1,error5_sum)
        ws1.write(line_sums,2,u'NCV mais sur des cases TVA hors N.C. OU VEN sur cases TVA de type N.C.')
        wb.save('vat_concordance.xls')
        result_file = open('vat_concordance.xls','rb').read()

        # give the result tos the user
        data['form']['vat_concordance.xls']=base64.encodestring(result_file)
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type': 'form', 'arch':form, 'fields': fields, 'state':[('end','Cancel'),('go','Create XLS') ]}
                },
         'go': {
            'actions': [_create_xls],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]},
                }
             }
vat_concordance_yearly('vat_concordance_yearly')
