##############################################################################
#
# Copyright (c) 2005-2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import wizard
import time
import datetime
import pooler
import base64
from osv import fields, osv
from tools.translate import _
from pyExcelerator import *

form_intra = """<?xml version="1.0"?>
<form string="CCI Partner VAT Intra">
    <notebook>
    <page string="General Information">
    <label string="This wizard will create an XML file for Vat Intra" colspan="4"/>
    <newline/>
    <field name="period_code"/>
    <newline/>
    <field name="period_ids"/>
    <newline/>
    <field name="mand_id" help="This identifies the representative of the sending company. This is a string of 14 characters"/>
    <newline/>
    </page>
    <page string="European Countries">
    <field name="country_ids" colspan="4" nolabel="1" />
    </page>
    </notebook>
</form>"""
fields_intra = {
    'period_code': {'string':'Period Code','type':'char','size':'6','required': True, 'help': '''This is where you have to set the period code for the intracom declaration using the format: ppyyyy

  PP can stand for a month: from '01' to '12'.
  PP can stand for a trimester: '31','32','33','34'
      The first figure means that it is a trimester,
      The second figure identify the trimester.
  PP can stand for a complete fiscal year: '00'.
  YYYY stands for the year (4 positions).
'''
},
    'period_ids': {'string': 'Period(s)', 'type': 'many2many', 'relation': 'account.period', 'required': True, 'help': 'Select here the period(s) you want to include in your intracom declaration'},
    'mand_id':{'string':'MandataireId','type':'char','size':'14'},
    #'test_xml':{'string':'Test XML file','type':'boolean','help':'Set the XML output as test file'},
    'country_ids': {
        'string': 'European Countries',
        'type': 'many2many',
        'relation': 'res.country',
        'required': False
    },
   }

msg_form = """<?xml version="1.0"?>
<form string="Notification">
    <separator string="XML File has been Created."  colspan="4"/>
    <field name="msg" colspan="4" nolabel="1"/>
    <field name="vat_intra.xml" colspan="4"/>
    <field name="vat_intra.xls" colspan="4"/>
</form>"""
msg_fields = {
  'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
  'vat_intra.xml':{'string': 'Save XML File',
        'type': 'binary',
        'readonly': True,},
  'vat_intra.xls':{'string': 'Save XLS File',
        'type': 'binary',
        'readonly': True,},
}

class cci_partner_vat_intra(wizard.interface):

    def _get_europe_country(self, cr, uid, data, context):
        country_ids = pooler.get_pool(cr.dbname).get('res.country').search(cr, uid, [('code', 'in', ['AT', 'BG', 'CY', 'CZ', 'DK', 'EE', 'FI', 'FR', 'DE', 'GR', 'HU', 'IE', 'IT', 'LV', 'LT', 'LU', 'MT', 'NL', 'PL', 'PT', 'RO', 'SK', 'SI', 'ES', 'SE', 'GB','CZ'])])
        return {'country_ids': country_ids}

    def _create_xml(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)
        data_cmpny = pool.get('res.users').browse(cr, uid, uid).company_id
        company_vat = data_cmpny.partner_id.vat
        if not company_vat:
            raise wizard.except_wizard('Data Insufficient','No VAT Number Associated with Main Company!')

        seq_controlref = pool.get('ir.sequence').get(cr, uid,'controlref')
        seq_declarantnum = pool.get('ir.sequence').get(cr, uid,'declarantnum')
        cref = company_vat[2:] + seq_controlref[-4:]
        dnum = company_vat[2:] + seq_declarantnum[-4:]
        if len(data['form']['period_code']) != 6:
            raise wizard.except_wizard(_('Wrong Period Code'), _('The period code you entered is not valid.'))
        if data['form']['period_code'][:2] not in ('00','01','02','03','04','05','06','07','08','09','10','11','12','31','32','33','34'):
            raise wizard.except_wizard(_('Wrong Period Code'), _('The period code you entered is not valid. It must begin with 00, 01 to 12, or 31 to 34. Then four digits for year.'))
        if data['form']['period_code'][2:] < '1980' or data['form']['period_code'][2:] > '2099':
            raise wizard.except_wizard(_('Wrong Period Code'), _('The period code you entered is not valid. It must begin with 00, 01 to 12, or 31 to 34. Then four digits for year.'))

        street = zip_city = country = zip_code = city = email_addr = ''
        addr = pool.get('res.partner').address_get(cr, uid, [data_cmpny.partner_id.id], ['invoice'])
        if addr.get('invoice',False):
            ads = pool.get('res.partner.address').browse(cr,uid,[addr['invoice']])[0]
            if ads.zip_id:
                zip_code = ads.zip_id.name
                city = ads.zip_id.city
            else:
                zip_code = ads.zip
                city = ads.city
            zip_city = (city or '') + ' ' + (zip_code or '')
            if zip_city== ' ':
                zip_city = ''
            if ads.street:
                street = ads.street
            if ads.street2:
                street += ' '
                street += ads.street2
            if ads.country_id:
                country = ads.country_id.code
            email_addr = ads.email or ''
            phone_addr = ads.phone or ''

        month_quarter = data['form']['period_code'][:2]
        year = data['form']['period_code'][2:]
        sender_date = time.strftime('%Y-%m-%d')
        comp_name = data_cmpny.name

        data_head = u"""<?xml version="1.0" encoding="UTF-8"?>
<ns2:IntraConsignment xmlns="http://www.minfin.fgov.be/InputCommon" xmlns:ns2="http://www.minfin.fgov.be/IntraConsignment" IntraListingsNbr="1">
    <ns2:Representative>
        <RepresentativeID identificationType="NVAT" issuedBy="%s">%s</RepresentativeID>
        <Name>%s</Name>
        <Street>%s</Street>
        <PostCode>%s</PostCode>
        <City>%s</City>
        <CountryCode>%s</CountryCode>
        <EmailAddress>%s</EmailAddress>
        <Phone>%s</Phone>
    </ns2:Representative>""" % (company_vat[:2],company_vat[2:],comp_name,street,zip_code,city,country,email_addr,phone_addr)
        if data['form']['mand_id']:
            data_head += u'\n\t\t<ns2:RepresentativeReference>%s</ns2:RepresentativeReference>' % (data['form']['mand_id'])
        data_comp_period = u'\n\t\t<ns2:Declarant>\n\t\t\t<VATNumber>%s</VATNumber>\n\t\t\t<Name>%s</Name>\n\t\t\t<Street>%s</Street>\n\t\t\t<PostCode>%s</PostCode>\n\t\t\t<City>%s</City>\n\t\t\t<CountryCode>%s</CountryCode>\n\t\t\t<EmailAddress>%s</EmailAddress>\n\t\t\t<Phone>%s</Phone>\n\t\t</ns2:Declarant>' % (company_vat[2:],comp_name,street,zip_code,city,country,email_addr,phone_addr)
        if month_quarter.startswith('3'):
            data_comp_period += u'\n\t\t<ns2:Period>\n\t\t\t<ns2:Quarter>'+month_quarter[1]+u'</ns2:Quarter> \n\t\t\t<ns2:Year>'+year+u'</ns2:Year>\n\t\t</ns2:Period>'
        elif month_quarter.startswith('0') and month_quarter.endswith('0'):
            data_comp_period+= u'\n\t\t<ns2:Period>\n\t\t\t<ns2:Year>'+year+u'</ns2:Year>\n\t\t</ns2:Period>'
        else:
            data_comp_period += u'\n\t\t<ns2:Period>\n\t\t\t<ns2:Month>'+month_quarter+u'</ns2:Month>\n\t\t\t<ns2:Year>'+year+u'</ns2:Year>\n\t\t</ns2:Period>'

        wb = Workbook()
        ws1 = wb.add_sheet('Customers')
        ws1.write(0,0,'Errors')
        ws1.write(0,1,'VAT Number')
        ws1.write(0,2,'Parner Name')
        ws1.write(0,3,'Partner ID')
        ws1.write(0,4,'Amount')
        ws1.write(0,5,'Intra Code')

        error_message = []
        seq = 0
        line = 0
        amount_sum = 0

        #list_partner = []
        data_clientinfo = ''
        dPartners = {}
        cr.execute('''SELECT l.partner_id AS partner_id, p.vat AS vat, t.code AS tax_code, pp.cci_temp_intra_code as intra_code, l.tax_amount AS amount, p.name AS pname 
                      FROM account_move_line l 
                      LEFT JOIN account_tax_code t ON (l.tax_code_id = t.id) 
                      LEFT JOIN res_partner p ON (l.partner_id = p.id) 
                      LEFT JOIN product_product pp on (l.product_id = pp.id)
                      WHERE t.code IN ('44','46','48') 
                       AND l.period_id IN %s
                      ORDER by l.partner_id''', (tuple(data['form']['period_ids'][0][2]), ))
        no_vat = []
        for row in cr.dictfetchall():
            amt = row['amount'] or 0.0
            #amt = int(amt * 100)
            intra_code = 'S' ## by default for the CCI
            if row['tax_code'] == '44':
                intra_code = 'S'
            elif row['tax_code'] == '46':
                intra_code = 'L'
            elif row['tax_code'] == '48' and row['intra_code']:
                intra_code = row['intra_code']
                amt = amt * -1
            listing_key = (row['partner_id'],intra_code)
            if dPartners.has_key( listing_key ):
                sum_record = dPartners[ listing_key ]
            else:
                sum_record = {'vat':row['vat'],'amt':0.0,'partner_name':row['pname']}
            sum_record['amt'] += amt
            if row['vat']: # records without VAT will be ignored later, so we do not add the amount to the final amount
                amount_sum += amt
            dPartners[ listing_key ] = sum_record

        for key,record in dPartners.items():
            line += 1
            if record['vat']:
                seq += 1
                data_clientinfo += u'\n\t\t<ns2:IntraClient SequenceNumber="%s">\n\t\t\t<ns2:CompanyVATNumber issuedBy="%s">%s</ns2:CompanyVATNumber>\n\t\t\t<ns2:Code>%s</ns2:Code>\n\t\t\t<ns2:Amount>%.2f</ns2:Amount>\n\t\t</ns2:IntraClient>' % (str(seq),record['vat'][:2],record['vat'][2:],key[1],record['amt'])
            else:
                no_vat.append(u'%s [%s]' % (record['partner_name'],str(key[0])))
                ws1.write(line,0,'!!!!No VAT!!!!')
            ws1.write(line,1,record['vat'] or '')
            ws1.write(line,2,record['partner_name'] or '')
            ws1.write(line,3,key[0])
            ws1.write(line,4,record['amt'])
            ws1.write(line,5,key[1])

        data_decl = u'\n\t<ns2:IntraListing SequenceNumber="1" ClientsNbr="'+ str(seq) +u'" DeclarantReference="'+ dnum + u'" AmountSum="'+ u"%.2f" % amount_sum +u'" >'
        data_file = data_head + data_decl + data_comp_period + data_clientinfo + u'\n\t</ns2:IntraListing>\n</ns2:IntraConsignment>'
        if no_vat:
            data['form']['msg'] = u'There is a list of customers without VAT :\n\n - %s\n\nIt is DANGEROUS to save the File with '".xml"' extension and send it without VERIFYING it deeply.' % ('\n - '.join(no_vat))
        else:
            data['form']['msg'] = u'Save the File with '".xml"' extension.'
        data['form']['vat_intra.xml'] = base64.encodestring(data_file.encode('utf-8'))
        wb.save('intra_listing.xls')
        result_file = open('intra_listing.xls','rb').read()

        # give the result tos the user
        data['form']['vat_intra.xls']=base64.encodestring(result_file)
        return data['form']

    states = {
        'init': {
            'actions': [_get_europe_country],
            'result': {'type': 'form', 'arch':form_intra, 'fields': fields_intra, 'state':[('end','Cancel'),('go','Create XML') ]}
                },
         'go': {
            'actions': [_create_xml],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]},
                }
             }
cci_partner_vat_intra('cci.vat.intra.xml')
