# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; withclass department(osv.osv):
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv,fields
import datetime
import netsvc

class budget(osv.osv):
    _name = 'cci_budget.budget'
    _description = 'Budget'
    _columns = {
        'name' : fields.char('Name', size=64, required=True),
        'official': fields.boolean('Official'),
        'fiscalyear_id': fields.many2one('account.fiscalyear', 'Fiscal Year' ),
        'creating_user_id': fields.many2one('res.users','Responsible User'),
        'validating_user_id': fields.many2one('res.users','Validate User', readonly=True),
        'state' : fields.selection([('draft','Draft'),('confirm','Confirmed'),('validate','Validated'),('done','Done'),('cancel', 'Cancelled')], 'Status'),
        'comments': fields.text('Comments'),
        'active': fields.boolean('Active'),
    }
    _defaults = {
        'state': lambda *a:'draft',
        'creating_user_id': lambda self,cr,uid,context: uid,
        'active': lambda *a: True,
    }
    def unlink(self,cr,uid,ids,context):
        dotation_obj = self.pool.get('cci_budget.dotation')
        per_dot_obj = self.pool.get('cci_budget.period_dotation')
        dotation_ids = dotation_obj.search(cr,uid,[('budget_id','in',ids)])
        if dotation_ids:
            per_dot_ids = per_dot_obj.search(cr,uid,[('dotation_id','in',dotation_ids)])
            if per_dot_ids:
                per_dot_obj.unlink(cr,uid,per_dot_ids,context)
            dotation_obj.unlink(cr,uid,dotation_ids,context)
        return super(osv.osv,self).unlink(cr,uid,ids,context)
    def dotation_by_period_for_post(self,cr,uid,budget_id,post_id):
        # dResults = {(period_id,post_id,account_id):{'period_name','post_name','dpt_name','dpt_sequence','account_name','group_name','category_name','category_id','budget'}}
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        budget_obj = self.pool.get('cci_budget.budget')
        period_obj = self.pool.get('account.period')
        account_obj = self.pool.get('account.account')
        aaccount_obj = self.pool.get('account.analytic.account')
        amline_obj = self.pool.get('account.move.line')
        dotation_obj = self.pool.get('cci_budget.dotation')
        per_dot_obj = self.pool.get('cci_budget.period_dotation')
        post_obj = self.pool.get('cci_budget.post')

        # get the fiscalyear and periods linked to the given budget
        budget = budget_obj.browse(cr,uid,budget_id)
        post = post_obj.browse(cr,uid,post_id)
        period_ids = period_obj.search(cr,uid,[('fiscalyear_id','=',budget.fiscalyear_id.id),('special','=',False)])
        periods = period_obj.read(cr,uid,period_ids,['name','date_start','date_stop'])
        dPeriods = {}
        for period in periods:
            dPeriods[period['id']] = period
            
        # create the dResults from dotations
        dResults = {}
        account_ids = account_obj.search(cr,uid,[('cci_budget_group_id','!=',False)])
        accounts = account_obj.read(cr,uid,account_ids,['cci_budget_group_id','code','name'])
        saccounts = sorted(accounts, key=lambda k: k['code'])
        aaccount_ids = aaccount_obj.search(cr,uid,[('cci_post_id','=',post_id)])

        dotation_ids = dotation_obj.search(cr,uid,[('budget_id','=',budget_id),('post_id','=',post_id)])
        dotations = dotation_obj.browse(cr,uid,dotation_ids)
        dot_period_ids = per_dot_obj.search(cr,uid,[('dotation_id','in',dotation_ids)])
        dot_periods = per_dot_obj.browse(cr,uid,dot_period_ids)
        dResults = {}
        for dot in dotations:
            for period_id in period_ids:
                dResults[(period_id,post_id,dot.account_id.id)] = {'dpt_name': dot.post_id.department_id.name,
                                                                   'dpt_sequence': dot.post_id.department_id.sequence,
                                                                   'post_name': dot.post_id.name,
                                                                   'category_name': dot.account_id.cci_budget_group_id.category_id.name,
                                                                   'group_name': dot.account_id.cci_budget_group_id.name,
                                                                   'account_name': dot.account_id.code + ' ' + dot.account_id.name,
                                                                   'period_name': dPeriods[period_id]['name'],
                                                                   'dotation_id': dot.id,
                                                                   'budget':0.0,
                                                                  }

        for dot_per in dot_periods:
            if dot_per.dotation_id.post_id and dot_per.dotation_id.account_id:
                if dResults.has_key((dot_per.period_id.id,dot_per.dotation_id.post_id.id,dot_per.dotation_id.account_id.id)):
                    record = dResults[(dot_per.period_id.id,dot_per.dotation_id.post_id.id,dot_per.dotation_id.account_id.id)]
                    record['budget'] += (dot_per.amount or 0.0)
                    dResults[(dot_per.period_id.id,dot_per.dotation_id.post_id.id,dot_per.dotation_id.account_id.id)] = record
        
        return dResults
    def dotation_real_by_period_for_post(self,cr,uid,budget_id,post_id):
        # dResults = {(period_id,post_id,account_id):{'period_name','post_name','dpt_name','dpt_sequence','account_name','group_name','category_name','category_id','budget','real'}}
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        budget_obj = self.pool.get('cci_budget.budget')
        period_obj = self.pool.get('account.period')
        account_obj = self.pool.get('account.account')
        aaccount_obj = self.pool.get('account.analytic.account')
        amline_obj = self.pool.get('account.move.line')
        dotation_obj = self.pool.get('cci_budget.dotation')
        per_dot_obj = self.pool.get('cci_budget.period_dotation')
        post_obj = self.pool.get('cci_budget.post')

        # get the fiscalyear and periods linked to the given budget
        budget = budget_obj.browse(cr,uid,budget_id)
        post = post_obj.browse(cr,uid,post_id)
        period_ids = period_obj.search(cr,uid,[('fiscalyear_id','=',budget.fiscalyear_id.id),('special','=',False)])
        periods = period_obj.read(cr,uid,period_ids,['name','date_start','date_stop'])
        dPeriods = {}
        for period in periods:
            dPeriods[period['id']] = period
            
        # create the dResults from dotations, but we can add some moves not on dotations if not on budget but in bookkeeping
        dResults = {}
        account_ids = account_obj.search(cr,uid,[('cci_budget_group_id','!=',False)])
        accounts = account_obj.read(cr,uid,account_ids,['cci_budget_group_id','code','name'])
        saccounts = sorted(accounts, key=lambda k: k['code'])
        aaccount_ids = aaccount_obj.search(cr,uid,[('cci_post_id','=',post_id)])

        dotation_ids = dotation_obj.search(cr,uid,[('budget_id','=',budget_id),('post_id','=',post_id)])
        dotations = dotation_obj.browse(cr,uid,dotation_ids)
        dot_period_ids = per_dot_obj.search(cr,uid,[('dotation_id','in',dotation_ids)])
        dot_periods = per_dot_obj.browse(cr,uid,dot_period_ids)
        dResults = {}
        for dot in dotations:
            for period_id in period_ids:
                dResults[(period_id,post_id,dot.account_id.id)] = {'dpt_name': dot.post_id.department_id.name,
                                                                   'dpt_sequence': dot.post_id.department_id.sequence,
                                                                   'post_name': dot.post_id.name,
                                                                   'category_name': dot.account_id.cci_budget_group_id.category_id.name,
                                                                   'group_name': dot.account_id.cci_budget_group_id.name,
                                                                   'account_name': dot.account_id.code + ' ' + dot.account_id.name,
                                                                   'period_name': dPeriods[period_id]['name'],
                                                                   'dotation_id': dot.id,
                                                                   'real':0.0,
                                                                   'budget':0.0,
                                                                  }

        for dot_per in dot_periods:
            if dot_per.dotation_id.post_id and dot_per.dotation_id.account_id:
                if dResults.has_key((dot_per.period_id.id,dot_per.dotation_id.post_id.id,dot_per.dotation_id.account_id.id)):
                    record = dResults[(dot_per.period_id.id,dot_per.dotation_id.post_id.id,dot_per.dotation_id.account_id.id)]
                    record['budget'] += (dot_per.amount or 0.0)
                    dResults[(dot_per.period_id.id,dot_per.dotation_id.post_id.id,dot_per.dotation_id.account_id.id)] = record
        
        amline_ids = amline_obj.search(cr,uid,[('period_id','in',period_ids)])
        amlines = amline_obj.browse(cr,uid,amline_ids)
        for amline in amlines:
            if amline.analytic_lines:
                for analytic_line in amline.analytic_lines:
                    if analytic_line.account_id.id in aaccount_ids:
                        if dResults.has_key((amline.period_id.id,post_id,amline.account_id.id)):
                            record = dResults[(amline.period_id.id,post_id,amline.account_id.id)]
                            record['real'] += analytic_line.amount
                            dResults[(amline.period_id.id,post_id,amline.account_id.id)] = record
                        else:
                            if amline.account_id and amline.account_id.cci_budget_group_id and amline.account_id.cci_budget_group_id.category_id:
                                for period_id in period_ids:
                                    dResults[(period_id,post_id,amline.account_id.id)] = {'dpt_name': post.department_id.name,
                                                                                          'dpt_sequence': post.department_id.sequence,
                                                                                          'post_name': post.name,
                                                                                          'category_name': amline.account_id.cci_budget_group_id.category_id.name,
                                                                                          'group_name': amline.account_id.cci_budget_group_id.name,
                                                                                          'account_name': amline.account_id.code + ' ' + amline.account_id.name,
                                                                                          'period_name': dPeriods[period_id]['name'],
                                                                                          'dotation_id':-1, # -1 = dotation to be created
                                                                                          'real':(period_id == amline.period_id.id) and analytic_line.amount or 0.0,
                                                                                          'budget':0.0,
                                                                                        }
        return dResults
budget()

class department(osv.osv):
    _name = 'cci_budget.department'
    _description = 'Budgetary Department'
    _columns = {
        'name' : fields.char('Name', size=64, required=True),
        'sequence': fields.integer('Sequence'),
        'post_ids': fields.one2many('cci_budget.post','department_id','Posts'),
        'active': fields.boolean('Active'),
    }
    _defaults = {
        'active': lambda *a: True,
    }
    _order = "sequence"
department()

class post(osv.osv):
    _name = 'cci_budget.post'
    _description = 'Budgetary Post'
    _columns = {
        'name' : fields.char('Name', size=64, required=True),
        'department_id': fields.many2one('cci_budget.department', 'Department' ),
        'analytic_account_ids': fields.one2many('account.analytic.account','cci_post_id','Analytic Accounts'),
        'active': fields.boolean('Active'),
    }
    _defaults = {
        'active': lambda *a: True,
    }
post()

#class dotation_type(osv.osv):
#    _name = 'cci_budget.dotation_type'
#    _description = 'Dotation Type'
#    _columns = {
#        'name' : fields.char('Name', size=64, required=True),
#    }
#dotation_type()

class budget_account_category(osv.osv):
    _name = 'cci_budget.budget_account_category'
    _description = 'Main Budget Category'
    _columns = {
        'name' : fields.char('Name', size=64, required=True),
        'sequence': fields.integer('Sequence',required=True),
        'is_ca': fields.boolean('Is C.A.'),
        'active': fields.boolean('Active'),
        'group_ids': fields.one2many('cci_budget.budget_account_group','category_id','Subgroups'),
        'class_type': fields.selection([('directsales','Direct Sales'),('indirectsales','Indirect Sales'),('outsales','Out/Side Sales'),('expenses','Expenses'),('outexpenses','Out/Side Expenses'),('other','Undefined/Special')],'Class Type')
    }
    _defaults = {
        'sequence': lambda *a: 1,
        'is_ca': lambda *a: False,
        'active': lambda *a: True,
    }
    _order = "sequence"
budget_account_category()

class budget_account_group(osv.osv):
    _name = 'cci_budget.budget_account_group'
    _description = 'Budget Sub Group of Accounts'
    _columns = {
        'name' : fields.char('Name', size=64, required=True),
        'category_id': fields.many2one('cci_budget.budget_account_category', 'Category' ),
        'active': fields.boolean('Active'),
        'account_ids': fields.one2many('account.account','cci_budget_group_id','Accounts'),
    }
    _defaults = {
        'active': lambda *a: True,
    }
budget_account_group()

class dotation(osv.osv):
    _name = 'cci_budget.dotation'
    _description = 'Budget Dotation'
    _columns = {
        'budget_id': fields.many2one('cci_budget.budget', 'Budget', required=True ),
        'post_id': fields.many2one('cci_budget.post', 'Post', required=True ),
        'account_id' : fields.many2one('account.account', 'Account', required=True ),
        'comments': fields.text('Comments'),
        'period_dotation_ids': fields.one2many('cci_budget.period_dotation','dotation_id','By Periods'),
        'active': fields.boolean('Active'),
    }
    def name_get(self,cr,uid,ids,context={}):
        res = []    
        for dot in self.browse(cr,uid,ids):
            res.append((dot.id,'"%s" : "%s %s" - "%s"' % (dot.budget_id.name,dot.account_id.code or '',dot.account_id.name or '',dot.post_id.name)))
        return res
    def name_search(self,cr,uid,name='',args=[],operator='ilike',context={},limit=80):
        ids = []
        if name:
            # as it is possible to filter on post name or account name with other fields,
            # we suppose that the user want to filter on budget name
            budget_ids = self.pool.get('cci_budget.budget').search(cr,uid,[('name',operator,name)])
            if budget_ids:
                ids = self.search(cr,uid,[('budget_id','in',budget_ids)] + args, limit=limit,context=context)
        return self.name_get(cr,uid,ids)
    _defaults = {
        'active': lambda *a: True,
    }
dotation()

class period_dotation(osv.osv):
    _name = 'cci_budget.period_dotation'
    _description = 'Budget Dotation By Period'
    _columns = {
        'dotation_id': fields.many2one('cci_budget.dotation', 'Dotation', required=True ),
        'period_id': fields.many2one('account.period', 'Period', required=True ),
        'amount' : fields.float('Amount', digits=(16, 2)),
        'comments': fields.text('Comments'),
        'budget_id': fields.related('dotation_id','budget_id',type='many2one',relation='cci_budget.budget',string='Budget',store=False,readonly=True),
        'post_id': fields.related('dotation_id','post_id',type='many2one',relation='cci_budget.post',string='Post',store=False,readonly=True),
        'account_id' : fields.related('dotation_id','account_id',type='many2one',relation='account.account',string='Account',store=False,readonly=True),
        'active': fields.boolean('Active'),
    }
    _defaults = {
        'active': lambda *a: True,
    }
period_dotation()

class account_account(osv.osv):
    _inherit ="account.account"
    _columns={
        'cci_budget_group_id' : fields.many2one('cci_budget.budget_account_group', 'Budget Group' ),
    }
account_account()

class account_analytic_account(osv.osv):
    _inherit ="account.analytic.account"
    _columns={
        'cci_post_id' : fields.many2one('cci_budget.post', 'Budget Project' ),
        'cci_department_id' : fields.many2one('cci_budget.department', 'Budget Department' ),
        'cci_budget' : fields.boolean('Is Planned in Budget' ),
        'cci_out_of_project' : fields.boolean('Out of Budget Project' ),
    }
    _defaults = {
        'cci_budget': lambda *a: True,
        'cci_out_of_project': lambda *a: False,
    }
account_analytic_account()

class raw_dotation(osv.osv):
    _name = 'cci_budget.raw_dotation'
    _description = 'Budget Dotation without Smoothing'
    _columns = {
        'budget_id': fields.many2one('cci_budget.budget', 'Budget', required=True ),
        'post_id': fields.many2one('cci_budget.post', 'Post', required=True ),
        'account_id': fields.many2one('account.account', 'Account', required=True ),
        'associated_account_id': fields.many2one('account.account', 'Account for Smoothed lines', required=True ),
        'comments': fields.text('Comments'),
        'raw_period_dotation_ids': fields.one2many('cci_budget.raw_period_dotation','raw_dotation_id','By Periods'),
        'active': fields.boolean('Active'),
    }
    def name_get(self,cr,uid,ids,context={}):
        res = []    
        for dot in self.browse(cr,uid,ids):
            res.append((dot.id,'"%s" : "%s %s" - "%s"' % (dot.budget_id.name,dot.account_id.code or '',dot.account_id.name or '',dot.post_id.name)))
        return res
    def name_search(self,cr,uid,name='',args=[],operator='ilike',context={},limit=80):
        ids = []
        if name:
            # as it is possible to filter on post name or account name with other fields,
            # we suppose that the user want to filter on budget name
            budget_ids = self.pool.get('cci_budget.budget').search(cr,uid,[('name',operator,name)])
            if budget_ids:
                ids = self.search(cr,uid,[('budget_id','in',budget_ids)] + args, limit=limit,context=context)
        return self.name_get(cr,uid,ids)
    _defaults = {
        'active': lambda *a: True,
    }
raw_dotation()

class raw_period_dotation(osv.osv):
    _name = 'cci_budget.raw_period_dotation'
    _description = 'Budget Dotation By Period without Smmothing'
    _columns = {
        'raw_dotation_id': fields.many2one('cci_budget.raw_dotation', 'Dotation', required=True ),
        'period_id': fields.many2one('account.period', 'Period', required=True ),
        'amount' : fields.float('Amount', digits=(16, 2)),
        'comments': fields.text('Comments'),
        'budget_id': fields.related('raw_dotation_id','budget_id',type='many2one',relation='cci_budget.budget',string='Budget',store=False,readonly=True),
        'post_id': fields.related('raw_dotation_id','post_id',type='many2one',relation='cci_budget.post',string='Post',store=False,readonly=True),
        'account_id' : fields.related('raw_dotation_id','account_id',type='many2one',relation='account.account',string='Account',store=False,readonly=True),
        'active': fields.boolean('Active'),
    }
    _defaults = {
        'active': lambda *a: True,
    }
raw_period_dotation()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
