# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import wizard
import time
import datetime
import tools
import pooler
import base64
import csv

form = """<?xml version="1.0"?>
<form string="Creation of salary from a yearly Sheet">
    <field name="new_budget_id" colspan="4" width="800" required="1"/>
    <field name="sheet" colspan="4" required="1"/>
    <newline/>
    <separator string="Multiplicateur mensuel pour congés payés et 13ème mois" colspan="4"/>
    <group col="12" colspan="4">
        <field name="second1" nolabel="1" required="1"/>
        <field name="second2" nolabel="1" required="1"/>
        <field name="second3" nolabel="1" required="1"/>
        <field name="second4" nolabel="1" required="1"/>
        <field name="second5" nolabel="1" required="1"/>
        <field name="second6" nolabel="1" required="1"/>
        <field name="second7" nolabel="1" required="1"/>
        <field name="second8" nolabel="1" required="1"/>
        <field name="second9" nolabel="1" required="1"/>
        <field name="second10" nolabel="1" required="1"/>
        <field name="second11" nolabel="1" required="1"/>
        <field name="second12" nolabel="1" required="1"/>
    </group>
    <field name="always_new"/>
</form>"""

fields = {
    'new_budget_id': {'string': 'New Budget', 'type':'many2one','relation': 'cci_budget.budget','required': True},
    'sheet':  {'type':'binary','string':'Salary CSV file','required':True},
    'second1':{'string':'Second line in January','type': 'float', 'default':1.0},
    'second2':{'string':'Second line in February','type': 'float', 'default':1.0},
    'second3':{'string':'Second line in March','type': 'float', 'default':1.0},
    'second4':{'string':'Second line in April','type': 'float', 'default':1.0},
    'second5':{'string':'Second line in May','type': 'float', 'default':1.0},
    'second6':{'string':'Second line in June','type': 'float', 'default':1.92},
    'second7':{'string':'Second line in July','type': 'float', 'default':1.0},
    'second8':{'string':'Second line in Augustus','type': 'float', 'default':1.0},
    'second9':{'string':'Second line in September','type': 'float', 'default':1.0},
    'second10':{'string':'Second line in October','type': 'float', 'default':1.0},
    'second11':{'string':'Second line in November','type': 'float', 'default':1.0},
    'second12':{'string':'Second line in December','type': 'float', 'default':2.0},
    'always_new':{'string':'Always Create','type':'boolean','help':'If checked, this wizard will always create new lines, never overwrite.','default':'True'},
   }

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="Results" colspan="4"/>
     <field name="msg" colspan="4" nolabel="1" readonly="1" width="800"/>
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'default':'All salaries created'},
}

class cci_budget_create_salary_yearly(wizard.interface):
    def _create_salary_yearly(self,cr,uid,data,context):
        budget_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget')
        account_obj = pooler.get_pool(cr.dbname).get('account.account')
        budget = budget_obj.browse(cr,uid,data['form']['new_budget_id'])
        #
        # read sheet of yearly salary
        #
        periods = []
        for period in budget.fiscalyear_id.period_ids:
            if not period.special:
                periods.append({'id':period.id,'date_start':period.date_start})
        speriods = sorted(periods, key= lambda k: k['date_start'])
        period_ids = [x['id'] for x in speriods]
        today = datetime.date.today().strftime('%d/%m/%Y')

        # first reading of first four lines to detect columns to manage and way to manage them
        inputfile = data['form']['sheet']
        inputdata = base64.decodestring(inputfile)
        num_ligne = 0
        used_col_nums = []
        account_codes = []
        salary_types = []
        data_lines = []
        distri_names = []
        fields = {}
        dAccounts = {}
        dAccountsID2Code = {}
        for line in inputdata.splitlines():
            num_ligne += 1
            columns = line.split(";")
            if num_ligne == 1:
                for col_num in range(0,len(columns)):
                    if columns[col_num]:
                        used_col_nums.append(col_num)
                        field_data = {'name':columns[col_num].strip(),
                                      'account_code':'',
                                      'type':'classic',
                                     }
                        fields[col_num] = field_data
                if not used_col_nums:
                    data['form']['msg'] = 'ERROR:\n\nLine 1 of csv file must have some fields name to specify the columns to use. These fields names will be used in comments; they can have multiple words and upper- or lowercase.\nOne of the columns must have the name "distri".'
                    break
            elif num_ligne == 2:
                all_cols = True
                for col_num in used_col_nums:
                    if columns[col_num] and len(columns[col_num].strip()) == 6:
                        field_data = fields[col_num]
                        field_data['account_code'] = columns[col_num].strip()
                        if columns[col_num].strip() not in account_codes:
                            account_codes.append(columns[col_num].strip())
                        fields[col_num] = field_data
                    else:
                        all_cols = False
                if all_cols:
                    account_ids = account_obj.search(cr,uid,[('code','in',account_codes)])
                    if account_ids:
                        if len(account_codes) == len(account_ids):
                            accounts = account_obj.read(cr,uid,account_ids,['code'])
                            for acc in accounts:
                                dAccounts[acc['code']] = acc['id']
                                dAccountsID2Code[acc['id']] = acc['code']
                        else:
                            data['form']['msg'] = 'ERROR:\n\nSome account codes are not found'
                            used_col_nums = []
                            break
                    else:
                        data['form']['msg'] = 'ERROR:\n\nNo account codes found on third line'
                        used_col_nums = []
                        break
                else:
                    data['form']['msg'] = 'ERROR:\n\nProblems with headers on line 2'
                    used_col_nums = []
                    break
            elif num_ligne == 3:
                # ligne with typology salary or not
                all_cols = True
                for col_num in used_col_nums:
                    if columns[col_num] and len(columns[col_num].strip()) >= 1:
                        salary_type = columns[col_num]
                        if salary_type in ['classic','salary']:
                            salary_types.append(salary_type)
                            field_data = fields[col_num]
                            field_data['type'] = salary_type
                            fields[col_num] = field_data
                        else:
                            all_cols = False
                    else:
                        all_cols = False
                if not all_cols:
                    data['form']['msg'] = 'ERROR:\n\nProblems with salary types on line 3'
                    used_col_nums = []
                    break
            elif num_ligne >= 4:
                if columns[0]:
                    data_line = {}
                    data_line['distri'] = columns[0].strip()
                    if line[0].strip() not in distri_names:
                        distri_names.append(columns[0].strip())
                    data_line['person'] = 'ligne'+str(num_ligne)
                    if columns[1]:
                        data_line['person'] = columns[1].strip()
                    elif columns[0]:
                        data_line['person'] += ' [' + columns[0][2:] + ']'
                    for col_num in used_col_nums:
                        field_name = fields[col_num]['name']
                        data_line[field_name] = float((columns[col_num] or '0.0').replace(',','.'))
                    data_lines.append(data_line)
        if used_col_nums:
            dist_obj = pooler.get_pool(cr.dbname).get('account.analytic.plan.instance')
            dist_ids = dist_obj.search(cr,uid,[('name','in',distri_names)])
            if dist_ids:
                dists = dist_obj.browse(cr,uid,dist_ids)
                dDistNames = {}
                for dist in dists:
                    dDistNames[dist['name']] = dist
                found_all_distri = True
                for name in distri_names:
                    if name not in dDistNames.keys():
                        data['form']['msg'] = 'ERROR:\n\nError(s) in distribution names'
                        found_all_distri = False
                        current_error = name
                if found_all_distri:
                    dotation_obj = pooler.get_pool(cr.dbname).get('cci_budget.dotation')
                    per_dot_obj = pooler.get_pool(cr.dbname).get('cci_budget.period_dotation')
                    for line in data_lines:
                        distri = dDistNames[line['distri']]
                        rates = 0
                        for dist_line in distri.account_ids:
                            if dist_line.analytic_account_id and dist_line.analytic_account_id.cci_post_id:
                                rates += dist_line.rate
                                for col_num in used_col_nums:
                                    comments = 'Injection RH ' + fields[col_num]['name'] + ' ' + line['person'] + ' ' + str(dist_line.rate or 0) + '%' + ' [' + today + ']'
                                    account_id = dAccounts[fields[col_num]['account_code']]
                                    dot_ids = dotation_obj.search(cr,uid,[('budget_id','=',budget.id),('post_id','=',dist_line.analytic_account_id.cci_post_id.id),('account_id','=',dAccounts[fields[col_num]['account_code']])])
                                    if not dot_ids:
                                        dot_id = dotation_obj.create(cr,uid,{'budget_id':budget.id,'post_id':dist_line.analytic_account_id.cci_post_id.id,'account_id':dAccounts[fields[col_num]['account_code']],'comments':comments})
                                    else:
                                        dot_id = dot_ids[0]
                                    if dot_id:
                                        index_period = 0
                                        for period_id in period_ids:
                                            month_multi = 1
                                            if fields[col_num]['type'] == 'salary':
                                                second_name = 'second'+str(index_period+1)
                                                month_multi = data['form'][second_name]
                                            amount_dotation = round(line[fields[col_num]['name']] * month_multi * (dist_line.rate or 0) / 100,2)
                                            comments = comments.encode("ASCII", 'ignore')
                                            per_dot_obj.create(cr,uid,{'dotation_id':dot_id,'period_id':period_id,'comments':comments,'amount':amount_dotation})
                                            index_period += 1
                        if rates != 100:
                            data['form']['msg'] = 'ERROR:\n\nProblems with the rates of some analytic distributions'
                else:
                    data['form']['msg'] = 'ERROR:\n\nNot possible to found all analytic distributions. At least : %s' % current_error
            else:
                data['form']['msg'] = 'ERROR:\n\nNot possible to found analytic distributions on basis of their names'
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('create','Create Salary')]},
        },
        'create': {
            'actions': [_create_salary_yearly],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }

cci_budget_create_salary_yearly('cci_budget_create_salary_yearly')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

