# -*- encoding: utf-8 -*-
# Version 1.0 Philmer
import wizard
import time
import datetime
import calendar
import tools
import pooler
import base64
import csv
from pyExcelerator import *
from osv import fields
from osv import osv
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Options">
    <field name="overwrite"/>
    <newline/>
    <field name="input" colspan="4" width="600"/>
    <separator string="Comments"/>
    <field name="comments" colspan="4" nolabel="1"/>
</form>"""

fields = {
    'input': {'type':'binary','string':'Input file','required':True},
    'overwrite':{'type':'boolean','string':'Overwrite (and create if not existing); else, will record just the changes','default':True},
    'comments':{'string':'Comments','type':'text'},
}

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="Importation result" colspan="4"/>
     <field name="msg" colspan="4" nolabel="1" width="800" heigth="600"/>
</form>"""

msg_fields = {
    'msg': {'string':'File imported', 'type':'text', 'readonly':True},
}

def _remove_accentuation(name):
    return name.strip().replace(u'é','e').replace(u'è','e').replace(u'ê','e').replace(u'ë','e').replace(u'ù','u').replace(u'û','u').replace(u'ü','u').replace(u'à','a').replace(u'â','a').replace(u'ä','a').replace(u'ç','c').replace(u'î','i').replace(u'ï','i').replace(u'ô','o').replace(u'ö','o')

def convert_to_float(string):
    res = string.strip().replace('.','').replace(',','.')
    if not res:
        res = '0.0'
    try:
        result = float(res)
    except:
        result = 0.0
    return result
def convert_to_int(string):
    res = string.strip()
    if not res:
        res = '0'
    try:
        result = int(res)
    except:
        result = 0
    return result
class cci_budget_read_back_csv(wizard.interface):
    def _import_data(self,cr,uid,data,context):
        budget_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget')
        dotation_obj = pooler.get_pool(cr.dbname).get('cci_budget.dotation')
        per_dot_obj = pooler.get_pool(cr.dbname).get('cci_budget.period_dotation')
        account_obj = pooler.get_pool(cr.dbname).get('account.account')
        period_obj = pooler.get_pool(cr.dbname).get('account.period')
        inputfile = data['form']['input']
        inputdata = base64.decodestring(inputfile).split('\n')
        hfInput = csv.DictReader(inputdata,delimiter=";",fieldnames=['dotation_id','group','subgroup','account','type','per01','per02','per03','per04','per05','per06','per07','per08','per09','per10','per11','per12','total'])
        post_id = 0
        new_budget_id = 0
        current_account = False
        line_id = 1
        all_ok = True
        overwritten = 0
        created = 0
        errors = []
        for record in hfInput:
            if line_id <= 7:
                if line_id == 4:
                    post_id = convert_to_int(record['group'])
                elif line_id == 5:
                    new_budget_id = convert_to_int(record['group'])
            else:
                # Only one time but after the first 7 lines
                if line_id == 8:
                    if post_id == 0 or new_budget_id == 0:
                        data['form']['msg'] = 'ERROR !\n\nLack of post_id (%i) or new_budget_id (%i)' % (post_id,new_budget_id)
                        all_ok = False
                        break
                    else:
                        budget = budget_obj.browse(cr,uid,new_budget_id)
                        budget_period_ids = [x.id for x in budget.fiscalyear_id.period_ids]
                        budget_periods = period_obj.read(cr,uid,budget_period_ids,['name','date_start','special'])
                        budget_periods = [x for x in budget_periods if not x['special']]
                        sbudget_periods = sorted(budget_periods, key=lambda k: k['date_start'])
                        dPeriods = {}
                        index_period = 1
                        for period in sbudget_periods:
                            dPeriods[index_period] = period['id']
                            index_period += 1
                if record['account']:
                    account_code_name = record['account'].decode('cp1252')
                    pos_space = account_code_name.find(' ')
                    if pos_space > -1:
                        account_code = account_code_name[0:pos_space]
                        account_name = account_code_name[pos_space+1:]
                        account_ids = account_obj.search(cr,uid,[('code','=',account_code),('name','=',account_name)])
                        if account_ids:
                            current_account = account_obj.browse(cr,uid,account_ids[0])
                        else:
                            current_account = False
                    else:
                        current_account = False
                first_column = convert_to_int(record['dotation_id'])
                if (first_column == -1 or first_column > 0) and current_account:
                    index = 1
                    amounts = {}
                    amount_total = 0
                    while index <= 12:
                        amounts[index] = convert_to_float(record['per'+str(index).rjust(2,'0')])
                        amount_total += amounts[index]
                        index += 1
                    total = convert_to_float(record['total'])
                    if abs(total-amount_total) < 0.01:
                        # write in DB
                        dotation = False
                        if first_column > 0:
                            dotation_ids = dotation_obj.search(cr,uid,[('id','=',first_column)])
                            if dotation_ids:
                                dotation = dotation_obj.browse(cr,uid,dotation_ids[0])
                                if dotation.budget_id.id == new_budget_id and dotation.post_id.id == post_id and dotation.account_id.id == current_account.id:
                                    # good dotation
                                    pass
                                else:
                                    dotation = False
                        if not dotation:
                            already_dot_ids = dotation_obj.search(cr,uid,[('budget_id','=',new_budget_id),('post_id','=',post_id),('account_id','=',current_account.id)])
                            if already_dot_ids:
                                dotation_id = already_dot_ids[0]
                            else:
                                dotation_id = dotation_obj.create(cr,uid,{'budget_id':new_budget_id,
                                                                          'post_id':post_id,
                                                                          'account_id':current_account.id,
                                                                          'comments':data['form']['comments'],
                                                                         })
                            if dotation_id:
                                dotation = dotation_obj.browse(cr,uid,dotation_id)
                        if dotation:
                            index_period = 1
                            while index_period <= 12:
                                per_dot_ids = per_dot_obj.search(cr,uid,[('dotation_id','=',dotation.id),('period_id','=',dPeriods[index_period])])
                                if data['form']['overwrite']:
                                    if per_dot_ids:
                                        per_dot_obj.write(cr,uid,[per_dot_ids[0],],{'amount':convert_to_float(record['per'+str(index_period).rjust(2,'0')]),'comments':data['form']['comments']})
                                        overwritten += 1
                                    else:
                                        per_dot_obj.create(cr,uid,{'dotation_id':dotation.id,
                                                                   'period_id':dPeriods[index_period],
                                                                   'amount':convert_to_float(record['per'+str(index_period).rjust(2,'0')]),
                                                                   'comments':data['form']['comments'],
                                                                  })
                                        created += 1
                                else:
                                    current_total = 0.0
                                    if per_dot_ids:
                                        per_dotations = per_dot_obj.browse(cr,uid,per_dot_ids)
                                        for per_dotation in per_dotations:
                                            current_total += (per_dotation.amount or 0.0)
                                    new_value = convert_to_float(record['per'+str(index_period).rjust(2,'0')])
                                    if abs(current_total - new_value) > 0.001:
                                        per_dot_obj.create(cr,uid,{'dotation_id':dotation.id,
                                                                   'period_id':dPeriods[index_period],
                                                                   'amount':round(new_value-current_total,2),
                                                                   'comments':data['form']['comments'],
                                                                  })
                                        created += 1
                                index_period += 1
                    else:
                        errors.append('error on total %f <> %f' % (total,amount_total))
                        all_ok = False
            line_id += 1
        if all_ok:
            data['form']['msg']=_('All amounts inserted into budget.')
        else:
            data['form']['msg']='ERRORS :\n' + '\n'.join(errors)
        return data['form']
    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('import','Import File')]},
        },
        'import': {
            'actions': [_import_data],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }

cci_budget_read_back_csv('cci_budget_read_back_csv')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

