# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import datetime
import wizard
import pooler
from tools.translate import _
import base64
from pyExcelerator import *
from pyExcelerator.Worksheet import *

form = '''<?xml version="1.0"?>
<form string="Select period">
    <field name="budget1_id" colspan="4"/>
    <field name="period_to"/>
    <newline/>
    <field name="budget2_id" colspan="4"/>
    <newline/>
    <field name="type"/>
</form>'''

fields = {
    'budget1_id': {'string': 'Budget', 'type': 'many2one', 'relation': 'cci_budget.budget', 'required': True},
    'period_to': {'string': 'To Period Budget 1', 'type': 'many2one', 'relation': 'account.period', 'required': True},
    'budget2_id': {'string': 'Budget', 'type': 'many2one', 'relation': 'cci_budget.budget', 'required': True},
    'type': {'string':'Type','type':'selection','selection':[('real','Real'),('ca','Only Turnover')],'required':True,'default':'real'}
}
msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="compare_budgets.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'compare_budgets.xls':{'string': 'Prepared file','type': 'binary','readonly': True,},
}

pyexcel_colours = {
        'aqua'    : 0x0F,
        'cyan'    : 0x0F,
        'black'   : 0x08,
        'blue'    : 0x0C,
        'brown'   : 0x10,
        'magenta' : 0x0E,
        'fuchsia' : 0x0E,
        'gray'    : 0x17,
        'grey'    : 0x17,
        'green'   : 0x11,
        'lime'    : 0x0B,
        'navy'    : 0x12,
        'orange'  : 0x35,
        'pink'    : 0x21,
        'purple'  : 0x14,
        'red'     : 0x0A,
        'silver'  : 0x16,
        'white'   : 0x09,
        'yellow'  : 0x0D,
}


def num_to_letter(line_number_in_pyexcel):
    # this method convert an integer to a line label in Excel
    # first line = 0 (beware on pyExcelerator, first line = 0)
    # 0 => 'A'
    # 26 => 'AA'
    # 27 => 'AB'
    line_number_in_pyexcel += 1
    if line_number_in_pyexcel < 26:
        result = chr(64+line_number_in_pyexcel)
    else:
        first = line_number_in_pyexcel_number_in_pyexcel // 26
        last = number % 26
        result = chr(64+first)+chr(64+last)
    return result
def full_cell(line_number_in_pyexcel,col_number_in_pyexcel,fixed='none'):
    # fixed = none, col, line, both
    result = ''
    if fixed in ('col','both'):
        result += '$'
    result += num_to_letter(col_number_in_pyexcel)
    if fixed in ('line','both'):
        result += '$'
    result += str(line_number_in_pyexcel+1)
    return result
class compare_cci_budgets(wizard.interface):
    def _get_defaults(self, cr, uid, data, context):
        today = datetime.date.today()
        fiscalyear_obj = pooler.get_pool(cr.dbname).get('account.fiscalyear')
        budget_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget')
        if today.month <= 2:
            fiscalyear_ids = fiscalyear_obj.search(cr,uid,[('date_start','=',str(today.year-1)+'-01-01')])
        else:
            fiscalyear_ids = fiscalyear_obj.search(cr,uid,[('date_start','=',str(today.year)+'-01-01')])
        if fiscalyear_ids:
            fiscalyear_id = fiscalyear_ids[0]
        else:
            fiscalyear_id = fiscalyear_obj.find(cr,uid)
        if fiscalyear_id:
            budget_ids = budget_obj.search(cr,uid,[('fiscalyear_id','=',fiscalyear_id),('official','=',True)])
            if budget_ids:
                data['form']['budget1_id'] = budget_ids[0]
            else:
                budget_ids = budget_obj.search(cr,uid,[('fiscalyear_id','=',fiscalyear_id)])
                if budget_ids:
                    data['form']['budget1_id'] = budget_ids[0]
        #
        if today.month <= 2:
            fiscalyear_ids = fiscalyear_obj.search(cr,uid,[('date_start','=',str(today.year-2)+'-01-01')])
        else:
            fiscalyear_ids = fiscalyear_obj.search(cr,uid,[('date_start','=',str(today.year-1)+'-01-01')])
        if fiscalyear_ids:
            fiscalyear_id = fiscalyear_ids[0]
        else:
            fiscalyear_id = fiscalyear_obj.find(cr,uid)
        if fiscalyear_id:
            budget_ids = budget_obj.search(cr,uid,[('fiscalyear_id','=',fiscalyear_id),('official','=',True)])
            if budget_ids:
                data['form']['budget2_id'] = budget_ids[0]
            else:
                budget_ids = budget_obj.search(cr,uid,[('fiscalyear_id','=',fiscalyear_id)])
                if budget_ids:
                    data['form']['budget2_id'] = budget_ids[0]
        return data['form']

    def _extract_excel(self, cr, uid, data, context):
        string_now = datetime.datetime.now().strftime('%d/%m/%Y - %H:%M:%S')
        # selected data
        output_type = data['form']['type']
        fiscalyear_obj = pooler.get_pool(cr.dbname).get('account.fiscalyear')
        budget_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget')
        period_obj = pooler.get_pool(cr.dbname).get('account.period')
        account_obj = pooler.get_pool(cr.dbname).get('account.account')
        aaccount_obj = pooler.get_pool(cr.dbname).get('account.analytic.account')
        amline_obj = pooler.get_pool(cr.dbname).get('account.move.line')
        dotation_obj = pooler.get_pool(cr.dbname).get('cci_budget.dotation')
        per_dot_obj = pooler.get_pool(cr.dbname).get('cci_budget.period_dotation')
        post_obj = pooler.get_pool(cr.dbname).get('cci_budget.post')
        
        budget1 = budget_obj.browse(cr,uid,data['form']['budget1_id'])
        budget2 = budget_obj.browse(cr,uid,data['form']['budget2_id'])
        fiscalyear1 = budget1.fiscalyear_id
        fiscalyear2 = budget2.fiscalyear_id
        period_from_id = period_obj.search(cr,uid,[('date_start','=',fiscalyear1.date_start),('special','=',False)])[0]
        period1_from = period_obj.browse(cr,uid,period_from_id)
        period1_to = period_obj.browse(cr,uid,data['form']['period_to'])
        budget1_period_ids = period_obj.search(cr,uid,[('date_start','>=',period1_from.date_start),('date_stop','<=',period1_to.date_stop),('special','=',False)])
        if budget1.fiscalyear_id.id != budget2.fiscalyear_id.id:
            # not same year => others periods also
            period_from_id = period_obj.search(cr,uid,[('date_start','=',fiscalyear2.date_start),('special','=',False)])[0]
            period2_from = period_obj.browse(cr,uid,period_from_id)
            date_start2 = fiscalyear2.date_start[0:4] + period1_to.date_start[4:]
            period_to_id = period_obj.search(cr,uid,[('date_start','=',date_start2),('special','=',False)])[0]
            period2_to = period_obj.browse(cr,uid,period_to_id)
            budget2_period_ids = period_obj.search(cr,uid,[('date_start','>=',period2_from.date_start),('date_stop','<=',period2_to.date_stop),('special','=',False)])
            concerned_period_ids = budget1_period_ids + budget2_period_ids
        else:
            concerned_period_ids = [x for x in budget1_period_ids]
        
        dResults = {}
        dAnalytic2Post = {}
        dAccount2Group = {}
        account_ids = account_obj.search(cr,uid,[('cci_budget_group_id','!=',False)])
        accounts = account_obj.read(cr,uid,account_ids,['cci_budget_group_id','code','name'])
        for account in accounts:
            dAccount2Group[account['id']] = account['cci_budget_group_id'][0]
        saccounts = sorted(accounts, key=lambda k: k['code'])
        aaccount_ids = aaccount_obj.search(cr,uid,[('cci_post_id','!=',False)])
        aaccounts = aaccount_obj.read(cr,uid,aaccount_ids,['cci_post_id'])
        for aaccount in aaccounts:
            dAnalytic2Post[aaccount['id']] = aaccount['cci_post_id'][0]

        dotation_ids = dotation_obj.search(cr,uid,[('budget_id','in',(budget1.id,budget2.id))])
        dotations = dotation_obj.browse(cr,uid,dotation_ids)
        dResults = {}
        concerned_account_ids = []
        concerned_aaccount_ids = []
        for dot in dotations:
            dResults[(dot.post_id.id,dot.account_id.id)] = {'dpt_name': dot.post_id.department_id.name,
                                                            'post_name': dot.post_id.name,
                                                            'category_name': dot.account_id.cci_budget_group_id.category_id.name,
                                                            'group_name': dot.account_id.cci_budget_group_id.name,
                                                            'account_name': dot.account_id.code + ' ' + dot.account_id.name,
                                                            'ca1':0.0,
                                                            'ca_budget1':0.0,
                                                            'turnover1':0.0,
                                                            'budget1':0.0,
                                                            'ca2':0.0,
                                                            'ca_budget2':0.0,
                                                            'turnover2':0.0,
                                                            'budget2':0.0}
            if dot.account_id.id not in concerned_account_ids:
                concerned_account_ids.append(dot.account_id.id)
        concerned_aaccount_ids = aaccount_obj.search(cr,uid,[('cci_post_id','!=',False)])
        period_dotation_ids = per_dot_obj.search(cr,uid,[('dotation_id','in',dotation_ids),('period_id','in',concerned_period_ids)])
        if period_dotation_ids:
            period_dotations = per_dot_obj.browse(cr,uid,period_dotation_ids)
            for per_dot in period_dotations:
                if per_dot.dotation_id.post_id and per_dot.dotation_id.account_id:
                    if dResults.has_key((per_dot.dotation_id.post_id.id,per_dot.dotation_id.account_id.id)):
                        record = dResults[(per_dot.dotation_id.post_id.id,per_dot.dotation_id.account_id.id)]
                        if per_dot.period_id.id in budget1_period_ids:
                            if per_dot.dotation_id.account_id.cci_budget_group_id.category_id.is_ca:
                                record['ca_budget1'] += (per_dot.amount or 0.0)
                            record['budget1'] += (per_dot.amount or 0.0)
                        if per_dot.period_id.id in budget2_period_ids:
                            if per_dot.dotation_id.account_id.cci_budget_group_id.category_id.is_ca:
                                record['ca_budget2'] += (per_dot.amount or 0.0)
                            record['budget2'] += (per_dot.amount or 0.0)
                        dResults[(per_dot.dotation_id.post_id.id,per_dot.dotation_id.account_id.id)] = record
        
        amline_ids = amline_obj.search(cr,uid,[('period_id','in',concerned_period_ids)])
        amlines = amline_obj.browse(cr,uid,amline_ids)
        for amline in amlines:
            if amline.analytic_lines:
                for analytic_line in amline.analytic_lines:
                    if analytic_line.account_id.id in concerned_aaccount_ids and dAnalytic2Post.has_key(analytic_line.account_id.id):
                        if dResults.has_key((dAnalytic2Post[analytic_line.account_id.id],amline.account_id.id)):
                            record = dResults[(dAnalytic2Post[analytic_line.account_id.id],amline.account_id.id)]
                            if amline.period_id.id in budget1_period_ids:
                                if amline.account_id.cci_budget_group_id.category_id.is_ca:
                                    record['ca1'] += analytic_line.amount
                                record['turnover1'] += analytic_line.amount
                            if amline.period_id.id in budget2_period_ids:
                                if amline.account_id.cci_budget_group_id.category_id.is_ca:
                                    record['ca2'] += analytic_line.amount
                                record['turnover2'] += analytic_line.amount
                            dResults[(dAnalytic2Post[analytic_line.account_id.id],amline.account_id.id)] = record
                        else:
                            post = post_obj.browse(cr,uid,dAnalytic2Post[analytic_line.account_id.id])
                            if amline.account_id and amline.account_id.cci_budget_group_id and amline.account_id.cci_budget_group_id.category_id:
                                dResults[(dAnalytic2Post[analytic_line.account_id.id],amline.account_id.id)] = \
                                            {'dpt_name': post.department_id.name,
                                             'post_name': post.name,
                                             'category_name': amline.account_id.cci_budget_group_id.category_id.name,
                                             'group_name': amline.account_id.cci_budget_group_id.name,
                                             'account_name': amline.account_id.code + ' ' + amline.account_id.name,
                                             'ca1':((amline.period_id.id in budget1_period_ids) and amline.account_id.cci_budget_group_id.category_id.is_ca) and analytic_line.amount or 0.0,
                                             'ca_budget1':0.0,
                                             'turnover1':(amline.period_id.id in budget1_period_ids) and analytic_line.amount or 0.0,
                                             'budget1':0.0,
                                             'ca2':((amline.period_id.id in budget2_period_ids) and amline.account_id.cci_budget_group_id.category_id.is_ca) and analytic_line.amount or 0.0,
                                             'ca_budget2':0.0,
                                             'turnover2':(amline.period_id.id in budget2_period_ids) and analytic_line.amount or 0.0,
                                             'budget2':0.0}
        wb1 = Workbook()

        # styles
        border_right = Borders()
        border_right.right = 2
        border_right_and_bottom = Borders()
        border_right_and_bottom.right = 2
        border_right_and_bottom.bottom = 2
        border_bottom = Borders()
        border_bottom.bottom = 2
        all_borders = Borders()
        all_borders.left = 2
        all_borders.right = 2
        all_borders.top = 2
        all_borders.bottom = 2
        borders_lr = Borders()
        borders_lr.left = 2
        borders_lr.right = 2
        borders_lrb = Borders()
        borders_lrb.left = 2
        borders_lrb.right = 2
        borders_lrb.bottom = 2
        al = Alignment()
        al.horz = Alignment.HORZ_CENTER
        al.vert = Alignment.VERT_CENTER
        font12 = Font()
        font12.height = 240
        font_title_project = Font()
        font_title_project.height = 300
        font_title_project.colour_index = 0x12 ## 'Navy'
        
        pattern_silver = Pattern()
        pattern_silver._pattern_back_colour = 0x16  ### 'silver'
        pattern_silver.pattern = 0x01 # SOLID
        pattern_grey = Pattern()
        pattern_grey._pattern_back_colour = 0x17  ### 'grey'
        pattern_grey.pattern = 0x01 # SOLID
        pattern_yellow = Pattern()
        pattern_yellow.pattern = 0x01 # SOLID
        pattern_yellow.set_pattern_fore_colour(0x0D)  ### 'yellow'
        pattern_aqua = Pattern()
        pattern_aqua.pattern = 0x01 # SOLID
        pattern_aqua.set_pattern_fore_colour(pyexcel_colours['aqua'])

        style_right = XFStyle()
        style_right.borders = border_right

        style_red_float = XFStyle()
        style_red_float.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_red_float.borders = border_right
        
        style_bordered_red_float = XFStyle()
        style_bordered_red_float.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_bordered_red_float.borders = borders_lr

        style_lr_red_float = XFStyle()
        style_lr_red_float.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_lr_red_float.borders = borders_lr

        style_red_float_wo_border = XFStyle()
        style_red_float_wo_border.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'

        style_total_red_float = XFStyle()
        style_total_red_float.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_total_red_float.font.bold = True

        style_red_float_aqua = XFStyle()
        style_red_float_aqua.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_red_float_aqua.pattern = pattern_aqua

        style_total_red_float_bordered = XFStyle()
        style_total_red_float_bordered.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_total_red_float_bordered.font.bold = True
        style_total_red_float_bordered.borders = all_borders

        style_total_red_float_right = XFStyle()
        style_total_red_float_right.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_total_red_float_right.font.bold = True
        style_total_red_float_right.borders = border_right

        style_total_red_float_wo_border = XFStyle()
        style_total_red_float.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_total_red_float_wo_border.font.bold = True

        style_float = XFStyle()
        style_float.num_format_str = '#,##0.00'

        style_float_aqua = XFStyle()
        style_float_aqua.num_format_str = '#,##0.00'
        style_float_aqua.pattern = pattern_aqua

        style_float_right = XFStyle()
        style_float_right.num_format_str = '#,##0.00'
        style_float_right.borders = border_right
        
        style_float_both = XFStyle()
        style_float_both.num_format_str = '#,##0.00'
        style_float_both.borders = borders_lr

        style_float_lrb = XFStyle()
        style_float_lrb.num_format_str = '#,##0.00'
        style_float_lrb.borders = borders_lrb

        style_total_float = XFStyle()
        style_total_float.num_format_str = '#,##0.00'
        style_total_float.font.bold = True

        style_total_float_bordered = XFStyle()
        style_total_float_bordered.num_format_str = '#,##0.00'
        style_total_float_bordered.font.bold = True
        style_total_float_bordered.borders = all_borders

        style_total_float_right = XFStyle()
        style_total_float_right.num_format_str = '#,##0.00'
        style_total_float_right.font.bold = True
        style_total_float_right.borders = border_right

        style_total = XFStyle()
        style_total.font.bold = True

        style_total_right = XFStyle()
        style_total_right.font.bold = True
        style_total_right.borders = border_right

        style_total_bordered = XFStyle()
        style_total_bordered.font.bold = True
        style_total_bordered.borders = all_borders

        style_pourcent = XFStyle()
        style_pourcent.num_format_str = '0.00%'

        style_pourcent_right = XFStyle()
        style_pourcent_right.num_format_str = '0.00%'
        style_pourcent_right.borders = border_right

        style_pourcent_total_bordered = XFStyle()
        style_pourcent_total_bordered.num_format_str = '0.00%'
        style_pourcent_total_bordered.font.bold = True
        style_pourcent_total_bordered.borders = all_borders

        style_pourcent_subtotal = XFStyle()
        style_pourcent_subtotal.num_format_str = '0.00%'
        style_pourcent_subtotal.font.bold = True
        style_pourcent_subtotal.borders = border_bottom

        style_pourcent_subtotal_right = XFStyle()
        style_pourcent_subtotal_right.num_format_str = '0.00%'
        style_pourcent_subtotal_right.font.bold = True
        style_pourcent_subtotal_right.borders = border_right_and_bottom

        style_grand_total = XFStyle()
        style_grand_total.font.bold = True
        style_grand_total.borders = all_borders
        style_grand_total.alignment = al
        style_grand_total.font = font12
        style_grand_total.pattern = pattern_yellow

        style_project_title = XFStyle()
        style_project_title.font = font_title_project
        
        ws1 = wb1.add_sheet('Global')
        ws1.write(0,0,_('Comparison between 2 budgets'))
        ws1.write(2,0,_('Extract date:'))
        #ws1.write(2,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws1.write(2,1,string_now)
        ws1.write(3,0,_('Budget 1:'))
        ws1.write(3,1,budget1.name)
        ws1.write(3,2,_('From'))
        ws1.write(3,3,period1_from.name)
        ws1.write(3,4,_('To'))
        ws1.write(3,5,period1_to.name)
        ws1.write(4,0,_('Budget 2:'))
        ws1.write(4,1,budget2.name)
        ws1.write(4,2,_('From'))
        ws1.write(4,3,period2_from.name)
        ws1.write(4,4,_('To'))
        ws1.write(4,5,period2_to.name)
        ws1.write(6,2,budget1.name)
        ws1.write(6,5,budget2.name)
        ws1.write(7,0,_('Categories'),style_total_bordered)
        ws1.write(7,1,_('Products'),style_total_bordered)
        if output_type == 'ca':
            ws1.write(7,2,_('C.A.'),style_total_bordered)
            ws1.write(7,3,_('C.A. Budget'),style_total_bordered)
        else:
            ws1.write(7,2,_('Real'),style_total_bordered)
            ws1.write(7,3,_('Budget'),style_total_bordered)
        ws1.write(7,4,_('Difference'),style_total_bordered)
        if output_type == 'ca':
            ws1.write(7,5,_('C.A.'),style_total_bordered)
            ws1.write(7,6,_('C.A. Budget'),style_total_bordered)
        else:
            ws1.write(7,5,_('Real'),style_total_bordered)
            ws1.write(7,6,_('Budget'),style_total_bordered)
        ws1.write(7,7,_('Difference'),style_total_bordered)
        ws1.write(7,9,_('Budget difference'),style_total_bordered)
        line = 8
        
        # now we get cumulated data by post, sorted by department and doing so, we create detail sheet for each post by group, under-group, account
        # for the general sheet we need the list of departments then post
        dpt_obj = pooler.get_pool(cr.dbname).get('cci_budget.department')
        dpt_ids = dpt_obj.search(cr,uid,[])
        dpts = dpt_obj.read(cr,uid,dpt_ids,['name','sequence'])
        sdpts = sorted(dpts, key=lambda k: k['sequence'])
        # for the detail sheets, we need the list of groups then subgroups then accounts
        group_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget_account_category')
        group_ids = group_obj.search(cr,uid,[])
        groups = group_obj.read(cr,uid,group_ids,['name','sequence'])
        sgroups = sorted(groups, key=lambda k: k['sequence'])
        subgroup_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget_account_group')
        subgroup_ids = subgroup_obj.search(cr,uid,[])
        subgroups = subgroup_obj.read(cr,uid,subgroup_ids,['name','category_id'])
        ssubgroups = sorted(subgroups, key=lambda k: k['name'])
        #
        gen_total_turnover = 0.0
        gen_total_budget = 0.0
        sum_gen_lines = []
        subtotal_pourcent_lines = []
        gen_membership = 0.0
        gen_sponsor_structural = 0.0
        gen_sponsor_product = 0.0
        gen_subsidies = 0.0
        for dpt in sdpts:
            post_ids = post_obj.search(cr,uid,[('department_id','=',dpt['id'])])
            line_beg_dpt = line
            if post_ids:
                posts = post_obj.read(cr,uid,post_ids,['name'])
                sposts = sorted(posts, key=lambda k: k['name'])
                post_num = 1
                dpt_total_ca1 = 0.0
                dpt_total_ca_budget1 = 0.0
                dpt_total_turnover1 = 0.0
                dpt_total_budget1 = 0.0
                dpt_total_ca2 = 0.0
                dpt_total_ca_budget2 = 0.0
                dpt_total_turnover2 = 0.0
                dpt_total_budget2 = 0.0
                # details
                post_total_turnover = 0.0
                for post in sposts:
                    dAccounts = {}
                    dSubGroups = {}
                    dGroups = {}
                    total_ca1 = 0.0
                    total_ca_budget1 = 0.0
                    total_turnover1 = 0.0
                    total_budget1 = 0.0
                    total_ca2 = 0.0
                    total_ca_budget2 = 0.0
                    total_turnover2 = 0.0
                    total_budget2 = 0.0
                    for key,record in dResults.items():
                        if key[0] == post['id']:
                            total_ca1 += record['ca1']
                            total_ca_budget1 += record['ca_budget1']
                            total_turnover1 += record['turnover1']
                            total_budget1 += record['budget1']
                            total_ca2 += record['ca2']
                            total_ca_budget2 += record['ca_budget2']
                            total_turnover2 += record['turnover2']
                            total_budget2 += record['budget2']
                            # put the same data in the detail sheet
                            account = account_obj.browse(cr,uid,key[1])
                            if account.id not in dAccounts.keys():
                                sub = {'id':account.id,'ca1':0.0,'ca_budget1':0.0,'turnover1':0.0,'budget1':0.0,'ca2':0.0,'ca_budget2':0.0,'turnover2':0.0,'budget2':0.0}
                            else:
                                sub = dAccounts[account.id]
                            sub['ca1'] += record['ca1']
                            sub['ca_budget1'] += record['ca_budget1']
                            sub['turnover1'] += record['turnover1']
                            sub['budget1'] += record['budget1']
                            sub['ca2'] += record['ca2']
                            sub['ca_budget2'] += record['ca_budget2']
                            sub['turnover2'] += record['turnover2']
                            sub['budget2'] += record['budget2']
                            dAccounts[account.id] = sub
                            if account.cci_budget_group_id.id not in dSubGroups.keys():
                                sub = {'id':account.id,'ca1':0.0,'ca_budget1':0.0,'turnover1':0.0,'budget1':0.0,'ca2':0.0,'ca_budget2':0.0,'turnover2':0.0,'budget2':0.0}
                            else:
                                sub = dSubGroups[account.cci_budget_group_id.id]
                            sub['ca1'] += record['ca1']
                            sub['ca_budget1'] += record['ca_budget1']
                            sub['turnover1'] += record['turnover1']
                            sub['budget1'] += record['budget1']
                            sub['ca2'] += record['ca2']
                            sub['ca_budget2'] += record['ca_budget2']
                            sub['turnover2'] += record['turnover2']
                            sub['budget2'] += record['budget2']
                            dSubGroups[account.cci_budget_group_id.id] = sub
                            if account.cci_budget_group_id.category_id.id not in dGroups.keys():
                                sub = {'id':account.id,'ca1':0.0,'ca_budget1':0.0,'turnover1':0.0,'budget1':0.0,'ca2':0.0,'ca_budget2':0.0,'turnover2':0.0,'budget2':0.0}
                            else:
                                sub = dGroups[account.cci_budget_group_id.category_id.id]
                            sub['ca1'] += record['ca1']
                            sub['ca_budget1'] += record['ca_budget1']
                            sub['turnover1'] += record['turnover1']
                            sub['budget1'] += record['budget1']
                            sub['ca2'] += record['ca2']
                            sub['ca_budget2'] += record['ca_budget2']
                            sub['turnover2'] += record['turnover2']
                            sub['budget2'] += record['budget2']
                            dGroups[account.cci_budget_group_id.category_id.id] = sub
                    if post_num == 1:
                        ws1.write(line,0,dpt['name'])
                    ws1.write(line,1,post['name'],style_right)
                    if output_type == 'ca':
                        ws1.write(line,2,round(total_ca1,2),style_float)
                        ws1.write(line,3,round(total_ca_budget1,2),style_float)
                        ws1.write(line,4,Formula("C%s-D%s" % (str(line+1),str(line+1))),style_red_float)
                        ws1.write(line,5,round(total_ca2,2),style_float)
                        ws1.write(line,6,round(total_ca_budget2,2),style_float_right)
                        ws1.write(line,7,Formula("F%s-G%s" % (str(line+1),str(line+1))),style_red_float)
                    else:
                        ws1.write(line,2,round(total_turnover1,2),style_float)
                        ws1.write(line,3,round(total_budget1,2),style_float)
                        ws1.write(line,4,Formula("C%s-D%s" % (str(line+1),str(line+1))),style_red_float)
                        ws1.write(line,5,round(total_turnover2,2),style_float)
                        ws1.write(line,6,round(total_budget2,2),style_float)
                        ws1.write(line,7,Formula("F%s-G%s" % (str(line+1),str(line+1))),style_red_float)
                    ws1.write(line,9,Formula("G%s-D%s" % (str(line+1),str(line+1))),style_lr_red_float)
                    post_num += 1
                    line += 1
                    dpt_total_ca1 += total_ca1
                    dpt_total_ca_budget1 += total_ca_budget1
                    dpt_total_turnover1 += total_turnover1
                    dpt_total_budget1 += total_budget1
                    dpt_total_ca2 += total_ca2
                    dpt_total_ca_budget2 += total_ca_budget2
                    dpt_total_turnover2 += total_turnover2
                    dpt_total_budget2 += total_budget2
                    
                    # put the details on a specific sheet by post
                    if dAccounts: # if there is any moves on this post
                        wso = wb1.add_sheet('Project '+post['name'])

                        wso.write(0,0,_('Comparison between 2 budgets'))
                        wso.write(2,0,_('Extract date:'))
                        wso.write(2,1,string_now)
                        wso.write(3,0,_('Budget 1:'))
                        wso.write(3,1,budget1.name)
                        wso.write(3,2,_('From'))
                        wso.write(3,3,period1_from.name)
                        wso.write(3,4,_('To'))
                        wso.write(3,5,period1_to.name)
                        wso.write(4,0,_('Budget 2:'))
                        wso.write(4,1,budget2.name)
                        wso.write(4,2,_('From'))
                        wso.write(4,3,period2_from.name)
                        wso.write(4,4,_('To'))
                        wso.write(4,5,period2_to.name)
                        wso.write(6,2,budget1.name)
                        wso.write(6,5,budget2.name)
                        
                        wso.write(8,0,'Project '+post['name'],style_project_title)
                        wso.write(10,1,budget1.name,style_total_bordered)
                        wso.write(10,4,budget2.name,style_total_bordered)
                        wso.write(11,0,_('Group/Sub/Account'),style_total_bordered)
                        if output_type == 'ca':
                            wso.write(11,1,_('CA'),style_total_bordered)
                            wso.write(11,2,_('CA Budget'),style_total_bordered)
                            wso.write(11,4,_('CA'),style_total_bordered)
                            wso.write(11,5,_('CA Budget'),style_total_bordered)
                        else:
                            wso.write(11,1,_('Real'),style_total_bordered)
                            wso.write(11,2,_('Budget'),style_total_bordered)
                            wso.write(11,4,_('Real'),style_total_bordered)
                            wso.write(11,5,_('Budget'),style_total_bordered)
                        wso.write(11,3,_('Difference'),style_total_bordered)
                        wso.write(11,6,_('Difference'),style_total_bordered)
                        wso.write(11,8,_('Budgets Difference'),style_total_bordered)
                        wso.write(12,0,_('TOTAL'),style_total_bordered)
                        line_other = 12
                        det_sum_lines = []
                        line_gen = line_other
                        for group in sgroups:
                            line_other += 1
                            wso.write(line_other,0,group['name'],style_total_right)
                            group_sum_lines = []
                            line_group = line_other
                            for subgroup in subgroups:
                                subgroup_present = False
                                if subgroup['category_id'][0] == group['id']:
                                    for acc in saccounts:
                                        if acc['cci_budget_group_id'][0] == subgroup['id'] and acc['id'] in dAccounts.keys():
                                            subgroup_present = True
                                if subgroup_present:
                                    line_other += 1
                                    wso.write(line_other,0,' >--'+subgroup['name'],style_total_right)
                                    subgroup_sum_lines = []
                                    line_subgroup = line_other
                                    for acc in saccounts:
                                        if acc['cci_budget_group_id'][0] == subgroup['id'] and acc['id'] in dAccounts.keys():
                                            line_other += 1
                                            wso.write(line_other,0,' >-- >--'+acc['code']+' '+acc['name']) ## ,style_total_right
                                            if output_type == 'ca':
                                                wso.write(line_other,1,round(dAccounts[acc['id']]['ca1'],2),style_float_aqua)
                                                wso.write(line_other,2,round(dAccounts[acc['id']]['ca_budget1'],2),style_float_aqua)
                                                wso.write(line_other,3,Formula("B%s-C%s" % (str(line_other+1),str(line_other+1))),style_red_float_aqua)
                                                wso.write(line_other,4,round(dAccounts[acc['id']]['ca2'],2),style_float_aqua)
                                                wso.write(line_other,5,round(dAccounts[acc['id']]['ca_budget2'],2),style_float_aqua)
                                                wso.write(line_other,6,Formula("E%s-F%s" % (str(line_other+1),str(line_other+1))),style_red_float_aqua)
                                                wso.write(line_other,8,Formula("F%s-C%s" % (str(line_other+1),str(line_other+1))),style_red_float_aqua)
                                            else:
                                                wso.write(line_other,1,round(dAccounts[acc['id']]['turnover1'],2),style_float_aqua)
                                                wso.write(line_other,2,round(dAccounts[acc['id']]['budget1'],2),style_float_aqua)
                                                wso.write(line_other,3,Formula("B%s-C%s" % (str(line_other+1),str(line_other+1))),style_red_float_aqua)
                                                wso.write(line_other,4,round(dAccounts[acc['id']]['turnover2'],2),style_float_aqua)
                                                wso.write(line_other,5,round(dAccounts[acc['id']]['budget2'],2),style_float_aqua)
                                                wso.write(line_other,6,Formula("E%s-F%s" % (str(line_other+1),str(line_other+1))),style_red_float_aqua)
                                                wso.write(line_other,8,Formula("F%s-C%s" % (str(line_other+1),str(line_other+1))),style_red_float_aqua)
                                            subgroup_sum_lines.append(line_other)
                                    for col in range(1,7)+[8,]:
                                        if col in (3,6):
                                            wso.write(line_subgroup,col,Formula("SUM(%s)" % ';'.join([full_cell(x,col) for x in subgroup_sum_lines])),style_total_float) # ,style_total_red_float_right
                                        elif col in (8,):
                                            wso.write(line_subgroup,col,Formula("SUM(%s)" % ';'.join([full_cell(x,col) for x in subgroup_sum_lines])),style_total_float)
                                        else:
                                            wso.write(line_subgroup,col,Formula("SUM(%s)" % ';'.join([full_cell(x,col) for x in subgroup_sum_lines])),style_total_float)
                                    group_sum_lines.append(line_subgroup)
                            if group_sum_lines:
                                for col in range(1,7)+[8,]:
                                    if col in (3,6):
                                        wso.write(line_group,col,Formula("SUM(%s)" % ';'.join([full_cell(x,col) for x in group_sum_lines])),style_total_float)
                                    elif col in (8,):
                                        wso.write(line_group,col,Formula("SUM(%s)" % ';'.join([full_cell(x,col) for x in group_sum_lines])),style_total_float)
                                    else:
                                        wso.write(line_group,col,Formula("SUM(%s)" % ';'.join([full_cell(x,col) for x in group_sum_lines])),style_total_float)
                            else:
                                for col in range(1,7)+[8,]:
                                    if col in (3,6):
                                        wso.write(line_group,col,0.0,style_total_float)
                                    elif col in (8,):
                                        wso.write(line_group,col,0.0,style_total_float)
                                    else:
                                        wso.write(line_group,col,0.0,style_total_float)
                            det_sum_lines.append(line_group)
                        for col in range(1,7)+[8,]:
                            if col in (3,6):
                                wso.write(line_gen,col,Formula("SUM(%s)" % ';'.join([full_cell(x,col) for x in det_sum_lines])),style_total_float)
                            elif col in (8,):
                                wso.write(line_gen,col,Formula("SUM(%s)" % ';'.join([full_cell(x,col) for x in det_sum_lines])),style_total_float)
                            else:
                                wso.write(line_gen,col,Formula("SUM(%s)" % ';'.join([full_cell(x,col) for x in det_sum_lines])),style_total_float)
                        wso = None
            else:
                # department without posts
                ws1.write(line,0,dpt['name'])
                ws1.write(line,1,'/')
                ws1.write(line,2,0.0,style_float_aqua)
                ws1.write(line,3,0.0,style_float_aqua)
                ws1.write(line,4,0.0,style_red_red_float_aqua)
                ws1.write(line,5,0.0,style_float_aqua)
                ws1.write(line,6,0.0,style_float_aqua)
                ws1.write(line,7,0.0,style_red_float_aqua)
                ws1.write(line,9,0.0,style_bordered_red_float)
            ws1.write(line,1,'',style_float_right)
            ws1.write(line,4,'',style_float_right)
            ws1.write(line,7,'',style_float_right)
            ws1.write(line,9,'',style_float_both)
            line += 1
            ws1.write(line,1,_('Sub total'),style_float_right)
            #ws1.write(line,2,round(dpt_total_turnover,2))
            #ws1.write(line,3,round(dpt_total_budget,2))
            #ws1.write(line,4,round(dpt_total_turnover-dpt_total_budget,2))
            ws1.write(line,2,Formula("SUM($C$%s:$C$%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_float)
            ws1.write(line,3,Formula("SUM($D$%s:$D$%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_float)
            ws1.write(line,4,Formula("SUM($E$%s:$E$%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_red_float_right)
            ws1.write(line,5,Formula("SUM($F$%s:$F$%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_float)
            ws1.write(line,6,Formula("SUM($G$%s:$G$%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_float)
            ws1.write(line,7,Formula("SUM($H$%s:$H$%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_red_float_right)
            ws1.write(line,9,Formula("SUM($J$%s:$J$%s)" % (str(line_beg_dpt+1),str(line-1))),style_lr_red_float)
            sum_gen_lines.append(line)
            line += 1
            ws1.write(line,0,'',style_pourcent_subtotal)
            ws1.write(line,1,_('%'),style_pourcent_subtotal_right) # the formulas will be written after, once the grand total line known
            ws1.write(line,4,'',style_float_right)
            ws1.write(line,7,'',style_float_right)
            ws1.write(line,9,'',style_float_both)
            subtotal_pourcent_lines.append(line)
            line += 1
            #gen_total_turnover += dpt_total_turnover
            #gen_total_budget += dpt_total_budget
        ws1.write(line,0,_('Grand Total'),style_grand_total)
        ws1.write(line,1,'',style_grand_total)
        for col in range(2,8)+[9,]:
            if col in (4,7):
                ws1.write(line,col,Formula("SUM(%s)" % ';'.join([full_cell(x,col) for x in sum_gen_lines])),style_total_red_float_right)
            elif col in (9,):
                ws1.write(line,col,Formula("SUM(%s)" % ';'.join([full_cell(x,col) for x in sum_gen_lines])),style_lr_red_float)
            else:
                ws1.write(line,col,Formula("SUM(%s)" % ';'.join([full_cell(x,col) for x in sum_gen_lines])),style_total_float)
        # pourcents of subtotals
        for line_num in subtotal_pourcent_lines:
            for col_num in range(2,8)+[9,]:
                if col_num in (4,7):
                    ws1.write(line_num,col_num,Formula("%s/%s" % (full_cell(line_num-1,col_num,'both'),full_cell(line,col_num,'both'))),style_pourcent_subtotal_right)
                elif col_num in (9,):
                    ws1.write(line_num,col_num,'',style_float_lrb)
                else:
                    ws1.write(line_num,col_num,Formula("%s/%s" % (full_cell(line_num-1,col_num,'both'),full_cell(line,col_num,'both'))),style_pourcent_subtotal)
        wb1.save('compare_budgets.xls')
        result_file = open('compare_budgets.xls','rb').read()

        # give the result tos the user
        data['form']['msg']='Save the File with '".xls"' extension.'
        data['form']['compare_budgets.xls']=base64.encodestring(result_file)
        return data['form']
    states = {
        'init': {
            'actions': [_get_defaults],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('extract','Extract') ]}
        },
        'extract': {
            'actions': [_extract_excel],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        }
    }
compare_cci_budgets('compare_cci_budgets')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

