# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import datetime
import wizard
import pooler
from tools.translate import _
import base64
from pyExcelerator import *
from pyExcelerator.Worksheet import *

form = '''<?xml version="1.0"?>
<form string="Select period">
    <field name="fiscalyear" colspan="4"/>
    <newline/>
    <field name="period_from"/>
    <field name="period_to"/>
    <newline/>
    <field name="budget_id" colspan="4"/>
    <field name="period_budget_from"/>
    <field name="period_budget_to"/>
    <newline/>
    <field name="direction"/>
    <field name="move_state"/>
</form>'''

fields = {
    'fiscalyear': {'string': 'Fiscal year', 'type': 'many2one', 'relation': 'account.fiscalyear','required': True},
    'period_from': {'string': 'From Period', 'type': 'many2one', 'relation': 'account.period', 'required': True, 'domain': "[('fiscalyear_id', '=', fiscalyear),('special','=',False)]"},
    'period_to': {'string': 'To Period', 'type': 'many2one', 'relation': 'account.period', 'required': True, 'domain': "[('fiscalyear_id', '=', fiscalyear),('special','=',False)]"},
    'budget_id': {'string': 'Budget', 'type': 'many2one', 'relation': 'cci_budget.budget', 'required': True},
    'period_budget_from': {'string': 'From Period', 'type': 'many2one', 'relation': 'account.period', 'required': True},
    'period_budget_to': {'string': 'To Period', 'type': 'many2one', 'relation': 'account.period', 'required': True},
    'direction': {'string': 'Direction', 'type': 'selection', 'selection': [('1-2','= Reel - Budget'),('2-1','= Budget - Reel')],'help':'if comparing current year with current budget, Reel - Budget; else, better Budget - Reel'},
    'move_state': {'string': 'Move State', 'type': 'selection', 'selection': [('posted','Only posted moves'),('draft','All moves, even draft')], 'default': 'posted', 'required': 'True'}
}
msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="suivi_budgetaire.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'suivi_budgetaire.xls':{'string': 'Prepared file',
        'type': 'binary',
        'readonly': True,},
}

pyexcel_colours = {
        'aqua'    : 0x0F,
        'cyan'    : 0x0F,
        'black'   : 0x08,
        'blue'    : 0x0C,
        'brown'   : 0x10,
        'magenta' : 0x0E,
        'fuchsia' : 0x0E,
        'gray'    : 0x17,
        'grey'    : 0x17,
        'green'   : 0x11,
        'lime'    : 0x0B,
        'navy'    : 0x12,
        'orange'  : 0x35,
        'pink'    : 0x21,
        'purple'  : 0x14,
        'red'     : 0x0A,
        'silver'  : 0x16,
        'white'   : 0x09,
        'yellow'  : 0x0D,
}

def num_to_letter(line_number_in_pyexcel):
    # this method convert an integer to a line label in Excel
    # first line = 0 (beware on pyExcelerator, first line = 0)
    # 0 => 'A'
    # 26 => 'AA'
    # 27 => 'AB'
    line_number_in_pyexcel += 1
    if line_number_in_pyexcel < 26:
        result = chr(64+line_number_in_pyexcel)
    else:
        first = line_number_in_pyexcel_number_in_pyexcel // 26
        last = number % 26
        result = chr(64+first)+chr(64+last)
    return result
def full_cell(line_number_in_pyexcel,col_number_in_pyexcel,fixed='none'):
    # fixed = none, col, line, both
    result = ''
    if fixed in ('col','both'):
        result += '$'
    result += num_to_letter(col_number_in_pyexcel)
    if fixed in ('line','both'):
        result += '$'
    result += str(line_number_in_pyexcel+1)
    return result
def _get_total_type(account_code):
    if account_code == '707000':
        return 'membership'
    elif account_code == '709001':
        return 'structural_sponsor'
    elif account_code == '709000':
        return 'product_sponsor'
    elif account_code[0:2] == '74':
        return 'subsidy'
    else:
        return 'normal'
class compare_turnover_cci_budget(wizard.interface):
    def _get_defaults(self, cr, uid, data, context):
        today = datetime.date.today()
        fiscalyear_obj = pooler.get_pool(cr.dbname).get('account.fiscalyear')
        if today.month <= 2:
            fiscalyear_ids = fiscalyear_obj.search(cr,uid,[('date_start','=',str(today.year-1)+'-01-01')])
        else:
            fiscalyear_ids = fiscalyear_obj.search(cr,uid,[('date_start','=',str(today.year)+'-01-01')])
        if fiscalyear_ids:
            fiscalyear_id = fiscalyear_ids[0]
        else:
            fiscalyear_id = fiscalyear_obj.find(cr,uid)
        data['form']['fiscalyear'] = fiscalyear_id
        return data['form']

    def _extract_excel(self, cr, uid, data, context):
        string_now = datetime.datetime.now().strftime('%d/%m/%Y - %H:%M:%S')
        # selected data
        fiscalyear_obj = pooler.get_pool(cr.dbname).get('account.fiscalyear')
        budget_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget')
        period_obj = pooler.get_pool(cr.dbname).get('account.period')
        account_obj = pooler.get_pool(cr.dbname).get('account.account')
        aaccount_obj = pooler.get_pool(cr.dbname).get('account.analytic.account')
        amline_obj = pooler.get_pool(cr.dbname).get('account.move.line')
        dotation_obj = pooler.get_pool(cr.dbname).get('cci_budget.dotation')
        per_dot_obj = pooler.get_pool(cr.dbname).get('cci_budget.period_dotation')
        post_obj = pooler.get_pool(cr.dbname).get('cci_budget.post')
        
        fiscalyear = fiscalyear_obj.browse(cr,uid,data['form']['fiscalyear'])
        budget = budget_obj.browse(cr,uid,data['form']['budget_id'])
        period_from = period_obj.browse(cr,uid,data['form']['period_from'])
        period_to = period_obj.browse(cr,uid,data['form']['period_to'])
        concerned_period_ids = period_obj.search(cr,uid,[('date_start','>=',period_from.date_start),('date_stop','<=',period_to.date_stop),('special','=',False)])
        period_budget_from = period_obj.browse(cr,uid,data['form']['period_budget_from'])
        period_budget_to = period_obj.browse(cr,uid,data['form']['period_budget_to'])
        concerned_period_budget_ids = period_obj.search(cr,uid,[('date_start','>=',period_budget_from.date_start),('date_stop','<=',period_budget_to.date_stop),('special','=',False)])
        
        dResults = {}
        dAnalytic2Post = {}
        dAccount2Group = {}
        account_ids = account_obj.search(cr,uid,[('cci_budget_group_id','!=',False)])
        accounts = account_obj.read(cr,uid,account_ids,['cci_budget_group_id','code','name'])
        for account in accounts:
            dAccount2Group[account['id']] = account['cci_budget_group_id'][0]
        saccounts = sorted(accounts, key=lambda k: k['code'])
        aaccount_ids = aaccount_obj.search(cr,uid,[('cci_post_id','!=',False)])
        aaccounts = aaccount_obj.read(cr,uid,aaccount_ids,['cci_post_id'])
        for aaccount in aaccounts:
            dAnalytic2Post[aaccount['id']] = aaccount['cci_post_id'][0]

        dotation_ids = dotation_obj.search(cr,uid,[('budget_id','=',budget.id)])
        dotations = dotation_obj.browse(cr,uid,dotation_ids)
        dResults = {}
        concerned_account_ids = []
        concerned_aaccount_ids = []
        for dot in dotations:
            dResults[(dot.post_id.id,dot.account_id.id)] = {'dpt_name': dot.post_id.department_id.name,
                                                            'post_name': dot.post_id.name,
                                                            'category_name': dot.account_id.cci_budget_group_id.category_id.name,
                                                            'group_name': dot.account_id.cci_budget_group_id.name,
                                                            'account_name': dot.account_id.code + ' ' + dot.account_id.name,
                                                            'ca':0.0,
                                                            'ca_budget':0.0,
                                                            'turnover':0.0,
                                                            'budget':0.0,
                                                            'type':_get_total_type(dot.account_id.code)}
            if dot.account_id.id not in concerned_account_ids:
                concerned_account_ids.append(dot.account_id.id)
            #new_aaccount_ids = aaccount_obj.search(cr,uid,[('cci_post_id','=',dot.post_id.id)])
            #if new_aaccount_ids:
            #    for new_aaccount_id in new_aaccount_ids:
            #        if new_aaccount_id not in concerned_aaccount_ids:
            #            concerned_aaccount_ids.append(new_aaccount_id)
        concerned_aaccount_ids = aaccount_obj.search(cr,uid,[('cci_post_id','!=',False)])
        period_dotation_ids = per_dot_obj.search(cr,uid,[('dotation_id','in',dotation_ids),('period_id','in',concerned_period_budget_ids)])
        if period_dotation_ids:
            period_dotations = per_dot_obj.browse(cr,uid,period_dotation_ids)
            for per_dot in period_dotations:
                if per_dot.dotation_id.post_id and per_dot.dotation_id.account_id:
                    if dResults.has_key((per_dot.dotation_id.post_id.id,per_dot.dotation_id.account_id.id)):
                        record = dResults[(per_dot.dotation_id.post_id.id,per_dot.dotation_id.account_id.id)]
                        if per_dot.dotation_id.account_id.cci_budget_group_id.category_id.is_ca:
                            record['ca_budget'] += (per_dot.amount or 0.0)
                        record['budget'] += (per_dot.amount or 0.0)
                        dResults[(per_dot.dotation_id.post_id.id,per_dot.dotation_id.account_id.id)] = record
        
        amline_ids = amline_obj.search(cr,uid,[('period_id','in',concerned_period_ids)])
        index = 0
        size = 1000
        while amline_ids[index:index+size]:
            amlines = amline_obj.browse(cr,uid,amline_ids[index:index+size])
            for amline in amlines:
                if amline.analytic_lines:
                    for analytic_line in amline.analytic_lines:
                        if analytic_line.account_id.id in concerned_aaccount_ids and dAnalytic2Post.has_key(analytic_line.account_id.id) and ((data['form']['move_state'] == 'draft') or (amline.move_id.state == data['form']['move_state'])):
                            if dResults.has_key((dAnalytic2Post[analytic_line.account_id.id],amline.account_id.id)):
                                record = dResults[(dAnalytic2Post[analytic_line.account_id.id],amline.account_id.id)]
                                if amline.account_id.cci_budget_group_id.category_id.is_ca:
                                    record['ca'] += analytic_line.amount
                                record['turnover'] += analytic_line.amount
                                dResults[(dAnalytic2Post[analytic_line.account_id.id],amline.account_id.id)] = record
                            else:
                                post = post_obj.browse(cr,uid,dAnalytic2Post[analytic_line.account_id.id])
                                if amline.account_id and amline.account_id.cci_budget_group_id and amline.account_id.cci_budget_group_id.category_id:
                                    dResults[(dAnalytic2Post[analytic_line.account_id.id],amline.account_id.id)] = \
                                                {'dpt_name': post.department_id.name,
                                                 'post_name': post.name,
                                                 'category_name': amline.account_id.cci_budget_group_id.category_id.name,
                                                 'group_name': amline.account_id.cci_budget_group_id.name,
                                                 'account_name': amline.account_id.code + ' ' + amline.account_id.name,
                                                 'ca':amline.account_id.cci_budget_group_id.category_id.is_ca and analytic_line.amount or 0.0,
                                                 'ca_budget':0.0,
                                                 'turnover':analytic_line.amount,
                                                 'budget':0.0,
                                                 'type':_get_total_type(amline.account_id.code)}
            index += size
        wb1 = Workbook()

        # styles
        border_right = Borders()
        border_right.right = 2
        border_right_and_bottom = Borders()
        border_right_and_bottom.right = 2
        border_right_and_bottom.bottom = 2
        border_bottom = Borders()
        border_bottom.bottom = 2
        all_borders = Borders()
        all_borders.left = 2
        all_borders.right = 2
        all_borders.top = 2
        all_borders.bottom = 2
        al = Alignment()
        al.horz = Alignment.HORZ_CENTER
        al.vert = Alignment.VERT_CENTER
        font12 = Font()
        font12.height = 240
        font_title_project = Font()
        font_title_project.height = 300
        font_title_project.colour_index = 0x12 ## 'Navy'
        
        pattern_silver = Pattern()
        pattern_silver._pattern_back_colour = 0x16  ### 'silver'
        pattern_silver.pattern = 0x01 # SOLID
        pattern_grey = Pattern()
        pattern_grey._pattern_back_colour = 0x17  ### 'grey'
        pattern_grey.pattern = 0x01 # SOLID
        pattern_yellow = Pattern()
        #pattern_yellow._pattern_back_colour = 0x0D  ### 'grey'
        pattern_yellow.pattern = 0x01 # SOLID
        pattern_yellow.set_pattern_fore_colour(0x0D)  ### 'yellow'
        pattern_aqua = Pattern()
        pattern_aqua.pattern = 0x01 # SOLID
        pattern_aqua.set_pattern_fore_colour(pyexcel_colours['aqua'])

        style_right = XFStyle()
        style_right.borders = border_right

        style_red_float = XFStyle()
        style_red_float.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_red_float.borders = border_right
        
        style_red_float_wo_border = XFStyle()
        style_red_float_wo_border.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'

        style_total_red_float = XFStyle()
        style_total_red_float.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_total_red_float.font.bold = True

        style_total_red_float_bordered = XFStyle()
        style_total_red_float_bordered.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_total_red_float_bordered.font.bold = True
        style_total_red_float_bordered.borders = all_borders

        style_total_red_float_right = XFStyle()
        style_total_red_float_right.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_total_red_float_right.font.bold = True
        style_total_red_float_right.borders = border_right

        style_total_red_float_wo_border = XFStyle()
        style_total_red_float_wo_border.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_total_red_float_wo_border.font.bold = True

        style_red_float_aqua = XFStyle()
        style_red_float_aqua.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_red_float_aqua.pattern = pattern_aqua

        style_float = XFStyle()
        style_float.num_format_str = '#,##0.00'

        style_float_aqua = XFStyle()
        style_float_aqua.num_format_str = '#,##0.00'
        style_float_aqua.pattern = pattern_aqua

        style_float_right = XFStyle()
        style_float_right.num_format_str = '#,##0.00'
        style_float_right.borders = border_right
        
        style_total_float = XFStyle()
        style_total_float.num_format_str = '#,##0.00'
        style_total_float.font.bold = True

        style_total_float_bordered = XFStyle()
        style_total_float_bordered.num_format_str = '#,##0.00'
        style_total_float_bordered.font.bold = True
        style_total_float_bordered.borders = all_borders

        style_total_float_right = XFStyle()
        style_total_float_right.num_format_str = '#,##0.00'
        style_total_float_right.font.bold = True
        style_total_float_right.borders = border_right

        style_total = XFStyle()
        style_total.font.bold = True

        style_total_right = XFStyle()
        style_total_right.font.bold = True
        style_total_right.borders = border_right

        style_total_bordered = XFStyle()
        style_total_bordered.font.bold = True
        style_total_bordered.borders = all_borders

        style_pourcent = XFStyle()
        style_pourcent.num_format_str = '0.00%'

        style_pourcent_right = XFStyle()
        style_pourcent_right.num_format_str = '0.00%'
        style_pourcent_right.borders = border_right

        style_pourcent_total_bordered = XFStyle()
        style_pourcent_total_bordered.num_format_str = '0.00%'
        style_pourcent_total_bordered.font.bold = True
        style_pourcent_total_bordered.borders = all_borders

        style_pourcent_subtotal = XFStyle()
        style_pourcent_subtotal.num_format_str = '0.00%'
        style_pourcent_subtotal.font.bold = True
        style_pourcent_subtotal.borders = border_bottom

        style_pourcent_subtotal_right = XFStyle()
        style_pourcent_subtotal_right.num_format_str = '0.00%'
        style_pourcent_subtotal_right.font.bold = True
        style_pourcent_subtotal_right.borders = border_right_and_bottom

        style_grand_total = XFStyle()
        style_grand_total.font.bold = True
        style_grand_total.borders = all_borders
        style_grand_total.alignment = al
        style_grand_total.font = font12
        style_grand_total.pattern = pattern_yellow

        style_project_title = XFStyle()
        style_project_title.font = font_title_project
        
        ws1 = wb1.add_sheet('Global')
        ws1.write(0,0,_('CCI PRODUCTS')+' '+period_from.name + ' -> ' +period_to.name)
        ws1.write(2,0,_('Extract date:'))
        ws1.write(2,1,string_now)
        ws1.write(3,0,_('Budget:'))
        ws1.write(3,1,budget.name + ' ' + period_budget_from.name + ' -> ' + period_budget_to.name  + ((data['form']['move_state'] == 'draft') and _(' (All even draft)') or _(' (only posted)')) )
        ws1.write(5,0,_('Categories'),style_total_bordered)
        ws1.write(5,1,_('Products'),style_total_bordered)
        ws1.write(5,2,_('C.A. Reel'),style_total_bordered)
        ws1.write(5,3,_('C.A. Budget'),style_total_bordered)
        ws1.write(5,4,_('C.A. Diff'),style_total_bordered)
        ws1.write(5,5,_('Reel Margin'),style_total_bordered)
        ws1.write(5,6,_('Budget Margin'),style_total_bordered)
        ws1.write(5,7,_('Difference'),style_total_bordered)
        ws1.write(5,8,_('Reel Struct & MBS'),style_total_bordered)
        ws1.write(5,9,_('Reel Spons Product & Subs'),style_total_bordered)
        ws1.write(5,10,_('Reel Margin 2'),style_total_bordered)
        ws1.write(5,11,_('Reel Margin 3'),style_total_bordered)
        
        col_final = 11
        
        ws2 = wb1.add_sheet(_('Details'))
        ws2.write(0,0,'post_id',style_total_bordered)
        ws2.write(0,1,'account_id',style_total_bordered)
        ws2.write(0,2,_('Department'),style_total_bordered)
        ws2.write(0,3,_('Project'),style_total_bordered)
        ws2.write(0,4,_('Group'),style_total_bordered)
        ws2.write(0,5,_('SubGroup'),style_total_bordered)
        ws2.write(0,6,_('Account'),style_total_bordered)
        ws2.write(0,7,_('Turnover'),style_total_bordered)
        ws2.write(0,8,_('Budget'),style_total_bordered)

        line = 6
        line2 = 1
        for key,record in dResults.items():
            ws2.write(line2,0,key[0])
            ws2.write(line2,1,key[1])
            ws2.write(line2,2,record['dpt_name'])
            ws2.write(line2,3,record['post_name'])
            ws2.write(line2,4,record['category_name'])
            ws2.write(line2,5,record['group_name'])
            ws2.write(line2,6,record['account_name'])
            ws2.write(line2,7,record['turnover'])
            ws2.write(line2,8,record['budget'])
            line2 += 1
        # now we get cumulated data by post, sorted by department and doing so, we create detail sheet for each post by group, under-group, account
        # for the general sheet we need the list of departments then post
        dpt_obj = pooler.get_pool(cr.dbname).get('cci_budget.department')
        dpt_ids = dpt_obj.search(cr,uid,[])
        dpts = dpt_obj.read(cr,uid,dpt_ids,['name','sequence'])
        sdpts = sorted(dpts, key=lambda k: k['sequence'])
        # for the detail sheets, we need the list of groups then subgroups then accounts
        group_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget_account_category')
        group_ids = group_obj.search(cr,uid,[])
        groups = group_obj.read(cr,uid,group_ids,['name','sequence'])
        sgroups = sorted(groups, key=lambda k: k['sequence'])
        subgroup_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget_account_group')
        subgroup_ids = subgroup_obj.search(cr,uid,[])
        subgroups = subgroup_obj.read(cr,uid,subgroup_ids,['name','category_id'])
        ssubgroups = sorted(subgroups, key=lambda k: k['name'])
        #
        gen_total_turnover = 0.0
        gen_total_budget = 0.0
        sum_gen_turnover = []
        sum_gen_budget = []
        sum_gen_diff = []
        sum_gen_ca = []
        sum_gen_ca_budget = []
        subtotal_pourcent_lines = []
        gen_membership = 0.0
        gen_sponsor_structural = 0.0
        gen_sponsor_product = 0.0
        gen_subsidies = 0.0
        for dpt in sdpts:
            post_ids = post_obj.search(cr,uid,[('department_id','=',dpt['id'])])
            line_beg_dpt = line
            if post_ids:
                posts = post_obj.read(cr,uid,post_ids,['name'])
                sposts = sorted(posts, key=lambda k: k['name'])
                post_num = 1
                dpt_total_turnover = 0.0
                dpt_total_budget = 0.0
                # details
                post_total_turnover = 0.0
                for post in sposts:
                    dAccounts = {}
                    dSubGroups = {}
                    dGroups = {}
                    total_ca = 0.0
                    total_ca_budget = 0.0
                    total_turnover = 0.0
                    total_budget = 0.0
                    total_struct_mbs = 0.0
                    total_sponsor_product = 0.0
                    total_subsidies = 0.0
                    for key,record in dResults.items():
                        if key[0] == post['id']:
                            total_ca += record['ca']
                            total_ca_budget += record['ca_budget']
                            total_turnover += record['turnover']
                            total_budget += record['budget']
                            if record['type'] == 'membership':
                                gen_membership += record['turnover']
                                total_struct_mbs += record['turnover']
                            elif record['type'] == 'structural_sponsor':
                                gen_sponsor_structural += record['turnover']
                                total_struct_mbs += record['turnover']
                            elif record['type'] == 'product_sponsor':
                                gen_sponsor_product += record['turnover']
                                total_sponsor_product += record['turnover']
                            elif record['type'] == 'subsidy':
                                gen_subsidies += record['turnover']
                                total_subsidies += record['turnover']
                            # put the same data in the detail sheet
                            account = account_obj.browse(cr,uid,key[1])
                            if account.id not in dAccounts.keys():
                                sub = {'id':account.id,'turnover':0.0,'budget':0.0}
                            else:
                                sub = dAccounts[account.id]
                            sub['turnover'] += record['turnover']
                            sub['budget'] += record['budget']
                            dAccounts[account.id] = sub
                            if account.cci_budget_group_id.id not in dSubGroups.keys():
                                sub = {'id':account.cci_budget_group_id.id,'turnover':0.0,'budget':0.0}
                            else:
                                sub = dSubGroups[account.cci_budget_group_id.id]
                            sub['turnover'] += record['turnover']
                            sub['budget'] += record['budget']
                            dSubGroups[account.cci_budget_group_id.id] = sub
                            if account.cci_budget_group_id.category_id.id not in dGroups.keys():
                                sub = {'id':account.cci_budget_group_id.category_id.id,'turnover':0.0,'budget':0.0}
                            else:
                                sub = dGroups[account.cci_budget_group_id.category_id.id]
                            sub['turnover'] += record['turnover']
                            sub['budget'] += record['budget']
                            dGroups[account.cci_budget_group_id.category_id.id] = sub
                    if post_num == 1:
                        ws1.write(line,0,dpt['name'])
                    ws1.write(line,1,post['name'],style_right)
                    ws1.write(line,2,round(total_ca,2),style_float)
                    ws1.write(line,3,round(total_ca_budget,2),style_float)
                    if data['form']['direction'] == '1-2':
                        ws1.write(line,4,Formula("C%s-D%s" % (str(line+1),str(line+1))),style_red_float)
                    else:
                        ws1.write(line,4,Formula("D%s-C%s" % (str(line+1),str(line+1))),style_red_float)
                    ws1.write(line,5,round(total_turnover,2),style_float)
                    ws1.write(line,6,round(total_budget,2),style_float)
                    if data['form']['direction'] == '1-2':
                        ws1.write(line,7,Formula("F%s-G%s" % (str(line+1),str(line+1))),style_red_float)
                    else:
                        ws1.write(line,7,Formula("G%s-F%s" % (str(line+1),str(line+1))),style_red_float)
                    ws1.write(line,8,round(total_struct_mbs,2),style_float)
                    ws1.write(line,9,round(total_sponsor_product+total_subsidies,2),style_float_right)
                    ws1.write(line,10,Formula("F%s-I%s" % (str(line+1),str(line+1))),style_float)
                    ws1.write(line,11,Formula("F%s-I%s-J%s" % (str(line+1),str(line+1),str(line+1))),style_float_right)
                    post_num += 1
                    line += 1
                    dpt_total_turnover += total_turnover
                    dpt_total_budget += total_budget
                    
                    # put the details on a specific sheet by post
                    if dAccounts: # if there is any moves on this post
                        wso = wb1.add_sheet('Project '+post['name'])
                        wso.write(0,0,'Project '+post['name'],style_project_title)
                        wso.write(2,0,_('Extract date:'))
                        wso.write(2,1,string_now)
                        wso.write(3,0,_('Budget:'))
                        wso.write(3,1,budget.name + ' ' + period_budget_from.name + ' -> ' + period_budget_to.name )
                        wso.write(5,0,_('Group/Sub/Account'),style_total_bordered)
                        wso.write(5,1,_('Turnover'),style_total_bordered)
                        wso.write(5,2,_('Budget'),style_total_bordered)
                        wso.write(5,3,_('Difference'),style_total_bordered)
                        wso.write(6,0,_('TOTAL'),style_total_bordered)
                        line_other = 6
                        det_sum_turnover = []
                        det_sum_budget = []
                        det_sum_diff = []
                        line_gen = line_other
                        for group in sgroups:
                            line_other += 1
                            wso.write(line_other,0,group['name'],style_total)
                            group_sum_turnover = []
                            group_sum_budget = []
                            group_sum_diff = []
                            line_group = line_other
                            for subgroup in subgroups:
                                subgroup_present = False
                                if subgroup['category_id'][0] == group['id']:
                                    for acc in saccounts:
                                        if acc['cci_budget_group_id'][0] == subgroup['id'] and acc['id'] in dAccounts.keys():
                                            subgroup_present = True
                                if subgroup_present:
                                    line_other += 1
                                    wso.write(line_other,0,' >--'+subgroup['name'],style_total_float)
                                    subgroup_sum_turnover = []
                                    subgroup_sum_budget = []
                                    subgroup_sum_diff = []
                                    line_subgroup = line_other
                                    for acc in saccounts:
                                        if acc['cci_budget_group_id'][0] == subgroup['id'] and acc['id'] in dAccounts.keys():
                                            line_other += 1
                                            wso.write(line_other,0,' >-- >--'+acc['code']+' '+acc['name'])
                                            wso.write(line_other,1,round(dAccounts[acc['id']]['turnover'],2),style_float_aqua)
                                            wso.write(line_other,2,round(dAccounts[acc['id']]['budget'],2),style_float_aqua)
                                            if data['form']['direction'] == '1-2':
                                                wso.write(line_other,3,Formula("B%s-C%s" % (str(line_other+1),str(line_other+1))),style_red_float_aqua)
                                            else:
                                                wso.write(line_other,3,Formula("C%s-B%s" % (str(line_other+1),str(line_other+1))),style_red_float_aqua)
                                            subgroup_sum_turnover.append('B' + str(line_other+1))
                                            subgroup_sum_budget.append('C' + str(line_other+1))
                                            subgroup_sum_diff.append('D' + str(line_other+1))
                                    wso.write(line_subgroup,1,Formula("SUM(%s)" % ';'.join(subgroup_sum_turnover)),style_total_float)
                                    wso.write(line_subgroup,2,Formula("SUM(%s)" % ';'.join(subgroup_sum_budget)),style_total_float)
                                    wso.write(line_subgroup,3,Formula("SUM(%s)" % ';'.join(subgroup_sum_diff)),style_total_red_float)
                                    group_sum_turnover.append('B' + str(line_subgroup+1))
                                    group_sum_budget.append('C' + str(line_subgroup+1))
                                    group_sum_diff.append('D' + str(line_subgroup+1))
                            if group_sum_turnover:
                                wso.write(line_group,1,Formula("SUM(%s)" % ';'.join(group_sum_turnover)),style_total_float)
                            else:
                                wso.write(line_group,1,0.0,style_total_float)
                            if group_sum_budget:
                                wso.write(line_group,2,Formula("SUM(%s)" % ';'.join(group_sum_budget)),style_total_float)
                            else:
                                wso.write(line_group,2,0.0,style_total_red_float)
                            if group_sum_diff:
                                wso.write(line_group,3,Formula("SUM(%s)" % ';'.join(group_sum_diff)),style_total_red_float)
                            else:
                                wso.write(line_group,3,0.0,style_total_red_float)
                            det_sum_turnover.append('B' + str(line_group+1))
                            det_sum_budget.append('C' + str(line_group+1))
                            det_sum_diff.append('D' + str(line_group+1))
                        wso.write(line_gen,1,Formula("SUM(%s)" % ';'.join(det_sum_turnover)),style_total_float)
                        wso.write(line_gen,2,Formula("SUM(%s)" % ';'.join(det_sum_budget)),style_total_float)
                        wso.write(line_gen,3,Formula("SUM(%s)" % ';'.join(det_sum_diff)),style_total_red_float)
                        wso = None
            else:
                # department without posts
                ws1.write(line,0,dpt['name'],style_total)
                ws1.write(line,1,'/',style_total)
                ws1.write(line,2,0.0,style_float)
                ws1.write(line,3,0.0,style_float)
                ws1.write(line,4,0.0,style_red_float)
                ws1.write(line,5,0.0,style_float)
                ws1.write(line,6,0.0,style_float)
                ws1.write(line,7,0.0,style_red_float)
                ws1.write(line,8,0.0,style_float)
                ws1.write(line,9,0.0,style_float_right)
                ws1.write(line,10,0.0,style_float)
                ws1.write(line,11,0.0,style_float_right)
            ws1.write(line,1,'',style_right)
            ws1.write(line,4,'',style_float_right)
            ws1.write(line,7,'',style_float_right)
            ws1.write(line,9,'',style_float_right)
            ws1.write(line,11,'',style_float_right)
            line += 1
            ws1.write(line,1,_('Sub total'),style_total_right)
            ws1.write(line,2,Formula("SUM(C%s:C%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_float)
            ws1.write(line,3,Formula("SUM(D%s:D%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_float)
            ws1.write(line,4,Formula("SUM(E%s:E%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_red_float_right)
            ws1.write(line,5,Formula("SUM(F%s:F%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_float)
            ws1.write(line,6,Formula("SUM(G%s:G%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_float)
            ws1.write(line,7,Formula("SUM(H%s:H%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_red_float_right)
            ws1.write(line,8,Formula("SUM(I%s:I%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_float)
            ws1.write(line,9,Formula("SUM(J%s:J%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_float_right)
            ws1.write(line,10,Formula("SUM(K%s:K%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_float)
            ws1.write(line,11,Formula("SUM(L%s:L%s)" % (str(line_beg_dpt+1),str(line-1))),style_total_float_right)
            sum_gen_ca.append("C%s" % str(line+1))
            sum_gen_ca_budget.append("D%s" % str(line+1))
            sum_gen_turnover.append("F%s" % str(line+1))
            sum_gen_budget.append("G%s" % str(line+1))
            sum_gen_diff.append("H%s" % str(line+1))
            line += 1
            ws1.write(line,0,'',style_pourcent_subtotal)
            ws1.write(line,1,_('%'),style_pourcent_subtotal_right) # the formulas will be written after, once the grand total line known
            subtotal_pourcent_lines.append(line)
            line += 1
            gen_total_turnover += dpt_total_turnover
            gen_total_budget += dpt_total_budget
        ws1.write(line,0,_('Grand Total'),style_grand_total)
        #ws1.write(line,2,round(gen_total_turnover,2))
        #ws1.write(line,3,round(gen_total_budget,2))
        #ws1.write(line,4,round(gen_total_turnover-gen_total_budget,2))
        ws1.write(line,1,'',style_grand_total)
        ws1.write(line,2,Formula("SUM(%s)" % ';'.join(sum_gen_ca)),style_total_float_bordered)
        ws1.write(line,3,Formula("SUM(%s)" % ';'.join(sum_gen_ca_budget)),style_total_float_bordered)
        sum_gen = [x.replace('F','E') for x in sum_gen_turnover]
        ws1.write(line,4,Formula("SUM(%s)" % ';'.join(sum_gen)),style_total_float_bordered)
        ws1.write(line,5,Formula("SUM(%s)" % ';'.join(sum_gen_turnover)),style_total_float_bordered)
        ws1.write(line,6,Formula("SUM(%s)" % ';'.join(sum_gen_budget)),style_total_float_bordered)
        ws1.write(line,7,Formula("SUM(%s)" % ';'.join(sum_gen_diff)),style_total_red_float_bordered)
        sum_gen = [x.replace('F','I') for x in sum_gen_turnover]
        ws1.write(line,8,Formula("SUM(%s)" % ';'.join(sum_gen)),style_total_float_bordered)
        sum_gen = [x.replace('F','J') for x in sum_gen_turnover]
        ws1.write(line,9,Formula("SUM(%s)" % ';'.join(sum_gen)),style_total_float_bordered)
        sum_gen = [x.replace('F','K') for x in sum_gen_turnover]
        ws1.write(line,10,Formula("SUM(%s)" % ';'.join(sum_gen)),style_total_float_bordered)
        sum_gen = [x.replace('F','L') for x in sum_gen_turnover]
        ws1.write(line,11,Formula("SUM(%s)" % ';'.join(sum_gen)),style_total_float_bordered)
        
        # pourcents of subtotals
        for line_num in subtotal_pourcent_lines:
            for col_num in range(2,12):
                if col_num in (4,7,9,11):
                    ws1.write(line_num,col_num,Formula("%s/%s" % (full_cell(line_num-1,col_num,'none'),full_cell(line,col_num,'none'))),style_pourcent_subtotal_right)
                else:
                    ws1.write(line_num,col_num,Formula("%s/%s" % (full_cell(line_num-1,col_num,'none'),full_cell(line,col_num,'none'))),style_pourcent_subtotal)
        # sub table with 4 types of "helps"
        ws1.write(5,col_final+2,_('HELPS'),style_total_bordered)
        ws1.write(5,col_final+3,_('Amount'),style_total_bordered)
        ws1.write(5,col_final+4,_('%'),style_total_bordered)
        ws1.write(6,col_final+2,_('Mbs'),style_total_bordered)
        ws1.write(7,col_final+2,_('S Structural'),style_total_bordered)
        ws1.write(8,col_final+2,_('S Product'),style_total_bordered)
        ws1.write(9,col_final+2,_('Subsidies'),style_total_bordered)
        ws1.write(10,col_final+2,_('TOTAL'),style_total_bordered)
        ws1.write(6,col_final+3,gen_membership,style_float)
        ws1.write(7,col_final+3,gen_sponsor_structural,style_float)
        ws1.write(8,col_final+3,gen_sponsor_product,style_float)
        ws1.write(9,col_final+3,gen_subsidies,style_float)
        gen_all = gen_membership + gen_sponsor_structural + gen_sponsor_product + gen_subsidies
        ws1.write(10,col_final+3,gen_all,style_total_float_bordered)
        col_numbers = num_to_letter(col_final+3)
        total_numbers = num_to_letter(col_final+4)
        ws1.write(6,col_final+4,Formula("$%s$7/$%s$11" % (col_numbers,col_numbers)),style_pourcent_right)
        ws1.write(7,col_final+4,Formula("$%s$8/$%s$11" % (col_numbers,col_numbers)),style_pourcent_right)
        ws1.write(8,col_final+4,Formula("$%s$9/$%s$11" % (col_numbers,col_numbers)),style_pourcent_right)
        ws1.write(9,col_final+4,Formula("$%s$10/$%s$11" % (col_numbers,col_numbers)),style_pourcent_right)
        ws1.write(10,col_final+4,Formula("SUM($%s$7:$%s$10)" % (total_numbers,total_numbers)),style_pourcent_total_bordered)
        wb1.save('suivi_budgetaire.xls')
        result_file = open('suivi_budgetaire.xls','rb').read()

        # give the result tos the user
        data['form']['msg']='Save the File with '".xls"' extension.'
        data['form']['suivi_budgetaire.xls']=base64.encodestring(result_file)
        return data['form']
    states = {
        'init': {
            'actions': [_get_defaults],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('extract','Extract') ]}
        },
        'extract': {
            'actions': [_extract_excel],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        }
    }
compare_turnover_cci_budget('compare_turnover_cci_budget')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

