# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import wizard
import time
import datetime
import tools
import pooler

form = """<?xml version="1.0"?>
<form string="Creation of a budget from another">
    <field name="old_budget_id" colspan="4" width="640"/>
    <field name="new_name" colspan="4"/>
    <field name="new_fiscalyear_id" />
    <newline/>
    <field name="reset"/>
</form>"""

fields = {
    'old_budget_id': {'string': 'Old Budget to Copy', 'type':'many2one','relation': 'cci_budget.budget','required': True},
    'new_name':{'string':'New Name','type':'char','size':64,'required':True},
    'new_fiscalyear_id': {'string':'New Fiscal Year (else same)', 'type':'many2one', 'relation': 'account.fiscalyear'},
    'reset':{'string':'Reset all amounts','type':'boolean'},
   }

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="Results" colspan="4"/>
     <field name="msg" colspan="4" nolabel="1" width="800"/>
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
}

class wizard_copy_budget(wizard.interface):
    def _defaults(self,cr,uid,data,contact):
        if data['id']:
            data['form']['old_budget_id'] = data['id']
            obj_budget = pooler.get_pool(cr.dbname).get('cci_budget.budget')
            old_budget = obj_budget.read(cr,uid,data['id'],['name'])
            data['form']['new_name'] = old_budget['name'] + ' (Copie)'
        return data['form']
    def _copy(self,cr,uid,data,context):
        old_budget_id = data['id']
        obj_budget = pooler.get_pool(cr.dbname).get('cci_budget.budget')
        obj_dotation = pooler.get_pool(cr.dbname).get('cci_budget.dotation')
        obj_period_dotation = pooler.get_pool(cr.dbname).get('cci_budget.period_dotation')
        old_budget = obj_budget.read(cr,uid,old_budget_id,['fiscalyear_id'])
        old_dotation_ids = obj_dotation.search(cr,uid,[('budget_id','=',data['id'])])
        if old_dotation_ids:
            old_period_dotation_ids = obj_period_dotation.search(cr,uid,[('dotation_id','in',old_dotation_ids)])
        else:
            old_dotation_ids = []
        dNewPeriods = {}
        if data['form']['new_fiscalyear_id'] and data['form']['new_fiscalyear_id'] <> old_budget['fiscalyear_id'][0] and len(old_period_dotation_ids) > 0:
            # prepare the periods translation table
            obj_period = pooler.get_pool(cr.dbname).get('account.period')
            obj_fiscalyear = pooler.get_pool(cr.dbname).get('account.fiscalyear')
            old_period_ids = obj_period.search(cr,uid,[('fiscalyear_id','=',old_budget['fiscalyear_id'][0]),('special','=',False)])
            old_periods = obj_period.read(cr,uid,old_period_ids,['id','date_start'])
            new_period_ids = obj_period.search(cr,uid,[('fiscalyear_id','=',data['form']['new_fiscalyear_id'])])
            new_periods = obj_period.read(cr,uid,new_period_ids,['id','date_start'])
            for old_period in old_periods:
                found_id = 0
                for new_period in new_periods:
                    if new_period['date_start'][5:10] == old_period['date_start'][5:10]: # same day and month but other year
                        found_id = new_period['id']
                        break
                if found_id:
                    dNewPeriods[old_period['id']] = found_id and found_id or old_period['id']
        else:
            obj_period = pooler.get_pool(cr.dbname).get('account.period')
            obj_fiscalyear = pooler.get_pool(cr.dbname).get('account.fiscalyear')
            old_period_ids = obj_period.search(cr,uid,[('fiscalyear_id','=',old_budget['fiscalyear_id'][0]),('special','=',False)])
            for old_period_id in old_period_ids:
                dNewPeriods[old_period_id] = old_period_id
        if len(dNewPeriods) != 12:
            raise wizard.except_wizard('Erreur !', 'Pas le bon nombre [12] de périodes dans l\'année fiscale cible [%i]' % len(dNewPeriods))
        # creation of the new budget
        new_id = obj_budget.create(cr,uid,{'name':data['form']['new_name'],'offical':False,'fiscalyear_id':data['form']['new_fiscalyear_id'] or old_budget['fiscalyear_id'][0],'validating_user_id':False})
        if new_id and len(old_dotation_ids) > 0:
            old_dotations = obj_dotation.read(cr,uid,old_dotation_ids,['post_id','account_id','comments'])
            dConvertDotations = {}
            for old_dot in old_dotations:
                new_dotation_id = obj_dotation.create(cr,uid,{'budget_id':new_id,'post_id':old_dot['post_id'][0],'account_id':old_dot['account_id'][0],'comments':old_dot['comments']})
                if new_dotation_id:
                    dConvertDotations[old_dot['id']] = new_dotation_id
            old_period_dotations = obj_period_dotation.read(cr,uid,old_period_dotation_ids,['dotation_id','period_id','amount','comments'])
            for old_perdot in old_period_dotations:
                if dConvertDotations.has_key(old_perdot['dotation_id'][0]):
                    if data['form']['reset']:
                        new_amount = 0.0
                    else:
                        new_amount = old_perdot['amount']
                    obj_period_dotation.create(cr,uid,{'dotation_id':dConvertDotations[old_perdot['dotation_id'][0]],
                                                       'period_id':dNewPeriods and dNewPeriods[old_perdot['period_id'][0]] or old_perdot['period_id'][0],
                                                       'amount':new_amount,
                                                       'comments':old_perdot['comments'],
                                                      })
        count_dotations = len(old_dotation_ids)
        count_period_dotations = len(old_period_dotation_ids)
        data['form']['msg']='Budget \'%s\' created with %i dotations and %i monthly dotations' % (data['form']['new_name'],count_dotations,count_period_dotations)
        return data['form']

    states = {
        'init': {
            'actions': [_defaults],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('copy','Create copy')]},
        },
        'copy': {
            'actions': [_copy],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }

wizard_copy_budget('wizard_copy_budget')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

