# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import datetime
import wizard
import pooler
from tools.translate import _
import base64
from pyExcelerator import *

form = '''<?xml version="1.0"?>
<form string="Select Budget">
    <field name="budget_id" colspan="4"/>
</form>'''

fields = {
    'budget_id': {'string': 'Budget', 'type': 'many2one', 'relation': 'cci_budget.budget'},
}
msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="detect_analytic_outside_budget.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'detect_analytic_outside_budget.xls':{'string': 'Prepared file',
        'type': 'binary',
        'readonly': True,},
}

class detect_analytic_outside_budget(wizard.interface):
    def _get_defaults(self, cr, uid, data, context):
        today = datetime.date.today()
        fiscalyear_obj = pooler.get_pool(cr.dbname).get('account.fiscalyear')
        budget_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget')
        if today.month <= 2:
            fiscalyear_ids = fiscalyear_obj.search(cr,uid,[('date_start','=',str(today.year-1)+'-01-01')])
        else:
            fiscalyear_ids = fiscalyear_obj.search(cr,uid,[('date_start','=',str(today.year)+'-01-01')])
        if fiscalyear_ids:
            fiscalyear_id = fiscalyear_ids[0]
        else:
            fiscalyear_id = fiscalyear_obj.find(cr,uid)
        budget_ids = budget_obj.search(cr,uid,[('fiscalyear_id','=',fiscalyear_id),('official','=',True)])
        if budget_ids:
            data['form']['budget_id'] = budget_ids[0]
        else:
            budget_ids = budget_obj.search(cr,uid,[('fiscalyear_id','=',fiscalyear_id)])
            if budget_ids:
                data['form']['budget_id'] = budget_ids[0]
        return data['form']

    def _extract_excel(self, cr, uid, data, context):
        # selected data
        fiscalyear_obj = pooler.get_pool(cr.dbname).get('account.fiscalyear')
        budget_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget')
        period_obj = pooler.get_pool(cr.dbname).get('account.period')
        account_obj = pooler.get_pool(cr.dbname).get('account.account')
        amline_obj = pooler.get_pool(cr.dbname).get('account.move.line')
        parameter_obj = pooler.get_pool(cr.dbname).get('cci_parameters.cci_value')

        budget = budget_obj.browse(cr,uid,data['form']['budget_id'])
        concerned_period_ids = period_obj.search(cr,uid,[('fiscalyear_id','=',budget.fiscalyear_id.id)])
        
        account_ids = account_obj.search(cr,uid,[('cci_budget_group_id','=',False)])
        accounts = account_obj.read(cr,uid,account_ids,['name','code'])
        saccounts = sorted(accounts, key=lambda k: k['code'])
        
        # list of exceptions
        param_values = parameter_obj.get_value_from_names(cr,uid,['AccountsOutsideBudget'])
        if param_values.has_key('AccountsOutsideBudget'):
            exception_code_accounts = param_values['AccountsOutsideBudget']
        else:
            exception_code_accounts = []
        
        wb1 = Workbook()
        ws1 = wb1.add_sheet('Detected errors')
        ws1.write(0,0,'Account with Analytic moves but not inserted into budget')
        ws1.write(1,0,_('Extract date:'))
        ws1.write(1,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws1.write(3,0,_('Account'))
        ws1.write(3,1,_('Mandatory Analytic'))
        ws1.write(3,2,_('Count'))
        line = 4
        ws2 = wb1.add_sheet('Details')
        ws2.write(0,0,'Analytic moves attached to account not inserted into budget')
        ws2.write(1,0,_('Extract date:'))
        ws2.write(1,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws2.write(3,0,_('Analytic Line ID'))
        ws2.write(3,1,_('Move Line ID'))
        ws2.write(3,2,_('Account Name'))
        ws2.write(3,3,_('Description'))
        ws2.write(3,4,_('Move Réf.'))
        ws2.write(3,5,_('Amount'))
        line2 = 4
        amline_ids = amline_obj.search(cr,uid,[('period_id','in',concerned_period_ids),('account_id','in',account_ids)])
        amlines = amline_obj.browse(cr,uid,amline_ids)
        dAccounts = {}
        for amline in amlines:
            if amline.account_id.code not in exception_code_accounts:
                if dAccounts.has_key(amline.account_id.id):
                    record = dAccounts[amline.account_id.id]
                else:
                    record = {'id': amline.account_id.id,
                              'name': amline.account_id.code + ' ' + amline.account_id.name,
                              'count_all': 0,
                              'count_anal': 0,
                              'mandatory': amline.account_id.cci_forced_analytic and 'OUI' or '',
                             }
                record['count_all'] += len(amline.analytic_lines)
                if amline.analytic_lines:
                    record['count_anal'] += len(amline.analytic_lines)
                    for analytic_line in amline.analytic_lines:
                        ws2.write(line2,0,analytic_line.id)
                        ws2.write(line2,1,amline.id)
                        ws2.write(line2,2,record['name'])
                        ws2.write(line2,3,amline.name or '')
                        ws2.write(line2,4,amline.move_id and (amline.move_id.name or amline.move_id.ref or '') or '')
                        ws2.write(line2,5,analytic_line.amount or 0.0)
                        line2 += 1
            dAccounts[amline.account_id.id] = record
        errors = 0
        for account in saccounts:
            if account['id'] in dAccounts:
                res_account = dAccounts[account['id']]
                if res_account['count_anal'] > 0:
                    ws1.write(line,0,res_account['name'])
                    ws1.write(line,1,res_account['mandatory'])
                    ws1.write(line,2,'%i / %i' % (res_account['count_anal'],res_account['count_all']))
                    errors += 1
                    line += 1
        wb1.save('detect_analytic_outside_budget.xls')
        result_file = open('detect_analytic_outside_budget.xls','rb').read()

        # give the result tos the user
        data['form']['msg']= '%s errors found.\n\nSave the File with '".xls"' extension.' % str(errors)
        data['form']['detect_analytic_outside_budget.xls']=base64.encodestring(result_file)
        return data['form']
    states = {
        'init': {
            'actions': [_get_defaults],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('extract','Extract') ]}
        },
        'extract': {
            'actions': [_extract_excel],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        }
    }
detect_analytic_outside_budget('detect_analytic_outside_budget')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

