# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import datetime
import wizard
import pooler
from tools.translate import _
import base64
from pyExcelerator import *

form = '''<?xml version="1.0"?>
<form string="Select Budget">
    <field name="budget_id" colspan="4"/>
</form>'''

fields = {
    'budget_id': {'string': 'Budget', 'type': 'many2one', 'relation': 'cci_budget.budget'},
}
msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="detect_lacking_dotation.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'detect_lacking_dotation.xls':{'string': 'Prepared file',
        'type': 'binary',
        'readonly': True,},
}

class detect_lacking_dotation(wizard.interface):
    def _get_defaults(self, cr, uid, data, context):
        today = datetime.date.today()
        fiscalyear_obj = pooler.get_pool(cr.dbname).get('account.fiscalyear')
        budget_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget')
        if today.month <= 2:
            fiscalyear_ids = fiscalyear_obj.search(cr,uid,[('date_start','=',str(today.year-1)+'-01-01')])
        else:
            fiscalyear_ids = fiscalyear_obj.search(cr,uid,[('date_start','=',str(today.year)+'-01-01')])
        if fiscalyear_ids:
            fiscalyear_id = fiscalyear_ids[0]
        else:
            fiscalyear_id = fiscalyear_obj.find(cr,uid)
        budget_ids = budget_obj.search(cr,uid,[('fiscalyear_id','=',fiscalyear_id),('official','=',True)])
        if budget_ids:
            data['form']['budget_id'] = budget_ids[0]
        else:
            budget_ids = budget_obj.search(cr,uid,[('fiscalyear_id','=',fiscalyear_id)])
            if budget_ids:
                data['form']['budget_id'] = budget_ids[0]
        return data['form']

    def _extract_excel(self, cr, uid, data, context):
        # selected data
        fiscalyear_obj = pooler.get_pool(cr.dbname).get('account.fiscalyear')
        budget_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget')
        period_obj = pooler.get_pool(cr.dbname).get('account.period')
        account_obj = pooler.get_pool(cr.dbname).get('account.account')
        aaccount_obj = pooler.get_pool(cr.dbname).get('account.analytic.account')
        amline_obj = pooler.get_pool(cr.dbname).get('account.move.line')
        dotation_obj = pooler.get_pool(cr.dbname).get('cci_budget.dotation')
        per_dot_obj = pooler.get_pool(cr.dbname).get('cci_budget.period_dotation')
        post_obj = pooler.get_pool(cr.dbname).get('cci_budget.post')
        
        budget = budget_obj.browse(cr,uid,data['form']['budget_id'])
        concerned_period_ids = period_obj.search(cr,uid,[('fiscalyear_id','=',budget.fiscalyear_id.id)])
        
        dResults = {}
        dAnalytic2Post = {}
        dAccount2Group = {}
        account_ids = account_obj.search(cr,uid,[('cci_budget_group_id','!=',False)])
        accounts = account_obj.read(cr,uid,account_ids,['cci_budget_group_id','code','name'])
        for account in accounts:
            dAccount2Group[account['id']] = account['cci_budget_group_id'][0]
        saccounts = sorted(accounts, key=lambda k: k['code'])
        aaccount_ids = aaccount_obj.search(cr,uid,[('cci_post_id','!=',False)])
        aaccounts = aaccount_obj.read(cr,uid,aaccount_ids,['cci_post_id'])
        for aaccount in aaccounts:
            dAnalytic2Post[aaccount['id']] = aaccount['cci_post_id'][0]

        dotation_ids = dotation_obj.search(cr,uid,[('budget_id','=',budget.id)])
        dotations = dotation_obj.browse(cr,uid,dotation_ids)
        dResults = {}
        for dot in dotations:
            dResults[(dot.post_id.id,dot.account_id.id)] = {'budget':0.0}

        wb1 = Workbook()
        ws1 = wb1.add_sheet('Detected errors')
        ws1.write(1,0,_('Extract date:'))
        ws1.write(1,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws1.write(3,0,_('Error'))
        ws1.write(3,1,_('Account'))
        ws1.write(3,2,_('Analytic Account'))
        ws1.write(3,3,_('Move Line ID'))
        line = 4
        amline_ids = amline_obj.search(cr,uid,[('period_id','in',concerned_period_ids)])
        amlines = amline_obj.browse(cr,uid,amline_ids)
        errors1 = []
        errors2 = []
        for amline in amlines:
            if amline.analytic_lines:
                for analytic_line in amline.analytic_lines:
                    if dAnalytic2Post.has_key(analytic_line.account_id.id):
                        key = (dAnalytic2Post[analytic_line.account_id.id],amline.account_id.id)
                        if (not dResults.has_key(key)) and (key not in errors1):
                            # lacking dotation !
                            ws1.write(line,0,_('No dotation'))
                            ws1.write(line,1,amline.account_id.code + ' ' + amline.account_id.name)
                            ws1.write(line,2,analytic_line.account_id.name)
                            ws1.write(line,3,amline.id)
                            errors1.append(key)
                            line += 1
                    else:
                        if analytic_line.account_id.id not in errors2:
                            # lacking project on analytic account !
                            ws1.write(line,0,_('Analytic account wo Project'))
                            ws1.write(line,1,amline.account_id.code + ' ' + amline.account_id.name)
                            ws1.write(line,2,analytic_line.account_id.name)
                            ws1.write(line,3,amline.id)
                            errors2.append(analytic_line.account_id.id)
                            line += 1
        wb1.save('detect_lacking_dotation.xls')
        result_file = open('detect_lacking_dotation.xls','rb').read()

        # give the result tos the user
        data['form']['msg']= '%s errors found.\n\nSave the File with '".xls"' extension.' % str(len(errors1)+len(errors2))
        data['form']['detect_lacking_dotation.xls']=base64.encodestring(result_file)
        return data['form']
    states = {
        'init': {
            'actions': [_get_defaults],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('extract','Extract') ]}
        },
        'extract': {
            'actions': [_extract_excel],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        }
    }
detect_lacking_dotation('detect_lacking_dotation')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

