# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution    
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import datetime
import wizard
import pooler
from tools.translate import _
import base64
from pyExcelerator import *
from pyExcelerator.Worksheet import *

form = """<?xml version="1.0"?>
<form string="Details of accounts by budget post">
    <separator string="Select the Periods for details" colspan="4"/>
    <field name="from_per"/>
    <field name="to_per"/>
    <newline/>
    <field name="account_selection"/>
    <field name="move_state"/>
</form>"""

form_fields = {
    'from_per': {'string': 'From','type': 'many2one','relation':'account.period','required':True},
    'to_per': {'string': 'To','type': 'many2one','relation':'account.period','required':True},
    'account_selection': {'string':'Selection','type':'selection','selection':[('sales','Class 7 - Sales'),('expenses','Class 6 - Expenses'),('all','Class 6, 7 and specials')],'default':'all'},
    'move_state': {'string': 'Move State', 'type': 'selection', 'selection': [('posted','Only posted moves'),('draft','All moves, even draft')], 'default': 'posted', 'required': 'True'}
}

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="details_departments.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'details_departments.xls':{'string': 'Prepared file','type': 'binary','readonly': True,},
}

def _get_type(amline):
    return result
def num_to_letter(line_number_in_pyexcel):
    # this method convert an integer to a line label in Excel
    # first line = 0 (beware on pyExcelerator, first line = 0)
    # 0 => 'A'
    # 26 => 'AA'
    # 27 => 'AB'
    first = (line_number_in_pyexcel // 26)
    last = (line_number_in_pyexcel % 26) + 1
    if first > 0:
        result = chr(64+first)+chr(64+last)
    else:
        result = chr(64+last)
    return result
def full_cell(line_number_in_pyexcel,col_number_in_pyexcel,fixed='none'):
    # fixed = none, col, line, both
    result = ''
    if fixed in ('col','both'):
        result += '$'
    result += num_to_letter(col_number_in_pyexcel)
    if fixed in ('line','both'):
        result += '$'
    result += str(line_number_in_pyexcel+1)
    return result
class extract_details_departments(wizard.interface):
    def _extract(self, cr, uid, data, context):
        period_obj = pooler.get_pool(cr.dbname).get('account.period')
        amline_obj = pooler.get_pool(cr.dbname).get('account.move.line')
        account_obj = pooler.get_pool(cr.dbname).get('account.account')
        aaccount_obj = pooler.get_pool(cr.dbname).get('account.analytic.account')
        main_group_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget_account_category')
        department_obj = pooler.get_pool(cr.dbname).get('cci_budget.department')
        post_obj = pooler.get_pool(cr.dbname).get('cci_budget.post')
        #if data['form']['account_selection'] == 'sales':
        #    accounts_beginning = ['7']
        #elif data['form']['account_selection'] == 'expenses':
        #    accounts_beginning = ['6']
        #else:
        #    accounts_beginning = ['7','6']
        # 
        main_group_ids = main_group_obj.search(cr,uid,[])
        main_groups = main_group_obj.browse(cr,uid,main_group_ids)
        
        account_ids = account_obj.search(cr,uid,[])
        accounts = account_obj.browse(cr,uid,account_ids)
        dAccount2Type = {}
        for acc in accounts:
            #if acc.code[0] in accounts_beginning:
               #concerned_account_ids.append( acc.id )
            acc_type = ''
            for mgroup in main_groups:
                for subgroup in mgroup.group_ids:
                    for defined_account in subgroup.account_ids:
                        if defined_account.id == acc.id:
                            acc_type = mgroup.class_type
                            break
            if not acc_type or acc_type == 'other':
                if acc.code[0] == '7':
                    acc_type = 'outsales'
                elif acc.code[0] == '6':
                    acc_type = 'outexpenses'
                else:
                    acc_type = 'other'
            dAccount2Type[acc.id] = acc_type

        # we will sum all analytic move by account_id and post_id
        period_from = period_obj.browse(cr,uid,data['form']['from_per'])
        period_to = period_obj.browse(cr,uid,data['form']['to_per'])
        if period_from.date_start > period_to.date_start:
            (period_from,period_to) = (period_to,period_from)
        concerned_period_ids = period_obj.search(cr,uid,[('date_start','>=',period_from.date_start),('date_stop','<=',period_to.date_stop)])

        wb1 = Workbook()
        ws1 = wb1.add_sheet(_('Crossing'))
        ws1.write(0,0,_('Detailed Posts by Accounts'))
        ws1.write(2,0,_('Extract date:'))
        ws1.write(2,1,datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S') )
        ws1.write(3,1,period_from.name + ' -> ' + period_to.name + ((data['form']['move_state'] == 'draft') and _(' (All even draft)') or _(' (only posted)')) )
        line = 5

        if concerned_period_ids:
            errors = []
            dResults = {}
            amline_ids = amline_obj.search(cr,uid,[('period_id','in',concerned_period_ids)])
            index = 0
            size = 1000
            used_dept_ids = []
            used_post_ids = []
            while amline_ids[index:index+size]:
                amlines = amline_obj.browse(cr,uid,amline_ids[index:index+size])
                for amline in amlines:
                    if amline.analytic_lines and ((data['form']['move_state'] == 'draft') or (amline.move_id.state == data['form']['move_state'])):
                        for analytic_line in amline.analytic_lines:
                            if analytic_line.account_id.cci_post_id:
                                if analytic_line.account_id.cci_post_id.department_id:
                                    if not dResults.has_key((analytic_line.account_id.cci_post_id.id,amline.account_id.id)):
                                        record = {'dpt_name': analytic_line.account_id.cci_post_id.department_id.name,
                                                  'post_name': analytic_line.account_id.cci_post_id.name,
                                                  'account_name': amline.account_id.code + ' ' + amline.account_id.name,
                                                  'amount':0.0,
                                                  'acc_type':dAccount2Type.has_key(analytic_line.general_account_id.id) and dAccount2Type[analytic_line.general_account_id.id] or 'other', # can be 'other' if inactive account used in the selected periods
                                                 }
                                        if analytic_line.account_id.cci_post_id.department_id.id not in used_dept_ids:
                                            used_dept_ids.append(analytic_line.account_id.cci_post_id.department_id.id)
                                        if analytic_line.account_id.cci_post_id.id not in used_post_ids:
                                            used_post_ids.append(analytic_line.account_id.cci_post_id.id)
                                    else:
                                        record = dResults[(analytic_line.account_id.cci_post_id.id,amline.account_id.id)]
                                    record['amount'] += (analytic_line.amount or 0.0)
                                    dResults[(analytic_line.account_id.cci_post_id.id,amline.account_id.id)] = record
                                else:
                                    # error : at least one analytic_account with post without department_id
                                    error = 'Analytic Account %s [%i] is linked to Budget Post \'%s\' is not linked to Department' % (analytic_line.account_id.name,analytic_line.account_id.id,analytic_line.account_id.cci_post_id.name)
                                    if error not in errors:
                                        errors.append(error)
                            else:
                                if analytic_line.account_id.parent_id.code != 'root_s':
                                    # error : at least one analytic_account without post_id
                                    error = 'Analytic Account %s [%i] is not linked to Budget Post' % (analytic_line.account_id.name,analytic_line.account_id.id)
                                    if error not in errors:
                                        errors.append(error)
                index += size
            if not errors:
                if data['form']['account_selection'] == 'sales':
                    selected_acc_types = ['directsales','outsales','indirectsales']
                elif data['form']['account_selection'] == 'expenses':
                    selected_acc_types = ['expenses','outexpenses']
                else:
                    selected_acc_types = ['directsales','outsales','indirectsales','expenses','outexpenses','other']
                dClassCode2Name = {}
                dClassCode2Name['directsales'] = _('Direct Sales')
                dClassCode2Name['indirectsales'] = _('Indirect Sales')
                dClassCode2Name['outsales'] = _('Out/Side Sales')
                dClassCode2Name['expenses'] = _('Expenses')
                dClassCode2Name['outexpenses'] = _('Out/Side Expenses')
                dClassCode2Name['other'] = _('Undefined/Special')
                
                # determining the list of post by department.sequence+post.name order
                # WRONG : Select only current budget depts and posts, but we can have some moves on older depts and/or posts
                #dept_ids = department_obj.search(cr,uid,[])
                #depts = department_obj.browse(cr,uid,dept_ids)
                #sorted_post_ids = []
                #for dept in depts:
                #    posts = []
                #    for post in dept.post_ids:
                #        posts.append({'id':post.id,'name':post.name})
                #    sorted_posts = sorted(posts, key=lambda x: x['name'])
                #    for spost in sorted_posts:
                #        sorted_post_ids.append(spost['id'])

                # determining the list of post by department.sequence+post.name order
                dept_ids = department_obj.search(cr,uid,[])
                for used_dept_id in used_dept_ids:
                    if used_dept_id not in dept_ids:
                        depts_ids.append(used_dept_id) ### to do same for posts
                depts = department_obj.browse(cr,uid,dept_ids)
                sorted_post_ids = []
                final_sorted_posts = []
                for dept in depts:
                    posts = []
                    def_post_ids = []
                    for post in dept.post_ids:
                        posts.append({'id':post.id,'name':post.name,'dept_id':post.department_id.id,'dept_name':post.department_id.name})
                        def_post_ids.append(post.id)
                    for used_post_id in used_post_ids:
                        if used_post_id not in def_post_ids:
                            post = post_obj.browse(cr,uid,used_post_id)
                            if post.department_id and post.department_id.id == dept.id:
                                posts.append({'id':used_post_id,'name':post.name,'dept_id':post.department_id.id,'dept_name':post.department_id.name})
                    sorted_posts = sorted(posts, key=lambda x: x['name'])
                    final_sorted_posts.extend(sorted_posts)
                    for spost in sorted_posts:
                        sorted_post_ids.append(spost['id'])

                
                # determining the list of account_ids sorted by name, class_type by class_type
                dClassTypes = {}
                for (key,res) in dResults.items():
                    class_type = res['acc_type']
                    if not dClassTypes.has_key(class_type):
                        record = []
                    else:
                        record = dClassTypes[class_type]
                    new = True
                    for account in record:
                        if account['id'] == key[1]:
                            new = False
                    if new:
                        record.append({'id':key[1],'name':res['account_name']})
                    dClassTypes[class_type] = record
                for (key,account_list) in dClassTypes.items():
                    sorted_account_list = sorted(account_list, key=lambda x: x['name'])
                    dClassTypes[key] = sorted_account_list

                # injecting data in excel worksheet
                top_line = line
                current_line = line + 2
                max_col = 2
                #for post_id in sorted_post_ids:
                for post in final_sorted_posts:
                    post_id = post['id']
                    current_col_type = 2
                    ws1.write(current_line,0,post['dept_name'])
                    ws1.write(current_line,1,post['name'])
                    for acc_type in selected_acc_types:
                        if dClassTypes.has_key(acc_type):
                            sorted_account_list = dClassTypes[acc_type]
                            index_account_inside_type = 1
                            total_amount_type = 0.0
                            for account in sorted_account_list:
                                account_id = account['id']
                                account_name = account['name']
                                if dResults.has_key((post_id,account_id)):
                                    result = dResults[(post_id,account_id)]
                                    #post_name = result['post_name']
                                    #dpt_name = result['dpt_name']
                                    amount = result['amount']
                                else:
                                    #post_name = ''
                                    #dpt_name = ''
                                    amount = 0.0
                                ws1.write(current_line,current_col_type+index_account_inside_type,round(amount,2))
                                ws1.write(top_line+1,current_col_type+index_account_inside_type,account_name)
                                if current_col_type+index_account_inside_type > max_col:
                                    max_col = current_col_type+index_account_inside_type
                                total_amount_type += amount
                                index_account_inside_type += 1
                            ws1.write(current_line,current_col_type,round(total_amount_type,2))
                            ws1.write(top_line,current_col_type,dClassCode2Name[acc_type])
                            ws1.write(top_line+1,current_col_type,_('Details->'))
                            current_col_type = current_col_type + index_account_inside_type
                    current_line += 1
                ws1.write(current_line,0,_('Total by account'))
                final_col = 2
                while final_col <= max_col:
                    ws1.write(current_line,final_col,Formula("SUM(%s:%s)" % (full_cell(top_line+2,final_col,'none'),full_cell(current_line-1,final_col,'none'))) )
                    final_col += 1
            else:
                ws1.write(line,0,_('PROBLEMS WITH SOME ANALYTIC ACCOUNTS :'))
                line += 1
                for err in errors:
                    ws1.write(line,1,err)
        else:
            ws1.write(line,0,_('PROBLEMS WITH PERIODS TO EXTRACT - Nothing extracted'))
        wb1.save('details_departments.xls')
        result_file = open('details_departments.xls','rb').read()

        # give the result tos the user
        data['form']['msg'] = 'Save the File with \'".xls"\' extension.'
        data['form']['details_departments.xls'] = base64.encodestring(result_file)
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type': 'form', 'arch':form, 'fields':form_fields, 'state': [('end', 'Cancel', 'gtk-cancel'),('extract', 'Extract','gtk-yes')]}
        },
        'extract': {
            'actions': [_extract],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        }
    }
extract_details_departments('extract_details_departments')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

