# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
# 2018-02-19 : now wa can also choose special periods for opening moves, if we want
import datetime
import wizard
import pooler
from tools.translate import _
import base64
from pyExcelerator import *
from pyExcelerator.Worksheet import *

form = '''<?xml version="1.0"?>
<form string="Select period">
    <field name="budget_id" colspan="4"/>
    <field name="period_budget_from"/>
    <field name="period_budget_to"/>
    <newline/>
    <field name="fiscalyear" colspan="4"/>
    <newline/>
    <field name="period_from"/>
    <field name="period_to"/>
    <newline/>
    <field name="direction"/>
    <field name="move_state"/>
</form>'''


fields = {
    'budget_id': {'string': 'Budget', 'type': 'many2one', 'relation': 'cci_budget.budget', 'required': True},
    'period_budget_from': {'string': 'From Period', 'type': 'many2one', 'relation': 'account.period', 'required': True},
    'period_budget_to': {'string': 'To Period', 'type': 'many2one', 'relation': 'account.period', 'required': True},
    'fiscalyear': {'string': 'Fiscal year', 'type': 'many2one', 'relation': 'account.fiscalyear'},
    'period_from': {'string': 'From Period', 'type': 'many2one', 'relation': 'account.period', 'domain': "[('fiscalyear_id', '=', fiscalyear)]"},
    'period_to': {'string': 'To Period', 'type': 'many2one', 'relation': 'account.period', 'domain': "[('fiscalyear_id', '=', fiscalyear)]"},
    'direction': {'string': 'Direction', 'type': 'selection', 'selection': [('1-2','= Reel - Budget'),('2-1','= Budget - Reel')],'help':'if comparing current year with current budget, Reel - Budget; else, better Budget - Reel'},
    'move_state': {'string': 'Move State', 'type': 'selection', 'selection': [('posted','Only posted moves'),('draft','All moves, even draft')], 'default': 'posted', 'required': 'True'}
}
msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="tableau_budget.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'tableau_budget.xls':{'string': 'Prepared file',
        'type': 'binary',
        'readonly': True,},
}

pyexcel_colours = {
        'aqua'    : 0x0F,
        'cyan'    : 0x0F,
        'black'   : 0x08,
        'blue'    : 0x0C,
        'brown'   : 0x10,
        'magenta' : 0x0E,
        'fuchsia' : 0x0E,
        'gray'    : 0x17,
        'grey'    : 0x17,
        'green'   : 0x11,
        'lime'    : 0x0B,
        'navy'    : 0x12,
        'orange'  : 0x35,
        'pink'    : 0x21,
        'purple'  : 0x14,
        'red'     : 0x0A,
        'silver'  : 0x16,
        'white'   : 0x09,
        'yellow'  : 0x0D,
}

def num_to_letter(line_number_in_pyexcel):
    # this method convert an integer to a line label in Excel
    # first line = 0 (beware on pyExcelerator, first line = 0)
    # 0 => 'A'
    # 26 => 'AA'
    # 27 => 'AB'
    line_number_in_pyexcel += 1
    if line_number_in_pyexcel < 26:
        result = chr(64+line_number_in_pyexcel)
    else:
        first = line_number_in_pyexcel_number_in_pyexcel // 26
        last = number % 26
        result = chr(64+first)+chr(64+last)
    return result
def full_cell(line_number_in_pyexcel,col_number_in_pyexcel,fixed='none'):
    # fixed = none, col, line, both
    result = ''
    if fixed in ('col','both'):
        result += '$'
    result += num_to_letter(col_number_in_pyexcel)
    if fixed in ('line','both'):
        result += '$'
    result += str(line_number_in_pyexcel+1)
    return result
#def _get_total_type(account_code):
#    if account_code in ['707000','709001','707500']:
#        return 'support'
#    elif account_code[0:1] == '7':
#        return 'turnover'
#    else:
#        return 'expense'
def _get_total_type(account): # parameter = account browse object
    result = 'other'
    if account.cci_budget_group_id and account.cci_budget_group_id.category_id and account.cci_budget_group_id.category_id.class_type:
        if account.cci_budget_group_id.category_id.class_type == 'directsales':
            result = 'turnover'
        elif account.cci_budget_group_id.category_id.class_type == 'indirectsales':
            result = 'support'
        elif account.cci_budget_group_id.category_id.class_type == 'expenses':
            result = 'expense'
    return result
def _get_total_subtype(account_code):
    if account_code == '707000':
        return 'membership'
    elif account_code == '709001':
        return 'structural_sponsor'
    elif account_code == '709000':
        return 'product_sponsor'
    elif account_code[0:2] == '74':
        return 'subsidy'
    else:
        return 'normal'
class extract_raw_splitted_data(wizard.interface):
    def _extract_excel(self, cr, uid, data, context):
        string_now = datetime.datetime.now().strftime('%d/%m/%Y - %H:%M:%S')
        # selected data
        fiscalyear_obj = pooler.get_pool(cr.dbname).get('account.fiscalyear')
        budget_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget')
        period_obj = pooler.get_pool(cr.dbname).get('account.period')
        account_obj = pooler.get_pool(cr.dbname).get('account.account')
        aaccount_obj = pooler.get_pool(cr.dbname).get('account.analytic.account')
        amline_obj = pooler.get_pool(cr.dbname).get('account.move.line')
        dotation_obj = pooler.get_pool(cr.dbname).get('cci_budget.dotation')
        per_dot_obj = pooler.get_pool(cr.dbname).get('cci_budget.period_dotation')
        raw_dotation_obj = pooler.get_pool(cr.dbname).get('cci_budget.raw_dotation')
        raw_per_dot_obj = pooler.get_pool(cr.dbname).get('cci_budget.raw_period_dotation')
        post_obj = pooler.get_pool(cr.dbname).get('cci_budget.post')
        
        if data['form']['fiscalyear']:
            fiscalyear = fiscalyear_obj.browse(cr,uid,data['form']['fiscalyear'])
            period_from = period_obj.browse(cr,uid,data['form']['period_from'])
            period_to = period_obj.browse(cr,uid,data['form']['period_to'])
            concerned_period_ids = period_obj.search(cr,uid,[('date_start','>=',period_from.date_start),('date_stop','<=',period_to.date_stop)])
            smoothing_journal_ids = []
            sm_account_ids = account_obj.search(cr,uid,[('smoothing_type','!=','none')])
            if sm_account_ids:
                sm_accounts = account_obj.read(cr,uid,sm_account_ids,['smoothing_journal_id'])
                for sm_acc in sm_accounts:
                    if sm_acc['smoothing_journal_id'] and sm_acc['smoothing_journal_id'][0] not in smoothing_journal_ids:
                        smoothing_journal_ids.append(sm_acc['smoothing_journal_id'][0])
        budget = budget_obj.browse(cr,uid,data['form']['budget_id'])
        period_budget_from = period_obj.browse(cr,uid,data['form']['period_budget_from'])
        period_budget_to = period_obj.browse(cr,uid,data['form']['period_budget_to'])
        concerned_period_budget_ids = period_obj.search(cr,uid,[('date_start','>=',period_budget_from.date_start),('date_stop','<=',period_budget_to.date_stop)])
        
        dResults = {}
        dAnalytic2Post = {}
        dAccount2Group = {}
        account_ids = account_obj.search(cr,uid,[('cci_budget_group_id','!=',False)])
        accounts = account_obj.read(cr,uid,account_ids,['cci_budget_group_id','code','name'])
        for account in accounts:
            dAccount2Group[account['id']] = account['cci_budget_group_id'][0]
        saccounts = sorted(accounts, key=lambda k: k['code'])
        aaccount_ids = aaccount_obj.search(cr,uid,[('cci_post_id','!=',False)])
        aaccounts = aaccount_obj.read(cr,uid,aaccount_ids,['cci_post_id'])
        for aaccount in aaccounts:
            dAnalytic2Post[aaccount['id']] = aaccount['cci_post_id'][0]

        # raw dotation => results put on associated_account
        dResults = {}
        concerned_account_ids = []
        concerned_aaccount_ids = []
        dAssociated = {}
        raw_dotation_ids = raw_dotation_obj.search(cr,uid,[('budget_id','=',budget.id)])
        if raw_dotation_ids:
            raw_dotations = raw_dotation_obj.browse(cr,uid,raw_dotation_ids)
            for raw_dot in raw_dotations:
                dResults[(raw_dot.post_id.id,raw_dot.associated_account_id.id)] = {'dpt_name': raw_dot.post_id.department_id.name,
                                                                'post_name': raw_dot.post_id.name,
                                                                'category_name': raw_dot.associated_account_id.cci_budget_group_id.category_id.name,
                                                                'group_name': raw_dot.associated_account_id.cci_budget_group_id.name,
                                                                'account_name': raw_dot.associated_account_id.code + ' ' + raw_dot.associated_account_id.name,
                                                                'amount_real':0.0,
                                                                'amount_budget':0.0,
                                                                'type':_get_total_type(raw_dot.associated_account_id),
                                                                'subtype':_get_total_subtype(raw_dot.associated_account_id.code)}
                if raw_dot.associated_account_id.id not in concerned_account_ids:
                    concerned_account_ids.append(raw_dot.associated_account_id.id)
                if raw_dot.account_id.id not in concerned_account_ids:
                    concerned_account_ids.append(raw_dot.account_id.id)
                dAssociated[(raw_dot.post_id.id,raw_dot.account_id.id)] = (raw_dot.post_id.id,raw_dot.associated_account_id.id)
            raw_period_dotation_ids = raw_per_dot_obj.search(cr,uid,[('raw_dotation_id','in',raw_dotation_ids),('period_id','in',concerned_period_budget_ids)])
            if raw_period_dotation_ids:
                raw_period_dotations = raw_per_dot_obj.browse(cr,uid,raw_period_dotation_ids)
                for raw_per_dot in raw_period_dotations:
                    if raw_per_dot.raw_dotation_id.post_id and raw_per_dot.raw_dotation_id.account_id and raw_per_dot.raw_dotation_id.associated_account_id:
                        if dResults.has_key((raw_per_dot.raw_dotation_id.post_id.id,raw_per_dot.raw_dotation_id.associated_account_id.id)):
                            record = dResults[(raw_per_dot.raw_dotation_id.post_id.id,raw_per_dot.raw_dotation_id.associated_account_id.id)]
                            record['amount_budget'] += (raw_per_dot.amount or 0.0)
                            dResults[(raw_per_dot.raw_dotation_id.post_id.id,raw_per_dot.raw_dotation_id.associated_account_id.id)] = record
        # normal dotation : every not associated
        dotation_ids = dotation_obj.search(cr,uid,[('budget_id','=',budget.id)])
        dotations = dotation_obj.browse(cr,uid,dotation_ids)
        #concerned_account_ids = []
        #concerned_aaccount_ids = []
        for dot in dotations:
            if (dot.post_id.id,dot.account_id.id) not in dAssociated.values():
                dResults[(dot.post_id.id,dot.account_id.id)] = {'dpt_name': dot.post_id.department_id.name,
                                                                'post_name': dot.post_id.name,
                                                                'category_name': dot.account_id.cci_budget_group_id.category_id.name,
                                                                'group_name': dot.account_id.cci_budget_group_id.name,
                                                                'account_name': dot.account_id.code + ' ' + dot.account_id.name,
                                                                'amount_real':0.0,
                                                                'amount_budget':0.0,
                                                                'type':_get_total_type(dot.account_id),
                                                                'subtype':_get_total_subtype(dot.account_id.code)}
                if dot.account_id.id not in concerned_account_ids:
                    concerned_account_ids.append(dot.account_id.id)
        concerned_aaccount_ids = aaccount_obj.search(cr,uid,[('cci_post_id','!=',False)])
        period_dotation_ids = per_dot_obj.search(cr,uid,[('dotation_id','in',dotation_ids),('period_id','in',concerned_period_budget_ids)])
        if period_dotation_ids:
            period_dotations = per_dot_obj.browse(cr,uid,period_dotation_ids)
            for per_dot in period_dotations:
                if per_dot.dotation_id.post_id and per_dot.dotation_id.account_id:
                    if (per_dot.dotation_id.post_id.id,per_dot.dotation_id.account_id.id) not in dAssociated.values():
                        if dResults.has_key((per_dot.dotation_id.post_id.id,per_dot.dotation_id.account_id.id)):
                            record = dResults[(per_dot.dotation_id.post_id.id,per_dot.dotation_id.account_id.id)]
                            record['amount_budget'] += (per_dot.amount or 0.0)
                            dResults[(per_dot.dotation_id.post_id.id,per_dot.dotation_id.account_id.id)] = record
        
        if data['form']['fiscalyear']:
            amline_ids = amline_obj.search(cr,uid,[('period_id','in',concerned_period_ids),('journal_id','not in',smoothing_journal_ids)])
            index = 0
            size = 1000
            while amline_ids[index:index+size]:
                amlines = amline_obj.browse(cr,uid,amline_ids[index:index+size])
                for amline in amlines:
                    if amline.analytic_lines:
                        for analytic_line in amline.analytic_lines:
                            if analytic_line.account_id.id in concerned_aaccount_ids and dAnalytic2Post.has_key(analytic_line.account_id.id):
                                final_account = amline.account_id
                                res_key = (dAnalytic2Post[analytic_line.account_id.id],amline.account_id.id)
                                if res_key in dAssociated.keys():
                                    res_key = dAssociated[res_key]
                                    final_account_id = res_key[1]
                                    final_account = account_obj.browse(cr,uid,final_account_id)
                                if final_account and final_account.cci_budget_group_id and final_account.cci_budget_group_id.category_id and ((data['form']['move_state'] == 'draft') or (amline.move_id.state == data['form']['move_state'])):
                                    if not dResults.has_key(res_key):
                                        post = post_obj.browse(cr,uid,dAnalytic2Post[analytic_line.account_id.id])
                                        record = {'dpt_name': post.department_id.name,
                                                  'post_name': post.name,
                                                  'category_name': final_account.cci_budget_group_id.category_id.name,
                                                  'group_name': final_account.cci_budget_group_id.name,
                                                  'account_name': final_account.code + ' ' + final_account.name,
                                                  'amount_real':0.0,
                                                  'amount_budget':0.0,
                                                  'type':_get_total_type(final_account),
                                                  'subtype':_get_total_subtype(final_account.code),
                                                 }
                                    else:
                                        record = dResults[res_key]
                                    record['amount_real'] += (analytic_line.amount or 0.0)
                                    dResults[res_key] = record
                index += size
        wb1 = Workbook()

        # styles
        border_right = Borders()
        border_right.right = 2
        border_right_and_bottom = Borders()
        border_right_and_bottom.right = 2
        border_right_and_bottom.bottom = 2
        border_bottom = Borders()
        border_bottom.bottom = 2
        all_borders = Borders()
        all_borders.left = 2
        all_borders.right = 2
        all_borders.top = 2
        all_borders.bottom = 2
        al = Alignment()
        al.horz = Alignment.HORZ_CENTER
        al.vert = Alignment.VERT_CENTER
        font12 = Font()
        font12.height = 240
        font_title_project = Font()
        font_title_project.height = 300
        font_title_project.colour_index = 0x12 ## 'Navy'
        
        pattern_silver = Pattern()
        pattern_silver._pattern_back_colour = 0x16  ### 'silver'
        pattern_silver.pattern = 0x01 # SOLID
        pattern_grey = Pattern()
        pattern_grey._pattern_back_colour = 0x17  ### 'grey'
        pattern_grey.pattern = 0x01 # SOLID
        pattern_yellow = Pattern()
        #pattern_yellow._pattern_back_colour = 0x0D  ### 'grey'
        pattern_yellow.pattern = 0x01 # SOLID
        pattern_yellow.set_pattern_fore_colour(0x0D)  ### 'yellow'
        pattern_aqua = Pattern()
        pattern_aqua.pattern = 0x01 # SOLID
        pattern_aqua.set_pattern_fore_colour(pyexcel_colours['aqua'])

        style_right = XFStyle()
        style_right.borders = border_right

        style_red_float = XFStyle()
        style_red_float.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_red_float.borders = border_right
        
        style_red_float_wo_border = XFStyle()
        style_red_float_wo_border.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'

        style_total_red_float = XFStyle()
        style_total_red_float.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_total_red_float.font.bold = True

        style_total_red_float_bordered = XFStyle()
        style_total_red_float_bordered.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_total_red_float_bordered.font.bold = True
        style_total_red_float_bordered.borders = all_borders

        style_total_red_float_right = XFStyle()
        style_total_red_float_right.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_total_red_float_right.font.bold = True
        style_total_red_float_right.borders = border_right

        style_total_red_float_wo_border = XFStyle()
        style_total_red_float_wo_border.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_total_red_float_wo_border.font.bold = True

        style_red_float_aqua = XFStyle()
        style_red_float_aqua.num_format_str = '_(#,##0.00_);[Red](#,##0.00)'
        style_red_float_aqua.pattern = pattern_aqua

        style_float = XFStyle()
        style_float.num_format_str = '#,##0.00'

        style_float_aqua = XFStyle()
        style_float_aqua.num_format_str = '#,##0.00'
        style_float_aqua.pattern = pattern_aqua

        style_float_right = XFStyle()
        style_float_right.num_format_str = '#,##0.00'
        style_float_right.borders = border_right
        
        style_total_float = XFStyle()
        style_total_float.num_format_str = '#,##0.00'
        style_total_float.font.bold = True

        style_total_float_bordered = XFStyle()
        style_total_float_bordered.num_format_str = '#,##0.00'
        style_total_float_bordered.font.bold = True
        style_total_float_bordered.borders = all_borders

        style_total_float_right = XFStyle()
        style_total_float_right.num_format_str = '#,##0.00'
        style_total_float_right.font.bold = True
        style_total_float_right.borders = border_right

        style_total_float_bottom = XFStyle()
        style_total_float_bottom.num_format_str = '#,##0.00'
        style_total_float_bottom.font.bold = True
        style_total_float_bottom.borders = border_bottom

        style_total_float_right_bottom = XFStyle()
        style_total_float_right_bottom.num_format_str = '#,##0.00'
        style_total_float_right_bottom.font.bold = True
        style_total_float_right_bottom.borders = border_right_and_bottom

        style_total = XFStyle()
        style_total.font.bold = True

        style_total_right = XFStyle()
        style_total_right.font.bold = True
        style_total_right.borders = border_right

        style_total_right_bottom = XFStyle()
        style_total_right_bottom.font.bold = True
        style_total_right_bottom.borders = border_right_and_bottom

        style_total_bottom = XFStyle()
        style_total_bottom.font.bold = True
        style_total_bottom.borders = border_bottom

        style_total_bordered = XFStyle()
        style_total_bordered.font.bold = True
        style_total_bordered.borders = all_borders

        style_pourcent = XFStyle()
        style_pourcent.num_format_str = '0.00%'

        style_pourcent_right = XFStyle()
        style_pourcent_right.num_format_str = '0.00%'
        style_pourcent_right.borders = border_right

        style_pourcent_total_bordered = XFStyle()
        style_pourcent_total_bordered.num_format_str = '0.00%'
        style_pourcent_total_bordered.font.bold = True
        style_pourcent_total_bordered.borders = all_borders

        style_pourcent_subtotal = XFStyle()
        style_pourcent_subtotal.num_format_str = '0.00%'
        style_pourcent_subtotal.font.bold = True
        style_pourcent_subtotal.borders = border_bottom

        style_pourcent_subtotal_right = XFStyle()
        style_pourcent_subtotal_right.num_format_str = '0.00%'
        style_pourcent_subtotal_right.font.bold = True
        style_pourcent_subtotal_right.borders = border_right_and_bottom

        style_grand_total = XFStyle()
        style_grand_total.font.bold = True
        style_grand_total.borders = all_borders
        style_grand_total.alignment = al
        style_grand_total.font = font12
        style_grand_total.pattern = pattern_yellow

        style_project_title = XFStyle()
        style_project_title.font = font_title_project
        
        ws1 = wb1.add_sheet('Global')
        ws1.write(0,0,_('Detailed Budget ') + budget.name)
        ws1.write(2,0,_('Extract date:'))
        ws1.write(2,1,string_now)
        ws1.write(3,0,_('Budget:'))
        ws1.write(3,1,budget.name + ' ' + period_budget_from.name + ' -> ' + period_budget_to.name)
        line = 3
        if data['form']['fiscalyear']:
            ws1.write(4,0,_('Accounting Data:'))
            ws1.write(4,1,fiscalyear.name + ' ' + period_from.name + ' -> ' + period_to.name + ((data['form']['move_state'] == 'draft') and _(' (All even draft)') or _(' (only posted)')) )
            ws1.write(6,2,budget.name,style_total_bordered)
            ws1.write(6,8,fiscalyear.name,style_total_bordered)
            line = 5
        ws1.write(line+2,0,_('Categories'),style_total_bordered)
        ws1.write(line+2,1,_('Products'),style_total_bordered)
        ws1.write(line+2,2,u'C.A. direct',style_total_bordered)
        ws1.write(line+2,3,u'Dépenses',style_total_bordered)
        ws1.write(line+2,4,u'Marge directe',style_total_bordered)
        ws1.write(line+2,5,u'Aides',style_total_bordered)
        ws1.write(line+2,6,u'Bénéfice net',style_total_bordered)        
        ws1.write(line+2,7,u'Exceptionnel',style_total_bordered)        
        col_final = 7
        if data['form']['fiscalyear']:
            ws1.write(line+2,8,u'C.A. direct',style_total_bordered)
            ws1.write(line+2,9,u'Dépenses',style_total_bordered)
            ws1.write(line+2,10,u'Marge directe',style_total_bordered)
            ws1.write(line+2,11,u'Aides',style_total_bordered)
            ws1.write(line+2,12,u'Bénéfice net',style_total_bordered)
            ws1.write(line+2,13,u'Exceptionnel',style_total_bordered)        
            col_final = 13
        
        ws2 = wb1.add_sheet(_('Details'))
        ws2.write(0,0,'post_id',style_total_bordered)
        ws2.write(0,1,'account_id',style_total_bordered)
        ws2.write(0,2,_('Department'),style_total_bordered)
        ws2.write(0,3,_('Project'),style_total_bordered)
        ws2.write(0,4,_('Group'),style_total_bordered)
        ws2.write(0,5,_('SubGroup'),style_total_bordered)
        ws2.write(0,6,_('Account'),style_total_bordered)
        ws2.write(0,7,'Budget',style_total_bordered)
        ws2.write(1,7,_('C.A. direct'),style_total_bordered)
        ws2.write(1,8,_('Dépenses'),style_total_bordered)
        ws2.write(1,9,_('Aides'),style_total_bordered)
        if data['form']['fiscalyear']:
            ws2.write(0,10,u'Réalisé',style_total_bordered)
            ws2.write(1,10,_('C.A. direct'),style_total_bordered)
            ws2.write(1,11,_('Dépenses'),style_total_bordered)
            ws2.write(1,12,_('Aides'),style_total_bordered)
            ws2.write(1,13,'Type')
            ws2.write(1,14,'SubType')
        else:
            ws2.write(1,10,'Type')
            ws2.write(1,11,'SubType')
        
        line += 3
        line2 = 2
        for key,record in dResults.items():
            ws2.write(line2,0,key[0])
            ws2.write(line2,1,key[1])
            ws2.write(line2,2,record['dpt_name'])
            ws2.write(line2,3,record['post_name'])
            ws2.write(line2,4,record['category_name'])
            ws2.write(line2,5,record['group_name'])
            ws2.write(line2,6,record['account_name'])
            if record['type'] == 'turnover':
                ws2.write(line2,7,record['amount_budget'])
                if data['form']['fiscalyear']:
                    ws2.write(line2,10,record['amount_real'])
            elif record['type'] == 'expense':
                ws2.write(line2,8,record['amount_budget'])
                if data['form']['fiscalyear']:
                    ws2.write(line2,11,record['amount_real'])
            elif record['type'] == 'support':
                ws2.write(line2,9,record['amount_budget'])
                if data['form']['fiscalyear']:
                    ws2.write(line2,12,record['amount_real'])
            if data['form']['fiscalyear']:
                ws2.write(line2,13,record['type'])
                ws2.write(line2,14,record['subtype'])
            else:
                ws2.write(line2,10,record['type'])
                ws2.write(line2,11,record['subtype'])
            line2 += 1
        # now we get cumulated data by post, sorted by department and doing so, we create detail sheet for each post by group, under-group, account
        # for the general sheet we need the list of departments then post
        dpt_obj = pooler.get_pool(cr.dbname).get('cci_budget.department')
        dpt_ids = dpt_obj.search(cr,uid,[])
        dpts = dpt_obj.read(cr,uid,dpt_ids,['name','sequence'])
        sdpts = sorted(dpts, key=lambda k: k['sequence'])
        # for the detail sheets, we need the list of groups then subgroups then accounts
        group_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget_account_category')
        group_ids = group_obj.search(cr,uid,[])
        groups = group_obj.read(cr,uid,group_ids,['name','sequence'])
        sgroups = sorted(groups, key=lambda k: k['sequence'])
        subgroup_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget_account_group')
        subgroup_ids = subgroup_obj.search(cr,uid,[])
        subgroups = subgroup_obj.read(cr,uid,subgroup_ids,['name','category_id'])
        ssubgroups = sorted(subgroups, key=lambda k: k['name'])
        #
        gen_lines = []

        gen_supports = {}
        gen_supports['membership'] = 0.0
        gen_supports['structural_sponsor'] = 0.0
        gen_supports['product_sponsor'] = 0.0
        gen_supports['subsidy'] = 0.0
        gen_supports['normal'] = 0.0
        #range_cols = [2,3,4,5,6]
        range_cols = [2,3,4,5,6,7]
        if data['form']['fiscalyear']:
            gen_real_supports = {}
            gen_real_supports['membership'] = 0.0
            gen_real_supports['structural_sponsor'] = 0.0
            gen_real_supports['product_sponsor'] = 0.0
            gen_real_supports['subsidy'] = 0.0
            gen_real_supports['normal'] = 0.0
            #range_cols = [2,3,4,5,6,7,8,9,10,11]
            range_cols = [2,3,4,5,6,7,8,9,10,11,12,13]

        for dpt in sdpts:
            post_ids = post_obj.search(cr,uid,[('department_id','=',dpt['id'])])
            line_beg_dpt = line
            if post_ids:
                posts = post_obj.read(cr,uid,post_ids,['name'])
                sposts = sorted(posts, key=lambda k: k['name'])
                post_num = 1
                first_dpt_line = line
                for post in sposts:
                    dAccounts = {}
                    dSubGroups = {}
                    dGroups = {}
                    total_real = {}
                    total_real['turnover'] = 0.0
                    total_real['expense'] = 0.0
                    total_real['support'] = 0.0
                    total_real['other'] = 0.0
                    total_budget = {}
                    total_budget['turnover'] = 0.0
                    total_budget['expense'] = 0.0
                    total_budget['support'] = 0.0
                    total_budget['other'] = 0.0
                    for key,record in dResults.items():
                        if key[0] == post['id']:
                            if data['form']['fiscalyear']:
                                total_real[record['type']] += record['amount_real']
                                gen_real_supports[record['subtype']] += record['amount_real']
                            total_budget[record['type']] += record['amount_budget']
                            gen_supports[record['subtype']] += record['amount_budget']

                            # sum the same data for the detail sheet
                            account = account_obj.browse(cr,uid,key[1])
                            if account.id not in dAccounts.keys():
                                sub = {'id':account.id,'real':0.0,'budget':0.0}
                            else:
                                sub = dAccounts[account.id]
                            sub['real'] += record['amount_real']
                            sub['budget'] += record['amount_budget']
                            dAccounts[account.id] = sub
                            if account.cci_budget_group_id.id not in dSubGroups.keys():
                                sub = {'id':account.cci_budget_group_id.id,'real':0.0,'budget':0.0}
                            else:
                                sub = dSubGroups[account.cci_budget_group_id.id]
                            sub['real'] += record['amount_real']
                            sub['budget'] += record['amount_budget']
                            dSubGroups[account.cci_budget_group_id.id] = sub
                            if account.cci_budget_group_id.category_id.id not in dGroups.keys():
                                sub = {'id':account.cci_budget_group_id.category_id.id,'real':0.0,'budget':0.0}
                            else:
                                sub = dGroups[account.cci_budget_group_id.category_id.id]
                            sub['real'] += record['amount_real']
                            sub['budget'] += record['amount_budget']
                            dGroups[account.cci_budget_group_id.category_id.id] = sub
                    if post_num == 1:
                        ws1.write(line,0,dpt['name'])
                    ws1.write(line,1,post['name'],style_right)
                    ws1.write(line,2,round(total_budget['turnover'],2),style_float)
                    ws1.write(line,3,round(total_budget['expense'],2),style_float)
                    ws1.write(line,4,Formula(full_cell(line,2)+'+'+full_cell(line,3)),style_float_right)
                    ws1.write(line,5,round(total_budget['support'],2),style_float)
                    ws1.write(line,6,Formula(full_cell(line,4)+'+'+full_cell(line,5)),style_float_right)
                    ws1.write(line,7,round(total_budget['other'],2),style_float_right)
                    if data['form']['fiscalyear']:
                        ws1.write(line,8,round(total_real['turnover'],2),style_float)
                        ws1.write(line,9,round(total_real['expense'],2),style_float)
                        ws1.write(line,10,Formula(full_cell(line,8)+'+'+full_cell(line,9)),style_float_right)
                        ws1.write(line,11,round(total_real['support'],2),style_float)
                        ws1.write(line,12,Formula(full_cell(line,10)+'+'+full_cell(line,11)),style_float_right)
                        ws1.write(line,13,round(total_real['other'],2),style_float_right)
                    post_num += 1
                    line += 1

                    # put the details on a specific sheet by post
                    if dAccounts: # if there is any moves on this post
                        wso = wb1.add_sheet(_('Project ')+post['name'])
                        wso.write(0,0,_('Project ')+post['name'],style_project_title)
                        wso.write(2,0,_('Extract date:'))
                        wso.write(2,1,string_now)
                        wso.write(3,0,_('Budget:'))
                        wso.write(3,1,budget.name + ' ' + period_budget_from.name + ' -> ' + period_budget_to.name )
                        if data['form']['fiscalyear']:
                            wso.write(4,0,_('Accounting Data:'))
                            wso.write(4,1,fiscalyear.name + ' ' + period_from.name + ' -> ' + period_to.name)
                        wso.write(6,0,_('Group/Sub/Account'),style_total_bordered)
                        if data['form']['fiscalyear']:
                            wso.write(6,1,_('Real'),style_total_bordered)
                        wso.write(6,2,_('Budget'),style_total_bordered)
                        if data['form']['fiscalyear']:
                            if data['form']['direction'] == '1-2':
                                col_name = _('Difference [Reel - Budget]')
                            else:
                                col_name = _('Difference [Budget - Reel]')
                            wso.write(6,3,col_name,style_total_bordered)
                        wso.write(7,0,_('TOTAL'),style_total_bordered)
                        line_other = 7
                        
                        det_sum_turnover = []
                        det_sum_budget = []
                        det_sum_diff = []
                        line_gen = line_other
                        for group in sgroups:
                            line_other += 1
                            wso.write(line_other,0,group['name'],style_total)
                            group_sum_turnover = []
                            group_sum_budget = []
                            group_sum_diff = []
                            line_group = line_other
                            for subgroup in subgroups:
                                subgroup_present = False
                                if subgroup['category_id'][0] == group['id']:
                                    for acc in saccounts:
                                        if acc['cci_budget_group_id'][0] == subgroup['id'] and acc['id'] in dAccounts.keys():
                                            subgroup_present = True
                                if subgroup_present:
                                    line_other += 1
                                    wso.write(line_other,0,' >--'+subgroup['name'],style_total_float)
                                    subgroup_sum_turnover = []
                                    subgroup_sum_budget = []
                                    subgroup_sum_diff = []
                                    line_subgroup = line_other
                                    for acc in saccounts:
                                        if acc['cci_budget_group_id'][0] == subgroup['id'] and acc['id'] in dAccounts.keys():
                                            line_other += 1
                                            wso.write(line_other,0,' >-- >--'+acc['code']+' '+acc['name'])
                                            if data['form']['fiscalyear']:
                                                wso.write(line_other,1,round(dAccounts[acc['id']]['real'],2),style_float_aqua)
                                            wso.write(line_other,2,round(dAccounts[acc['id']]['budget'],2),style_float_aqua)
                                            if data['form']['fiscalyear']:
                                                if data['form']['direction'] == '1-2':
                                                    wso.write(line_other,3,Formula("%s-%s" % (full_cell(line_other,1),full_cell(line_other,2))),style_red_float_aqua)
                                                else:
                                                    wso.write(line_other,3,Formula("%s-%s" % (full_cell(line_other,2),full_cell(line_other,1))),style_red_float_aqua)
                                            subgroup_sum_turnover.append('B' + str(line_other+1))
                                            subgroup_sum_budget.append('C' + str(line_other+1))
                                            subgroup_sum_diff.append('D' + str(line_other+1))
                                    if data['form']['fiscalyear']:
                                        wso.write(line_subgroup,1,Formula("SUM(%s)" % ';'.join(subgroup_sum_turnover)),style_total_float)
                                        group_sum_turnover.append('B' + str(line_subgroup+1))
                                    wso.write(line_subgroup,2,Formula("SUM(%s)" % ';'.join(subgroup_sum_budget)),style_total_float)
                                    group_sum_budget.append('C' + str(line_subgroup+1))
                                    if data['form']['fiscalyear']:
                                        wso.write(line_subgroup,3,Formula("SUM(%s)" % ';'.join(subgroup_sum_diff)),style_total_red_float)
                                        group_sum_diff.append('D' + str(line_subgroup+1))
                            if data['form']['fiscalyear']:
                                if group_sum_turnover:
                                    wso.write(line_group,1,Formula("SUM(%s)" % ';'.join(group_sum_turnover)),style_total_float)
                                else:
                                    wso.write(line_group,1,0.0,style_total_float)
                            if group_sum_budget:
                                wso.write(line_group,2,Formula("SUM(%s)" % ';'.join(group_sum_budget)),style_total_float)
                            else:
                                wso.write(line_group,2,0.0,style_total_float)
                            if data['form']['fiscalyear']:
                                if group_sum_diff:
                                    wso.write(line_group,3,Formula("SUM(%s)" % ';'.join(group_sum_diff)),style_total_float)
                                else:
                                    wso.write(line_group,3,0.0,style_total_float)
                            if data['form']['fiscalyear']:
                                det_sum_turnover.append('B' + str(line_group+1))
                            det_sum_budget.append('C' + str(line_group+1))
                            if data['form']['fiscalyear']:
                                det_sum_diff.append('D' + str(line_group+1))
                        if data['form']['fiscalyear']:
                            wso.write(line_gen,1,Formula("SUM(%s)" % ';'.join(det_sum_turnover)),style_total_float)
                        wso.write(line_gen,2,Formula("SUM(%s)" % ';'.join(det_sum_budget)),style_total_float)
                        if data['form']['fiscalyear']:
                            wso.write(line_gen,3,Formula("SUM(%s)" % ';'.join(det_sum_diff)),style_total_float)
                        wso = None
                last_dpt_line = line - 1
                
            ws1.write(line,1,'',style_total_right)
            ws1.write(line,4,'',style_total_right)
            ws1.write(line,6,'',style_total_right)
            ws1.write(line,7,'',style_total_right)
            if data['form']['fiscalyear']:
                ws1.write(line,10,'',style_total_right)
                ws1.write(line,12,'',style_total_right)
                ws1.write(line,13,'',style_total_right)
            line += 1
            ws1.write(line,0,'',style_total_bottom)
            ws1.write(line,1,_('Sub total'),style_total_right_bottom)
            for col in range_cols:
                if col in [4,6,7,10,12,13]:
                    ws1.write(line,col,Formula("SUM(%s:%s)" % (full_cell(first_dpt_line,col),full_cell(last_dpt_line,col))),style_total_float_right_bottom)
                else:
                    ws1.write(line,col,Formula("SUM(%s:%s)" % (full_cell(first_dpt_line,col),full_cell(last_dpt_line,col))),style_total_float_bottom)
            gen_lines.append(line)
            line += 1
            #ws1.write(line,0,'',style_pourcent_subtotal)
            #ws1.write(line,1,_('%'),style_pourcent_subtotal_right) # the formulas will be written after, once the grand total line known
            #subtotal_pourcent_lines.append(line)
            #line += 1
            #gen_total_turnover += dpt_total_turnover
            #gen_total_budget += dpt_total_budget
        ws1.write(line,0,_('Grand Total'),style_grand_total)
        ws1.write(line,1,'',style_grand_total)
        for col in range_cols:
                if col in [4,6,7,10,12,13]:
                    ws1.write(line,col,Formula("SUM(%s)" % ';'.join([full_cell(l,col) for l in gen_lines])),style_total_float_right_bottom)
                else:
                    ws1.write(line,col,Formula("SUM(%s)" % ';'.join([full_cell(l,col) for l in gen_lines])),style_total_float_bottom)
        
        # pourcents of subtotals
        #for line_num in subtotal_pourcent_lines:
        #    for col_num in range(2,12):
        #        if col_num in (4,7,9,11):
        #            ws1.write(line_num,col_num,Formula("%s/%s" % (full_cell(line_num-1,col_num,'none'),full_cell(line,col_num,'none'))),style_pourcent_subtotal_right)
        #        else:
        #            ws1.write(line_num,col_num,Formula("%s/%s" % (full_cell(line_num-1,col_num,'none'),full_cell(line,col_num,'none'))),style_pourcent_subtotal)
        # sub table with 4 types of "helps"
        ws1.write(5,col_final+2,_('BUDGET SUPPORTS'),style_total_bordered)
        ws1.write(5,col_final+3,_('Amount'),style_total_bordered)
        ws1.write(5,col_final+4,_('%'),style_total_bordered)
        ws1.write(6,col_final+2,_('Mbs'),style_total_bordered)
        ws1.write(7,col_final+2,_('S Structural'),style_total_bordered)
        ws1.write(8,col_final+2,_('S Product'),style_total_bordered)
        ws1.write(9,col_final+2,_('Subsidies'),style_total_bordered)
        ws1.write(10,col_final+2,_('TOTAL'),style_total_bordered)
        ws1.write(6,col_final+3,gen_supports['membership'],style_float)
        ws1.write(7,col_final+3,gen_supports['structural_sponsor'],style_float)
        ws1.write(8,col_final+3,gen_supports['product_sponsor'],style_float)
        ws1.write(9,col_final+3,gen_supports['subsidy'],style_float)
        gen_all = gen_supports['membership'] + gen_supports['structural_sponsor'] + gen_supports['product_sponsor'] + gen_supports['subsidy']
        ws1.write(10,col_final+3,gen_all,style_total_float_bordered)
        col_numbers = num_to_letter(col_final+3)
        total_numbers = num_to_letter(col_final+4)
        ws1.write(6,col_final+4,Formula("%s$7/%s$11" % (col_numbers,col_numbers)),style_pourcent_right)
        ws1.write(7,col_final+4,Formula("%s$8/%s$11" % (col_numbers,col_numbers)),style_pourcent_right)
        ws1.write(8,col_final+4,Formula("%s$9/%s$11" % (col_numbers,col_numbers)),style_pourcent_right)
        ws1.write(9,col_final+4,Formula("%s$10/%s$11" % (col_numbers,col_numbers)),style_pourcent_right)
        ws1.write(10,col_final+4,Formula("SUM(%s7:%s10)" % (total_numbers,total_numbers)),style_pourcent_total_bordered)
        if data['form']['fiscalyear']:
            ws1.write(12,col_final+2,_('REAL SUPPORTS'),style_total_bordered)
            ws1.write(12,col_final+3,_('Amount'),style_total_bordered)
            ws1.write(12,col_final+4,_('%'),style_total_bordered)
            ws1.write(13,col_final+2,_('Mbs'),style_total_bordered)
            ws1.write(14,col_final+2,_('S Structural'),style_total_bordered)
            ws1.write(15,col_final+2,_('S Product'),style_total_bordered)
            ws1.write(16,col_final+2,_('Subsidies'),style_total_bordered)
            ws1.write(17,col_final+2,_('TOTAL'),style_total_bordered)
            ws1.write(13,col_final+3,gen_real_supports['membership'],style_float)
            ws1.write(14,col_final+3,gen_real_supports['structural_sponsor'],style_float)
            ws1.write(15,col_final+3,gen_real_supports['product_sponsor'],style_float)
            ws1.write(16,col_final+3,gen_real_supports['subsidy'],style_float)
            gen_all = gen_real_supports['membership'] + gen_real_supports['structural_sponsor'] + gen_real_supports['product_sponsor'] + gen_real_supports['subsidy']
            ws1.write(17,col_final+3,gen_all,style_total_float_bordered)
            col_numbers = num_to_letter(col_final+3)
            total_numbers = num_to_letter(col_final+4)
            ws1.write(13,col_final+4,Formula("%s14/%s18" % (col_numbers,col_numbers)),style_pourcent_right)
            ws1.write(14,col_final+4,Formula("%s15/%s18" % (col_numbers,col_numbers)),style_pourcent_right)
            ws1.write(15,col_final+4,Formula("%s16/%s18" % (col_numbers,col_numbers)),style_pourcent_right)
            ws1.write(16,col_final+4,Formula("%s17/%s18" % (col_numbers,col_numbers)),style_pourcent_right)
            ws1.write(17,col_final+4,Formula("SUM(%s14:%s17)" % (total_numbers,total_numbers)),style_pourcent_total_bordered)
        
        wb1.save('tableau_budget.xls')
        result_file = open('tableau_budget.xls','rb').read()

        # give the result tos the user
        data['form']['msg']='Save the File with '".xls"' extension.'
        data['form']['tableau_budget.xls']=base64.encodestring(result_file)
        return data['form']
    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('extract','Extract') ]}
        },
        'extract': {
            'actions': [_extract_excel],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        }
    }
extract_raw_splitted_data('extract_raw_splitted_data')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

