# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import wizard
import time
import datetime
import tools
import pooler
import base64
import csv

form = """<?xml version="1.0"?>
<form string="Inject new Amounts on a RAW Dotation">
    <field name="budget_id" colspan="4" width="800" required="1"/>
    <field name="account_id" colspan="4" required="1"/>
    <field name="analytic_distri_id" colspan="4" required="1"/>
    <field name="associated_account_id" colspan="4" required="1"/>
    <newline/>
    <separator string="Amounts by month" colspan="4"/>
    <group col="12" colspan="4">
        <field name="amount1" nolabel="1"/>
        <field name="amount2" nolabel="1"/>
        <field name="amount3" nolabel="1"/>
        <field name="amount4" nolabel="1"/>
        <field name="amount5" nolabel="1"/>
        <field name="amount6" nolabel="1"/>
        <field name="amount7" nolabel="1"/>
        <field name="amount8" nolabel="1"/>
        <field name="amount9" nolabel="1"/>
        <field name="amount10" nolabel="1"/>
        <field name="amount11" nolabel="1"/>
        <field name="amount12" nolabel="1"/>
    </group>
    <field name="comments" colpspan="4"/>
</form>"""

fields = {
    'budget_id': {'string': 'Budget', 'type':'many2one','relation': 'cci_budget.budget','required': True},
    'account_id':  {'string':'Account', 'type':'many2one','relation': 'account.account','required': True},
    'associated_account_id':  {'string':'Associated Account', 'type':'many2one','relation': 'account.account','required': True},
    'analytic_distri_id':  {'string':'Analytic Distribution', 'type':'many2one','relation': 'account.analytic.plan.instance','required': True},
    'amount1':{'string':'Amount in January','type': 'float', 'default':0.0},
    'amount2':{'string':'Amount in February','type': 'float', 'default':0.0},
    'amount3':{'string':'Amount in March','type': 'float', 'default':0.0},
    'amount4':{'string':'Amount in April','type': 'float', 'default':0.0},
    'amount5':{'string':'Amount in May','type': 'float', 'default':0.0},
    'amount6':{'string':'Amount in June','type': 'float', 'default':0.0},
    'amount7':{'string':'Amount in July','type': 'float', 'default':0.0},
    'amount8':{'string':'Amount in Augustus','type': 'float', 'default':0.0},
    'amount9':{'string':'Amount in September','type': 'float', 'default':0.0},
    'amount10':{'string':'Amount in October','type': 'float', 'default':0.0},
    'amount11':{'string':'Amount in November','type': 'float', 'default':0.0},
    'amount12':{'string':'Amount in December','type': 'float', 'default':0.0},
    'comments':{'string':'Comments','type':'text'},
   }

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="Results" colspan="4"/>
     <field name="msg" colspan="4" nolabel="1" readonly="1" width="800"/>
</form>"""

msg_fields = {
    'msg': {'string':'Result', 'type':'text', 'default':'Amounts injected'},
}

class inject_new_raw_amounts(wizard.interface):
    def _inject(self,cr,uid,data,context):
        budget_obj = pooler.get_pool(cr.dbname).get('cci_budget.budget')
        account_obj = pooler.get_pool(cr.dbname).get('account.account')
        distri_obj = pooler.get_pool(cr.dbname).get('account.analytic.plan.instance')
        raw_dotation_obj = pooler.get_pool(cr.dbname).get('cci_budget.raw_dotation')
        raw_per_dot_obj = pooler.get_pool(cr.dbname).get('cci_budget.raw_period_dotation')
        #
        budget = budget_obj.browse(cr,uid,data['form']['budget_id'])
        #
        periods = []
        for period in budget.fiscalyear_id.period_ids:
            if not period.special:
                periods.append({'id':period.id,'date_start':period.date_start})
        speriods = sorted(periods, key= lambda k: k['date_start'])
        #period_ids = [x['id'] for x in speriods]
        today = datetime.date.today().strftime('%d/%m/%Y')
        amounts = [data['form']['amount1'],
                   data['form']['amount2'],
                   data['form']['amount3'],
                   data['form']['amount4'],
                   data['form']['amount5'],
                   data['form']['amount6'],
                   data['form']['amount7'],
                   data['form']['amount8'],
                   data['form']['amount9'],
                   data['form']['amount10'],
                   data['form']['amount11'],
                   data['form']['amount12']]
        account = account_obj.browse(cr,uid,data['form']['account_id'])
        associated_account = account_obj.browse(cr,uid,data['form']['associated_account_id'])
        distri = distri_obj.browse(cr,uid,data['form']['analytic_distri_id'])
        sum_rates = 0
        for dist_line in distri.account_ids:
            if dist_line.analytic_account_id and dist_line.analytic_account_id.cci_post_id:
                sum_rates += dist_line.rate or 0
        if int(sum_rates) == 100:
            for dist_line in distri.account_ids:
                if dist_line.analytic_account_id.cci_post_id:
                    raw_dot_ids = raw_dotation_obj.search(cr,uid,[('budget_id','=',budget.id),('post_id','=',dist_line.analytic_account_id.cci_post_id.id),('account_id','=',account.id),('associated_account_id','=',associated_account.id)])
                    if not raw_dot_ids:
                        raw_dotation_id = raw_dotation_obj.create(cr,uid,{'budget_id':budget.id,
                                                                          'post_id':dist_line.analytic_account_id.cci_post_id.id,
                                                                          'account_id':account.id,
                                                                          'associated_account_id':associated_account.id,
                                                                          'comments':data['form']['comments'] or ''})
                    else:
                        raw_dotation_id = raw_dot_ids[0]
                    if raw_dotation_id:
                        index_period = 0
                        for period in speriods:
                            if dist_line.analytic_account_id:
                                amount_dotation = round(amounts[index_period] * (dist_line.rate or 0) / 100,2)
                                new_id = raw_per_dot_obj.create(cr,uid,{'raw_dotation_id':raw_dotation_id,'period_id':period['id'],'comments':data['form']['comments'] or '','amount':amount_dotation})
                            index_period += 1
            data['form']['msg'] = 'Amounts injected'
        else:
            data['form']['msg'] = 'ERROR:\n\nAnalytic distribution not full 100 pourcent'
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('inject','Inject Amounts')]},
        },
        'inject': {
            'actions': [_inject],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }

inject_new_raw_amounts('inject_new_raw_amounts')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

