# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008 CCI Connect ASBL. (http://www.cciconnect.be) All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
from osv import fields, osv
import datetime
import tools
import string
import unicodedata
import re

def french_date(string):
    # convert date in format 'YYYY-MM-DD' to 'DD/MM/YYYY'
    if string and len(string) == 10:
        result = string[8:10]+'/'+string[5:7]+'/'+string[0:4]
    else:
        result = ''
    return result
    
class cci_cleaning_task(osv.osv):
    _name = 'cci_cleaning.task'
    _description = "Class to launch task of cleaning for CCIs"
    _columns = {
        'name':fields.char('Name',size=20),
    }
    def _ccilvn_daily(self,cr,uid):
        # we remove the default values to the field Date of res.partner.history if this default is 'NULL'
        # Start log
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_CCILVN_Daily')
        final_result = True
        # remove defaults values set to 'none' on date on 'res.partner.history'
        sql_sentence = """DELETE FROM ir_values
                            WHERE model = 'res.partner.history' 
                                and key = 'default' 
                                and name = 'date' 
                                and substr(value,1,1) != 'S'"""
        cr.execute(sql_sentence)
        # remove unuseful spaces on job function labels
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Removing unusefull spaces on jobs function_label :\n')
        sql_sentence = """SELECT id from res_partner_job WHERE function_label ilike ' %'"""
        cr.execute(sql_sentence)
        res = cr.fetchall()
        if res:
            job_ids = [x[0] for x in res]
            jobs = self.pool.get('res.partner.job').read(cr,uid,job_ids,['function_label'])
            for job in jobs:
                self.pool.get('res.partner.job').write(cr,uid,[job['id'],],{'function_label':job['function_label'].strip()})
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,str(len(job_ids)) + ' leading spaces\n')
        sql_sentence = """SELECT id from res_partner_job WHERE function_label ilike '% '"""
        cr.execute(sql_sentence)
        res = cr.fetchall()
        if res:
            job_ids = [x[0] for x in res]
            jobs = self.pool.get('res.partner.job').read(cr,uid,job_ids,['function_label'])
            for job in jobs:
                self.pool.get('res.partner.job').write(cr,uid,[job['id'],],{'function_label':job['function_label'].strip()})
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,str(len(job_ids)) + ' trailing spaces\n')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Deleted Default Empty Dates on res.partner.history\n')
        # remove defaults values set on 'cci_missions=>product.lines' because they interfere with the defintion of the VAT code associated with the product
        sql_sentence = """DELETE FROM ir_values
                            WHERE model = 'product.lines' 
                                and key = 'default'"""
        cr.execute(sql_sentence)
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Deleted Default Values on cci_missions=>product.lines\n')
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_special_periods_unreconciled(self,cr,uid):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_Unreconciled_SpecialPeriods')
        final_result = True
        period_ids = self.pool.get('account.period').search(cr,uid,[('special','=',True)])
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking %s special periods\n' % len(period_ids) )
        if period_ids:
            sentence = """SELECT period_id
                              FROM account_move_line
                              WHERE period_id in (%s) AND reconcile_id IS NULL
                              GROUP BY period_id
                       """ % ','.join([str(x) for x in period_ids])
            cr.execute(sentence)
            res = cr.fetchall()
            if res:
                final_result = False
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nSpecial Periods Unreconciled : %s [%s]' % (str(len(res)),','.join(str(x[0]) for x in res)) )
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _correct_budget_items_not_active(self,cr,uid):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Correct_Budget_items_not_active')
        final_result = True
        #
        sentence = """SELECT id
                          FROM cci_budget_dotation
                          WHERE active IS NULL
                   """
        cr.execute(sentence)
        res = cr.fetchall()
        if res:
            final_result = False
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nDotation(s) with active = NULL : %s' % ','.join(str(x[0]) for x in res) )
            if self.pool.get('cci_budget.dotation').write(cr,uid,[x[0] for x in res],{'active':True}):
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nCorrected\n' )
        #
        sentence = """SELECT id
                          FROM cci_budget_period_dotation
                          WHERE active IS NULL
                   """
        cr.execute(sentence)
        res = cr.fetchall()
        if res:
            final_result = False
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nDotation(s) By Period with active = NULL : %s' % ','.join(str(x[0]) for x in res) )
            if self.pool.get('cci_budget.period_dotation').write(cr,uid,[x[0] for x in res],{'active':True}):
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nCorrected\n' )
        #
        sentence = """SELECT id
                          FROM cci_budget_budget
                          WHERE active IS NULL
                   """
        cr.execute(sentence)
        res = cr.fetchall()
        if res:
            final_result = False
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nBudget(s) with active = NULL : %s' % ','.join(str(x[0]) for x in res) )
            if self.pool.get('cci_budget.budget').write(cr,uid,[x[0] for x in res],{'active':True}):
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nCorrected\n' )
        #
        sentence = """SELECT id
                          FROM cci_budget_department
                          WHERE active IS NULL
                   """
        cr.execute(sentence)
        res = cr.fetchall()
        if res:
            final_result = False
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nDepartment(s) with active = NULL : %s' % ','.join(str(x[0]) for x in res) )
            if self.pool.get('cci_budget.department').write(cr,uid,[x[0] for x in res],{'active':True}):
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nCorrected\n' )
        #
        sentence = """SELECT id
                          FROM cci_budget_post
                          WHERE active IS NULL
                   """
        cr.execute(sentence)
        res = cr.fetchall()
        if res:
            final_result = False
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nPost(s) with active = NULL : %s' % ','.join(str(x[0]) for x in res) )
            if self.pool.get('cci_budget.post').write(cr,uid,[x[0] for x in res],{'active':True}):
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nCorrected\n' )
        #
        sentence = """SELECT id
                          FROM cci_budget_budget_account_category
                          WHERE active IS NULL
                   """
        cr.execute(sentence)
        res = cr.fetchall()
        if res:
            final_result = False
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nCategory(ies) with active = NULL : %s' % ','.join(str(x[0]) for x in res) )
            if self.pool.get('cci_budget.budget_account_category').write(cr,uid,[x[0] for x in res],{'active':True}):
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nCorrected\n' )
        #
        sentence = """SELECT id
                          FROM cci_budget_budget_account_group
                          WHERE active IS NULL
                   """
        cr.execute(sentence)
        res = cr.fetchall()
        if res:
            final_result = False
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nGroup(s) with active = NULL : %s' % ','.join(str(x[0]) for x in res) )
            if self.pool.get('cci_budget.budget_account_group').write(cr,uid,[x[0] for x in res],{'active':True}):
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nCorrected\n' )
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_modified_membership_line(self,cr,uid,concerned_years=[2012,2013,2014,2015],ignore_ids=[]):
        # Concerned Years : for theses years, we check if all concerned invoice lines are linked to a membership line
        #                   by default, at minimum, pas year, current year and next year, but we can force others years
        current_year = int(datetime.datetime.today().strftime('%Y'))
        if current_year not in concerned_years:
            concerned_years.append(current_year)
        if (current_year-1) not in concerned_years:
            concerned_years.append(current_year-1)
        if (current_year+1) not in concerned_years:
            concerned_years.append(current_year+1)
        # ignore_ids : sometimes, it's desired to have a partner on membership line different from the partner_id on invoice. Such membership_line_id are given here
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_MembershipLine_Modified')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Tested Membership Years : [%s]\n\n' % ','.join(str(x) for x in concerned_years) )
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Membership lines\n')
        final_result = True
        errors = 0
        all_product_ids = self.pool.get('product.product').search(cr,uid,[('membership','=',True),'|',('active','=',True),('active','=',False)])
        products = self.pool.get('product.product').read(cr,uid,all_product_ids,['name','membership_year','membership_date_from','membership_date_to'])
        dProducts = {}
        for prod in products:
            dProducts[prod['id']] = prod
        #
        linked_invoice_line_ids = []
        mline_ids = self.pool.get('membership.membership_line').search(cr,uid,[])
        if mline_ids:
            mlines = self.pool.get('membership.membership_line').browse(cr,uid,mline_ids)
            for mline in mlines:
                linked_invoice_line_ids.append(mline.account_invoice_line.id)
                if mline.id not in ignore_ids:
                    if not mline['partner']:
                        errors += 1
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nMembership Line without Partner : [%s]' % (str(mline.id),) )
                    elif not mline['account_invoice_line']:
                        errors += 1
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nMembership Line without Attached Invoice Line : [%s]' % (str(mline.id),) )
                    else:
                        if mline.account_invoice_line.invoice_id.id != 1:
                            invoice_line = mline.account_invoice_line
                            invoice = invoice_line.invoice_id
                            if invoice.partner_id.id != mline.partner.id and invoice.partner_id.id != 2:
                                errors += 1
                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nMembership Line with other partner than invoice : [%s] %s (%s) - Invoice %s : %s (%s)' % 
                                                    (str(mline.id),mline.partner.name,str(mline.partner.id),invoice.number or '',invoice.partner_id.name,str(invoice.partner_id.id)) )
                            elif ( not invoice_line.product_id) or (invoice_line.product_id and invoice_line.product_id.id not in all_product_ids):
                                errors += 1
                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nMembership Line with product non membership on invoice %s : [%s] %s (%s) for "%s"' % 
                                                    (invoice.number or '',str(mline.id),invoice_line.product_id.name,str(invoice_line.product_id.id),invoice_line.name or '') )
                            else:
                                product = dProducts[invoice_line.product_id.id]
                                if (mline.date_from < product['membership_date_from'] or mline.date_to <> product['membership_date_to']) and product['membership_year'] in concerned_years:
                                    errors += 1
                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nMembership Line with diff date_from/to : [%s] from %s to %s - Product "%s" from %s to %s - Invoice %s' % 
                                                    (str(mline.id),mline.date_from,mline.date_to,product['name'],product['membership_date_from'],product['membership_date_to'],invoice.number or '') )
        # search for membership invoice lines without linked membership_line
        searched_prod_ids = []
        for (prod_id,prod) in dProducts.items():
            if prod['membership_year'] in concerned_years:
                searched_prod_ids.append(prod_id)
        iline_ids = self.pool.get('account.invoice.line').search(cr,uid,[('product_id','in',searched_prod_ids),('id','not in',linked_invoice_line_ids)])
        if iline_ids:
            ilines = self.pool.get('account.invoice.line').browse(cr,uid,iline_ids)
            incorrect_invoices = []
            for iline in ilines:
                if iline.invoice_id and iline.invoice_id.state in ['paid','open'] and iline.invoice_id.journal_id and iline.invoice_id.journal_id.code == 'VEN':
                    incorrect_invoices.append(iline.invoice_id.number)
            if incorrect_invoices:









                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nFactures de membership sans lignes membership associées : [%s]' % (str(incorrect_invoices),) )
            else:
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nToutes les factures de membership ont une ligne membership associée' )
        # search for recent membership invoices linked to 'refuse membership' partners
        #iline_ids = self.pool.get('account.invoice.line').search(cr,uid,[('product_id','in',searched_prod_ids)])
        #if iline_ids:
        #    ilines = self.pool.get('account.invoice.line').browse(cr,uid,iline_ids)
        #    for iline in ilines:
        #        if iline.invoice_id and iline.invoice_id.state in ['paid','open'] and iline.invoice_id.journal_id and iline.invoice_id.journal_id.code == 'VEN' and iline.invoice_id.id not in all_invoice_ids:
        #            all_invoice_ids.append(iline.invoice
        #            print iline.invoice_id.create_date
        #
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['PastDaysMembershipInvoiceTested'])
        if param_values.has_key('PastDaysMembershipInvoiceTested'):
            past_days = param_values['PastDaysMembershipInvoiceTested']
        else:
            past_days = 365
        today = datetime.datetime.today()
        first_date = today + datetime.timedelta(days=(past_days*-1))
        journal_ids = self.pool.get('account.journal').search(cr,uid,[('code','=','VEN')])
        sentence = """SELECT id, partner_id, number, create_date
                          FROM account_invoice
                          WHERE state in ('draft','paid','open') AND 
                                journal_id = %s AND 
                                create_date BETWEEN '%s' AND '%s' AND 
                                id in (SELECT invoice_id FROM account_invoice_line WHERE product_id in (%s) ) AND
                                partner_id in (SELECT id FROM res_partner WHERE active AND refuse_membership)
                          ORDER BY create_date DESC""" % (str(journal_ids[0]),
                                        first_date.strftime('%Y-%m-%d 00:00:01'),
                                        today.strftime('%Y-%m-%d 23:59:59'),
                                        ','.join(str(x) for x in searched_prod_ids))
        cr.execute(sentence)
        res = cr.fetchall()
        if res:
            errors += len(res)
            incorrect_partner_ids = []
            for incorrect_inv in res:
                if incorrect_inv[1] not in incorrect_partner_ids:
                    incorrect_partner_ids.append(incorrect_inv[1])
            partners = self.pool.get('res.partner').read(cr,uid,incorrect_partner_ids,['id','name'])
            dPartners = {}
            for partner in partners:
                dPartners[partner['id']] = partner['name']
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\n\nFactures de membership liées à des partenaires qui refusent le membership :')
            for incorrect_inv in res:
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\n%s - %s - Partenaire %s [%i]' % (incorrect_inv[3][0:10],
                                                                                                                        incorrect_inv[2],
                                                                                                                        dPartners[incorrect_inv[1]],
                                                                                                                        incorrect_inv[1] ) )
        # detect parters with indication of 'forced_membership' but also 'refuse_membership' ...
        partner_obj = self.pool.get('res.partner')
        partner_ids = partner_obj.search(cr,uid,[('next_membership_bill_forced','=',True),('refuse_membership','=',True)])
        if partner_ids:
            errors += len(partner_ids)
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\n\nPartenaires indiqués comme \'Prochaine facture forcée\' mais aussi \'Refus membership\' ... : ' + ','.join([str(x) for x in partner_ids]))
        if errors > 0:
            final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_partners_multiple_default_addresses(self,cr,uid):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_Multiple_Defaults')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Partners with Multiple Default Addresses\n')
        final_result = True
        sentence = """SELECT partner_id as id, count(partner_id) as count_defaults 
                          FROM res_partner_address
                          WHERE type = 'default' AND active
                          GROUP by partner_id 
                          HAVING count(partner_id) > 1"""
        cr.execute(sentence)
        res = cr.fetchall()
        if res:
            errors = [str(x[0]) for x in res]
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Incorrect Partners IDs : [%s]\n' % ','.join(errors) )
            final_result = False
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Partners with address but no Default Address\n')
        sentence = """SELECT id FROM res_partner
                          WHERE id IN (SELECT DISTINCT(partner_id) as id FROM res_partner_address WHERE type <> 'default' AND active)
                                AND state_id = 1 AND active
                      EXCEPT
                      SELECT id FROM res_partner
                          WHERE id IN (SELECT DISTINCT(partner_id) as id FROM res_partner_address WHERE type = 'default' AND active)
        """
        cr.execute(sentence)
        res = cr.fetchall()
        if res:
            errors = [str(x[0]) for x in res]
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Incorrect Partners IDs : [%s]\n' % ','.join(errors) )
            final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _check_open_invoices_reconciled(self,cr,uid):
        # detect the invoices with the state 'open' but reconciled
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_Reconciled_Open_Invoices')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking for Invoices still Open, but reconciled...\n')
        INV_TYPES = ['out_invoice','out_refund']
        INV_ACCOUNT = 145 # = '400000'
        #search all the objects
        inv_ids = self.pool.get('account.invoice').search(cr,uid,[('state','=','open'),('type','in',INV_TYPES)])
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Invoices to control : %s' % str(len(inv_ids)) )
        errors = []
        final_result = True
        if inv_ids:
            invoices = self.pool.get('account.invoice').browse(cr,uid,inv_ids)
            for invoice in invoices:
                if invoice.reconciled or invoice.payment_ids:
                    # Check if this is a partial reconcile or a full reconcile
                    if invoice.move_id:
                        for mline in invoice.move_id.line_id:
                            if mline.account_id and mline.account_id.id == INV_ACCOUNT:
                                if mline.reconcile_id:
                                    errors.append({'invoice_id':invoice.id,'number':invoice.number,'partner':invoice.partner_id.name})
        if errors:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nIncorrect Invoices :\n' )
            for error in errors:
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nInvoice %s [%s] for partner %s' % (error['number'], str(error['invoice_id']), error['partner']) )
            final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _check_invoices_numbers(self,cr,uid,exceptions=[],additional_names=[]):
        # exception = array of tuples (type,year,number) in 'F/N/FA/NA' and integers
        def _extract_struct_number(inputstring):
            for add_name in additional_names:
                if add_name in inputstring:
                    inputstring = inputstring.replace(add_name,'')
            error = 0
            struct = inputstring.split('/')
            if len(struct) == 2:
                if struct[0][0:2] in ['FA','NA']:
                    inv_type = struct[0][0:2]
                elif struct[0][0:1] in ['F','N']:
                    inv_type = struct[0][0:1]
                else:
                    inv_type = ''
                    error = 2
                if error == 0:
                    try:
                        inv_year = int(struct[0][len(inv_type):])
                    except ValueError:
                        error = 3
                        inv_year = 0
                    try:
                        inv_number = int(struct[1])
                    except ValueError:
                        error = 4
                        inv_number = 0
                else:
                    inv_year = 0
                    inv_number = 0
            else:
                error = 1
                inv_type = ''
                inv_year = 0
                inv_number = 0
            return (error,inv_type,inv_year,inv_number)
        # detect the holes in numbers of invoices and credit notas
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_Holes_in_Invoices')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nBe careful : maybe, a hole in numbers say the number is used but on a \'canceled\' or \'proforma\' invoice wich is also an error but the correction differs...')
        jnl_ids = self.pool.get('account.journal').search(cr,uid,[('type','in',['sale','purchase'])])
        jnls = self.pool.get('account.journal').read(cr,uid,jnl_ids,['id','code'])
        errors = 0
        final_result = True
        for jnl in jnls:
            # detected canceled invoices with numbers beyond 2013
            inv_ids = self.pool.get('account.invoice').search(cr,uid,[('journal_id','=',jnl['id']),('state','=','cancel'),('date_invoice','>','2013-12-31')])
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nChecking for Numbered Canceled Invoices in Journal\n')
            invs = self.pool.get('account.invoice').read(cr,uid,inv_ids,['number','state','date_invoice'])
            for invoice in invs:
                if invoice['number']:
                    errors += 1
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nFacture avec numéro mais annulée, au delà de 2013 [%s] (%i)' % (invoice['number'],invoice['id']))
            #
            inv_ids = self.pool.get('account.invoice').search(cr,uid,[('journal_id','=',jnl['id']),('state','not in',['draft','proforma','proforma2']),('number','>=','F'),('date_invoice','>','2013-12-31')])
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nChecking for Numbers in Journal %s : %i lines to test\n' % (jnl['code'],len(inv_ids)))
            invs = self.pool.get('account.invoice').read(cr,uid,inv_ids,['number','state','date_invoice'])
            inv_numbers = sorted([(x['number'],x['id'],x['date_invoice'],x['state']) for x in invs], key=lambda x: x[0])
            last_one = inv_numbers[0][0]
            (error,last_one_type,last_one_year,last_one_number) = _extract_struct_number(last_one or '')
            if error:
                errors += 1
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nError %i with invoice [%s] (%i)' % (error,last_one,inv_numbers[0][1]))
            for inv in inv_numbers[1:]:
                (error,current_type,current_year,current_number) = _extract_struct_number(inv[0] or '')
                if error:
                    errors += 1
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nError %i with invoice [%s] (%i)' % (error,inv[0],inv[1]))
                else:
                    if current_type != last_one_type:
                        errors += 1
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nRupture sur les types %s -> %s avec la facture [%s] (%i)' % (last_one_type,current_type,inv[0],inv[1]))
                    else:
                        if current_year != last_one_year:
                            if (current_year != last_one_year + 1 ) or ((current_number % 10000) != 1):
                                errors += 1
                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nPassage d\'année sans recommencer à zéro avec les factures [%s->%s] (%i)' % (last_one,inv[0],inv[1]))
                        else:
                            if current_number != (last_one_number + 1) and ((last_one_type,last_one_year,last_one_number+1) not in exceptions):
                                errors += 1
                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nRupture dans la numérotation avec les factures [%s->%s] (%i)' % (last_one,inv[0],inv[1]))
                last_one = inv[0]
                last_one_type = current_type
                last_one_year = current_year
                last_one_number = current_number
        if errors:
            final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_tax_problem(self,cr,uid,checked_ids=[],correct_line_ids=[],tax_code='V50'):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_Tax_Problems_On_Refund')
        final_result = True


        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Purchase Refunds with taxes at 50% for cars\n')
        # Detect the purchase refunds with tax 50% on cars
        atax_obj = self.pool.get('account.tax')
        atax_ids = atax_obj.search(cr,uid,[('description','ilike',tax_code)])
        if atax_ids:
            sentence = """SELECT ai.id FROM account_invoice as ai, account_invoice_line as ail, account_invoice_line_tax as ailt
                            WHERE ai.journal_id = 14 AND ailt.tax_id in (%s) AND ai.id = ail.invoice_id AND ail.id = ailt.invoice_line_id
                       """ % ','.join([str(x) for x in atax_ids])
            cr.execute(sentence)
            res = cr.fetchall()
            if res:
                epured_res = []
                for record in res:
                    if record[0] not in checked_ids:
                        epured_res.append(str(record[0]))
                if epured_res:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'Errors - NCA avec des TVA à 50pourcents : [%s]\n' % ','.join(epured_res) )
                    final_result = False
        # Detect the purchase refunds with wrong amount of tax
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nDetection of account move line without tax_amount but tax_id !\n')
        mline_ids = self.pool.get('account.move.line').search(cr,uid,[('journal_id','=',14),('tax_code_id','<>',False),('tax_amount','<',0.009),('tax_amount','>',-0.009),('id','not in',correct_line_ids)])
        if mline_ids:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Errors - Lines with amount of tax = 0 while tax to apply : [%s]\n' % ','.join([str(x) for x in mline_ids]) )
            final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_old_draft_moves(self,cr,uid,correct_move_ids=[]):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_Old_Draft_Moves')
        final_result = True
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Draft Moves Older than current month : ')
        today = datetime.date.today()
        if int(today.strftime('%d')) <= 8:
            today = today-datetime.timedelta(days=8)
        current_month = today.strftime('%Y-%m') + '-01'
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,current_month)
        move_ids = self.pool.get('account.move').search(cr,uid,[('state','=','draft'),('id','not in',correct_move_ids),('date','<',current_month)])
        if move_ids:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nErrors - Moves still in draft : [%s]\n' % ','.join([str(x) for x in move_ids]) )
            final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_partners_wo_salesman(self,cr,uid):
        # detect the partners without attached salesman. If any address, it's an error. If no address, signal it wituout raising an error
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_Partners_wo_Salesman')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Partners without salesman, with ou without address\n')
        final_result = True
        errors = []
        warnings = []
        partner_ids = self.pool.get('res.partner').search(cr,uid,[('user_id','=',False),('active','=',True),('state_id','=',1)])
        if partner_ids:
            partners = self.pool.get('res.partner').browse(cr,uid,partner_ids)
            for partner in partners:
                if partner.address:
                    errors.append(str(partner.id))
                else:
                    warnings.append(str(partner.id))
        if warnings:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Partners IDs without salesman and without address : [%s]\n' % ','.join(warnings) )
        if errors:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Partners IDs without salesman but WITH address : [%s]\n' % ','.join(errors) )
            final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _wrong_year(self,cr,uid,years=[],all_dates_important=False):
        # parameters management
        # 1. if years is empty, create it with past year, current year and next year
        # 2. if all_dates_important is false, but the current_date is the 1 day of the month, make it important
        if not years:
            current_year = int(datetime.date.today().strftime('%Y'))
            years = [current_year-1,current_year,current_year+1]
        if not all_dates_important:
            current_day = int(datetime.date.today().strftime('%d'))
            if current_day == 1:
                all_dates_important = True
        
        # detect the move line with date in another year of the period
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Wrong_Year')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nChecking move lines with different year than the year of the period')
        final_result = True
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nChecked Years : %s' % years )
        all_fiscalyear_ids = []
        for check_year in years:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nCurrent year : %s\n' % str(check_year) )
            fiscal_code = 'FY' + str(check_year)
            fiscal_year_id = self.pool.get('account.fiscalyear').search(cr,uid,[('code','=',fiscal_code)])
            if fiscal_year_id:
                all_fiscalyear_ids.append(fiscal_year_id[0])
                period_ids = self.pool.get('account.period').search(cr,uid,[('fiscalyear_id','=',fiscal_year_id[0]),('special','=',False)])
                if period_ids:
                    sentence = """SELECT id, date, move_id FROM account_move_line WHERE date_part('year',date) <> %s and period_id in (%s)""" % (str(check_year),','.join([str(x) for x in period_ids]) )
                    cr.execute(sentence)
                    res = cr.fetchall()
                    for line in res:
                        final_result = False
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nLine ID %s - Move ID %s - Date : %s' % (str(line[0]),str(line[2]),line[1]) )
                else:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nNo period in this year' )
            else:
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nNo fiscal year detected with the code \'%s\'' % fiscal_code )
        if all_fiscalyear_ids:
            # detect all move lines with date outside of period
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\n\nLESS IMPORTANT OR URGENT :\nChecking move lines with date outside period')
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nChecked Fiscal Years IDs : %s' % ','.join([str(x) for x in all_fiscalyear_ids]) )
            sentence = """select aml.id, aml.move_id, aml.date, ap.date_start, ap.date_stop
                            FROM account_move_line as aml, account_period as ap
                            WHERE aml.period_id = ap.id and ap.fiscalyear_id in (%s) and ap.special = false and (aml.date < ap.date_start or aml.date > ap.date_stop);""" % ','.join([str(x) for x in all_fiscalyear_ids])
            cr.execute(sentence)
            res = cr.fetchall()
            for line in res:
                if all_dates_important:
                    final_result = False
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nLine ID : %s - Move ID : %s - Date : %s - Period : %s to %s' % (str(line[0]),str(line[1]),line[2],line[3],line[4]) )
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _test_connect_mysql(self,cr,uid):
        # try to connect to a MySQL DB
        import pymysql
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'Test Connect MySQL')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Test if the connection to MySQL DB is possible\n')
        final_result = True
        try:
            con = pymysql.connect(host='149.5.47.63', port=3306, user='intotw', passwd='1784&Mary@', db='laravel');
            cur = con.cursor()
            cur.execute("SELECT VERSION()")
            ver = cur.fetchone()
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Database version : %s' % ver)

        except pymysql.err.Error, e:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Connection Error %d: %s' % (e.args[0],e.args[1]))
            final_result = False
        finally:
            if cur:
                cur.close()
            if con:
                con.close()
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_wrong_dom_mandates(self,cr,uid):
        # detect the partners indicated as domiciled but with no valid mandate or the contrary
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_Wrong_Domiciled_Partners')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Partners domiciled comparing to valid mandate\n')
        final_result = True
        valid_mandate_ids = self.pool.get('cci.sdd.mandate').search(cr,uid,[('state','=','valid')])
        valid_mandates = self.pool.get('cci.sdd.mandate').read(cr,uid,valid_mandate_ids,['partner_id'])
        valid_mandates_partner_ids = [x['partner_id'][0] for x in valid_mandates]
        dom_partner_ids = self.pool.get('res.partner').search(cr,uid,[('domiciliation','<>',False)])
        if dom_partner_ids or valid_mandates_partner_ids:
            errors_dom = [str(x) for x in dom_partner_ids if x not in valid_mandates_partner_ids]
            errors_mand = [str(x) for x in valid_mandates_partner_ids if x not in dom_partner_ids]
            if errors_dom or errors_mand:
                if errors_dom:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Partners domiciled without valid mandates : [%s]\n' % ','.join(errors_dom) )
                if errors_mand:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Partners with mandates but not marked as domiciled : [%s]\n' % ','.join(errors_mand) )
                final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_event_names(self,cr,uid):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Event_Names')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Detect Event with different names by languages\n')
        final_result = True
        errors = []
        event_obj = self.pool.get('event.event')
        event_ids = event_obj.search(cr,uid,[])
        if event_ids:
            events_EN = event_obj.read(cr,uid,event_ids, ['id','name'],{'lang':'en_EN'})
            dEventsEN = {}
            for event in events_EN:
                dEventsEN[event['id']] = event['name']
            #
            events_FR = event_obj.read(cr,uid,event_ids, ['id','name'],{'lang':'fr_FR'})
            dEventsFR = {}
            for event in events_FR:
                dEventsFR[event['id']] = event['name']
            #
            events_NL = event_obj.read(cr,uid,event_ids, ['id','name'],{'lang':'nl_NL'})
            dEventsNL = {}
            for event in events_NL:
                dEventsNL[event['id']] = event['name']
            #
            events_DE = event_obj.read(cr,uid,event_ids, ['id','name'],{'lang':'de_DE'})
            dEventsDE = {}
            for event in events_DE:
                dEventsDE[event['id']] = event['name']
            #
            for event_id in event_ids:
                if dEventsEN.has_key(event_id) and dEventsFR.has_key(event_id) and dEventsNL.has_key(event_id) and dEventsDE.has_key(event_id):
                    if ( dEventsEN[event_id] != dEventsFR[event_id] ) or ( dEventsEN[event_id] != dEventsNL[event_id] ) or ( dEventsEN[event_id] != dEventsDE[event_id] ) or \
                       ( dEventsFR[event_id] != dEventsNL[event_id] ) or ( dEventsFR[event_id] != dEventsDE[event_id] ) or ( dEventsNL[event_id] != dEventsDE[event_id] ):
                        errors.append(str(event_id))
        if errors:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Incorrect Event IDs : [%s]\n' % ','.join(errors) )
            final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_product_names(self,cr,uid,ignored=[688]):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Product_Names')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Detect Products with different names by languages\n')
        final_result = True
        errors = []
        prod_obj = self.pool.get('product.product')
        prod_ids = prod_obj.search(cr,uid,[('id','not in',ignored)])
        if prod_ids:
            prods_EN = prod_obj.read(cr,uid,prod_ids, ['id','name'],{'lang':'en_EN'})
            dProdsEN = {}
            for product in prods_EN:
                dProdsEN[product['id']] = product['name']
            #
            prods_FR = prod_obj.read(cr,uid,prod_ids, ['id','name'],{'lang':'fr_FR'})
            dProdsFR = {}
            for product in prods_FR:
                dProdsFR[product['id']] = product['name']
            #
            prods_NL = prod_obj.read(cr,uid,prod_ids, ['id','name'],{'lang':'nl_NL'})
            dProdsNL = {}
            for product in prods_NL:
                dProdsNL[product['id']] = product['name']

            #
            prods_DE = prod_obj.read(cr,uid,prod_ids, ['id','name'],{'lang':'de_DE'})
            dProdsDE = {}
            for product in prods_DE:

                dProdsDE[product['id']] = product['name']
            #
            for prod_id in prod_ids:
                if dProdsEN.has_key(prod_id) and dProdsFR.has_key(prod_id) and dProdsNL.has_key(prod_id) and dProdsDE.has_key(prod_id):
                    if ( dProdsEN[prod_id] != dProdsFR[prod_id] ) or ( dProdsEN[prod_id] != dProdsNL[prod_id] ) or ( dProdsEN[prod_id] != dProdsDE[prod_id] ) or \
                       ( dProdsFR[prod_id] != dProdsNL[prod_id] ) or ( dProdsFR[prod_id] != dProdsDE[prod_id] ) or ( dProdsNL[prod_id] != dProdsDE[prod_id] ):
                        errors.append(str(prod_id))
        if errors:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Incorrect Products IDs : [%s]\n' % ','.join(errors) )
            final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _check_account_account(self,cr,uid):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Check_Accounts')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Accounts\nCheck if every account of class 6 or 7 are linked to a budget_group.\n')

        final_result = True
        errors = 0
        # check if all account of class 6 or 7 have an cci_budget_group_id
        acc_obj = self.pool.get('account.account')
        acc_ids = acc_obj.search(cr,uid,['|',('code','ilike','6%'),('code','ilike','7%')])
        accounts = acc_obj.browse(cr,uid,acc_ids)
        for acc in accounts:
            if acc.user_type and acc.user_type.code <> 'view':
                if not acc.cci_budget_group_id:
                    errors += 1
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nCompte \'%s\' non lié à un groupe budgétaire' % (acc.code or 'no code',) )
                else:
                    if not acc.cci_budget_group_id.active:
                        errors += 1
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nCompte \'%s\' avec groupe lié inactif : %s' % (acc.code or 'no code',acc.cci_budget_group_id.name or 'no name') )
        if errors > 0:
            final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _check_analytic(self,cr,uid,root_ids=[171,2],date_from='2014-01-01',has_budget=True):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Check_Analytic')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Analytics\n')
        final_result = True
        errors = 0
        # check if all analytic distribution are for 100% and on two or three analytic plans
        dist_obj = self.pool.get('account.analytic.plan.instance')
        dist_ids = dist_obj.search(cr,uid,[])
        distributions = dist_obj.browse(cr,uid,dist_ids)
        for distri in distributions:
            error_in_distri = False
            if not distri.account_ids:
                error_in_distri = True
                errors += 1
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nDistribution sans comptes liés : [ID distri : %s] Nom : %s' % (str(distri.id),distri.name) )
            else:
                dSumRoots = {}
                for root_id in root_ids:
                    dSumRoots[root_id] = 0.0
                for dist_line in distri.account_ids:
                    aaccount = dist_line.analytic_account_id
                    if not aaccount:
                        error_in_distri = True
                        errors += 1
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nLigne de distribution sans compte lié : [ID distri : %s] Nom : %s' % (str(distri.id),distri.name) )
                    else:
                        if not aaccount.active:
                            error_in_distri = True
                            errors += 1
                            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nLigne de distribution avec compte lié inactif : [ID distri : %s] Nom : %s' % (str(distri.id),distri.name) )
                        else:
                            root_account = False
                            if aaccount.parent_id:
                                root_account = aaccount.parent_id
                            while root_account.parent_id:
                                root_account = root_account.parent_id
                            if not root_account:
                                error_in_distri = True
                                errors += 1
                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nLigne avec compte lié sans root : [%s] %s' % (str(distri.id),distri.name) )
                            else:
                                if (root_account.id not in root_ids) or (not dSumRoots.has_key(root_account.id)):
                                    error_in_distri = True
                                    errors += 1
                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nLigne avec compte lié à une mauvaise root : [%s] %s' % (str(distri.id),distri.name) )
                                else:
                                    dSumRoots[root_account.id] += dist_line.rate
            if not error_in_distri:
                error_in_sum = False
                for root_id in root_ids:
                    if dSumRoots[root_id] < 99.999 or dSumRoots[root_id] > 100.001:
                        error_in_sum = True
                        break
                if error_in_sum:
                    errors += 1
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nLigne pas à 100 pourcents sur chaque plan : [%s] %s %s' % (str(distri.id),distri.name,str(dSumRoots)) )

        # check if all move lines on specific accounts are linked to an analytic distribution
        account_ids = self.pool.get('account.account').search(cr,uid,[('cci_forced_analytic','=',True),'|',('active','=',True),('active','=',False)])
        if account_ids:
            mline_ids = self.pool.get('account.move.line').search(cr,uid,[('account_id','in',account_ids),('analytics_id','=',False),('date','>=',date_from)])
            if mline_ids:
                errors += len(mline_ids)
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\n\nLigne sur comptes forcés mais sans lignes analytiques : [%s]' % (','.join([str(x) for x in mline_ids]),) )
            else:
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\n\nAucune ligne sur comptés forcés sans distribution => test OK' )
        else:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\n\nAucun compte forcé d\'être analytique => reste du test ignoré' )

        # check if all move lines on specific accounts are linked to 100% of analytic
        # and linked to the same account.account->account_id
        if account_ids:
            mline2_ids = self.pool.get('account.move.line').search(cr,uid,[('account_id','in',account_ids),('date','>=',date_from),('state','=','valid')])
            if mline2_ids:
                start = 0
                sslice = 200
                while mline2_ids[start:start+sslice]:
                    mlines = self.pool.get('account.move.line').browse(cr,uid,mline2_ids[start:start+sslice])
                    for mline in mlines:
                        if mline.analytic_lines:
                            dSumRoots = {}
                            for root_id in root_ids:
                                dSumRoots[root_id] = 0.0
                            for aline in mline.analytic_lines:
                                if (not aline.general_account_id):
                                    errors += 1
                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nLigne analytique liée sans compte comptable lié : [id ligne comptable %s] %s' % (str(mline.id),aline.name or '') )
                                elif aline.general_account_id.id != mline.account_id.id:
                                    errors += 1
                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nLigne analytique liée avec un autre compte comptable : [id ligne comptable %s] %s' % (str(mline.id),aline.name or '') )
                                aaccount = aline.account_id
                                if not aaccount:
                                    errors += 1
                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nLigne analytique liée sans compte analytique lié : [id ligne comptable %s] %s' % (str(mline.id),aline.name or '') )
                                else:
                                    root_account = False
                                    if aaccount.parent_id:
                                        root_account = aaccount.parent_id
                                    while root_account.parent_id:
                                        root_account = root_account.parent_id
                                    if not root_account:
                                        errors += 1
                                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nLigne avec compte lié hors plan analytique : [id ligne comptable %s] %s' % (str(mline.id),aline.name or '') )
                                    else:
                                        if (root_account.id not in root_ids) or (not dSumRoots.has_key(root_account.id)):
                                            errors += 1
                                            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nLigne avec compte lié à un mauvais plan analytique : [%s] %s' % (str(mline.id),aline.name or '') )
                                        else:
                                            dSumRoots[root_account.id] += aline.amount
                            error_in_sum = False
                            for root_id in root_ids:
                                ref_amount = ( mline.credit - mline.debit )
                                if abs(dSumRoots[root_id]) < (abs(ref_amount) - 0.001) or abs(dSumRoots[root_id]) > (abs(ref_amount) + 0.001 ):
                                    error_in_sum = True
                                    break
                            if error_in_sum:
                                errors += 1
                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nLigne pas à 100 pourcents sur chaque plan : [id ligne comptable %s] %s %s != %s' % 
                                                                                    (str(mline.id),aline.name,str(ref_amount),str(dSumRoots)) )
                        else:
                            if mline.id not in mline_ids: # this error has already been detected so not counted once again
                                errors += 1
                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nLigne de mouvement sans lignes analytiques liées : %s' % str(mline.id) )
                    start += sslice
        # check if all active analytic accounts are linked to budget posts and departments
        aaccount_ids = self.pool.get('account.analytic.account').search(cr,uid,['|',('active','=',True),('active','=',False)])
        today = datetime.datetime.today().strftime('%Y-%m-%d')
        if has_budget:
            ended_aaccounts = []
            aaccounts = self.pool.get('account.analytic.account').browse(cr,uid,aaccount_ids)
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nComptes contrôlés : %s' % str(len(aaccount_ids)) )
            for aaccount in aaccounts:
                currently_active = (aaccount.cci_budget and aaccount.active)
                if aaccount.date_start and aaccount.date_start > today:
                    currently_active = False
                if aaccount.date and aaccount.date < today[0:4]+'01-01':
                    currently_active = False
                if currently_active:
                    if ( not aaccount.cci_post_id ) or (not aaccount.cci_department_id ):
                        errors += 1
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nCompte analytique actif mais non lié à un projet ou département budgétaire [%i] : %s' % (aaccount.id,aaccount.code and (aaccount.code+' ') or '' )+aaccount.name )
                if aaccount.date:
                    ended_aaccounts.append((aaccount.id,aaccount.date,(aaccount.code and (aaccount.code+' ') or '' )+aaccount.name))
        else:
            ended_aaccounts = []
            aaccounts = self.pool.get('account.analytic.account').browse(cr,uid,aaccount_ids)
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nComptes contrôlés : %s' % str(len(aaccount_ids)) )
            for aaccount in aaccounts:
                if aaccount.date:
                    ended_aaccounts.append((aaccount.id,aaccount.date,(aaccount.code and (aaccount.code+' ') or '' )+aaccount.name))
        # check if all inactive analytic accounts are not used beyond their end date
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\n\nComptes clôturés contrôlés : %s' % str(len(ended_aaccounts)) )
        ended_errors = 0

        for (aa_id, aa_date, aa_name) in ended_aaccounts:
            aaline_ids = self.pool.get('account.analytic.line').search(cr,uid,[('account_id','=',aa_id),('date','>',aa_date)])
            if aaline_ids:
                linked_move_line_ids = []
                aalines = self.pool.get('account.analytic.line').read(cr,uid,aaline_ids,['move_id'])
                for aaline in aalines:
                    if aaline['move_id']:
                        linked_move_line_ids.append(aaline['move_id'][0])
                ended_errors += len(aaline_ids)
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\n(%i) lignes analytiques après la date de clôture \'%s\' de son compte analytique  [%i] %s\n  -> [%s]' % (len(aaline_ids),aa_date,aa_id,aa_name,','.join([str(x) for x in linked_move_line_ids])) )
        # NEW 2018-11-06
        # check if all inactive analytic distributions are not used outside their begin and end date
        distri_ids = self.pool.get('account.analytic.plan.instance').search(cr,uid,[])
        distris = self.pool.get('account.analytic.plan.instance').browse(cr,uid,distri_ids)
        dates_distri_ids = []
        for distri in distris:
            if distri.date_start or distri.date_stop:
                dates_distri_ids.append(distri.id)
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\n\nDates d\'utilisation des distributions analytiques contrôlées : %s' % str(len(dates_distri_ids)) )
        ddates_errors = 0
        dates_distris = self.pool.get('account.analytic.plan.instance').browse(cr,uid,dates_distri_ids)
        for distri in dates_distris:
            if distri.date_start and distri.date_stop:
                aline_ids = self.pool.get('account.move.line').search(cr,uid,[('date','>=',date_from),('analytics_id','=',distri.id),'|',('date','<',distri.date_start),('date','>',distri.date_stop)])
            elif distri.date_start and not distri.date_stop:
                aline_ids = self.pool.get('account.move.line').search(cr,uid,[('date','>=',date_from),('analytics_id','=',distri.id),('date','<',distri.date_start)])
            elif distri.date_stop and not distri.date_start:
                aline_ids = self.pool.get('account.move.line').search(cr,uid,[('date','>=',date_from),('analytics_id','=',distri.id),('date','>',distri.date_stop)])
            else:
                aline_ids = []
            if aline_ids:
                if len(aline_ids) < 20:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\n(%i) lignes comptables hors des dates de début-fin de la distribution analytique [%i] \'%s\'\n  -> [%s]' % (len(aline_ids),distri.id,distri.name,','.join([str(x) for x in aline_ids])) )
                else:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\n(%i) lignes comptables hors des dates de début-fin de la distribution analytique [%i] \'%s\'\n  -> [%s ...]' % (len(aline_ids),distri.id,distri.name,','.join([str(x) for x in aline_ids[0:20]])) )
                ddates_errors += len(aline_ids)
        if ddates_errors > 0:
            errors += ddates_errors
        else:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nOK - Aucun lignes comptables avec une date hors dates de distributions analytiques')
        if errors > 0:
            final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_wrong_characters(self,cr,uid,verified_fields=[('res.partner','activity_description',{'lang':'fr_FR'}),
                                                              ('res.partner.address','activity_description',{'lang':'fr_FR'}),
                                                              ('res.partner.job','function_label','')],
                                             good_characters=string.printable+u'àâäéèêëîïôöùûüç°²³€ÁÀÄÂÉÈÊËÏÎÔÖÛÜ®©'):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Wrong_Characters')
        final_wrongs = 0
        for (table_name,field_name,field_context) in verified_fields:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nCorrecting %s::%s \n' % (table_name,field_name) )
            ids_to_verify = self.pool.get(table_name).search(cr,uid,[(field_name,'ilike','e')])
            if field_context:
                obj_to_verify = self.pool.get(table_name).read(cr,uid,ids_to_verify,[field_name],field_context)
            else:
                obj_to_verify = self.pool.get(table_name).read(cr,uid,ids_to_verify,[field_name])
            if obj_to_verify:
                wrongs = 0
                corrections = 0
                for record in obj_to_verify:
                    obj_id = record['id']
                    obj_field = record[field_name]
                    wrong = False
                    corrected = False
                    new_value = ''
                    for carac in obj_field:
                        if carac not in good_characters:
                            if carac == u'\u201c':
                                new_carac = '"'
                                corrected = True
                            elif carac == u'\u201d':
                                new_carac = '"'
                                corrected = True
                            elif carac == u'\u00ab':
                                new_carac = '"'
                                corrected = True
                            elif carac == u'\u00bb':
                                new_carac = '"'
                                corrected = True
                            elif carac == u'\u00b4':
                                new_carac = "'"
                                corrected = True
                            elif carac == u'\u2018':
                                new_carac = "'"
                                corrected = True
                            elif carac == u'\u2019':
                                new_carac = "'"
                                corrected = True
                            elif carac == u'\u2013':
                                new_carac = "-"
                                corrected = True
                            elif carac == u'\u0153':
                                new_carac = "oe"
                                corrected = True
                            elif carac == u'\u0153':
                                new_carac = "oe"
                                corrected = True
                            elif carac == u'\u0152':
                                new_carac = "OE"
                                corrected = True
                            elif carac == u'\u0178':
                                new_carac = "Y"
                                corrected = True
                            elif carac == u'\u202a':
                                new_carac = ""
                                corrected = True
                            elif carac == u'\u00ec':
                                new_carac = ""
                                corrected = True
                            elif carac == u'\u200b':
                                new_carac = ""
                                corrected = True
                            elif carac == u'\uf0fc':
                                new_carac = ""
                                corrected = True
                            elif carac == u'\uf0d8':
                                new_carac = ""
                                corrected = True
                            elif carac == u'\u00df':
                                new_carac = "ss"
                                corrected = True
                            elif carac == u'\u2026':
                                new_carac = "..."
                                corrected = True

                            else:
                                unicode_name = ''
                                try:
                                    unicode_name = unicodedata.name(carac)
                                except:
                                    unicode_name = 'Undetermined'
                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n' + str(obj_id).rjust(6,'0') + ' ' + '%04x' % ord(carac) + ' ' + unicode_name)
                                wrong = True
                            if corrected:
                                new_value = new_value + new_carac
                        else:
                            new_value = new_value + carac
                    if wrong:
                        wrongs += 1
                    else:
                        if corrected:
                            if field_context:
                                self.pool.get(table_name).write(cr,uid,[obj_id],{field_name:new_value},field_context)
                            else:
                                self.pool.get(table_name).write(cr,uid,[obj_id],{field_name:new_value})
                            corrections += 1
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nStill wrongs fields : %s' % str(wrongs) )
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nAuto corrected fields : %s\n\n' % str(corrections) )
                final_wrongs += wrongs
            else:
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nNothing to check\n\n')
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_wrongs==0,'\n---end---')
        return True
    def _detect_wrong_email(self,cr,uid):
        # we detect a serie of common error on email addresses
        # some errors can be automaticaly corrected; some are just signaled to admin to correct later manually
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_wrong_email')
        final_result = True
        #
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nAutomatic corrections')
        cr.execute("""UPDATE res_partner_job SET email = btrim(email) WHERE email ilike '% %'""")
        # this will not correct all; just first and last spaces, not if inside
        obj_addr = self.pool.get('res.partner.address')
        obj_job = self.pool.get('res.partner.job')
        obj_contact = self.pool.get('res.partner.contact')
        obj_subscriber = self.pool.get('cci_newsletter.subscriber')
        #
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nAutomatic corrections made one by one')
        corrected_emails = []
        errors_by_users = {}

        #
        cr.execute("""select id, email, write_uid, create_uid from res_partner_address where email ilike '% %' OR email ilike '%\n%' OR email ilike '%\r%' OR email ilike '%\t%' or email ilike '%\\'%'""")
        res = cr.fetchall()
        for line in res:
            current_id = line[0]
            old_email = line[1]
            new_email = old_email.replace('\n','').replace('\r','').replace('\t','').replace(' ','').replace("'","")
            obj_addr.write(cr,uid,[current_id],{'email':new_email})
            corrected_emails.append(new_email)
            resp_id = line[2] or line[3]
            if resp_id:
                errors = errors_by_users.get(resp_id,[])
                if old_email not in errors:
                    errors.append(old_email)
                    errors_by_users[resp_id] = errors
        #
        cr.execute("""select id, email, write_uid, create_uid from res_partner_job where email ilike '% %' OR email ilike '%\r%' OR email ilike '%\n%' OR email ilike '%\t%' or email ilike '%\\'%'""")
        res = cr.fetchall()
        for line in res:
            current_id = line[0]
            old_email = line[1]
            new_email = old_email.replace('\n','').replace('\t','').replace('\t','').replace(' ','').replace("'","")
            obj_job.write(cr,uid,[current_id],{'email':new_email})
            corrected_emails.append(new_email)
            resp_id = line[2] or line[3]
            if resp_id:
                errors = errors_by_users.get(resp_id,[])
                if old_email not in errors:
                    errors.append(old_email)
                    errors_by_users[resp_id] = errors
        #
        cr.execute("""select id, email, write_uid, create_uid from res_partner_contact where email ilike '% %' OR email ilike '%\r%' OR email ilike '%\n%' OR email ilike '%\t%' or email ilike '%\\'%'""")
        res = cr.fetchall()
        for line in res:
            current_id = line[0]
            old_email = line[1]
            new_email = old_email.replace('\n','').replace('\t','').replace('\t','').replace(' ','').replace("'","")
            obj_contact.write(cr,uid,[current_id],{'email':new_email})
            corrected_emails.append(new_email)
            resp_id = line[2] or line[3]
            if resp_id:
                errors = errors_by_users.get(resp_id,[])
                if old_email not in errors:
                    errors.append(old_email)
                    errors_by_users[resp_id] = errors
        #
        cr.execute("""select id, email, write_uid, create_uid from cci_newsletter_subscriber where email ilike '% %' OR email ilike '%\r%' OR email ilike '%\n%' OR email ilike '%\t%' or email ilike '%\\'%'""")
        res = cr.fetchall()
        for line in res:
            current_id = line[0]
            old_email = line[1]
            new_email = old_email.replace('\n','').replace('\t','').replace('\t','').replace(' ','').replace("'","")
            obj_subscriber.write(cr,uid,[current_id],{'email':new_email})
            corrected_emails.append(new_email)
            resp_id = line[2] or line[3]
            if resp_id:
                errors = errors_by_users.get(resp_id,[])
                if old_email not in errors:
                    errors.append(old_email)
                    errors_by_users[resp_id] = errors
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nCorrected emails :\n')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n'.join([x for x in corrected_emails]))
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nConcerned users warned by email :')
        obj_users = self.pool.get('res.users')
        user_ids = obj_users.search(cr,uid,[('email','<>',False)])
        users = obj_users.read(cr,uid,user_ids,['id','email'])
        dUsers = {}
        for user in users:
            dUsers[user['id']] = user['email']
        for (user_id,error_list) in errors_by_users.items():
            if dUsers.has_key(user_id):
                visible_errors = [x.replace(' ','[space]').replace('\n','[new_line]').replace('\t','[tabulation]') for x in error_list]
                visibles = '<li>' + ('</li><li>'.join(visible_errors) ) + '</li>'
                tools.email_send('no-reply@ccilvn.be', [dUsers[user_id]], u'Erreur(s) dans l''encodage des emails', u'<p>Vous avez mal encod&eacute;s les emails suivants. Ils ont <b>&eacute;t&eacute; corrig&eacute;s</b> mais soyez prudents &agrave; l''avenir : de mauvais emails peuvent bloquer des emails group&eacute;s !</p><p><ul>%s</ul></p>' % visible_errors, subtype='html' )
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nEmail [%s] has received %s errors by mail' % (dUsers[user_id],len(error_list)) )
            else:
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nUser without email : ' + str(user_id) + ' had ' + str(len(error_list)) + ' errors')
                final_result = False
        #
        searched = [u'%@%@%',u'%à%',u'%â%',u'%ä%',u'%é%',u'%è%',u'%ê%',u'%ë%',u'%î%',u'%ï%',u'%ô%',u'%ö%',u'%ù%',u'%û%',u'%ü%',u'%ç%',u'%<%',u'%>%',u'%;%',u'%,%',u'%|%',u'%(%',u'%)%',u'%:%',u'%#%',u'%$%']
        tables = ['res.partner.address','res.partner.job','res.partner.contact','cci_newsletter.subscriber']
        mail_to_check = []
        for table in tables:
            wrong_ids = []
            for exp in searched:
                cr.execute("""select id from %s
                                WHERE email ilike '%s'""" % (table.replace('.','_'),exp) )
                res= cr.fetchall()
                if res:
                    for line in res:
                        if line[0] not in wrong_ids:
                            wrong_ids.append(line[0])
            wrong_emails = self.pool.get(table).read(cr,uid,wrong_ids,['email'])
            for email in wrong_emails:
                if email['email'] not in mail_to_check:
                    mail_to_check.append(email['email'])
        if mail_to_check:
            final_result = False

        control_regex = r"(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-_]+\.[a-zA-Z0-9-._]+$)"
        cregex = re.compile(control_regex)
        tables = ['res.partner.address','res.partner.job','res.partner.contact','cci_newsletter.subscriber']
        mail_to_check = []
        for table in tables:
            wrong_ids = []
            cr.execute("""select id, email from %s
                            WHERE length(email) > 0""" % table.replace('.','_') )
            res= cr.fetchall()
            for res_line in res:
                if not cregex.match(res_line[1]):
                    mail_to_check.append(res_line[1]+'  ->  '+table+'[%i]' % res_line[0])
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nemails to correct manually :')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n----------------------------\n')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n'.join(mail_to_check))
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_addresses_wo_zip_id(self,cr,uid):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_Addresses_Without_Zip_ID')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Addresses without Zip ID\n')
        final_result = True
        obj_addr = self.pool.get('res.partner.address')
        addr_ids = obj_addr.search(cr,uid,[('zip_id','=',False)])
        addresses = obj_addr.browse(cr,uid,addr_ids)
        errors1 = []
        errors2 = []
        errors3 = []
        aweekago = ( datetime.datetime.now() - datetime.timedelta(days=7) ).strftime('%Y-%m-%d %H:%M:%S')
        for addr in addresses:
            if addr.write_date >= aweekago:
                errors3.append(str(addr.id))
            if (addr.partner_id and addr.partner_id.id == 3) or not addr.partner_id:
                if addr.job_ids:
                    errors2.append(str(addr.id))
                else:
                    try:
                        obj_addr.undelete(cr,uid,[addr.id,])
                    except:
                        obj_addr.write(cr,uid,[addr.id,],{'active':False})
            else:
                errors1.append(str(addr.id))
        if errors1 or errors2:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Addresses without zip_id :\n\nRecent ones :\nCount : %i\n\n%s\n\nCount with partners: %i\n\n%s\n\nCount without partners : %i\n\n%s' % (len(errors3),','.join(errors3),len(errors1),','.join(errors1),len(errors2),','.join(errors2)) )
            if errors3:
                final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_strange_sex(self,cr,uid):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_Strange_sex')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking after Stranges Sexs...\n')
        final_result = True
        obj_contact = self.pool.get('res.partner.contact')
        normal_sexes = [('Monsieur','man'),('Madame','women')]
        for couple in normal_sexes:
            contact_ids = obj_contact.search(cr,uid,[('title','=',couple[0]),('gender','!=',couple[1])])
            if contact_ids:
                final_result = False
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nERROR = %s not %s' % (couple[0],couple[1]) )
                contacts = obj_contact.read(cr,uid,contact_ids,['title','gender','first_name','name'])
                for cont in contacts:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n[%i] %s %s : %s %s' % (cont['id'],cont['name'],cont['first_name'] or '-',cont['title'] or '-',cont['gender'] or '-'))
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_accounts_outside_bilan(self,cr,uid,report_ids=[],excluded_final_line_ids=[],excluded_account_ids=[]):
        # this tool detects accounts outside of bilan
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_Accounts_outside_bilan')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Accounts not found on \'Bilan\' or \'Compte de résultats\' belgian reports\n')
        final_result = True
        obj_account = self.pool.get('account.account')
        account_ids = obj_account.search(cr,uid,[('type','!=','view')])
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Accounts to verify : %i (non-views)\n' % len(account_ids) )
        accounts = obj_account.browse(cr,uid,account_ids)
        account_report_obj = self.pool.get('account.low.level.report')
        # we exclude some lines of reports because they sum all accounts (final results lines)
        #excluded_final_line_ids = [82,96,101,103,153]
        #report_ids = [1,56] # 1 = bilan; 2 = comptes de résultats
        # to be less specific, we search for all reports without parent_id. Can be a complete solution (if we use low level report for others things but today it's ok pour CCILVN and CCIH
        if not report_ids:
            report_ids = account_report_obj.search(cr,uid,[('parent_id','=',False)])
        # search for all headers defined under these reports in the hierarchy
        all_account_report_ids = []
        new_children_ids = list(report_ids)
        while new_children_ids:
            all_account_report_ids.extend(new_children_ids)
            new_children_ids = account_report_obj.search(cr,uid,[('parent_id','in',new_children_ids)])
        account_report_lines = account_report_obj.browse(cr,uid,all_account_report_ids)
        for acc in accounts:
            found = False
            for line in account_report_lines:
                linked_account_ids = [x.id for x in line.account_ids]
                if acc.id in linked_account_ids:
                    found = True
                    break
            if not found:
                acc_parent_ids = []
                current = acc
                while current.parent_id:
                    acc_parent_ids.append(current.parent_id.id)
                    current = current.parent_id
                for line in account_report_lines:
                    if line.id not in excluded_final_line_ids:
                        for parent_id in acc_parent_ids:
                            linked_account_ids = [x.id for x in line.account_ids]
                            if parent_id in linked_account_ids:
                                found = True
                                break
                    if found:
                        break
                if not found:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nerror %s - %s [%i]' % (acc.code,acc.name,acc.id))
                    final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_accounts_outside_budget(self,cr,uid,excluded_account_ids=[]):
        # this tool detects accounts of class 6 or 7 outside of global budget
        # the detection is done on all period of the current year but also on open pasts periods
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_Accounts_outside_budget')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Accounts not found on Budget\n')
        final_result = True
        obj_period = self.pool.get('account.period')
        begin_current_year = datetime.date.today().strftime('%Y')+'-01-01'
        end_current_year = datetime.date.today().strftime('%Y')+'-12-31'
        #
        period_ids = obj_period.search(cr,uid,[('date_start','>=',begin_current_year),('date_stop','<=',end_current_year),('special','=',False)])
        period2_ids = obj_period.search(cr,uid,[('date_stop','<=',begin_current_year),('state','=','draft'),('special','=',False)])
        period_ids = period_ids + period2_ids
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Periods to verify : [%s]\n' % ','.join([str(x) for x in period_ids]) )
        #
        obj_account = self.pool.get('account.account')
        account_ids = obj_account.search(cr,uid,[('type','!=','view')])
        accounts = obj_account.read(cr,uid,account_ids,['code'])
        good_account_ids = []
        for account in accounts:
            if account['code'][0] in ['6','7']:
                good_account_ids.append(account['id'])
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Accounts to verify : %i (non-views)\n' % len(account_ids) )
        if period_ids and account_ids:
            obj_mline = self.pool.get('account.move.line')
            ml_ids = obj_mline.search(cr,uid,[('period_id','in',period_ids),('state','=','valid'),('account_id','in',good_account_ids)])
            if ml_ids:
                mlines = obj_mline.read(cr,uid,ml_ids,['account_id'])
                moved_account_ids = []
                for line in mlines:
                    if line['account_id'] and line['account_id'][0] not in moved_account_ids:
                        moved_account_ids.append(line['account_id'][0])
                move_accounts = obj_account.read(cr,uid,moved_account_ids,['code','name','cci_budget_group_id'])
                for account in move_accounts:
                    if not account['cci_budget_group_id'] and account['id'] not in excluded_account_ids:
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Error with account : %s - %s\n' % (account['code'],account['name'] or '') )
                        final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_sales_with_strange_account(self,cr,uid,special_accepted_account_ids=[],excluded_line_ids=[]):
        # this tool detects accounts of class other than 7 on sales journal (except special_accepted_account_ids)
        # the detection is done on all period of the current year but also on open pasts periods
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_Sales_With_Strange_Accounts')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Move Lines on Sales Journals with Strange Accounts\n')
        final_result = True
        obj_period = self.pool.get('account.period')
        begin_current_year = datetime.date.today().strftime('%Y')+'-01-01'
        end_current_year = datetime.date.today().strftime('%Y')+'-12-31'
        #
        period_ids = obj_period.search(cr,uid,[('date_start','>=',begin_current_year),('date_stop','<=',end_current_year),('special','=',False)])
        period2_ids = obj_period.search(cr,uid,[('date_stop','<=',begin_current_year),('state','=','draft'),('special','=',False)])
        period_ids = period_ids + period2_ids
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Periods to verify : [%s]\n' % ','.join([str(x) for x in period_ids]) )
        #
        obj_account = self.pool.get('account.account')
        account_ids = obj_account.search(cr,uid,[('type','!=','view')])
        accounts = obj_account.read(cr,uid,account_ids,['code'])
        good_account_ids = []
        for account in accounts:
            if account['code'][0] == '7':
                good_account_ids.append(account['id'])
        good_account_ids.extend(special_accepted_account_ids)
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Accounts to verify : %i (non-views)\n' % len(good_account_ids) )
        #
        obj_jnl = self.pool.get('account.journal')
        jnl_ids = obj_jnl.search(cr,uid,[('type','=','sale')])
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Journals to verify : [%s]\n\n\nLines :\n' % ','.join([str(x) for x in jnl_ids]) )
        #
        if period_ids and good_account_ids and jnl_ids:
            obj_mline = self.pool.get('account.move.line')
            ml_ids = obj_mline.search(cr,uid,[('period_id','in',period_ids),('state','=','valid'),('account_id','not in',good_account_ids),('journal_id','in',jnl_ids),('id','not in',excluded_line_ids)])
            if ml_ids:
                mlines = obj_mline.browse(cr,uid,ml_ids)
                for line in mlines:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Line [%i] (%s) on %s : account %s %s\n' % (line.id,
                                                                                                                                   line.name or '',
                                                                                                                                   line.move_id and line.move_id.name or '',
                                                                                                                                   line.account_id and line.account_id.code or '',
                                                                                                                                   line.account_id and line.account_id.name or '',
                                                                                                                                  ) )
                final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _detect_address_not_in_belgium(self,cr,uid,TYPE_ID=5,NAME='Belgique',COUNTRY_ID=20):
        # this tool detects address with country_id = 'Belgique' but a zip_code not in 'Code postaux étrangers = Belgique'
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Detect_Addresses_not_in_Belgium')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Addresses not in Belgium\n')
        final_result = True
        obj_zip_group = self.pool.get('res.partner.zip.group')
        zip_group_ids = obj_zip_group.search(cr,uid,[('type_id','=',TYPE_ID),('name','=',NAME)])
        if zip_group_ids and len(zip_group_ids) == 1:
            sentence = """SELECT id
                              FROM res_partner_zip
                              WHERE id not in (
                                  SELECT zip_id
                                      FROM partner_zip_group_rel1
                                      WHERE group_id = %i 
                                  )
                       """ % zip_group_ids[0]
            cr.execute(sentence)
            res = cr.fetchall()
            if res:
                stranger_zip_ids = [x[0] for x in res]
                obj_address = self.pool.get('res.partner.address')
                address_ids = obj_address.search(cr,uid,[('zip_id','in',stranger_zip_ids),('country_id','=',COUNTRY_ID)])
                if address_ids:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'[%i] Wrong address ids : %s\n' % (len(address_ids),','.join([str(x) for x in address_ids])) )
                    final_result = False
        else:
            final_result = False
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Unable to make the test : ZIp Group [%i] name = \'%s\' not found\n' % (TYPE_ID,NAME) )
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
    def _check_partner_subscription(self,cr,uid,ignored_ids=[]):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'CCICleaning_Check_Partner_Subscription')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Checking Partner Subscriptions\n')
        final_result = True
        sub_type_obj = self.pool.get('partner_subscription.type')
        sub_type_ids = sub_type_obj.search(cr,uid,[('code','=','SOCREPELEC')])
        obj_psub = self.pool.get('partner_subscription')
        psub_ids = obj_psub.search(cr,uid,[('state','in',['draft','current']),('type_id','in',sub_type_ids)])
        stoday = datetime.date.today().strftime('%Y-%m-%d')
        if psub_ids:
            closeds = []
            gifts = []
            drafts = []
            no_invoices = []
            negatives = []
            canceled = []
            crossings = []
            inv_sale_journal_ids = self.pool.get('account.journal').search(cr,uid,[('type','=','sale'),('refund_journal','=',False)])
            refund_sale_journal_ids = self.pool.get('account.journal').search(cr,uid,[('type','=','sale'),('refund_journal','=',True)])
            psubs = obj_psub.browse(cr,uid,psub_ids)
            sub_by_partner = {}
            for psub in psubs:
                if psub.state == 'draft' and psub.begin < stoday:
                    drafts.append(u'[%i] - %s for %s € beginning at %s' % (psub.id,
                                                                           psub.partner_id and psub.partner_id.name or '/no partner/',
                                                                           psub.price and str(psub.price) or '0.0',
                                                                           psub.begin and french_date(psub.begin) or '--/--/----'))
                    final_result = False
                elif psub.state == 'current':
                    if psub.end < stoday:
                        closeds.append(u'[%i] - %s for %s € beginning at %s' % (psub.id,
                                                                                psub.partner_id and psub.partner_id.name or '/no partner/',
                                                                                psub.price and str(psub.price) or '0.0',
                                                                                psub.begin and french_date(psub.begin) or '--/--/----'))
                        # we correct this in the subscriptions
                        obj_psub.write(cr, uid, [psub.id,], {'state':'closed','close_date':stoday,'close_user_id':uid,'close_source':'Automatic - Ended'})
                    elif psub.price and psub.price <= -0.001:
                        negatives.append(u'[%i] - %s for %s € beginning at %s' % (psub.id,
                                                                                  psub.partner_id and psub.partner_id.name or '/no partner/',
                                                                                  psub.price and str(psub.price) or '0.0',
                                                                                  psub.begin and french_date(psub.begin) or '--/--/----'))
                        final_result = False
                    elif (psub.price and psub.price >= -0.001 and psub.price <= 0.001) or not psub.price:
                        gifts.append(u'[%i] - %s beginning at %s and ending at %s' % (psub.id,
                                                                              psub.partner_id and psub.partner_id.name or '/no partner/',
                                                                              psub.begin and french_date(psub.begin) or '--/--/----',
                                                                              psub.end and french_date(psub.end) or '--/--/----'))
                    elif psub.price and psub.price >= 0.001:
                        if not psub.invoice_id:
                            no_invoices.append(u'[%i] - %s beginning at %s and ending at %s' % (psub.id,
                                                                                                psub.partner_id and psub.partner_id.name or '/no partner/',
                                                                                                psub.begin and french_date(psub.begin) or '--/--/----',
                                                                                                psub.end and french_date(psub.end) or '--/--/----'))
                            final_result = False
                        else:
                            # we consider it is not paid only by refund
                            only_paid_with_refund = False
                            invoice = psub.invoice_id
                            if invoice.journal_id and invoice.journal_id.id in inv_sale_journal_ids and invoice.reconciled:
                                if invoice.payment_ids:
                                    # it is paid, so we consider it is paid only with refund, except if you found another payment
                                    only_paid_with_refund = True
                                    for pay in invoice.payment_ids:
                                        if pay.journal_id and pay.journal_id.id not in refund_sale_journal_ids:
                                            only_paid_with_refund = False
                            if only_paid_with_refund:
                                canceled.append(u'[%i] - %s beginning at %s and ending at %s' % (psub.id,
                                                                                                 psub.partner_id and psub.partner_id.name or '/no partner/',
                                                                                                 psub.begin and french_date(psub.begin) or '--/--/----',
                                                                                                 psub.end and french_date(psub.end) or '--/--/----'))
                                final_result = False
                                # we correct this in the subscriptions
                                obj_psub.write(cr, uid, [psub.id,], {'state':'closed','close_date':stoday,'close_user_id':uid,'close_source':'Automatic - Canceled by refund'})
                            else:
                                # we keep this good subscription to see if there is no crossing between two subscription for the same partner
                                if psub.partner_id:
                                    if sub_by_partner.has_key(psub.partner_id.id):
                                        data = sub_by_partner[psub.partner_id.id]
                                        data['subs'] = data['subs'].append((psub.id,psub.begin,psub.end))
                                    else:
                                        data = {}
                                        data['subs'] = [(psub.id,psub.begin,psub.end)]
                                    sub_by_partner[psub.partner_id.id] = data
            multiple_subs_partners = 0
            if sub_by_partner:
                for (partner_id,data) in sub_by_partner.items():
                    if len(data) > 1:
                        multiple__subs_by_partner += 1
                        current = 0
                        while current < len(data)-1:
                            (sub_id,sub_begin,sub_end) = data[current]
                            test = current + 1
                            while test < len(data):
                                (test_id,test_begin,test_end) = data[test]
                                if (sub_begin > test_begin and sub_begin < test_end) or (sub_end > test_begin and sub_end < test_end):
                                    crossings.append(u'Partner [%i] has two subscription with crossing dates' % partner_id)
                                    test = len(data)+1
                                    current = len(data)+1
                                test += 1
                            current += 1
            if gifts:
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nGifts : not an error, just FYI\n' + '\n'.join(gifts))
            if closeds:
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nClosed : not an error, just FYI\n' + '\n'.join(closeds))
            if drafts:
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nDrafts with past data\n' + '\n'.join(drafts))
            if negatives:
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nNegative amounts\n' + '\n'.join(gifts))
            if no_invoices:
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nAmounts without linked invoice\n' + '\n'.join(no_invoices))
            if canceled:
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nCanceled by Refund - automatic correction\n' + '\n'.join(canceled))
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nGood partners with current subscriptions : %i' % len(sub_by_partner))
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n  with %i partners with multiple current subscriptions' % multiple_subs_partners)
            if crossings:
                final_result = False
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nDate crossings for the same partner\n' + '\n'.join(crossing))
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return final_result
cci_cleaning_task()
