# -*- encoding: utf-8 -*-

from pyExcelerator import *
import tempfile
import os
import base64
import unicodedata

# Convert a list of records into a dictionary
#
# @param index the index used in the dictionary
# @param records the records to put in the dictionary
def make_dictionary(index,records):

    dic = {}
    for record in records:    
    
        key = record[index]
        if type(key) is list:
            key = key[0]
    
        dic[key] = record   
    return dic

def strip_accents(text):
    try:
        text2 = unicode(text,"utf-8")
    except TypeError:
        text2 = unicode(text.encode('utf-8'),"utf-8")
    return ''.join((c for c in unicodedata.normalize('NFD', text2) if unicodedata.category(c) != 'Mn'))



def generate_xls_file(sheet_contents):
    
    style_normal = XFStyle()
        
    style_headers = XFStyle()
    style_headers.font.bold = True
    
    style_error = XFStyle()
    style_error.font.colour_index = Formatting.get_colour_val('red')
    style_error.font.bold = True
    
    wb = Workbook()
    
    for sheet_content in sheet_contents:
    
        ws = wb.add_sheet(unicode(sheet_content['name'],"utf-8"))
        
        i = 0
        j = 0
        
        # Print summary in file
        for line in sheet_content['summary']:
            j = 0
            for value in line:
                ws.write(i, j, unicode(value,"utf-8"))
                j = j + 1
            i = i + 1
        i = i + 1
        
        # Print headers in file
        j = 0
        for header in sheet_content['headers']:
            ws.write(i, j, unicode(header,"utf-8"), style_headers)
            j = j + 1
        
        # Print each line in file
        i = i + 1
        for line in sheet_content['data']:
            j = 0
            for value in line:
                if value:
                    try:
                        ws.write(i, j, unicode(value,"utf-8"), style_normal)
                    except TypeError:
                        ws.write(i, j, unicode(value.encode('utf-8'),"utf-8"), style_normal)
                else:
                    if sheet_content['show_missing']:
                        ws.write(i, j, unicode("INVALID".encode('utf-8'),"utf-8"), style_error)
                j = j + 1
            i = i + 1
        
    # Return file content
    tmp_filename = tempfile.mkstemp()[1]
    wb.save(tmp_filename)
    with open(tmp_filename,mode='r') as tmp_xls:
        xls = base64.encodestring(tmp_xls.read())
    os.remove(tmp_filename)
    return xls
