# -*- encoding: utf-8 -*-

from osv import fields, osv
import wizard
import pooler
import os
import base64
import time
import datetime
from pyExcelerator import *
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Export Sessions List">
    <field name="type_id" colspan="4" width="600"/>
    <field name="from_date"/>
    <field name="to_date"/>
</form>"""

fields = {
    'type_id': {'string':'Type of clubs', 'type':'many2one', 'relation':'cci_club.club_type','required':True},
    'from_date': {'string':'From', 'type':'date','required':True},
    'to_date': {'string':'To', 'type':'date','required':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Results">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="sessions_list.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'sessions_list.xls':{'string': 'Prepared file','type': 'binary','readonly': True,},
}

class export_sessions_list(wizard.interface):
    def _extract_file(self,cr,uid,data,context):
        # create the workbook
        wb = Workbook()
        ws1 = wb.add_sheet(_('Sessions'))
        ws1.write(0,0,_('ID Session'))
        ws1.write(0,1,_('Club Name'))
        ws1.write(0,2,_('Date'))
        ws1.write(0,3,_('Number'))
        ws1.write(0,4,_('Session State'))
        ws1.write(0,5,_('Comments'))
        ws1.write(0,6,_('Host Address'))
        ws1.write(0,7,_('Host Contact'))
        ws2 = wb.add_sheet(_('Parameters'))
        ws2.write(0,0,_('Date'))
        ws2.write(1,0,_('Club Type'))
        ws2.write(2,0,_('From'))
        ws2.write(3,0,_('To'))
        # save the parameters
        club_type_obj = pooler.get_pool(cr.dbname).get('cci_club.club_type')
        session_obj = pooler.get_pool(cr.dbname).get('cci_club.session')
        ws2.write(0,1,datetime.datetime.today().strftime('%d/%m/%Y'))
        club = club_type_obj.browse(cr,uid,[data['form']['type_id']])[0]
        ws2.write(1,1,club.name)
        date_from = data['form']['from_date']
        date_to = data['form']['to_date']
        if date_to < date_from:
            (date_to,date_from) = (date_from,date_to)
        ws2.write(2,1,date_from[8:]+'/'+date_from[5:7]+'/'+date_from[0:4])
        ws2.write(3,1,date_to[8:]+'/'+date_to[5:7]+'/'+date_to[0:4])
        sentence = """SELECT sess.id, sess.date, sess.number, sess.state_id, sstate.name, sess.comments, sess.group_id, club.name
                        FROM cci_club_session as sess, cci_club_session_state as sstate, cci_club_club as club 
                        WHERE sess.group_id = club.id AND sess.state_id = sstate.id AND club.type_id = %s AND club.active AND sess.date BETWEEN '%s' AND '%s'
                        ORDER BY club.name, sess.date
        """ % (str(club.id),date_from,date_to)
        cr.execute(sentence)
        lines = cr.fetchall()
        session_ids = [x[0] for x in lines]
        sessions = session_obj.browse(cr,uid,session_ids)
        dSessions = {}
        for sess in sessions:
            dSessions[sess.id] = sess
        linenum = 1
        for line in lines:
            ws1.write(linenum,0,line[0])
            ws1.write(linenum,1,line[7])
            ws1.write(linenum,2,line[1][8:]+'/'+line[1][5:7]+'/'+line[1][0:4])
            ws1.write(linenum,3,line[2] or '')
            ws1.write(linenum,4,line[4] or '')
            ws1.write(linenum,5,(line[5] or '')[0:250].replace('\n','-'))
            if dSessions.has_key(line[0]):
                session = dSessions[line[0]]
                if session.host_address_id:
                    address = session.host_address_id
                else:
                    if session.group_id.host_address_id:
                        address = session.group_id.host_address_id
                    else:
                        address = False
                if address:
                    full_address = address.street
                    if address.street2:
                        full_address += (' '+address.street2)
                    if address.zip_id:
                        full_address += (', '+address.zip_id.name + ' ' + address.zip_id.city)
                    ws1.write(linenum,6,full_address)
                if session.host_contact_id:
                    contact = session.host_contact_id.name
                    if session.host_contact_id.first_name:
                        contact += (' '+session.host_contact_id.first_name)
                    if session.host_contact_id.mobile:
                        contact += (' - GSM : '+session.host_contact_id.mobile)
                    ws1.write(linenum,7,contact)
            linenum += 1
        # give the result tos the user
        wb.save('sessions_list.xls')
        result_file = open('sessions_list.xls','rb').read()
        data['form']['msg']=_('File created')
        data['form']['sessions_list.xls']=base64.encodestring(result_file)
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('extract','Extract')]},
        },
        'extract': {
            'actions': [_extract_file],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }
export_sessions_list('export_sessions_list')
