# -*- encoding: utf-8 -*-

from osv import fields, osv
import wizard
import pooler
import os
import base64
import time
import datetime
from tools.translate import _
from pyExcelerator import *

form = """<?xml version="1.0"?>
<form string="Score Presences By Sessions">
    <field name="type_id" colspan="4" width="600"/>
    <field name="from_date"/>
    <field name="to_date"/>
</form>"""

fields = {
    'type_id': {'string':'Type of clubs', 'type':'many2one', 'relation':'cci_club.club_type','required':True},
    'from_date': {'string':'From', 'type':'date','required':True},
    'to_date': {'string':'To', 'type':'date','required':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Results">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="score_presences_and_sessions.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'score_presences_and_sessions.xls':{'string': 'Prepared file','type': 'binary','readonly': True,},
}

class score_presence_by_session(wizard.interface):
    def _score_presence_by_session(self,cr,uid,data,context):
        '''
        This method checks the scoring of presences of participers to clubs for session between two dates
        '''
        type_id = data['form']['type_id']
        from_date = data['form']['from_date']
        to_date = data['form']['to_date']
        # creating excel file for output
        ExcelFileName = 'score_presences.xls'

        wb = Workbook()
        ws1 = wb.add_sheet(_('Clubs'))
        ws2 = wb.add_sheet(_('Persons'))
        ws3 = wb.add_sheet(_('Sessions'))
        ws4 = wb.add_sheet(_('Parameters'))
        
        # recording parameters in excel file for 'debugging' if necessary
        ws4.write(0,0,'Types')
        ws4.write(1,0,'From')
        ws4.write(2,0,'To')
        ws4.write(3,0,'Dates')
        club_type = pooler.get_pool(cr.dbname).get('cci_club.club_type').browse(cr,uid,[type_id])[0]
        ws4.write(0,1,'%s (%i)' % (club_type.name,club_type.id) )
        ws4.write(1,1,from_date)
        ws4.write(2,1,to_date)

        # construct the variavbles to store data to display later
        club_ids = []
        all_session_ids = []
        part_ids = []
        dResClubs = {}
        dResParts = {}
        dResSess = {}
        # we extract all presences for sessions between the two dates and club of the given type
        selection = """SELECT attend.participation_id, session.group_id, state.count_as, session.id
                           FROM cci_club_attendance as attend, cci_club_session as session, cci_club_attendance_state as state, cci_club_club as club
                           WHERE attend.state = state.id AND attend.session_id = session.id AND attend.active AND
                                 session.group_id = club.id AND club.type_id = %s AND session.date BETWEEN '%s' AND '%s'
                    """ % ( str(type_id), from_date, to_date )
        cr.execute(selection)
        res = cr.fetchall()
        for line in res:
            # add this sum in the count for the clubs
            if dResClubs.has_key(line[1]):
                resClub = dResClubs[line[1]]
            else:
                resClub = {'present':0,'absent':0,'undefined':0}
                club_ids.append(line[1])
            resClub[line[2]] += 1
            dResClubs[line[1]] = resClub
            # add this sum in the count for the participers
            if dResParts.has_key(line[0]):
                resPart = dResParts[line[0]]
            else:
                resPart = {'present':0,'absent':0,'undefined':0}
            resPart[line[2]] += 1
            dResParts[line[0]] = resPart
            # add this sum in the count for the sessions
            if dResSess.has_key(line[3]):
                resSession = dResSess[line[3]]
            else:
                resSession = {'present':0,'absent':0,'undefined':0}
                all_session_ids.append(line[3])
            resSession[line[2]] += 1
            dResSess[line[3]] = resSession
        # sort the results by ascending scoring (the worst scoring at the beginning)
        # with the scoring = present/(present+absent)
        scoring_clubs = []
        for (key,result) in dResClubs.items():
            if result['present'] + result['absent'] > 0:
                scoring_clubs.append( ( key,int(result['present']*100/(result['present']+result['absent'])) ) )
            else:
                scoring_clubs.append( ( key, 0 ) )
        sorted_scoring_clubs = sorted(scoring_clubs, key=lambda k: k[1] )
        scoring_parts = []
        for (key,result) in dResParts.items():
            if result['present'] + result['absent'] > 0:
                scoring_parts.append( ( key,int(result['present']*100/(result['present']+result['absent'])) ) )
            else:
                scoring_parts.append( ( key, 0 ) )
            part_ids.append( key )
        sorted_scoring_parts = sorted(scoring_parts, key=lambda k: k[1] )
        scoring_sessions = []
        for (key,result) in dResSess.items():
            if result['present'] + result['absent'] > 0:
                scoring_sessions.append( ( key,int(result['present']*100/(result['present']+result['absent'])) ) )
            else:
                scoring_sessions.append( ( key, 0 ) )
        sorted_scoring_sessions = sorted(scoring_sessions, key=lambda k: k[1] )
        # record results
        # first tab : scoring by clubs
        ws1.write(0,0,'Club name')
        ws1.write(0,1,'Presences')
        ws1.write(0,2,'Absences')
        ws1.write(0,3,'Undefined')
        ws1.write(0,4,'Scoring')
        ws1.write(0,5,'Encoded past sessions')
        ws1.write(0,6,'Undefined past session')
        ws1.write(0,7,'Future sessions')
        ws1.write(0,8,'School')
        if club_ids:
            clubs = pooler.get_pool(cr.dbname).get('cci_club.club').browse(cr,uid,club_ids)
            dClubs = {}
            for club in clubs:
                dClubs[club.id] = {'name':club.name,'school':club.school_id and club.school_id.name or ''}
            obj_session = pooler.get_pool(cr.dbname).get('cci_club.session')
            today = datetime.date.today().strftime('%Y-%m-%d')
            line = 1
            for (key,result) in sorted_scoring_clubs:
                ws1.write(line,0,dClubs[key]['name'])
                ws1.write(line,1,dResClubs[key]['present'])
                ws1.write(line,2,dResClubs[key]['absent'])
                ws1.write(line,3,dResClubs[key]['undefined'])
                ws1.write(line,4,result)
                # reading of the state of sessions of this clubs
                session_ids = obj_session.search(cr,uid,[('group_id','=',key)])
                sessions = obj_session.browse(cr,uid,session_ids)
                encoded = 0
                undefined = 0
                future = 0
                for session in sessions:
                    if session.date >= today:
                        future += 1
                    else:
                        if session.attendance_ids:
                            encoded += 1
                        else:
                            undefined += 1
                ws1.write(line,5,encoded)
                ws1.write(line,6,undefined)
                ws1.write(line,7,future)
                ws1.write(line,8,dClubs[key]['school'])
                line += 1
        # second tab : scoring by participation
        ws2.write(0,0,'Participer')
        ws2.write(0,1,'Company')
        ws2.write(0,2,'Salesman')
        ws2.write(0,3,'Club')
        ws2.write(0,4,'Present')
        ws2.write(0,5,'Absent')
        ws2.write(0,6,'Undefined')
        ws2.write(0,7,'Scoring')
        if part_ids:
            parts = pooler.get_pool(cr.dbname).get('cci_club.participation').browse(cr,uid,part_ids)
            dParts = {}
            for participer in parts:
                if participer.contact_id:
                    full_name = participer.contact_id.name
                    if participer.contact_id.first_name:
                        full_name += ( ' ' + participer.contact_id.first_name )
                else:
                    full_name = ''
                salesman_name = ''
                if participer.partner_id:
                    if participer.partner_id.user_id:
                        salesman_name = participer.partner_id.user_id.name or ''
                dParts[participer.id] = {'name':full_name,'company':participer.partner_id and participer.partner_id.name or '','salesman':salesman_name,'club':participer.group_id and participer.group_id.name or ''}
            line = 1
            for (key,result) in sorted_scoring_parts:
                ws2.write(line,0,dParts[key]['name'])
                ws2.write(line,1,dParts[key]['company'])
                ws2.write(line,2,dParts[key]['salesman'])
                ws2.write(line,3,dParts[key]['club'])
                ws2.write(line,4,dResParts[key]['present'])
                ws2.write(line,5,dResParts[key]['absent'])
                ws2.write(line,6,dResParts[key]['undefined'])
                ws2.write(line,7,result)
                line += 1
        # third tab : scoring by session
        ws3.write(0,0,'Session')
        ws3.write(0,1,'Presences')
        ws3.write(0,2,'Absences')
        ws3.write(0,3,'Undefined')
        ws3.write(0,4,'Scoring')
        ws3.write(0,5,'School')
        if session_ids:
            sessions = pooler.get_pool(cr.dbname).get('cci_club.session').browse(cr,uid,all_session_ids)
            dSessions = {}
            for sess in sessions:
                dSessions[sess.id] = {'name':sess.date + ' - ' + sess.group_id.name,'school':sess.group_id.school_id and sess.group_id.school_id.name or ''}
            obj_session = pooler.get_pool(cr.dbname).get('cci_club.session')
            today = datetime.date.today().strftime('%Y-%m-%d')
            line = 1
            for (key,result) in sorted_scoring_sessions:
                ws3.write(line,0,dSessions[key]['name'])
                ws3.write(line,1,dResSess[key]['present'])
                ws3.write(line,2,dResSess[key]['absent'])
                ws3.write(line,3,dResSess[key]['undefined'])
                ws3.write(line,4,result)
                ws3.write(line,5,dSessions[key]['school'])
                line += 1
        # 
        # save the created excel file
        wb.save('score_presences_and_sessions.xls')
        
        result_file = open('score_presences_and_sessions.xls','rb').read()
        # give the result tos the user
        data['form']['msg']='File created \'score_presences_and_sessions.xls\''
        data['form']['score_presences_and_sessions.xls']=base64.encodestring(result_file)
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('calculate','Score')]},
        },
        'calculate': {
            'actions': [_score_presence_by_session],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }
score_presence_by_session('score_presence_by_session')
