# -*- encoding: utf-8 -*-
# 2013-03-08 Philmer manage inactive participation and 0,00 latitude - longitude
# 2013-04-22 Philmer test if partner with address, if not, don't bug, just ignore
from osv import fields, osv
import wizard
import pooler
import os
import base64
import time
from pyExcelerator import *
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Search for Groups">
    <field name="type" colspan="4"/>
    <field name="club_state" colspan="4"/>
    <field name="part_state" colspan="4"/>
    <field name="level_max"/>
    <field name="dist_max"/>
    <field name="min_size"/>
    <field name="max_size"/>
    <field name="langage_id"/>
</form>"""

fields = {
    'type': {'string':'Type of clubs', 'type':'many2one', 'relation':'cci_club.club_type','required':True},
    'club_state':{'string':'State of clubs', 'type':'selection', 'selection':[('draft','Waiting to begin'),('current','Current'),('done','Closed')],'required':True},
    'part_state':{'string':'State of participers', 'type':'many2one', 'relation':'cci_club.participation_state','required':True},
    'level_max':{'string':'Max difference of level', 'type':'integer','default':1},
    'dist_max':{'string':'Max Distance in Kms', 'type':'float', 'required':True},
    'min_size':{'string':'Min Size', 'type':'integer'},
    'max_size':{'string':'Max Size', 'type':'integer'},
    'langage_id':{'string':'Limit to langage', 'type':'many2one', 'relation':'cci_club.language'},
}

msg_form = """<?xml version="1.0"?>
<form string="Results">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="groups.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'groups.xls':{'string': 'Prepared file','type': 'binary','readonly': True,},
}

def choose_iter(elements, length):
    for i in xrange(len(elements)):
        if length == 1:
            yield (elements[i],)
        else:
            for next in choose_iter(elements[i+1:len(elements)], length-1):
                yield (elements[i],) + next

class search_groups(wizard.interface):
    def _valid_group(self,cr,uid,list_of_parts,dist_max,level_max):
        # first check if this group has not yet been accepted
        # or is contained in a bigger gruop already accepted
        # if so, we consider it not valid, AGAIN...
        slist = sorted( [x.id for x in list_of_parts] )
        if slist in self.lResults:
            return False
        newset = set(slist)
        for result in self.lResults:
            if newset.issubset(set(result)):
                return False
        # check if the group if valid and if valid, return the maximum distance and the maximum difference of levels
        zip_obj = pooler.get_pool(cr.dbname).get('res.partner.zip')
        current = 0
        max_distance = 0.0
        max_levels = 0
        error = False
        for main in list_of_parts:
            for second in list_of_parts[current+1:]:
                if abs(main.level_id.sequence - second.level_id.sequence) <= level_max:
                    max_levels = max(max_levels,abs(main.level_id.sequence - second.level_id.sequence))
                    current_distance = zip_obj.distance_in_kilometers(cr,uid,self.dParts[main.id]['job_zip_id'],self.dParts[second.id]['job_zip_id'])
                    if current_distance <= dist_max:
                        max_distance = max(max_distance,current_distance)
                    else:
                        error = True
                        break
                else:
                    error = True
                    break
            if error:
                break
            else:
                current += 1
        if error:
            result = False
        else:
            # no problem with distance or level, check if we have at least one common available day for all participers
            days = [True,True,True,True,True]
            for prosp in list_of_parts:
                days[0] = days[0] and prosp.monday
                days[1] = days[1] and prosp.tuesday
                days[2] = days[2] and prosp.wednesday
                days[3] = days[3] and prosp.thursday
                days[4] = days[4] and prosp.friday
            if days[0] or days[1] or days[2] or days[3] or days[4]:
                avails = []
                if days[0]:
                    avails.append(_('Monday'))
                if days[1]:
                    avails.append(_('Tuesday'))
                if days[2]:
                    avails.append(_('Wednesday'))
                if days[3]:
                    avails.append(_('Thursday'))
                if days[4]:
                    avails.append(_('Friday'))
                availables = ','.join(avails)
                result = (max_distance,max_levels,availables)
            else:
                result = False
        return result
    def _search_for_groups(self,cr,uid,data,context):
        # 1. extraction of all concerned participations
        if data['form']['langage_id']:
            selection = """SELECT parti.id, parti.language_id
                                FROM cci_club_participation as parti, cci_club_club as club 
                                WHERE club.type_id = %i AND club.state = '%s' AND parti.state_id = %i AND parti.language_id = %i AND parti.group_id = club.id
                                      AND club.active AND parti.active""" % (data['form']['type'],data['form']['club_state'],data['form']['part_state'],data['form']['langage_id'])
        else:
            selection = """SELECT parti.id 
                                FROM cci_club_participation as parti, cci_club_club as club 
                                WHERE club.type_id = %i AND club.state = '%s' AND parti.state_id = %i AND parti.group_id = club.id
                                      AND club.active AND parti.active""" % (data['form']['type'],data['form']['club_state'],data['form']['part_state'])
        cr.execute(selection)
        res = cr.fetchall()
        part_ids = [x[0] for x in res]
        max_possibles = 250
        parts = pooler.get_pool(cr.dbname).get('cci_club.participation').browse(cr,uid,part_ids)
        
        # 2. Extract the zip code for the latitude longitude
        self.dParts = {}
        for part in parts:
            if part.partner_id.address:
                if len(part.partner_id.address) == 1:
                    if part.partner_id.address[0].zip_id:
                        self.dParts[part.id] = {'job_zip_id':part.partner_id.address[0].zip_id.id}
                    else:
                        self.dParts[part.id] = {'job_zip_id':False}
                else:
                    zip_id = False
                    for addr in part.partner_id.address:
                        for job in addr.job_ids:
                            if job.contact_id.id == part.contact_id.id:
                                if addr.zip_id:
                                    zip_id = addr.zip_id.id
                                    break
                    self.dParts[part.id] = {'job_zip_id':zip_id}
        # Creation of the result file
        wb = Workbook()
        ws1 = wb.add_sheet(_('Groups'))
        ws2 = wb.add_sheet(_('Persons'))
        ws3 = wb.add_sheet(_('Isolated'))
        ws4 = wb.add_sheet(_('Problems'))
        # 3. For each participant, search if zip code has latitude, longitude
        ws4.write(0,0,_('Zip codes without latitude and longitude'))
        ws4.write(1,0,_('Code'))
        ws4.write(1,1,_('City'))
        line4 = 2
        zip_ids = pooler.get_pool(cr.dbname).get('res.partner.zip').search(cr,uid,[])
        zips = pooler.get_pool(cr.dbname).get('res.partner.zip').read(cr,uid,zip_ids,['id','name','city','latitude','longitude'])
        dZips = {}
        for zip_code in zips:
            dZips[zip_code['id']] = zip_code
        for (id,part) in self.dParts.items():
            if part['job_zip_id']:
                zip_code = dZips[part['job_zip_id']]
                if not zip_code['latitude'] or not zip_code['longitude'] or ( abs(zip_code['latitude']) < 0.001 and abs(zip_code['longitude']) < 0.001):
                    ws4.write(line4,0,zip_code['name'])
                    ws4.write(line4,1,zip_code['city'])
                    line4 += 1
        line4 += 1
        ws4.write(line4,0,_('Participers without level'))
        line4 += 1
        ws4.write(line4,0,_('Name'))
        ws4.write(line4,1,_('First Name'))
        ws4.write(line4,2,_('Company'))
        ws4.write(line4,3,_('Zip code'))
        ws4.write(line4,4,_('City'))
        line4 += 1
        for part in parts:
            if not part.level_id:
                zip_id = self.dParts[part.id]['job_zip_id']
                ws4.write(line4,0,part.contact_id.name)
                ws4.write(line4,1,part.contact_id.first_name or '')
                ws4.write(line4,2,part.partner_id.name)
                ws4.write(line4,3,zip_id and dZips[zip_id]['name'] or '')
                ws4.write(line4,4,zip_id and dZips[zip_id]['city'] or '')
                line4 += 1
        line4 += 1
        ws4.write(line4,0,_('Participers without language'))
        line4 += 1
        ws4.write(line4,0,_('Name'))
        ws4.write(line4,1,_('First Name'))
        ws4.write(line4,2,_('Company'))
        ws4.write(line4,3,_('Zip code'))
        ws4.write(line4,4,_('City'))
        line4 += 1
        for part in parts:
            if not part.language_id:
                zip_id = self.dParts[part.id]['job_zip_id']
                ws4.write(line4,0,part.contact_id.name)
                ws4.write(line4,1,part.contact_id.first_name or '')
                ws4.write(line4,2,part.partner_id.name)
                ws4.write(line4,3,zip_id and dZips[zip_id]['name'] or '')
                ws4.write(line4,4,zip_id and dZips[zip_id]['city'] or '')
                line4 += 1

        # 4. search for biggest groups
        #  for each person, we search all others persons compatible with language, levels, distance and days
        #  after that, we try to create biggest groups with theses persons and keep each group
        #  where level, distance and days are compatible for the entire group
        ws1.write(0,0,_('Group name'))
        ws1.write(0,1,_('Langage'))
        ws1.write(0,2,_('Size'))
        ws1.write(0,3,_('Max Distance'))
        ws1.write(0,4,_('Max diff level'))
        ws1.write(0,5,_('Available days'))
        group_id = 1
        #
        ws2.write(0,0,_('Group name'))
        ws2.write(0,1,_('Name'))
        ws2.write(0,2,_('First name'))
        ws2.write(0,3,_('Company'))
        ws2.write(0,4,_('Zip Code'))
        ws2.write(0,5,_('City'))
        ws2.write(0,6,_('Langage'))
        ws2.write(0,7,_('Level'))
        prospect_id = 1
        #
        zip_obj = pooler.get_pool(cr.dbname).get('res.partner.zip')
        dGroups = {}
        self.lResults = []
        for part in parts:
            #print '---new part---' + str(part.id)
            compatibles = []
            if self.dParts.has_key( part.id ) and self.dParts[part.id]['job_zip_id'] and part.level_id and part.language_id:
                # the person has a zip and a level, we can try to cross it with others
                for other in parts:
                    if part.id != other.id:
                        if self.dParts.has_key( other.id ) and self.dParts[other.id]['job_zip_id'] and other.language_id and other.language_id.id == part.language_id.id:
                            if zip_obj.distance_in_kilometers(cr,uid,self.dParts[part.id]['job_zip_id'],self.dParts[other.id]['job_zip_id']) <= data['form']['dist_max']:
                                # in range from this person
                                if other.level_id and abs(other.level_id.sequence - part.level_id.sequence ) <= data['form']['level_max']:
                                    # in range in term of level too, this other person is compatible
                                    compatibles.append( other )
                if compatibles and len(compatibles) >= (data['form']['min_size']-1):
                    # enought compatibles to try to create groups
                    # we first try to create as much groups as biggest as possible
                    # then if we don't succeed, we try with smaller groups
                    # we stop to try for this person with the biggest size that give at least a result
                    current_size = min(data['form']['max_size'],len(compatibles)+1)
                    results = []
                    while current_size >= data['form']['min_size'] and not results:
                        all_compatibles = compatibles + [part]
                        if len(all_compatibles) == current_size:
                            # the only group to check if a group with all prospects
                            check = self._valid_group(cr,uid,all_compatibles,data['form']['dist_max'],data['form']['level_max'])
                            if check:
                                #print str(group_id)+' '+str(check)
                                results.append((all_compatibles,check))
                                self.lResults.append( sorted([x.id for x in all_compatibles]) )
                                #print self.lResults
                        else:
                            # there is more prospects than places in the group, we must check for all possible combinations
                            # but we must care about combination explosion, so we limit the size of the all_compatibles to 150%
                            # of the max_size, and to select the compatibles inside the group, we classify them with the distance
                            # to the current participer
                            if len(all_compatibles) <= int(data['form']['max_size'] * 1.5 ):
                                #print 'possibles %s for size %s' % (str(len(all_compatibles)),str(current_size))
                                possibles = list(choose_iter(all_compatibles,current_size))
                                if len(possibles) <= max_possibles:
                                    #print len(possibles)
                                    for possible in possibles:
                                        #print 'check'
                                        check = self._valid_group(cr,uid,possible,data['form']['dist_max'],data['form']['level_max'])
                                        if check:
                                            #print str(group_id)+' '+str(check)
                                            results.append((possible,check))
                                            self.lResults.append( sorted([x.id for x in possible]) )
                                            #print self.lResults
                            else:
                                # we search closer participers until max-size * 150%
                                #print 'too much possibles'
                                new_list = [part]
                                list_of_compatibles = []
                                max_list = (int(data['form']['max_size']*1.5)-1)
                                for compatible in compatibles:
                                    list_of_compatibles.append( {'distance':zip_obj.distance_in_kilometers(cr,uid,self.dParts[part.id]['job_zip_id'],self.dParts[compatible.id]['job_zip_id']),
                                                                 'participant':compatible} )
                                    slist_of_compatibles = sorted(list_of_compatibles, key=lambda k: k['distance'])
                                    #print 'slist_of_compatibles'
                                    #print slist_of_compatibles
                                new_list = new_list + [x['participant'] for x in slist_of_compatibles[0:max_list] ]
                                #print 'new_list'
                                #print new_list
                                #print 'len(new_list)'
                                #print len(new_list)
                                #print 'possibles %s for size %s' % (str(len(new_list)),str(current_size))
                                possibles = list(choose_iter(new_list,current_size))
                                #print 'len(possibles)'
                                #print len(possibles)
                                if len(possibles) <= max_possibles:
                                    for possible in possibles:
                                        #print 'check'
                                        check = self._valid_group(cr,uid,possible,data['form']['dist_max'],data['form']['level_max'])
                                        if check:
                                            #print str(group_id)+' '+str(check)
                                            results.append((possible,check))
                                            self.lResults.append( sorted([x.id for x in possible]) )
                                            #print self.lResults
                        if results:
                            for result in results:
                                # record the detected valid groups in excel file
                                group_parts = result[0]
                                maxs = result[1]
                                ws1.write(group_id,0,_('Group ') +str(group_id))
                                ws1.write(group_id,1,group_parts[0].language_id.name)
                                ws1.write(group_id,2,len(group_parts))
                                ws1.write(group_id,3,maxs[0])
                                ws1.write(group_id,4,maxs[1])
                                ws1.write(group_id,5,maxs[2])
                                for prospect in group_parts:
                                    zip_id = self.dParts[prospect.id]['job_zip_id']
                                    ws2.write(prospect_id,0,'Group ' +str(group_id))
                                    ws2.write(prospect_id,1,prospect.contact_id.name)
                                    ws2.write(prospect_id,2,prospect.contact_id.first_name or '')
                                    ws2.write(prospect_id,3,prospect.partner_id.name)
                                    ws2.write(prospect_id,4,zip_id and dZips[zip_id]['name'] or '')
                                    ws2.write(prospect_id,5,zip_id and dZips[zip_id]['city'] or '')
                                    ws2.write(prospect_id,6,prospect.language_id and prospect.language_id.name or '')
                                    ws2.write(prospect_id,7,prospect.level_id and prospect.level_id.name or '')
                                    ws2.write(prospect_id,8,prospect.state_id and prospect.state_id.name or '')
                                    prospect_id += 1
                                group_id += 1
                                prospect_id += 1 # to create an empty line between groups in ws2
                        else:
                            current_size -= 1
        #print group_id
        # 5. manage the isolated
        ws3.write(0,0,_('Persons without any groups'))
        ws3.write(1,0,_('Name'))
        ws3.write(1,1,_('First Name'))
        ws3.write(1,2,_('Company'))
        ws3.write(1,3,_('Zip code'))
        ws3.write(1,4,_('City'))
        ws3.write(1,5,_('Level'))
        ws3.write(1,6,_('Language'))
        line3 = 2
        for part in parts:
            ingroup = False
            for (id,group) in dGroups:
                if part.id in group['part_ids']:
                    ingroup = True
                    break
            if not ingroup:
                zip_id = self.dParts.has_key(part.id) and self.dParts[part.id]['job_zip_id'] or False
                ws3.write(line3,0,part.contact_id.name)
                ws3.write(line3,1,part.contact_id.first_name or '')
                ws3.write(line3,2,part.partner_id.name)
                ws3.write(line3,3,zip_id and dZips[zip_id]['name'] or '')
                ws3.write(line3,4,zip_id and dZips[zip_id]['city'] or '')
                ws3.write(line3,5,part.level_id and part.level_id.name or '')
                ws3.write(line3,6,part.language_id and part.language_id.name or '')
                ws3.write(line3,7,part.state_id and part.state_id.name or '')
                line3 += 1

        # 6. Show the results
        wb.save('groups.xls')
        result_file = open('groups.xls','rb').read()

        # give the result tos the user
        data['form']['msg']=_('Save the File with '".xls"' extension.\n\nPossible detected groups : %i\n\nProblems to checks : %i') % (group_id-1,line4-8)
        data['form']['groups.xls']=base64.encodestring(result_file)
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('search','Search')]},
        },
        'search': {
            'actions': [_search_for_groups],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }
search_groups('search_groups')
