# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2008 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import netsvc
import pooler
from tools.translate import _
from osv import fields, osv

form = """<?xml version="1.0"?>
<form string="Send Email based on Template">
    <field name="template_id" colspan="4" width="650"/>
</form>
"""
fields = {
    'template_id': {'string':'Template','type':'many2one','relation':'cci_email_template','required':True},
}

param_form = """
<form string="Modify Template">
    <field name="email_from" colspan="4" width="650"/>
    <field name="email_to" colspan="4"/>
    <field name="subject" colspan="4"/>
    <field name="body" colspan="4" width="650" height="300"/>
</form>
"""

param_fields = {
  'subject': {'string':'Subject', 'type':'char', 'size':64, 'required':True},
  'email_from': {'string':'Email From', 'type':'char', 'size':200, 'required':True},
  'email_to': {'string':'Email To', 'type':'char', 'size':200, 'help':'If not empty, FORCE the sending to this address(s), not participant address !'},
  'body': {'string':'Body', 'type':'text', 'required':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Result">
    <field name="message" width="650" height="50"/>
</form>
"""
msg_fields = {
      'message': {'string':'Result', 'type':'text', 'readonly':True}
}

def _init_param_form(self,cr,uid,data,context):
    pool = pooler.get_pool(cr.dbname)
    template = pool.get('cci_email_template').browse(cr, uid, [data['form']['template_id']], context)[0]
    current_user = pool.get('res.users').browse(cr, uid, [uid], context)[0]
    data['form']['subject'] = template.subject
    data['form']['email_from'] = current_user.email or 'info@ccilvn.be'
    data['form']['body']= template.body
    return data['form']
    
def _sendemail(self, cr, uid, data, context):
    pool = pooler.get_pool(cr.dbname)
    template_obj = pool.get('cci_email_template')
    template = template_obj.read(cr,uid,[data['form']['template_id']],['type'])[0]
    part_obj = pool.get('cci_club.participation')
    sended_mails = part_obj.send_emails(cr,uid,data['ids'],data['form']['email_from'],data['form']['email_to'],data['form']['subject'],data['form']['body'],template['type'])
    message = _('''%s mails have been successfully sent\n''') % (sended_mails)
    return {'message': message}

class send_email_template(wizard.interface):
    states = {
        'init' : {
           'actions' : [],
           'result': {'type': 'form', 'arch': form, 'fields': fields, 'state':[('end','Cancel'), ('param','Personalize')]}
        },
        'param' : {
           'actions' : [_init_param_form],
           'result': {'type': 'form', 'arch': param_form, 'fields': param_fields, 'state':[('end','Cancel'), ('send','Send')]}
        },
        'send': {
            'actions': [_sendemail],
            'result': {'type':'form', 'arch': msg_form, 'fields': msg_fields, 'state':[('end','Ok')]}
        },
    }
send_email_template("send_email_template")

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
