# -*- encoding: utf-8 -*-

from osv import fields, osv
import wizard
import pooler
import cci_tools
import os
import tempfile
import base64
import struct
import time

_form_init="""<?xml version="1.0"?>
<form title="Traform">
    <separator string="Parameters" colspan="4"/>
    <field name="date_start"/>
    <field name="date_end"/>
    <newline/>
    <field name="operator_code"/>
    <field name="write"/>
    <newline/>
    <field name="year_old"/>
    <field name="year"/>
    <newline/>
</form>
"""
_fields_init = {
    'date_start'    : {
        'string':'Date start',
        'type':'date',
        'required':True,
        'default':'1900-01-01'},
    'date_end'      : {
        'string':'Date end',   
        'type':'date',
        'required':True,
        'default':'2099-12-31'},
    'operator_code' : {
        'string'    : 'Operator',
        'type'      : 'char',
        'required'  : True,},
    'write' : {
        'string'    : 'Write changes',
        'type'      : 'boolean',
        'help'		: 'Marquer les chèques comme encodés dans Traform',
        'required'  : True,},
    'year_old' : {
        'string'    : 'Traform Year Old',
        'type'      : 'integer',
        'help'		: 'Année à récupérer (0 = chèques non encodés dans Traform)',
        'required'  : True,},
    'year' : {
        'string'    : 'Traform Year',
        'type'      : 'integer',
        'help'		: 'Année à encoder dans la base de données',
        'default'   : 2011,
        'required'  : True,}
}

_form_output="""<?xml version="1.0"?>
<form title="Traform">
    <separator string="Parameters" colspan="4"/>
    <field name="date_start" readonly="True"/>
    <field name="date_end" readonly="True"/>
    <newline/>
    <field name="operator_code" readonly="True"/>
    <field name="write" readonly="True"/>
    <newline/>
    <field name="year_old" readonly="True"/>
    <field name="year" readonly="True"/>
    <newline/>
    <separator string="Files" colspan="4"/>
    <field name="traform.xls" readonly="True" colspan="4"/>
    <field name="traform.for" readonly="True" colspan="4"/>
    <newline/>
    <separator string="Log" colspan="4"/>
    <field name="log" readonly="True" colspan="4" nolabel="1"/>
</form>
"""
_fields_output = {
    'traform.xls' : {
        'string':'xls file',
        'type':'binary'},
    'traform.for' : {
        'string':'traform file',
        'type':'binary'},
    'log' : {
        'string':'log',
        'type':'text'},
}

class cci_traform_wizard(wizard.interface):
    
    headers = [
        'Nom',
        'Prénom',
        'Code postal',
        'Localité',
        'Statut',
        'Sexe',
        'Date de naissance',
        'Études',
        'Expérience professionnelle',
        'Nationalité',
        'Registre national',
        'Numéro unique',
        'Entreprise',
        'Code postal entreprise',
        'Localité entreprise',
        'Formation',
        'Nombre d\'heures',
        'Nombre de chèques',
        'Nbre de chèques langue',
        'Opérateur',
        'Prix',
        'Type',
        'participation_id',
        'club_id',
        'contact_id',
        'partner_id',
        'final_date',
        'invoice_data',
    ]    
    
    fse_work_status = {
        'worker'         : 'OUVRIER',
        'employee'       : 'EMPLOYE',
        'manager'        : 'CADRE',
        'independent'    : 'INDEPENDANT',
        'temporary'      : 'INTERIMAIRE',
        'workingmarried' : 'CONJOINT-AIDANT',
    }
    
    fse_studies = {
        'primary'        : 'PRIMAIRE',
        'lowersecondary' : 'SECONDAIRE INF',
        'uppersecondary' : 'SECONDAIRE SUP',
        'non-academic'   : 'SUP NON UNIVERSITAIRE',
        'academic'       : 'UNIVERSITAIRE',
        'prof_exp'       : 'EXPERIENCE PROFESSIONNELLE',
    }
    
    fse_work_experience = {
        'few'      : '<5 ANS',
        'year59'   : '5-9 ANS',
        'year1014' : '10-14 ANS',
        'year1519' : '15-19 ANS',
        'more'     : '20 ANS ET PLUS'
    }
    
    fse_nationality = {
        'belgian'   : 'BELGE',
        'european'  : u'UNION EUROPÉENNE',
        'world'     : u'HORS UNION EUROPÉENNE',
    }    
    
    def _get_checks(self, cr, uid, wizard_content):
        
        # TODO: add a constraint: only the chack that 
        # have a valid traform code ("Langue" ou "Formation")
        
        year_old = wizard_content['form']['year_old']
        if year_old == 0:
			year_old = False
        
        check_constraints = [
            ('date_submission','>=',wizard_content['form']['date_start']),
            ('date_submission','<=',wizard_content['form']['date_end']),
            ('traform_year','=',year_old),
            ('state','in',['asked','paid'])]
            
        check_ids = pooler.get_pool(cr.dbname).get('cci_club.club_check').search(cr,uid,check_constraints)
        
        if len(check_ids) == 0:
            return []
            
        checks = pooler.get_pool(cr.dbname).get('cci_club.club_check').browse(cr,uid,check_ids)
        return checks
    
    def _get_participations_from_checks(self,cr,uid,checks):
        
        participations    = []
        participation_ids = []
        
        for check in checks:
            
            participation_id = check.participation_id.id
            if not participation_id in participation_ids:
                participation_ids.append(participation_id)
                participations.append(check.participation_id)
                
        return participations
                
    
    def _generate_files(self, cr, uid, data, res_get=False):
        
        pool = pooler.get_pool(cr.dbname)
        
        checks = self._get_checks(cr,uid,data)
        participations = self._get_participations_from_checks(cr,uid,checks)

        operator_code = data['form']['operator_code']
        
        contact_ids    = []
        partner_ids    = []
        agreement_nbrs = []
        nbr_check_tot_general = 0
        
        participation_data = []
        for participation in participations:
            
            if not participation.contact_id.id in contact_ids:
                contact_ids.append(participation.contact_id.id)
                
            if not participation.partner_id.id in partner_ids:
                partner_ids.append(participation.partner_id.id)
                
            current_participation_data = []
            
            # Nom
            try:
                current_participation_data.append(participation.contact_id.name)
            except:
                current_participation_data.append(False)
            # Prénom
            try:
                current_participation_data.append(participation.contact_id.first_name)
            except:
                current_participation_data.append(False)
            # Code postal
            try:
                current_participation_data.append(participation.contact_id.fse_zip_id.name)
            except:
                current_participation_data.append(False)
            # Localité
            try:
                current_participation_data.append(participation.contact_id.fse_zip_id.old_city)
            except:
                current_participation_data.append(False)
            # Statut
            try:
                status = participation.contact_id.fse_work_status
                current_participation_data.append(self.fse_work_status[status])
            except:
                current_participation_data.append(False)
            # Sexe
            try:
                sex = participation.contact_id.gender
                if sex:
                    if sex == 'man':
                        sex = 'M'
                    else:
                        sex = 'F'
                current_participation_data.append(sex)
            except:
                current_participation_data.append(False)
            # Date de naissance
            try:
                current_participation_data.append(participation.contact_id.birthdate)
            except:
                current_participation_data.append(False)
            # Études
            try:
                studies = participation.contact_id.fse_studies
                current_participation_data.append(self.fse_studies[studies])
            except:
                current_participation_data.append(False)
            # Expérience professionnelle
            try:
                work_experience = participation.contact_id.fse_work_experience
                current_participation_data.append(self.fse_work_experience[work_experience])
            except:
                current_participation_data.append(False)
            # Nationalité
            try:
                nationality = participation.contact_id.fse_nationality
                current_participation_data.append(self.fse_nationality[nationality])
            except:
                current_participation_data.append(False)
            # Registre national
            try:
                national_number = participation.contact_id.national_number
                current_participation_data.append(participation.contact_id.national_number)
            except:
                current_participation_data.append(False)
            # Numéro unique
            try:
                raw_uniq_number = participation.partner_id.vat[2:]

                uniq_number = "%s.%s.%s" % (raw_uniq_number[0:4],raw_uniq_number[4:7],raw_uniq_number[7:10])
                current_participation_data.append(uniq_number)
            except:
                current_participation_data.append(False)
            # Entreprise
            try:
                current_participation_data.append(participation.partner_id.name)
            except:
                current_participation_data.append(False)
            # Code postal entreprise & Localité entreprise
            try:
                zip = False
                city = False
                for job in participation.contact_id.job_ids:
                    if job.name and job.address_id and job.name.id == participation.partner_id.id:
                        zip = job.address_id.zip_id.name
                        city = job.address_id.zip_id.old_city
                current_participation_data.append(zip)
                current_participation_data.append(city)   
            except:
                current_participation_data.append(False)
                current_participation_data.append(False)
            # Formation
            try:
                agreement_nbr = operator_code + "-" + participation.group_id.agreement_nbr
                if len(agreement_nbr) == 14:
                    current_participation_data.append(agreement_nbr)
                    if not agreement_nbr in agreement_nbrs:
                        agreement_nbrs.append(agreement_nbr)
                else:
                    current_participation_data.append(False)
            except:
                current_participation_data.append(False)
            # Nombre d' heures
            nbr_check_langue = 0
            nbr_check_formation = 0
            for check in participation.check_ids:
                if check in checks:
                    if check.type_id.traform_column == 2: # TODO: check the value
                        nbr_check_langue = nbr_check_langue + check.unit_nbr
                    if check.type_id.traform_column == 1: # TODO: check the value
                        nbr_check_formation = nbr_check_formation + check.unit_nbr
            
            nbr_check_tot = nbr_check_langue + nbr_check_formation
            nbr_check_tot_general = nbr_check_tot_general + nbr_check_tot
            
            if nbr_check_tot == 0:
                current_participation_data.append(False)
            else:
                current_participation_data.append(str(nbr_check_tot))

            
            # Nombre de chèques
            current_participation_data.append(str(nbr_check_formation))
            # Nbre de chèques langue
            current_participation_data.append(str(nbr_check_langue))
            # Opérateur
            current_participation_data.append(operator_code)
            # Prix
            try:
                price = participation.final_turnover
                vat = participation.group_id.product_id.product_tmpl_id.taxes_id[0].amount
                current_participation_data.append("%.2f" % ((1+vat)*price))
            except:
                current_participation_data.append(False)
            # Type
            try:
                current_participation_data.append(participation.group_id.traform_type)
            except:
                current_participation_data.append(False)
            
            current_participation_data.append(str(participation.id))
            current_participation_data.append(str(participation.group_id.id))
            current_participation_data.append(str(participation.contact_id.id))
            current_participation_data.append(str(participation.partner_id.id))
            if participation.forced_date_out:
                final_date = participation.forced_date_out
            else:
                final_date = participation.group_id.date_end
            current_participation_data.append(final_date or '')
            current_participation_data.append(participation.invoice_data or '')
            participation_data.append(current_participation_data)
        
        summary = []
        summary.append(['Participations',str(len(participations))])
        summary.append(['Contacts',str(len(contact_ids))])
        summary.append(['Partenaires',str(len(partner_ids))])
        summary.append(['Chèques (total)',str(nbr_check_tot_general)])
        summary.append(['Formations',str(len(agreement_nbrs))])
            
        traform_xls_data = {
            'name'         : 'Résumé',
            'headers'      : self.headers,
            'summary'      : summary,
            'data'         : participation_data,
            'show_missing' : True
        }

        checks_xls_data = {
            'name'         : 'Chèques',
            'headers'      : ['Chèque id','Nom','Prénom','Participation id','Date de soumission','Nombre','Type','Colonne Traform'],
            'summary'      : [],
            'data'         : [],
            'show_missing' : True
        }
        
        check_tot = 0
        check_tot_langue = 0
        check_tot_formation = 0
        for check in checks:
            current_check_data = []
            # Chèque id
            current_check_data.append(str(check.id))
            # Nom
            current_check_data.append(check.participation_id.contact_id.name)
            # Prénom
            current_check_data.append(check.participation_id.contact_id.first_name)
            # Participation id
            current_check_data.append(str(check.participation_id.id))
            # Date de soumission
            current_check_data.append(check.date_submission)
            # Nombre
            unit_nbr = check.unit_nbr
            if unit_nbr:
                check_tot = check_tot + check.unit_nbr
                if check.type_id.traform_column == 2: # TODO: check the value
                    check_tot_langue = check_tot_langue + check.unit_nbr
                if check.type_id.traform_column == 1: # TODO: check the value
                    check_tot_formation = check_tot_formation + check.unit_nbr
                current_check_data.append(str(check.unit_nbr))
            else:
                current_check_data.append(False)
            # Type
            current_check_data.append(check.type_id.name)
            # Traform code
            if check.type_id.traform_column > 0:
                current_check_data.append(str(check.type_id.traform_column))
            else:
                current_check_data.append(False)
            
            checks_xls_data['data'].append(current_check_data)

        summary = []
        summary.append(['Chèques (encodage)',str(len(checks))])
        summary.append(['Chèques (total)',str(check_tot)])
        summary.append(['Chèques (Formation)',str(check_tot_formation)])
        summary.append(['Chèques (Langue)',str(check_tot_langue)])
        checks_xls_data['summary'] = summary

        (errors, traform_file_content) = self.generate_traform_file(traform_xls_data['data'])
        data['traform.for'] = traform_file_content
        
        data['traform.xls'] = cci_tools.generate_xls_file([traform_xls_data,checks_xls_data,errors])
        # TODO: write traform file
        
        # Stats
        stats = ""
        count_errors = 0
        for entry in traform_xls_data['data']:
            if False in entry:
                count_errors = count_errors + 1
                #print entry
        stats = stats + "Erreurs dans les participations : \t" + str(count_errors) + "\n"
        
        count_errors = 0
        for entry in checks_xls_data['data']:
            if False in entry:
                count_errors = count_errors + 1
                #print entry
        stats = stats + "Erreurs dans les chèques : \t\t" + str(count_errors) + "\n"
        
        stats = stats + "Erreurs dans Traform : \t\t\t" + str(len(errors['data'])) + "\n"
     
        data['log'] = stats
        
        if data['form']['write']:
			check_ids = []
			for check in checks_xls_data['data']:
				if not False in check:
					check_ids.append(check[0])
			if len(check_ids) > 0:
				pooler.get_pool(cr.dbname).get('cci_club.club_check').write(cr,uid,check_ids,{'traform_year':data['form']['year']})
			stats = stats + "\n" + "Chèques modifiés : \t\t\t" + str(len(check_ids)) + "\n"
        else:
			pass
			#print "Write inactif"
        
        return data
    
    def generate_traform_file(self,data):
    
        errors = {
            'name'         : 'Erreurs Traform',
            'headers'      : ['Champ','Erreur','Valeur rejetée'],
            'summary'      : [],
            'data'         : [],
            'show_missing' : False
        }    
    
        tmp_filename = tempfile.mkstemp()[1]
        
        data2 = map(lambda x: x[0:10] + x[11:18] + x[19:22] + [time.strftime("%d/%m/%Y %H:%M:%S")] + x[10:11] + x[18:19] ,data)
        
        data3 = data2
        
        #with open(tmp_filename,mode='w') as tmp_traform_file:
        import codecs
        with codecs.open(tmp_filename, mode = 'w', encoding = 'cp1252') as tmp_traform_file:   
            nullbyte = struct.pack('B', 0)
            sizes    = [50,50,10,40,50,2,5,50,50,50,13,50,10,40,40,7,7,50,10,6,20,20,7]
            
            for line in data:
                
                line_ok = line[0:10] + line[11:18] + line[19:22] + [time.strftime("%d/%m/%Y %H:%M:%S")] + line[10:11] + line[18:19]

                has_false = False
                for value in line_ok:
                    if isinstance(value,bool) and not value:
                        errors['data'].append([
                            '*',
                            'Certains champs ne sont pas remplis',
                            'participation id = %s' % line[22]
                        ])
                        has_false = True
                        
                        break
                if has_false:
                    continue
                
                for i in range(0,len(sizes)):
                    
                    size  = sizes[i]
                    name  = self.headers[i]
                    
                    value = line_ok[i]
                    if len(value) > size - 1:
                        errors['data'].append([self.headers[i],'Trop grand (max = %s)' % (size-1),value])
                        value = ''
                    
                    value_ok = value
                    if i != 9:
                        value_ok = cci_tools.strip_accents(value).upper().encode('cp1252')
                    
                    if i in [3,13]:
                        value_ok = value_ok + "\n"
                    #print value_ok
                    
                    tmp_traform_file.write(value_ok)
                    for j in range(len(value_ok),size):
                        tmp_traform_file.write(nullbyte)
        
        errors['summary'] = [['Nombre d\'erreurs',str(len(errors['data']))]]
        
        
        with open(tmp_filename,mode='r') as tmp_traform_file:
            traform_file_content = base64.encodestring(tmp_traform_file.read())
        os.remove(tmp_filename)
        return (errors,traform_file_content)
    
    states = {
        'init' : {
            'actions' : [],
            'result'  : {   'type': 'form',
                            'arch': _form_init,
                            'fields': _fields_init,
                            'state':[('end','Cancel'),('output','Create files')]}},
        'output' : {
            'actions' : [_generate_files],
            'result'  : {   'type': 'form',
                            'arch': _form_output,
                            'fields': dict(_fields_init.items() + _fields_output.items()),
                            'state':[('end','Cancel'),('init','Edit values')]}}
    }
    
cci_traform_wizard('cci.club.extend.wizard.traform')
