# -*- encoding: utf-8 -*-
from osv import fields, osv
import wizard
import pooler
import cci_tools
import os
import tempfile
import base64
import struct
import time
import datetime
from dateutil.relativedelta import relativedelta
from pyExcelerator import *
from tools.translate import _

_form_init="""<?xml version="1.0"?>
<form string="Traform v2.0">
    <separator string="Parameters" colspan="4"/>
    <field name="date_start"/>
    <field name="date_end"/>
</form>
"""
_fields_init = {
    'date_start': {'string':'Date start','type':'date','required':True,'default':'1900-01-01'},
    'date_end': {'string':'Date end','type':'date','required':True,'default':'2099-12-31'},
}

_form_output="""<?xml version="1.0"?>
<form string="Traform v2.0">
    <separator string="Excel File" colspan="4"/>
    <field name="traform2.xls" colspan="4" readonly="1"/>
    <newline/>
    <separator string="Log" colspan="4"/>
    <field name="log" readonly="True" colspan="4" nolabel="1"/>
</form>
"""
_fields_output = {
    'traform2.xls': {'string':'xls file','type':'binary'},
    'log' : {'string':'log','type':'text'},
}

pyexcel_colours = {
        'aqua'    : 0x0F,
        'cyan'    : 0x0F,
        'black'   : 0x08,
        'blue'    : 0x0C,
        'brown'   : 0x10,
        'magenta' : 0x0E,
        'fuchsia' : 0x0E,
        'gray'    : 0x17,
        'grey'    : 0x17,
        'green'   : 0x11,
        'lime'    : 0x0B,
        'navy'    : 0x12,
        'orange'  : 0x35,
        'pink'    : 0x21,
        'purple'  : 0x14,
        'red'     : 0x0A,
        'silver'  : 0x16,
        'white'   : 0x09,
        'yellow'  : 0x0D,
}

class wizard_traform2(wizard.interface):
    
    headers = [
        'Nom',
        'Prénom',
        'Code postal',
        'Localité',
        'Statut',
        'Sexe',
        'Date de naissance',
        'Études',
        'Expérience professionnelle',
        'Nationalité',
        'Registre national',
        'Numéro unique',
        'Entreprise',
        'Code postal entreprise',
        'Localité entreprise',
        'Formation',
        'Nombre d\'heures',
        'Nombre de chèques',
        'Nbre de chèques langue',
        'Opérateur',
        'Prix',
        'Type',
        'participation_id',
        'club_id',
        'contact_id',
        'partner_id',
        'final_date',
        'invoice_data',
    ]    
    
    fse_work_status = {
        'worker'         : 'OUVRIER',
        'employee'       : 'EMPLOYE',
        'manager'        : 'CADRE',
        'independent'    : 'INDEPENDANT',
        'temporary'      : 'INTERIMAIRE',
        'workingmarried' : 'CONJOINT-AIDANT',
    }
    
    fse_studies = {
        'primary'        : 'PRIMAIRE',
        'lowersecondary' : 'SECONDAIRE INF',
        'uppersecondary' : 'SECONDAIRE SUP',
        'non-academic'   : 'SUP NON UNIVERSITAIRE',
        'academic'       : 'UNIVERSITAIRE',
        'prof_exp'       : 'EXPERIENCE PROFESSIONNELLE',
    }
    
    fse_work_experience = {
        'few'      : '<5 ANS',
        'year59'   : '5-9 ANS',
        'year1014' : '10-14 ANS',
        'year1519' : '15-19 ANS',
        'more'     : '20 ANS ET PLUS'
    }
    
    fse_nationality = {
        'belgian'   : 'BELGE',
        'european'  : u'UNION EUROPÉENNE',
        'world'     : u'HORS UNION EUROPÉENNE',
    }
    def _generate_files(self,cr,uid,data,context):
        pool = pooler.get_pool(cr.dbname)

        # check if we have all necessary parameters
        obj_parameter = pool.get('cci_parameters.cci_value')
        param_values = obj_parameter.get_value_from_names(cr,uid,['TRAFORMOperatorCode'])
        if param_values.has_key('TRAFORMOperatorCode'):
            operator_code = param_values['TRAFORMOperatorCode']
                
            # extract the checks between theses two dates : all
            obj_check = pool.get('cci_club.club_check')
            check_ids = obj_check.search(cr,uid,[('date_submission','>=',data['form']['date_start']),('date_submission','<=',data['form']['date_end']),('state','in',['asked','paid'])])
            if len(check_ids) > 0:
                checks = obj_check.browse(cr,uid,check_ids)
            
                dParticipations = {}           
            
                contact_ids    = []
                partner_ids    = []
                agreement_nbrs = []
                nbr_check_tot_general = 0
                errors_found = 0
                check_tot = 0
                check_tot_formation = 0
                check_tot_langue = 0
                                
                wb = Workbook()

                # styles
                pattern_yellow = Pattern()
                pattern_yellow.pattern = 0x01 # SOLID
                pattern_yellow.set_pattern_fore_colour(pyexcel_colours['yellow'])
                #
                pattern_red = Pattern()
                pattern_red.pattern = 0x01 # SOLID
                pattern_red.set_pattern_fore_colour(pyexcel_colours['red'])
                #
                pattern_white = Pattern()
                #pattern_white.pattern = 0x01 # SOLID
                pattern_white.set_pattern_fore_colour(pyexcel_colours['white'])
                #
                pattern_green = Pattern()
                pattern_green.pattern = 0x01 # SOLID
                pattern_green.set_pattern_fore_colour(pyexcel_colours['lime'])
                #
                style_yellow = XFStyle()
                style_yellow.pattern = pattern_yellow
                #
                style_red = XFStyle()
                style_red.font.bold = True
                style_red.pattern = pattern_red
                #
                style_white = XFStyle()
                style_white.pattern = pattern_white
                #
                style_green = XFStyle()
                style_green.pattern = pattern_green

                ws1 = wb.add_sheet(u'Participations')
                ws1.write(0,0,_('Participations:'))
                ws1.write(1,0,_('Contacts:'))
                ws1.write(2,0,_('Partners'))
                ws1.write(3,0,_('Checks count:'))
                ws1.write(4,0,_('Formations'))
                #
                ws1.write(6,0,_('Errors Count'))
                ws1.write(6,1,_('Contact Name'))
                ws1.write(6,2,_('Contact First Name'))
                ws1.write(6,3,_('Contact Zip Code'))
                ws1.write(6,4,_('Contact City'))
                ws1.write(6,5,_('Status'))
                ws1.write(6,6,_('Sex'))
                ws1.write(6,7,_('Birthdate'))
                ws1.write(6,8,_('Studies'))
                ws1.write(6,9,_('Working Experience'))
                ws1.write(6,10,_('Nationality'))
                ws1.write(6,11,_('National Number'))
                ws1.write(6,12,_('Company Number'))
                ws1.write(6,13,_('Company name'))
                ws1.write(6,14,_('Company Zip Code'))
                ws1.write(6,15,_('Company City'))
                ws1.write(6,16,_('Formation'))
                ws1.write(6,17,_('Hours'))
                ws1.write(6,18,_('Checks\' count'))
                ws1.write(6,19,_('Language Checks\' count'))
                ws1.write(6,20,_('Operator'))
                ws1.write(6,21,_('Price'))
                ws1.write(6,22,_('Type'))
                ws1.write(6,23,_('Participation ID'))
                ws1.write(6,24,_('Club ID'))
                ws1.write(6,25,_('Contact ID'))
                ws1.write(6,26,_('Partner ID'))
                ws1.write(6,27,_('Job ID'))
                ws1.write(6,28,_('Final Date'))
                ws1.write(6,29,_('Invoice Data'))
                line1 = 7

                ws2 = wb.add_sheet(u'Chèques')
                ws2.write(0,0,_(u'Chèques (encodage)'))
                ws2.write(1,0,_(u'Chèques (total)'))
                ws2.write(2,0,_(u'Chèques (Formation)'))
                ws2.write(3,0,_(u'Chèques (Langue)'))
                ws2.write(5,0,_('Color Code'))
                ws2.write(6,0,_('Yellow'),style_yellow)
                ws2.write(6,1,_('Already injected'),style_yellow)
                ws2.write(7,0,_('Red'),style_red)
                ws2.write(7,1,_('Error - Injection not possible'),style_red)
                ws2.write(8,0,_('Green'),style_green)
                ws2.write(8,1,_('To inject'),style_green)
                ws2.write(9,0,_('White'),style_white)
                ws2.write(9,1,_('Good but not ready for injection'),style_white)
                
                ws2.write(11,0,_('Check ID'))
                ws2.write(11,1,_('Participation ID'))
                ws2.write(11,2,_('Contact Name'))
                ws2.write(11,3,_('Contact First Name'))
                ws2.write(11,4,_('Contact Zip Code'))
                ws2.write(11,5,_('Contact City'))
                ws2.write(11,6,_('Birthdate'))
                ws2.write(11,7,_('Sex'))
                ws2.write(11,8,_('Nationality'))
                ws2.write(11,9,_('National Number'))
                ws2.write(11,10,_('Status'))
                ws2.write(11,11,_('Working Experience'))
                ws2.write(11,12,_('Studies'))
                ws2.write(11,13,_('Formation'))
                ws2.write(11,14,_('Count'))
                ws2.write(11,15,_('Type'))
                ws2.write(11,16,_('Price'))
                ws2.write(11,17,_('Invoice Data'))
                ws2.write(11,18,_('Final Date'))
                ws2.write(11,19,_('True Submission Date'))
                ws2.write(11,20,_('Company Number'))
                ws2.write(11,21,_('Company name'))
                ws2.write(11,22,_('Company Zip Code'))
                ws2.write(11,23,_('Company City'))
                ws2.write(11,24,_('Check State'))
                ws2.write(11,25,_('Website already injected'))
                ws2.write(11,26,_('Participation problems'))
                ws2.write(11,27,_('Injected today'))
                ws2.write(11,28,_('Type'))
                line2 = 12
            
                ws3 = wb.add_sheet(u'Erreurs')
                ws3.write(0,0,u'Nombre d\'erreurs :')
                ws3.write(2,0,'Participation id')
                ws3.write(2,1,'check id')
                ws3.write(2,2,_('Field'))
                ws3.write(2,3,_('Wrong value'))
                ws3.write(2,4,_('Error description'))
                line3 = 3
                
                for check in checks:
                    if check.participation_id:
                        if not check.participation_id.contact_id.id in contact_ids:
                            contact_ids.append(check.participation_id.contact_id.id)
                        
                        if not check.participation_id.partner_id.id in partner_ids:
                            partner_ids.append(check.participation_id.partner_id.id)
                        if dParticipations.has_key(check.participation_id.id):
                            current_participation_data = dParticipations[check.participation_id.id]
                        else:
                            current_participation_data = {}
                            current_part_errors = 0
                            job_id = False
                            # Nom
                            try:
                                current_participation_data['contact_name'] = check.participation_id.contact_id.name
                            except:
                                current_participation_data['contact_name'] = False
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Contact Name'))
                                ws3.write(line3,4,_('Empty field'))
                                line3 += 1
                            # Prénom
                            try:
                                current_participation_data['contact_first_name'] = check.participation_id.contact_id.first_name
                            except:
                                current_participation_data['contact_first_name'] = False
                            if not current_participation_data['contact_first_name']:
                                errors_found += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Contact First Name'))
                                ws3.write(line3,4,_('Empty field'))
                                line3 += 1
                            # Code postal
                            try:
                                current_participation_data['contact_zip_code'] = check.participation_id.contact_id.fse_zip_id.name
                            except:
                                current_participation_data['contact_zip_code'] = False
                            if not current_participation_data['contact_zip_code']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Contact Zip Code'))
                                ws3.write(line3,4,_('Empty field'))
                                line3 += 1
                            # Localité
                            try:
                                current_participation_data['contact_zip_city'] = check.participation_id.contact_id.fse_zip_id.old_city
                            except:
                                current_participation_data['contact_zip_city'] = False
                            if not current_participation_data['contact_zip_city']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Contact City'))
                                ws3.write(line3,4,_('Empty field'))
                                line3 += 1
                            # Statut
                            try:
                                current_participation_data['work_status'] = self.fse_work_status[ check.participation_id.contact_id.fse_work_status ]
                            except:
                                current_participation_data['work_status'] = False
                            if not current_participation_data['work_status']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Work Status'))
                                ws3.write(line3,4,_('Empty field'))
                                line3 += 1
                            # Sexe
                            try:
                                if check.participation_id.contact_id.gender:
                                    if check.participation_id.contact_id.gender == 'man':
                                        sex = 'M'
                                    else:
                                        sex = 'F'
                                else:
                                    sex = False
                                    current_part_errors += 1
                                    ws3.write(line3,0,check.participation_id.id)
                                    ws3.write(line3,1,check.id)
                                    ws3.write(line3,2,_('Sex'))
                                    ws3.write(line3,4,_('Undefined'))
                                    line3 += 1
                                current_participation_data['sex'] = sex
                            except:
                                current_participation_data['sex'] = False
                            if not current_participation_data['sex']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Sex'))
                                ws3.write(line3,4,_('Empty field'))
                                line3 += 1
                            # Date de naissance
                            try:
                                current_participation_data['birthdate'] = check.participation_id.contact_id.birthdate
                            except:
                                current_participation_data['birthdate'] = False
                            if not current_participation_data['birthdate']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Contact Birthdate'))
                                ws3.write(line3,4,_('Empty field'))
                                line3 += 1
                            # Études
                            try:
                                current_participation_data['studies'] = self.fse_studies[ check.participation_id.contact_id.fse_studies ]
                            except:
                                current_participation_data['studies'] = False
                            if not current_participation_data['studies']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Studies'))
                                ws3.write(line3,4,_('Empty field'))
                                line3 += 1
                            # Expérience professionnelle
                            try:
                                current_participation_data['work_exp'] = self.fse_work_experience[ check.participation_id.contact_id.fse_work_experience ]
                            except:
                                current_participation_data['work_exp'] = False
                            if not current_participation_data['work_exp']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Work Experience'))
                                ws3.write(line3,4,_('Empty field'))
                                line3 += 1
                            # Nationalité
                            try:
                                current_participation_data['nationality'] = self.fse_nationality[ check.participation_id.contact_id.fse_nationality ]
                            except:
                                current_participation_data['nationality'] = False
                            if not current_participation_data['nationality']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Nationality'))
                                ws3.write(line3,4,_('Empty field'))
                                line3 += 1
                            # Registre national
                            try:
                                current_participation_data['national_number'] = check.participation_id.contact_id.national_number
                            except:
                                current_participation_data['national_number'] = False
                            if not current_participation_data['national_number']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('National Number'))
                                ws3.write(line3,4,_('Empty field'))
                                line3 += 1
                            # Numéro unique
                            try:
                                raw_uniq_number = check.participation_id.partner_id.vat[2:]
                                current_participation_data['uniq_number'] = "%s.%s.%s" % (raw_uniq_number[0:4],raw_uniq_number[4:7],raw_uniq_number[7:10])
                            except:
                                current_participation_data['uniq_number'] = False
                            if not current_participation_data['uniq_number']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Company Number'))
                                ws3.write(line3,4,_('Empty field'))
                                line3 += 1
                            # Entreprise
                            try:
                                current_participation_data['partner_name'] = check.participation_id.partner_id.name
                            except:
                                current_participation_data['partner_name'] = False
                            if not current_participation_data['partner_name']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Company Name'))
                                ws3.write(line3,4,_('Empty field'))
                                line3 += 1
                            # Code postal entreprise & Localité entreprise
                            try:
                                pzip = False
                                pcity = False
                                for job in check.participation_id.contact_id.job_ids:
                                    if job.address_id and job.address_id.partner_id and job.address_id.partner_id.id == check.participation_id.partner_id.id:
                                        pzip = job.address_id.zip_id.name
                                        pcity = job.address_id.zip_id.old_city
                                        job_id = job.id
                                current_participation_data['partner_zip_code'] = pzip
                                current_participation_data['partner_zip_city'] = pcity   
                            except:
                                current_participation_data['partner_zip_code'] = False
                                current_participation_data['partner_zip_city'] = False
                            if not current_participation_data['partner_zip_code']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Company Zip Code'))
                                ws3.write(line3,4,_('Empty field'))
                            if not current_participation_data['partner_zip_city']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Company City'))
                                ws3.write(line3,4,_('Empty field'))
                            # Formation
                            try:
                                agreement_nbr = operator_code + "-" + check.participation_id.group_id.agreement_nbr
                                if len(agreement_nbr) == 14:
                                    current_participation_data['agreement_nbr'] = agreement_nbr
                                    if not agreement_nbr in agreement_nbrs:
                                        agreement_nbrs.append(agreement_nbr)
                                else:
                                    current_participation_data['agreement_nbr'] = False
                            except:
                                current_participation_data['agreement_nbr'] = False
                            if not current_participation_data['agreement_nbr']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Formation Agreement number'))
                                ws3.write(line3,4,_('Empty field'))
                                line3 += 1
                            current_participation_data['nbr_check_formation'] = 0
                            current_participation_data['nbr_check_langue'] = 0
                            # Opérateur
                            current_participation_data['operator_code'] = operator_code
                            # Prix
                            try:
                                price = check.participation_id.final_turnover
                                vat = check.participation_id.group_id.product_id.product_tmpl_id.taxes_id[0].amount
                                current_participation_data['price'] = "%.2f" % ((1+vat)*price)
                            except:
                                current_participation_data['price'] = False
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Price'))
                                ws3.write(line3,4,_('Problem with the price'))
                                line3 += 1
                            # Type
                            try:
                                current_participation_data['traform_type'] = check.participation_id.group_id.traform_type
                            except:
                                current_participation_data['traform_type'] = False
                            if not current_participation_data['traform_type']:
                                current_part_errors += 1
                                ws3.write(line3,0,check.participation_id.id)
                                ws3.write(line3,1,check.id)
                                ws3.write(line3,2,_('Type TRAFORM'))
                                ws3.write(line3,4,_('Undefined'))
                                line3 += 1
                            current_participation_data['participation_id'] = check.participation_id.id
                            current_participation_data['club_id'] = check.participation_id.group_id.id
                            current_participation_data['contact_id'] = check.participation_id.contact_id.id
                            current_participation_data['partner_id'] = check.participation_id.partner_id.id
                            current_participation_data['job_id'] = job_id
                            if check.participation_id.forced_date_out:
                                part_final_date = check.participation_id.forced_date_out
                            else:
                                part_final_date = check.participation_id.group_id.date_end
                            current_participation_data['final_date'] = part_final_date or False
                            current_participation_data['invoice_data'] = check.participation_id.invoice_data or False
                            current_participation_data['current_part_errors'] = current_part_errors
                            errors_found += current_part_errors
                            # End of initialisation of current_participation_data if new one
                            
                        # Nombre d'heures
                        if check.type_id.traform_column == 2: # TODO: check the value
                            current_participation_data['nbr_check_langue'] += (check.unit_nbr or 0)
                        if check.type_id.traform_column == 1: # TODO: check the value
                            current_participation_data['nbr_check_formation'] += (check.unit_nbr or 0)
                        # nbr heures = sum (nbr_check_langue + nbr_check_formation)
                                            
                        # TO DO if this check has unit_br <= 0 or False => error
                        #nbr_check_tot = nbr_check_langue + nbr_check_formation
                        #nbr_check_tot_general = nbr_check_tot_general + nbr_check_tot
                        # 
                        #if nbr_check_tot == 0:
                        #    current_participation_data.append(False)
                        #else:
                        #    current_participation_data.append(str(nbr_check_tot))

                    
                        dParticipations[check.participation_id.id] = current_participation_data

                        # Enregistrement du chèque AVANT son état pour bien montrer ce qui est encodé, ce qui peut l'être et ce qui ne peut pas
                        current_step = u'Etape 1'
                        if check.step_id:
                            current_step = check.step_id.name
                        if current_step == u'Etape 1':
                            check_final_date = check.participation_id._get_invoice_first_date(check.participation_id.id)
                            if check_final_date:
                                d_check_final_date = datetime.date(int(check_final_date[0:4]),int(check_final_date[5:7]),int(check_final_date[8:10]))
                                d_true_check_final_date = d_check_final_date + relativedelta(months=5)
                                while int(d_true_check_final_date.strftime('%m')) in [7,8]:
                                    d_true_check_final_date = d_true_check_final_date + relativedelta(months=1)
                                check_final_date = d_true_check_final_date.strftime('%Y-%m-%d')
                        else:
                            check_final_date = check.participation_id.date_out
                        current_style = style_white
                        if check.traform_year > 0:
                            current_style = style_yellow
                        elif current_participation_data['current_part_errors'] > 0:
                            current_style = style_red
                        elif check.state == 'paid' or check.state == 'asked':
                            if (check_final_date >= data['form']['date_start']) and (check_final_date <= data['form']['date_end']):
                                current_style = style_green
                        ws2.write(line2,0,check.id,current_style)
                        ws2.write(line2,1,check.participation_id.id,current_style)
                        ws2.write(line2,2,current_participation_data['contact_name'] or '',current_style)
                        ws2.write(line2,3,current_participation_data['contact_first_name'] or '',current_style)
                        ws2.write(line2,4,current_participation_data['contact_zip_code'] or '',current_style)
                        ws2.write(line2,5,current_participation_data['contact_zip_city'] or '',current_style)
                        ws2.write(line2,6,current_participation_data['birthdate'] or '',current_style)
                        ws2.write(line2,7,current_participation_data['sex'] or '',current_style)
                        ws2.write(line2,8,current_participation_data['nationality'] or '',current_style)
                        ws2.write(line2,9,current_participation_data['national_number'] or '',current_style)
                        ws2.write(line2,10,current_participation_data['work_status'] or '',current_style)
                        ws2.write(line2,11,current_participation_data['work_exp'] or '',current_style)
                        ws2.write(line2,12,current_participation_data['studies'] or '',current_style)
                        ws2.write(line2,13,current_participation_data['agreement_nbr'] or '',current_style)
                        ws2.write(line2,14,check.unit_nbr,current_style)
                        ws2.write(line2,15,check.type_id.name,current_style)
                        ws2.write(line2,16,current_participation_data['price'] or '',current_style)
                        ws2.write(line2,17,current_participation_data['invoice_data'] or '',current_style)
                        ws2.write(line2,18,check_final_date or '',current_style)
                        ws2.write(line2,19,check.date_submission,current_style)
                        ws2.write(line2,20,current_participation_data['uniq_number'] or '',current_style)
                        ws2.write(line2,21,current_participation_data['partner_name'] or '',current_style)
                        ws2.write(line2,22,current_participation_data['partner_zip_code'] or '',current_style)
                        ws2.write(line2,23,current_participation_data['partner_zip_city'] or '',current_style)
                        ws2.write(line2,24,check.state,current_style)
                        ws2.write(line2,25,check.traform_year > 0 and check.traform_year or '',current_style)
                        ws2.write(line2,26,current_participation_data['current_part_errors'],current_style)
                        ws2.write(line2,27,'',current_style)
                        ws2.write(line2,28,current_participation_data['traform_type'] or '',current_style)
                        line2 += 1
                for part_data in dParticipations.values():
                    if part_data['current_part_errors'] > 0:
                        ws1.write(line1,0,part_data['current_part_errors'],style_red)
                    if part_data['contact_name']:
                        ws1.write(line1,1,part_data['contact_name'])
                    else:
                        ws1.write(line1,1,'INVALID',style_red)
                    if part_data['contact_first_name']:
                        ws1.write(line1,2,part_data['contact_first_name'])
                    else:
                        ws1.write(line1,2,'INVALID',style_red)
                    if part_data['contact_zip_code']:
                        ws1.write(line1,3,part_data['contact_zip_code'])
                    else:
                        ws1.write(line1,3,'INVALID',style_red)
                    if part_data['contact_zip_city']:
                        ws1.write(line1,4,part_data['contact_zip_city'])
                    else:
                        ws1.write(line1,4,'INVALID',style_red)
                    if part_data['work_status']:
                        ws1.write(line1,5,part_data['work_status'])
                    else:
                        ws1.write(line1,5,'INVALID',style_red)
                    if part_data['sex']:
                        ws1.write(line1,6,part_data['sex'])
                    else:
                        ws1.write(line1,6,'INVALID',style_red)
                    if part_data['birthdate']:
                        ws1.write(line1,7,part_data['birthdate'])
                    else:
                        ws1.write(line1,7,'INVALID',style_red)
                    if part_data['studies']:
                        ws1.write(line1,8,part_data['studies'])
                    else:
                        ws1.write(line1,8,'INVALID',style_red)
                    if part_data['work_exp']:
                        ws1.write(line1,9,part_data['work_exp'])
                    else:
                        ws1.write(line1,9,'INVALID',style_red)
                    if part_data['nationality']:
                        ws1.write(line1,10,part_data['nationality'])
                    else:
                        ws1.write(line1,10,'INVALID',style_red)
                    if part_data['national_number']:
                        ws1.write(line1,11,part_data['national_number'])
                    else:
                        ws1.write(line1,11,'INVALID',style_red)
                    if part_data['uniq_number']:
                        ws1.write(line1,12,part_data['uniq_number'])
                    else:
                        ws1.write(line1,12,'INVALID')
                    if part_data['partner_name']:
                        ws1.write(line1,13,part_data['partner_name'])
                    else:
                        ws1.write(line1,13,'INVALID',style_red)
                    if part_data['partner_zip_code']:
                        ws1.write(line1,14,part_data['partner_zip_code'])
                    else:
                        ws1.write(line1,14,'INVALID',style_red)
                    if part_data['partner_zip_city']:
                        ws1.write(line1,15,part_data['partner_zip_city'])
                    else:
                        ws1.write(line1,15,'INVALID',style_red)
                    if part_data['agreement_nbr']:
                        ws1.write(line1,16,part_data['agreement_nbr'])
                    else:
                        ws1.write(line1,16,'INVALID',style_red)
                    ws1.write(line1,17,part_data['nbr_check_formation']+part_data['nbr_check_langue'])
                    ws1.write(line1,18,part_data['nbr_check_formation'])
                    ws1.write(line1,19,part_data['nbr_check_langue'])
                    ws1.write(line1,20,part_data['operator_code'])
                    if part_data['price']:
                        ws1.write(line1,21,part_data['price'])
                    else:
                        ws1.write(line1,21,'INVALID',style_red)
                    if part_data['traform_type']:
                        ws1.write(line1,22,part_data['traform_type'])
                    else:
                        ws1.write(line1,22,'INVALID',style_red)
                    ws1.write(line1,23,part_data['participation_id'])
                    ws1.write(line1,24,part_data['club_id'])
                    ws1.write(line1,25,part_data['contact_id'])
                    ws1.write(line1,26,part_data['partner_id'])
                    if part_data['job_id']:
                        ws1.write(line1,27,part_data['job_id'])
                    else:
                        ws1.write(line1,27,'INVALID - NO MATCH FOUND',style_red)
                    if part_data['final_date']:
                        ws1.write(line1,28,part_data['final_date'])
                    else:
                        ws1.write(line1,28,'INVALID',style_red)
                    if part_data['invoice_data']:
                        ws1.write(line1,29,part_data['invoice_data'])
                    else:
                        ws1.write(line1,29,'INVALID',style_red)
                    line1 += 1
                ws1.write(0,1,len(dParticipations.keys()))
                ws1.write(1,1,len(contact_ids))
                ws1.write(2,1,len(partner_ids))
                ws1.write(3,1,nbr_check_tot_general)
                ws1.write(4,1,len(agreement_nbrs))

                ws2.write(0,1,len(checks))
                ws2.write(1,1,check_tot)
                ws2.write(2,1,check_tot_formation)
                ws2.write(3,1,check_tot_langue)

                # Stats
                stats = ""
                stats = stats + "Erreurs dans les participations : \t" + str(errors_found) + "\n"
                
                count_errors = 0
                stats = stats + "Erreurs dans les chèques : \t\t" + str(count_errors) + "\n"
                
                #stats = stats + "Erreurs dans Traform : \t\t\t" + str(len(errors['data'])) + "\n"
             
                data['form']['log'] = stats
                
                #if data['form']['write']:
			    #    check_ids = []
			    #    for check in checks_xls_data['data']:
				#        if not False in check:
				#	        check_ids.append(check[0])
			    #    if len(check_ids) > 0:
				#        pooler.get_pool(cr.dbname).get('cci_club.club_check').write(cr,uid,check_ids,{'traform_year':data['form']['year']})
			    #    stats = stats + "\n" + "Chèques modifiés : \t\t\t" + str(len(check_ids)) + "\n"
                #else:
			    #    pass
			    #    #print "Write inactif"
                wb.save('traform2.xls')
                result_file = open('traform2.xls','rb').read()
                data['form']['msg']=_('File created')
                data['form']['traform2.xls']=base64.encodestring(result_file)
            else:
                data['form']['log'] = 'No found checks'
        else:
            data['form']['log'] = 'Some parameters lack'
        return data['form']
        
    
    states = {
        'init' : {
            'actions' : [],
            'result'  : {'type': 'form',
                         'arch': _form_init,
                         'fields': _fields_init,
                         'state':[('end','Cancel'),('output','Create files')]}},
        'output' : {
            'actions' : [_generate_files],
            'result'  : {'type': 'form',
                         'arch': _form_output,
                         'fields': _fields_output,
                         'state':[('end','Cancel'),]}}
    }
    
wizard_traform2('wizard_traform2')
