# -*- encoding: utf-8 -*-

from osv import fields, osv
import wizard
import pooler
import os
import tools
import re
import base64
import csv
import StringIO

form = """<?xml version="1.0"?>
<form string="Send Template from CSV records">
     <field name="addr_from" colspan="4"/>
     <field name="addrs_to" colspan="4"/>
     <field name="bcc" colspan="4"/>
     <!--<field name="record"/>-->
     <newline/>
     <separator string="CSV File" colspan="4"/>
     <field name="sep_char"/>
     <field name="codepage"/>
     <field name="input_csv" colspan="4"/>
</form>"""

fields = {
    'addr_from': {'string':'From','type':'char','size':200,'required':True},
    'addrs_to': {'string':'To','type':'char','size':200,'required':True},
    'bcc': {'string':'BCC','type':'char','size':200},
    #'record':{'string':'Record sendings','type':'boolean','default':True},
    'sep_char':{'string':'Separator','type':'char','size':1,'default':';'},
    'codepage':{'string':'CodePage','type':'char','size':10,'default':'cp1252'},
    'input_csv': {'type':'binary','string':'CSV file','required':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Results">
     <field name="msg" colspan="4" nolabel="1" width="600"/>
</form>"""

msg_fields = {
    'msg': {'string':'Finish', 'type':'text','readonly':True},
}

def _convert_tags(contents,fields,row):
    results = []
    for content in contents:
        for field in fields:
            content = content.replace("{PERSO %s}" % field, row[field] or '')
        results.append(content)
    return results
def _is_valid_email(email_address):
    result = True
    if not re.match(r"^[A-Za-z0-9\.\+_-]+@[A-Za-z0-9\._-]+\.[a-zA-Z]*$", email_address):
        result = False
    return result
def _is_valid_from(email_from_address):
    return _is_valid_email(email_from_address)
def _is_valid_to(list_of_email_to_address):
    result = True
    if type(list_of_email_to_address) is type([]):
        for x in list_of_email_to_address:
            if not _is_valid_email(x):
                result = False
    else:
        result = False
    return result
def _is_valid_bcc(list_of_email_bcc_address):
    if list_of_email_bcc_address:
        return _is_valid_to(list_of_email_bcc_address)
    else:
        return True ## An empty list if also correct
class send_template_with_csv(wizard.interface):
    def _send(self,cr,uid,data,context):
        obj_template = pooler.get_pool(cr.dbname).get('cci_email_template')
        template = obj_template.browse(cr,uid,data['id'])
        
        # A. Get all dynamic fields from template, 'from' address, 'to' address
        tags = re.compile('{PERSO .*?}',re.IGNORECASE)
        dynamic_fields = []
        dynamic_contents = [template.body or '',template.subject or '',data['form']['addr_from'] or '',data['form']['addrs_to'] or '',data['form']['bcc'] or '']
        for content in dynamic_contents:
            for tag in tags.findall(content):
                tag_name = str(tag)
                dyn_name = tag_name[7:-1]
                if dyn_name not in dynamic_fields:
                    dynamic_fields.append(dyn_name)
        
        # B. get all field names from CSV
        inputfile = data['form']['input_csv']
        inputdata = base64.decodestring(inputfile)
        inputdata = inputdata.decode(data['form']['codepage']).encode('ascii', 'xmlcharrefreplace')
        f = StringIO.StringIO(inputdata)
        reader = csv.DictReader(f,delimiter=data['form']['sep_char'],quotechar='"')
        csv_field_names = reader.fieldnames
        
        fields_not_found = []
        for field_name in dynamic_fields:
            if field_name not in csv_field_names:
                fields_not_found.append(field_name)
        if not fields_not_found:
            # C. If all dynamic fields in fields names, then launch the sending
            # C.1. Get the environment
            parameter_obj = pooler.get_pool(cr.dbname).get('cci_parameters.cci_value')
            param_values = parameter_obj.get_value_from_names(cr,uid,['BDDeTest'])
            bd_de_test = False
            if param_values.has_key('BDDeTest'):
                bd_de_test = param_values['BDDeTest']
            sended = 0
            from_errors = []
            to_errors = []
            bcc_errors = []
            empty_errors = []
            sending_errors = []
            current_line = 2
            for row in reader:
                (perso_body,perso_subject,perso_from,perso_to,perso_bcc) = _convert_tags(dynamic_contents,dynamic_fields,row)
                perso_from = perso_from.strip()
                perso_to = perso_to.strip()
                perso_bcc = perso_bcc.strip()
                perso_subject = perso_subject.strip()

                if ',' in perso_to:
                    l_perso_to = perso_to.split(',')
                elif ';' in perso_to:
                    l_perso_to = perso_to.split(';')
                else:
                    l_perso_to = [perso_to,]
                l_perso_to = [x.strip() for x in l_perso_to]

                if ',' in perso_bcc:
                    l_perso_bcc = perso_bcc.split(',')
                elif ';' in perso_bcc:
                    l_perso_bcc = perso_bcc.split(';')
                elif not perso_bcc:
                    l_perso_bcc = []
                else:
                    l_perso_bcc = [perso_bcc,]
                l_perso_bcc = [x.strip() for x in l_perso_bcc]
                if _is_valid_from(perso_from):
                    if _is_valid_to(l_perso_to):
                        if _is_valid_bcc(l_perso_bcc):
                            if perso_body and perso_subject:
                                if bd_de_test:
                                    print '--------------------------'
                                    print 'Type: ' + template.type
                                    print 'From: ' + perso_from
                                    print 'To : ' + ','.join(l_perso_to)
                                    print 'Bcc: ' + ','.join(l_perso_bcc)
                                    print 'Subject: ' + perso_subject
                                    print 'Body: ' + perso_body
                                    sended += 1
                                else:
                                    try:
                                        tools.email_send(perso_from, l_perso_to, perso_subject, perso_body, subtype=template.type, email_bcc=l_perso_bcc)
                                    except Exception, e:
                                        sending_errors.append((current_line,str(e)))
                                    else:
                                        sended += 1
                            else:
                                empty_errors.append((current_line,perso_subject or '--none--',perso_body or '--none--'))
                        else:
                            bcc_errors.append((current_line,perso_bcc or '--none--'))
                    else:
                        to_errors.append((current_line,perso_to or '--none--'))
                else:
                    from_errors.append((current_line,perso_from or '--none--'))
                current_line += 1
            data['form']['msg'] = '%i sended mail(s)' % sended
            if from_errors:
                data['form']['msg'] += "\n\nErrors in 'From' addresses :"
                for (line,addr) in from_errors:
                    data['form']['msg'] += "\n[line %i] %s" % (line,addr)
            if to_errors:
                data['form']['msg'] += "\n\nErrors in 'To' addresses :"
                for (line,addr) in to_errors:
                    data['form']['msg'] += "\n[line %i] %s" % (line,addr)
            if bcc_errors:
                data['form']['msg'] += "\n\nErrors in 'BCC' addresses :"
                for (line,addr) in bcc_errors:
                    data['form']['msg'] += "\n[line %i] %s" % (line,addr)
            if empty_errors:
                data['form']['msg'] += "\n\nErrors because empty subject or empty body :"
                for (line,addr) in empty_errors:
                    data['form']['msg'] += "\n[line %i]\n  Subject : %s\n  Body :%s" % (line,subject,body)
            if sending_errors:
                data['form']['msg'] += "\n\nErrors while sending :"
                for (line,error) in sending_errors:
                    data['form']['msg'] += "\n[line %i] %s" % (line,error)
        else:
            data['form']['msg'] = 'The mails are NOT SENDED, because the template has fields not found in the CSV file : %s.' % ','.join('"' + x + '"' for x in fields_not_found)
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('send','Send Personalised Emails')]},
        },
        'send': {
            'actions' : [_send],
            'result' : {'type' : 'form','arch': msg_form,'fields' : msg_fields,'state' : [('end','Close')],}
        }
    }
send_template_with_csv('send_template_with_csv')
