# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
# Version 1.0 Philmer
# this script extract counting of different companies AND differents people for a list of events
import wizard
import time
import datetime
import tools
import pooler
import base64
from pyExcelerator import *
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Options">
    <field name="date_from"/>
    <field name="date_to"/>
    <field name="exclude_not_done" colspan="4"/>
    <field name="type_ids" colspan="4" width="600"/>
</form>"""

fields = {
    'date_from': {'string': 'From', 'type':'date','required': True},
    'date_to': {'string': 'To', 'type':'date','required': True},
    'type_ids': {'string': 'Types', 'type':'many2many','relation': 'event.type'},
    'exclude_not_done': {'string':'Exclude Canceled and Draft Events','type':'boolean'},
   }

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="count_differents.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'count_differents.xls':{'string': 'Prepared file','type': 'binary','readonly': True,},
}

def french_date(date_as_english_string):
    # '1997-05-02' -> '02/05/1997'
    return date_as_english_string[8:10]+'/'+date_as_english_string[5:7]+'/'+date_as_english_string[0:4]
    
class wizard_count_differents(wizard.interface):

    def _get_file(self,cr,uid,data,context):
        obj_event = pooler.get_pool(cr.dbname).get('event.event')
        obj_reg = pooler.get_pool(cr.dbname).get('event.registration')
        obj_type = pooler.get_pool(cr.dbname).get('event.type')
        wb1 = Workbook()
        ws1 = wb1.add_sheet(_('General'))
        ws1.write(0,0,_('Count of Different Companies and People'))
        ws1.write(2,0,_('Types of Events:'))
        ws1.write(3,0,_('From:'))
        ws1.write(4,0,_('To:'))
        ws1.write(5,0,_('Exclude Draft and Canceled:'))
        ws1.write(6,0,_('Average by Company:'))
        ws1.write(7,0,_('Average by Contact:'))
        ws1.write(9,0,'event_id')
        ws1.write(9,1,_('Event Name'))
        ws1.write(9,2,_('Even Date'))
        ws1.write(9,3,_('Registrations'))
        ws1_line = 10

        from_date = data['form']['date_from']
        to_date = data['form']['date_to']
        if from_date > to_date:
            (from_date,to_date) = (to_date,from_date)
        ws1.write(3,1,french_date(from_date))
        ws1.write(4,1,french_date(to_date))
        ws1.write(5,1,data['form']['exclude_not_done'] and _('YES') or _('No'))

        type_ids = data['form']['type_ids'][0][2]
        if type_ids:
            # output the names of the chosen types in the excel file
            types = obj_type.read(cr,uid,type_ids,['name'])
            type_names = [x['name'] for x in types]
            ws1.write(2,1,','.join(type_names))
            
            # search for the company to exclmud it from the count
            excluded_partner_ids = []
            obj_company = pooler.get_pool(cr.dbname).get('res.company')
            this_company_ids = obj_company.search(cr,uid,[])
            if this_company_ids:
                this_companies = obj_company.browse(cr,uid,this_company_ids)
                for company in this_companies:
                    if company.partner_id and company.partner_id.id not in excluded_partner_ids:
                        excluded_partner_ids.append(company.partner_id.id)
            # extract all events between the two given dates
            select_sentence = [('date_begin','>=',from_date),('date_begin','<=',to_date),('type','in',type_ids)]
            if data['form']['exclude_not_done']:
                select_sentence.append(('state','not in',['draft','cancel']))
            event_ids = obj_event.search(cr,uid,select_sentence)
            event_names = []
            if event_ids:
                events = obj_event.browse(cr,uid,event_ids)
                dCompanies = {}
                dContacts = {}
                for event in events:
                    ws1.write(ws1_line,0,event.id)
                    ws1.write(ws1_line,1,event.name)
                    ws1.write(ws1_line,2,french_date(event.date_begin))
                    reg_ids = obj_reg.search(cr,uid,[('event_id','=',event.id),('state','!=','cancel'),('partner_id','not in',excluded_partner_ids)])
                    ws1.write(ws1_line,3,len(reg_ids))
                    ws1_line += 1
                    regs = obj_reg.read(cr,uid,reg_ids,['partner_id','contact_id'])
                    for reg in regs:
                        if reg['partner_id']:
                            if dCompanies.has_key(reg['partner_id'][0]):
                                dCompanies[reg['partner_id'][0]] = dCompanies[reg['partner_id'][0]] + 1
                            else:
                                dCompanies[reg['partner_id'][0]] = 1
                        if reg['contact_id']:
                            if dContacts.has_key(reg['contact_id'][0]):
                                dContacts[reg['contact_id'][0]] = dContacts[reg['contact_id'][0]] + 1
                            else:
                                dContacts[reg['contact_id'][0]] = 1
                ws2 = wb1.add_sheet(_('By Company'))
                ws2.write(0,0,'partner_id')
                ws2.write(0,1,_('Partner Name'))
                ws2.write(0,2,_('Membership'))
                ws2.write(0,3,_('Salesman'))
                ws2.write(0,4,_('Count'))
                ws2_line = 1
                obj_partner = pooler.get_pool(cr.dbname).get('res.partner')
                company_count = len(dCompanies.keys())
                company_sum = 0
                for key,count in dCompanies.items():
                    partner = False
                    partner = obj_partner.read(cr,uid,key,['name','membership_state','user_id'])
                    ws2.write(ws2_line,0,key)
                    ws2.write(ws2_line,1,partner and partner['name'] or '-')
                    ws2.write(ws2_line,2,partner and partner['membership_state'] or '-')
                    ws2.write(ws2_line,3,(partner and partner['user_id']) and partner['user_id'][1] or '-')
                    ws2.write(ws2_line,4,count)
                    ws2_line += 1
                    company_sum += count
                ws1.write(6,1,'%.3f (%i/%i)' % (round(float(company_sum)/float(company_count),3), company_sum, company_count))
                
                contact_count = len(dContacts.keys())
                contact_sum = 0
                obj_contact = pooler.get_pool(cr.dbname).get('res.partner.contact')
                ws3 = wb1.add_sheet(_('By Contact'))
                ws3.write(0,0,'contact_id')
                ws3.write(0,1,_('Contact Name'))
                ws3.write(0,2,_('Count'))
                ws3_line = 1
                for key,count in dContacts.items():
                    contact = False
                    contact = obj_contact.read(cr,uid,key,['name','first_name'])
                    contact_name = False
                    if contact:
                        contact_name = contact['name'].strip()
                        if contact['first_name']:
                            contact_name += (' '+contact['first_name'])
                    ws3.write(ws3_line,0,key)
                    ws3.write(ws3_line,1,contact_name or '-')
                    ws3.write(ws3_line,2,count)
                    ws3_line += 1
                    contact_sum += count
                ws1.write(7,1,'%.3f (%i/%i)' % (round(float(contact_sum)/float(contact_count),3), contact_sum, contact_count))
                data['form']['msg'] = _('Save the File with '".xls"' extension.')
            else:
                data['form']['msg'] = _('No events of theses types between these two dates.')
        else:
            data['form']['msg'] = _('No types of events given.\nNo results possible.')
        wb1.save('count_differents.xls')
        result_file = open('count_differents.xls','rb').read()
        data['form']['count_differents.xls']=base64.encodestring(result_file)
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('getfile','Get Excel File')]},
        },
        'getfile': {
            'actions': [_get_file],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }

wizard_count_differents('wizard_count_differents')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

