# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
# Version 1.0 Philmer
# this script extract counting of inscriptions done by a certain user (the web site) for events between two dates
import wizard
import time
import datetime
import tools
import pooler
import base64
from pyExcelerator import *

form = """<?xml version="1.0"?>
<form string="Options">
    <field name="from"/>
    <field name="to"/>
    <field name="user_id" colspan="4" width="600"/>
</form>"""

fields = {
    'from': {'string': 'From', 'type':'date','required': True},
    'to': {'string': 'To', 'type':'date','required': True},
    'user_id': {'string': 'Site User', 'type':'many2one','relation': 'res.users','required': True},
   }

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="site_counting.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'site_counting.xls':{'string': 'Prepared file',
        'type': 'binary',
        'readonly': True,},
}

class wizard_count_site_inscriptions(wizard.interface):

    def _get_file(self,cr,uid,data,context):
        obj_event = pooler.get_pool(cr.dbname).get('event.event')
        obj_registration = pooler.get_pool(cr.dbname).get('event.registration')

        wb1 = Workbook()
        ws1 = wb1.add_sheet('Count of Site Registrations by Event')
        ws1.write(0,0,'event_id')
        ws1.write(0,1,'event_name')
        ws1.write(0,2,'count_site')
        ws1.write(0,3,'count_total')
        line = 1

        # extract all events between the two given dates
        event_ids = obj_event.search(cr,uid,[('date_begin','>=',data['form']['from']),('date_begin','<=',data['form']['to'])])
        if event_ids:
            events = obj_event.browse(cr,uid,event_ids)
            sentence = """SELECT count(event_id), event_id
                            FROM event_registration as reg, crm_case as crm 
                            WHERE reg.event_id in (%s) 
                                AND (crm.state = 'open' OR crm.state = 'done')
                                AND reg.create_uid = %s
                                AND reg.case_id = crm.id
                            GROUP BY event_id""" % ( ','.join([str(x) for x in event_ids]),
                                                                str(data['form']['user_id']) )
            cr.execute(sentence)
            count_site = {}
            count_total = {}
            for event_id in event_ids:
                count_site[event_id] = 0
                count_total[event_id] = 0
            res = cr.fetchall()
            for count in res:
                count_site[count[1]] = count[0]
            sentence = """SELECT count(event_id), event_id
                            FROM event_registration as reg, crm_case as crm 
                            WHERE reg.event_id in (%s) 
                                AND (crm.state = 'open' OR crm.state = 'done')
                                AND reg.case_id = crm.id
                            GROUP BY event_id""" % ( ','.join([str(x) for x in event_ids]))
            cr.execute(sentence)
            res = cr.fetchall()
            for count in res:
                count_total[count[1]] = count[0]
            for event in events:
                ws1.write(line,0,event.id)
                ws1.write(line,1,event.name or 'inconnu' )
                ws1.write(line,2,count_site[event.id])
                ws1.write(line,3,count_total[event.id])
                line += 1
            wb1.save('site_counting.xls')
            result_file = open('site_counting.xls','rb').read()

            # give the result to the user
            data['form']['msg']='Save the File with '".xls"' extension.'
            data['form']['site_counting.xls']=base64.encodestring(result_file)
        else:
            wb1.save('site_counting.xls')
            result_file = open('site_counting.xls','rb').read()
            data['form']['msg']='No events between these two dates.'
            data['form']['site_counting.xls']=base64.encodestring(result_file)
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('getfile','Get Excel File')]},
        },
        'getfile': {
            'actions': [_get_file],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }

wizard_count_site_inscriptions('cci_event_extend.extract_site_count')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

