# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2008 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from osv import fields,osv
from osv import orm

class cci_event_proxy_registration(osv.osv):
    _name = 'cci_event_proxy.registration'
    _description='cci_event_proxy_registration'
    _columns={
            "event_id":fields.many2one('event.event','Event'),
            "site_id":fields.integer('Event ID on WebSite'),
            "event_name":fields.char('Event Name',size=120),
            "first_name": fields.char('First Name',size=120),
            "last_name": fields.char('Last Name',size=120),
            "company_name":fields.char('Company Name',size=120),
            "vat":fields.char('VAT/Company ID',size=50),
            "comments":fields.text("User Comments"),
            "price_wo_vat":fields.float("Price wo VAT",digits=(15,2)),
            "price_w_vat":fields.float("Price with VAT",digits=(15,2)),
            "state":fields.selection([('draft','Draft'),('confirm','Confirmed'),('done','Done'),('cancel','Canceled')], 'Status'),
            "email":fields.char('Email',size=200),
            "street":fields.char('Street',size=120),
            "zip_code":fields.char('Zip Code',size=10),
            "zip_city":fields.char('Zip City',size=60),
            "phone":fields.char('Phone',size=40),
            "courtesy":fields.char('Courtesy',size=30),
            "title":fields.char('Title',size=240),
            "mobile":fields.char('Mobile',size=60),
            "login":fields.char('Login Premium',size=100),
            "motpasse":fields.char('Password Premium',size=100),
            "partner_id":fields.many2one('res.partner','Linked Partner'),
            "contact_id":fields.many2one('res.partner.contact','Linked Contact'),
            "active":fields.boolean('Active'),
            "registration_id":fields.many2one('event.registration','Registration'),
    }
    _defaults = {
        'active': lambda *a: True,
        'state': lambda *a: 'draft',
    }
    
    def _search_partner_on_vat_or_name(self,cr,uid,vat,company_name,context=None):
        partner_id = False
        correct_vat = False
        if vat:
            vat = vat.replace('.','').replace('-','').replace(' ','').replace('/','').upper()
            if len(vat) > 2:
                if vat[0:2] == 'BE':
                    if len(vat) == 12:
                        # on a un numéro bien formaté
                        correct_vat = vat
                    else:
                        if len(vat) == 11:
                            # on suppose que c'est l'erreur classique : le user a indiqué BE et le numéro de TVA à 9 chiffres
                            correct_vat = vat[0:2] + '0' + vat[2:]
                        else:
                            # on est en terrain inconnu => on n'a pas de vat valable pour faire une recherche
                            correct_vat = False
                else:
                    # on vérifie si on a 9 chiffres et seulement cela
                    if len(vat) == 9 and vat.isdigit():
                        correct_vat = 'BE0' + vat
                    elif len(vat) == 10 and vat.isdigit():
                        correct_vat = 'BE' + vat
                    elif vat[0:2] in ['FR','NL','LU']:
                        # on va essayer comme cela
                        correct_vat = vat
                    else:
                        correct_vat = False
        if correct_vat:
            partner_ids = self.pool.get('res.partner').search(cr,uid,[('vat','=',correct_vat)])
            if partner_ids and len(partner_ids) == 1:
                partner_id = partner_ids[0]
        if not partner_id:
            # The search on vat is unsuccessfull, we'll try on name
            partner_ids = self.pool.get('res.partner').search(cr,uid,[('name','=',company_name.strip().upper())])
            if partner_ids and len(partner_ids) == 1:
                partner_id = partner_ids[0]
        return partner_id
    def _search_contact(self,cr,uid,partner_id,last_name,first_name,login,motpasse,context=None):
        contact_id = False
        if login and motpasse:
            contact_ids = self.pool.get('res.partner').search(cr,uid,[('premium_login','=',login),('premium_mp','=',motpasse)])
            if contact_ids and len(ontact_ids) == 1:
                contact_id = contact_ids[0]
        if not contact_id and last_name:
            partner = self.pool.get('res.partner').browse(cr,uid,partner_id)
            last_name = last_name.upper()
            first_name = (first_name or '').upper()
            if first_name:
                # if first_name, we search on exact comparison on name and first_name
                if partner.address:
                    for addr in partner.address:
                        if addr.job_ids:
                            for job in addr.job_ids:
                                if job.contact_id and job.contact_id.name.strip().upper() == last_name.strip().upper() and job.contact_id.first_name.strip().upper() == first_name.strip().upper():
                                    contact_id = job.contact_id.id
                                    break
                        if contact_id:
                            break
            else:
                # no first name => we search if we have one and only one contact with given name
                contact_ids = []
                if partner.address:
                    for addr in partner.address:
                        if addr.job_ids:
                            for job in addr.job_ids:
                                if job.contact_id and job.contact_id.name.strip().upper() == last_name.strip().upper():
                                    contact_ids.append( job.contact_id.id )
                if contact_ids and len(contact_ids) == 1:
                    contact_id = contact_ids[0]
        return contact_id
    def create(self,cr,uid,vals,context=None):
        #if vals.has_key('vat') and vals['vat']:
        #    partner_id = self._search_partner_on_vat_or_name(cr,uid,vals['vat'],vals['company_name'],context)
        #    if partner_id:
        #        print 'partner found'
        #        vals['partner_id'] = partner_id
        #        contact_id = self._search_contact(cr,uid,partner_id,vals['last_name'],vals['first_name'],context)
        #        if contact_id:
        #            vals['contact_id'] = contact_id
        #        # we search if we must apply member or non-member price following partner_id
        #        #if vals['event_id']:
        #        #    print 'event_id'
        #        #    print vals['event_id']
        #        #    partner = self.pool.get('res.partner').browse(cr,uid,[partner_id])[0]
        #        #    event = self.pool.get('event.event').browse(cr,uid,[vals['event_id']])[0]
        #        #    price = 0.0
        #        #    if event.product_id:
        #        #        context.update({'partner_id':partner.id})
        #        #        context.update({'pricelist':partner.property_product_pricelist.id})
        #        #        price = self.pool.get('product.product')._product_price(cr, uid, [event.product_id.id], False, False, context )[event.product_id.id]
        #        #        print 'price'
        #        #        print price
        #        #    vals['price_wovat'] = price ## TODO check it, not change it
        #print "create vals"
        #print vals
        if vals['site_id'] and not vals.get('event_name',False):
            event_ids = self.pool.get('event.event').search(cr,uid,[('site_id','=',vals['site_id'])])
            if event_ids:
                event = self.pool.get('event.event').read(cr,uid,event_ids,['name'])[0]
                vals['event_name'] = event['name']
        return super(osv.osv,self).create(cr, uid, vals, context)
    def write(self, cr, uid, ids,vals,context=None):
        # TODO actionner les recherches si les valeurs ont changés et que partner_id et/ou contact_id sont vides
        #current_value = self.pool.get('cci_event_proxy.registration').browse(cr,uid,ids)[0]
        #print 'partner_id'
        #print current_value.partner_id.id
        #print "vals['partner_id']"
        #if vals.get('partner_id',False):
        #    print vals['partner_id']
        #else:
        #    print 'no value'
        #print "write vals"
        #print vals
        #print 'get'
        #print vals.get('partner_id',False)
        #if current_value.partner_id and current_value.partner_id.id > 0 and not vals.get('partner_id',False):
        #    print u'Manually deleted by user => do nothing'
        #elif vals.get('partner_id',False) and current_value.partner_id and current_value.partner_id.id > 0:
        #    print u"un partenaire a été encodé manuellement => on garde cette valeur"
        #elif not vals.get('partner_id',False) and not current_value.partner_id:
        #    print u"il n'y avait pas de valeur et il n'y en a toujours pas => on recherche"
        #    partner_id = self._search_partner_on_vat_or_name(cr,uid,vals['vat'],vals['company_name'],context)
        #    if partner_id:
        #        pass
        return super(osv.osv,self).write(cr, uid, ids,vals, context)
    def unlink(self, cr, uid, ids, context=None):
        self.write(cr,uid, ids, {'active':False})
        return True
cci_event_proxy_registration()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

