# -*- encoding: utf-8 -*-

####################################################################################################
# Ce fichier a deux fonctions:
#      - Fournir la liste des relations vers des objets des règles d'extraction
#      - Générer les vues correspondantes
####################################################################################################

from osv import fields, osv
import tools
import cci_tools

parameter_object_prefix = 'parameter_object_'
required_suffix = '_required'

def get_fields_objects():
    
    fields_ok = {}
    
    # Integer
    for (field_name,field) in fields_integer.items():
        fields_ok['parameter_' + field_name] = fields.integer(
            field['string'],
        )
    
    # Text
    for (field_name,field) in fields_text.items():
        fields_ok['parameter_' + field_name] = fields.char(
            field['string'],
            size = field['size']
        )

    # Date
    for (field_name,field) in fields_date.items():
        fields_ok['parameter_' + field_name] = fields.date(
            field['string']
        )
        
    # Selection
    for (field_name,field) in fields_selection.items():
        fields_ok['parameter_' + field_name] = fields.selection(
            field['selection'],
            field['string']
        )
    
    # Relations
    for (field_name,field) in fields_relation.items():
        new_name = parameter_object_prefix + field_name
        fields_ok[new_name] = fields.many2one(
            field['relation'],
            field['string'],
            ondelete='set null')
            
    return fields_ok
    
# TODO
def get_fields_objects_names(type_filter='all',suffix=''):
    
    names = []
    
    # Integer
    if type_filter in ['all','integer']:
        names.extend(
            ['parameter_' + x + suffix for x in fields_integer.keys()]
        )
    
    # Text
    if type_filter in ['all','text']:
        names.extend(
            ['parameter_' + x + suffix for x in fields_text.keys()]
        )

    # Date
    if type_filter in ['all','date']:
        ordered_names = ['date_start','date_end']
        names.extend(
            ['parameter_' + x + suffix for x in ordered_names]
        )
        names.extend(
            ['parameter_' + x + suffix for x in fields_date.keys() if x not in ordered_names]
        )
 
    # Selection
    if type_filter in ['all','selection']:
        names.extend(
            ['parameter_' + x + suffix for x in fields_selection.keys()]
        )
    
    # Relations
    if type_filter in ['all','relation']:
        l1 = []
        l2 = []
        for (field_key,field) in fields_relation.items():
            new_name = parameter_object_prefix + field_key + suffix
            if 'answer' in new_name:
                l2.append(new_name)
            else:
                l1.append(new_name)
        names.extend(l1 + l2)
    
    return names

def get_fields_required():
    
    fields_ok = {}
    
    # Integer
    for (field_name,field) in fields_integer.items():
        new_name = 'parameter_' + field_name + required_suffix
        fields_ok[new_name] = fields.boolean(field['string'] + ' requis?') 
    
    # Text
    for (field_name,field) in fields_text.items():
        new_name = 'parameter_' + field_name + required_suffix
        fields_ok[new_name] = fields.boolean(field['string'] + ' requis?')

    # Date
    for (field_name,field) in fields_date.items():
        new_name = 'parameter_' + field_name + required_suffix
        fields_ok[new_name] = fields.boolean(field['string'] + ' requis?')
        
    # Selection
    for (field_name,field) in fields_selection.items():
        new_name = 'parameter_' + field_name + required_suffix
        fields_ok[new_name] = fields.boolean(field['string'] + ' requis?')
    
    # Relations
    for (field_name,field) in fields_relation.items():
        new_name = parameter_object_prefix + field_name + required_suffix
        fields_ok[new_name] = fields.boolean(field['string'] + ' requis?')
       
    return fields_ok

def get_fields_required_related(field_to_follow):
    
    fields_ok = {}
    
    # Integer
    for (field_name,field) in fields_integer.items():
        new_name = 'parameter_' + field_name + required_suffix
        fields_ok[new_name] = fields.related(
            field_to_follow,
            new_name,
            string=field['string'] + ' requis?',
            type='boolean',
            readonly=True)
             
    # Text
    for (field_name,field) in fields_text.items():
        new_name = 'parameter_' + field_name + required_suffix
        fields_ok[new_name] = fields.related(
            field_to_follow,
            new_name,
            string=field['string'] + ' requis?',
            type='boolean',
            readonly=True)

    # Date
    for (field_name,field) in fields_date.items():
        new_name = 'parameter_' + field_name + required_suffix
        fields_ok[new_name] = fields.related(
            field_to_follow,
            new_name,
            string=field['string'] + ' requis?',
            type='boolean',
            readonly=True)
        
    # Selection
    for (field_name,field) in fields_selection.items():
        new_name = 'parameter_' + field_name + required_suffix
        fields_ok[new_name] = fields.related(
            field_to_follow,
            new_name,
            string=field['string'] + ' requis?',
            type='boolean',
            readonly=True)
    
    # Relations
    for (field_name,field) in fields_relation.items():
        new_name = parameter_object_prefix + field_name + required_suffix
        fields_ok[new_name] = fields.related(
            field_to_follow,
            new_name,
            string=field['string'] + ' requis?',
            type='boolean',
            readonly=True)
            
    return fields_ok
  
def get_fields_required_names(type_filter='all'): 
    return get_fields_objects_names(type_filter=type_filter,suffix='_required')

def get_fields_for_wizard():
    
    fields = {}
    
    # Integer
    for (field_name,field) in fields_integer.items():
        fields['parameter_' + field_name] = field
        fields['parameter_' + field_name]['type'] = 'integer'
    
    # Text
    for (field_name,field) in fields_text.items():
        fields['parameter_' + field_name] = field
        fields['parameter_' + field_name]['type'] = 'char'

    # Date
    for (field_name,field) in fields_date.items():
        fields['parameter_' + field_name] = field
        fields['parameter_' + field_name]['type'] = 'date'
        
    # Selection
    for (field_name,field) in fields_selection.items():
        fields['parameter_' + field_name] = field
        fields['parameter_' + field_name]['type'] = 'selection'
    
    # Relational
    for (field_name,field) in fields_relation.items():
        fields['parameter_object_' + field_name] = field
        fields['parameter_object_' + field_name]['type'] = 'many2one'
        
    return fields

################################################################################
# Text integers
################################################################################

fields_integer = {}

fields_integer['integer'] = {
    'string' : 'Parameter integer',
}

################################################################################
# Text fields
################################################################################

fields_text = {}

fields_text['text'] = {
    'string' : 'Parameter text',
    'size'   : 64
}

################################################################################
# Date fields
################################################################################

fields_date = {}

fields_date['date_start'] = {
    'string' : 'Parameter date start',
}
fields_date['date_end'] = {
    'string' : 'Parameter date end',
} 

################################################################################
# Selection fields
################################################################################

fields_selection = {}

fields_selection['membership_state'] = {
    'string' : 'État d\'adhésion',
    'selection': [
        ('yes','Membre (Facturé, Libre, Cotisant)'),
        ('no','Non membre (ni Facuté, ni Libre, ni Cotisant)'),
        ('all','Tous les partenaires'),
        ('none', 'Non Membre'),
        ('canceled','Membre Annulé'),
        ('old','Ancien Membre'),
        ('waiting','Membre en attente'),
        ('invoiced','Membre Facturé'),
        ('free','Membre Libre'),
        ('paid','Membre Cotisant')] } 
    
fields_selection['address_type'] = {
    'string' : 'Type d\'adresse',
    'selection': [
        ('default','Défaut'),
        ('invoice','Facturation'), 
        ('delivery','Livraison'), 
        ('contact','Contact'), 
        ('other','Autre')] }

fields_selection['email_availability'] = {
    'string' : 'Présence des emails',
    'selection': [
        ('professional','Professionnel'),
        ('private','Privé'), 
        ('both','Les deux'), 
        ('professional_only','Seulement professionnel'), 
        ('private_only','Seulement privé'),
        ('none','Aucun')] } 
        
fields_selection['ata_status'] = {
    'string' : 'État du carnet ATA',
    'selection': [
        ('all','Tous les carnets'),
        ('draft','Ouvert'), 
        ('created','Créé'), 
        ('pending','En attente'), 
        ('dispute','En litige'),
        ('correct','Correct'),
        ('closed','Clôturé')] } 
        
fields_selection['certificat_status'] = {
    'string' : 'État du certificat',
    'selection': [
        ('all','Tous les certificats'),
        ('draft','Confirmé'), 
        ('invoiced','Facturé'), 
        ('cancel_customer','Annulé par le client'), 
        ('cancel_cci','Annulé par la CCI')] } 

fields_selection['translation_status'] = {
    'string' : 'État de la traduction',
    'selection': [
        ('all','Toutes les traductions'),
        ('draft','Brouillon'), 
        ('confirmed','Confirmé'), 
        ('invoiced','Facturé'), 
        ('done','Terminé'), 
        ('cancel_cci','Annulé')] } 
        
fields_selection['contact_gender'] = {
    'string' : 'Sexe du contact',
    'selection': [
        ('man','Man'),
        ('women','Women')] } 

################################################################################
# Relational fields
################################################################################

fields_relation = {}

# Questions and answers
for (target,table) in { 'contact':'res.partner.contact',
                        'partner':'res.partner',
                        'address':'res.partner.address',
                        'job'    :'res.partner.job'}.items():
        
    fields_relation['crm_profiling_question_%s' % target] = {
        'relation' : 'crm_profiling.question',
        'string'   : 'Tag (%s - question)' % target,
        'domain'   : [('target','=',table)],
    }
    fields_relation['crm_profiling_answer_%s' % target] = {
        'relation' : 'crm_profiling.answer',
        'string'   : 'Tag (%s - reponse)' % target, 
        'domain'   : [('question_id','!=',False),
                      ('question_id','=', '%scrm_profiling_question_%s'
                            % (parameter_object_prefix,target))],
    }

# Others relational fields
fields_relation['res_users'] = {
    'relation' : 'res.users',
    'string'   : 'Commercial',
}
fields_relation['event_type'] = {
    'relation' : 'event.type',
    'string'     : 'Type d\'event',
}
fields_relation['res_partner_activity'] = {
    'relation' : 'res.partner.activity',
    'string'     : 'Secteur d\'activite',
}
fields_relation['res_partner_category'] = {
    'relation' : 'res.partner.category',
    'string'     : 'Catégorie',
}
fields_relation['res_partner_zip_group'] = {
    'relation' : 'res.partner.zip.group',
    'string'     : 'Groupe de codes postaux',
}
fields_relation['res_partner_title_partner'] = {
    'relation' : 'res.partner.title',
    'string'   : 'Forme juridique',
    'domain'   : [('domain','=','partner')],
}

fields_relation['cci_club_club_type'] = {
    'relation' : 'cci_club.club_type',
    'string'   : 'Type de club',
}

