# -*- encoding: utf-8 -*-

from osv import fields, osv
from mx import DateTime
import tools
import time
import pooler
import re
from tools.translate import _
import thread
import traceback
import datetime
import sys

import cci_tools
import cci_extract_bootstrap

class cci_extract(osv.osv):
    
    _name = "cci.extract"
    _description = "cci.extract"
    _columns = {
        # Informations générales
        'name'        : fields.char('Name',size=128, required=True, select=1),
        'user_id'     : fields.many2one('res.users', 'User', select=1),
        'model'       : fields.boolean("Model", select=2),
        'description' : fields.text("Description"),

        # Résultats
        'save_result' : fields.boolean("Save results"),
        'result_ids'  : fields.one2many('cci.extract.result','extract_id','Results'),
        'requery'       : fields.boolean("Requery"),
        'delay_requery' : fields.integer("Delay requery",
            help="L'extraction sera lancée automatiquement tous ... jours à partir de la date de prochain lancement"),
        'start_requery' : fields.datetime("Start requery date"),

        # Règles
        'rule_ids'    : fields.one2many('cci.extract.rule','extract_id','Rules'),
        'rules_priority_partner' : fields.char('Rules priority (Partner)',size=1024),       
        'rules_priority_address' : fields.char('Rules priority (Address)',size=1024),       
        'rules_priority_job'     : fields.char('Rules priority (Job)',    size=1024),       
        'use_incomplete_partner' : fields.boolean(
            'Use incomplete',
            help="Ajouter aux résultats les partenaires pour lesquels aucune adresse n'a été sélectionnée"),
        'use_incomplete_address' : fields.boolean(
            'Use incomplete',
            help="Ajouter aux résultats les adresses pour lesquels aucun job n'a été sélectionné"),
            
        # Synchronisation
        'synchro1'       : fields.boolean("Synchronisation"),
        'target_name1'   : fields.char('Centre d\'intérêt',size=64),
        'synchro_begin1' : fields.date('Début'),
        'synchro_end1'   : fields.date('Fin'),
        
        # Other fields
        'show' : fields.boolean("Show extraction"),
    }
    _defaults = {
        # Informations générales
        'model'   : lambda *a: False,
        'user_id' : lambda self,cr,uid,context: uid,
        # Résultats
        'save_result'   : lambda *a: True,
        'requery'       : lambda *a: False,
        'delay_requery' : lambda *a: 1,
        'start_requery' : lambda *a: False,
        # Règles
        'use_incomplete_partner' : lambda *a: True,
        'use_incomplete_address' : lambda *a: True,
        # Other fields
        'show' : lambda *a: True,
    }
    _sql_constraints = [('target_name1_uniq','unique(target_name1)', 'Target name must be unique!')]
    
    def copy(self, cr, uid, source_id, values=None, context=None):
        
        source = cci_tools.fast_read(cr,uid,'cci.extract',[source_id],['name'])[0]

        # => On copie vers une extraction personnalisée
        values['model']        = False
        values['save_result'] = True
        values['icentral'] = False
        
        values['requery'] = False
        values['delay_requery'] = False
        values['start_requery'] = False
        
        values['synchro1'] = False
        values['target_name1'] = False
        values['synchro_begin1'] = False
        values['synchro_end1'] = False

        # Il ne faut pas recopier les résultats
        values['result_ids'] = False
        values['user_id']    = uid
        
        # Modifie le nom
        name = source['name']
        values['name'] = name.encode("utf-8") + time.strftime(" (dupliqué le %A, %d %B %Y %H:%M:%S)", time.gmtime())
        
        #TODO: 1 => 51
        return super(cci_extract,self).copy(cr, 1, source_id, values, context)
    
    # Prend en entrée une liste d'ids et renvoie un dictionnaire avec chaque id en clé
    def create_dict_of_ids(self,ids):
        dict = {}
        for id in ids:
            dict[id] = True
        return dict
    
    # Prend en entrée deux dictionnaires et renvoie un dictionnaire contenant les clés
    # présentes dans les deux dictionnaires de départ
    def AND(self,target_model, dict1, dict2):    
        dict = {}
        for key in dict1.keys():
            if dict2.has_key(key):
                dict[key] = True
        return dict
        
    # Prend en entrée deux dictionnaires et renvoie un dictionnaire contenant les clés
    # présentes dans au moins un des dictionnaires de départ        
    def OR(self,target_model, dict1, dict2):
        return dict(dict1.items() + dict2.items())
    
    def NOT(self, cr, uid, target_model, dict1):
        
        tables = {'address':'res_partner_address','job':'res_partner_job','partner':'res_partner'}
        table  = tables[target_model] 
        
        if len(dict1.keys()) > 0:
            request = """
            select id
            from %s
            where id not in (%s) and active = true;""" % (
                table,
                ",".join(map(str,dict1.keys())))
        else:
            request = """
            select id
            from %s where active = true""" % (table)

        cr.execute(request)
        
        final_dict = {}
        for key in cr.fetchall():
            final_dict[key[0]] = True
            
        return final_dict
        
    def PRIORITY(self, cr, uid, target_model, dict1, dict2):
        
        tables = {'address':'res_partner_address','job':'res_partner_job'}
        id_p   = {'address':'partner_id','job':'address_id'}
        
        priority_key = id_p[target_model]
        table = tables[target_model] 

        subrequest = """
            select id, %s as id_group, 1 as priority 
            from %s 
            where id in (%s) 
            UNION
            select id, %s as id_group, 2 as priority 
            from %s 
            where id in (%s)""" % (
            priority_key,
            table,
            ",".join(map(str,[-1] + dict1.keys())),
            priority_key,
            table,
            ",".join(map(str,[-1] + dict2.keys()))) 

        request = """
        select id
        from
            (select id_group, MIN(priority) as priority
             from (%s)a
             group by id_group)b
        natural join
            (%s)c
        order by id_group""" % (subrequest,subrequest)  

        #print request
        cr.execute(request)
        
        final_dict = {}
        count = 0
        for key in cr.fetchall():
            count = count + 1
            final_dict[key[0]] = True
             
        return final_dict
        
    def ONE(self, cr, uid, target_model, dict1):
        
        tables = {'address':'res_partner_address','job':'res_partner_job'}
        id_p   = {'address':'partner_id','job':'address_id'}
        seq_p  = {'address':'sequence_partner','job':'sequence_partner'}
        
        priority_key = id_p[target_model]
        table = tables[target_model] 
        sequence = seq_p[target_model]
          
        request = """
        select MIN(id) as id, %s 
        from
            (select %s, MIN(CASE WHEN %s IS NULL THEN 0 ELSE %s END) as %s
            from %s
            where id in (%s)
            GROUP BY %s)a
        natural join
            (select id, %s, CASE WHEN %s IS NULL THEN 0 ELSE %s END
            from %s
            where id in (%s))b
        group by %s;""" % (
                priority_key,
                priority_key,
                sequence,
                sequence,
                sequence, 
                table,
                ",".join(map(str,[-1] + dict1.keys())),
                priority_key,
                priority_key,
                sequence,
                sequence, 
                table,
                ",".join(map(str,[-1] + dict1.keys())),
                priority_key)   

        #print request
        cr.execute(request)
        
        final_dict = {}
        for key in cr.fetchall():
            final_dict[key[0]] = True
            
        return final_dict
        
    
    def throw_AND_OR_except(self,message):
        title = "Regles de priorite invalides"
        title_encoded = title.encode('utf-8','ignore')
        message_encoded = message.encode('utf-8','ignore')
        raise osv.except_osv(title_encoded,message_encoded)  
        
    def throw_extract_except(self,message):
        title = "Erreur lors de l'extraction"
        title_encoded = title.encode('utf-8','ignore')
        message_encoded = message.encode('utf-8','ignore')
        raise osv.except_osv(title_encoded,message_encoded)  
    
    # Vérifie la validité de certaines valeurs
    #   - Paramètres des règles
    #   - Nom des règles
    #   - Priorités
    def check_values(self, cr, uid, ids, context={}):
        

        # Récupère l'extraction
        extract_id = ids[0]
        extract_fields = [
            'rule_ids',
            'use_incomplete_partner',
            'use_incomplete_address',
            'rules_priority_partner',
            'rules_priority_address',
            'rules_priority_job']
        extract_data   = cci_tools.fast_read(cr,uid,'cci.extract',[extract_id],extract_fields)[0]

        # Récupère les règles
        rule_ids    = extract_data['rule_ids']
        rule_fields = cci_extract_bootstrap.get_fields_objects_names() + [
            'name',
            'code_to_evaluate',
            'target_model',
            'model_id'] + cci_extract_bootstrap.get_fields_required_names()
        rules       = cci_tools.fast_read(cr,uid,'cci.extract.rule',rule_ids,rule_fields)

        # Vérification de l'unicité des noms des règles pour une même extraction
        names = []
        for rule in rules:
            if not re.match("^[a-zA-Z0-9_]+$",rule['name']):
                self.throw_extract_except("Les noms des regles ne peut contenir que des lettres, des chiffres et le signe _")
            if rule['name'] in names:
                self.throw_extract_except("Les noms des regles d'une meme extraction doivent etre differents")
            else:
                names.append(rule['name'])
                
        # Vérifie que tous les paramètres requis ont bien été insérés
        wizard_fields = cci_extract_bootstrap.get_fields_for_wizard()
        for rule in rules:
            for key in rule.keys():
                if '_required' in key:
                    field_name = key[0:len(key)-len('_required')]
                    if rule[key] and not rule[field_name] and not wizard_fields[field_name]['type'] == 'integer':
                        self.throw_extract_except("Il manque au moins une valeur a la regle %s" % rule['name'])
                        
        # Vérifie le contenu des règles
        for rule in rules:
            self.pool.get('cci.extract.rule').check_values(cr,uid,rule)

        # Vérifie que les priorités sont correctement encodées
        and_or_regex = "ids\['[a-zA-Z0-9_]+'\]"
        rules_dict = cci_tools.make_dictionary('name',rules)
        for target_model in ['partner','address','job']:
            # Vérification syntaxique
            and_or = self.transform_AND_OR(target_model,extract_data['rules_priority_' + target_model],extract_data['rules_priority_' + target_model])
            # Extraction des noms des règles
            for key in re.findall(and_or_regex,and_or):
                key = key[len("ids['"):len(key)-len("']")]
                if key not in ['selected_partners','selected_addresses']:
                    # Existence du nom
                    if not rules_dict.has_key(key):
                        self.throw_AND_OR_except("La regle \"%s\" n'existe pas (%s)" % (key,target_model))
                    # Correspondance par rapport au modèle
                    elif rules_dict[key]['target_model'] != target_model:
                        self.throw_AND_OR_except("La regle \"%s\" ne correspond pas au modele (%s)" % (key,target_model))

        # Vérifie qu'il n'y a pas de ONE ou PRIORITY demandée sur le partenaire
        regex = "ONE|PRIORITY"
        if re.search(regex,extract_data['rules_priority_partner'] or ''):
            self.throw_AND_OR_except("Les regles pour le partenaire ne peuvent pas contenir de ONE ou de PRIORITY")
   

    

    # Transforme une priorité en du code éxécutable 
    def transform_AND_OR(self,target_model,expression, full_expression):
        
        if expression == False or expression == "":
            return "{}"
        elif re.match('^[a-zA-Z0-9_]+$',expression):
            return "ids['" + expression + "']"
        elif re.match("^[a-zA-Z0-9_\(\),_]+$",expression) and expression.startswith(('ONE(','NOT(','OR(','AND(','PRIORITY(')) and expression.endswith(')'):
            prefix = ''
            prefix_rep = ''
            if expression.startswith('AND('):
                prefix = 'AND('
                prefix_rep = 'AND('
            elif expression.startswith('OR('):
                prefix = 'OR('
                prefix_rep = 'OR('
            elif expression.startswith('NOT('):
                prefix = 'NOT('
                prefix_rep = 'NOT(cr,uid,'
            elif expression.startswith('ONE('):
                prefix = 'ONE('
                prefix_rep = 'ONE(cr,uid,'
            else:
                prefix = 'PRIORITY('
                prefix_rep = 'PRIORITY(cr,uid,'
                
            level = 0
            start_i = len(prefix)
            i = start_i
            exp1  = ''
            # Premier paramètre
            while i < len(expression)-1 and (expression[i] != ',' or level > 0):
                exp1 = exp1 + expression[i]
                if expression[i] == '(':
                    level = level + 1
                if expression[i] == ')':
                    level = level - 1
                i = i + 1
            exp2 = ''
            i = i + 1
            # Second paramètre
            while i < len(expression)-1:
                exp2 = exp2 + expression[i]
                i = i + 1
            if level == 0 and exp1 != '' and not (exp2 != '' and prefix in ['NOT(','ONE(']):
                if exp2 != '':
                    return "self." + prefix_rep + '"' + target_model+'",' + self.transform_AND_OR(target_model,exp1,full_expression) + "," + self.transform_AND_OR(target_model,prefix+exp2+")",full_expression) + ")"
                else:
                    if prefix in ['NOT(','ONE(']:
                        return "self." + prefix_rep + '"' + target_model+'",' + self.transform_AND_OR(target_model,exp1,full_expression) + ")"
                    else:
                        return self.transform_AND_OR(target_model,exp1,full_expression)
        
        self.throw_AND_OR_except("Syntaxe AND, OR invalide : %s" %  full_expression)    

    def run_extract_now(self, cr, uid, ids, context={}):
    
        extract_id = ids[0]
    
        self.check_values(cr,uid,[extract_id],context)
    
        values               = {}
        values['extract_id'] = extract_id
        values['name']       = ''
        values['state']      = 'running'
        
        result_id = self.pool.get('cci.extract.result').create(cr, uid, values)
        cr.commit()
        self.run_extract(cr.dbname, uid, extract_id, result_id, context)
        
    def run_extract_now_in_thread(self, cr, uid, ids, context={}):
    
        extract_id = ids[0]
    
        self.check_values(cr,uid,[extract_id],context)
    
        values               = {}
        values['extract_id'] = extract_id
        values['name']       = ''
        values['state']      = 'running'
        
        result_id = self.pool.get('cci.extract.result').create(cr, uid, values)
        
        thread.start_new_thread(self.run_extract, (cr.dbname, uid, extract_id, result_id, context))
    

    def run_extract(self, dbname, uid, extract_id, result_id, context={}):
        
        log = ''
        
        try:
            cr = pooler.get_db(dbname).cursor()
            #print "step"
            init_time = time.time()
            #print "step"
            log = log + "[0.000000] Début de l'extraction\n"
            #print "step"
            
            self.check_values(cr,uid,[extract_id],context)
            #print "step"
            #print log 
            # Récupère l'extraction
            extract_fields = [
                'name',
                'rule_ids',
                'use_incomplete_partner',
                'use_incomplete_address',
                'rules_priority_partner',
                'rules_priority_address',
                'rules_priority_job',
                'save_result']
            extract_data   = cci_tools.fast_read(cr,uid,'cci.extract',[extract_id],extract_fields)[0]
            
            # Récupère les règles
            rule_ids    = extract_data['rule_ids']
            rule_fields = cci_extract_bootstrap.get_fields_objects_names() + [
                'name',
                'code_to_evaluate',
                'target_model',
                'model_id']
            rules       = cci_tools.fast_read(cr,uid,'cci.extract.rule',rule_ids,rule_fields)
            #print log 
            # Calcule les contraintes
            (dict_ids, log) = self.get_constraints(cr,uid,rules,log)
            
            # Récupération des partenaires
            # Application des AND et des OR
            ids    = dict_ids['partner']
            and_or = self.transform_AND_OR(
                'partner',
                extract_data['rules_priority_partner'],
                extract_data['rules_priority_partner'])
            #print traceback.print_stack()
            exec("partner_dict_ids = " + and_or)
            
            log = log + "[%f] Partenaires trouvés : %d\n" % (time.time()-init_time,len(partner_dict_ids.keys()))

            if len(partner_dict_ids.keys()) == 0:
                pass
                #self.throw_extract_except("Pas de partenaires trouves")
            #print log     
            # Récupération des adresses
            # Application des AND et des OR
            ids    = dict_ids['address']
            if len(partner_dict_ids.keys()) > 0:
                ids['selected_partners'] = self.create_dict_of_ids(cci_tools.search(
                    cr,
                    uid,
                    'res.partner.address',
                    [('partner_id','in',partner_dict_ids.keys())]))
            else:
                ids['selected_partners'] = {}
            and_or = self.transform_AND_OR(
                'address',
                extract_data['rules_priority_address'],
                extract_data['rules_priority_address'])
            #print traceback.print_stack()
            exec("address_dict_ids = " + and_or)

            log = log + "[%f] Adresses trouvées   : %d\n" % (time.time()-init_time,len(address_dict_ids.keys()))
            #print log 
            if len(address_dict_ids.keys()) == 0:
                pass
                #self.throw_extract_except("Pas d'adresses trouvees")
            #print log 
            # Récupération des jobs
            # Application des AND et des OR
            ids    = dict_ids['job']
            if len(address_dict_ids.keys()) > 0:
                ids['selected_addresses'] = self.create_dict_of_ids(cci_tools.search(
                    cr,
                    uid,
                    'res.partner.job',
                    [('address_id','in',address_dict_ids.keys())]))
            else:
                ids['selected_addresses'] = {}
            and_or = self.transform_AND_OR(
                'job',
                extract_data['rules_priority_job'],
                extract_data['rules_priority_job'])
            #print traceback.print_stack()
            exec("job_dict_ids = " + and_or)
            
            log = log + "[%f] Jobs trouvés        : %d\n" % (time.time()-init_time,len(job_dict_ids.keys()))
            #print log
            #print log 
            if len(job_dict_ids.keys()) == 0:
                pass
                #self.throw_extract_except("Pas de jobs trouves")
                
            # Création des tuples
            jobs = cci_tools.fast_read(cr,uid,'res.partner.job',job_dict_ids.keys(),['id','name','address_id','contact_id'])
            log = log + "[%f] Lecture des jobs ok\n" % (time.time()-init_time)
            #print log
            tuples = []
            used_ids = {'partner' : [], 'address' : []}
            has_incomplete_jobs = False
            for job in jobs:
                if not job['name'] or not job['address_id'] or not job['contact_id']:
                    if not has_incomplete_jobs:
                        has_incomplete_jobs = True
                        log = log + "[Warning] => Jobs incomplets : (exclus des résultats)\n"
                    t = ['---','---',job['id'],'---']
                    if job['name']:
                        t[0] = job['name'][0]
                    if job['address_id']:
                        t[1] = job['address_id'][0]
                    if job['contact_id']:
                        t[3] = job['contact_id'][0]
                    log = log + "[Warning]     "
                    log = log + "(partner_id: %s	address_id: %s	job_id: %s	contact_id: %s)\n" % (t[0],t[1],t[2],t[3])
                    pass
                else:
                    tuples.append((job['name'][0],job['address_id'][0],job['id'],job['contact_id'][0]))
                    used_ids['partner'].append(job['name'][0])
                    used_ids['address'].append(job['address_id'][0])         
            

            log = log + "[%f] Jobs complets : %d\n" % (time.time()-init_time,len(tuples))
            log = log + "[%f] Adresses utilisées   : %d\n" % (time.time()-init_time,len(used_ids['address']))
            log = log + "[%f] Partenaires utilisés : %d\n" % (time.time()-init_time,len(used_ids['partner']))
            #print log 
            # Ajout des adresses sans job
            
            #print log 
            if extract_data['use_incomplete_address'] and len(address_dict_ids.keys()) > 0:
                addresses =  cci_tools.fast_read(cr,uid,'res.partner.address',address_dict_ids.keys(),['partner_id'])
                addresses_count = 0
                for address in addresses:
                    if not address['id'] in used_ids['address']:
                        if address['partner_id']:    
                            addresses_count = addresses_count + 1
                            tuples.append((address['partner_id'][0],address['id'],0,0))
                            used_ids['partner'].append(address['partner_id'][0])
                            used_ids['address'].append(address['id'])
                        else:
                            log = log + "[debug] adresse incomplete : " + str(address) + "\n"
                log = log + "[%f] Ajout des adresses sans job : %d\n" % (time.time()-init_time,addresses_count)
                log = log + "[%f] Adresses utilisées   : %d\n" % (time.time()-init_time,len(used_ids['address']))
                log = log + "[%f] Partenaires utilisés : %d\n" % (time.time()-init_time,len(used_ids['partner']))
            else:
                log = log + "[%f] Ajout des adresses sans job : non demandé\n" % (
                    time.time()-init_time)
            #print log 
               

            # Ajout des partenaires sans adresses
            if extract_data['use_incomplete_partner'] and len(partner_dict_ids.keys()) > 0:
                partners =  cci_tools.fast_read(cr,uid,'res.partner',partner_dict_ids.keys(),['id'])
                partners_count = 0
                for partner in partners:
                    if not partner['id'] in used_ids['partner']:
                        partners_count = partners_count + 1
                        tuples.append((partner['id'],0,0,0))
                        used_ids['partner'].append(partner['id'])
                log = log + "[%f] Ajout des partenaires sans adresse : %d\n" % (time.time()-init_time,partners_count)
                log = log + "[%f] Partenaires utilisés : %d\n" % (time.time()-init_time,len(used_ids['partner']))
            else:
                log = log + "[%f] Ajout des partenaires sans adresse : non demandé\n" % (
                    time.time()-init_time)

            not_active_partners = self.create_dict_of_ids(cci_tools.search(
                cr,
                uid,
                'res.partner',[('id','in',used_ids['partner']),('state_id','!=',1)]))
            
            if len(not_active_partners) > 0:
                log = log + "[Warning] => Certains partenaires sélectionnés ne sont plus "
                log = log + "en activité : (présents dans les résultats)\n"
                for key in not_active_partners.keys():
                    for t in tuples:
                        if t[1] == key:
                            log = log + "[Warning]     (partner_id: %s	address_id: %s	job_id: %s	contact_id: %s)\n" % t

            not_delivered_addresses = self.create_dict_of_ids(cci_tools.search(
                cr,
                uid,
                'res.partner.address',[('id','in',used_ids['address']),('notdelivered','!=',False)]))
            
            if len(not_delivered_addresses) > 0:
                log = log + "[Warning] => Certaines adresses ont un retour poste : (présents dans les résultats)\n"
                for key in not_delivered_addresses.keys():
                    for t in tuples:
                        if t[1] == key:
                            log = log + "[Warning]     (partner_id: %s	address_id: %s	job_id: %s	contact_id: %s)\n" % t
            
            log = log + "[%f] Nbr final de lignes  : %s\n" % (time.time()-init_time,len(tuples))
            #print log 
            #print log 
            lines_data = []
            if extract_data['save_result']:
                lines_data = self.pool.get('cci.extract.result').extract_data(cr, uid, 0, tuples)
                log = log + "[%f] Écriture des données : oui\n" % (time.time()-init_time)
            else:
                log = log + "[%f] Écriture des données : non demandé\n" % (time.time()-init_time)
                
            d = time.strftime('%Y-%m-%d')
            t = time.strftime('%H:%M:%S')
            deletion_date = time.strftime(
                "%Y-%m-%d",
                (datetime.datetime.now() + datetime.timedelta(days=8)).timetuple())
            self.pool.get('cci.extract.result').write(cr, uid,[result_id],{
                'line_ids':lines_data,
                'size':len(tuples),
                'state':'done',
                'deletion_date' : deletion_date,
                'execution_date': "%s %s" % (d,t),
                'name': "Extraction n°%s terminée le %s à %s.xls" % (extract_id,d,t)})
            
            
            log = log + "[%f] Fin de l'extraction" % (time.time()-init_time)
        except Exception as ex:
            self.pool.get('cci.extract.result').write(cr, uid,[result_id],{'state':'error'})
            log = log + "[debug] %s\n" % ex
            (exc_type, exc_value, exc_traceback) = sys.exc_info()
            
            emailFrom = 'nepasrepondre@cciconnect.be'
            emailTo   = 'nitr@cciconnect.be'
            subject   = u"CCI extract - Erreur lors d'une extraction"
            
            def escape(text):
                import cgi
                return cgi.escape(text).encode('ascii', 'xmlcharrefreplace')
            
            msgHTML = "<h3>Erreur lors d'une extraction</h3>"
            
            msgHTML = msgHTML + "<table style=\"broder:1pxsolid black;\">"
            msgHTML = msgHTML + "   <tr>"
            msgHTML = msgHTML + "      <td>"
            msgHTML = msgHTML + escape(u"Base de données")
            msgHTML = msgHTML + "      </td>"
            msgHTML = msgHTML + "      <td>"
            msgHTML = msgHTML + cr.dbname
            msgHTML = msgHTML + "      </td>"
            msgHTML = msgHTML + "   </tr>"
            msgHTML = msgHTML + "   <tr>"
            msgHTML = msgHTML + "      <td>"
            msgHTML = msgHTML + escape(u"Utilisateur")
            msgHTML = msgHTML + "      </td>"
            msgHTML = msgHTML + "      <td>"
            msgHTML = msgHTML + str(uid) + " - " + escape(self.pool.get('res.users').read(cr, uid,uid,['name'])['name'])
            msgHTML = msgHTML + "      </td>"
            msgHTML = msgHTML + "   </tr>"
            msgHTML = msgHTML + "   <tr>"
            msgHTML = msgHTML + "      <td>"
            msgHTML = msgHTML + escape(u"Extraction")
            msgHTML = msgHTML + "      </td>"
            msgHTML = msgHTML + "      <td>"
            msgHTML = msgHTML + str(extract_data['id']) + " - " + escape(extract_data['name'])
            msgHTML = msgHTML + "      </td>"
            msgHTML = msgHTML + "   </tr>"
            msgHTML = msgHTML + "   <tr>"
            msgHTML = msgHTML + "      <td>"
            msgHTML = msgHTML + escape(u"Résultat (id)")
            msgHTML = msgHTML + "      </td>"
            msgHTML = msgHTML + "      <td>"
            msgHTML = msgHTML + str(result_id)
            msgHTML = msgHTML + "      </td>"
            msgHTML = msgHTML + "   </tr>"
            msgHTML = msgHTML + "   </table>"      
            
            self.send_mail(emailFrom,emailTo,subject,msgHTML)
            
            #print traceback.print_stack()
            log = log + "[debug] " + str(traceback.format_tb(exc_traceback))
        finally:
            self.pool.get('cci.extract.result').write(cr, uid,[result_id],{'log':log})
            cr.commit()
            cr.close()
        
        return True   

    def send_mail(self,emailFrom,emailTo,subject,messageHTML):
    
        import smtplib
        from email.MIMEMultipart import MIMEMultipart
        from email.MIMEText import MIMEText
    
        smtp = smtplib.SMTP()
        smtp.connect('mail.infomaniak.ch',2525)
        smtp.login('nepasrepondre@cciconnect.be', 'H4yk751E')

        # Create the root message and fill in the from, to, and subject headers
        msgRoot = MIMEMultipart('related')
        msgRoot['Subject'] = subject
        msgRoot['From'] = emailFrom
        msgRoot['To'] = emailTo

        msgAlternative = MIMEMultipart('alternative')
        msgRoot.attach(msgAlternative)
        msg = MIMEText(messageHTML, 'html')
        msgAlternative.attach(msg)

        smtp.sendmail(emailFrom, emailTo, msgRoot.as_string())
        smtp.quit()
    ##
    # Renvoie un dictionnaire contenant les clés 'partner,'address','job'
    # auxquelles sont associées les contraintes à respecter (listes
    # de triplets)
    def get_constraints(self,cr,uid,rules,log):

        cons_to_apply = {'partner': {}, 'address': {}, 'job': {}}
        tables = {'partner': 'res.partner', 'address': 'res.partner.address', 'job': 'res.partner.job'}
        for rule in rules:
            
            parameter = {}
            
            for key in rule.keys():

                if key.startswith('parameter_object_'):
                    new_key = key[len('parameter_object_'):]
                    if isinstance(rule[key],tuple):
                        parameter[new_key] = rule[key][0]
                    else:
                        parameter[new_key] = rule[key]
                        
                elif key.startswith('parameter_'):
                    new_key = key[len('parameter_'):]  
                    parameter[new_key] = rule[key]     
                        
                        
            ids = []
            constraints = []
            
            exec(rule['code_to_evaluate'])

            # Permet de ne générer que les contraintes...
            if len(constraints) > 0:
                ids = ids + cci_tools.search(cr,uid,tables[rule['target_model']],constraints)
                
            cons_to_apply[rule['target_model']][rule['name']] = self.create_dict_of_ids(ids)     
        return (cons_to_apply,log)
    
    # Tâches CRON effectuant les extractions
    def _run_extracts(self, cr, uid, ids=False, context={}):

        # Lancements automatiques
        current_date = time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime())
        extracts = cci_tools.get_records(
            cr,
            uid,
            'cci.extract',
            [('start_requery','<=',current_date),('requery','=',True),('delay_requery','>',0)],
            ['start_requery','delay_requery'])[1]
        extracts_to_write = []
        for extract in extracts: 
            values               = {}
            values['extract_id'] = extract['id']
            values['name']       = ''
            values['scheduled_execution_date'] = extract['start_requery']
            values['creation_mode'] = 'automatic'
            pooler.get_pool(cr.dbname).get('cci.extract.result').create(cr, uid, values)
            
            start_requery = extract['start_requery']
            while(start_requery <= current_date):
                start_requery = time.strftime(
                        "%Y-%m-%d %H:%M:%S",
                        (datetime.datetime.strptime(start_requery, "%Y-%m-%d %H:%M:%S") 
                            + datetime.timedelta(days=extract['delay_requery'])).timetuple())
            pooler.get_pool(cr.dbname).get('cci.extract').write(cr, uid, [extract['id']], 
                {'start_requery' : start_requery})                
        cr.commit()


        results = cci_tools.get_records(cr,uid,'cci.extract.result',[],['id','scheduled_execution_date','extract_id','state','user_id'])[1]
        for result in results:
            if result['scheduled_execution_date'] != False and result['scheduled_execution_date'] <= current_date and result['state'] == 'scheduled':
                try:
                    pass
                    self.run_extract(cr.dbname,result['user_id'][0],result['extract_id'][0],result['id'])
                except Exception as ex:
                    #print ex
                    self.pool.get('cci.extract.result').write(cr, uid,[result['id']],{'state':'error'})
                    #print 'error'
                    pass
                    
        # Synchronisations périmées
        extract_ids = cci_tools.search(
            cr,
            uid,
            'cci.extract',
            [('synchro1','=',True),('synchro_end1','<',current_date[0:10])])
        pooler.get_pool(cr.dbname).get('cci.extract').write(
            cr, 
            uid, 
            extract_ids, 
            {'synchro1' : False})
            
        return True
        
    def on_change_synchro1(self, cr, uid, ids, synchro1):
        values = {}
        if synchro1:
            values['synchro_begin1'] = time.strftime(
                "%Y-%m-%d",
                (datetime.datetime.now() + datetime.timedelta(days=0)).timetuple())
            values['synchro_end1'] = time.strftime(
                "%Y-%m-%d",
                (datetime.datetime.now() + datetime.timedelta(days=7)).timetuple())
        return {'value': values}
     
cci_extract()
