# -*- encoding: utf-8 -*

from osv import fields, osv

import cci_tools
import time
import base64
from pyExcelerator import *
import traceback
import tempfile

import cci_tools

class cci_extract_result(osv.osv):
    
    ##
    # Prend en entrée un dictionnaire et une liste de clés
    # Renvoie la liste des valeurs correspondantes
    ##
    def convert_dictionary_to_list(self,dictionary,keys):
        final_list = []
        for key in keys:
            final_list.append(dictionary[key])
        return final_list
    
    ##
    # Prend en entrée une feuille d'un document xls
    # et une liste d'objets à écrire sur la ligne i
    ##
    def write_line_in_xls(self,worksheet,values,i):
        j = 0
        for value in values:
            if isinstance(value, tuple):
                value = value[1]
            if isinstance(value, bool):
                if value:
                    value = str(value)
                else:
                    value = ''
            worksheet.write(i,j,value)
            j = j + 1
        
    
    def generate_result_as_xls(self,cr,uid,id):
        
        fields = [
            'partner_membership_state',
            'partner_title',
            'partner_name',
            'contact_title',
            'contact_name',
            'contact_first_name',
            'job_function_code_label',
            'job_function_label',
            'address_street',
            'address_street2',
            'address_zip',
            'address_city',
            'address_country',
            'address_type',
            
            'partner_id',
            'address_id',
            'job_id',
            'contact_id',
            
            'job_phone',
            'job_fax',
            'job_email',
            'contact_mobile',
            'contact_email',
            'address_phone',
            'address_fax',
            'address_email',
            
            'partner_vat',
            'partner_user',
            'partner_state',
            'partner_employee_nbr',
            'partner_website',
            'partner_full_name',
            'partner_category',
            
        ]
        
        # Récupération de lignes
        (line_ids,lines) = cci_tools.get_records(cr,uid,'cci.extract.result.line',[('result_id','=',id)],fields)
            
        # Conversion en xls
        wb = Workbook()
        ws = wb.add_sheet('Résultats de l\'extraction')
        # Écriture des headers
        self.write_line_in_xls(ws,fields,0)
        # Écriture des lignes
        i = 1
        for line in lines:
            values = self.convert_dictionary_to_list(line,fields)
            self.write_line_in_xls(ws,values,i)
            i = i + 1
        
        filename = tempfile.mkstemp()[1]
        wb.save(filename)
        return base64.encodestring(open(filename,'rb').read())
    
    def _get_result_as_xls(self, cr, uid, ids, name, args, context=None):
        res = {}
        for result_id in ids:
            res[result_id] = self.generate_result_as_xls(cr,uid,result_id)
            #print 'xls généré (%s)' % len(res[result_id])
        return res

    _name = "cci.extract.result"
    _description = "cci.extract.result"
    _columns = {
        'name'          : fields.char('Name',size=64, required=True),
        'extract_id'    : fields.many2one('cci.extract','Extraction', required=True,ondelete='cascade', select=1),
        'line_ids'      : fields.one2many('cci.extract.result.line','result_id','Result Lines'),
        'state'         : fields.selection([
            ('scheduled','Scheduled'),
            ('error','Error'),
            ('done','Done'),
            ('running','Running')],'State',select=1),
        'creation_mode' : fields.selection([
            ('manual','Manual'),
            ('automatic','Automatic'),
            ('synchro','Synchronisation'),],'Creation mode'),
        'size'          : fields.integer('# Result Lines', select=1),
        'user_id'       : fields.related('extract_id','user_id',type='many2one',relation='res.users',readonly=True),
        'log'           : fields.text('Log'),
        'scheduled_execution_date' : fields.datetime('Scheduled execution date'),
        'execution_date' : fields.datetime('Execution date'),
        'deletion_date' : fields.date('Deletion date'),
        
        'result_as_xls' : fields.function(
            _get_result_as_xls,
            type="binary",
            method=True, 
            string="Download result",
            store=True)
    }
    _defaults = {
        'state': lambda self,cr,uid,context: 'scheduled',
        'size' : lambda self,cr,uid,context: 0,
        'creation_mode' : lambda self,cr,uid,context: 'manual',
    }
 
    ##
    # Prend en entrée une liste de tuples 
    # (partner_id, address_id, job_id, contact_id) 
    # et copie le contenu des objets correspondants
    # dans un résultat
    #
    # Renvoie l'id du résultat créé
    ##
    def extract_data(self, cr, uid, id, tuples):
        
        ids    = [[],[],[],[]]
        fields = [{
            'id':'partner_id',
            'name':'partner_name',
            'title':'partner_title',
            'membership_state':'partner_membership_state',
            'address_type':'partner_address_type',
            'vat':'partner_vat',
            'user_id':'partner_user',
            'state_id':'partner_state',
            'employee_nbr':'partner_employee_nbr',
            'category_id':'partner_category',
            'website':'partner_website',
        },{
            'id':'address_id',
            'name':'address_name',
            'type':'address_type',
            'street':'address_street',
            'street2':'address_street2',
            'zip':'address_zip',
            'city':'address_city',
            'country_id':'address_country',
            'phone':'address_phone',
            'fax':'address_fax',
            'email':'address_email'
        },{
            'id':'job_id',
            'function_label':'job_function_label',
            'function_code_label':'job_function_code_label',
            'phone':'job_phone',
            'fax':'job_fax',
            'email':'job_email'
        },{
            'id':'contact_id',
            'name':'contact_name',
            'first_name':'contact_first_name',
            'title':'contact_title',
            'mobile':'contact_mobile',
            'email':'contact_email',
        }]
        tables = [  'res.partner',
                    'res.partner.address',
                    'res.partner.job',
                    'res.partner.contact']
        
        # Récupération des ids par objet (partenaire,adresse,job,contact)
        for tuple in tuples:
            for i in range(0,len(tuple)):
                ids[i].append(tuple[i])
        
        results = [[],[],[],[]]
        for i in [0,1,2,3]:
            results[i] = cci_tools.make_dictionary(
                'id',cci_tools.read(cr, uid,tables[i],ids[i],['id']+fields[i].keys()))
        
        # Récupération des catégories des partenaires
        try:   
            categories_ok = cci_tools.get_sub_categories_with_depth(
                cr,
                uid,
                cci_tools.search(cr,uid,'res.partner.category',[('name','in',[u"Activity Sector",u"Secteur d'activité"])],{'lang':'en_EN'})[0])
        except:
            raise Exception('La catégorie "Secteur d\'activité" n\'existe pas')
        
        result = []
        for t in tuples:
            record_to_write = {'name':'result'}
            # partner + address + job + contact => 1 record
            for i in [0,1,2,3]:
                record_id = t[i]
                if record_id != 0:
                    record_from_db = results[i].get(record_id)
                    for field_name_in_db in record_from_db.keys():
                        if fields[i].has_key(field_name_in_db):
                            field_name_to_write = fields[i][field_name_in_db]
                            field_content_in_db = record_from_db[field_name_in_db]
                            try:
                                if str(field_content_in_db)[0] == "(":
                                    field_content_in_db = field_content_in_db[1]
                            except:
                                pass
                            record_to_write[field_name_to_write] = field_content_in_db
                for field_name_to_write in fields[i].values():
                    if not record_to_write.has_key(field_name_to_write):
                        record_to_write[field_name_to_write] = False
            # fields that need to be modified
            for field_name in record_to_write.keys():
                if 'phone' in field_name or 'fax' in field_name:
                    record_to_write[field_name] = cci_tools.convert_phone(record_to_write[field_name])
            # additional fields
            # partner_full_name
            if record_to_write['address_name'] == False or record_to_write['address_name'] == '':
                record_to_write['partner_full_name'] = record_to_write['partner_name']
            elif record_to_write['address_name'].startswith("- "):
                record_to_write['partner_full_name'] = record_to_write['partner_name'] + " " + record_to_write['address_name']
            elif record_to_write['address_name'].startswith(" - "):
                record_to_write['partner_full_name'] = record_to_write['partner_name'] + record_to_write['address_name']
            elif record_to_write['address_name'].startswith("-"):
                record_to_write['partner_full_name'] = " " + record_to_write['address_name']
            else:
                record_to_write['partner_full_name'] = record_to_write['address_name']
            # partner_category
            categories_to_write = []
            for category_id in record_to_write['partner_category']:
                if categories_ok.has_key(category_id):
                    categories_to_write.append(categories_ok[category_id]['name'])
            record_to_write['partner_category'] = ",".join(categories_to_write)
            result.append((0,0,record_to_write))
        return result
        
    def _delete_old_results(self, cr, uid, ids=False, context={}):
        
        #print "DEL"
        results = cci_tools.get_records(
            cr,
            uid,
            'cci.extract.result',[],
            ['id','deletion_date'])[1]
        current_date = time.strftime("%Y-%m-%d", time.gmtime())
        
        to_delete = []
        
        for result in results:
            #print result['deletion_date']
            if result['deletion_date'] != False and result['deletion_date'] <= current_date:
                to_delete.append(result['id'])
                
        self.unlink(cr,uid,to_delete,context)
        
        return True        
    """
    # Order: 
    #   1. Running sorted by 'id'
    #   2. Scheduled sorted by 'scheduled_execution_date'
    #   3. Others sorted by 'execution_date'
    def search(self,cr, user, args=[], offset=0, limit=None, order=None, context=None, count=False):
        if order == None:
            new_arg = args + [('state','=','running')]
            ids_running = super(cci_extract_result,self).search(
                cr, user, new_arg, offset, limit, 'id', context, count)
            new_arg = args + [('state','=','scheduled')]
            ids_scheduled = super(cci_extract_result,self).search(
                cr, user, new_arg, offset, limit, 'scheduled_execution_date', context, count)
            new_arg = args + [('state','not in',['running','scheduled'])]
            ids_others = super(cci_extract_result,self).search(
                cr, user, new_arg, offset, limit, 'execution_date', context, count)

            return ids_running + ids_scheduled + ids_others
        else:
            ret = super(cci_extract_result,self).search(cr, user, args, offset, limit, order, context, count)
            return ret
    """
    
    """
    # Order: 
    #   1. Running sorted by 'id'
    #   2. Scheduled sorted by 'scheduled_execution_date'
    #   3. Others sorted by 'execution_date'
    def read(self, cr, user, ids, fields_to_read=None, context=None, load='_classic_read'):

        result = super(cci_extract_result,self).read(cr, user, ids, fields_to_read, context, load)
        result_as_dic = cci_tools.make_dictionary('id',result)
        
        ids_ok = []
        
        args = [('id','in',ids),('state','=','running')]
        ids_running = super(cci_extract_result,self).search(cr, user, args, order='id')
        args = [('id','in',ids),('state','=','scheduled')]
        ids_scheduled = super(cci_extract_result,self).search(cr, user, args, order='scheduled_execution_date')
        args = [('id','in',ids),('state','not in',['running','scheduled'])]
        ids_others = super(cci_extract_result,self).search(cr, user, args, order='execution_date')

        ids_ok = ids_running + ids_scheduled + ids_others
        
        result_ok = []
        for result_id in ids_ok:
            result_ok.append(result_as_dic[result_id])
        
        return result_ok
    """
    
cci_extract_result()
