# -*- encoding: utf-8 -*-

from osv import fields, osv
import time
from tools.translate import _
import re
import pooler
import tools
import string
import copy
import cci_extract_bootstrap
import os
import cci_tools

columns = {   

    'name' : fields.char(
        'Name',
        size=256, 
        required=True, 
        select=1),
    'extract_id' : fields.many2one(
        'cci.extract',
        'Extraction',
        ondelete='cascade', 
        select=1),
    'model_id' : fields.many2one(
        'cci.extract.rule.model',
        'Rule',
        ondelete='cascade', 
        select=1),

    # Fields from the model
    'readonly' : fields.related(
        'extract_id',
        'model',
        readonly=True),
    'code_to_evaluate' : fields.related(
        'model_id',
        'code_to_evaluate',
        readonly=True),
    'target_model' : fields.related(
        'model_id',
        'target_model',
        type='char',
        string='Target model',
        readonly=True),
    'parameter_regex' : fields.related(
        'model_id',
        'parameter_regex',
        type='char',
        string='Parameter Regex',
        readonly=True),
    'parameter_fail_message' : fields.related(
        'model_id',
        'parameter_fail_message',
        type='text',
        string='Parameter Fail Message',
        readonly=True),
    'description' : fields.related(
        'model_id',
        'description',
        type='text',
        string='Description',
        readonly=True),      
}  
columns = dict(
    columns.items() + 
    cci_extract_bootstrap.get_fields_required_related('model_id').items() + 
    cci_extract_bootstrap.get_fields_objects().items() )

class cci_extract_rule(osv.osv):
    
    old_context = {}
    
    def _get_parameter(self, cr, uid, ids, name, args, context=None):
        res = {}
        for result_id in ids:
            res[result_id] = self.generate_parameter(cr,uid,result_id)
        return res
    
    def _default_extract(self, cr, uid, context={}):
        # Il y a perte du contexte à la fin du wizard lors de la création
        # d'une nouvelle règle...
        if context.has_key('extract_id'):
            self.old_context[cr.dbname + ' ' + str(uid)] = context['extract_id']
        return self.old_context.get(cr.dbname + ' ' + str(uid),False)
    
    def _default_model(self, cr, uid, context={}):
        return cci_tools.search(cr,uid,'cci.extract.rule.model',[
            ('name','=',"L'entreprise doit être « En activité »")])[0]
    
    _name = "cci.extract.rule"
    
    _description = "cci.extract.rule"
    
    _columns = dict(columns.items() + {
        'parameter' : fields.function(_get_parameter,type="char",size=128,method=True,string="Parameters")
    }.items())
    
    _defaults = {
        'extract_id'     : lambda self, cr, uid, context={}: self._default_extract(cr,uid,context),
        'model_id'       : lambda self, cr, uid, context={}: self._default_model(cr,uid,context),
    }

    def copy(self, cr, uid, source_id, default=None, context=None):
        return source_id     
    
    def check_values(self,cr,uid,values):
        model = self.extract_model(cr,uid,values['model_id'][0])        
        self.check_parameter_text_match_regex(model,values)

    def extract_model(self,cr,uid,model_id):
        return self.pool.get('cci.extract.rule.model').copy_data(cr, uid, model_id)[0]
    
    def check_parameter_text_match_regex(self,model,values):
        
        #print "************************************************************************************"
        #print model
        #print "************************************************************************************"
        #print model['parameter_text_required']
        
        if model['parameter_text_required'] == False:
            return
        
        parameter = ''
        if values.has_key('parameter_text') and values['parameter_text']:
            parameter = values['parameter_text']
        
        regex = False
        if model['parameter_regex']:
            regex = model['parameter_regex']
        
        #print "PARAMETER %s" % parameter
        #print "REGEX %s" % regex
        
        if model['parameter_text_required'] == 1 and regex and not re.match(regex,parameter):
            #print regex
            #print parameter
            message = values['parameter_text']
            
            message = model['parameter_fail_message'] + "\n\n"
            message_encoded = message.encode('utf-8','ignore')
            
            raise osv.except_osv(_('Validate Error'),message_encoded)

    def generate_parameter(self,cr,uid,id):
        
        fields = cci_extract_bootstrap.get_fields_for_wizard()
        #print fields
        
        field_names = fields.keys() + cci_extract_bootstrap.get_fields_required_names(type_filter='integer')
        
        rule = {}
        rule = cci_tools.read(cr,uid,'cci.extract.rule',[id],field_names)[0]
        ret  = ""
        # cci_extract_bootstrap.get_fields_objects_names() nécessaire pour avoir
        # le bon ordre dans les dates
        for field_name in cci_extract_bootstrap.get_fields_objects_names():
            if rule.has_key(field_name) and (rule[field_name] != False or fields[field_name]['type'] == 'integer'):
            
                if isinstance(rule[field_name],tuple):
                    ret = ret + rule[field_name][1] + " "
                    
                elif fields[field_name]['type'] == 'selection':
                    selection_values = dict(fields[field_name]['selection'])
                    ret = ret + str(selection_values[rule[field_name]]) + " "
                
                elif fields[field_name]['type'] == 'integer':
                    #print rule
                    #print field_name + '_required'
                    import sys
                    sys.stdout.flush()
                    if rule[field_name + '_required']:
                        ret = ret + str(rule[field_name] or 0) + " "
                    
                else:
                    ret = ret + str(rule[field_name]) + " "
        
        if ret == "":
            ret = "/"

        return ret
        
cci_extract_rule() 
