# -*- encoding: utf-8 -*-

import pooler
import tools
import traceback

def search(cr,uid,table,constraints,context={}):
    return pooler.get_pool(cr.dbname).get(table).search(cr, uid,constraints,0,None,None,context)

def read(cr,uid,table,ids,fields,context={}):
    return pooler.get_pool(cr.dbname).get(table).read(cr, uid,ids,fields,context)

def write(cr,uid,table,ids,values,context={}):
    return pooler.get_pool(cr.dbname).get(table).write(cr, uid,ids,values,context)

##
# Renvoie un tuple (record_ids,records) contenant une liste des ids
# des enregistrements respectant les contraintes et une liste de ces
# enregistrements limités aux champs demandés
##
def get_records(cr,uid,table,constraints,fields):
    
    record_ids = pooler.get_pool(cr.dbname).get(table).search(cr, uid,constraints)
    records    = fast_read(cr, uid,table,record_ids,fields)
    
    return (record_ids, records)

def print_dictionary(dictionary,prefix=''):
    
    if tools.config['debug_mode']:
        #print "--------------------------------------------------------"
        for (key, value) in dictionary.iteritems():
            #print "%s%s" % (prefix,key)
            if type(value) is dict:
                print_dictionary(value,prefix + "\t")
            else:
                pass
                #print "%s==> %s" % (prefix,value)
        #print "--------------------------------------------------------"

def fast_read(cr,uid,table,ids,fields,max_ids=1000):
    
    #print "read start on table %s" % table
    
    if not ids:
        return []
        
    ids.sort()
    
    # Division en plusieurs sous requêtes pour accélérer le traitement
    i       = 0
    sub_ids = []
    while max_ids * i < len(ids):
        if max_ids * (i + 1) >= len(ids):
            sub_ids.append(ids[max_ids*i:])
        else:
            sub_ids.append(ids[(max_ids*i):(max_ids*(i+1))])
        i = i + 1
    # Requêtes
    i     = 0
    ids_l = len(ids)
    ids_s = str(ids_l)
    res   = []
    for r_ids in sub_ids:

        i   = i + len(r_ids)
        i_s = str(i)
        while len(i_s) < len(ids_s):
            i_s = " " + i_s
        
        p = str("%.2f" % ((100.0*i)/ids_l))
        while len(p) < 6:
            p = " " + p

        #print "read %s/%s %s%s" % (i_s,ids_s,p,'%')

        x = pooler.get_pool(cr.dbname).get(table).read(cr, uid,r_ids,fields)
        res = res + x

        #print "read %s/%s %s%s ok" % (i_s,ids_s,p,'%')
    
    #print "read end"
    
    return res

def convert_phone(string):
    if string and len(string) > 0:
        if len(string) == 9:
            if string[0:2] in ['02','03','04','09']:
                result = string[0:2] + "/" + string[2:5] + "." + string[5:7] + "." + string[7:]
            else:
                result = string[0:3] + "/" + string[3:5] + "." + string[5:7] + "." + string[7:]
        elif len(string) == 10:
            result = string[0:4] + "/" + string[4:6] + "." + string[6:8] + "." + string[8:]
        else:
            result = string
    else:
        result = ''
    return result

# Convert a list of records into a dictionary
#
# @param index the index used in the dictionary
# @param records the records to put in the dictionary
def make_dictionary(index,records):

    dic = {}
    for record in records:    
    
        key = record[index]
        if type(key) is list:
            key = key[0]
    
        dic[key] = record   
    return dic
    
def get_sub_categories_with_depth(cr,uid,main_categ_id):

    category_ids = search(cr,uid,'res.partner.category',[])
    categories = make_dictionary(
        'id',
        read(cr,uid,'res.partner.category',category_ids,['name','parent_id'],{'lang':'fr_FR'}))
    
    categories_ok = {}
    for category in categories.values():
        current_category = category
        category_id = category['id']
        depth = 0
        while category:
            if category['id'] == main_categ_id:
                current_category['depth'] = depth
                categories_ok[category_id] = current_category
                category = False
            else:
                depth = depth + 1
                if category['parent_id']:
                    category = categories.get(category['parent_id'][0],False)
                else:
                    category = False
    return categories_ok
