# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008 CCI Connect ASBL. (http://www.cciconnect.be) All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import time
import datetime
from osv import fields, osv

class cci_gdpr_address(osv.osv):
    _name = "cci_gdpr.address"
    _description = "Address (email, mobile, ...) to protect"
    _columns = {
        'name' : fields.char('Name',size=250,required=True),
        'addr_type': fields.selection([('email','EMail'),('mobile','Mobile'),('innr','Id Number National Register')],'Type'),
        'create_date': fields.datetime('Creation Time'),
        'create_uid' : fields.many2one('res.users','Creator',help='The user who has created this record'),
        'write_date' : fields.datetime('Last Modification'),
        'write_uid' : fields.many2one('res.users','Last Modifier',help='The last person who has modified this record'),
    }
cci_gdpr_address()

class cci_gdpr_usage(osv.osv):
    _name = "cci_gdpr.usage"
    _description = "Usage on wich the user can give his agreement"
    _columns = {
        'name' : fields.char('Name',size=100,required=True),
        'code': fields.char('Code',size=15,required=True),
        'description': fields.text('Internal description'),
        'user_help':fields.text('Quick help for the user to ,understand the concerned usage'),
        'parent_id' : fields.many2one('cci_gdpr.usage','Parent'),
    }
cci_gdpr_usage()

class cci_gdpr_agreement(osv.osv):
    _name = "cci_gdpr.agreement"
    _description = "Agreement for an address and one usage"
    _columns = {
        'name' : fields.char('Name',size=128,required=True),
        'address_id': fields.many2one('cci_gdpr.address', 'Address', required=True),
        'usage_id': fields.many2one('cci_gdpr.usage', 'Usage', required=True),
        'agreement' : fields.selection([(' ','Undefined'),('Y','Yes'),('N','No')],'Agreement',required=True),
        'active': fields.boolean('Active'),
        'create_date': fields.datetime('Creation Time'),
        'create_uid' : fields.many2one('res.users','Creator',help='The user who has created this record'),
        'write_date' : fields.datetime('Last Modification'),
        'write_uid' : fields.many2one('res.users','Last Modifier',help='The last person who has modified this record'),
        'create_ip': fields.char('IP Address of Creator',size=39), # ipv4 = 15 char, ipv6 = 39 char
    }
    _defaults = {
        'active': lambda *a: True,
        'agreement' : lambda *a: ' ',
    }

    def create(self, cr, uid, vals, *args, **kwargs):
        vals['name'] = '%s - %s = %s' % (self.pool.get('cci_gdpr.address').browse(cr, uid, vals['address_id']).name,
                                         self.pool.get('cci_gdpr.usage').browse(cr, uid, vals['usage_id']).code,
                                         vals['agreement'],
                                        )
        new_id = super(osv.osv,self).create(cr, uid, vals, *args, **kwargs)
        return new_id
cci_gdpr_agreement()

class cci_gdpr_token(osv.osv):
    _name = "cci_gdpr.token"
    _description = "Token for limited access to agreements of a user"
    _columns = {
        'name' : fields.char('Name',size=128,required=True),
        'address_id': fields.many2one('cci_gdpr.address', 'Address', required=True),
        'expiry': fields.datetime('Expiration Time'),
        'active': fields.boolean('Active'),
    }
    _defaults = {
        'active': lambda *a: True,
        'expiry' : lambda *a: datetime.datetime.now() + datetime.timedelta(days=1),
    }
cci_gdpr_token()

