# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import datetime
from osv import fields,osv
import wizard
import pooler
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Change Address">
    <field name="new_address_id" colspan="4"/>
</form>"""

fields = {
    'new_address_id' : {'string':'New Address','type':'many2one','relation':'res.partner.address','required':True},
   }

class wizard_change_inv_addr(wizard.interface):
    def _get_defaults(self, cr, uid, data, context):
        pool_obj = pooler.get_pool(cr.dbname)
        invoice_obj = pool_obj.get('account.invoice')
        current_inv = invoice_obj.browse(cr,uid,data['ids'][0])
        fields['new_address_id']['domain'] = ( "[('partner_id','=',%s)]" % str(current_inv.partner_id.id) )
        return data['form']

    def _change_all_lines(self, cr, uid, data, context):
        lContinue = False
        pool_obj = pooler.get_pool(cr.dbname)
        if data['ids']:
            if len(data['ids']) > 1:
                inv_obj = pool_obj.get('account.invoice')
                invoices = inv_obj.read(cr,uid,data['ids'],['id','partner_id'])
                current_partner_id = invoices[0]['partner_id'][0]
                for inv in invoices[1:]:
                    if inv['partner_id'][0] != current_partner_id:
                        raise osv.except_osv(_('Error'), _('You can change the address of multiple invoices ONLY if ALL invoices concern the same partner !'))
                lContinue = True
            else:
                lContinue = True
        if lContinue:
            # correct the account_invoice
            new_addr_id = data['form']['new_address_id'] or 0
            if new_addr_id > 0:
                inv_obj = pool_obj.get('account.invoice')
                invoices = inv_obj.read(cr,uid,data['ids'],['id','address_invoice_id'])
                old_addr_id = invoices[0]['address_invoice_id'][0]
                addr_obj = pool_obj.get('res.partner.address')
                addr_ids = [new_addr_id,old_addr_id]
                addresses = addr_obj.read(cr,uid,addr_ids,['id','street','street2','zip_id'])
                dAddr = {}
                for addr in addresses:
                    dAddr[addr['id']] = addr
                list_ids = ','.join([str(x) for x in data['ids']])
                #
                correction = "UPDATE account_invoice set address_invoice_id = %s where id in (%s)" % ( str(new_addr_id), list_ids )
                cr.execute( correction )

                # recording of the asked changes in the CCI Logs
                log_obj = pool_obj.get('cci_logs.line')
                log_obj.create(cr,uid,{'name':'Change Invoice Address',
                                       'datetime':datetime.datetime.now(),
                                       'user_id':uid,
                                       'message':'Address changed for invoice : from id [%s] to id [%s] for ids [%s]' 
                                            % ( str(old_addr_id), 
                                                str(new_addr_id), 
                                                list_ids ),
                                      })
        return {'type':'state','state':'end'}
    states = {
        'init': {
            'actions': [_get_defaults],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('change','Change')]},
        },
        'change': {
            'actions': [_change_all_lines],
            'result' : {'type':'state', 'state':'end'}
        },
    }
wizard_change_inv_addr("cci_invoice_extend_change_inv_addr")
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
