# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; withclass department(osv.osv):
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from osv import osv,fields
import datetime
import time
import netsvc

class job_asked(osv.osv):
    ## this object record the data about an opportunity to give a partial job for some days per week
    ## it specifies details about this job to permit the research of similar possibilities
    ## that can combine together to create a full time job usable by an employment alliance
    #
    ## This object is mainly defined in OpenERP to manage the data
    ## but the CRUD operations will be mainly management by an web site
    _name = 'cci_job_alliance.job_asked'
    _description = 'A partial job/profile researched by a partner'
    _columns = {
        'name' : fields.char('Reference', size=30, required=True), ## used to give a reference
        'sector': fields.char('Activity Sector', size=120), ## this field will be filed by values restricted by a list managed by web site
        'profile': fields.char('Profile', size=120), ## this field will be filed by values restricted by a list managed by web site
        'frequency_days': fields.float('Days by week',digits=(2,1)), ## this field give the number of days per week of occupation. Can be partial (0.5)
        'begin_date': fields.date('Begin Date'), 
        'end_date': fields.date('End Date'),
        'recurrent': fields.boolean('Recurrent',help='If begin and end dates, indicates if recurrent over years or not'),
        'partner_id': fields.many2one('res.partner','Partner'),
        'address_id': fields.many2one('res.partner.address','Address'),
        'date': fields.date('Asking Date'),
        'active': fields.boolean('Active'),
    }
    _defaults = {
        'date': lambda *a: time.strftime("%Y-%m-%d %H:%M:%S"),
        'recurrent': lambda *a : False,
        'active': lambda *a : True,
    }
job_asked()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

