# -*- coding: utf-8 -*-
##############################################################################
#    
#    Copyright (c) 2009 CCI  ASBL. (<http://www.ccilconnect.be>).
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
# Redefinition of magazine_subscription_source as fields.selection() better than fields.char()
from osv import fields, osv
import time
import datetime
from tools.translate import _

class subscription_source(osv.osv):
    _name = "cci_magazine.subscription_source"
    _description = "Source of Subscription for the CCI Mag"
    _columns = {
        'name': fields.char('Name', required=True, size=64),
        'code': fields.char('Code', required=True, size=30),
        'to_control': fields.boolean('To control regularly'),
        'last_control': fields.date('Date of the last control'),
        'manager_id': fields.many2one('res.users','Manager'),
        'description': fields.text('Description'),
    }
    _order = 'code'
    _defaults = {
        'to_control': lambda *a: True,
    }

    def write(self, cr, uid, ids,vals,context=None):
        this = self.browse(cr,uid,ids[0])  ## to get the old value
        old = this.code
        new = vals.has_key('code') and vals['code'] or old
        res = super(osv.osv,self).write(cr, uid, ids,vals, context)
        if old != new:
            cr.execute("UPDATE res_partner_job set magazine_subscription_source = '%s' where magazine_subscription_source = '%s'" % (new,old) )
            cr.execute("UPDATE res_partner_address set magazine_subscription_source = '%s' where magazine_subscription_source = '%s'" % (new,old) )
        return res

    def copy(self, cr, uid, id, default={}, context={}):
        this = self.read(cr, uid, [id], ['name','code'])[0]
        old_name = this['name']
        old_code = this['code']
        default.update({'name': old_name+ _(' (copy)'), 'code': old_code + _(' (copy)') })
        return super(osv.osv, self).copy(cr, uid, id, default, context)

    def unlink(self,cr,uid,ids,context):
        for source in self.browse(cr,uid,ids):
            cr.execute("UPDATE res_partner_job set magazine_subscription_source = NULL where magazine_subscription_source = '%s'" % source.code )
            cr.execute("UPDATE res_partner_address set magazine_subscription_source = NULL where magazine_subscription_source = '%s'" % source.code )
        return super(osv.osv,self).unlink(cr,uid,ids,context)
subscription_source()

def _get_subscription_sources(self, cr, uid, context={}):
    obj = self.pool.get('cci_magazine.subscription_source')
    ids = obj.search(cr, uid, [])
    res = obj.read(cr, uid, ids, ['code','name'], context)
    return [(r['code'], r['name']) for r in res] +  [('','')]

class res_partner_address(osv.osv):
    _inherit = "res.partner.address"
    _columns = {
        'magazine_lastprospection':fields.date("Magazine last Sending as Prospection"),
        'magazine_subscription_source':fields.selection(_get_subscription_sources,'Source',size=30),
    }
res_partner_address()

class res_partner_job(osv.osv):
    _inherit = "res.partner.job"
    def _inactivate_old(self, cr, uid, ids=False, context={}):
        # this method makes jobs with pas 'date-stop' inactive
        # it's launched every day by a cron task but can also be launched manually (not important but for testing)
        # all checks are recorded to keep track of theses automatic changes
        today = datetime.date.today().strftime('%Y-%m-%d')
        obj_job = self.pool.get('res.partner.job')
        job_ids = obj_job.search(cr, uid, [('date_stop','<',today)], context=context)
        if job_ids:
            jobs = obj_job.write(cr,uid,job_ids,{'active':False})
        past_job_check = self.pool.get('cci_magazine.past_job_check')
        today = datetime.datetime.today()
        id = past_job_check.create(cr, uid, {
            'name': today.strftime('%Y-%m-%d-%H:%M:%S'),
            'count' : len(job_ids),
            'status': '[' + ','.join( [str(x) for x in job_ids] ) + ']',
            })
        return job_ids

    _columns = {
        'magazine_subscription_source':fields.selection(_get_subscription_sources,'Source',size=30),
        'magazine_lastprospection':fields.date("Magazine last Sending as Prospection"),
    }
res_partner_job()

class cci_mag_subscription(osv.osv):
    _name = "cci_mag_subscription"
    _description = "Subscription to the CCI Mag"
    _columns = {
        'source':fields.char('Source',size=30),
        'type':fields.char("Type",size=30),
        'model':fields.char("Model",size=30),
        'partner_name':fields.char("Partner name",size=120),
        'membership_state':fields.char("Membership state",size=30),
        'street':fields.char("Street",size=120),
        'street2':fields.char("Street2",size=120),
        'city':fields.char("City",size=120),
        'contact_name':fields.char("Contact name",size=60),
        'first_name':fields.char("First name",size=60),
        'partner_id':fields.many2one('res.partner',"Partner data"),
        'address_id':fields.many2one('res.partner.address',"Address data"),
        'job_id':fields.many2one('res.partner.job',"Function data"),
        'contact_id':fields.many2one('res.partner.contact',"Contact data"),
    }
cci_mag_subscription()

class past_job_check(osv.osv):
    _name = 'cci_magazine.past_job_check'
    _description = '''Recording of a check of past done job'''
    _columns = {
        'name' : fields.char('Date',size=19,required=True),
        'count': fields.integer('Changes count'),
        'status' : fields.text('Status'),
    }
    _order = 'name desc'
past_job_check()

class standard_street(osv.osv):
    _name = 'cci_magazine.standard_street'
    _description = '''Standardisation of street name with zip codes'''
    _columns = {
        'name' : fields.char('Wrong Name',size=200,required=True),
        'full_zip': fields.char('Wrong Zip Code and Name',size=60), # not required => if not given, all streets will use the same 'standardisation'
        'good_name': fields.char('SUMO Name',size=200,required=True),
        'good_zip_code': fields.char('SUMO ZIP code',size=60),
        'good_zip_municipality': fields.char('SUMO ZIP Municipality',size=60),
    }
    def _check_zips(self, cr, uid, ids, context=None):
        for val in self.read(cr, uid, ids, ['full_zip','good_zip_code','good_zip_municipality'], context=context):
            if val['full_zip'] and not (val['good_zip_code'] or val['good_zip_municipality']):
                return False
            elif (val['good_zip_code'] or val['good_zip_municipality']) and not val['full_zip']:
                return False
        return True

    _order = 'full_zip, name'

    _constraints = [(_check_zips, 'If you give wrong or good zip codes, you MUST give BOTH.', ['full_zip','good_zip_code','good_zip_municipality']), ]
    
    def standardize(self,cr,uid, street, sumo_zip_code, sumo_zip_municipality):
        std_street_obj = self.pool.get('cci_magazine.standard_street')
        street = street.strip()
        result = (street,sumo_zip_code,sumo_zip_municipality)
        std_street_ids = std_street_obj.search(cr,uid,[('name','=',street),('full_zip','=',(sumo_zip_code.strip()+' '+sumo_zip_municipality.strip()).strip())])
        if std_street_ids:
            std_street = std_street_obj.read(cr,uid,std_street_ids[0],['good_name','good_zip_code','good_zip_municipality'])
            result = (std_street['good_name'],std_street['good_zip_code'],std_street['good_zip_municipality'])
        else:
            std_street_ids = std_street_obj.search(cr,uid,[('name','=',street)])
            if std_street_ids:
                std_street = std_street_obj.read(cr,uid,std_street_ids[0],['good_name','good_zip_code','good_zip_municipality'])
                result = (std_street['good_name'],sumo_zip_code,sumo_zip_municipality)
        return result
standard_street()
