# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import netsvc
import pooler
import base64
import datetime
from osv import fields, osv

form = """<?xml version="1.0"?>
<form string="Create Invoices from a CSV file of Partner IDs">
    <field name="explanation" colspan="4" nolabel="1"/>
    <field name="date"/>
    <field name="period_id"/>
    <field name="input" colspan="4" width="600"/>
</form>
"""
fields = {
    'date': {'type':'date','string':'Invoice Date','required':'True'},
    'period_id':{'type':'many2one','relation':'account.period','string':'Period','required':'True'},
    'input': {'type':'binary','string':'CSV file','required':True},
    'explanation': {'string':'Explanation', 'type':'text', 'readonly':True, 'default':u"Vous devez fournir un fichier CSV avec au moins une colonne nommée 'partner_id' et contenant les IDs des partenaires à facturer.\nPour créer un fichier CSV, vous pouvez utiliser un fichier Excel et 'enregistrer sous...' avec le type 'CSV'."},
}


res_form = """<?xml version="1.0"?>
<form string="Results">
    <newline />
    <field name="inv_created"/>
    <newline />
    <field name="inv_rejected"/>
    <newline />
    <field name="inv_rej_reason" width="400"/>
</form>
"""
res_fields = {
    'inv_created': {'string':'Invoice Created', 'type':'char', 'readonly':True},
    'inv_rejected': {'string':'Invoice Rejected', 'type':'char', 'readonly':True},
    'inv_rej_reason': {'string':'Error Messages', 'type':'text', 'readonly':True},
}

def _make_invoices(self, cr, uid, data, context):
    inputfile = data['form']['input']
    inputdata = base64.decodestring(inputfile)
    lines = inputdata.splitlines()
    first_line = lines[0].replace('\n','').replace('\r','')
    separator = ';'
    only_one_column = False
    if len(first_line) == 10 and first_line == 'partner_id':
        separator = ' ' # can't be empty...
        only_one_column = True
    else:
        if separator not in first_line:
            separator = ','
            if separator not in first_line:
                  separator = '' # cancel the rest of treatment
    if separator or only_one_column:
        field_names = first_line.split(separator)
        if 'partner_id' in field_names:
            index_partner_id = field_names.index('partner_id')
            
            inv_rej_reason = ''
            final_result = True
            pool_obj = pooler.get_pool(cr.dbname)
            
            invoices = {}
            invoice_ids = []
            create_ids=[]
            inv_create = 0
            inv_reject = 0
            inv_rej_reason = ""
            list_inv = []
            obj_lines=pool_obj.get('account.invoice.line')
            obj_partner = pool_obj.get('res.partner')
            obj_inv = pool_obj.get('account.invoice')
            obj_product = pool_obj.get('product.product')
            prod_ids = data['ids']
            for line in lines[1:]:
                elements = line.split(separator)
                partner_id = int(elements[index_partner_id])
                if partner_id and partner_id > 0:
                    partner = obj_partner.browse(cr,uid,partner_id)
                    val_invoice = pool_obj.get('account.invoice').onchange_partner_id(cr, uid, [], 'out_invoice', partner_id, False, False)
                    val_invoice['value'].update({'partner_id': partner_id})
                    partner_address_id = val_invoice['value']['address_invoice_id']
                    if not partner_address_id:
                        inv_rej_reason += u"Given partner doesn't have an address to make the invoice.\nInvoice not generated for the partner id %i \n" % partner_id
                        inv_reject += 1
                    else:
                        inv_line_ids = []
                        context['force_member'] = partner.membership_state in ['free','paid','invoiced']
                        for product_id in data['ids']:
                            unit_price = obj_product.price_get(cr, uid, [product_id], ptype='member_price',context=context)[product_id]
                            product = obj_product.browse(cr,uid,product_id)
                            value = obj_lines.product_id_change(cr, uid, [], product_id,uom =False, partner_id=partner_id, fposition_id=partner.property_account_position.id)
                            tax_ids=[]
                            for tax in product.taxes_id:
                                tax_ids.append(tax.id)
                            vals = value['value']
                            vals.update({
                                'price_unit': unit_price,
                                'quantity': 1,
                                'product_id': product_id,
                                'invoice_line_tax_id': [(6,0,tax_ids)],
                            })
                            inv_line_ids.append(obj_lines.create(cr, uid,vals))
                        val_invoice['value'].update({
                            'date_invoice':data['form']['date'],
                            'period_id':data['form']['period_id'],
                            'origin': '',
                            'reference': False,
                            'invoice_line': [(6,0,inv_line_ids)],
                            'comment': "",
                        })
                        new_id_invoice = obj_inv.create(cr, uid, val_invoice['value'])
                        inv_create += 1
                        list_inv.append(new_id_invoice)
                else:
                    inv_rej_reason += 'The partner %i is not more in the ERP database\n' % partner_id
                    inv_reject += 1
        else:
            inv_rej_reason = u'Le fichier CSV semble mal formé : pas de colone \'partner_id\' trouvée.'
    else:
        inv_rej_reason = u'Le fichier CSV semble mal formé : pas de séparateur trouvé.'
    return {'inv_created' : str(inv_create) , 'inv_rejected' : str(inv_reject) , 'invoice_ids':  list_inv, 'inv_rej_reason': inv_rej_reason}

class invoice_from_partner_file(wizard.interface):

    def _set_default(self, cr, uid, data, context):
        today = datetime.datetime.today().strftime('%Y-%m-%d')
        data['form']['date'] = datetime.datetime.today().strftime('%Y-%m-%d')
        pool_obj = pooler.get_pool(cr.dbname)
        period_ids = pool_obj.get('account.period').search(cr,uid,[('date_start','<=',today),('date_stop','>=',today)])
        if period_ids:
            data['form']['period_id'] = period_ids[0]
        return data['form']
        
    def _list_invoice(self, cr, uid, data, context):
        pool_obj = pooler.get_pool(cr.dbname)
        model_data_ids = pool_obj.get('ir.model.data').search(cr,uid,[('model','=','ir.ui.view'),('name','=','invoice_form')])
        resource_id = pool_obj.get('ir.model.data').read(cr,uid,model_data_ids,fields=['res_id'])[0]['res_id']
        return {
            'domain': "[('id','in', ["+','.join(map(str,data['form']['invoice_ids']))+"])]",
            'name': 'Invoices',
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'account.invoice',
            'views': [(False,'tree'),(resource_id,'form')],
            'context': "{'type':'out_invoice'}",
            'type': 'ir.actions.act_window'
        }

    states = {
        'init': {
            'actions': [_set_default],
            'result': {'type': 'form', 'arch':form, 'fields': fields, 'state':[('end','Cancel'),('make_invoices', 'Invoice')]}

        },
        'make_invoices' : {
               'actions' : [_make_invoices],
               'result': {'type': 'form', 'arch': res_form, 'fields': res_fields, 'state':[('end','Ok'),('open','Open')]}
            },

        'open': {
            'actions': [],
            'result': {'type':'action', 'action':_list_invoice, 'state':'end'}
        }

    }

invoice_from_partner_file("invoice_from_partner_file")

