# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008 CCI Connect ASBL. (http://www.cciconnect.be) All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import datetime
from osv import fields, osv
from tools.translate import _
import tools

SUBJECT_MSG_JOB = u"Confirmation départ de %s de l'entreprise %s ?"
MSG_JOB = u"""
<p>En validant les modifications apportées par le membre à sa propre fiche, via le site Web, l'ERP a constaté qu'il a tenté d'effacer %s de son personnel alors qu'il/elle %s '%s' dont vous êtes responsable.</p>
<p>Cet 'effacement' a été annulé et juste remplacé par une simple absence du répertoire des membres papier et une exclusion du répertoire de l'app mobile CCI.</p>
<p>Mais il faudrait peut-être vérifier si ce départ de l'entreprise n'est pas réel.</p>
<p>Si tel est le cas, merci de bien effacer vous-même manuellement la présence de %s dans la fiche de l'entreprise %s.</p>
<p>Merci d'avance.</p>
<p>L'ERP...</p>
"""

def _only_digits(string):
    result = [x for x in string if x.isdigit()]
    return ''.join(result)
def _correct_escape(string):
    return string.replace("\\" + "'","'").replace("\\" + '"','"')
def _remove_stop(string):
    return string.replace("STOP!","").strip()
def _check_digit_1_digit(digital_string):
    digit_sum = 0
    for carac in digital_string:
        if carac.isdigit():
            digit_sum += int(carac)
    return str(digit_sum)[-1:]
def _link_id(record_id):
    partner = str(record_id).rjust(5,'0')
    inv_partner = ''
    for carac in partner:
        inv_partner = carac + inv_partner
    year = (7 * int(inv_partner[3:4])) + int(inv_partner[2:3]) + 13 ## year was the last two caracters of the current year
    link_id = inv_partner[0:3] + str(year) + inv_partner[3:4]
    link_id += _check_digit_1_digit(link_id)
    link_id += inv_partner[4:5]
    rest = int(link_id) % 93
    link_id += _check_digit_1_digit(link_id)
    link_id = link_id + str(rest).rjust(2,'0')
    return link_id    
def _convert_phone(string,PHONE_COUNTRY_PREFIX):
    def only_digits(string):
        cleaned = ''
        for carac in string:
            if carac in '0123456789':
                cleaned += carac
        return cleaned
    result = ''
    string = only_digits(string)
    if len(string) > 0:
        if len(string) == 9:
            if string[0:2] in ['02','03','04','09']:
                result = string[0:2] + "/" + string[2:5] + "." + string[5:7] + "." + string[7:]
            else:
                result = string[0:3] + "/" + string[3:5] + "." + string[5:7] + "." + string[7:]
        elif len(string) == 10:
            result = string[0:4] + "/" + string[4:6] + "." + string[6:8] + "." + string[8:]
        else:
            # international number
            if string[0:2] == '00':
                # search after a country with this prefix
                prefix = string[2:4]
                if prefix not in PHONE_COUNTRY_PREFIX:
                    prefix = string[2:5]
                    if prefix not in PHONE_COUNTRY_PREFIX:
                        prefix = string[2:6]
                        if prefix not in PHONE_COUNTRY_PREFIX:
                            prefix = ''
                if prefix:
                    result = '+' + string[2:2+len(prefix)] + ' ' + string[2+len(prefix):4+len(prefix)]
                    rest = string[4+len(prefix):]
                    while len(rest) > 3:
                        result += '.' + rest[0:2]
                        rest = rest[2:]
                    result += '.' + rest
                else:
                    result = 'International:'+string
    return result
def _convert_vat(vat):
    if vat and vat[0:2] == 'BE':
        result = vat[0:2]+'-'+vat[2:6]+'.'+vat[6:9]+'.'+vat[9:]
    else:
        result = vat
    return result
def split_address(string):
    if string.count(',') == 1:
        (street,number_and_box) = string.split(',')
        street = street.strip()
        number_and_box = number_and_box.strip()
        if ' BTE ' in number_and_box:
            (number,box) = number_and_box.split(' BTE ')
            number = number.strip()
            box = box.strip()
        else:
            number = number_and_box
            box = ''
    else:
        street = string.strip()
        number = ''
        box = ''
    return (street,number,box)
def get_activity_sectors(partner,oldcateg_ids,dOldSectID2NewID):
    secteurs = [False,False,False]
    current_secteur = 0
    if partner.category_id:
        for oldcateg in partner.category_id: ## category_id is, not like his name define, an array of category ids
            if (oldcateg.id in oldcateg_ids) and (oldcateg.id not in secteurs) and dOldSectID2NewID.has_key(oldcateg.id):
                secteurs[current_secteur] = dOldSectID2NewID[oldcateg.id]
                current_secteur += 1
                if current_secteur > 2:
                    break
    return secteurs

class directory_address_access(osv.osv):
    _name = "directory.address.access"
    _description = "Record the access to address.proxy from site"
    _columns = {
        'address_id': fields.many2one('res.partner.address','Address',ondelete='cascade'),
        'type': fields.char('Type of access',size=6),
        'server_time': fields.datetime('Server Time'),
        'ip_address': fields.char('IP Address',size=39),
    }
    _defaults = {
        'server_time' : lambda *a : datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
        'type' : lambda *a : 'read',
    }
directory_address_access()

class directory_address_proxy(osv.osv):
    _name = "directory.address.proxy"
    _description = "proxy partner and address 4 yearly members'directory"
    _columns = {
        'partner_id': fields.many2one('res.partner','Partner',required=True,ondelete='cascade'),
        'address_id': fields.many2one('res.partner.address','Address',required=True,ondelete='cascade'),
        'link_id': fields.char('Link ID',size=13,required=True),
        'complete_name': fields.char('Complete Name',size=200),
        'partner_name': fields.char('Partner Name',size=200),
        'address_name': fields.char('Address Name',size=200),
        'dir_show_name': fields.char('Dir Show Name',size=200),
        'dir_sort_name': fields.char('Dir Sort Name',size=200),
        'street': fields.char('Street',size=200),
        'street_number': fields.char('Street Number',size=20),
        'street_box': fields.char('Street Box',size=20),
        'zip_code': fields.char('Zip Code',size=15),
        'city': fields.char('City',size=50),
        'phone': fields.char('Phone',size=40),
        'fax': fields.char('Fax',size=40),
        'email': fields.char('eMail',size=200),
        'web': fields.char('Web',size=200),
        'vat_num': fields.char('VAT',size=40),
        'employee': fields.integer('Employee'),
        'desc_activ': fields.text('Activity'),
        'origin_activ':fields.char('Origin Activ',size=10), # 'defpartner' : from partner on default address; 'partner' = from partner on other address; 'address' : from address record
        'comments': fields.text('User Comments'),
        'sector1': fields.char('Sector1',size=6),
        'sector2': fields.char('Sector2',size=6),
        'sector3': fields.char('Sector3',size=6),
        'new_complete_name': fields.char('Complete Name',size=200),
        'new_street': fields.char('Street',size=200),
        'new_street_number': fields.char('New Street Number',size=20),
        'new_street_box': fields.char('New Street Box',size=20),
        'new_zip_code': fields.char('Zip Code',size=4),
        'new_city': fields.char('City',size=50),
        'new_phone': fields.char('Phone',size=40),
        'new_fax': fields.char('Fax',size=40),
        'new_email': fields.char('eMail',size=200),
        'new_web': fields.char('Web',size=200),
        'new_vat_num': fields.char('VAT',size=40),
        'new_employee': fields.integer('Employee'),
        'new_desc_activ': fields.text('Activity'),
        'new_sector1': fields.char('Sector1',size=6),
        'new_sector2': fields.char('Sector2',size=6),
        'new_sector3': fields.char('Sector3',size=6),
        'final_partner_name': fields.char('Partner Name',size=200),
        'final_address_name': fields.char('Address Name',size=200),
        'final_dir_show_name': fields.char('Dir Show Name',size=200),
        'final_dir_sort_name': fields.char('Dir Sort Name',size=200),
        'final_street': fields.char('Street',size=200),
        'final_street_number': fields.char('Final Street Number',size=20),
        'final_street_box': fields.char('Final Street Box',size=20),
        'final_zip_id': fields.many2one('res.partner.zip','Zip Code'),
        'final_phone': fields.char('Phone',size=40),
        'final_fax': fields.char('Fax',size=40),
        'final_email': fields.char('eMail',size=200),
        'final_web': fields.char('Web',size=200),
        'final_vat_num': fields.char('VAT',size=40),
        'final_employee': fields.integer('Employee'),
        'final_sector1': fields.many2one('res.partner.activsector','Sector1',),
        'final_sector2': fields.many2one('res.partner.activsector','Sector2',),
        'final_sector3': fields.many2one('res.partner.activsector','Sector3',),
        'final_desc_activ': fields.text('Final Desc Activ'),
        'final_addr_desc_activ': fields.text('Final Addr Desc Activ'),
        'not_in_directory': fields.boolean('Not in directory'),
        'final_not_in_directory': fields.boolean('Final Not in directory'),
        'user_validated': fields.boolean('Validated by User'),
        'internal_validated': fields.boolean('Validated by CCI'),
        'sending_address': fields.char('Sending Address',size=200),
        'sending_name': fields.char('Sending Name',size=60),
        'sending_courtesy': fields.char('Sending Courtesy',size=60),
        'send_initial_mail': fields.datetime('Send Initial Mail'),
        'send_first_followup': fields.datetime('Send Followup Mail'),
        'job_ids': fields.one2many('directory.job.proxy','address_proxy_id','Jobs'),
        #'deleted_job_ids': fields.one2many('directory.job2delete','address_proxy_id','Jobs marked for deletion'),
        'change_date':fields.datetime('Change Date'),
        'full_page':fields.boolean('Full Page for this edition'), # to remove later not more necessary
        'full_page_app':fields.boolean('Full Page on App Directory'),
        'active':fields.boolean('Active'),
        # the following field indicate that the address has already been validated by the user and the CCI since the beginning of the last directory
        'already_validated':fields.boolean('Already validated for Directory'),
        'edition':fields.char('Edition',size=10),
        'write_date' : fields.datetime('Last Modification'),
        'write_uid' : fields.many2one('res.users','Last Modifier',help='The last person who has modified this contact'),
        'create_date' : fields.datetime('Creation Date'),
        'create_uid' : fields.many2one('res.users','Creator',help='The person who has created this record'),
    }
    _defaults = {
        'not_in_directory' : lambda *a : False,
        'user_validated' : lambda *a : False,
        'internal_validated' : lambda *a : False,
        'full_page': lambda *a: False,
        'full_page_app': lambda *a: False,
        'already_validated': lambda *a: False,
        'active': lambda *a: True,
    }
    def name_get(self, cr, uid, ids, context=None):
        if not len(ids):
            return []
        reads = self.read(cr, uid, ids, ['complete_name'], context=context)
        res = []
        for record in reads:
            name = record['complete_name']
            res.append((record['id'], name))
        return res
    def read(self, cr, uid, ids, fields, *args, **kwargs): # , context = none
        if uid == 74:
            # read as admin to avid recursion
            currents = self.read(cr,1,ids,['address_id'])
            obj_access = self.pool.get('directory.address.access')
            for current in currents:
                if current['address_id']:
                    obj_access.create(cr,uid,{'address_id':current['address_id'][0],'type':'read'})
        return super(osv.osv,self).read(cr,uid,ids,fields, *args, **kwargs)
    def write(self, cr, uid, ids,vals,context=None):
        result = False
        if vals.has_key('user_validated') and vals['user_validated'] == False and len(vals) == 1:
            result = super(osv.osv,self).write(cr, uid, ids,vals, context)
        else:
            current = self.pool.get('directory.address.proxy').browse(cr,1,ids)[0]
            if vals.has_key('new_complete_name'):
                vals.update({'final_partner_name':'','final_address_name':'','final_dir_show_name':'','final_dir_sort_name':'','internal_validated':False})
            if vals.has_key('new_web'):
                vals.update({'final_web':(vals['new_web'] or '').replace('http://',''),'internal_validated':False})
            if vals.has_key('new_vat_num'):
                new_vat_num = ( vals['new_vat_num'] or '').replace('/','').replace('-','').replace('.','').replace(' ','')
                if len(new_vat_num) == 9:
                    new_vat_num = 'BE0' + new_vat_num
                elif len(new_vat_num) == 10:
                    new_vat_num = 'BE' + new_vat_num
                elif len(new_vat_num) == 11 and new_vat_num[2:3] != '0':
                    new_vat_num = 'BE0' + new_vat_num[3:]
                vals.update({'final_vat_num':new_vat_num,'internal_validated':False})
            if vals.has_key('new_street'):
                vals.update({'final_street':vals['new_street'],'internal_validated':False})
            if vals.has_key('new_street_number'):
                vals.update({'final_street_number':vals['new_street_number'],'internal_validated':False})
            if vals.has_key('new_street_box'):
                vals.update({'final_street_box':vals['new_street_box'],'internal_validated':False})
            if vals.has_key('new_zip_code') or vals.has_key('new_city'):
                zip_obj = self.pool.get('res.partner.zip')
                new_zip_code = vals.get('new_zip_code',current.zip_code) or ''
                new_city = vals.get('new_city',current.city) or ''
                zip_ids = zip_obj.search(cr,uid,[('name','=',new_zip_code),('city','ilike',new_city)])
                if not zip_ids:
                    # search in other names
                    zip_ids = zip_obj.search(cr,uid,[('name','=',new_zip_code),('other_names','ilike',new_city)])
                if zip_ids and len(zip_ids) == 1:
                    vals.update({'final_zip_id':zip_ids[0],'internal_validated':False})
                else:
                    found = False
                    zips = zip_obj.read(cr,uid,zip_ids,['name','city'])
                    for zip_record in zips:
                        if len(zip_record['city']) == len(new_city):
                            vals.update({'final_zip_id':zip_record['id'],'internal_validated':False})
                            found = True
                            break
                    if not found:
                        vals.update({'final_zip_id':False,'internal_validated':False})
            if vals.has_key('new_sector1') or vals.has_key('new_sector2') or vals.has_key('new_sector3'):
                sector_obj = self.pool.get('res.partner.activsector')
            if vals.has_key('new_sector1'):
                if not vals['new_sector1']:
                    vals.update({'final_sector1':False,'internal_validated':False})
                else:
                    sector_ids = sector_obj.search(cr,uid,[('code','=',vals['new_sector1'])])
                    if sector_ids and len(sector_ids) == 1:
                        vals.update({'final_sector1':sector_ids[0],'internal_validated':False})
            if vals.has_key('new_sector2'):
                if not vals['new_sector2']:
                    vals.update({'final_sector2':False,'internal_validated':False})
                else:
                    sector_ids = sector_obj.search(cr,uid,[('code','=',vals['new_sector2'])])
                    if sector_ids and len(sector_ids) == 1:
                        vals.update({'final_sector2':sector_ids[0],'internal_validated':False})
            if vals.has_key('new_sector3'):
                if not vals['new_sector3']:
                    vals.update({'final_sector3':False,'internal_validated':False})
                else:
                    sector_ids = sector_obj.search(cr,uid,[('code','=',vals['new_sector3'])])
                    if sector_ids and len(sector_ids) == 1:
                        vals.update({'final_sector3':sector_ids[0],'internal_validated':False})
            if vals.has_key('new_phone'):
                new_phone = _only_digits(vals['new_phone'] or '')
                vals.update({'final_phone':new_phone,'internal_validated':False})
            if vals.has_key('new_fax'):
                new_fax = _only_digits(vals['new_fax'] or '')
                vals.update({'final_fax':new_fax,'internal_validated':False})
            if vals.has_key('new_employee') and vals['new_employee'] != 0:
                vals.update({'final_employee':vals['new_employee'],'internal_validated':False})
            if vals.has_key('new_desc_activ'):
                if current.origin_activ in ('defpartner','partner'):

                    vals.update({'final_desc_activ':vals['new_desc_activ'],'internal_validated':False})
                else:
                    vals.update({'final_addr_desc_activ':vals['new_desc_activ'],'internal_validated':False})
            if vals.has_key('new_email'):
                vals.update({'final_email':vals['new_email'],'internal_validated':False})
            if vals.has_key('new_courtesy'):
                vals.update({'final_courtesy':vals['new_courtesy'],'final_gender':vals['new_courtesy']=='Madame' and 'women' or 'man','internal_validated':False})
            if vals.has_key('not_in_directory'):
                vals.update({'final_not_in_directory':vals['not_in_directory'],'internal_validated':False})
            result = super(osv.osv,self).write(cr, uid, ids,vals, context)
        return result
    def but_confirm_changes(self, cr, uid, ids, *args):
        current = self.browse(cr,uid,ids)[0]
        if current.internal_validated:
            raise osv.except_osv(_('Error !'), _('This record has already been updated.'))
        else:
            new_partner_data = {}
            new_address_data = {}
            if current.final_partner_name:
                new_partner_data['name'] = _correct_escape(current.final_partner_name)
            if current.final_web or (current.web and not current.final_web):
                new_partner_data['website'] = _correct_escape(current.final_web)
            if current.final_employee:
                new_partner_data['employee_nbr'] = current.final_employee ## CORRECT BOTH TO KEEP THEM SYNCHRO
                new_partner_data['employee_nbr_total'] = current.final_employee
            if current.final_vat_num and current.final_vat_num != 'BE0000000000':
                new_partner_data['vat'] = current.final_vat_num
            if current.final_desc_activ and current.origin_activ == 'defpartner':
                new_partner_data['activity_description'] = _remove_stop(_correct_escape(current.final_desc_activ))
            if current.final_address_name:
                new_address_data['name'] = ( current.final_address_name == '/' ) and '' or  _correct_escape(current.final_address_name)
            else:
                new_address_data['name'] = ''
            if current.final_dir_show_name:
                new_address_data['dir_show_name'] = ( current.final_dir_show_name == '/' ) and '' or  _correct_escape(current.final_dir_show_name)
            else:
                new_address_data['dir_show_name'] = ''
            if current.final_dir_sort_name:
                new_address_data['dir_sort_name'] = ( current.final_dir_sort_name == '/' ) and '' or  _correct_escape(current.final_dir_sort_name)
            else:
                new_address_data['dir_sort_name'] = ''
            if current.final_street or current.final_street_number or current.final_street_box:
                new_address_data['street'] =  _correct_escape(current.final_street or '')
                if current.final_street_number:
                    new_address_data['street'] += ', ' +  _correct_escape(current.final_street_number)
                if current.final_street_box:
                    new_address_data['street'] += ' BTE ' +  _correct_escape(current.final_street_box)
            if current.final_zip_id:
                new_address_data['zip_id'] = current.final_zip_id.id
            if current.final_phone or (current.phone and not current.final_phone):
                new_address_data['phone'] = ( current.final_phone == '/' ) and '' or current.final_phone
            if current.final_fax or (current.fax and not current.final_fax):
                new_address_data['fax'] = ( current.final_fax == '/' ) and '' or current.final_fax
            new_address_data['sector1'] = current.final_sector1 and current.final_sector1.id or False
            new_address_data['sector2'] = current.final_sector1 and current.final_sector2.id or False
            new_address_data['sector3'] = current.final_sector1 and current.final_sector3.id or False
            if new_address_data['sector3'] and new_address_data['sector2'] and new_address_data['sector3'] == new_address_data['sector2']:
                new_address_data['sector3'] = False
            if new_address_data['sector3'] and new_address_data['sector1'] and new_address_data['sector3'] == new_address_data['sector1']:
                new_address_data['sector3'] = False
            if new_address_data['sector2'] and new_address_data['sector1'] and new_address_data['sector2'] == new_address_data['sector1']:
                new_address_data['sector2'] = new_address_data['sector3']
            if current.final_email or (current.email and not current.final_email):
                new_address_data['email'] = ( current.final_email == '/' ) and '' or current.final_email
            if current.final_addr_desc_activ:
                new_address_data['activity_description'] =  _remove_stop(_correct_escape(( current.final_addr_desc_activ == '/' ) and '' or current.final_addr_desc_activ))
            if current.final_not_in_directory:
                new_address_data['dir_exclude'] = current.final_not_in_directory
            new_partner_data['date_lastcheck'] = datetime.datetime.today().strftime('%Y-%m-%d')
            self.pool.get('res.partner').write(cr,uid,[current.partner_id.id,],new_partner_data,{'lang':'fr_FR'})
            self.pool.get('res.partner.address').write(cr,uid,[current.address_id.id,],new_address_data,{'lang':'fr_FR'})
            # Update of contacts linked to the proxy record
            job_obj = self.pool.get('res.partner.job')
            contact_obj = self.pool.get('res.partner.contact')
            part_obj = self.pool.get('cci_club.participation')
            reg_obj = self.pool.get('event.registration')
            #address_obj = self.pool.get('res.partner.address')
            parameter_obj = self.pool.get('cci_parameters.cci_value')
            param_values = parameter_obj.get_value_from_names(cr,uid,['BDDeTest'])
            if param_values and param_values.has_key('BDDeTest'):
                BD_DE_TEST = param_values['BDDeTest']
            else:
                BD_DE_TEST = False
            #partner_obj = self.pool.get('res.partner.partner')
            jobs_in_sequence = []
            proxy_jobs_treated = []
            current_date = {'year':int(datetime.date.today().strftime('%Y')),'month':int(datetime.date.today().strftime('%m'))}
            if current_date['month'] == 1:
                current_date['year'] -= 1
                current_date['month'] = 13
            current_date['month'] -= 1
            A_MONTH_AGO = str(current_date['year'])+'-'+str(current_date['month']).rjust(2, '0')+'-01'
            for job_proxy in current.job_ids:
                proxy_jobs_treated.append(job_proxy.id)
                if job_proxy.marked_for_deletion:
                    if job_proxy.job_id and job_proxy.contact_id:
                        # we check if this person is not on a club or an event before 'deleting'
                        job_to_keep = False
                        part_ids = part_obj.search(cr,uid,[('partner_id','=',job_proxy.address_id.partner_id.id),('contact_id','=',job_proxy.contact_id.id)])
                        if part_ids:
                            participations = part_obj.browse(cr,uid,part_ids)
                            for participation in participations:
                                if participation.state_id and participation.state_id.active_presence:
                                    # this person is still active
                                    if participation.group_id and (participation.group_id.state in ['draft','current'] or participation.group_id.waiting_club):
                                        # the club is still active
                                        # the deleting of this contact is still transformed in absence for directory while a human will valid this
                                        if participation.group_id.responsible_id and participation.group_id.responsible_id.email:
                                            resp_email = participation.group_id.responsible_id.email
                                            contact_name = participation.contact_id.name
                                            if participation.contact_id.first_name:
                                                contact_name = participation.contact_id.first_name + ' ' + contact_name
                                            company_name = participation.partner_id.name + ' [' + str(participation.partner_id.id) + ']'
                                            subject = SUBJECT_MSG_JOB % (contact_name,company_name)
                                            content = MSG_JOB % (contact_name,u'fait partie du club',participation.group_id.name,contact_name,company_name)
                                            if BD_DE_TEST:
                                                print resp_email
                                                print subject
                                                print content
                                            else:
                                                tools.email_send('no-reply@ccilvn.be', [resp_email,], subject, content, subtype='html', email_bcc=['philmer@ccilvn.be',])
                                        job_obj.write(cr,uid,[job_proxy.job_id.id],{'sequence_directory':99,'mobile_exclude':True})
                                        job_to_keep = True
                        reg_ids = reg_obj.search(cr,uid,[('partner_id','=',job_proxy.address_id.partner_id.id),('contact_id','=',job_proxy.contact_id.id)])
                        if reg_ids:
                            regs = reg_obj.browse(cr,uid,reg_ids)
                            for reg in regs:
                                if reg.state in ['draft','confirm','done']:
                                    if reg.event_id and reg.event_id.state in ['draft','fixed','open','confirm','running','done'] and reg.event_id.date_begin >= A_MONTH_AGO:
                                        # the deleting of this contact is still transformed in absence for directory while a human will valid this
                                        if reg.event_id.user_id and reg.event_id.user_id.email:
                                            resp_email = reg.event_id.user_id.email
                                            contact_name = reg.contact_id.name
                                            if reg.contact_id.first_name:
                                                contact_name = reg.contact_id.first_name + ' ' + contact_name
                                            company_name = reg.partner_id.name + ' [' + str(reg.partner_id.id) + ']'

                                            subject = SUBJECT_MSG_JOB % (contact_name,company_name)
                                            content = MSG_JOB % (contact_name,u'est inscrit à l\'event',reg.event_id.name,contact_name,company_name)
                                            if BD_DE_TEST:
                                                print resp_email
                                                print subject
                                                print content
                                            else:
                                                tools.email_send('no-reply@ccilvn.be', [resp_email,], subject, content, subtype='html', email_bcc=['philmer@ccilvn.be',])
                                        job_obj.write(cr,uid,[job_proxy.job_id.id],{'sequence_directory':99,'mobile_exclude':True})
                                        job_to_keep = True
                        if not job_to_keep:
                            if job_proxy.contact_id.job_ids and len(job_proxy.contact_id.job_ids) == 1 and job_proxy.contact_id.job_ids[0].contact_id.id == job_proxy.job_id.id:
                                contact_obj.write(cr,uid,[job_proxy.contact_id.id],{'active':False})
                            job_obj.write(cr,uid,[job_proxy.job_id.id],{'active':False})
                else:
                    if job_proxy.job_id: ## and job_proxy.final_last_name == job_proxy.job_id.contact_id.name:
                        #print 'put changes in db'
                        jobs_in_sequence.append(job_proxy.job_id.id)
                        # we put the changes in db
                        def_mobile = False
                        if job_proxy.final_mobile and job_proxy.final_mobile != '0400000000':
                            def_mobile = job_proxy.final_mobile
                            if job_proxy.final_mobile_confidential:
                                def_mobile += ' (conf)'
                        contact_obj.write(cr,uid,[job_proxy.contact_id.id],{'name': _correct_escape(job_proxy.final_last_name),
                                                                            'first_name':  _correct_escape(job_proxy.final_first_name).strip(),
                                                                            'title':  _correct_escape(job_proxy.final_courtesy or 'Monsieur'),
                                                                            'gender': (job_proxy.final_gender or '').strip(),
                                                                            'mobile': def_mobile, ## TODO nettoyage formatage
                                                                            })
                        job_obj.write(cr,uid,[job_proxy.job_id.id],{'sequence_directory':job_proxy.final_sequence or job_proxy.new_sequence or job_proxy.sequence,
                                                                    'function_label':  _correct_escape(job_proxy.final_title or '').strip(),
                                                                    'email': (job_proxy.final_email or '').strip(),
                                                                    'function_code_label': job_proxy.final_categs,
                                                                   })
                    else:
                        if job_proxy.new_last_name:
                            # we have to create a new contact and link him to this company
                            # or search for an existing contact in the same address
                            existing_job_id = 0
                            existing_contact_id = 0
                            if current.address_id:
                                for job in current.address_id.job_ids:
                                    inserted_last_name = (job_proxy.new_last_name or '').strip()
                                    inserted_first_name = (job_proxy.final_first_name or '').strip()
                                    if job.contact_id and job.contact_id.name.strip() == inserted_last_name and ( job.contact_id.first_name or '').strip() == inserted_first_name:
                                        existing_job_id = job.id
                                        existing_contact_id = job.contact_id.id
                                        break
                            if existing_job_id == 0 or existing_contact_id == 0:
                                #print 'create new contact and job'
                                def_mobile = False
                                if job_proxy.final_mobile and job_proxy.new_mobile != '0400000000':
                                    def_mobile = job_proxy.final_mobile
                                    if job_proxy.final_mobile_confidential:
                                        def_mobile += ' (conf)'
                                new_contact_id = contact_obj.create(cr,uid,{'name': _correct_escape(job_proxy.new_last_name),
                                                                            'first_name': _correct_escape(job_proxy.new_first_name or '').strip(),
                                                                            'title': (job_proxy.final_courtesy or '').strip(),
                                                                            'gender': (job_proxy.final_gender or '').strip(),
                                                                            'mobile': def_mobile or False, ## TODO nettoyage
                                                                           })
                                new_job_id = job_obj.create(cr,uid,{'sequence_directory':job_proxy.final_sequence or job_proxy.new_sequence or job_proxy.sequence,
                                                                    'sequence_partner':99,
                                                                    'sequence_contact':1,
                                                                    'sequence_yearbook':job_proxy.new_sequence or job_proxy.sequence,
                                                                    'function_label':  _correct_escape(job_proxy.new_title).strip(),
                                                                    'email': (job_proxy.new_email or '').strip(),
                                                                    'function_code_label': (job_proxy.final_categs or '').strip(),
                                                                    'contact_id':new_contact_id,
                                                                    'address_id':current.address_id.id,
                                                                    'partner_id':current.partner_id.id,
                                                                   })
                            else:
                                #print 'change data for found job'
                                changes = {}
                                if job_proxy.final_courtesy:
                                    changes['title'] =  _correct_escape(job_proxy.final_courtesy).strip()
                                if job_proxy.final_gender:
                                    changes['gender'] = (job_proxy.final_gender or '').strip()
                                def_mobile = False
                                if job_proxy.final_mobile and job_proxy.new_mobile != '0400000000':
                                    def_mobile = job_proxy.final_mobile
                                    if job_proxy.final_mobile_confidential:
                                        def_mobile += ' (conf)'
                                changes['mobile'] = def_mobile
                                if changes:
                                    contact_obj.write(cr,uid,[existing_contact_id],changes)
                                changes = {}
                                changes['sequence_directory'] = job_proxy.final_sequence or job_proxy.new_sequence or job_proxy.sequence
                                if job_proxy.final_title:
                                    changes['function_label'] =  _correct_escape(job_proxy.final_title).strip()
                                if job_proxy.final_email:
                                    changes['email'] = (job_proxy.final_email or '').strip()
                                if job_proxy.final_categs:
                                    changes['function_code_label'] = (job_proxy.final_categs or '').strip()
                                job_obj.write(cr,uid,[existing_job_id],changes)
                                new_job_id = existing_job_id
                            #jobs_in_sequence.append(new_job_id)
            if proxy_jobs_treated:
                self.pool.get('directory.job.proxy').write(cr,uid,proxy_jobs_treated,{'internal_validated':True})
            #self.write(cr, uid, ids, {'internal_validated':True})
            #self.write(cr, uid, ids, {'user_validated':False}) # we put the record once again modifiable
            # we keep this record but as 'inactive' to keep track of the user's work
            # so to refresh the record and put it again modifiable thru website, we create new record for this address
            self.write(cr, uid, ids, {'internal_validated':True,'active':False}) # we put the record once again modifiable by making the original record inactive
            self._synchro_address(cr,uid,current.address_id,current.partner_id,[current.id,],current.full_page_app and [current.id,] or [],False,False,False,False,False,False)
        return True
    def get_phone_country_prefix(self,cr,uid):
        result = []
        obj_country = self.pool.get('cci.country')
        country_ids = obj_country.search(cr,uid,[('phoneprefix','!=',False),('phoneprefix','!=',0)])
        if country_ids:
            countries = obj_country.read(cr,uid,country_ids,['phoneprefix'])
            result = [str(x['phoneprefix']) for x in countries]
        return result
    def _get_sector_dicts(self,cr,uid):
        obj_categ = self.pool.get('res.partner.activsector')
        categ_ids = obj_categ.search(cr,uid,[])
        categs = obj_categ.read(cr,uid,categ_ids,['code'])
        dCategs = {}
        dSectCode2ID = {}
        for categ in categs:
            dCategs[categ['id']] = categ['code']
            dSectCode2ID[categ['code']] = categ['id']
        # table for converting old activity sectors to new ones
        dOldSectID2NewID = {}
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['ActivityCodeRootID'])
        if param_values.has_key('ActivityCodeRootID'):
            activityCodeRootID = param_values['ActivityCodeRootID']
            obj_oldcateg = self.pool.get('res.partner.category')
            old_len = 0
            oldcateg_ids = [ activityCodeRootID ]
            while len(oldcateg_ids) > old_len:
                new_ids = oldcateg_ids[old_len:] # ids of categories added last time
                old_len = len(oldcateg_ids) # the new frontier ...
                new_categs = obj_oldcateg.read(cr,uid,new_ids,['child_ids'])
                for categ in new_categs:
                    if categ['child_ids']:
                        oldcateg_ids.extend(categ['child_ids'])
            oldcategs = obj_oldcateg.read(cr,uid,oldcateg_ids,['name'],{'lang':'fr_FR'})
            for oldcateg in oldcategs:
                formated_name = oldcateg['name']
                posA = formated_name.rfind('[')
                posB = formated_name.rfind(']')
                if posA >= 0 and posB > 0 and posA < posB:
                    code = formated_name[posA+1:posB]
                    if dSectCode2ID.has_key(code):
                        dOldSectID2NewID[oldcateg['id']] = dSectCode2ID[code]
        return (dCategs,dSectCode2ID,dOldSectID2NewID,oldcateg_ids)
    def _synchro_address(self,cr,uid,addr,partner,modifiable_proxy_ids,full_page_partner_ids,always_create,user_website_id,dCategs,dSectCode2ID,dOldSectID2NewID,oldcateg_ids):
        # addr = browse object, partner = browse object or false (then extracted from addr)
        # dCategs = dictionnaire des nouveaux secteurs d'activités, mais peut-être à 'False'
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['LastDirectoryBeginDate','LastDirectoryEdition'])
        if param_values.has_key('LastDirectoryBeginDate'):
            BEGIN_DIRECTORY_DATE = param_values['LastDirectoryBeginDate']
        else:
            BEGIN_DIRECTORY_DATE = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        if param_values.has_key('LastDirectoryEdition'):
            DIRECTORY_EDITION = param_values['LastDirectoryEdition']
        else:
            DIRECTORY_EDITION = str(datetime.date.today().year+1)
        partner_no_email = ''
        addr_count = 0
        modif_count = 0
        email_count = 0
        cont_count = 0
        if not partner:
            partner = addr.partner_id
        if not dCategs:
            (dCategs,dSectCode2ID,dOldSectID2NewID,oldcateg_ids) = self._get_sector_dicts(cr,uid)
        if not user_website_id:
            user_website_id = self.pool.get('res.users').search(cr,uid,[('name','=','Noomia')])[0]
        if (addr.type <> 'invoice' or addr.dir_presence) and partner:
            addr_proxy_obj = self.pool.get('directory.address.proxy')
            cont_proxy_obj = self.pool.get('directory.job.proxy')
            link_id = _link_id(addr.id)
            PHONE_COUNTRY_PREFIX = self.get_phone_country_prefix(cr,uid)
            sector_ids = []
            if addr.sector1:
                sector_ids.append(addr.sector1.id)
                if addr.sector2:
                    sector_ids.append(addr.sector2.id)
                    if addr.sector3:
                        sector_ids.append(addr.sector3.id)
            else:
                (sector1,sector2,sector3) = get_activity_sectors(partner,oldcateg_ids,dOldSectID2NewID)
                sector_ids = [sector1,sector2,sector3]
            addr_data = {'link_id':link_id,'partner_id':partner.id,'address_id':addr.id}
            if addr.dir_show_name:
                addr_data['complete_name'] = addr.dir_show_name
            else:
                if addr.name:
                    if addr.name[0:2] == '- ' or addr.name[0:3] == ' - ':
                        addr_data['complete_name'] = partner.name.strip() + ' ' + addr.name.strip()
                    else:
                        addr_data['complete_name'] = addr.name.strip()
                else:
                    addr_data['complete_name'] = partner.name.strip()
            addr_data['partner_name'] = partner.name.strip()
            addr_data['address_name'] = ( addr.name or '' ).strip()
            addr_data['dir_show_name'] = ( addr.dir_show_name or '' ).strip()
            addr_data['dir_sort_name'] = ( addr.dir_sort_name or '' ).strip()
            (addr_data['street'],addr_data['street_number'],addr_data['street_box']) = split_address( addr.street or '' )
            addr_data['zip_code'] = ( addr.zip_id and addr.zip_id.name or '' )
            addr_data['city'] = ( addr.zip_id and addr.zip_id.city or '' )
            addr_data['phone'] = _convert_phone( addr.phone or '', PHONE_COUNTRY_PREFIX )
            addr_data['fax'] = _convert_phone( addr.fax or '', PHONE_COUNTRY_PREFIX )
            addr_data['email'] = ( addr.email or '' )
            addr_data['web'] = ( partner.website or '' )
            addr_data['vat_num'] = _convert_vat( partner.vat or '' )
            addr_data['employee'] = ( partner.employee_nbr_total or 0 )
            if addr.activity_description:
                if len(addr.activity_description) > 250:
                    addr_data['desc_activ'] = addr.activity_description[0:244]+' STOP!'
                else:
                    addr_data['desc_activ'] = addr.activity_description
                addr_data['origin_activ'] = 'address'
            else:
                if partner.activity_description and len(partner.activity_description) > 250:
                    addr_data['desc_activ'] = partner.activity_description[0:244]+' STOP!'
                else:
                    addr_data['desc_activ'] = partner.activity_description or ''
                addr_data['origin_activ'] = (addr.type == 'default') and 'defpartner' or 'partner'
            addr_data['not_in_directory'] = partner.dir_exclude or addr.dir_exclude
            addr_data['final_partner_name'] = partner.name.strip()
            addr_data['final_address_name'] = ( addr.name or '' ).strip()
            addr_data['final_dir_show_name'] = ( addr.dir_show_name or '' ).strip()
            addr_data['final_dir_sort_name'] = ( addr.dir_sort_name or '' ).strip()
            (addr_data['final_street'],addr_data['final_street_number'],addr_data['final_street_box']) = split_address( addr.street or '' )
            addr_data['final_zip_id'] = ( addr.zip_id and addr.zip_id.id or False )
            addr_data['final_phone'] = addr.phone or ''
            addr_data['final_fax'] = addr.fax or ''
            addr_data['final_email'] = ( addr.email or '' )
            addr_data['final_web'] = ( partner.website or '' )
            addr_data['final_vat_num'] = partner.vat or ''
            addr_data['final_employee'] = ( partner.employee_nbr_total or 0 )
            addr_data['final_desc_activ'] = ''
            addr_data['final_dir_show_name'] = addr.dir_show_name or ''
            addr_data['final_dir_sort_name'] = addr.dir_sort_name or ''
            addr_data['final_not_in_directory'] = partner.dir_exclude or addr.dir_exclude
            addr_data['sector1'] = False
            addr_data['sector2'] = False
            addr_data['sector3'] = False
            addr_data['final_sector1'] = False
            addr_data['final_sector2'] = False
            addr_data['final_sector3'] = False
            addr_data['full_page_app'] = (partner.id in full_page_partner_ids)
            if sector_ids and sector_ids[0]:
                addr_data['sector1'] = dCategs[sector_ids[0]]
                addr_data['final_sector1'] = sector_ids[0]
                if len(sector_ids) > 1 and sector_ids[1]:
                    addr_data['sector2'] = dCategs[sector_ids[1]]
                    addr_data['final_sector2'] = sector_ids[1]
                    if len(sector_ids) > 2 and sector_ids[2]:
                        addr_data['sector3'] = dCategs[sector_ids[2]]
                        addr_data['final_sector3'] = sector_ids[2]
            addr_data['edition'] = DIRECTORY_EDITION
            # either we create a new record proxy for address, either we change the old one, either, we do noting
            address_method = 'create'
            addr_proxy_id = False
            if not always_create:
                existing_ids = addr_proxy_obj.search(cr,uid,[('address_id','=',addr_data['address_id'])])
                if len(existing_ids) == 1:
                    existing = addr_proxy_obj.read(cr,uid,existing_ids,['partner_name','address_name','dir_show_name','dir_sort_name','street','zip_code','city','full_page_app',
                                                                        'phone','fax','email','web','vat_num','employee','desc_activ','origin_activ','sector1','sector2','sector3',
                                                                        'link_id','user_validated','internal_validated','write_uid'])[0]
                    the_same = True
                    if not existing['user_validated'] and not existing['internal_validated'] and existing['write_uid'] != user_website_id:
                        for field_name in ['partner_name','address_name','dir_show_name','dir_sort_name','street','zip_code','city','full_page_app','phone','fax','email','web','vat_num','employee','desc_activ','origin_activ','link_id']:
                            if existing[field_name] != addr_data[field_name]:
                                the_same = False
                    if not the_same:
                        address_method = 'write'
                        addr_proxy_id = existing_ids[0]
                    else:
                        address_method = 'nothing'
            if address_method == 'create':
                # if we create a new record, we must check if we already don't have an existing record inactive but validated since the beginning of the last directory creation
                already_validated_ids = addr_proxy_obj.search(cr,uid,[('address_id','=',addr.id),('user_validated','=',True),('internal_validated','=',True),('create_date','>=',BEGIN_DIRECTORY_DATE),('active','=',False)])
                if already_validated_ids:
                    addr_data['already_validated'] = True
                addr_proxy_id = addr_proxy_obj.create(cr,uid,addr_data)
                modifiable_proxy_ids.append(addr_proxy_id)
                addr_count += 1
            elif address_method == 'write' and addr_proxy_id and addr_proxy_id in modifiable_proxy_ids:
                addr_proxy_obj.write(cr,uid,[addr_proxy_id,],addr_data)
                modif_count += 1
            # extraction of contacts
            # we extract two kinds of contacts : the contacts to be published and the contact allowed to modify the record
            if addr_proxy_id and addr_proxy_id in modifiable_proxy_ids:
                selected_jobs = []
                sending_address = ''
                sending_name = ''
                sending_courtesy = ''
                sending_sequence = 9999
                if addr.job_ids:
                    for job in addr.job_ids:
                        if job.contact_id:
                            new_job = {'sequence':job.sequence_directory or 99,
                                       'last_name':job.contact_id.name,
                                       'first_name':job.contact_id.first_name or '',
                                       'address_proxy_id':addr_proxy_id,
                                       'job_id':job.id,
                                       'contact_id':job.contact_id.id,
                                       'address_id':addr.id,
                                       'email': job.email or job.contact_id.email or '',
                                       'mobile': _convert_phone( job.contact_id.mobile or '', PHONE_COUNTRY_PREFIX ),
                                       'mobile_confidential':'conf' in (job.contact_id.mobile or ''),
                                       'title': job.function_label or '',
                                       'courtesy': job.contact_id.title or 'Monsieur',
                                       'gender': job.contact_id.gender or 'man',
                                       'categs': job.function_code_label or '',
                                       'final_last_name':job.contact_id.name,
                                       'final_first_name':job.contact_id.first_name or '',
                                       'final_email': job.email or job.contact_id.email or '',
                                       'final_mobile': _convert_phone( job.contact_id.mobile or '', PHONE_COUNTRY_PREFIX ),
                                       'final_mobile_confidential':'conf' in (job.contact_id.mobile or ''),
                                       'final_title': job.function_label or '',
                                       'final_categs': job.function_code_label or '',
                                       'final_gender': job.contact_id.gender or 'man',
                                       'final_courtesy': job.contact_id.title or 'Monsieur',
                                       'edition':DIRECTORY_EDITION,
                                      }
                            selected_jobs.append(new_job)
                if selected_jobs:
                    selected_jobs = sorted(selected_jobs,key= lambda x: x['sequence'])
                found_g = False
                new_sequence = 1
                # before searching for first G, we must exclude the cases where there is a '@' contact on this address or a '!' contact on the partner
                if addr.job_ids:
                    for addr_job in addr.job_ids:
                        if addr_job.function_code_label and ('@' in addr_job.function_code_label) and addr_job.contact_id and (addr_job.email or addr_job.contact_id.email):
                            sending_address = addr_job.email or addr_job.contact_id.email
                            sending_courtesy = addr_job.contact_id.title or ''
                            sending_name = addr_job.contact_id.name
                            found_g = True
                            break
                if (not sending_address) and addr.partner_id and (not found_g):
                    for partner_addr in addr.partner_id.address:
                        if partner_addr.job_ids:
                            for partner_job in partner_addr.job_ids:
                                if partner_job.function_code_label and ('!' in partner_job.function_code_label) and partner_job.contact_id and (partner_job.email or partner_job.contact_id.email):
                                    sending_address = partner_job.email or partner_job.contact_id.email
                                    sending_courtesy = partner_job.contact_id.title or ''
                                    sending_name = partner_job.contact_id.name
                                    found_g = True
                                    break
                        if found_g:
                            break
                for new_job in selected_jobs:
                    if new_job['email'] and (not sending_address or (sending_address and (not found_g) and ('G' in new_job['categs']))):
                        sending_address = new_job['email']
                        sending_courtesy = new_job['courtesy']
                        sending_name = new_job['last_name']
                        found_g = ('G' in new_job['categs'])

                    cont_count += 1
                    if new_job['sequence'] <= 5 and new_sequence <= 5:
                        new_job['sequence'] = new_sequence
                    else:
                        new_job['sequence'] = 99
                    cont_proxy_ids = cont_proxy_obj.search(cr,uid,[('address_proxy_id','=',addr_proxy_id),('job_id','=',new_job['job_id'])])
                    if cont_proxy_ids:
                        cont_proxy = cont_proxy_obj.read(cr,uid,cont_proxy_ids,['sequence','last_name','first_name','email','mobile','mobile_confidential','title','categs'])[0]
                        the_same = True
                        for field_name in ['sequence','last_name','first_name','email','mobile','mobile_confidential','title','categs']:
                            if cont_proxy[field_name] != new_job[field_name]:
                                the_same = False
                        if not the_same:
                            cont_proxy_obj.write(cr,uid,cont_proxy_ids,new_job)
                    else:
                        cont_proxy_obj.create(cr,uid,new_job)
                    new_sequence += 1
                if not sending_address and addr.email and addr.email[0:5] != 'info@':
                    sending_address = addr.email
                    sending_courtesy = 'Cher'
                    sending_name = 'membre'
                # new on 2015-12-11
                # if member with a zip_code or a vat number not belgian, no sending_address, it must be corrected by paper sending
                # because the web form can't manage strangers VAT ou zip codes
                if (partner.vat and partner.vat[0:2].upper() != 'BE'):
                    sending_address = ''
                    sending_courtesy = ''
                    sending_name = ''
                else:
                    belgian = False
                    if addr.zip_id and addr.zip_id.groups_id:
                        for group in addr.zip_id.groups_id:
                            if group.name == u'Belgique' and len(group.name) == 8:
                                belgian = True
                    if not belgian:
                        sending_address = ''
                        sending_courtesy = ''
                        sending_name = ''
                if sending_address and addr_proxy_id in modifiable_proxy_ids:
                    addr_proxy_obj.write(cr,uid,[addr_proxy_id,],{'sending_address':sending_address,'sending_name':sending_name,'sending_courtesy':sending_courtesy})
                    email_count += 1
                else:
                    # adding to the 'no email address text'
                    streets = addr.street or ''
                    if addr.street2:
                        streets += '\n  ' + addr.street2
                    partner_no_email += '\n  Adresse :\n  %s\n  %s' % (streets,(addr.zip_id.name or '') + ' ' + (addr.zip_id.city or ''))
                    if selected_jobs:
                        partner_no_email += '\n    Personnes de contact sans adresse de courriel:'
                        for job in selected_jobs:
                            partner_no_email += '\n    ' + job['courtesy'] + ' ' + job['last_name'] + ' ' + job['first_name']
            if addr_proxy_id and addr_proxy_id not in modifiable_proxy_ids:
                modifiable_proxy_ids = modifiable_proxy_ids.append(addr_proxy_id)
        return (partner_no_email,addr_count,modif_count,email_count,cont_count)
    def _daily_update(self,cr,uid,new_table=False,raz_exclude=False):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'Memberdirectory_DailyUpdate')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Daily updates of the memberdirectory\'s tables\n')
        #
        addr_proxy_obj = self.pool.get('directory.address.proxy')
        cont_proxy_obj = self.pool.get('directory.job.proxy')
        access_proxy_obj = self.pool.get('directory.address.access')
        # extract the ID of the website
        obj_user = self.pool.get('res.users')
        website_id = obj_user.search(cr,uid,[('name','=','Noomia')])[0]
        #
        # before checking for modifiable addresses, we re-active the addresses modified by user but confirmed by the CCI
        #if not new_table: # we are not re-creating all the table
        #    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nReactivating records validated by user AND CCI :\n')
        #    addr_ids = addr_ids = addr_proxy_obj.search(cr,uid,[('user_validated','=',True),('internal_validated','=',True)])
        #    if addr_ids:
        #        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,','.join(str(x) for x in addr_ids) )
        #
        modifiable_proxy_ids = []
        if new_table:
            cont_ids = cont_proxy_obj.search(cr,uid,[])
            cont_proxy_obj.write(cr,uid,cont_ids,{'active':False})
            addr_ids = addr_proxy_obj.search(cr,uid,[])
            addr_proxy_obj.write(cr,uid,addr_ids,{'active':False})
            used_res_partner_address_ids = []
        else:
            existing_addr_ids = addr_proxy_obj.search(cr,uid,[])
            existing_addresses = addr_proxy_obj.read(cr,uid,existing_addr_ids,['address_id','write_uid','user_validated','internal_validated'])
            used_res_partner_address_ids = [x['address_id'][0] for x in existing_addresses]
            modifiable_proxy_ids = [x['id'] for x in existing_addresses if not x['write_uid'] or (x['write_uid'] and x['write_uid'][0] <> website_id and not x['user_validated'] and not x['internal_validated'])]
        # extract all ids of activity sector categories and remove '[303]' from name
        obj_categ = self.pool.get('res.partner.activsector')
        categ_ids = obj_categ.search(cr,uid,[])
        categs = obj_categ.read(cr,uid,categ_ids,['code'])
        dCategs = {}
        dSectCode2ID = {}
        for categ in categs:
            dCategs[categ['id']] = categ['code']
            dSectCode2ID[categ['code']] = categ['id']
        # table for converting old activity sectors to new ones
        dOldSectID2NewID = {}
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['ActivityCodeRootID'])
        if param_values.has_key('ActivityCodeRootID'):
            activityCodeRootID = param_values['ActivityCodeRootID']
            obj_oldcateg = self.pool.get('res.partner.category')
            old_len = 0
            oldcateg_ids = [ activityCodeRootID ]
            while len(oldcateg_ids) > old_len:
                new_ids = oldcateg_ids[old_len:] # ids of categories added last time
                old_len = len(oldcateg_ids) # the new frontier ...
                new_categs = obj_oldcateg.read(cr,uid,new_ids,['child_ids'])
                for categ in new_categs:
                    if categ['child_ids']:
                        oldcateg_ids.extend(categ['child_ids'])
            oldcategs = obj_oldcateg.read(cr,uid,oldcateg_ids,['name'],{'lang':'fr_FR'})
            for oldcateg in oldcategs:
                formated_name = oldcateg['name']
                posA = formated_name.rfind('[')
                posB = formated_name.rfind(']')
                if posA >= 0 and posB > 0 and posA < posB:
                    code = formated_name[posA+1:posB]
                    if dSectCode2ID.has_key(code):
                        dOldSectID2NewID[oldcateg['id']] = dSectCode2ID[code]
        
        # Extraction of data
        partner_obj = self.pool.get('res.partner')
        addr_obj = self.pool.get('res.partner.address')
        PHONE_COUNTRY_PREFIX = self.get_phone_country_prefix(cr,uid)
        addr_count = 0
        modif_count = 0
        not_in_directory_count = 0
        cont_count = 0
        email_count = 0
        deleted_count = 0
        member_wo_address = []
        partners_wo_emails = ''
        partner_ids = partner_obj.search(cr,uid,[('state_id','=',1),('membership_state','in',['free','invoiced','paid'])])
        # adding the current partners with subscriptions to electronic memberdirectory for full page
        # while keeping their address_ids for setting 'full_page_app'
        full_page_partner_ids = []
        subs_partner_ids = {}
        in_five_days = (datetime.datetime.today()+datetime.timedelta(days=5)).strftime('%Y-%m-%d')
        thelastday = datetime.datetime.today().strftime('%Y-%m-%d')
        sub_type_obj = self.pool.get('partner_subscription.type')
        sub_type_ids = sub_type_obj.search(cr,uid,[('code','=','SOCREPELEC')])
        subscription_obj = self.pool.get('partner_subscription')
        subscription_ids = subscription_obj.search(cr,uid,[('type_id','in',sub_type_ids),('state','=','current')])
        subscriptions = subscription_obj.browse(cr,uid,subscription_ids)
        for subs in subscriptions:
            if subs.state == 'current' and subs.begin and (subs.begin <= in_five_days) and subs.end and (subs.end >= thelastday) and subs.partner_id:
                if not subs.partner_id.dir_exclude:
                    if subs.partner_id.id not in partner_ids:
                        partner_ids.append(subs.partner_id.id)
                    if subs.partner_id.id not in subs_partner_ids:
                        data_sub = {'id':subs.partner_id.id,
                                'amount':subs.price,
                                'begindate':subs.begin,
                                'enddate':subs.end,
                               }
                        subs_partner_ids[subs.partner_id.id] = data_sub
                    else:
                        data_sub = subs_partner_ids[subs.partner_id.id]
                        data_sub['amount'] += subs.price
                        data_sub['begindate'] = min(data_sub['begindate'],subs.begin)
                        data_sub['enddate'] = max(data_sub['enddate'],subs.end)
                        subs_partner_ids[subs.partner_id.id] = data_sub
        for (partner_id,data_sub) in subs_partner_ids.items():
            full_page_partner_ids.append(partner_id)
            if data_sub['amount'] > 0.001 and partner_id not in partner_ids:
                partner_ids.append(partner_id)
        # 2017-03-14 adding all manually added partners within free 'publication' dates
        added_obj = self.pool.get('directory.manual_add')
        ctoday = datetime.date.today().strftime('%Y-%m-%d')
        added_ids = added_obj.search(cr,uid,[('from_date','<=',ctoday),('to_date','>=',ctoday)])
        if added_ids:
            addeds = added_obj.read(cr,uid,added_ids,['partner_id'])
            for added_partner in addeds:
                print added_partner
                if added_partner['partner_id'] and added_partner['partner_id'][0] not in partner_ids:
                    partner_ids.append(added_partner['partner_id'][0])
        # adding the current freemium members with subscriptions to electronic memberdirectory for full page
        # while keeping their address_ids for setting 'full_page_app'
        full_page_contact_ids = []
        subs_contact_partner_ids = {}
        in_five_days = (datetime.datetime.today()+datetime.timedelta(days=5)).strftime('%Y-%m-%d')
        thelastday = datetime.datetime.today().strftime('%Y-%m-%d')
        sub_type_obj = self.pool.get('premium_subscription.type')
        sub_type_ids = sub_type_obj.search(cr,uid,[('code','=','REPMEMBRE')])
        subscription_obj = self.pool.get('premium_subscription')
        subscription_ids = subscription_obj.search(cr,uid,[('type_id','in',sub_type_ids),('state','=','current')])
        subscriptions = subscription_obj.browse(cr,uid,subscription_ids)
        for subs in subscriptions:
            if subs.state == 'current' and subs.begin and (subs.begin <= in_five_days) and subs.end and (subs.end >= thelastday) and subs.contact_id and subs.contact_id.job_ids:
                for possible_job in subs.contact_id.job_ids:
                    if possible_job.address_id and not possible_job.address_id.dir_exclude and possible_job.address_id.active and possible_job.address_id.partner_id \
                        and not possible_job.address_id.partner_id.dir_exclude and possible_job.address_id.partner_id.state_id.id == 1 and possible_job.address_id.partner_id.active:
                            if subs.partner_id.id not in partner_ids:
                                partner_ids.append(subs.partner_id.id)
                            if subs.partner_id.id not in subs_contact_partner_ids:
                                data_sub = {'id':possible_job.address_id.partner_id.id,
                                        'amount':subs.price,
                                        'begindate':subs.begin,
                                        'enddate':subs.end,
                                       }
                                subs_contact_partner_ids[possible_job.address_id.partner_id.id] = data_sub
                            else:
                                data_sub = subs_contact_partner_ids[possible_job.address_id.partner_id.id]
                                data_sub['amount'] += subs.price
                                data_sub['begindate'] = min(data_sub['begindate'],subs.begin)
                                data_sub['enddate'] = max(data_sub['enddate'],subs.end)
                                subs_contact_partner_ids[possible_job.address_id.partner_id.id] = data_sub
        for (partner_id,data_sub) in subs_contact_partner_ids.items():
            full_page_partner_ids.append(partner_id)
            if data_sub['amount'] > 0.001 and partner_id not in partner_ids:
                partner_ids.append(partner_id)
        #
        partners = partner_obj.browse(cr,uid,partner_ids,{'lang':'fr_FR'})
        for partner in partners:
            if partner.dir_exclude and not raz_exclude:
                to_del_addr_proxy_ids = addr_proxy_obj.search(cr,uid,[('partner_id','=',partner.id)])
                if to_del_addr_proxy_ids:
                    to_del_addr_proxies = addr_proxy_obj.read(cr,uid,to_del_addr_proxy_ids,['address_id'])
                    to_del_addr_ids = []
                    for to_del_addr_proxy in to_del_addr_proxies:
                        if to_del_addr_proxy['address_id']:
                            to_del_addr_ids.append(to_del_addr_proxy['address_id'][0])
                    to_del_job_proxy_ids = cont_proxy_obj.search(cr,uid,[('address_proxy_id','in',to_del_addr_proxy_ids)])
                    if to_del_addr_ids:
                        to_del_access_proxy_ids = access_proxy_obj.search(cr,uid,[('address_id','in',to_del_addr_ids)])
                        if to_del_access_proxy_ids:
                            access_proxy_obj.unlink(cr,1,to_del_access_proxy_ids)
                            deleted_count += len(to_del_access_proxy_ids)
                    if to_del_job_proxy_ids:
                        cont_proxy_obj.unlink(cr,uid,to_del_job_proxy_ids)
                        deleted_count += len(to_del_job_proxy_ids)
                    addr_proxy_obj.unlink(cr,uid,to_del_addr_proxy_ids)
                    deleted_count += len(to_del_addr_proxy_ids)
                not_in_directory_count += 1
            else:
                if partner.dir_exclude and raz_exclude:
                    correction = {'dir_exclude':False}
                    partner_obj.write(cr,uid,[partner.id,],correction)
                if partner.address:
                    partner_no_email = ''
                    for addr in partner.address:
                        if addr.dir_exclude and raz_exclude:
                            correction = {'dir_exclude':False}
                            addr_obj.write(cr,uid,[addr.id,],correction)
                        if addr.dir_exclude and not raz_exclude:
                            to_del_addr_proxy_ids = addr_proxy_obj.search(cr,uid,[('address_id','=',addr.id)])
                            if to_del_addr_proxy_ids:
                                to_del_addr_proxies = addr_proxy_obj.read(cr,uid,to_del_addr_proxy_ids,['address_id'])
                                to_del_addr_ids = []
                                for to_del_addr_proxy in to_del_addr_proxies:
                                    if to_del_addr_proxy['address_id']:
                                        to_del_addr_ids.append(to_del_addr_proxy['address_id'][0])
                                to_del_job_proxy_ids = cont_proxy_obj.search(cr,uid,[('address_proxy_id','in',to_del_addr_proxy_ids)])
                                if to_del_addr_ids:
                                    to_del_access_proxy_ids = access_proxy_obj.search(cr,uid,[('address_id','in',to_del_addr_ids)])
                                    if to_del_access_proxy_ids:
                                        access_proxy_obj.unlink(cr,1,to_del_access_proxy_ids)
                                        deleted_count += len(to_del_access_proxy_ids)
                                if to_del_job_proxy_ids:
                                    cont_proxy_obj.unlink(cr,uid,to_del_job_proxy_ids)
                                    deleted_count += len(to_del_job_proxy_ids)
                                addr_proxy_obj.unlink(cr,uid,to_del_addr_proxy_ids)
                                deleted_count += len(to_del_addr_proxy_ids)
                            not_in_directory_count += 1
                        else:
                            (errors,res_addr_count,res_modif_count,res_email_count,res_cont_count) = addr_proxy_obj._synchro_address(cr,uid,addr,partner,modifiable_proxy_ids,full_page_partner_ids,new_table,website_id,dCategs,dSectCode2ID,dOldSectID2NewID,oldcateg_ids)
                            if errors:
                                partner_no_email = '\n-----------------------------\nPartenaire : %s [%s] %s' % (partner.name,partner.id,errors)
                                partners_wo_emails += partner_no_email
                            addr_count += res_addr_count
                            modif_count += res_modif_count
                            email_count += res_email_count
                            cont_count += res_cont_count
                else:
                    member_wo_address.append(partner.id)
        results = 'Selected Partners : %s\nPublished Addresses : %s\nDirectory Exclude cleaned : %s\nDeleted records because excluded : %s\nContacts found : %s\nEmail Count : %s\nMembers without addresses : [%s]' % (str(len(partner_ids)),
                                                                                                                                               str(addr_count),
                                                                                                                                               str(not_in_directory_count),
                                                                                                                                               str(deleted_count),
                                                                                                                                               str(cont_count),
                                                                                                                                               str(email_count),
                                                                                                                                               ','.join([str(x) for x in member_wo_address]))
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,results)
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,True,'\n---end---')
        return True
    def send_email(self,cr,uid,addr_ids,body_template,body_add_message,body_item_message,from_email,title,recording):
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['BDDeTest'])
        if param_values.has_key('BDDeTest'):
            bddetest = param_values['BDDeTest']
        else:
            bddetest = True
        #
        addr_proxy_obj = self.pool.get('directory.address.proxy')
        sended_emails = []
        sended_ids = []
        errors = []
        if addr_ids:
            proxies = addr_proxy_obj.browse(cr,uid,addr_ids)
            for proxy in proxies:
                partner_address = proxy.final_street
                if proxy.final_street_number:
                    partner_address += ( ' ' + proxy.final_street_number )
                if proxy.final_street_box:
                    partner_address += ( ' BTE ' + proxy.final_street_box )
                if proxy.final_zip_id:
                    partner_address += ( ' - ' + proxy.final_zip_id.name + ' ' + proxy.final_zip_id.city )
                email_content = body_template.replace('[courtesy]',proxy.sending_courtesy or '').replace('[name]',proxy.sending_name or 'cher Membre').replace('[link_id]',proxy.link_id)
                email_content = email_content.replace('[partner_name]',proxy.final_partner_name or '').replace('[partner_address]',partner_address or '')
                if '[others_data]' in body_template and proxy.partner_id:
                    other_ids = addr_proxy_obj.search(cr,uid,[('partner_id','=',proxy.partner_id.id),('id','<>',proxy.id),('sending_address','<>',False),('user_validated','=',False)])
                    if other_ids:
                        others = addr_proxy_obj.read(cr,uid,other_ids,['complete_name','sending_courtesy','sending_name','sending_address','link_id'])
                        others_list = []
                        for other in others:
                            others_list.append(body_item_message % (other['complete_name'],
                                                                    other['sending_name'] and (u'"' + other['sending_courtesy']+ ' ' + other['sending_name'] + '"') or u'l\'adresse du siège',
                                                                    other['sending_address'],
                                                                    other['link_id'],
                                                                    other['link_id']))
                        optional_message = body_add_message % u'</li><li>'.join(others_list)
                        email_content = email_content.replace('[others_data]',optional_message)
                    else:
                        email_content = email_content.replace('[others_data]','')
                email_address = proxy.sending_address
                if email_address:
                    #email_address = 'philmer@ccilvn.be' ## TODO delete
                    try:
                        if bddetest:
                            print 'mocked email send'
                            print 'From : \'%s\'' % from_email
                            print 'To : [\'%s\']' % email_address
                            print 'Subject : %s' % title
                            print 'Body :\n%s' % email_content
                        else:
                            tools.email_send(from_email, [email_address], title, email_content, subtype='html')
                        self.pool.get('cci_email_sended').create(cr,uid,{'name':u'Envoi groupé fiche MAJ partenaire',
                                                                         'subject':title,
                                                                         'body':email_content,
                                                                         'ressource':'directory.address.proxy:'+str(proxy.id)+':',
                                                                         'addr_from':from_email,
                                                                         'addrs_to':'[' + email_address +',]',
                                                                        })
                        #print email_content
                        sended_emails.append( email_address)
                        sended_ids.append(proxy.id)
                        # recording of the sending after each one, so, if we have a problem, we can determine what is already sent...
                        if recording == 'initial':
                            addr_proxy_obj.write(cr,uid,[proxy.id,],{'send_initial_mail':datetime.date.today().strftime('%Y-%m-%d')})
                        elif recording == '1followup':
                            addr_proxy_obj.write(cr,uid,[proxy.id,],{'send_first_followup':datetime.date.today().strftime('%Y-%m-%d')})
                    except:
                        errors.append('Wrong address : %s' % email_address)
                else:
                    errors.append('No email address : proxy id [%s]' % str(proxy.id))
            results = (sended_ids,sended_emails,errors)
        else:
            results = False
        return results
    def send_partner_email(self,cr,uid,partner_id,body_template,body_add_message,body_item_message,from_email,to_email,subject):
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['BDDeTest'])
        bddetest = False
        if param_values.has_key('BDDeTest'):
            bddetest = (param_values['BDDeTest'] == 1)
        #
        errors = []
        if partner_id:
            partner = self.pool.get('res.partner').browse(cr,uid,[partner_id,])[0]
            proxy_ids = self.pool.get('directory.address.proxy').search(cr,uid,[('partner_id','=',partner.id)])
            if proxy_ids:
                first_proxy = False
                proxies = self.pool.get('directory.address.proxy').browse(cr,uid,proxy_ids)
                for proxy in proxies:
                    if proxy.address_id and proxy.address_id.type == 'default':
                        first_proxy = proxy
                if not first_proxy:
                    first_proxy = proxies[0]
                if first_proxy:
                    partner_address = first_proxy.final_street
                    if first_proxy.final_street_number:
                        partner_address += ( ' ' + first_proxy.final_street_number )
                    if first_proxy.final_street_box:
                        partner_address += ( ' BTE ' + first_proxy.final_street_box )
                    if first_proxy.final_zip_id:
                        partner_address += ( ' - ' + first_proxy.final_zip_id.name + ' ' + first_proxy.final_zip_id.city )
                    email_content = body_template.replace('[name]',u'cher Membre').replace('[link_id]',first_proxy.link_id)
                    email_content = email_content.replace('[partner_name]',first_proxy.final_partner_name or '').replace('[partner_address]',partner_address or '')
                    if '[others_data]' in body_template and first_proxy.partner_id and len(proxy_ids) > 1:
                        other_ids = [x.id for x in proxies if x.id <> first_proxy.id]
                        if other_ids:
                            others = self.pool.get('directory.address.proxy').read(cr,uid,other_ids,['complete_name','sending_courtesy','sending_name','sending_address','link_id'])
                            others_list = []
                            for other in others:
                                others_list.append(body_item_message % (other['complete_name'],
                                                                        other['sending_name'] and (u'"' + other['sending_courtesy']+ ' ' + other['sending_name'] + '"') or u'l\'adresse du siège',
                                                                        other['sending_address'],
                                                                        other['link_id'],
                                                                        other['link_id']))
                            optional_message = body_add_message % u'</li><li>'.join(others_list)
                            email_content = email_content.replace('[others_data]',optional_message)
                        else:
                            email_content = email_content.replace('[others_data]','')
                    email_address = to_email
                    sended_emails = []
                    if email_address:
                        #email_address = 'philmer@ccilvn.be' ## TODO delete
                        try:
                            if bddetest:
                                print 'mocked email send'
                                print 'From : \'%s\'' % from_email
                                print 'To : [\'%s\']' % email_address
                                print 'Subject : %s' % subject
                                print 'Body :\n%s' % email_content
                            else:
                                tools.email_send(from_email, [email_address], subject, email_content, subtype='html')
                            #print email_content
                            sended_emails.append(email_address)
                            sended_ids.append(proxy.id)
                        except:
                            errors.append('Wrong address : %s' % email_address)
                    else:
                        errors.append('No email address : [%s]' % to_email)
                results = (proxy_ids,sended_emails,errors)
            else:
                results = False
        else:
            results = False
        return results
directory_address_proxy()

class directory_job_proxy(osv.osv):
    _name = "directory.job.proxy"
    _description = "proxy job 4 yearly directory"
    def _title_get(self,cr, user, context={}):
        obj = self.pool.get('res.partner.title')
        ids = obj.search(cr, user, [])
        res = obj.read(cr, user, ids, ['shortcut', 'name','domain'], context)
        res = [(r['shortcut'], r['name']) for r in res if r['domain']=='contact']
        return res
    def name_get(self, cr, uid, ids, context=None):
        if not len(ids):
            return []
        jobs = self.browse(cr, uid, ids, context=context)
        res = []
        for record in reads:
            name = record.contact_id.name + ' ' + (record.contact_id.first_name or '')
            res.append((record['id'], name))
        return res
    _columns = {
        'address_proxy_id': fields.many2one('directory.address.proxy','Partner Proxy',required=True,ondelete='cascade'),
        'address_id': fields.many2one('res.partner.address','Address',ondelete='cascade'),
        'job_id': fields.many2one('res.partner.job','Job',ondelete='cascade'),
        'contact_id': fields.many2one('res.partner.contact','Contact',ondelete='cascade'),
        'sequence': fields.integer('Sequence',required=True),
        'last_name': fields.char('Name', size=30),
        'first_name': fields.char('First Name', size=30),
        'email': fields.char('EMail', size=240),
        'mobile':fields.char('Mobile',size=40),
        'mobile_confidential':fields.boolean('Mobile confidential'),
        'title': fields.char('Title', size=250),
        #'birthdate':fields.date('Birthdate'),
        #'linkedin_url':fields.char('LinkedIN',size=200),
        'categs': fields.char('Categs',size=10),
        'courtesy': fields.selection(_title_get, 'Courtesy'),
        'gender':fields.selection([('man','Man'),('women','Women')],"Gender"),
        'new_sequence': fields.integer('New Sequence'),
        'new_last_name': fields.char('New Name', size=30),
        'new_first_name': fields.char('New First Name', size=30),
        'new_email': fields.char('New EMail', size=240),
        'new_mobile':fields.char('New Mobile',size=40),
        'new_mobile_confidential':fields.boolean('New Mobile confidential'),
        'new_title': fields.char('New Title', size=250),
        #'new_birthdate':fields.date('New Birthdate'),
        #'new_linkedin_url':fields.char('New LinkedIN',size=200),
        'new_courtesy': fields.selection(_title_get, 'Courtesy'),
        'final_last_name': fields.char('Final Name', size=30),
        'final_first_name': fields.char('Final First Name', size=30),
        'final_email': fields.char('Final EMail', size=240),
        'final_mobile':fields.char('Final Mobile',size=40),
        'final_mobile_confidential':fields.boolean('Final Mobile confidential'),
        'final_title': fields.char('Final Title', size=250),
        'final_sequence': fields.integer('Final Sequence'),
        #'final_birthdate':fields.date('Final Birthdate'),
        #'final_linkedin_url':fields.char('Final LinkedIN',size=200),
        'final_courtesy': fields.selection(_title_get, 'Final Title'),
        'final_gender':fields.selection([('man','Man'),('women','Women')],"Final Gender"),
        'final_categs': fields.char('Final Categs',size=10),
        'internal_validated': fields.boolean('Internal Validated'),
        'new_record': fields.boolean('New Contact'),
        'change_date':fields.datetime('Change Date'),
        'marked_for_deletion':fields.boolean('Marked For Deletion'),
        'active':fields.boolean('Active'),
        'edition':fields.char('Edition',size=10),
    }
    _defaults = {
        'internal_validated' : lambda *a : False,
        'new_record' : lambda *a : False,
        'mobile_confidential': lambda *a: False,
        'new_mobile_confidential': lambda *a: False,
        'marked_for_deletion': lambda *a: False,
        'active': lambda *a: True,
    }
    def create(self, cr, uid, vals,*args, **kwargs):
        if not vals.has_key('sequence'):
            vals.update({'sequence':99}) # because sequence is required ...
        if not vals.has_key('edition'):
            parameter_obj = self.pool.get('cci_parameters.cci_value')
            param_values = parameter_obj.get_value_from_names(cr,uid,['LastDirectoryEdition'])
            if param_values.has_key('LastDirectoryEdition'):
                DIRECTORY_EDITION = param_values['LastDirectoryEdition']
            else:
                DIRECTORY_EDITION = str(datetime.date.today().year+1)
            vals.update({'edition':DIRECTORY_EDITION})
        if vals.has_key('new_last_name'):
            vals.update({'final_last_name':vals['new_last_name'].upper()})
        if vals.has_key('new_first_name'):
            vals.update({'final_first_name':vals['new_first_name'].title()})
        if vals.has_key('new_email'):
            vals.update({'final_email':vals['new_email'].lower()})
        if vals.has_key('new_sequence'):
            vals.update({'final_sequence':vals['new_sequence']})
        if vals.has_key('new_title'):
            vals.update({'final_title':vals['new_title']})
        if vals.has_key('new_mobile'):
            vals.update({'final_mobile':_only_digits(vals['new_mobile'] or '')})
        if vals.has_key('new_mobile_confidential'):
            vals.update({'final_mobile_confidential':vals['new_mobile_confidential']})
        if vals.has_key('new_courtesy'):
            vals.update({'final_courtesy':vals['new_courtesy']})
            vals.update({'final_gender':(vals['new_courtesy']=='Madame') and 'women' or 'man'})
        return super(osv.osv,self).create(cr, uid, vals, *args, **kwargs)
    def write(self, cr, uid, ids,vals,context=None):
        current = self.pool.get('directory.job.proxy').browse(cr,uid,ids)[0]
        if vals.has_key('job_id'):
            if not vals['job_id']:
                vals.update({'new_record':True,
                             'final_last_name':vals.get('new_last_name',''),
                             'final_first_name':vals.get('new_first_name',''),
                             'final_email':vals.get('new_email',''),
                             'final_title':vals.get('new_title',''),
                             'final_categs':'',
                             'final_gender':(vals.get('new_courtesy','Monsieur')=='Madame') and 'women' or 'man',
                             'final_courtesy':vals.get('new_courtesy','Monsieur'),
                            })
            else:
                if not vals.has_key('contact_id'):
                    job = self.pool.get('res.partner.job').browse(cr,uid,[vals['job_id']])[0]
                    vals.update({'contact_id':job.contact_id.id})
        if vals.has_key('new_last_name'):
            vals.update({'final_last_name':vals['new_last_name']})
        if vals.has_key('new_first_name'):
            vals.update({'final_first_name':vals['new_first_name'].title()})
        if vals.has_key('new_email'):
            vals.update({'final_email':vals['new_email']})
        if vals.has_key('new_sequence'):
            vals.update({'final_sequence':vals['new_sequence']})
        if vals.has_key('new_title'):
            vals.update({'final_title':vals['new_title']})
        if vals.has_key('new_mobile'):
            vals.update({'final_mobile':_only_digits(vals['new_mobile'] or '')})
        if vals.has_key('new_mobile_confidential'):
            vals.update({'final_mobile_confidential':vals['new_mobile_confidential']})
        if vals.has_key('new_courtesy'):
            vals.update({'final_courtesy':vals['new_courtesy']})
            vals.update({'final_gender':(vals['new_courtesy']=='Madame') and 'women' or 'man'})
        vals.update({'internal_validated':False})
        return super(osv.osv,self).write(cr, uid, ids,vals, context)
directory_job_proxy()

class directory_job2delete(osv.osv):
    _name = "directory.job2delete"
    _description = "initial jobs marked for deletion"
    _columns = {
        'address_proxy_id': fields.many2one('directory.address.proxy','Partner Proxy',ondelete='cascade'),
        'job_id': fields.integer('Job ID'),
        'last_name': fields.char('Last Name',size=50),
        'first_name': fields.char('First Name',size=50),
        'active': fields.boolean('Active'),
    }
    _defaults = {
        'active' : lambda *a : True,
    }
    def create(self, cr, uid, vals,context=None):
        if vals.has_key('job_id'):
            job_ids = self.pool.get('directory.job.proxy').search(cr,uid,[('job_id','=',vals['job_id'])])
            if job_ids and len(job_ids) == 1:
                job = self.pool.get('directory.job.proxy').read(cr,uid,[job_ids[0]],['address_proxy_id','last_name','first_name'])[0]
                vals.update({'address_proxy_id':job['address_proxy_id'][0],'last_name':job['last_name'] or '','first_name':job['first_name'] or ''})
        return super(osv.osv,self).create(cr, uid, vals, context)
directory_job2delete()

class directory_complex(osv.osv):
    _name = "directory.complex"
    _description = 'Partner ID of complex cases to isolate them from simple ones'
    _columns = {
        'partner_id':fields.many2one('res.partner','Partner',required=True),
    }
directory_complex()

class directory_manual_add(osv.osv):
    _name = "directory.manual_add"
    _description = 'Partner ID of partners to be added between two dates'
    _columns = {
        'partner_id':fields.many2one('res.partner','Partner',required=True),
        'from_date': fields.date('From',required=True),
        'to_date':fields.date('To',required=True),
        'asker_id':fields.many2one('res.users','Asker',required=True),
    }
    _defaults = {
        'asker_id': lambda obj, cr, uid, context: uid,
        'from_date': lambda *a: datetime.datetime.today().strftime('%Y-%m-%d'),
        'to_date': lambda *a: (datetime.datetime.today()+datetime.timedelta(days=92)).strftime('%Y-%m-%d'),
    }
directory_manual_add()

class directory_fullpage(osv.osv):
    _name = "directory.fullpage"
    _description = "folder to follow the prospects of full pages in directory"
    def _get_salesman(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for fullpage in self.browse(cr,uid,ids):
            if fullpage.partner_id:
                res[fullpage.id] = fullpage.partner_id.user_id.name
            else:
                res[fullpage.id] = ''
        return res
    def _get_sale_order_amount(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for fullpage in self.browse(cr,uid,ids):
            if fullpage.sale_order_id and fullpage.sale_order_id.state not in ['draft','cancel']:
                res[fullpage.id] = fullpage.sale_order_id.amount_untaxed
            else:
                res[fullpage.id] = 0.0
        return res
    def _get_sale_order_state(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for fullpage in self.browse(cr,uid,ids):
            if fullpage.sale_order_id:
                res[fullpage.id] = fullpage.sale_order_id.state
                if fullpage.sale_order_id.invoiced:
                    res[fullpage.id] += ' (paid)'
            else:
                res[fullpage.id] = 'None'
        return res
    def _get_invoices_amount(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for fullpage in self.browse(cr,uid,ids):
            res[fullpage.id] = 0.0
            if fullpage.invoice_ids:
                for inv in fullpage.invoice_ids:
                    multiplier = 1
                    if inv.journal_id and inv.journal_id.refund_journal:
                        multiplier = -1
                    res[fullpage.id] += ( inv.amount_untaxed * multiplier )
        return res
    def _get_products(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for fullpage in self.browse(cr,uid,ids):
            products = ''
            if fullpage.sale_order_id and fullpage.sale_order_id.order_line:
                for so_line in fullpage.sale_order_id.order_line:
                    if so_line.product_id and so_line.product_id.code:
                        if products:
                            products += ( ', ' + so_line.product_id.code )
                        else:
                            products = so_line.product_id.code
            res[fullpage.id] = products
        return res
    def _get_email_so(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for fullpage in self.browse(cr,uid,ids):
            email = ''
            if fullpage.partner_id and fullpage.so_contact_id:
                if fullpage.partner_id.address:
                    for addr in fullpage.partner_id.address:
                        if addr.job_ids:
                            for job in addr.job_ids:
                                if job.contact_id and job.contact_id.id == fullpage.so_contact_id.id:
                                    email = job.email or job.contact_id.email or ''
                                    if email:
                                        break
                        if email:
                            break
            res[fullpage.id] = email
        return res
    def _get_email_material(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for fullpage in self.browse(cr,uid,ids):
            email = ''
            if fullpage.partner_id and fullpage.material_contact_id:
                if fullpage.partner_id.address:
                    for addr in fullpage.partner_id.address:
                        if addr.job_ids:
                            for job in addr.job_ids:
                                if job.contact_id and job.contact_id.id == fullpage.material_contact_id.id:
                                    email = job.email or job.contact_id.email or ''
                                    if email:
                                        break
                        if email:
                            break
            res[fullpage.id] = email
        return res
    def _salesman_search(self, cr, uid, ids, field_name, args, context=None):
        if context is None:
            context = {}
        if not len(args):
            return []
        operator = args[0][1]
        value = args[0][2]
        u_ids= self.pool.get('res.users').search(cr,uid,[('name', operator, value)])
        fp_ids = []
        if u_ids:
            sql = """SELECT fp.id FROM directory_fullpage as fp, res_partner as rp
                        WHERE fp.partner_id = rp.id AND rp.user_id in (%s)""" % (','.join([str(x) for x in u_ids]))
            cr.execute(sql)        
            res = cr.fetchall()
            if res:
                fp_ids = [x[0] for x in res]
        if not fp_ids:
            return [('id','=','0')]
        return [('id','in',fp_ids)]
    _columns = {
        'partner_id': fields.many2one('res.partner','Partner',required=True),
        'sorted_name': fields.char('Sorted Name',size=250),
        'so_contact_id': fields.many2one('res.partner.contact','SO Contact'),
        'material_contact_id': fields.many2one('res.partner.contact','Material Contact'),
        'email_so': fields.function(_get_email_so,type='string',size=254,method=True,string='Email Contact SO'),
        'email_material': fields.function(_get_email_material,type='string',size=254,method=True,string='Email Contact Material'),
        'sale_order_id': fields.many2one('sale.order','Sale Order'),
        'invoice_ids': fields.many2many('account.invoice', 'account_fullage_inv_rel', 'fullpage_id', 'invoice_id', 'Invoices'),
        'state': fields.selection([('draft','Prospect'),('confirm','Full Page confirmed'),('waiting_material','Waiting material'),('gfs_creation','Good for Shooting in creation'),('gfs_sended','Good for Shooting Sended'),('good_gfs','GFS OK'),('finish','GFS OK and verified')],'State'),
        'type': fields.selection([('normal','Normal'),('official','Official Partner'),('product','Product Partner'),('exchange','Other - Exchange')],'Type'),
        'value_amount': fields.float('Value Amount',digits=(15,2)),
        'so_amount': fields.function(_get_sale_order_amount,type='float',digits=(15,2),method=True,string='Order Amount'),
        'so_state': fields.function(_get_sale_order_state,type='char',size=30,method=True,string='Order state'),
        'inv_amount' : fields.function(_get_invoices_amount,type='float',digits=(15,2),method=True,string='Invoices Amount'),
        'products': fields.function(_get_products,type='char',size=50,method=True,string='Ordered products'),
        'salesman': fields.function(_get_salesman,type='char',size=50,method=True,string='Salesman', fnct_search=_salesman_search),
        'active': fields.boolean('Active'),
        'history_ids': fields.one2many('fullpage.history','fullpage_id','History'),
        'comments': fields.text('Comments'),
        'current_gfs': fields.integer('Current GfS number'),
    }
    _defaults = {
        'state': lambda *a: 'draft',
        'active' : lambda *a : True,
        'type': lambda *a: 'normal',
        'current_gfs': lambda *a: 0,
    }
    def set_draft(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state':'draft'})
        self._log_event(cr, uid, ids, _('Reset as prospect'))
        return True
    def set_confirm(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state':'confirm'})
        self._log_event(cr, uid, ids, _('Confirmed'))
        return True
    def set_waiting_material(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state':'waiting_material'})
        self._log_event(cr, uid, ids, _('Waiting for material'))
        return True
    def set_gfs_creation(self, cr, uid, ids, *args):
        current_gfs = self.read(cr,uid,ids[0],['current_gfs'])['current_gfs']
        current_gfs += 1
        self.write(cr, uid, ids, {'state':'gfs_creation','current_gfs':current_gfs})
        self._log_event(cr, uid, ids, _('Starting creation of Good for Shooting ') + str(current_gfs) )
        return True
    def set_gfs_sended(self, cr, uid, ids, *args):
        current_gfs = self.read(cr,uid,ids[0],['current_gfs'])['current_gfs']
        self.write(cr, uid, ids, {'state':'gfs_sended'})
        self._log_event(cr, uid, ids, _('Sending of Good for Shooting ') + str(current_gfs) )
        return True
    def set_good_gfs(self, cr, uid, ids, *args):
        current_gfs = self.read(cr,uid,ids[0],['current_gfs'])['current_gfs']
        self.write(cr, uid, ids, {'state':'good_gfs'})
        self._log_event(cr, uid, ids, _('Good for Shooting %i accepted as good') % current_gfs)
        return True
    def set_finish(self, cr, uid, ids, *args):
        current_gfs = self.read(cr,uid,ids[0],['current_gfs'])['current_gfs']
        self.write(cr, uid, ids, {'state':'finish'})
        self._log_event(cr, uid, ids, _('Good for Shooting %i verified in database') % current_gfs)
        return True
    def _log_event(self, cr, uid, ids, name):
        for fullpage_id in ids:
            event = {
                'fullpage_id': fullpage_id,
                'user_id': uid,
                'short': name,
                'hour': datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                'desc': '',
            }
            self.pool.get('fullpage.history').create(cr, uid, event)
    def onchange_partner_id(self, cr, uid, ids, partner_id, sorted_name):
        if not sorted_name and partner_id:
            partner = self.pool.get('res.partner').browse(cr, uid, partner_id)
            base_sorted_name = ''
            if partner.address:
                for addr in partner.address:
                    if addr.dir_sort_name:
                        if base_sorted_name:
                            if addr.dir_sort_name.strip() == base_sorted_name:
                                continue
                            else:
                                base_sorted_name = ''
                                break
                        else:
                            base_sorted_name = addr.dir_sort_name.strip()
            if not base_sorted_name:
                base_sorted_name = partner.name.strip()
            result = {'value': {'sorted_name': base_sorted_name,}}
            return result
        else:
            return True
    
directory_fullpage()

class fullpage_history(osv.osv):
    _name = "fullpage.history"
    _description = "log event in fullpage"
    _columns = {
        'fullpage_id': fields.many2one('directory.fullpage','Full Page',required=True),
        'user_id': fields.many2one('res.users','User'),
        'hour': fields.datetime('When'),
        'short': fields.char('What',size=100),
        'desc': fields.text('Description'),
    }
    _defaults = {
        'hour': lambda *a: datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
        'user_id' : lambda object,cr,uid,context: uid,
    }
    _order = "hour desc"
fullpage_history()

class res_partner_address(osv.osv):
    _inherit = "res.partner.address"
    _description = 'res.partner.address'
    _columns = {
        'dir_presence':fields.boolean('Dir. Presence',help='Force the presence of this address in the directory of the members for members but invoice address'),
    }
    _defaults = {
        'dir_presence' : lambda *a: False,
    }
res_partner_address()

class res_partner(osv.osv):
    _inherit = "res.partner"
    def unshadow_partner_id(self, cr, uid, shadowed_id):
        if len(shadowed_id) != 11:
            partner_id = False
        else:
            if int(shadowed_id[1]) != ((int(shadowed_id[10])+int(shadowed_id[8])) % 10):
                partner_id = False
            else:
                if int(shadowed_id[3]) != ((int(shadowed_id[8])+int(shadowed_id[6])) % 10):
                    partner_id = False
                else:
                    if int(shadowed_id[5]) != ((int(shadowed_id[6])+int(shadowed_id[4])) % 10):
                        partner_id = False
                    else:
                        if int(shadowed_id[7]) != ((int(shadowed_id[4])+int(shadowed_id[2])) % 10):
                            partner_id = False
                        else:
                            if int(shadowed_id[9]) != ((int(shadowed_id[2])+int(shadowed_id[0])) % 10):
                                partner_id = False
                            else:
                                partner_id = int(shadowed_id[10]+shadowed_id[8]+shadowed_id[6]+shadowed_id[4]+shadowed_id[2]+shadowed_id[0])
        return partner_id
    def get_shadowed_id_data(self, cr, uid, shadowed_partner_id):
        partner_id = self.unshadow_partner_id(cr, uid , shadowed_partner_id)
        result = []
        if partner_id and partner_id > 0:
            partner = self.pool.get('res.partner').browse(cr, uid, partner_id)
            if partner.address:
                for addr in partner.address:
                    if addr.dir_exclude and not addr.dir_presence:
                        full_name = partner.name.strip()
                        if addr.name:
                            if addr.name.strip()[0:2] == '- ':
                                full_name += (' ' + addr.name.strip())
                            else:
                                full_name = addr.name.strip()
                        full_street = addr.street and addr.street.strip() or ''
                        if addr.street2:
                            full_street = full_street + ' - ' + addr.street2.strip()
                        zip_city = ''
                        if addr.zip_id:
                            if addr.zip_id.name and addr.zip_id.city:
                                zip_city = addr.zip_id.name.strip() + ' ' + addr.zip_id.city.strip()
                        if addr.country_id and addr.country_id.code != 'BE':
                            zip_city += ' [' + addr.country_id.name.strip() + ']'
                        result.append({'id':addr.id,'name':full_name,'street':full_street,'zip_city':zip_city})
        return result
    def set_shadowed_id_addr_state(self, cr, uid, shadowed_partner_id, addr_id, state):
        result = True
        if state != "excluded":
            addr_obj = self.pool.get('res.partner.address')
            partner_id = self.unshadow_partner_id(cr,uid,shadowed_partner_id)
            if partner_id and partner_id > 0:
                partner = self.pool.get('res.partner').browse(cr, uid, partner_id)
                if partner.address:
                    for addr in partner.address:
                        if addr_id == addr.id:
                            if state == "active":
                                addr_obj.write(cr,uid,[addr_id,],{'dir_exclude':False,'dir_presence':True})
                            elif state == "delete":
                                addr_obj.write(cr,uid,[addr_id,],{'active':False})
                            else:
                                result = False
                            break
        return result
res_partner()
