# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
# 1.1 Doesn't warn if show name = (full_name + legal status) and if sort_name is shorter than full name and begin with the same characters
import wizard
import time
import datetime
import pooler
import tools
from pyExcelerator import *
import codecs
import base64
from operator import itemgetter
import re
import urllib
import os

form = """<?xml version="1.0"?>
<form string="Check Deleted Jobs">
    <field name="edition" colspan="4" width="400"/>
    <field name="min_deleted" colspan="4"/>
    <field name="minimum" colspan="4"/>
</form>"""

fields = {
    'edition': {'string': 'Edition', 'type':'integer','required':True},
    'min_deleted': {'string':'minimum of deleted','type':'integer','default':2},
    'minimum': {'string': '% minimum of deleted', 'type':'integer','default':20},
}

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File(s) has(have) been created."  colspan="4"/>
     <field name="check_deleted_jobs.xls" colspan="4" />
</form>"""

msg_fields = {
    'check_deleted_jobs.xls':{'string': 'Excel File','type': 'binary','readonly': True,},
}

class dir_check_deleted_jobs(wizard.interface):
    def _set_default(self,cr,uid,data,context):
        obj_pooler = pooler.get_pool(cr.dbname)
        parameter_obj = obj_pooler.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['LastDirectoryEdition'])
        if param_values.has_key('LastDirectoryEdition'):
            data ['form']['edition'] = int(param_values['LastDirectoryEdition'])
        else:
            data['form']['edition'] = datetime.date.today().year+1
        return data['form']
    def _extract(self,cr,uid,data,context):
        obj_pooler = pooler.get_pool(cr.dbname)

        # Create empty Excel file
        wb1 = Workbook()
        ws1 = wb1.add_sheet('Detected Possible Wrong Deleted Jobs')
        
        # create Excel Headers
        ws1.write(0,0,'partner_id')
        ws1.write(0,1,'address_id')
        ws1.write(0,2,'Partner name')
        ws1.write(0,3,'Street')
        ws1.write(0,4,'Zip code')
        ws1.write(0,5,'Pourcent')
        ws1.write(0,6,'job_id')
        ws1.write(0,7,'statut')
        ws1.write(0,8,'Contact name')
        xl_line = 1
        
        sql = """SELECT djp.job_id, djp.address_id, djp.marked_for_deletion FROM directory_job_proxy as djp
                    WHERE djp.edition = '%i'""" % data['form']['edition']
        cr.execute(sql)        
        res = cr.fetchall()
        dDeleted = {}
        for line in res:
            key = (line[0], line[1])
            if dDeleted.has_key(key):
                if line[2]:
                    dDeleted[key] = True
            else:
                dDeleted[key] = line[2]
        dTotByAddress = {}
        for (key,deleted) in dDeleted.items():
            (job_id,address_id) = key
            if dTotByAddress.has_key(address_id):
                total = dTotByAddress[address_id]
                total['count_jobs'] += 1
                total['job_ids'].append(job_id)
                if deleted:
                    total['deleted_jobs'] += 1
                dTotByAddress[address_id] = total
            else:
                dTotByAddress[address_id] = {'count_jobs':1,'deleted_jobs':deleted and 1 or 0,'job_ids':[job_id,],'pourcent':0}
        for (key,total) in dTotByAddress.items():
            total['pourcent'] = int(100 * total['deleted_jobs'] / total['count_jobs'])
            dTotByAddress[key] = total
        addr_ids = []
        job_ids = []
        to_sort = []
        for (address_id,total) in dTotByAddress.items():
            if total['pourcent'] >= data['form']['minimum'] and total['deleted_jobs'] >= data['form']['min_deleted']:
                addr_ids.append(address_id)
                job_ids.extend(total['job_ids'])
                to_sort.append((total['pourcent'],address_id))
        if addr_ids:
            addresses = obj_pooler.get('res.partner.address').read(cr,uid,addr_ids,['partner_id','street','zip_id'],{'lang':'fr_FR'})
            dAddrs = {}
            for addr in addresses:
                dAddrs[addr['id']] = addr
        if job_ids:
            jobs = obj_pooler.get('res.partner.job').read(cr,uid,job_ids,['contact_id'])
            dJobs = {}
            for job in jobs:
                dJobs[job['id']] = job
        to_sort.sort(key=lambda x: x[0],reverse=True)
        for (pourcent,address_id) in to_sort:
            total = dTotByAddress[address_id]
            for job_id in total['job_ids']:
                address = False
                if dAddrs.has_key(address_id):
                    address = dAddrs[address_id]
                job = False
                if dJobs.has_key(job_id):
                    job = dJobs[job_id]
                ws1.write(xl_line,0,address and address['partner_id'][0] or 0)
                ws1.write(xl_line,1,address_id)
                ws1.write(xl_line,2,address and address['partner_id'][1] or '')
                ws1.write(xl_line,3,address and address['street'] or '')
                ws1.write(xl_line,4,address and (address['zip_id'] and address['zip_id'][1] or '') or '')
                ws1.write(xl_line,5,str(total['pourcent']) + ' (' + str(total['deleted_jobs']) + '/' + str(total['count_jobs']) + ')' )
                ws1.write(xl_line,6,job_id)
                ws1.write(xl_line,7,dDeleted[(job_id,address_id)] and 'deleted' or 'active')
                ws1.write(xl_line,8,job and job['contact_id'][1] or '')
                xl_line += 1
        wb1.save('check_deleted_jobs.xls')
        result_file = open('check_deleted_jobs.xls','rb').read()
        data['form']['check_deleted_jobs.xls'] = base64.encodestring(result_file)
        return data['form']

    states = {
        'init': {
            'actions': [_set_default],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel','gtk-stop'),('check','Check','gtk-find',True)]},
        },
        'check': {
            'actions': [_extract],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Close')]}
        },
    }
dir_check_deleted_jobs('dir_check_deleted_jobs')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

