# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import wizard
import time
import datetime
import pooler
import tools
from pyExcelerator import *
import codecs
import base64
from operator import itemgetter

form = """<?xml version="1.0"?>
<form string="Check Employee Global Number for multi-site partners">
    <field name="warning" colspan="4" width="700" nolabel="1" readonly="1"/>
</form>"""

fields = {
    'warning': {'string': 'Warning', 'type':'text','default':'This tool extract the list of partners of at least one address has been validated, and there is some differences between employee global number of the partner record and one of the validated addresses.\nCet outil sort en Excel la liste des partenares dont au moins une adresse a été validée et pour lequel il existe une différence entre le nombre global d\'employés enregistrés sur la fiche du partenaire et les nombres d\'employés locaux enregistrés au niveau des adresses répertoires validées.'},
}

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File(s) has(have) been created."  colspan="4"/>
     <field name="check_employee.xls" colspan="4" />
</form>"""

msg_fields = {
    'check_employee.xls':{'string': 'Excel File','type': 'binary','readonly': True,},
}

class cci_memberdirectory_check_employee(wizard.interface):
    def _extract(self,cr,uid,data,context):
        obj_pooler = pooler.get_pool(cr.dbname)

        parameter_obj = obj_pooler.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['LastDirectoryBeginDate'])
        if param_values.has_key('LastDirectoryBeginDate'):
            BEGIN_DIRECTORY_DATE = param_values['LastDirectoryBeginDate']
        else:
            BEGIN_DIRECTORY_DATE = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')

        # Create empty Excel file
        wb1 = Workbook()
        ws1 = wb1.add_sheet('Detected Possible Errors')
        
        # create Excel Headers
        ws1.write(0,0,'partner_id')
        ws1.write(0,1,'partner_name')
        ws1.write(0,2,'global_employee')
        ws1.write(0,3,'validated_directory_address_id')
        ws1.write(0,4,'street_zip')
        ws1.write(0,5,'validated')
        ws1.write(0,6,'active_addr')
        ws1.write(0,7,'initial_employee')
        ws1.write(0,8,'given_employee')
        line = 1
        
        # extraction de toutes les adresses nécessaires
        obj_partner = obj_pooler.get('res.partner')
        obj_dir_addr = obj_pooler.get('directory.address.proxy')
        partner_ids = obj_partner.search(cr,uid,[('membership_state','in',['free','invoiced','paid']),('state_id','=',1),('dir_exclude','=',False)])
        partners = obj_partner.read(cr,uid,partner_ids,['address','name','employee_nbr_total'])
        dPartners = {}
        address_ids = []
        for partner in partners:
            if partner['address'] and len(partner['address']) > 1:
                # partner with multiple addresses => we search if we have at least one validated directory record validated for the current directory campaign
                dir_addr_ids = obj_dir_addr.search(cr,uid,[('partner_id','=',partner['id']),('create_date','>=',BEGIN_DIRECTORY_DATE),('user_validated','=',True),'|',('active','=',1),('active','=',False)])
                if len(dir_addr_ids) > 0:
                    address_ids.extend(partner['address'])
                    dPartners[partner['id']] = partner
                    dir_addr_ids = obj_dir_addr.search(cr,uid,[('partner_id','=',partner['id']),('create_date','>=',BEGIN_DIRECTORY_DATE),'|',('active','=',True),('active','=',False)])
                    if dir_addr_ids and len(dir_addr_ids) > 1:
                        dir_addrs = obj_dir_addr.read(cr,uid,dir_addr_ids,['user_validated','internal_validated','already_validated','employee','final_employee','active','final_street','final_street_number','final_street_box'])
                        discordance = False
                        for dir_addr in dir_addrs:
                            if dir_addr['final_employee'] != partner['employee_nbr_total']:
                                discordance = True
                                break
                        if discordance:
                            # writing line on excel file
                            ws1.write(line,0,partner['id'])
                            ws1.write(line,1,partner['name'])
                            ws1.write(line,2,partner['employee_nbr_total'])
                            for dir_addr in dir_addrs:
                                ws1.write(line,3,dir_addr['id'])
                                final_addr = ''
                                if dir_addr['final_street']:
                                    final_addr = dir_addr['final_street']
                                    if dir_addr['final_street_number']:
                                        final_addr += ( ', ' + dir_addr['final_street_number'] )
                                    if dir_addr['final_street_box']:
                                        final_addr += ( ' BTE ' + dir_addr['final_street_box'] )
                                ws1.write(line,4,final_addr)
                                ws1.write(line,5,(dir_addr['user_validated'] or dir_addr['internal_validated']) and 'OUI' or '')
                                ws1.write(line,6,dir_addr['active'] and 'ACTIVE' or 'non')
                                ws1.write(line,7,dir_addr['employee'])
                                ws1.write(line,8,dir_addr['final_employee'])
                                line += 1
        wb1.save('check_employee.xls')
        result_file = open('check_employee.xls','rb').read()
        data['form']['check_employee.xls'] = base64.encodestring(result_file)
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel','gtk-stop'),('extract','Extract','gtk-find',True)]},
        },
        'extract': {
            'actions': [_extract],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Close')]}
        },
    }

cci_memberdirectory_check_employee('cci_memberdirectory_check_employee')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

