# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import wizard
import time
import datetime
import pooler
import tools
from pyExcelerator import *
import codecs
import base64
import csv
from operator import itemgetter

form = """<?xml version="1.0"?>
<form string="Export data for Members' Directory">
    <field name="format" colspan="4" width="700"/>
    <field name="codepage"/>
    <field name="separator"/>
    <field name="finalize"/>
</form>"""

fields = {
    'format': {'string': 'Format', 'type':'selection','selection':[('xls','Excel seul'),('indesign','Excel et InDesign pour fusion')],'required':True,'default':'indesign'},
    'codepage': {'string':'CodePage (DirectoryExportCodePage as default)','type':'char','size':10,'required':True,'default':'utf-16'},
    'separator': {'string':'Separator (DirectoryExportDelimiter as default)','type':'char','size':1,'required':True,'default':'@'},
    'finalize': {'string':'End activities with \'.\'','type':'boolean','default':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File(s) has(have) been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="directory.xls" colspan="4" />
     <field name="directory.csv" colspan="4" />
     <field name="index.csv" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'directory.xls':{'string': 'Excel File',
        'type': 'binary',
        'readonly': True,},
    'directory.csv':{'string': 'Directory CSV File',
        'type': 'binary',
        'readonly': True,},
    'index.csv':{'string': 'Index CSV File',
        'type': 'binary',
        'readonly': True,},
}

def _strip_sort_name(name):
    # this function has been added after the extrazction of January 2017 for the next edition
    # asker : Sabine Renaville
    # goal : transform T INTERIM into TINTERIM for a better alphabetic order so we have T INTERIM between TIMBERLAND and TIOLEC
    #        and no more berore TAC because of the 'space' of the 'caret' ...
    stripped = name.strip().replace(' ','').replace('.','').replace('-','').replace('_','').replace('@','A').replace("'","").replace('"','').replace('&','').replace('+','')
    stripped = stripped.replace(u'Á','A').replace(u'À','A').replace(u'Â','A').replace(u'Ä','A')
    stripped = stripped.replace(u'É','E').replace(u'È','E').replace(u'Ê','E').replace(u'Ë','E')
    stripped = stripped.replace(u'Î','I').replace(u'Ï','I')
    stripped = stripped.replace(u'Ô','O').replace(u'Ö','O')
    stripped = stripped.replace(u'Û','U').replace(u'Ü','U').replace (u'Ù','U')
    return stripped
def _show_name(partner_name,address_name,dir_show_name,dir_sort_name):
    if dir_show_name and dir_show_name.strip():
        return dir_show_name.strip()
    else:
        return _complete_name(partner_name,address_name)
def _sort_name(partner_name,address_name,dir_show_name,dir_sort_name):
    if dir_sort_name and dir_sort_name.strip():
        return _strip_sort_name(dir_sort_name.strip())
    else:
        if dir_show_name and dir_show_name.strip():
            return _strip_sort_name(dir_show_name.strip())
        else:
            return _strip_sort_name(_complete_name(partner_name,address_name))
def _bold_show_name(partner_name,address_name,dir_show_name,dir_sort_name,company_codes):
    # first search if sort_name inside show_name
    show_name = _show_name(partner_name,address_name,dir_show_name,dir_sort_name)
    sort_name = _sort_name(partner_name,address_name,dir_show_name,dir_sort_name)
    if len(sort_name) < len(show_name):
        pos = show_name.find(sort_name)
        if pos > -1:
            before = ''
            inside = ''
            after = ''
            if pos > 0:
                before = show_name[0:pos]
            inside = show_name[pos:pos+len(sort_name)]
            if pos+len(sort_name) < len(show_name):
                after = show_name[pos+len(sort_name):]
            show_name = before + '<gras>' + inside + '<fingras>' + after
    #second search if end of the name if a type of company
    for company_code in company_codes:
        if show_name[-1*len(company_code):] == company_code:
            if not '<fingras>' in show_name:
                show_name = '<gras>' + show_name[0:-1*len(company_code)] + '<fingras>' + show_name[-1*len(company_code):]
            else:
                # nothing to do the end of bold if already set inside
                pass
        elif show_name[-1*len(company_code+'<fingras>'):] == (company_code+'<fingras>'):
            show_name = show_name[0:-1*len(company_code+'<fingras>')] + '<fingras>' + company_code
    if '<gras>' not in show_name:
        show_name = '<gras>' + show_name + '<fingras>'
    return show_name
def _complete_name(partner_name,address_name):
    result = partner_name.strip()
    if address_name and address_name.strip():
        if len(address_name) >= 3 and (address_name[0:2] == '- ' or address_name[0:3] == ' - '):
            result = partner_name.strip() + ' ' + address_name.strip()
        else:
            result = address_name.strip()
    return result
def _phone_format(string,PHONE_COUNTRY_PREFIX):
    def only_digits(string):
        cleaned = ''
        for carac in string:
            if carac in '0123456789':
                cleaned += carac
        return cleaned
    result = ''
    is_confidential = (' (conf)' in (string or ''))
    string = only_digits(string.replace('+','00'))
    if len(string) > 0:
        if len(string) == 9:
            if string[0:2] in ['02','03','04','09']:
                result = string[0:2] + "/" + string[2:5] + "." + string[5:7] + "." + string[7:]
            else:
                result = string[0:3] + "/" + string[3:5] + "." + string[5:7] + "." + string[7:]
        elif len(string) == 10:
            result = string[0:4] + "/" + string[4:6] + "." + string[6:8] + "." + string[8:]
        else:
            # international number
            if string[0:2] == '00':
                # search after a country with this prefix
                prefix = string[2:4]
                if prefix not in PHONE_COUNTRY_PREFIX:
                    prefix = string[2:5]
                    if prefix not in PHONE_COUNTRY_PREFIX:
                        prefix = string[2:6]
                        if prefix not in PHONE_COUNTRY_PREFIX:
                            prefix = ''
                if prefix:
                    result = '+' + string[2:2+len(prefix)] + ' ' + string[2+len(prefix):4+len(prefix)]
                    rest = string[4+len(prefix):]
                    while len(rest) > 3:
                        result += '.' + rest[0:2]
                        rest = rest[2:]
                    result += '.' + rest
                else:
                    result = 'International:'+string
    if is_confidential:
        result += ' (conf)'
    return result
def _strip_http(url):
    url = (url or '').strip()
    if url.upper()[0:7] == 'HTTP://':
        result = url.strip()[7:]
    elif url.upper()[0:8] == 'HTTPS://':
        result = url.strip()[8:]
    else:
        result = url.strip()
    return result
def _strip_endlines(text):
    text = (text or '').strip()
    text = text.replace('\n',' ').replace('\r',' ').replace('\t',' ')
    pos = text.find('  ')
    while pos > -1:
        text = text.replace('  ',' ')
        pos = text.find('  ')
    return text
def _vat_format(number):
    number = (number or '').strip().upper()
    if number[0:2] == 'BE':
        result = number[0:2]+'-'+number[2:6]+'.'+number[6:9]+'.'+number[9:]
    else:
        result = number[0:2]+'-'+number[2:]
    return result
def _full_street(street,zip_id):
    result = ''
    if street:
        result = street.strip()
    if zip_id:
        if result:
            result += ' - '
        result += ( zip_id.name + ' ' + zip_id.city )
    return result
def _extract_categories(cr,uid):
    catDict = {}
    obj_categ = pooler.get_pool(cr.dbname).get('res.partner.category')
    obj_ids = obj_categ.search(cr,uid,[])
    result = obj_categ.read(cr,uid,obj_ids,['id','name', 'complete_name'],{'lang':'fr_FR'} )
    for record in result:
        if record['complete_name'] and ( 'Secteur d\'activit' in record['complete_name'] ):
            fname = record['name']
            if fname.rfind('[') > 0:
                sectcode = fname[fname.rfind('[')+1 : fname.rfind(']') ]
                if fname.rfind(' / ') > 0:
                    sectlib = fname[fname.rfind(' / ')+3 : fname.rfind('[') ]
                else:
                    sectlib = fname[0 : fname.rfind('[') ]
            else:
                sectcode = ''
                sectlib = fname
            catDict[ record['id'] ] = ( sectcode, sectlib )
    return catDict
def _dict_sectors(cr,uid):
    obj_sector = pooler.get_pool(cr.dbname).get('res.partner.activsector')
    sector_ids = obj_sector.search(cr,uid,[])
    sectors = obj_sector.read(cr,uid,sector_ids,['code','name'])
    names = []
    result = {}
    for sector in sectors:
        result[sector['code']] = sector['name']
        names.append(sector['name'])
    names = sorted(names)
    return (result,names)
def _dict_title_companies(cr,uid):
    obj_title = pooler.get_pool(cr.dbname).get('res.partner.title')
    title_ids = obj_title.search(cr,uid,[('domain','=','partner')])
    titles = obj_title.read(cr,uid,title_ids,['shortcut'])
    names = []
    for title in titles:
        if title['shortcut'] and title['shortcut'].strip():
            names.append(' ' +title['shortcut'].strip().upper())
    return names
def _full_page_partner_ids(cr,uid):
    result = []
    #obj_directory = pooler.get_pool(cr.dbname).get('directory.address.proxy')
    #obj_ids = obj_directory.search(cr,uid,[('full_page','=',True)])
    #if obj_ids:
    #    obj_data = obj_directory.read(cr,uid,obj_ids,['partner_id'])
    #    for data in obj_data:
    #        if data['partner_id'] and data['partner_id'][0] not in result:
    #            result.append(data['partner_id'][0])
    obj_fp = pooler.get_pool(cr.dbname).get('directory.fullpage')
    fp_ids = obj_fp.search(cr,uid,[])
    fps = obj_fp.read(cr,uid,fp_ids,['partner_id'])
    for fp in fps:
        if fp['partner_id'] and fp['partner_id'][0] not in result:
            result.append(fp['partner_id'][0])
    return result

class cci_memberdirectory_output(wizard.interface):
    def _get_default(self,cr,uid,data,context):
        param_delimiter = '#'
        param_codepage = 'cp1251'
        parameter_obj = pooler.get_pool(cr.dbname).get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['DirectoryExportDelimiter'])
        if param_values.has_key('DirectoryExportDelimiter'):
            param_delimiter = param_values['DirectoryExportDelimiter']
        param_values = parameter_obj.get_value_from_names(cr,uid,['DirectoryExportCodePage'])
        if param_values.has_key('DirectoryExportCodePage'):
            param_codepage = param_values['DirectoryExportCodePage']
        data['form']['separator'] = param_delimiter
        data['form']['codepage'] = param_codepage
        return data['form']
    def get_phone_country_prefix(self,cr,uid):
        result = []
        obj_country = pooler.get_pool(cr.dbname).get('cci.country')
        country_ids = obj_country.search(cr,uid,[('phoneprefix','!=',False),('phoneprefix','!=',0)])
        if country_ids:
            countries = obj_country.read(cr,uid,country_ids,['phoneprefix'])
            result = [str(x['phoneprefix']) for x in countries]
        return result
    def _extract(self,cr,uid,data,context):
        MAX_JOBS = 5
        MAX_SECTORS = 3
        output_format = data['form']['format']
        dCategories = _extract_categories(cr,uid)
        lfull_page_ids = _full_page_partner_ids(cr,uid)
        (dSectorsByCode,lSortedSectorNames) = _dict_sectors(cr,uid)
        company_codes = _dict_title_companies(cr,uid)
        PHONE_COUNTRY_PREFIX = self.get_phone_country_prefix(cr,uid)
        
        # Create empty Excel file
        wb1 = Workbook()
        ws1 = wb1.add_sheet('Directory')
        
        # create Excel Headers
        fix_headers = [(0,'partner_id'),(1,'address_id'),(2,'partner_name'),(3,'address_name'),(4,'dir_show_name'),(5,'dir_sort_name'),
                       (6,'full_name'),(7,'group_full_name'),(8,'sort_name'),(9,'group_sort_name'),(10,'bold_full_name'),(11,'group_bold_full_name'),
                       (12,'street'),(13,'full_street'),(14,'zipcode'),(15,'city'),
                       (16,'phone_address'),(17,'fax_address'),(18,'email_address'),(19,'web'),(20,'vat_num'),(21,'employee'),(22,'desc_activ'),(23,'orig_descactiv'),
                       (24,'sector1code'),(25,'sector1lib'),(26,'sector2code'),(27,'sector2lib'),(28,'sector3code'),(29,'sector3lib'),(30,'full_page'),(31,'image_name'),(32,'page_name')]
        var_headers = [(0,'courtesy'),(1,'firstname'),(2,'lastname'),(3,'title'),(4,'email'),(5,'mobile'),
                       (6,'mob_conf'),(7,'published_mobile'),(8,'job_id'),(9,'contact_id')]
        index_headers = [(0,'sectorcode'),(1,'sectorname'),(2,'show_name'),(3,'full_page'),(4,'page_name'),(5,'sort_name'),(6,'group_sort_name'),('7','bold_full_name'),(8,'group_bold_full_name'),(9,'isectorcode')]
        for (index,name) in fix_headers:
            ws1.write(0,index,name)
        for count_job in range(1,MAX_JOBS+1):
            for (index,name) in var_headers:
                ws1.write(0,index+len(fix_headers)+(len(var_headers)*(count_job-1)),name+str(count_job))
        obj_pooler = pooler.get_pool(cr.dbname)
        if output_format == 'indesign':
            param_delimiter = ','
            param_codepage = 'utf-8'
            parameter_obj = obj_pooler.get('cci_parameters.cci_value')
            param_values = parameter_obj.get_value_from_names(cr,uid,['DirectoryExportDelimiter'])
            if param_values.has_key('DirectoryExportDelimiter'):
                param_delimiter = param_values['DirectoryExportDelimiter']
            param_values = parameter_obj.get_value_from_names(cr,uid,['DirectoryExportCodePage'])
            if param_values.has_key('DirectoryExportCodePage'):
                param_codepage = param_values['DirectoryExportCodePage']
            if data['form']['separator']:
                param_delimiter = data['form']['separator']
            if data['form']['codepage']:
                param_codepage = data['form']['codepage']
            hfDir = open('directory.csv','w')
            csvDir = csv.writer(hfDir,delimiter=param_delimiter[0].encode('ascii','ignore'),quotechar='"',quoting=csv.QUOTE_NONNUMERIC,lineterminator='\n')
            record = []
            for (index,name) in fix_headers:
                record.append(name)
            for count_job in range(0,MAX_JOBS):
                for (index,name) in var_headers:
                    record.append(name+str(count_job+1))
            csvDir.writerow(record)
            hfIndex = open('index.csv','w')
            csvIndex = csv.writer(hfIndex,delimiter=param_delimiter[0].encode('ascii','ignore'),quotechar='"',quoting=csv.QUOTE_NONNUMERIC,lineterminator='\n')
            record = []
            for (index,name) in index_headers:
                record.append(name)
            csvIndex.writerow(record)

        # Preparing the list of companies by activity sector for the index later in the output
        dCompBySectors = {} # key = sector name; value = list of company names
        lCompNamesFullPage = []
        
        # extraction de toutes les adresses nécessaires
        obj_partner = obj_pooler.get('res.partner')
        obj_addr = obj_pooler.get('res.partner.address')
        partner_ids = obj_partner.search(cr,uid,[('membership_state','in',['free','invoiced','paid']),('state_id','=',1),('dir_exclude','=',False)])
        partner2_ids = obj_partner.search(cr,uid,[('dir_presence','=',True),('dir_exclude','=',False)])
        for pid in partner2_ids:
            if pid not in partner_ids:
                partner_ids.append(pid)
        address_ids = obj_addr.search(cr,uid,[('partner_id','in',partner_ids),('dir_exclude','=',False),('notdelivered','=',False),'|',('type','<>','invoice'),('dir_presence','=',True)])
        addresses = obj_addr.browse(cr,uid,address_ids,{'lang':'fr_FR'})
        # tri de ces addresses sur base du nom final
        # avec comptage du nombre d'adresses publiées par partenaires
        count_by_partner = {}
        lAddresses = []
        for addr in addresses:
            lAddresses.append( (_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name),addr) )
            if addr.partner_id.id not in count_by_partner:
                count_by_partner[addr.partner_id.id] = 1
            else:
                count_by_partner[addr.partner_id.id] += 1
        lAddresses = sorted(lAddresses,key=itemgetter(0))
        line = 1
        for (key,addr) in lAddresses:
            # calculation of some values to extract with conditions
            final_employee = addr.partner_id.employee_nbr_total or 0
            final_activ = ( addr.activity_description or addr.partner_id.activity_description or '' ).strip(' \n')
            final_activ = final_activ.replace('\n',' ')
            while '  ' in final_activ:
                final_activ = final_activ.replace('  ',' ')
            if data['form']['finalize'] and final_activ and final_activ[-1:] not in ['.','!','?',',',';']:
                final_activ += '.'
            full_page = (addr.partner_id.id in lfull_page_ids)
            
            # extraction of activity sector
            sectors = []
            if addr.sector1 or addr.sector2 or addr.sector3:
                # extract for local cci activity sector
                if addr.sector1:
                    sectors.append( (addr.sector1.code,addr.sector1.name) )
                if addr.sector2:
                    sectors.append( (addr.sector2.code,addr.sector2.name) )
                if addr.sector3:
                    sectors.append( (addr.sector3.code,addr.sector3.name) )
            else:
                # search if we have activity sector in partners categories
                if addr.partner_id and addr.partner_id.category_id:
                    for categ_id in addr.partner_id.category_id:
                        if dCategories.has_key(categ_id.id):
                            categ = dCategories[categ_id.id]
                            sectors.append( (categ[0],categ[1]) )
            # keeping selected sectors to create the index by activity sectors later
            sector_num = 1
            for sector in sectors:
                if sector_num <= MAX_SECTORS:
                    if dCompBySectors.has_key(sector[1]):
                        values = dCompBySectors[sector[1]]
                        if _complete_name(addr.partner_id.name,addr.name) not in values:
                            values.append(_complete_name(addr.partner_id.name,addr.name))
                            dCompBySectors[sector[1]] = values
                    else:
                        dCompBySectors[sector[1]] = [_complete_name(addr.partner_id.name,addr.name),]
                sector_num += 1
            if (sector_num > 1) and full_page and (_complete_name(addr.partner_id.name,addr.name) not in lCompNamesFullPage):
                lCompNamesFullPage.append(_complete_name(addr.partner_id.name,addr.name))
            # extraction and sorting of contacts
            lContacts = []
            if addr.job_ids:
                for job in addr.job_ids:
                    if job.sequence_directory and job.sequence_directory > 0 and job.sequence_directory < 99 and job.contact_id and job.contact_id.active:
                        lContacts.append( (job.sequence_directory,job) )
            lContacts = sorted(lContacts,key=itemgetter(0))
            
            # 2016-10-07 extracting list of partner_ids with a Full Page
            obj_fp = obj_pooler.get('directory.fullpage')
            fp_ids = obj_fp.search(cr,uid,[])
            fps = obj_fp.read(cr,uid,fp_ids,['partner_id',])
            fp_partner_ids = []
            for fp in fps:
                if fp['partner_id'] and fp['partner_id'][0] not in fp_partner_ids:
                    fp_partner_ids.append(fp['partner_id'][0])
                    
            # writing line on excel file
            ws1.write(line,0,addr.partner_id.id)
            ws1.write(line,1,addr.id)
            ws1.write(line,2,addr.partner_id.name.strip())
            ws1.write(line,3,(addr.name or '').strip())
            ws1.write(line,4,(addr.dir_show_name or '').strip())
            ws1.write(line,5,(addr.dir_sort_name or '').strip())
            ws1.write(line,6,_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name))
            if count_by_partner[addr.partner_id.id] > 1:
                ws1.write(line,7,addr.partner_id.name.strip() + ' # ' + _show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name))
            else:
                ws1.write(line,7,_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name))
            ws1.write(line,8,_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name))
            if count_by_partner[addr.partner_id.id] > 1:
                ws1.write(line,9,_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name) + ' # ' + _sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name))
            else:
                ws1.write(line,9,_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name))
            ws1.write(line,10,_bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes))
            if count_by_partner[addr.partner_id.id] > 1:
                ws1.write(line,11,_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name) + ' # ' + _bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes))
            else:
                ws1.write(line,11,_bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes))
            ws1.write(line,12,addr.street or '')
            if addr.street2:
                ws1.write(line,13,(addr.street or '').strip() + ' - ' + addr.street2.strip())
            else:
                ws1.write(line,13,(addr.street or '').strip())
            ws1.write(line,14,(addr.zip_id and (' '+addr.zip_id.name) or '').strip())
            ws1.write(line,15,(addr.zip_id and addr.zip_id.city or '').strip())
            ws1.write(line,16,_phone_format(addr.phone or '',PHONE_COUNTRY_PREFIX))
            ws1.write(line,17,_phone_format(addr.fax or '',PHONE_COUNTRY_PREFIX))
            ws1.write(line,18,(addr.email or '').strip())
            ws1.write(line,19,_strip_http(addr.partner_id.website))
            ws1.write(line,20,_vat_format(addr.partner_id.vat))
            ws1.write(line,21,final_employee)
            ws1.write(line,22,_strip_endlines(final_activ))
            ws1.write(line,23,final_activ)
            if len(sectors) >= 1:
                ws1.write(line,24,sectors[0][0])
                ws1.write(line,25,sectors[0][1])
            else:
                ws1.write(line,24,'')
                ws1.write(line,25,'')
            if len(sectors) >= 2:
                ws1.write(line,26,sectors[1][0])
                ws1.write(line,27,sectors[1][1])
            else:
                ws1.write(line,26,'')
                ws1.write(line,27,'')
            if len(sectors) >= 3:
                ws1.write(line,28,sectors[2][0])
                ws1.write(line,29,sectors[2][1])
            else:
                ws1.write(line,27,'')
                ws1.write(line,29,'')
            ws1.write(line,30,(addr.partner_id.id in fp_partner_ids) and 'OUI' or '')
            ws1.write(line,31,(addr.partner_id.id in fp_partner_ids) and '<signePP>' or '<signeM>')
            ws1.write(line,32,(addr.partner_id.id in fp_partner_ids) and 'Page XXX' or '')
            for count_job in range(1,min(len(lContacts),MAX_JOBS)+1):
                (sequence_directory,job) = lContacts[count_job-1]
                ws1.write(line,len(fix_headers)+(len(var_headers)*(count_job-1)),job.contact_id.title or 'Monsieur')
                ws1.write(line,1+len(fix_headers)+(len(var_headers)*(count_job-1)),job.contact_id.first_name or '')
                ws1.write(line,2+len(fix_headers)+(len(var_headers)*(count_job-1)),job.contact_id.name or '')
                ws1.write(line,3+len(fix_headers)+(len(var_headers)*(count_job-1)),job.function_label or '')
                ws1.write(line,4+len(fix_headers)+(len(var_headers)*(count_job-1)),job.email or job.contact_id.email or '')
                mobile = _phone_format(job.contact_id.mobile or '',PHONE_COUNTRY_PREFIX)
                ws1.write(line,5+len(fix_headers)+(len(var_headers)*(count_job-1)),mobile)
                if ' (conf)' in mobile:
                    ws1.write(line,6+len(fix_headers)+(len(var_headers)*(count_job-1)),'OUI')
                    ws1.write(line,7+len(fix_headers)+(len(var_headers)*(count_job-1)),'')
                else:
                    ws1.write(line,6+len(fix_headers)+(len(var_headers)*(count_job-1)),'')
                    ws1.write(line,7+len(fix_headers)+(len(var_headers)*(count_job-1)),mobile)
                ws1.write(line,8+len(fix_headers)+(len(var_headers)*(count_job-1)),job.id)
                ws1.write(line,9+len(fix_headers)+(len(var_headers)*(count_job-1)),job.contact_id.id)

            line += 1
            if output_format == 'indesign':
                record = []
                record.append(addr.partner_id.id)
                record.append(addr.id)
                record.append((addr.partner_id.name.strip()).encode(param_codepage))
                record.append((addr.name or '').strip().encode(param_codepage))
                record.append((addr.dir_show_name or '').strip().encode(param_codepage))
                record.append((addr.dir_sort_name or '').strip().encode(param_codepage))
                record.append(_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name).encode(param_codepage))
                if count_by_partner[addr.partner_id.id] > 1:
                    record.append((addr.partner_id.name.strip() + ' # ' + _show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name)).encode(param_codepage))
                else:
                    record.append(_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name).encode(param_codepage))
                record.append(_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name).encode(param_codepage))
                if count_by_partner[addr.partner_id.id] > 1:
                    record.append((_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name) + ' # ' + _sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name)).encode(param_codepage))
                else:
                    record.append(_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name).encode(param_codepage))
                record.append(_bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes).encode(param_codepage))
                if count_by_partner[addr.partner_id.id] > 1:
                    record.append((addr.partner_id.name.strip() + ' # ' + _bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes)).encode(param_codepage))
                else:
                    record.append(_bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes).encode(param_codepage))
                record.append((addr.street or '').strip().encode(param_codepage))
                if addr.street2:
                    record.append(((addr.street or '').strip() + ' - ' + addr.street2.strip()).encode(param_codepage))
                else:
                    record.append(((addr.street or '').strip()).encode(param_codepage))
                record.append((addr.zip_id and (' '+addr.zip_id.name) or '').strip().encode(param_codepage))
                record.append((addr.zip_id and addr.zip_id.city or '').strip().encode(param_codepage))
                record.append(_phone_format(addr.phone or '',PHONE_COUNTRY_PREFIX).encode(param_codepage))
                record.append(_phone_format(addr.fax or '',PHONE_COUNTRY_PREFIX).encode(param_codepage))
                record.append((addr.email or '').strip().encode(param_codepage))
                record.append(_strip_http(addr.partner_id.website).encode(param_codepage))
                record.append(_vat_format(addr.partner_id.vat).encode(param_codepage))
                record.append(final_employee)
                record.append(_strip_endlines(final_activ).encode(param_codepage))
                record.append(final_activ.encode(param_codepage))
                if len(sectors) >= 1:
                    record.append(sectors[0][0].strip().encode(param_codepage))
                    record.append(sectors[0][1].strip().encode(param_codepage))
                    sector = []
                    sector.append(('C'+(sectors[0][0].strip())).encode(param_codepage))
                    sector.append((sectors[0][1].strip()+' ('+sectors[0][0].strip()+')').encode(param_codepage))
                    sector.append(_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name).encode(param_codepage))
                    sector.append((addr.partner_id.id in fp_partner_ids) and 'OUI' or '')
                    sector.append((addr.partner_id.id in fp_partner_ids) and 'Page XXX' or '')
                    sector.append(_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name).encode(param_codepage))
                    if count_by_partner[addr.partner_id.id] > 1:
                        sector.append((_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name) + ' # ' + _sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name)).encode(param_codepage))
                    else:
                        sector.append(_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name).encode(param_codepage))
                    sector.append(_bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes).encode(param_codepage))
                    if count_by_partner[addr.partner_id.id] > 1:
                        sector.append((_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name) + ' # ' + _bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes)).encode(param_codepage))
                    else:
                        sector.append(_bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes).encode(param_codepage))
                    sector.append(sectors[0][0].strip().encode(param_codepage))
                    csvIndex.writerow(sector)
                else:
                    record.append('')
                    record.append('')
                if len(sectors) >= 2:
                    record.append(sectors[1][0].strip().encode(param_codepage))
                    record.append(sectors[1][1].strip().encode(param_codepage))
                    sector = []
                    sector.append(('C'+(sectors[1][0].strip())).encode(param_codepage))
                    sector.append((sectors[1][1].strip()+' ('+sectors[1][0].strip()+')').encode(param_codepage))
                    sector.append(_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name).encode(param_codepage))
                    sector.append((addr.partner_id.id in fp_partner_ids) and 'OUI' or '')
                    sector.append((addr.partner_id.id in fp_partner_ids) and 'Page XXX' or '')
                    sector.append(_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name).encode(param_codepage))
                    if count_by_partner[addr.partner_id.id] > 1:
                        sector.append((_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name) + ' # ' + _sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name)).encode(param_codepage))
                    else:
                        sector.append(_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name).encode(param_codepage))
                    sector.append(_bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes).encode(param_codepage))
                    if count_by_partner[addr.partner_id.id] > 1:
                        sector.append((_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name) + ' # ' + _bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes)).encode(param_codepage))
                    else:
                        sector.append(_bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes).encode(param_codepage))
                    sector.append(sectors[1][0].strip().encode(param_codepage))
                    csvIndex.writerow(sector)
                else:
                    record.append('')
                    record.append('')
                if len(sectors) >= 3:
                    record.append(sectors[2][0].strip().encode(param_codepage))
                    record.append(sectors[2][1].strip().encode(param_codepage))
                    sector = []
                    sector.append(('C'+(sectors[2][0].strip())).encode(param_codepage))
                    sector.append((sectors[2][1].strip()+' ('+sectors[2][0]+')').encode(param_codepage))
                    sector.append(_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name).encode(param_codepage))
                    sector.append((addr.partner_id.id in fp_partner_ids) and 'OUI' or '')
                    sector.append((addr.partner_id.id in fp_partner_ids) and 'Page XXX' or '')
                    sector.append(_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name).encode(param_codepage))
                    if count_by_partner[addr.partner_id.id] > 1:
                        sector.append((_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name) + ' # ' + _sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name)).encode(param_codepage))
                    else:
                        sector.append(_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name).encode(param_codepage))
                    sector.append(_bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes).encode(param_codepage))
                    if count_by_partner[addr.partner_id.id] > 1:
                        sector.append((_sort_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name) + ' # ' + _bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes)).encode(param_codepage))
                    else:
                        sector.append(_bold_show_name(addr.partner_id.name,addr.name,addr.dir_show_name,addr.dir_sort_name,company_codes).encode(param_codepage))
                    sector.append(sectors[2][0].strip().encode(param_codepage))
                    csvIndex.writerow(sector)
                else:
                    record.append('')
                    record.append('')
                record.append((addr.partner_id.id in fp_partner_ids) and 'OUI' or '')
                record.append((addr.partner_id.id in fp_partner_ids) and '<signePP>' or '<signeM>')
                record.append((addr.partner_id.id in fp_partner_ids) and 'Page XXX' or '')
                for count_job in range(1,min(len(lContacts),MAX_JOBS)+1):
                    (sequence_directory,job) = lContacts[count_job-1]
                    record.append((job.contact_id.title or 'Monsieur').encode(param_codepage))
                    record.append((job.contact_id.first_name or '').encode(param_codepage))
                    record.append((job.contact_id.name or '').encode(param_codepage))
                    record.append((job.function_label or '').encode(param_codepage))
                    record.append((job.email or job.contact_id.email or '').encode(param_codepage))
                    mobile = _phone_format(job.contact_id.mobile or '',PHONE_COUNTRY_PREFIX)
                    record.append(mobile.encode(param_codepage))
                    if ' (conf)' in mobile:
                        record.append('OUI')
                        record.append('')
                    else:
                        record.append('')
                        record.append(mobile.encode(param_codepage))
                    record.append(job.id)
                    record.append(job.contact_id.id)
                csvDir.writerow(record)
        wb1.save('directory.xls')
        result_file = open('directory.xls','rb').read()
        data['form']['directory.xls'] = base64.encodestring(result_file)
        if output_format == 'indesign':
            hfDir.close()
            hfIndex.close()
            result_file = open('directory.csv','rb').read()
            data['form']['directory.csv'] = base64.encodestring(result_file)
            result_file = open('index.csv','rb').read()
            data['form']['index.csv'] = base64.encodestring(result_file)
        return data['form']

    states = {
        'init': {
            'actions': [_get_default],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel','gtk-stop'),('extract','Extract','gtk-find',True)]},
        },
        'extract': {
            'actions': [_extract],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Close')]}
        },
    }

cci_memberdirectory_output('cci_memberdirectory_output')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

