# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import wizard
import time
import datetime
import pooler
import tools
from tools.translate import _
from pyExcelerator import *
import base64

form = """<?xml version="1.0"?>
<form string="Send Emails for members with excluded addresses">
    <field name="invoice" colspan="4"/>
    <field name="sending_address" colspan="4" width='600'/>
    <field name="template_id" colspan="4"/>
    <field name="excel" colspan="4"/>
</form>"""

fields = {
    'invoice': {'string':'Exclude invoice before', 'type':'boolean', 'help':'Exclude all invoice addresses not already confirmed (by forced status)','default':True},
    'sending_address': {'string':'Sending Address', 'type':'char','size':200,'help':'If empty, no email sending, just excluding invoice address and creation of Excel file'},
    'template_id':{'string':'Template','type':'many2one','relation':'cci_email_template'},
    'excel': {'string': 'Excel File', 'type':'boolean', 'help':'Create an Excel file with the sending and detections','default':True},
}

inter_form = """<?xml version="1.0"?>
<form string="Counting">
    <field name="newly" readonly="1" colspan="4"/>
    <field name="addresses" readonly="1"/>
    <field name="partners" readonly="1"/>
    <field name="already_sended" readonly="1" colspan="4"/>
    <field name="max_to_send" colspan="4"/>
</form>"""

inter_fields = {
    'newly': {'string':'Newly excluded', 'type':'integer', 'help':'Count of newly excluded invoice addresses of members'},
    'addresses': {'string':'Addresses', 'type':'integer', 'help':'Count of detected excluded addresses'},
    'partners': {'string':'Partners', 'type':'integer', 'help':'Count of detected partners with excluded addresses'},
    'already_sended': {'string':'Already Sended', 'type':'integer', 'help':'Already sended this year'},
    'max_to_send': {'string':'Max emails to send', 'type':'integer', 'help':'Send maximum this number of emails before stopping the sending, for sending in multiple waves','default':9999},
}

final_form = """<?xml version="1.0"?>
<form string="Results">
    <field name="results" colspan="4" nolabel="1"/>
    <field name="excluded_addresses_send.xls" colspan="4" />
</form>"""

final_fields = {
    'results':{'string':'Results','type':'text','readonly':1,},
    'excluded_addresses_send.xls':{'string': 'Excel File','type': 'binary','readonly': True,},
}

def partner_id_shadow(partner_id):
    base = str(partner_id).rjust(6,'0')
    inv = ''.join(reversed(base))
    interm1 = (int(inv[5])+int(inv[4]))%10
    interm2 = (int(inv[4])+int(inv[3]))%10
    interm3 = (int(inv[3])+int(inv[2]))%10
    interm4 = (int(inv[2])+int(inv[1]))%10
    interm5 = (int(inv[1])+int(inv[0]))%10
    shadowed_id = inv[0]+str(interm1)+inv[1]+str(interm2)+inv[2]+str(interm3)+inv[3]+str(interm4)+inv[4]+str(interm5)+inv[5]
    return shadowed_id
def unshadow_partner_id(shadowed_id):
    if len(shadowed_id) != 11:
        partner_id = False
    else:
        if int(shadowed_id[1]) != ((int(shadowed_id[10])+int(shadowed_id[8])) % 10):
            partner_id = False
        else:
            if int(shadowed_id[3]) != ((int(shadowed_id[8])+int(shadowed_id[6])) % 10):
                partner_id = False
            else:
                if int(shadowed_id[5]) != ((int(shadowed_id[6])+int(shadowed_id[4])) % 10):
                    partner_id = False
                else:
                    if int(shadowed_id[7]) != ((int(shadowed_id[4])+int(shadowed_id[2])) % 10):
                        partner_id = False
                    else:
                        if int(shadowed_id[9]) != ((int(shadowed_id[2])+int(shadowed_id[0])) % 10):
                            partner_id = False
                        else:
                            partner_id = int(shadowed_id[10]+shadowed_id[8]+shadowed_id[6]+shadowed_id[4]+shadowed_id[2]+shadowed_id[0])
    return partner_id
class send_emails_excluded_address(wizard.interface):
    _addr_ids = []
    _partner_ids = []
    _new_excluded_addr_ids = []
    _already_sended = []
    _parameter_id = 0
    
    def _count_concerned(self,cr,uid,data,context):
        # set empty at the begin of each usage
        self._addr_ids = []
        self._partner_ids = []
        self._new_excluded_addr_ids = []
        self._already_sended = []
        self._parameter_id = 0
        addr_obj = pooler.get_pool(cr.dbname).get('res.partner.address')
        if data['form']['invoice']:
            # correct before counting
            addr_to_correct_ids = addr_obj.search(cr,uid,[('dir_exclude','=',False),('dir_presence','=',False),('type','=','invoice')])
            addrs = addr_obj.browse(cr,uid,addr_to_correct_ids)
            for addr in addrs:
                if addr.partner_id and addr.partner_id.membership_state in ['free','invoiced','paid'] and not addr.partner_id.dir_exclude:
                    self._new_excluded_addr_ids.append(addr.id)
            if self._new_excluded_addr_ids:
                addr_obj.write(cr,uid,self._new_excluded_addr_ids,{'dir_exclude':True})
        # count to show the result to the user
        addr_ids = addr_obj.search(cr,uid,[('dir_exclude','=',True),('dir_presence','=',False)])
        addrs = addr_obj.browse(cr,uid,addr_ids)
        for addr in addrs:
            if addr.partner_id and addr.partner_id.membership_state in ['free','invoiced','paid']:
                self._addr_ids.append(addr.id)
                if addr.partner_id.id not in self._partner_ids:
                    self._partner_ids.append(addr.partner_id.id)
        # get back the list of already sended ID
        parameter_obj = pooler.get_pool(cr.dbname).get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['LastDirectoryExcludedAddrSended'])
        if param_values.has_key('LastDirectoryExcludedAddrSended'):
            self._already_sended = param_values['LastDirectoryExcludedAddrSended']
            self._parameter_id = parameter_obj.search(cr,uid,[('name','=','LastDirectoryExcludedAddrSended')])[0]
        else:
            # create the parameters
            self._parameter_id = parameter_obj.create(cr,uid,{'name':'LastDirectoryExcludedAddrSended','type':'exp','value':'[,]','description':'List of Partners IDS for already sended mails for excluded addresses in Members\' Directory'})
        data['form']['addresses'] = len(self._addr_ids)
        data['form']['partners'] = len(self._partner_ids)
        data['form']['newly'] = len(self._new_excluded_addr_ids)
        data['form']['already_sended'] = len(self._already_sended)
        return data['form']
    def _send_emails(self,cr,uid,data,context):
        if self._addr_ids and self._partner_ids:
            if data['form']['sending_address'] and data['form']['template_id'] and data['form']['max_to_send'] > 0:
                sending_address = data['form']['sending_address']
                template_id = data['form']['template_id']
                template_obj = pooler.get_pool(cr.dbname).get('cci_email_template')
                template = template_obj.browse(cr,uid,template_id)
                #
                mail_sended_obj = parameter_obj = pooler.get_pool(cr.dbname).get('cci_email_sended')
                #
                bd_de_test = True
                parameter_obj = pooler.get_pool(cr.dbname).get('cci_parameters.cci_value')
                param_values = parameter_obj.get_value_from_names(cr,uid,['BDDeTest'])
                if param_values.has_key('BDDeTest'):
                    bd_de_test = (param_values['BDDeTest'] == 1)
            else:
                sending_address = False
                template = False
            # Create empty Excel file
            if data['form']['excel']:
                wb1 = Workbook()
                ws1 = wb1.add_sheet('Excluded Addresses to ask for forced presence')
                ws1.write(0,0,'Here you find the excluded addresses, not forced, to ask for un-exclusion to the member')
                ws1.write(1,0,'If member un-exclude one or more addresses, they will be \'forced presence\' indicated so we don\'t ask for it anymore next year.')
                ws1.write(3,0,'Partner ID')
                ws1.write(3,1,'Address ID')
                ws1.write(3,2,'Job ID')
                ws1.write(3,3,'Contact ID')
                ws1.write(3,4,'Partner Name')
                ws1.write(3,5,'Partner Membership State')
                ws1.write(3,6,'Full Name')
                ws1.write(3,7,'Street')
                ws1.write(3,8,'Zip City')
                ws1.write(3,9,'Newly Excluded')
                ws1.write(3,10,'Used Email')
                ws1.write(3,11,'Contact')
                ws1.write(3,12,'Salesman')
                ws1.write(3,13,'Mail sended')
                ws1.write(3,14,'Manual')
                line = 4
            #
            addr_obj = pooler.get_pool(cr.dbname).get('res.partner.address')
            partner_obj = pooler.get_pool(cr.dbname).get('res.partner')
            partners = partner_obj.browse(cr,uid,self._partner_ids)
            sended_this_time = 0
            for partner in partners:
                job_id = False
                contact_id = False
                final_email = ''
                contact_courtesy = ''
                contact_name = ''
                contact_firstname = ''
                if sending_address:
                    # search the person to use for emailing if emailing
                    # first : contact for cotisation for this partner
                    # second : contact for cotisation for default address
                    # third : contact for cotisation first address not default
                    # fourth : first G or $ on default addresse, else other addresses
                    # fifth : first B in default address, else other addresses
                    # sixth : first job on default address, else other addresses
                    # seventh : search addr email first on default, then on others
                    if partner.address:
                        #1
                        found_job = False
                        for addr in partner.address:
                            if addr.job_ids:
                                for job in addr.job_ids:
                                    if job.function_code_label and '!' in job.function_code_label and job.contact_id and (job.email or job.contact_id.email):
                                        found_job = job
                                        break
                            if not found_job:
                                #2
                                for addr in partner.address:
                                    if addr.type == 'default' and addr.job_ids:
                                        for job in addr.job_ids:
                                            if job.function_code_label and '@' in job.function_code_label and job.contact_id and (job.email or job.contact_id.email):
                                                found_job = job
                                                break
                            if not found_job:
                                #3
                                for addr in partner.address:
                                    if addr.type != 'default' and addr.job_ids:
                                        for job in addr.job_ids:
                                            if job.function_code_label and '@' in job.function_code_label and job.contact_id and (job.email or job.contact_id.email):
                                                found_job = job
                                                break
                            if not found_job:
                                #4
                                min_seq = 9999
                                for addr in partner.address:
                                    if addr.type == 'default' and addr.job_ids:
                                        for job in addr.job_ids:
                                            if job.function_code_label and ('G' in job.function_code_label or '$' in job.function_code_label) and job.contact_id and (job.email or job.contact_id.email):
                                                if (job.sequence_partner or 0) < min_seq:
                                                    found_job = job
                                                    min_seq = job.sequence_partner
                                        break
                            if not found_job:
                                min_seq = 9999
                                for addr in partner.address:
                                    if addr.type != 'default' and addr.job_ids:
                                        for job in addr.job_ids:
                                            if job.function_code_label and ('G' in job.function_code_label or '$' in job.function_code_label) and job.contact_id and (job.email or job.contact_id.email):
                                                if (job.sequence_partner or 0) < min_seq:
                                                    found_job = job
                                                    min_seq = job.sequence_partner
                                        if found_job:
                                            break
                            if not found_job:
                                #5
                                min_seq = 9999
                                for addr in partner.address:
                                    if addr.type == 'default' and addr.job_ids:
                                        for job in addr.job_ids:
                                            if job.function_code_label and 'B' in job.function_code_label and job.contact_id and (job.email or job.contact_id.email):
                                                if (job.sequence_partner or 0) < min_seq:
                                                    found_job = job
                                                    min_seq = job.sequence_partner
                                        break
                            if not found_job:
                                min_seq = 9999
                                for addr in partner.address:
                                    if addr.type != 'default' and addr.job_ids:
                                        for job in addr.job_ids:
                                            if job.function_code_label and 'B' in job.function_code_label and job.contact_id and (job.email or job.contact_id.email):
                                                if (job.sequence_partner or 0) < min_seq:
                                                    found_job = job
                                                    min_seq = job.sequence_partner
                                        if found_job:
                                            break
                            if not found_job:
                                #6
                                min_seq = 9999
                                for addr in partner.address:
                                    if addr.type == 'default' and addr.job_ids:
                                        for job in addr.job_ids:
                                            if job.contact_id and (job.email or job.contact_id.email):
                                                if (job.sequence_partner or 0) < min_seq:
                                                    found_job = job
                                                    min_seq = job.sequence_partner
                                        break
                            if not found_job:
                                min_seq = 9999
                                for addr in partner.address:
                                    if addr.type != 'default' and addr.job_ids:
                                        for job in addr.job_ids:
                                            if job.contact_id and (job.email or job.contact_id.email):
                                                if (job.sequence_partner or 0) < min_seq:
                                                    found_job = job
                                                    min_seq = job.sequence_partner
                                        if found_job:
                                            break
                            found_addr_email = False
                            if not found_job:
                                # 7
                                for addr in partner.address:
                                    if addr.type == 'default' and addr.email:
                                        found_addr_email = addr
                                        break
                            if not found_addr_email:
                                # 7bis
                                for addr in partner.address:
                                    if addr.type != 'default' and addr.email:
                                        found_addr_email = addr
                                        break
                            if found_job:
                                job_id = found_job.id
                                contact_id = found_job.contact_id.id
                                final_email = found_job.email or found_job.contact_id.email or ''
                                contact_courtesy = found_job.contact_id.title or ''
                                contact_name = found_job.contact_id.name
                                contact_firstname = found_job.contact_id.first_name or ''
                            else:
                                if found_addr_email:
                                    job_id = False
                                    contact_id = False
                                    final_email = found_addr_email.email
                                    contact_courtesy = 'Cher'
                                    contact_name = 'Membre'
                                    contact_firstname = ''
                first_addr = True
                first_line = 999999
                addr_text = ''
                for addr in partner.address:
                    if addr.id in self._addr_ids:
                        full_name = partner.name.strip()
                        if addr.name:
                            if addr.name.strip()[0:2] == '- ':
                                full_name += (' ' + addr.name.strip())
                            else:
                                full_name = addr.name.strip()
                        full_street = addr.street and addr.street.strip() or ''
                        if addr.street2:
                            full_street = full_street + ' - ' + addr.street2.strip()
                        zip_city = ''
                        if addr.zip_id:
                            if addr.zip_id.name and addr.zip_id.city:
                                zip_city = addr.zip_id.name.strip() + ' ' + addr.zip_id.city.strip()
                        if addr.country_id and addr.country_id.code != 'BE':
                            zip_city += ' [' + addr.country_id.name.strip() + ']'
                        if data['form']['excel']:
                            ws1.write(line,0,partner.id)
                            ws1.write(line,1,addr.id)
                            ws1.write(line,2,job_id and job_id or 0)
                            ws1.write(line,3,contact_id and contact_id or 0)
                            ws1.write(line,4,first_addr and partner.name)
                            ws1.write(line,5,first_addr and partner.membership_state)
                            ws1.write(line,6,full_name)
                            ws1.write(line,7,addr.street or '')
                            ws1.write(line,8, zip_city.strip())
                            ws1.write(line,9,(addr.id in self._new_excluded_addr_ids) and 'YES' or '')
                            ws1.write(line,10,(first_addr and final_email) and final_email or '')
                            ws1.write(line,11,(first_addr and contact_name) and (contact_courtesy+' '+contact_name+' '+contact_firstname).strip())
                            ws1.write(line,12,(first_addr and partner.user_id) and partner.user_id.name or '')
                            if first_addr:
                                first_line = line
                            line += 1
                        addr_text += '<li><strong>%s</strong><br/>%s<br/>%s</li>' % (full_name,full_street,zip_city)
                        first_addr = False
                # send of email
                if sended_this_time < data['form']['max_to_send']:
                    sending_ok = True
                    if sending_address and final_email and sending_ok:
                        if partner.id not in self._already_sended:
                            if True: #try:
                                full_texts = [template.subject,template.body % (contact_courtesy,contact_name,partner_id_shadow(partner.id),addr_text)]
                                if bd_de_test:
                                    print '---EMail Mocking---'
                                    print 'From : ' + sending_address
                                    print 'To : ' + final_email
                                    print 'Subject : ' + full_texts[0]
                                    print 'Body : ' + full_texts[1]
                                else:
                                    tools.email_send(sending_address, [final_email,], full_texts[0], full_texts[1], subtype=template.type)
                                mail_sended_obj.create(cr,uid,{
                                    'name':'memberdirectory.mailReactivateExcludedAddress',
                                    'subject':full_texts[0],
                                    'body':full_texts[1],
                                    'ressource':'res.partner:'+str(partner.id),
                                    'addr_from':sending_address,
                                    'addrs_to': '[%s,]' % final_email,
                                   })
                                self._already_sended.append(partner.id)
                                new_value = '[' + ','.join([str(x) for x in self._already_sended]) + ']'
                                parameter_obj.write(cr,uid,[self._parameter_id,],{'value':'[' + ','.join([str(x) for x in self._already_sended]) + ']'})
                                sended_this_time += 1
                            #except Exception,e:
                            #    sending_ok = False
                            if data['form']['excel']:
                                rec_line = first_line
                                while rec_line < line:
                                    if rec_line == first_line:
                                        ws1.write(rec_line,13,(sending_address and sending_ok and final_email) and 'OK' or '--No--')
                                        manual = not (sending_address and final_email and sending_ok)
                                    else:
                                        ws1.write(rec_line,13,'-')
                                    ws1.write(rec_line,14,manual and 'TODO' or '')
                                    rec_line += 1
                        else:
                            if data['form']['excel']:
                                rec_line = first_line
                                while rec_line < line:
                                    if rec_line == first_line:
                                        ws1.write(rec_line,13,'Already sended')
                                        manual = False
                                    else:
                                        ws1.write(rec_line,13,'-')
                                    ws1.write(rec_line,14,manual and 'TODO' or '')
                                    rec_line += 1
                    else:
                        if data['form']['excel']:
                            rec_line = first_line
                            while rec_line < line:
                                if rec_line == first_line:
                                    ws1.write(rec_line,13,'--No--')
                                    manual = False
                                else:
                                    ws1.write(rec_line,13,'-')
                                ws1.write(rec_line,14,manual and 'TODO' or '')
                                rec_line += 1
                else:
                    if data['form']['excel']:
                        rec_line = first_line
                        while rec_line < line:
                            if rec_line == first_line:
                                ws1.write(rec_line,13,'--No more credits--')
                                manual = not (sending_address and final_email)
                            else:
                                ws1.write(rec_line,13,'-')
                            ws1.write(rec_line,14,manual and 'TODO' or '')
                            rec_line += 1
            if data['form']['excel']:
                wb1.save('excluded_addresses_send.xls')
                result_file = open('excluded_addresses_send.xls','rb').read()
                data['form']['excluded_addresses_send.xls'] = base64.encodestring(result_file)
            results = u'Don\'t forget to treat partners without email address'
        else:
            results = u'No excluded addresses on members'
        data['form']['results'] = results
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel','gtk-stop'),('show','Count Concerned','gtk-find',True)]},
        },
        'show': {
            'actions': [_count_concerned],
            'result': {'type':'form', 'arch':inter_form, 'fields':inter_fields, 'state':[('end','Cancel','gtk-stop'),('send','Send','terp-crm',True)]},
        },
        'send': {
            'actions': [_send_emails],
            'result': {'type':'form', 'arch':final_form, 'fields':final_fields, 'state':[('end','Close')]}
        },
    }

send_emails_excluded_address('send_emails_excluded_address')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

