# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from osv import fields, osv
import datetime
import time
import tools
from tools.translate import _
from pyExcelerator import *
import calendar

pyexcel_colours = {
        'aqua'    : 0x0F,
        'cyan'    : 0x0F,
        'black'   : 0x08,
        'blue'    : 0x0C,
        'brown'   : 0x10,
        'magenta' : 0x0E,
        'fuchsia' : 0x0E,
        'gray'    : 0x17,
        'grey'    : 0x17,
        'green'   : 0x11,
        'lime'    : 0x0B,
        'navy'    : 0x12,
        'orange'  : 0x35,
        'pink'    : 0x21,
        'purple'  : 0x14,
        'red'     : 0x0A,
        'silver'  : 0x16,
        'white'   : 0x09,
        'yellow'  : 0x0D,
}

def date_french(date):
    return date[8:10]+'/'+date[5:7]+'/'+date[0:4]
def _months_between(date_soon,date_late):
    months_late = (int(date_late[0:4])*12)+int(date_late[5:7])
    months_soon = (int(date_soon[0:4])*12)+int(date_soon[5:7])
    return months_late-months_soon
def _end_of_month(date):
    year = int(date[0:4])
    month = int(date[5:7])
    return datetime.date(year,month,calendar.monthrange(year,month)[1]).strftime('%Y-%m-%d')
def _french_date(english_date):
    return english_date[8:10]+'/'+english_date[5:7]+'/'+english_date[0:4]
def _substract_a_year(date):
    # date = string YYYY-MM-DD
    year = int(date[0:4])
    rest = date[5:]
    if rest == '02-29':
        rest = '02-28'
    return str(year-1)+'-'+rest
def _beginning_of_month(date):
    # date = string YYYY-MM-DD
    return date[0:8]+'01'
    
class res_partner(osv.osv):
    _inherit = 'res.partner'
    _description = 'Partner'

    def _membership_state(self, cr, uid, ids, name, args, context=None):
        #the call to super is deactivated because of unresolved conflicts with the 5.0 version
        #of the membership module in state priorities. It is replaced by the ugly copy/paste below
        #res = super(res_partner, self)._membership_state(cr, uid, ids, name, args, context)
        res = {}
        for id in ids:
            res[id] = 'none'
        today = time.strftime('%Y-%m-%d')
        for id in ids:
            partner_data = self.browse(cr,uid,id)
            # commented by Philmer - the membership_cancel field is always filled if there is only one membership_line
            # with a date_cancel even if there is more membership_line more recent not canceled
            #if partner_data.membership_cancel and today > partner_data.membership_cancel:
            #    res[id] = 'canceled'
            #    continue
            if partner_data.state_id.id not in (1,) or not partner_data.active:
                res[id] = 'none'
                continue
            if partner_data.refuse_membership:
                res[id] = 'canceled'
                continue
            if partner_data.free_member:
                res[id] = 'free'
                continue
            s = 4
            if partner_data.member_lines:
                for mline in partner_data.member_lines:
                    if mline.date_from <= today and mline.date_to >= today and mline.account_invoice_line and mline.account_invoice_line.invoice_id and mline.account_invoice_line.invoice_id.type == 'out_invoice':
                            mstate = mline.account_invoice_line.invoice_id.state
                            if mstate == 'paid':
                                #TOCHECK : if paid by a credit nota : s = 2
                                # A quick way to check this is to check the state of the membership_line is 'canceled' or not
                                if mline.state == 'canceled':
                                    if s!=0 and s!=1:
                                        s = 2
                                else:
                                    s = 0
                                    break
                            elif mstate == 'open' and s!=0:
                                s = 1
                            elif mstate == 'cancel' and s!=0 and s!=1:
                                s = 2
                            elif  (mstate == 'draft' or mstate == 'proforma') and s!=0 and s!=1:
                                s = 3
                if s==4:
                    for mline in partner_data.member_lines:
                        if mline.date_from < today and mline.date_to < today and mline.date_from<=mline.date_to and mline.account_invoice_line and mline.account_invoice_line.invoice_id.state == 'paid' and mline.account_invoice_line.invoice_id.type == 'out_invoice' and mline.state == 'paid':
                            s = 5
                        else:
                            s = 6
                if s==0:
                    res[id] = 'paid'
                elif s==1:
                    res[id] = 'invoiced'
                elif s==2:
                    res[id] = 'canceled'
                elif s==3:
                    res[id] = 'waiting'
                elif s==5:
                    res[id] = 'old'
                elif s==6:
                    res[id] = 'none'
            #if partner_data.membership_stop and today > partner_data.membership_stop:
            #    res[id] = 'old'
            #    print 'changed to old'
            #    continue
            if partner_data.associate_member and res[id] not in ('paid','invoiced','waiting'):
                res_state = self._membership_state(cr, uid, [partner_data.associate_member.id], name, args, context)
                res[id] = res_state[partner_data.associate_member.id]
        return res

    def _get_membership_state_at_date(self,cr,uid,partner_id,searched_date):
        # result = membership_state at this date if member, else 'non-member' but also 'undefined' if before beginning of this count
        if searched_date < '2017-03-28':
            result = 'undefined'
        else:
            membership_state_obj = self.pool.get('membership_state_change')
            state_ids = membership_state_obj.search(cr,uid,[('partner_id','=',partner_id),'|',('active','=',True),('active','=',False)])
            result = 'non-member'
            if state_ids:
                states = membership_state_obj.read(cr,uid,state_ids,['date_from','date_to','state','active']) 
                for state in states:
                    if state['date_from'] <= searched_date and state['date_to'] > searched_date:
                        result = state['state']
                    if state['date_from'] <= searched_date and (not state['date_to']) and state['active']:
                        result = state['state']
        return result
    def _get_membership_state_change_at_date(self,cr,uid,partner_id,searched_date):
        # result = id of record in membership_state at this date; else False
        if searched_date < '2017-03-28':
            result = False
        else:
            membership_state_obj = self.pool.get('membership_state_change')
            state_ids = membership_state_obj.search(cr,uid,[('partner_id','=',partner_id),'|',('active','=',True),('active','=',False)])
            result = False
            if state_ids:
                states = membership_state_obj.read(cr,uid,state_ids,['date_from','date_to','state','active']) 
                for state in states:
                    if state['date_from'] <= searched_date and state['date_to'] > searched_date:
                        result = state['id']
                    if state['date_from'] <= searched_date and (not state['date_to']) and state['active']:
                        result = state['id']
        return result
    def _membership_state_job(self, cr, uid, ids=False, context={}):
        today = datetime.datetime.today().strftime('%Y-%m-%d')
        partner_obj = self.pool.get('res.partner')
        cr.execute("""SELECT id, active, membership_state, cci_date_start_membership, cci_date_stop_membership FROM res_partner""")
        res = cr.fetchall()
        index = 0
        size = 1000
        while res[index:index+size]:
            partners = [{'id':x[0],'active':x[1],'membership_state':x[2],'cci_date_start_membership':x[3],'cci_date_stop_membership':x[4]} for x in res[index:index+size]]
            partner_ids = [x[0] for x in res[index:index+size]]
            new_mstates = partner_obj._membership_state(cr, uid, partner_ids, '', args=None, context=None)
            changed_lines = []
            for partner in partners:
                if new_mstates.has_key( partner['id'] ):
                    if partner['membership_state'] <> new_mstates[partner['id']]:
                        # record the change
                        partner_name = partner_obj.read(cr, uid, [partner['id']], ['name'] )[0]['name']
                        try:
                            partner_obj.write(cr, uid, [partner['id']], {}, context )
                            changed_lines.append( u"Partenaire '%s' (id=%s) passe de '%s' à '%s'" % (partner_name, str(partner['id']),partner['membership_state'],new_mstates[partner['id']]) )
                        except Exception,e:
                            changed_lines.append( u"Partenaire '%s' (id=%s) connait des problemes d'ecritures" % (partner_name, str(partner['id'])) )
                    if ( not partner['cci_date_start_membership'] ) and ( not partner['cci_date_stop_membership'] ) and new_mstates[partner['id']] in ('paid','free','invoiced'):
                        # new member today
                        try:
                            partner_obj.write(cr, uid, [partner['id']], {'cci_date_start_membership':today}, context )
                        except Exception,e:
                            changed_lines.append( u"Partenaire '%s' (id=%s) connait des problemes d'ecritures" % (partner_name, str(partner['id'])) )
                    elif ( partner['cci_date_start_membership'] ) and ( not partner['cci_date_stop_membership'] ) and new_mstates[partner['id']] not in ('paid','free','invoiced'):
                        # old member today
                        try:
                            partner_obj.write(cr, uid, [partner['id']], {'cci_date_stop_membership':today}, context )
                        except Exception,e:
                            changed_lines.append( u"Partenaire '%s' (id=%s) connait des problemes d'ecritures" % (partner_name, str(partner['id'])) )
                    elif ( partner['cci_date_start_membership'] ) and ( partner['cci_date_stop_membership'] ) and new_mstates[partner['id']] in ('paid','free','invoiced'):
                        # member today again
                        try:
                            partner_obj.write(cr, uid, [partner['id']], {'cci_date_stop_membership':False}, context )
                        except Exception,e:
                            changed_lines.append( u"Partenaire '%s' (id=%s) connait des problemes d'ecritures" % (partner_name, str(partner['id'])) )
                    elif ( partner['cci_date_start_membership'] ) and ( partner['cci_date_stop_membership'] ) and (partner['cci_date_start_membership'] >= partner['cci_date_stop_membership']) and new_mstates[partner['id']] not in ('paid','free','invoiced'):
                        # non member today again
                        try:
                            partner_obj.write(cr, uid, [partner['id']], {'cci_date_stop_membership':today}, context )
                        except Exception,e:
                            changed_lines.append( u"Partenaire '%s' (id=%s) connait des problemes d'ecritures" % (partner_name, str(partner['id'])) )
            index += size
        if changed_lines:
            final_text = u'Changements hebdomadaires : \n' + ( u'\n'.join( changed_lines ) )
        else:
            final_text = u"Changements hebdomadaires : aucun"
        membership_check_obj = self.pool.get('cci_membership.membership_check')
        id = membership_check_obj.create(cr, uid, {
            'name': datetime.datetime.today().strftime('%Y-%m-%d-%H:%M:%S'),
            'count' : len(changed_lines),
            'status': final_text,
            })
        return True
    def onchange_membership_explanation(self, cr, uid, ids):
        newvalue = {'read_before_next_membership_bill':True}
        return {'value':newvalue}
    def _record_membership_state(self,cr,uid,partner_id,partner_state,partner_free,partner_refuse,partner_billed_id,current_state_record_id=None):
        # current_state_record_id = 0 means : no current state record, no need to search
        # this method is called if there is changes detected
        obj_state = self.pool.get('membership_state_change')
        today = datetime.date.today().strftime('%Y-%m-%d')
        if current_state_record_id is None:
            state_ids = obj_state.search(cr,uid,[('partner_id','=',partner_id)])
            if state_ids and len(state_ids) == 1:
                current_state_id = state_ids[0]
            else:
                current_state_id = 0
        else:
            current_state_id = current_state_record_id
        if partner_state in ('free','invoiced','paid'):
            if current_state_id != 0:
                # we have to close the old state before creating a new one
                obj_state.write(cr,uid,[current_state_id,],{'date_to':today,'active':False})
            # we have to create a new state 'member'
            (membership_past_year,membership_current_year,membership_next_year) = self._get_membership_amounts(cr,uid,partner_id,int(today[0:4]))
            new_state = {'partner_id':partner_id,
                         'date_from':today,
                         'state':partner_state,
                         'free':partner_free,
                         'refuse':partner_refuse,
                         'billed_partner_id':partner_billed_id,
                         'amount_past_year': membership_past_year,
                         'amount_current_year': membership_current_year,
                         'amount_next_year': membership_next_year,
                         'active':True,
                        }
            obj_state.create(cr,uid,new_state)
        else:
            if current_state_id == 0:
                # nothing to do : non member and no current 'member' state
                pass
            else:
                # we have to close the current state
                obj_state.write(cr,uid,[current_state_id,],{'date_to':today,'active':False})
        return True
    def _get_membership_amounts(self,cr,uid,partner_id,year):
        # get the amount of membership  invoiced for the three years for this partner : current year, past year and next year
        journal_ids = self.pool.get('account.journal').search(cr,uid,[('type','=','sale')])
        if journal_ids:
            obj_product = self.pool.get('product.product')
            past_year_prod_ids = obj_product.search(cr,uid,[('membership_year','=',year-1)])
            curr_year_prod_ids = obj_product.search(cr,uid,[('membership_year','=',year)])
            next_year_prod_ids = obj_product.search(cr,uid,[('membership_year','=',year+1)])
            obj_amline = self.pool.get('account.move.line')
            result = []
            for prod_ids in (past_year_prod_ids,curr_year_prod_ids,next_year_prod_ids):
                if prod_ids:
                    amline_ids = obj_amline.search(cr,uid,[('partner_id','=',partner_id),('product_id','in',prod_ids)])
                    total_this_year = 0.0
                    if amline_ids:
                        amlines = obj_amline.read(cr,uid,amline_ids,['debit','credit'])
                        for amline in amlines:
                            total_this_year += ((amline['credit'] or 0.0)- (amline['debit'] or 0.0))
                    result.append(total_this_year)
                else:
                    result.append(0.0)
        else:
            result = (0.0,0.0,0.0)
        return result
    def _find_membership_state_changes(self,cr,uid):
        # Start log
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'Record Membership State Changes')

        # 1. Get current states of members
        dCurrentStates = {}
        obj_membership_state = self.pool.get('membership_state_change')
        current_state_ids = obj_membership_state.search(cr,uid,[])
        if current_state_ids:
            current_states = obj_membership_state.read(cr,uid,current_state_ids, ['partner_id','state','free','refuse','billed_partner_id'])
            for cstate in current_states:
                dCurrentStates[cstate['partner_id'][0]] = cstate
        current_state_partner_ids = dCurrentStates.keys()
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n\nFound Current Active States : %i' % len(current_state_ids) )
        # 2. get current members with data
        obj_partner = self.pool.get('res.partner')
        member_ids = obj_partner.search(cr,uid,[('membership_state','in',['free','invoiced','paid'])])
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nDetected members : %i' % len(member_ids) )
        difference_ids = []
        new_member_ids = []
        new_paid_free_member_ids = []
        if member_ids:
            members = obj_partner.read(cr,uid,member_ids,['membership_state','free_member','refuse_membership','associate_member'])
            for member in members:
                if member['id'] in current_state_partner_ids:
                    current_state = dCurrentStates[member['id']]
                    if current_state['state'] <> member['membership_state'] or \
                       current_state['free'] <> member['free_member'] or \
                       current_state['refuse'] <> member['refuse_membership'] or \
                       (current_state['billed_partner_id'] and current_state['billed_partner_id'][0] or False) <> (member['associate_member'] and member['associate_member'][0] or False):
                        #difference => record new state
                        self._record_membership_state(cr,uid,member['id'],member['membership_state'],member['free_member'],member['refuse_membership'],member['associate_member'] and member['associate_member'][0] or False,current_state['id'])
                        difference_ids.append(member['id'])
                        if current_state['state'] not in ['free','paid'] and member['membership_state'] in ['free','paid']:
                            new_paid_free_member_ids.append(member['id'])
                else:
                    # not already record => new state recording
                    self._record_membership_state(cr,uid,member['id'],member['membership_state'],member['free_member'],member['refuse_membership'],member['associate_member'] and member['associate_member'][0] or False,0)
                    new_member_ids.append(member['id'])
                    if member['membership_state'] in ['free','paid']:
                        new_paid_free_member_ids.append(member['id'])
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nNew Members IDs : [%s]' % ','.join([str(x) for x in new_member_ids]) )
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nDifference IDs : [%s]' % ','.join([str(x) for x in difference_ids]) )
        # 3. inactivate states of old members
        old_member_ids = []
        for partner_id in current_state_partner_ids:
            if partner_id not in member_ids:
                old_member_ids.append(partner_id)
        old_members = obj_partner.read(cr,uid,old_member_ids,['membership_state','free_member','refuse_membership','associate_member'])
        for old_member in old_members:
            # old member => inactivate old state
            self._record_membership_state(cr,uid,old_member['id'],old_member['membership_state'],old_member['free_member'],old_member['refuse_membership'],old_member['associate_member'] and old_member['associate_member'][0] or False,dCurrentStates[old_member['id']]['id'])
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nNew Old Member IDs : [%s]' % ','.join([str(x) for x in old_member_ids]) )
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,True,'\n---end---')
        return (new_member_ids,new_paid_free_member_ids,old_member_ids)
    def _send_new_members_followup(self,cr,uid,emails_list_all,emails_list_paid):
        persons_followup = []
        if emails_list_all:
            persons_followup = emails_list_all.split(',')
        (new_member_ids,new_paid_free_member_ids,old_member_ids) = self._find_membership_state_changes(cr,uid)
        bd_de_test = True
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['BDDeTest'])
        if param_values.has_key('BDDeTest'):
            if param_values['BDDeTest'] == 0:
                bd_de_test = False
        if new_member_ids and persons_followup:
            members = self.pool.get('res.partner').browse(cr,uid,new_member_ids)
            dSalesmans = {}
            datas = []
            for member in members:
                # prepare data to publish
                new_member_data = u'<li>%s [%i]' % (member.name,member.id)
                if member.address:
                    found_address = False
                    for addr in member.address:
                        if addr.type == 'default':
                            new_member_data += u'<br/>%s - %s %s' % (addr.street or '',
                                                                   addr.zip_id and addr.zip_id.name or '',
                                                                   addr.zip_id and addr.zip_id.city or '')
                        new_member_data += '</li>'
                        found_address = True
                        break
                    if not found_address:
                        addr = member.address[0]
                        new_member_data += u'<br/>%s - %s %s' % (addr.street or '',
                                                               addr.zip_id and addr.zip_id.name or '',
                                                               addr.zip_id and addr.zip_id.city or '')
                        new_member_data += '</li>'
                datas.append(new_member_data)
                if member.user_id and member.user_id.email:
                    salesman = member.user_id.email
                    if dSalesmans.has_key(salesman):
                        salesman_data = dSalesmans[salesman]
                        salesman_data.append(new_member_data)
                    else:
                        salesman_data = [new_member_data,]
                    dSalesmans[salesman] = salesman_data
            if datas and persons_followup:
                subject = 'Nouveau(s) membre(s) du jour'.encode('ascii', 'xmlcharrefreplace')
                body = ("""<p>Voici la liste des nouveaux membres du jour :</p><ul>%s</ul>""" % '\n'.join(datas)).encode('ascii', 'xmlcharrefreplace')
                if not bd_de_test:
                    tools.email_send('no-reply@ccilvn.be', persons_followup, subject, body, subtype="html")
                else:
                    print '--- EMail Mockup ---'
                    print 'To : ' + ' ; '.join(persons_followup)
                    print 'Subject : ' + subject
                    print 'Body : ' + body
            if dSalesmans:
                for (key,datas) in dSalesmans.items():
                    subject = 'Vos nouveau(s) membre(s) du jour'.encode('ascii', 'xmlcharrefreplace')
                    body = ("""<p>Voici la liste de vos nouveaux membres du jour :</p><ul>%s</ul>""" % '\n'.join(datas)).encode('ascii', 'xmlcharrefreplace')
                    if not bd_de_test:
                        tools.email_send('no-reply@ccilvn.be', [key,], subject, body, subtype="html")
                    else:
                        print '--- EMail Mockup ---'
                        print 'To : ' + key
                        print 'Subject : ' + subject
                        print 'Body : ' + body
        if emails_list_paid:
            persons_followup = emails_list_paid.split(',')
        if new_paid_free_member_ids:
            members = self.pool.get('res.partner').browse(cr,uid,new_paid_free_member_ids)
            dSalesmans = {}
            datas = []
            for member in members:
                # prepare data to publish
                new_member_data = u'<li>%s [%i] - State : %s' % (member.name,member.id,member.membership_state)
                if member.address:
                    found_address = False
                    for addr in member.address:
                        if addr.type == 'default':
                            new_member_data += u'<br/>%s - %s %s' % (addr.street or '',
                                                                   addr.zip_id and addr.zip_id.name or '',
                                                                   addr.zip_id and addr.zip_id.city or '')
                        new_member_data += '</li>'
                        found_address = True
                        break
                    if not found_address:
                        addr = member.address[0]
                        new_member_data += u'<br/>%s - %s %s' % (addr.street or '',
                                                               addr.zip_id and addr.zip_id.name or '',
                                                               addr.zip_id and addr.zip_id.city or '')
                        new_member_data += '</li>'
                datas.append(new_member_data)
                if member.user_id and member.user_id.email:
                    salesman = member.user_id.email
                    if dSalesmans.has_key(salesman):
                        salesman_data = dSalesmans[salesman]
                        salesman_data.append(new_member_data)
                    else:
                        salesman_data = [new_member_data,]
                    dSalesmans[salesman] = salesman_data
            if datas and persons_followup:
                subject = 'Nouveau(s) membre(s) PAYANTS (ou GRATUITS) du jour'.encode('ascii', 'xmlcharrefreplace')
                body = ("""<p>Voici la liste des nouveaux membres payants ou gratuits du jour :</p><ul>%s</ul>""" % '\n'.join(datas)).encode('ascii', 'xmlcharrefreplace')
                if not bd_de_test:
                    tools.email_send('no-reply@ccilvn.be', persons_followup, subject, body, subtype="html", email_bcc=['philmer@ccilvn.be',])
                else:
                    print '--- EMail Mockup ---'
                    print 'To : ' + ' ; '.join(persons_followup)
                    print 'Subject : ' + subject
                    print 'Body : ' + body
            if dSalesmans:
                for (key,datas) in dSalesmans.items():
                    subject = 'Vos nouveau(s) membre(s) PAYANTS (ou GRATUITS) du jour'.encode('ascii', 'xmlcharrefreplace')
                    body = ("""<p>Voici la liste de vos nouveaux membres du jour :</p><ul>%s</ul>""" % '\n'.join(datas)).encode('ascii', 'xmlcharrefreplace')
                    if not bd_de_test:
                        tools.email_send('no-reply@ccilvn.be', [key,], subject, body, subtype="html", email_bcc=['philmer@ccilvn.be',])
                    else:
                        print '--- EMail Mockup ---'
                        print 'To : ' + key
                        print 'Subject : ' + subject
                        print 'Body : ' + body
        if old_member_ids and persons_followup:
            old_members = self.pool.get('res.partner').browse(cr,uid,old_member_ids)
            dSalesmans = {}
            datas = []
            for old_member in old_members:
                # prepare data to publish
                old_member_data = u'<li>%s [%i]' % (old_member.name,old_member.id)
                if old_member.address:
                    found_address = False
                    for addr in old_member.address:
                        if addr.type == 'default':
                            old_member_data += u'<br/>%s - %s %s' % (addr.street or '',
                                                                     addr.zip_id and addr.zip_id.name or '',
                                                                     addr.zip_id and addr.zip_id.city or '')
                        old_member_data += '</li>'
                        found_address = True
                        break
                    if not found_address:
                        addr = old_member.address[0]
                        old_member_data += u'<br/>%s - %s %s' % (addr.street or '',
                                                                 addr.zip_id and addr.zip_id.name or '',
                                                                 addr.zip_id and addr.zip_id.city or '')
                        old_member_data += '</li>'
                datas.append(old_member_data)
                if old_member.user_id and old_member.user_id.email:
                    salesman = old_member.user_id.email
                    if dSalesmans.has_key(salesman):
                        salesman_data = dSalesmans[salesman]
                        salesman_data.append(old_member_data)
                    else:
                        salesman_data = [old_member_data,]
                    dSalesmans[salesman] = salesman_data
            if datas and persons_followup:
                subject = 'Membre(s) perdu(s) du jour'.encode('ascii', 'xmlcharrefreplace')
                body = ("""<p>Voici la liste des membres perdus du jour :</p><ul>%s</ul>""" % '\n'.join(datas)).encode('ascii', 'xmlcharrefreplace')
                if not bd_de_test:
                    tools.email_send('no-reply@ccilvn.be', persons_followup, subject, body, subtype="html")
                else:
                    print '--- EMail Mockup ---'
                    print 'To : ' + ' ; '.join(persons_followup)
                    print 'Subject : ' + subject
                    print 'Body : ' + body
            if dSalesmans:
                for (key,datas) in dSalesmans.items():
                    subject = 'Vos membre(s) perdu(s) du jour'.encode('ascii', 'xmlcharrefreplace')
                    body = ("""<p>Voici la liste de vos membres perdus du jour :</p><ul>%s</ul>""" % '\n'.join(datas)).encode('ascii', 'xmlcharrefreplace')
                    if not bd_de_test:
                        tools.email_send('no-reply@ccilvn.be', [key,], subject, body, subtype="html")
                    else:
                        print '--- EMail Mockup ---'
                        print 'To : ' + key
                        print 'Subject : ' + subject
                        print 'Body : ' + body
        return True
    def _member_states_by_dates(self,cr,uid,partner_id):
        mstate_obj = self.pool.get('membership_state_change')
        partner_obj = self.pool.get('res.partner')
        partner = partner_obj.read(cr,uid,[partner_id,],['membership_state'])[0]
        mstate_ids = mstate_obj.search(cr,uid,[('partner_id','=',partner_id),'|',('active','=',True),('active','=',False)])
        current_state = 'none'
        date_from = '2017-03-27'
        date_to = ''
        result = []
        if mstate_ids:
            mstates = mstate_obj.read(cr,uid,mstate_ids,['state','date_from','date_to'])
            s_mstates = sorted(mstates,key=lambda row: row['date_from'])
            for mstate in s_mstates:
                if date_to:
                    result.append({'state':current_state,
                                   'date_from':date_from,
                                   'date_to':(datetime.datetime.strptime(date_to,'%Y-%m-%d')+datetime.timedelta(days=-1)).strftime('%Y-%m-%d'),
                                  })
                else:
                    result.append({'state':current_state,
                                   'date_from':date_from,
                                   'date_to':(datetime.datetime.strptime(mstate['date_from'],'%Y-%m-%d')+datetime.timedelta(days=-1)).strftime('%Y-%m-%d'),
                                  })
                current_state = mstate['state']
                date_from = mstate['date_from']
                date_to = mstate['date_to']
            if date_to and date_to < datetime.date.today().strftime('%Y-%m-%d'):
                result.append({'state':current_state,
                               'date_from':date_from,
                               'date_to':(datetime.datetime.strptime(date_to,'%Y-%m-%d')+datetime.timedelta(days=-1)).strftime('%Y-%m-%d'),
                              })
                result.append({'state':partner['membership_state'],
                               'date_from':date_to,
                               'date_to':datetime.date.today().strftime('%Y-%m-%d'),
                              })
            else:
                result.append({'state':current_state,
                               'date_from':date_from,
                               'date_to':datetime.date.today().strftime('%Y-%m-%d'),
                              })
        else:
            result.append({'state':current_state,
                           'date_from':date_from,
                           'date_to':datetime.date.today().strftime('%Y-%m-%d'),
                          })
        return result
    def _member_types_by_dates(self,cr,uid,partner_id):
        states = self._member_states_by_dates(cr,uid,partner_id)
        dConvStates = {'free':'member','invoiced':'member','paid':'member','none':'nonmember','old':'nonmember','canceled':'nonmember','waiting':'nonmember'}
        res = []
        if len(states) == 1:
            res = states
            states[0]['type'] = dConvStates[state[0]]
        else:
            current = states[0]
            current['type'] = dConvStates[current['state']]
            for state in states[1:]:
                if current['type'] == dConvStates[state['state']]:
                    current['state'] = state['state'] ##  state == last state of this type
                    current['date_to'] = state['date_to']
                else:
                    current['type'] = dConvStates[current['state']]
                    res.append(current)
                    current = state.copy()
                    current['type'] = dConvStates[current['state']]
            res.append(current)
        return res
    def _changed_membership_type(self,cr,uid,partner_id,date_from,date_to):
        # detect if partner change from member (paid,invoiced,free) to non-member (other states) or the contrary and the date of the last change
        # result : (change_type,date_change) with change_type = 'same','new', or 'lost'
        types = self._member_types_by_dates(cr,uid,partner_id)
        change_type = 'same'
        date_change = ''
        for type_member in reversed(types):
            if type_member['date_from'] >= date_from and type_member['date_from'] <= date_to:
                change_type = (type_member['type'] == 'member') and 'new' or 'lost'
                date_change = type_member['date_from']
                break
        return (change_type,date_change)
    def _auto_changed_members(self,cr,uid,emails=['philmer@ccilvn.be',]):
        if emails:
            partner_obj = self.pool.get('res.partner')
            today = datetime.date.today()
            first_of_current_month = today.replace(day=1)
            date_to_as_date = first_of_current_month + datetime.timedelta(days=-1)
            date_from_as_date = date_to_as_date.replace(day=1)
            date_from = date_from_as_date.strftime('%Y-%m-%d')
            date_to = date_to_as_date.strftime('%Y-%m-%d')
            (news,losts) = self._changed_members_between_2_dates(cr,uid,date_from,date_to,'date_from','date_from')
            conso_shemas = ['FreeEvents','PaidEvents','PaidAE','PaidCNN','PaidTC','PaidMS','PaidPublication','PaidSponsoring','PaidMissions']
            date_conso_to = date_to
            date_conso_from = max('2017-03-29',_substract_a_year(_beginning_of_month(date_to)))
            new_members_conso = partner_obj._new_members_between_2_dates(cr,uid,date_conso_from,date_conso_to,'date_from')	
            new_member_conso_ids = [x['partner_id'] for x in new_members_conso]
            dResCons = partner_obj._extract_consumption(cr,uid,date_conso_from,date_conso_to,conso_shemas,new_member_conso_ids)
            consos = []
            for member in new_members_conso:
                member.update(dResCons[member['partner_id']])
                consos.append(member)
            filename = 'chg_members'+datetime.datetime.now().strftime('%Y%m%d_%H%M%S')+'.xls'
            self._members_followup_excel_file(cr,uid,filename,date_from,date_to,today,news,losts,consos,_french_date(date_from) + ' -> ' + _french_date(date_to))
            # sending of the results
            BD_TEST = True
            parameter_obj = self.pool.get('cci_parameters.cci_value')
            param_values = parameter_obj.get_value_from_names(cr,uid,['BDDeTest'])
            if param_values.has_key('BDDeTest'):
                if param_values['BDDeTest'] == 0:
                    BD_TEST = False
            subject = 'Liste mensuelle de suivi des nouveaux membres (et des perdus...)'
            body = """<p>Voici la liste des nouveaux membres du mois passé, avec également les membres perdus le même mois.</p>
                      <p>Avec leur consommation de produits CCI.</p>
                      <p>Bon suivi ....</p><p>L'ERP</p>"""
            for email_addr in emails:
                if BD_TEST:
                    print '--- EMail Mockup ---'
                    print 'To : ' + email_addr
                    print 'Subject : ' + subject
                    print 'Body : ' + body
                else:
                    tools.email_send('philmer@ccilvn.be', [email_addr,], subject, body, subtype="html", attach=[(filename,open(filename,'rb').read())])
        return True
    def _changed_members_between_2_dates(self,cr,uid,date_from,date_to,sort_field_new='date_from',sort_field_lost='date_from'):
        # this method extract the list of all partners whose membership_state changed between two dates.
        # date_from = 'YYYY-MM-DD'
        # date_to = 'YYYY-MM-DD'
        # sort_field_new = name of the field to sort the result list of new members
        # sort_field_lost = name of the field to sort the result list of lost members
        # result = [{'partner_id','partner_name','today_membership','date_from','date_to','billed_partner_name','salesman','amountYYYY'}] with multiples amountXXX
        if date_from > date_to:
            date_to,date_from = date_from,date_to
        if date_from <= '2017-03-28':
            date_from = '2017-03-28'
        year_from = int(date_from[0:4])-1
        year_to  = int(date_to[0:4])+1
        ctr = year_from
        years = range(year_from,year_to)
        amounts_base = {}
        for year in years:
            amounts_base['amount_'+str(year)] = 0.0
        #
        news = []
        losts = []
        #
        mstate_obj = self.pool.get('membership_state_change')
        mstate_ids = mstate_obj.search(cr,uid,['|',('active','=',True),('active','=',False)])
        partner_ids = []
        if mstate_ids:
            mstates = mstate_obj.read(cr,uid,mstate_ids,['partner_id'])
            for mstate in mstates:
                if mstate['partner_id'] and mstate['partner_id'][0] not in partner_ids:
                    partner_ids.append(mstate['partner_id'][0])
        if partner_ids:
            partner_obj = self.pool.get('res.partner')
            partners = partner_obj.browse(cr,uid,partner_ids)
            dNewPartners = {}
            dLostPartners = {}
            for partner in partners:
                (change_type,date_change) = partner_obj._changed_membership_type(cr,uid,partner.id,date_from,date_to)
                if change_type == 'new':
                    if dNewPartners.has_key(partner.id):
                        record = dNewPartners[partner.id]
                    else:
                        record = {'partner_id':partner.id,
                                  'partner_name':partner.name,
                                  'today_membership':partner.membership_state,
                                  'membership_type_date_to':'',
                                  'date_from':date_change,
                                  'billed_partner_name':0,   ##mstate.billed_partner_id.name,
                                  'salesman':partner.user_id and partner.user_id.name or ''}
                        record.update(amounts_base)
                        #current_year = int(mstate['date_from'][0:4])
                    dNewPartners[partner.id] = record
                elif change_type == 'lost':
                    if dLostPartners.has_key(partner.id):
                        record = dLostPartners[partner.id]
                    else:
                        record = {'partner_id':partner.id,
                                  'partner_name':partner.name,
                                  'today_membership':partner.membership_state,
                                  'membership_type_date_to':'',
                                  'date_from':date_change,
                                  'billed_partner_name':0,  ##mstate.billed_partner_id.name,
                                  'salesman':partner.user_id and partner.user_id.name or ''}
                        record.update(amounts_base)
                    dLostPartners[partner.id] = record
            # TODO : set the amount of membership by concerned years
            if dNewPartners:
               news = dNewPartners.values()
               if sort_field_new in dNewPartners.values()[0].keys():
                   news.sort(key=lambda row: row[sort_field_new],reverse=(sort_field_new in ['date_from',]))
            if dLostPartners:
               losts = dLostPartners.values()
               if sort_field_lost in dLostPartners.values()[0].keys():
                   losts.sort(key=lambda row: row[sort_field_lost],reverse=(sort_field_lost in ['date_from',]))
        return (news,losts)
    def _new_members_between_2_dates(self,cr,uid,date_from,date_to,sort_field='date_from'):
        # this method extract the list of all partners set as members between two dates. They must still be members at the date_to
        # date_from = 'YYYY-MM-DD'
        # date_to = 'YYYY-MM-DD'
        # sort = name of the field to sort the result list
        # result = [{'partner_id','partner_name','today_membership','membership_type_date_to','date_from','date_to','billed_partner_name','salesman','amountYYYY'}] with multiples amountXXX
        if date_from > date_to:
            date_to,date_from = date_from,date_to
        year_from = int(date_from[0:4])-1
        year_to  = int(date_to[0:4])+1
        ctr = year_from
        years = range(year_from,year_to)
        amounts_base = {}
        for year in years:
            amounts_base['amount_'+str(year)] = 0.0
        #
        mstate_obj = self.pool.get('membership_state_change')
        mstate_ids = mstate_obj.search(cr,uid,[('date_from','>=',date_from),('date_from','<=',date_to),'|',('active','=',True),('active','=',False)])
        result = []
        dPartners = {}
        if mstate_ids:
            mstates = mstate_obj.browse(cr,uid,mstate_ids)
            for mstate in mstates:
                if dPartners.has_key(mstate.partner_id.id):
                    record = dPartners[mstate.partner_id.id]
                else:
                    record = {'partner_id':mstate.partner_id.id,
                              'partner_name':mstate.partner_id.name,
                              'today_membership':mstate.partner_id.membership_state,
                              'membership_type_date_to':mstate.state,
                              'date_from':mstate['date_from'] or '',
                              'date_to':mstate.date_to or '',
                              'billed_partner_name':mstate.billed_partner_id.name,
                              'salesman':mstate.partner_id.user_id and mstate.partner_id.user_id.name or ''}
                    record.update(amounts_base)
                    current_year = int(mstate['date_from'][0:4])
                    record['amount_'+str(current_year-1)] = mstate.amount_past_year or 0.0
                    record['amount_'+str(current_year)] = mstate.amount_current_year or 0.0
                    record['amount_'+str(current_year+1)] = mstate.amount_next_year or 0.0
                dPartners[mstate.partner_id.id] = record
            #
            result = dPartners.values()
            if sort_field in dPartners.values()[0].keys():
                result.sort(key=lambda row: row[sort_field],reverse=(sort_field in ['date_from','date_to']))
        return result
    def _lost_members_between_2_dates(self,cr,uid,date_from,date_to,sort_field='date_to'):
        # this method extract the list of all partners set as non members between two dates.
        # date_from = 'YYYY-MM-DD'
        # date_to = 'YYYY-MM-DD'
        # sort = name of the field to sort the result list
        # result = [{'partner_id','partner_name','today_membership','membership_type_date_to','date_from','date_to','billed_partner_name','salesman','amountYYYY'}] with multiples amountXXX
        if date_from > date_to:
            date_to,date_from = date_from,date_to
        year_from = int(date_from[0:4])-1
        year_to  = int(date_to[0:4])+1
        ctr = year_from
        years = range(year_from,year_to)
        amounts_base = {}
        for year in years:
            amounts_base['amount_'+str(year)] = 0.0
        #
        mstate_obj = self.pool.get('membership_state_change')
        mstate_ids = mstate_obj.search(cr,uid,[('date_to','>=',date_from),('date_to','<=',date_to),'|',('active','=',True),('active','=',False)])
        result = []
        dPartners = {}
        if mstate_ids:
            mstates = mstate_obj.browse(cr,uid,mstate_ids)
            for mstate in mstates:
                if dPartners.has_key(mstate.partner_id.id):
                    record = dPartners[mstate.partner_id.id]
                else:
                    record = {'partner_id':mstate.partner_id.id,
                              'partner_name':mstate.partner_id.name,
                              'today_membership':mstate.partner_id.membership_state,
                              'membership_type_date_to':mstate.state,
                              'date_from':mstate['date_from'] or '',
                              'date_to':mstate.date_to or '',
                              'billed_partner_name':mstate.billed_partner_id.name,
                              'salesman':mstate.partner_id.user_id and mstate.partner_id.user_id.name or ''}
                    record.update(amounts_base)
                    current_year = int(mstate['date_from'][0:4])
                    record['amount_'+str(current_year-1)] = mstate.amount_past_year or 0.0
                    record['amount_'+str(current_year)] = mstate.amount_current_year or 0.0
                    record['amount_'+str(current_year+1)] = mstate.amount_next_year or 0.0
                dPartners[mstate.partner_id.id] = record
            #
            result = dPartners.values()
            if sort_field in dPartners.values()[0].keys():
                result.sort(key=lambda row: row[sort_field],reverse=(sort_field in ['date_from','date_to']))
        return result
    def _extract_consumption(self,cr,uid,date_from,date_to,shemas,partner_ids):
        # shemas = list of keywords identifying an substract of data
        # partner_ids = limit the calculation with this list of partners (new members for example)
        #               if none or empty list : all partners
        # dResult = dict of partner ids with shemas fields filled
        # TODO : if no schema, give only the amount of sales
        partner_obj = self.pool.get('res.partner')
        if partner_ids is None or not partner_ids:
            partner_ids = partner_obj.search(cr,uid,[])
        if date_from > date_to:
            (date_from,date_to) = (date_to,date_from)
        dResult = {}
        for partner_id in partner_ids:
            dResult[partner_id] = {'id':partner_id}
        if 'FreeEvents' in shemas:
            event_obj = self.pool.get('event.event')
            registration_obj = self.pool.get('event.registration')
            event_ids = event_obj.search(cr,uid,[('date_begin','>=',date_from),('date_begin','<=',date_to),('state','<>','cancel')])
            if event_ids:
                reg_ids = registration_obj.search(cr,uid,[('event_id','in',event_ids),('state','<>','cancel'),('unit_price','>',-0.01),('unit_price','<',0.01)])
                if reg_ids:
                    regs = registration_obj.read(cr,uid,reg_ids,['partner_invoice_id'])
                    for line in regs:
                        if line['partner_invoice_id'] and line['partner_invoice_id'][0] in partner_ids:
                            result = dResult[line['partner_invoice_id'][0]]
                            if result.has_key('FreeEvents'):
                                result['FreeEvents'] += 1
                            else:
                                result['FreeEvents'] = 1
                            dResult[line['partner_invoice_id'][0]] = result
        if 'PaidEvents' in shemas:
            event_obj = self.pool.get('event.event')
            registration_obj = self.pool.get('event.registration')
            event_ids = event_obj.search(cr,uid,[('date_begin','>=',date_from),('date_begin','<=',date_to),('state','<>','cancel')])
            if event_ids:
                reg_ids = registration_obj.search(cr,uid,[('event_id','in',event_ids),('state','<>','cancel'),('unit_price','>',0.01)])
                if reg_ids:
                    regs = registration_obj.read(cr,uid,reg_ids,['partner_invoice_id','unit_price'])
                    for line in regs:
                        if line['partner_invoice_id'] and line['partner_invoice_id'][0] in partner_ids:
                            result = dResult[line['partner_invoice_id'][0]]
                            if result.has_key('PaidEvents'):
                                result['PaidEvents'] += (line['unit_price'] or 0.0)
                            else:
                                result['PaidEvents'] = (line['unit_price'] or 0.0)
                            dResult[line['partner_invoice_id'][0]] = result
        if 'PaidAE' in shemas or 'PaidMS' in shemas or 'PaidCNN' in shemas or 'PaidTC' in shemas:
            club_obj = self.pool.get('cci_club.club')
            club_type_obj = self.pool.get('cci_club.club_type')
            aml_obj = self.pool.get('account.move.line')
            journal_obj = self.pool.get('account.journal')
            jnl_sales_ids = journal_obj.search(cr,uid,[('type','=','sale')])
            type_clubs = list(set(shemas) & set(['PaidAE','PaidMS','PaisTC','PaidCNN']))
            dTypes = {'PaidAE':'ALTER EGO','PaidCNN':'CNN','PaidTC':'TABLE DE CONVERSATION','PaidMS':'MENS SANA'}
            for type_club in type_clubs:
                product_ids = []
                name_type = dTypes[type_club]
                club_type_ids = club_type_obj.search(cr,uid,[('name','=',name_type)])
                if club_type_ids:
                    club_ids = club_obj.search(cr,uid,[('type_id','in',club_type_ids)])
                    if club_ids:
                        clubs = club_obj.read(cr,uid,club_ids,['product_id'])
                        for club in clubs:
                            if club['product_id'] and club['product_id'][0] not in product_ids:
                                product_ids.append(club['product_id'][0])
                if product_ids:
                    aml_ids = aml_obj.search(cr,uid,[('product_id','in',product_ids),('journal_id','in',jnl_sales_ids),('date','>=',date_from),('date','<=',date_to)])
                    if aml_ids:
                        amlines = aml_obj.read(cr,uid,aml_ids,['partner_id','credit','debit'])
                        for amline in amlines:
                            if amline['partner_id'] and amline['partner_id'][0] in partner_ids:
                                result = dResult[amline['partner_id'][0]]
                                if result.has_key(type_club):
                                    result[type_club] += ((amline['credit'] or 0.0)-(amline['debit'] or 0.0))
                                else:
                                    result[type_club] = ((amline['credit'] or 0.0)-(amline['debit'] or 0.0))
                                dResult[amline['partner_id'][0]] = result
        if 'PaidPublication' in shemas:
            pcateg_obj = self.pool.get('product.category')
            pcateg_ids = pcateg_obj.search(cr,uid,[])
            pcategs = pcateg_obj.read(cr,uid,pcateg_ids,['id','complete_name'])
            select_categ_ids = []
            for pcateg in pcategs:
                if 'Produits des Publications' in pcateg['complete_name']:
                    select_categ_ids.append(pcateg['id'])
            if select_categ_ids:
                product_obj = self.pool.get('product.product')
                product_ids = product_obj.search(cr,uid,[('categ_id','in',select_categ_ids)])
                journal_obj = self.pool.get('account.journal')
                jnl_sales_ids = journal_obj.search(cr,uid,[('type','=','sale')])
                if product_ids and jnl_sales_ids:
                    aml_ids = aml_obj.search(cr,uid,[('product_id','in',product_ids),('journal_id','in',jnl_sales_ids),('date','>=',date_from),('date','<=',date_to)])
                    if aml_ids:
                        amlines = aml_obj.read(cr,uid,aml_ids,['partner_id','credit','debit'])
                        for amline in amlines:
                            if amline['partner_id'] and amline['partner_id'][0] in partner_ids:
                                result = dResult[amline['partner_id'][0]]
                                if result.has_key('PaidPublication'):
                                    result['PaidPublication'] += ((amline['credit'] or 0.0)-(amline['debit'] or 0.0))
                                else:
                                    result['PaidPublication'] = ((amline['credit'] or 0.0)-(amline['debit'] or 0.0))
                                dResult[amline['partner_id'][0]] = result
        if 'PaidSponsoring' in shemas:
            pcateg_obj = self.pool.get('product.category')
            pcateg_ids = pcateg_obj.search(cr,uid,[])
            pcategs = pcateg_obj.read(cr,uid,pcateg_ids,['id','complete_name'])
            select_categ_ids = []
            for pcateg in pcategs:
                if 'Produits commerciaux' in pcateg['complete_name']:
                    select_categ_ids.append(pcateg['id'])
            if select_categ_ids:
                product_obj = self.pool.get('product.product')
                product_ids = product_obj.search(cr,uid,[('categ_id','in',select_categ_ids),('membership_year','=',False)])
                journal_obj = self.pool.get('account.journal')
                jnl_sales_ids = journal_obj.search(cr,uid,[('type','=','sale')])
                if product_ids and jnl_sales_ids:
                    aml_ids = aml_obj.search(cr,uid,[('product_id','in',product_ids),('journal_id','in',jnl_sales_ids),('date','>=',date_from),('date','<=',date_to)])
                    if aml_ids:
                        amlines = aml_obj.read(cr,uid,aml_ids,['partner_id','credit','debit'])
                        for amline in amlines:
                            if amline['partner_id'] and amline['partner_id'][0] in partner_ids:
                                result = dResult[amline['partner_id'][0]]
                                if result.has_key('PaidSponsoring'):
                                    result['PaidSponsoring'] += ((amline['credit'] or 0.0)-(amline['debit'] or 0.0))
                                else:
                                    result['PaidSponsoring'] = ((amline['credit'] or 0.0)-(amline['debit'] or 0.0))
                                dResult[amline['partner_id'][0]] = result
        if 'PaidMissions' in shemas:
            pcateg_obj = self.pool.get('product.category')
            pcateg_ids = pcateg_obj.search(cr,uid,[])
            pcategs = pcateg_obj.read(cr,uid,pcateg_ids,['id','complete_name'])
            select_categ_ids = []
            for pcateg in pcategs:
                if u'Produits pour les missions déléguées' in pcateg['complete_name']:
                    select_categ_ids.append(pcateg['id'])
            if select_categ_ids:
                product_obj = self.pool.get('product.product')
                product_ids = product_obj.search(cr,uid,[('categ_id','in',select_categ_ids)])
                journal_obj = self.pool.get('account.journal')
                jnl_sales_ids = journal_obj.search(cr,uid,[('type','=','sale')])
                if product_ids and jnl_sales_ids:
                    aml_ids = aml_obj.search(cr,uid,[('product_id','in',product_ids),('journal_id','in',jnl_sales_ids),('date','>=',date_from),('date','<=',date_to)])
                    if aml_ids:
                        amlines = aml_obj.read(cr,uid,aml_ids,['partner_id','credit','debit'])
                        for amline in amlines:
                            if amline['partner_id'] and amline['partner_id'][0] in partner_ids:
                                result = dResult[amline['partner_id'][0]]
                                if result.has_key('PaidMissions'):
                                    result['PaidMissions'] += ((amline['credit'] or 0.0)-(amline['debit'] or 0.0))
                                else:
                                    result['PaidMissions'] = ((amline['credit'] or 0.0)-(amline['debit'] or 0.0))
                                dResult[amline['partner_id'][0]] = result
        return dResult
    def _members_followup_excel_file(self,cr,uid,filename,date_from,date_to,today,news,olds,consos,month_name_or_dates):
        pattern_yellow = Pattern()
        pattern_yellow.pattern = 0x01 # SOLID
        pattern_yellow.set_pattern_fore_colour(pyexcel_colours['yellow'])
        pattern_green = Pattern()
        pattern_green.pattern = 0x01 # SOLID
        pattern_green.set_pattern_fore_colour(pyexcel_colours['green'])
        pattern_orange = Pattern()
        pattern_orange.pattern = 0x01 # SOLID
        pattern_orange.set_pattern_fore_colour(pyexcel_colours['orange'])
        pattern_red = Pattern()
        pattern_red.pattern = 0x01 # SOLID
        pattern_red.set_pattern_fore_colour(pyexcel_colours['red'])

        style_alert1 = XFStyle()
        style_alert1.pattern = pattern_green
        style_alert2 = XFStyle()
        style_alert2.pattern = pattern_yellow
        style_alert3 = XFStyle()
        style_alert3.pattern = pattern_orange
        style_alert4 = XFStyle()
        style_alert4.pattern = pattern_red
        styles = {}
        styles[0] = ''
        styles[1] = style_alert1
        styles[2] = style_alert2
        styles[3] = style_alert3
        styles[4] = style_alert4
        wb1 = Workbook()
        ws1 = wb1.add_sheet(_('New Members'))
        ws1bis = wb1.add_sheet(_('Lost Members'))
        if consos:
            ws2 = wb1.add_sheet(_('Usage on past 6 months'))
            ws3 = wb1.add_sheet(_('Usage 6 months -> 1 year'))
        line = 0
        ws1.write(line,0,_('Monthly followup of new members'))
        line = 2
        if '->' in month_name_or_dates:
            ws1.write(line,0,_('From-To :'))
        else:
            ws1.write(line,0,_('Month :'))
        ws1.write(line,1,month_name_or_dates)
        ws1.write(line+1,0,_('Date :'))
        ws1.write(line+1,1,today.strftime('%d/%m/%Y'))
        
        line_bis = 0
        ws1bis.write(line_bis,0,_('Monthly followup of lost members'))
        line_nis = 2
        if '->' in month_name_or_dates:
            ws1bis.write(line_bis,0,_('From-To :'))
        else:
            ws1bis.write(line_bis,0,_('Month :'))
        ws1bis.write(line_bis,1,month_name_or_dates)
        ws1bis.write(line_bis+1,0,_('Date :'))
        ws1bis.write(line_bis+1,1,today.strftime('%d/%m/%Y'))
        line_bis = 5

        line = 5
        
        ws1.write(line,0,_('New Members : '))
        line += 1
        ws1.write(line,0,_('Partner_ID'))
        ws1.write(line,1,_('Date'))
        ws1.write(line,2,_('Salesman'))
        ws1.write(line,3,_('Company'))

        partner_obj = self.pool.get('res.partner')
        new_member_ids = []
        line += 1
        for member in news:
            ws1.write(line,0,member['partner_id'])
            ws1.write(line,1,date_french(member['date_from']))
            ws1.write(line,2,member['salesman'])
            ws1.write(line,3,member['partner_name'])
            partner = partner_obj.browse(cr,uid,member['partner_id'])
            if partner.address:
                for addr in partner.address:
                    found_job = False
                    contact = ''
                    min_sequence = 99999
                    if addr.type == 'default':
                        if addr.job_ids:
                            for job in addr.job_ids:
                                if job.function_code_label and (('G' in job.function_code_label) or ('$' in job.function_code_label)) and ((job.sequence_partner or 0) < min_sequence) and job.contact_id:
                                    min_sequence = job.sequence_partner
                                    contact = job.contact_id.name.strip()
                                    if job.contact_id.first_name:
                                        contact += (' ' + job.contact_id.first_name.strip())
                                    if job.contact_id.title:
                                        contact += ('(' + job.contact_id.title + ')')
                                    if job.function_label:
                                        contact += (', ' + job.function_label.strip())
                                    found_job = True
                            if found_job:
                                ws1.write(line,4,contact)
                                break
            if member['partner_id'] not in new_member_ids:
                new_member_ids.append(member['partner_id'])
            line += 1
        line += 1

        ws1bis.write(line_bis,0,_('Lost Members : '))
        line_bis += 1
        ws1bis.write(line_bis,0,_('Partner_ID'))
        ws1bis.write(line_bis,1,_('Date'))
        ws1bis.write(line_bis,2,_('Salesman'))
        ws1bis.write(line_bis,3,_('Company'))
        line_bis += 1
        for member in olds:
            ws1bis.write(line_bis,0,member['partner_id'])
            ws1bis.write(line_bis,1,date_french(member['date_from']))
            ws1bis.write(line_bis,2,member['salesman'])
            ws1bis.write(line_bis,3,member['partner_name'])
            line_bis += 1
        if consos:
            ws2.write(0,0,_('Followup of consumption of products for new members of the last year'))
            ws2.write(1,0,_('This sheet show all new members of the past 6 months, with, for each one, his billing on various products.'))
            ws2.write(2,0,_('Members since 3 or 6  months without consumption will be shown with ALERT LEVEL 1 or 2 respectivly.'))
            ws2.write(3,0,_('New:'))
            ws2.write(3,4,_('ALERT LEVEL'))
            ws2.write(3,5,_('Free Events'))
            ws2.write(3,6,_('Events (euros)'))
            ws2.write(3,7,_('AE'))
            ws2.write(3,8,_('CNN'))
            ws2.write(3,9,_('TC'))
            ws2.write(3,10,_('MS'))
            ws2.write(3,11,_('Publications'))
            ws2.write(3,12,_('Sponsoring'))
            ws2.write(3,13,_('Missions'))
            ws2.write(3,14,_('Employee Sharing'))
            ws2.write(3,15,_('Others'))
            ws3.write(1,0,_('This sheet show all new members since 6 months to 1 year, with, for each one, his billing on various products.'))
            ws3.write(2,0,_('Members since 9 or 12 months without consumption will be shown whit ALERT LEVEL 3 or 4 respectivly.'))
            ws3.write(3,0,_('New:'))
            ws3.write(3,4,_('ALERT LEVEL'))
            ws3.write(3,5,_('Free Events'))
            ws3.write(3,6,_('Events'))
            ws3.write(3,7,_('AE'))
            ws3.write(3,8,_('CNN'))
            ws3.write(3,9,_('TC'))
            ws3.write(3,10,_('MS'))
            ws3.write(3,11,_('Publications'))
            ws3.write(3,12,_('Sponsoring'))
            ws3.write(3,13,_('Missions'))
            ws3.write(3,14,_('Employee Sharing'))
            ws3.write(3,15,_('Others'))
            line2 = 4
            line3 = 4
            for consumption in consos:
                current_months = _months_between(consumption['date_from'],_end_of_month(date_to))
                effective_conso = False
                if consumption.has_key('FreeEvents') and consumption['FreeEvents'] > 0:
                    effective_conso = True
                if consumption.has_key('PaidEvents') and consumption['PaidEvents'] > 0.01:
                    effective_conso = True
                if consumption.has_key('PaidAE') and consumption['PaidAE'] > 0.01:
                    effective_conso = True
                if consumption.has_key('PaidCNN') and consumption['PaidCNN'] > 0.01:
                    effective_conso = True
                if consumption.has_key('PaidTC') and consumption['PaidTC'] > 0.01:
                    effective_conso = True
                if consumption.has_key('PaidPublication') and consumption['PaidPublication'] > 0.01:
                    effective_conso = True
                if consumption.has_key('PaidSponsoring') and consumption['PaidSponsoring'] > 0.01:
                    effective_conso = True
                # no PaidMissions for effective consumption because captive market
                if current_months < 6:
                    ws2.write(line2,0,consumption['id'])
                    ws2.write(line2,1,consumption['date_from'])
                    ws2.write(line2,2,consumption['salesman'])
                    ws2.write(line2,3,consumption['partner_name'])
                    if current_months//3 > 0 and not effective_conso:
                        ws2.write(line2,4,current_months//3,styles[current_months//3])
                    ws2.write(line2,5,consumption.has_key('FreeEvents') and consumption['FreeEvents'] > 0 and consumption['FreeEvents'] or 0)
                    ws2.write(line2,6,consumption.has_key('PaidEvents') and consumption['PaidEvents'] or 0.0)
                    ws2.write(line2,7,consumption.has_key('PaidAE') and consumption['PaidAE'] or 0.0)
                    ws2.write(line2,8,consumption.has_key('PaidCNN') and consumption['PaidCNN'] > 0 or 0.0)
                    ws2.write(line2,9,consumption.has_key('PaidTC') and consumption['PaidTC'] > 0 or 0.0)
                    ws2.write(line2,10,consumption.has_key('PaidMS') and consumption['PaidMS'] or 0.0)
                    ws2.write(line2,11,consumption.has_key('PaidPublication') and round(consumption['PaidPublication'],2) or 0.0)
                    ws2.write(line2,12,consumption.has_key('PaidSponsoring') and round(consumption['PaidSponsoring'],2) or 0.0)
                    ws2.write(line2,13,consumption.has_key('PaidMissions') and round(consumption['PaidMissions'],2) or 0.0)
                    line2 += 1
                else:
                    ws3.write(line3,0,consumption['id'])
                    ws3.write(line3,1,consumption['date_from'])
                    ws3.write(line3,2,consumption['salesman'])
                    ws3.write(line3,3,consumption['partner_name'])
                    if current_months//3 > 0 and not effective_conso:
                        ws3.write(line3,4,current_months//3,styles[current_months//3])
                    ws3.write(line3,5,consumption.has_key('FreeEvents') and consumption['FreeEvents'] > 0 and consumption['FreeEvents'] or 0)
                    ws3.write(line3,6,consumption.has_key('PaidEvents') and consumption['PaidEvents'] > 0 and consumption['PaidEvents'] or 0.0)
                    ws3.write(line3,7,consumption.has_key('PaidAE') and consumption['PaidAE'] > 0 and consumption['PaidAE'] or 0.0)
                    ws3.write(line3,8,consumption.has_key('PaidCNN') and consumption['PaidCNN'] > 0 and consumption['PaidCNN'] or 0.0)
                    ws3.write(line3,9,consumption.has_key('PaidTC') and consumption['PaidTC'] > 0 and consumption['PaidTC'] or 0.0)
                    ws3.write(line3,10,consumption.has_key('PaidMS') and consumption['PaidMS'] > 0 and consumption['PaidMS'] or 0.0)
                    ws3.write(line3,11,consumption.has_key('PaidPublication') and round(consumption['PaidPublications'],2) or 0.0)
                    ws3.write(line3,12,consumption.has_key('PaidSponsoring') and round(consumption['PaidSponsoring'],2) or 0.0)
                    ws3.write(line3,13,consumption.has_key('PaidMissions') and round(consumption['PaidMissions'],2) or 0.0)
                    line3 += 1
        wb1.save(filename)
        return True
    _columns = {
        'site_membership': fields.integer('Additional sites for membership'),
        'desc_add_site': fields.char('Explanation of additional sites',help='Text inserted on invoice to explain the number of additional sites',size=200),
        'cci_date_start_membership': fields.date('Start of Membership',help='Date of first membership'), ### first date of membership. Doesn't change if become non member or new member again
        'cci_date_stop_membership': fields.date('End of Membership',help='Last date of membership loss'),  ### last date of becoming non-member
        'next_membership_bill_forced':fields.boolean('Force the next automatic bill',help="The next time we'll use the automatic bill or call for membreship, this partner will be taken into account"),
        'read_before_next_membership_bill':fields.boolean('Re-read before billing',help="Selected if this text must be re-read before the next automatic billing"),
    }
    _default = {
        'site_membership': lambda *a: 0,
        'next_membership_bill_forced': lambda *a: False,
        'read_before_next_membership_bill': lambda *a: False,
    }
res_partner()

class membership_range(osv.osv):
    _name = 'cci_membership.membership_range'
    _description = '''Range of employees with standard membership price'''
    _columns = {
        'year' : fields.integer('Year',required=True),
        'date_from': fields.integer('From',help='Lower number of local employees for this range',required=True), # incorrect name : date_from > employee_from
        'date_to' : fields.integer('To',help='Upper number of employees for this range',required=True), # incorrect name : date_to > employee_to for the next version implementation if possible
        'amount' : fields.float('Standard amount',digits=(15,2),help='Amount to invoice by default for this range of employees'),
    }
membership_range()

class membership_askedused(osv.osv):
    _name = 'cci_membership.membership_askedused'
    _description = '''Used asked amount of membership'''
    _columns = {
        'amount' : fields.float('Amount',digits=(15,2),readonly=True),
        'count': fields.integer('Count',readonly=True),
        'total_value': fields.float('Total Value',digits=(15,2),readonly=True),
        'type': fields.char('Type of selection',size=7),
    }
membership_askedused()

class membership_call(osv.osv):
    _name = 'cci_membership.membership_call'
    _description = '''Data for calling membership'''
    _columns = {
        'partner_id' : fields.many2one('res.partner','Partner'),
        'address_id': fields.many2one('res.partner.address','Address'),
        'job_id': fields.many2one('res.partner.job','Job'),
        'contact_id': fields.many2one('res.partner.contact','Contact'),
        'partner_name': fields.char('Sending Name',size=200),
        'street': fields.char('Street',size=250),
        'street2':fields.char('Street2',size=250),
        'zip_code':fields.char('Zip Code',size=30),
        'city':fields.char('City',size=80),
        'email_addr':fields.char('Email Address',size=200),
        'phone_addr':fields.char('Phone Address',size=50),
        'fax_addr':fields.char('Fax Address',size=50),
        'name':fields.char('Contact Name',size=120),
        'first_name':fields.char('Contact First Name',size=120),
        'courtesy':fields.char('Courtesy',size=20),
        'title':fields.char('Title',size=1000),
        'title_categs':fields.char('Categs',size=20),
        'email_contact':fields.char('Professionnal Email',size=200),
        'base_amount':fields.float('Base Amount',digits=(15,2)),
        'count_add_sites':fields.integer('Additional Sites'),
        'unit_price_site':fields.float('Unit Price per Site',digits=(15,2)),
        'desc_add_site':fields.char('Explanation Additional Sites',size=200),
        'wovat_amount':fields.float('Invoiced Price without VAT',digits=(15,2)),
        'char_wovat_amount':fields.char('WoVAT Printable price',size=15),
        'wvat_amount':fields.float('Invoiced Price with VAT',digits=(15,2)),
        'char_wvat_amount':fields.char('WVAT Printable price',size=15),
        'salesman':fields.char('Salesman',size=45),
        'salesman_phone':fields.char('Salesman Phone',size=45),
        'salesman_fax':fields.char('Salesman Fax',size=45),
        'salesman_mobile':fields.char('Salesman Mobile',size=45),
        'salesman_email':fields.char('Salesman Email',size=45),
        'salesman_linkedin':fields.char('Salesman LinkedIN',size=250),
        'salesman_login':fields.char('Salesman Login',size=40),
        'mailing_email':fields.char('Mailing EMail Address',size=250),
        'mailing_complete_name':fields.char('Mailing Complete Name',size=250),
        'mailing_courtesy':fields.char('Mailing Courtesy',size=30),
        'mailing_name':fields.char('Mailing Name',size=45),
        'mailing_firstname':fields.char('Mailing First Name',size=45),
        'mailing_job_id': fields.many2one('res.partner.job','Job'),
        'mailing_contact_id': fields.many2one('res.partner.contact','Contact'),
        'mailing_send_date': fields.datetime('Sended on'),
        'vcs':fields.char('VCS',size=30),
        'vat_num':fields.char('VAT',size=30),
    }
membership_call()

class membership_by_partner(osv.osv):
    _name = 'cci_membership.membership_by_partner'
    _description = '''Subtotal Membership by Partner'''
    _columns = {
        'partner_id' : fields.many2one('res.partner','Partner'),
        'user_id': fields.many2one('res.users','Salesman'),
        'year': fields.integer('Membership Year'),
        'invoiced': fields.float('Invoiced',digits=(15,2)),
        'paid': fields.float('Paid',digits=(15,2)),
        'canceled': fields.float('Canceled',digits=(15,2)),
        'open': fields.float('Open',digits=(15,2)),
    }
membership_by_partner()

class membership_state_change(osv.osv):
    _name = 'membership_state_change'
    _description = 'log the date of change of membership state on partners'
    _columns = {
        'partner_id': fields.many2one('res.partner','Partner',required=True),
        'date_from': fields.date('Date of change if new member'),
        'date_to' : fields.date('Date of end of membership'),
        'state': fields.char('State',size=15),
        'active': fields.boolean('Active'), # if member change to another state of membershuip => new record withh old one inactivated
                                            # if member change to non-member => indicate 'to' date and inactivate this record
        'amount_past_year': fields.float('Amount Past Year',digits=(15,2)), # amount of membership for the past year in case of a new 'from' dateor of the 'to' date if there is a to date
        'amount_current_year': fields.float('Amount Current Year',digits=(15,2)),
        'amount_next_year': fields.float('Amount Next Year',digits=(15,2)),
        'free': fields.boolean('Free Member'), # state of the free_membership at the moment of the 'from' date or of the 'to' date if there is a to date
        'refuse': fields.boolean('Refuse Membership'), # state of this field at the from 'date' or of the 'to' date if there is a to date
        'billed_partner_id': fields.many2one('res.partner','Billed Partner'), # state of this field at the from 'date' or of the 'to' date if there is a to date
    }
    _default = {
        'amount_past_year': lambda *a: 0.0,
        'amount_current_year': lambda *a: 0.0,
        'amount_next_year': lambda *a: 0.0,
        'free': lambda *a: False,
        'refuse': lambda *a: False,
        'active': lambda *a: True,
    }
membership_state_change()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

