# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import wizard
import pooler
import datetime
import base64
from pyExcelerator import *
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Check Renew">
    <field name="year_from" colspan="4" width="800"/>
    <field name="amount_from"/>
    <field name="amount_to" nolabel="1"/>
    <field name="year_to"/>
</form>
"""

fields = {
    'year_from':{'string':'Origin Year','type':'integer','help':'Origin Year, in wich the amount is checked','required':True},
    'amount_from':{'string':'Amount','type':'float','help':'For 0, indicate -0.01 and 0.01.'},
    'amount_to':{'string':'Amount','type':'float','help':'For 0, indicate -0.01 and 0.01.'},
    'year_to':{'string':'To this Year','type':'integer','help':'Final Year','required':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="check_renew.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'check_renew.xls':{'string': 'Prepared file','type': 'binary','readonly': True,},
}

class check_renew(wizard.interface):
    def _defaults(self, cr, uid, data, context):
        data['form']['year_to'] = datetime.datetime.today().year
        data['form']['year_from'] = datetime.datetime.today().year - 1
        return data['form']

    def _create_excel_file(self, cr, uid, data, context):
        # we extract the 'membership products' for the tw concerned years
        # for theses products, we extract all account.invoice.line.
        # For each invoice.line we check if invoice type is sale and if the move linked to the invoice is reconciled on 'receivable' account
        # We keep the id and the amount for each such invoice.line and the concerned years
        # with the invoice_lines ids, we extract concerned partners
        # and for each concerned partners we cumulate by years and compare the two years in excel file
        # we do this because the 'paid' state of invoices is not trustable at 100%
        if data['form']['year_from'] <= data['form']['year_to']:
            from_year = data['form']['year_from']
            to_year = data['form']['year_to']
        else:
            from_year = data['form']['year_to']
            to_year = data['form']['year_from']
        product_obj = pooler.get_pool(cr.dbname).get('product.product')
        ailine_obj = pooler.get_pool(cr.dbname).get('account.invoice.line')
        product_ids = product_obj.search(cr,uid,[('membership','=',True),('membership_year','=',from_year),'|',('active','=',True),('active','=',False)])
        if not product_ids:
            raise wizard.except_wizard('Warning','There is no membership products for year %s' % from_year)

        # extract concerned lines and partners
        concerned_years = range(from_year,to_year+1)
        wb = Workbook()
        ws1 = wb.add_sheet('Lines')
        ws1.write(0,0,u'Cette liste donne toutes les lignes de factures de vente avec un montant compris entre %.2f et %.2f pour des produits membership %i.' % (data['form']['amount_from'],data['form']['amount_to'],from_year))
        ws1.write(3,0,u'ailine_id')
        ws1.write(3,1,'partner_id')
        ws1.write(3,2,'Partner Name')
        ws1.write(3,3,'Partner Current State')
        ws1.write(3,4,'Partner Current Membership State')
        ws1.write(3,5,u'Concerned Amount')
        col = 6
        for current_year in concerned_years:
            ws1.write(3,col,'Total Amount %i' % current_year)
            col += 1
        line = 4
        ailine_ids = ailine_obj.search(cr,uid,[('product_id','in',product_ids),('price_unit','>=',data['form']['amount_from']),('price_unit','<=',data['form']['amount_to'])])
        print len(ailine_ids)
        if ailine_ids:
            ailines = ailine_obj.browse(cr,uid,ailine_ids)
            for ailine in ailines:
                if ailine.invoice_id and ailine.invoice_id.partner_id:
                    ws1.write(line,0,ailine.id,)
                    ws1.write(line,1,ailine.invoice_id.partner_id.id)
                    ws1.write(line,2,ailine.invoice_id.partner_id.name)
                    ws1.write(line,3,ailine.invoice_id.partner_id.state_id.name)
                    ws1.write(line,4,ailine.invoice_id.partner_id.membership_state)
                    ws1.write(line,5,ailine.price_unit or 0.0)
                    col = 6
                    for current_year in concerned_years:
                        product_ids = product_obj.search(cr,uid,[('membership','=',True),('membership_year','=',current_year),'|',('active','=',True),('active','=',False)])
                        partner_year_line_ids = ailine_obj.search(cr,uid,[('product_id','in',product_ids),('invoice_id.partner_id','=',ailine.invoice_id.partner_id.id)])
                        partner_year_sum = 0.0
                        if partner_year_line_ids:
                            partner_year_lines = ailine_obj.browse(cr,uid,partner_year_line_ids,)
                            for pyline in partner_year_lines:
                                partner_year_sum += ( (pyline.price_unit or 0.0) * (pyline.invoice_id.journal_id.refund_journal and -1 or 1) )
                        ws1.write(line,col,partner_year_sum)
                        col += 1
                    line += 1
        wb.save('check_renew.xls')
        result_file = open('check_renew.xls','rb').read()

        # give the result tos the user
        data['form']['msg']='Save the File with '".xls"' extension.'
        data['form']['check_renew.xls']=base64.encodestring(result_file)
        return data['form']

    states = {
        'init' : {
            'actions' : [_defaults],
            'result' : {'type' : 'form' , 'arch' : form,
                    'fields' : fields,
                    'state' : [('end', 'Cancel'), ('getfile', 'Get File')]}
        },
        'getfile': {
            'actions': [_create_excel_file],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },

    }
check_renew("check_renew")
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

