# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import wizard
import pooler
import datetime
import base64
from pyExcelerator import *
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Compare two Years of paid Members">
    <field name="year" colspan="4" width="800"/>
</form>
"""

fields = {
    'year':{'string':'Last year','type':'integer','help':'Last of the two years; the first will be calculed from the last ... difficult ...','required':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="compare2paidmembers.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'compare2paidmembers.xls':{'string': 'Prepared file','type': 'binary','readonly': True,},
}

class compare_paid2years(wizard.interface):
    def _defaults(self, cr, uid, data, context):
        data['form']['year'] = datetime.datetime.today().year - 1
        return data['form']

    def _create_excel_file(self, cr, uid, data, context):
        # manage parameters
        if not data['form']['year']:
            raise wizard.except_wizard('Warning','You must give the year of membership concerned; between 1900 and the next year')
        if data['form']['year'] and int(data['form']['year']) < 1900:
            raise wizard.except_wizard('Warning','You must give the year of membership concerned; between 1900 and the next year')
        # we extract the 'membership products' for the tw concerned years
        # for theses products, we extract all account.invoice.line.
        # For each invoice.line we check if invoice type is sale and if the move linked to the invoice is reconciled on 'receivable' account
        # We keep the id and the amount for each such invoice.line and the concerned years
        # with the invoice_lines ids, we extract concerned partners
        # and for each concerned partners we cumulate by years and compare the two years in excel file
        # we do this because the 'paid' state of invoices is not trustable at 100%
        year1 = data['form']['year']-1
        year2 = data['form']['year']
        concerned_years = [year1,year2]
        product_obj = pooler.get_pool(cr.dbname).get('product.product')
        product_ids = product_obj.search(cr,uid,[('membership','=',True),('membership_year','in',concerned_years),'|',('active','=',True),('active','=',False)])
        if not product_ids:
            raise wizard.except_wizard('Warning','There is no membership products for the concerned years : %s and %s' % (concerned_years[0],concerned_years[1]))
        products = product_obj.read(cr,uid,product_ids,['id','membership_year'])
        dProducts = {}
        for prod in products:
            dProducts[prod['id']] = prod['membership_year']
        ailine_obj = pooler.get_pool(cr.dbname).get('account.invoice.line')
        ailine_ids = ailine_obj.search(cr,uid,[('product_id','in',product_ids)])
        if not ailine_ids:
            raise wizard.except_wizard('Warning','There is no invoices for the concerned years : %s and %s' % (concerned_years[0],concerned_years[1]))
        account_obj = pooler.get_pool(cr.dbname).get('account.account')
        receivable_account_ids = account_obj.search(cr,uid,[('type','=','receivable')])
        ailines = ailine_obj.browse(cr,uid,ailine_ids)
        concerned_ailine_ids = []
        dAILines = {}
        for ailine in ailines:
            move_reconciled = False
            if ailine.invoice_id and ailine.invoice_id.state in ('open','paid') and ailine.invoice_id.journal_id and ailine.invoice_id.journal_id.type == 'sale':
                if ailine.invoice_id.move_id and ailine.invoice_id.move_id.line_id:
                    for mline in ailine.invoice_id.move_id.line_id:
                        if mline.account_id.id in receivable_account_ids and mline.reconcile_id:
                            move_reconciled = True
            if move_reconciled:
                concerned_ailine_ids.append(ailine.id)
                dAILines[ailine.id] = {'amount':ailine.price_subtotal,'year':dProducts[ailine.product_id.id]}
        mmline_obj = pooler.get_pool(cr.dbname).get('membership.membership_line')
        mmline_ids = mmline_obj.search(cr,uid,[('account_invoice_line','in',concerned_ailine_ids)])
        mmlines = mmline_obj.browse(cr,uid,mmline_ids)
        dPartners = {}
        for mmline in mmlines:
            if mmline.partner:
                if dPartners.has_key(mmline.partner.id):
                    partner = dPartners[mmline.partner.id]
                else:
                    partner = {'name':mmline.partner.name,'year1':0.0,'year2':0.0}
                if dAILines[mmline.account_invoice_line.id]['year'] == year1:
                    partner['year1'] += dAILines[mmline.account_invoice_line.id]['amount']
                else:
                    partner['year2'] += dAILines[mmline.account_invoice_line.id]['amount']
                dPartners[mmline.partner.id] = partner
        wb = Workbook()
        ws1 = wb.add_sheet('Partners')
        ws1.write(0,0,u'Cette liste est principalement basée sur les lignes de membership mais en vérifiant qu\'elles sont liées à des factures acquittées (par un paiement ou un NC).')
        ws1.write(1,0,u'Elle peut donc comporter quelques différences avec des extractions basées sur les factures de cotisations.')
        ws1.write(3,0,u'partner_id')
        ws1.write(3,1,u'name')
        ws1.write(3,2,u'Paid %s' % str(year1))
        ws1.write(3,3,u'Paid %s' % str(year2))
        ws1.write(3,4,u'Status')
        ws1.write(3,5,u'Amount Diff')
        line = 4
        count_lost = 0
        count_new = 0
        count_less = 0
        count_more = 0
        count_year1 = 0
        count_year2 = 0
        count_year1_faithfull = 0
        amount_year1 = 0.0
        amount_year2 = 0.0
        for (partner_id,partner) in dPartners.items():
            ws1.write(line,0,partner_id)
            ws1.write(line,1,partner['name'])
            ws1.write(line,2,partner['year1'])
            ws1.write(line,3,partner['year2'])
            if partner['year1'] >= 0.001 and partner['year2'] <= 0.001:
                ws1.write(line,4,'lost')
                count_lost += 1
            elif partner['year1'] <= 0.001 and partner['year2'] >= 0.001:
                ws1.write(line,4,'new')
                count_new += 1
            elif partner['year1'] >= 0.001 and partner['year2'] >= 0.001 and partner['year1'] > partner['year2']:
                ws1.write(line,4,'less money')
                count_less += 1
            elif partner['year1'] >= 0.001 and partner['year2'] >= 0.001 and partner['year1'] < partner['year2']:
                ws1.write(line,4,'more money')
                count_more += 1
            if partner['year1'] >= 0.001:
                count_year1 += 1
            if partner['year2'] >= 0.001:
                count_year2 += 1
            amount_year1 += partner['year1']
            amount_year2 += partner['year2']
            if partner['year1'] >= 0.001 and partner['year2'] >= 0.001:
                count_year1_faithfull += 1
            ws1.write(line,5,partner['year2']-partner['year1'])
            line += 1
        # record the global data on worsheet 2
        ws2 = wb.add_sheet('Global Data')
        ws2.write(0,0,'Global Results')
        ws2.write(1,0,'Date')
        ws2.write(1,1,datetime.datetime.today().strftime('%d/%m/%Y'))
        ws2.write(2,0,'Lost')
        ws2.write(2,1,count_lost)
        ws2.write(3,0,'New')
        ws2.write(3,1,count_new)
        ws2.write(4,0,'Less money')
        ws2.write(4,1,count_less)
        ws2.write(5,0,'More money')
        ws2.write(5,1,count_more)
        ws2.write(6,0,'Count Year1')
        ws2.write(6,1,count_year1)
        ws2.write(7,0,'Count Year2')
        ws2.write(7,1,count_year2)
        ws2.write(8,0,'Year1 faithfull')
        ws2.write(8,1,count_year1_faithfull)
        ws2.write(9,0,'Fidelity Rate')
        if count_year1:
            ws2.write(9,1,int(count_year1_faithfull*100/count_year1))
        else:
            ws2.write(9,1,0)
        ws2.write(10,0,'Amount Year 1')
        ws2.write(10,1,amount_year1)
        ws2.write(11,0,'Amount Year 2')
        ws2.write(11,1,amount_year2)
        wb.save('compare2paidmembers.xls')
        result_file = open('compare2paidmembers.xls','rb').read()

        # give the result tos the user
        data['form']['msg']='Save the File with '".xls"' extension.'
        data['form']['compare2paidmembers.xls']=base64.encodestring(result_file)
        return data['form']

    states = {
        'init' : {
            'actions' : [_defaults],
            'result' : {'type' : 'form' , 'arch' : form,
                    'fields' : fields,
                    'state' : [('end', 'Cancel'), ('getfile', 'Get File')]}
        },
        'getfile': {
            'actions': [_create_excel_file],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },

    }
compare_paid2years("compare_paid2years")
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

