# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import wizard
import pooler
import datetime
import base64
from pyExcelerator import *
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Count Paid Members between two dates">
    <field name="date_from" colspan="4"/>
    <field name="date_to" colspan="4"/>
</form>
"""

fields = {
    'date_from':{'string':'From','type':'date','required':True},
    'date_to':{'string':'To','type':'date','required':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="count_paid_partners_between_2_dates.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'count_paid_partners_between_2_dates.xls':{'string': 'Prepared file','type': 'binary','readonly': True,},
}

class count_paid_members(wizard.interface):
    def _create_excel_file(self, cr, uid, data, context):
        # manage parameters
        date_from = data['form']['date_from']
        date_to = data['form']['date_to']
        if date_from > date_to:
            (date_to,date_from) = (date_from,date_to)
        french_date_from = date_from[8:10]+'/'+date_from[5:7]+'/'+date_from[0:4]
        french_date_to = date_to[8:10]+'/'+date_to[5:7]+'/'+date_to[0:4]
        # A paid member is an invoiced member between theses two dates with total invoiced amount > 0 (to take care of invoice canceled by credit nota into account; but also of invoice of zero copunting as paid member)
        journal_obj = pooler.get_pool(cr.dbname).get('account.journal')
        partner_obj = pooler.get_pool(cr.dbname).get('res.partner')
        credit_ids = journal_obj.search(cr,uid,[('type','=','sale'),('refund_journal','=',True)])
        product_obj = pooler.get_pool(cr.dbname).get('product.product')
        product_ids = product_obj.search(cr,uid,[('membership','=',True),'|',('active','=',True),('active','=',False)])
        ailine_obj = pooler.get_pool(cr.dbname).get('account.invoice.line')
        ailine_ids = ailine_obj.search(cr,uid,[('product_id','in',product_ids),('invoice_id.date_invoice','>=',date_from),('invoice_id.date_invoice','<=',date_to)])
        ailines = ailine_obj.browse(cr,uid,ailine_ids)
        dPartners = {}
        for ailine in ailines:
            if ailine.invoice_id and ailine.invoice_id.partner_id:
                if dPartners.has_key(ailine.invoice_id.partner_id.id):
                    partner_data = dPartners[ailine.invoice_id.partner_id.id]
                else:
                    state = partner_obj._get_membership_state_at_date(cr,uid,ailine.invoice_id.partner_id.id,date_to)
                    partner_data = {'name':ailine.invoice_id.partner_id.name,'amount':0.0,'count_ailines':0,'count_refund':0,'member_end_date':state,'id':ailine.invoice_id.partner_id.id}
                partner_data['count_ailines'] += 1
                mult = 1
                if ailine.invoice_id.journal_id.id in credit_ids:
                    mult = -1
                    partner_data['count_refund'] += 1
                partner_data['amount'] += (ailine.price_subtotal * mult)
                dPartners[ailine.invoice_id.partner_id.id] = partner_data
        wb = Workbook()
        ws1 = wb.add_sheet(_('Results'))
        ws1.write(0,0,u'Cette liste donne les membres payants entre le \'%s\' et le \'%s\'.' % (french_date_from,french_date_to))
        ws1.write(1,0,u'Un partenaire est considéré comme membre payant entre ces deux dates s\'il a été facturé pour un montant global positif entre ces deux dates,')
        ws1.write(2,0,u'avec l\'exception qu\'un partenaire n\'ayant reçu qu\'une seule facture (et pas de note de crédit) sur cette période, avec un montant de 0 est considéré comme membre payant AUSSI entre ces deux dates.')
        ws1.write(3,0,u'Si un partenaire n\'est plus membre au \'%s\', il est néanmoins comptabilisé positivement dans ce nombre final avec une mention spéciale dans les détails.' % french_date_to)
        ws1.write(4,0,u'Date de calcul : ' + datetime.date.today().strftime('%d/%m/%Y'))
        ws1.write(6,0,u'Nombre :')
        ws1.write(7,0,u'Spéciaux :')
        ws2 = wb.add_sheet(_('Partners'))
        ws2.write(3,0,u'partner_id')
        ws2.write(3,1,u'name')
        ws2.write(3,2,u'Amount invoiced between thses two dates')
        ws2.write(3,3,u'Number of documents')
        ws2.write(3,4,u'Number of refunds')
        ws2.write(3,5,u'Membership state at %s' % french_date_to)
        ws2.write(3,6,u'Special if not membre at %s' % french_date_to)
        line = 4
        specials = 0
        for (partner_id,partner) in dPartners.items():
            if partner['amount'] > 0.001 or ((partner['amount'] > -0.001) and (partner['count_ailines'] == 1) and (partner['count_refund'] == 0)):
                ws2.write(line,0,partner_id)
                ws2.write(line,1,partner['name'])
                ws2.write(line,2,partner['amount'])
                ws2.write(line,3,partner['count_ailines'])
                ws2.write(line,4,partner['count_refund'])
                ws2.write(line,5,partner['member_end_date'] or 'Unknown')
                if partner['member_end_date'] not in ['free','paid','invoiced']:
                    ws2.write(line,6,'SPECIAL')
                    specials += 1
                line += 1
        ws1.write(6,1,line - 4)
        ws1.write(7,1,specials)
        wb.save('count_paid_partners_between_2_dates.xls')
        result_file = open('count_paid_partners_between_2_dates.xls','rb').read()

        # give the result tos the user
        data['form']['msg'] = 'Save the File with '".xls"' extension.\n\nCount : %i (with %i specials).' % (line - 4,specials)
        data['form']['count_paid_partners_between_2_dates.xls'] = base64.encodestring(result_file)
        return data['form']

    states = {
        'init' : {
            'actions' : [],
            'result' : {'type' : 'form' , 'arch' : form,
                    'fields' : fields,
                    'state' : [('end', 'Cancel'), ('getfile', 'Get File')]}
        },
        'getfile': {
            'actions': [_create_excel_file],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },

    }
count_paid_members("count_paid_members")
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

