# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import datetime
from osv import fields
import wizard
import pooler
import base64
from pyExcelerator import *
from tools.translate import _

wizard_arch= """<?xml version="1.0"?>
<form string="Choose year">
    <field name="year" colspan="4"/>
    <field name="target"/>
</form>"""

wizard_fields = {
    'year': {'string': 'Membership year','type': 'integer', 'required':True},
    'target': {'string': 'Target', 'type':'selection', 'selection':[('all','All - Members or not'),('members','Only Members')], 'required':True, 'default':'all'},
}

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="benefice_members.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'benefice_members.xls':{'string': 'Prepared file','type': 'binary','readonly': True,},
}


class extract_consommation_for_members(wizard.interface):
    def _extract(self, cr, uid, data, context):
        # manage parameters
        current_year = int(datetime.date.today().strftime('%Y'))
        year = data['form']['year']
        if year < 1980 or year > current_year:
            year = current_year
        target = data['form']['target']
        
        wb1 = Workbook()
        ws1 = wb1.add_sheet('Gains as member')
        
        line = 1
        ws1.write(line,0,'Year : ')
        ws1.write(line,1,year)
        line += 1
        ws1.write(line,0,'Target')
        ws1.write(line,1,target)
        line += 2
        
        # get the products linked to the membership (and the possible year)
        prod_obj = pooler.get_pool(cr.dbname).get('product.product')

        # get the list of sales journals and refund_sales journals
        jnl_obj = pooler.get_pool(cr.dbname).get('account.journal')
        # get the list of 'customer accounts'
        type_ids = pooler.get_pool(cr.dbname).get('account.account.type').search(cr,uid,[('code','=','tiers -rec')])
        cust_account_ids = pooler.get_pool(cr.dbname).get('account.account').search(cr,uid,[('user_type','in',type_ids)])
        NCV_ids = jnl_obj.search(cr,uid,[('type','=','sale'),('refund_journal','=',True)])
        VEN_ids = jnl_obj.search(cr,uid,[('type','=','sale'),('refund_journal','=',False)])
        membership_prod_ids = prod_obj.search(cr,uid,[('membership','=',True),('membership_year','=',year)])
        next_year_prod_ids = prod_obj.search(cr,uid,[('membership','=',True),('membership_year','=',year+1)])
        prod_ids = membership_prod_ids + next_year_prod_ids
        dYearPartners = {}
        if prod_ids:
            # extract all lines concerned by these products
            iline_obj = pooler.get_pool(cr.dbname).get('account.move.line')
            iline_ids = iline_obj.search(cr,uid,[('product_id','in',prod_ids)])
            ilines = iline_obj.browse(cr,uid,iline_ids)
            for iline in ilines:
                key = (iline.product_id.membership_year or 1900,iline.partner_id.id or 0)
                if not dYearPartners.has_key( key ):
                    dYearPartners[ key ] = 0.0
                value = dYearPartners[ key ]
                dYearPartners[ key ] += ( iline.credit - iline.debit )
        else:
            ws1.write(line,0,'No membership products found for year %s' % str(year))
            line += 2

        partner_ids = []
        partner_obj = pooler.get_pool(cr.dbname).get('res.partner')
        period_obj = pooler.get_pool(cr.dbname).get('account.period')
        period_ids = period_obj.search(cr,uid,[('date_start','>=',str(year)+'-01-01'),('date_start','<=',str(year)+'-12-31'),('special','=',False)])
        dPartners = {}
        if period_ids:
            if target == 'all': # all customers
                iline_ids = iline_obj.search(cr,uid,[('period_id','in',period_ids),('journal_id','in',NCV_ids+VEN_ids),('account_id','not in',cust_account_ids)])
                if iline_ids:
                    ilines = iline_obj.browse(cr,uid,iline_ids)
                    for iline in ilines:
                        if iline.partner_id and iline.product_id:
                            if dPartners.has_key(iline.partner_id.id):
                                vals = dPartners[iline.partner_id.id]
                                vals['amount'] += (iline.credit or 0.0) - (iline.debit or 0.0)
                                if iline.product_id.id in (membership_prod_ids+next_year_prod_ids) or (iline.product_id.list_price <= 0.001):
                                    vals['product_list'] += (iline.credit or 0.0) - (iline.debit or 0.0)
                                    vals['product_member'] += (iline.credit or 0.0) - (iline.debit or 0.0)
                                else:
                                    vals['product_list'] += ((iline.product_id.list_price or 0.0) * (iline.quantity or 1))
                                    vals['product_member'] += ((iline.product_id.member_price or iline.product_id.list_price or 0.0) * (iline.quantity or 1))
                            else:
                                if iline.product_id.id in (membership_prod_ids+next_year_prod_ids) or (iline.product_id.list_price <= 0.001):
                                    vals = {'partner_id':iline.partner_id.id,
                                            'name':iline.partner_id.name,
                                            'membership_state':iline.partner_id.membership_state or 'unknown',
                                            'asked':iline.partner_id.membership_amount or 0.0,
                                            'amount': (iline.credit or 0.0) - (iline.debit or 0.0),
                                            'product_list': ((iline.product_id.list_price or 0.0) * (iline.quantity or 1)),
                                            'product_member': ((iline.product_id.member_price or iline.product_id.list_price or 0.0) * (iline.quantity or 1)),
                                           }
                                else:
                                    vals = {'partner_id':iline.partner_id.id,
                                            'name':iline.partner_id.name,
                                            'membership_state':iline.partner_id.membership_state or 'unknown',
                                            'asked':iline.partner_id.membership_amount or 0.0,
                                            'amount': (iline.credit or 0.0) - (iline.debit or 0.0),
                                            'product_list': ((iline.product_id.list_price or 0.0) * (iline.quantity or 1)),
                                            'product_member': ((iline.product_id.member_price or iline.product_id.list_price or 0.0) * (iline.quantity or 1)),
                                           }
                            dPartners[iline.partner_id.id] = vals
            else: ## target == 'members'
                member_ids = partner_obj.search(cr,uid,[('membership_state','in',['paid','invoiced','free'])])
                iline_ids = iline_obj.search(cr,uid,[('period_id','in',period_ids),('journal_id','in',NCV_ids+VEN_ids),('partner_id','in',member_ids),('account_id','not in',cust_account_ids)])
                if iline_ids:
                    dPartners = {}
                    ilines = iline_obj.browse(cr,uid,iline_ids)
                    for iline in ilines:
                        if iline.product_id:
                            if dPartners.has_key(iline.partner_id.id):
                                vals = dPartners[iline.partner_id.id]
                                vals['amount'] += (iline.credit or 0.0) - (iline.debit or 0.0)
                                if iline.product_id.id in (membership_prod_ids+next_year_prod_ids) or (iline.product_id.list_price <= 0.001):
                                    vals['product_list'] += (iline.credit or 0.0) - (iline.debit or 0.0)
                                    vals['product_member'] += (iline.credit or 0.0) - (iline.debit or 0.0)
                                else:
                                    vals['product_list'] += ((iline.product_id.list_price or 0.0) * (iline.quantity or 1))
                                    vals['product_member'] += ((iline.product_id.member_price or iline.product_id.list_price or 0.0) * (iline.quantity or 1))
                            else:
                                if iline.product_id.id in (membership_prod_ids+next_year_prod_ids) or (iline.product_id.list_price <= 0.001):
                                    vals = {'partner_id':iline.partner_id.id,
                                            'name':iline.partner_id.name,
                                            'membership_state':iline.partner_id.membership_state or 'unknown',
                                            'asked':iline.partner_id.membership_amount or 0.0,
                                            'amount': (iline.credit or 0.0) - (iline.debit or 0.0),
                                            'product_list': (iline.credit or 0.0) - (iline.debit or 0.0),
                                            'product_member': (iline.credit or 0.0) - (iline.debit or 0.0),
                                           }
                                else:
                                    vals = {'partner_id':iline.partner_id.id,
                                            'name':iline.partner_id.name,
                                            'membership_state':iline.partner_id.membership_state or 'unknown',
                                            'asked':iline.partner_id.membership_amount or 0.0,
                                            'amount': (iline.credit or 0.0) - (iline.debit or 0.0),
                                            'product_list': ((iline.product_id.list_price or 0.0) * (iline.quantity or 1)),
                                            'product_member': ((iline.product_id.member_price or iline.product_id.list_price or 0.0) * (iline.quantity or 1)),
                                           }
                            dPartners[iline.partner_id.id] = vals
            ws1.write(line,0,u'partner_id')
            ws1.write(line,1,u'partner name')
            ws1.write(line,2,u'membership_state')
            ws1.write(line,3,u'asked membership')
            ws1.write(line,4,u'membership in %s' % str(year))
            ws1.write(line,5,u'membership in %s' % str(year+1))
            ws1.write(line,6,u'CA in %s' % str(year))
            ws1.write(line,7,u'non-member price for products')
            ws1.write(line,8,u'member price for products')
            ws1.write(line,9,u'gain if member')
            ws1.write(line,10,u'saldo (asked - gain)')
            line += 1
            for (partner_id,vals) in dPartners.items():
                membership_key1 = (year,partner_id)
                membership_key2 = (year+1,partner_id)
                ws1.write(line,0,partner_id)
                ws1.write(line,1,vals['name'])
                ws1.write(line,2,vals['membership_state'])
                ws1.write(line,3,round(vals['asked'],2))
                ws1.write(line,4,round(dYearPartners.has_key(membership_key1) and dYearPartners[membership_key1] or 0.0,2))
                ws1.write(line,5,round(dYearPartners.has_key(membership_key2) and dYearPartners[membership_key2] or 0.0,2))
                ws1.write(line,6,round(vals['amount'],2))
                ws1.write(line,7,round(vals['product_list'],2))
                ws1.write(line,8,round(vals['product_member'],2))
                ws1.write(line,9,round(vals['product_list']-vals['product_member'],2))
                ws1.write(line,10,round(vals['product_list']-vals['product_member']-vals['asked'],2))
                line += 1
            wb1.save('benefice_members.xls')
            result_file = open('benefice_members.xls','rb').read()

            # give the result to the user
            data['form']['msg']='Save the File with '".xls"' extension.'
            data['form']['benefice_members.xls']=base64.encodestring(result_file)
        else:
            raise wizard.except_wizard('Warning',"There is no periods in the year '%s' !" % str(year) )
        return data['form']
    states = {
        'init' : {
            'actions' : [],
            'result' : {'type':'form','arch':wizard_arch,'fields':wizard_fields,'state':[('end', 'Cancel'),('ok', 'Confirm') ]}
        },
        'ok' : {
            'actions' : [_extract],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }
extract_consommation_for_members("extract_consommation_for_members")
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
