# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import time

from osv import fields
import wizard
import pooler
from tools.translate import _

def _extract_totals_by_partner(self, cr, uid, data, context):
    # manage parameters
    year = data['form']['year']
    # get the products linked to the membership (and the possible year)
    prod_obj = pooler.get_pool(cr.dbname).get('product.product')
    if year:
        prod_ids = prod_obj.search(cr,uid,[('membership','=',True),('membership_year','=',year)])
    else:
        prod_ids = prod_obj.search(cr,uid,[('membership','=',True)])
    if prod_ids:
        # get the list of sales journals and refund_sales journals
        jnl_obj = pooler.get_pool(cr.dbname).get('account.journal')
        NCV_ids = jnl_obj.search(cr,uid,[('type','=','sale'),('refund_journal','=',True)])
        VEN_ids = jnl_obj.search(cr,uid,[('type','=','sale'),('refund_journal','=',False)])
        # get the list of 'customer accounts'
        type_ids = pooler.get_pool(cr.dbname).get('account.account.type').search(cr,uid,[('code','=','tiers -rec')])
        account_ids = pooler.get_pool(cr.dbname).get('account.account').search(cr,uid,[('user_type','in',type_ids)])
        # extract all lines concerned by these products
        iline_obj = pooler.get_pool(cr.dbname).get('account.move.line')
        iline_ids = iline_obj.search(cr,uid,[('product_id','in',prod_ids)])
        ilines = iline_obj.browse(cr,uid,iline_ids)
        dYearPartners = {}
        for iline in ilines:
            key = (iline.product_id.membership_year or 1900,iline.partner_id.id or 0)
            if not dYearPartners.has_key( key ):
                newrecord = {'partner_id':iline.partner_id.id or 0, 'user_id':iline.partner_id.user_id.id or 0, 'year':iline.product_id.membership_year or 1900, 'invoiced':0.0,'paid':0.0,'canceled':0.0,'open':0.0}
                dYearPartners[ key ] = newrecord
            if iline.journal_id.id in VEN_ids:
                dYearPartners[ key ]['invoiced'] += ( iline.credit - iline.debit )
                # for the sales lines, we must check if paid by another line than sales_refund
                lPaid = False
                lRefund = False
                if iline.move_id and iline.move_id.line_id:
                    for assoc_line in iline.move_id.line_id:
                        if assoc_line.account_id.id in account_ids and assoc_line.reconcile_id.id:
                            # check the journal of the lines linked to this one : if there is a refund, consider the invoice as refunded
                            for other_line in assoc_line.reconcile_id.line_id:
                                lPaid = True
                                if other_line.id <> iline.id:
                                    if other_line.journal_id.id in NCV_ids:
                                        lRefund = True
                if lPaid:
                    if not lRefund:
                        dYearPartners[ key ]['paid'] += ( iline.credit - iline.debit )
                    else:
                        dYearPartners[ key ]['canceled'] += ( iline.credit - iline.debit )
                else:
                    dYearPartners[ key ]['open'] += ( iline.credit - iline.debit )
            elif iline.journal_id.id in NCV_ids:
                #dYearPartners[ key ]['canceled'] += ( iline.debit - iline.credit )
                pass
            else:
                print iline.journal_id.id
        # record in the database to show it to the user
        record_obj = pooler.get_pool(cr.dbname).get('cci_membership.membership_by_partner')
        list_ids = []
        for data in dYearPartners.values():
            list_ids.append( record_obj.create(cr,uid,{'partner_id':data['partner_id'],'user_id':data['user_id'],'year':data['year'],'invoiced':data['invoiced'],'paid':data['paid'],'canceled':data['canceled'],'open':data['open']}) )
    else:
        raise wizard.except_wizard('Warning',"There is no membership products associated with the year '%s' !" % str(year) )
        list_ids = []
    value = {
            'domain': [('id', 'in', list_ids),],
            'name': _('Total membership(s) by Partner by year'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'cci_membership.membership_by_partner',
            'type': 'ir.actions.act_window',
            'limit': len(list_ids),
        }
    return value

wizard_arch= """<?xml version="1.0"?>
<form string="Choose membership year">
    <field name="year" colspan="4"/>
</form>"""

wizard_fields = {
    'year': {'string': 'Membership year','type': 'integer'},
}

class extract_membership_by_partner(wizard.interface):
    states = {
        'init' : {
            'actions' : [],
            'result' : {'type':'form','arch':wizard_arch,'fields':wizard_fields,'state':[('end', 'Cancel'),('ok', 'Confirm') ]}
        },
        'ok' : {
            'actions' : [],
            'result' : {'type' : 'action','action':  _extract_totals_by_partner,'state' : 'end'},
        },
    }
extract_membership_by_partner("extract_membership_by_partner")

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
