# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import wizard
import pooler
import datetime
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Extract Strange Amounts" width="300">
    <field name="year" width="300"/>
    <newline/>
    <field name="only_members"/>
    <newline/>
    <field name="only_active"/>
    <newline/>
    <label string="Extraction of partners with an asked membership different of any of the amount given by ranges." colspan="2" width="300"/>
</form>
"""

fields = {
    'year':{'string':'Membership Year','type':'integer','required':True},
    'only_members':{'string':'Only for current members','type':'boolean','default':True,'help':'If not checked, the list will give ALL members with strange membership amount...'},
    'only_active':{'string':'Only for active partners','type':'boolean','default':True,'help':'If not checked, the list will give ALL partners with strange membership amount even if inactive...'},
}

class extract_strange_amounts(wizard.interface):

    def _defaults(self, cr, uid, data, context):
        data['form']['year'] = datetime.datetime.today().year
        return data['form']

    def _open_window_selected_partners(self, cr, uid, data, context):
        if not data['form']['year']:
            raise wizard.except_wizard('Warning','You must give the year of membership concerned; between 1900 and the next year')
        if data['form']['year'] and int(data['form']['year']) < 1900:
            raise wizard.except_wizard('Warning','You must give the year of membership concerned; between 1900 and the next year')
        # get the standard ranges
        mranges_obj = pooler.get_pool(cr.dbname).get('cci_membership.membership_range')
        mrange_ids = mranges_obj.search(cr,uid,[('year','=',data['form']['year'])])
        if not mrange_ids:
            raise wizard.except_wizard('Warning','The given membership year has no membership ranges defined')
        mranges = mranges_obj.read(cr,uid,mrange_ids,['amount'])
        amounts = []
        for mrange in mranges:
            amounts.append(mrange['amount'])
            
        # for each ranges, we check the amount of partners inside this range and 
        # collect partners with a different amount
        partner_ids = []
        for mrange in mranges:
            where_clause = "active AND membership_amount <> 0.0 and membership_amount not in (%s)" % ','.join([str(x) for x in amounts])
            if data['form']['only_members']:
                where_clause += " AND membership_state in ('free','invoiced','paid')"
            if data['form']['only_active']:
                where_clause += " AND state_id = 1"
            selection = "SELECT id FROM res_partner WHERE %s" % where_clause
            cr.execute( selection )
            result_lines = cr.fetchall()
            current_ids = [x[0] for x in result_lines]
            partner_ids.extend(current_ids)
        result = {
            'domain': [('id', 'in', partner_ids)],
            'name': _('Strange Membership Amounts'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'res.partner',
            'context': { },
            'type': 'ir.actions.act_window'
        }
        return result

    states = {
        'init' : {
            'actions' : [_defaults],
            'result' : {'type' : 'form' , 'arch' : form,
                    'fields' : fields,
                    'state' : [('end', 'Cancel'), ('go', 'Go')]}
        },
        'go': {
            'actions': [],
            'result': {'type': 'action', 'action':_open_window_selected_partners, 'state':'end'}
        }

    }
extract_strange_amounts("extract_strange_amounts")
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

