# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
# 2012-02-16 : some 'job.' -> 'sel_job.'
import wizard
import pooler
import datetime
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Calls for Membership">
    <field name="year" colspan="4" width="800"/>
    <field name="also_invoiced"/>
    <field name="also_forced"/>
    <field name="price_site" colspan="4"/>
    <newline/>
    <separator string="Address and Contact For the paper"/>
    <field name="address_type" colspan="4"/>
    <field name="job_categs" colspan="2"/>
    <label string="If you want to choose the address of the Membership Contact, you MUST give his letter as first category" colspan="2"/>
    <newline/>
    <separator string="Address and Contact For the eMailing"/>
    <field name="mailing_address_type" colspan="4"/>
    <field name="mailing_job_categs" colspan="2"/>
    <label string="If you want to choose the address of the Membership Contact, you MUST give his letter as first category. Beware, the first with an email address will be selected." colspan="2"/>
</form>
"""

fields = {
    'year':{'string':'New year','type':'integer','help':'New year of the called membership...','required':True},
    'address_type':{'string':'Type of address','type':'selection','selection':[('default','Default address'),('invoice','Invoice address otherwise defaut address'),('contact_default','Membership Contact else default address'),('contact_invoice','Membership Contact else invoice address otherwise defaut address')],'default':'invoice'},
    'also_invoiced':{'string':'Also the invoiced members, else only paid','type':'boolean','default':False},
    'also_forced':{'string':'Also the forced partners','type':'boolean','default':True},
    'job_categs':{'string':'Categories of searched contacts','type':'char','size':20,'required':True},
    'price_site':{'string':'W/o VAT Price per Site','type':'float','digits':(15,2),'required':True},
    'mailing_address_type':{'string':'Type of address','type':'selection','selection':[('default','Default address'),('invoice','Invoice address otherwise defaut address'),('contact_default','Membership Contact else default address'),('contact_invoice','Membership Contact else invoice address otherwise defaut address')],'default':'default'},
    'mailing_job_categs':{'string':'Categories of searched contacts','type':'char','size':20,'required':True},
}

def float_to_char(value):
    result = '%.2f' % value
    (before,after) = result.split('.')
    if len(before) > 6:
        before_with_sep = before[0:len(before)-6]+'.'+before[len(before)-6:len(before)-3]+'.'+before[-3:]
    elif len(before) > 3:
        before_with_sep = before[0:len(before)-3]+'.'+before[-3:]
    else:
        before_with_sep = before
    if after != '00':
        result = before_with_sep + ',' + after
    else:
        result = before_with_sep
    return result
    
class membership_calls(wizard.interface):

    def get_phone_country_prefix(self,cr,uid):
        result = []
        obj_country = pooler.get_pool(cr.dbname).get('cci.country')
        country_ids = obj_country.search(cr,uid,[('phoneprefix','!=',False),('phoneprefix','!=',0)])
        if country_ids:
            countries = obj_country.read(cr,uid,country_ids,['phoneprefix'])
            result = [str(x['phoneprefix']) for x in countries]
        return result
    def convert_phone(self,string,PHONE_COUNTRY_PREFIX):
        def only_digits(string):
            cleaned = ''
            for carac in string:
                if carac in '0123456789':
                    cleaned += carac
            return cleaned
        result = ''
        string = only_digits(string)
        if len(string) > 0:
            if len(string) == 9:
                if string[0:2] in ['02','03','04','09']:
                    result = string[0:2] + "/" + string[2:5] + "." + string[5:7] + "." + string[7:]
                else:
                    result = string[0:3] + "/" + string[3:5] + "." + string[5:7] + "." + string[7:]
            elif len(string) == 10:
                result = string[0:4] + "/" + string[4:6] + "." + string[6:8] + "." + string[8:]
            else:
                # international number
                #print string
                if string[0:2] == '00':
                    # search after a country with this prefix
                    prefix = string[2:4]
                    if prefix not in PHONE_COUNTRY_PREFIX:
                        prefix = string[2:5]
                        if prefix not in PHONE_COUNTRY_PREFIX:
                            prefix = string[2:6]
                            if prefix not in PHONE_COUNTRY_PREFIX:
                                prefix = ''
                    if prefix:
                        result = '+' + string[2:2+len(prefix)] + ' ' + string[2+len(prefix):4+len(prefix)]
                        rest = string[4+len(prefix):]
                        while len(rest) > 3:
                            result += '.' + rest[0:2]
                            rest = rest[2:]
                        result += '.' + rest
                    else:
                        result = 'International:'+string
        return result
    def _defaults(self, cr, uid, data, context):
        data['form']['year'] = datetime.datetime.today().year + 1
        return data['form']

    def _open_window_results(self, cr, uid, data, context):
        # manage parameters
        if not data['form']['year']:
            raise wizard.except_wizard('Warning','You must give the year of membership concerned; between 1900 and the next year')
        if data['form']['year'] and int(data['form']['year']) < 1900:
            raise wizard.except_wizard('Warning','You must give the year of membership concerned; between 1900 and the next year')
        if data['form']['also_invoiced']:
            mstates = ['invoiced','paid']
        else:            
            mstates = ['paid']
        unit_price = data['form']['price_site']
        address_type = data['form']['address_type']
        if address_type[0:8] == 'contact_':
            if len(data['form']['job_categs']) > 0:
                membership_categ = data['form']['job_categs'][0]
            else:
                address_type = address_type[8:] # we forget to search on membership categ because this is not given as parameters
        if len(data['form']['job_categs']) == 0:
            categs = 'G'
            len_categs = 1
        else:
            categs = data['form']['job_categs']
            len_categs = len(categs)
        mailing_address_type = data['form']['mailing_address_type']
        if mailing_address_type[0:8] == 'contact_':
            if len(data['form']['mailing_job_categs']) > 0:
                mailing_membership_categ = data['form']['mailing_job_categs'][0]
            else:
                mailing_address_type = mailing_address_type[8:] # we forget to search on membership categ because this is not given as parameters
        if len(data['form']['mailing_job_categs']) == 0:
            mailing_categs = 'G'
            len_mailing_categs = 1
        else:
            mailing_categs = data['form']['mailing_job_categs']
            len_mailing_categs = len(mailing_categs)
        # get the usual VAT on membership
        tax_rate = 0.0
        mprod_obj = pooler.get_pool(cr.dbname).get('product.product')
        mprod_ids = mprod_obj.search(cr,uid,[('membership','=',True),('membership_year','=',int(data['form']['year']))])
        if mprod_ids:
            mprods = mprod_obj.browse(cr,uid,mprod_ids)
            for mproduct in mprods:
                if mproduct.product_tmpl_id.taxes_id:
                    tax_rate = mproduct.product_tmpl_id.taxes_id[0].amount
                    if tax_rate > 0.0:
                        break;
        # delete all old values
        selection = "DELETE FROM cci_membership_membership_call"
        cr.execute( selection )
        selection = '' # unusefull just for the case of
        # prepare the inserting of new data
        call_obj = pooler.get_pool(cr.dbname).get('cci_membership.membership_call')
        PREFIXES = self.get_phone_country_prefix(cr,uid)
        # get the data about all the salesman from cci_salesman.cci.contact
        salesman_obj = pooler.get_pool(cr.dbname).get('res.users')
        salesman_ids = salesman_obj.search(cr,uid,[])
        salesmen = salesman_obj.browse(cr,uid,salesman_ids)
        dSalesman = {}
        for sman in salesmen:
            dSalesman[sman.id] = sman
        # get this list of partners already invoiced for the next year of membership
        # event the inactive products
        mprod_ids = mprod_obj.search(cr,uid,[('membership','=',True),('membership_year','=',int(data['form']['year'])),'|',('active','=',True),('active','=',False)])
        excluded_partner_ids = []
        if len(mprod_ids) > 0:
            selection = "SELECT distinct(partner_id) FROM account_move_line where product_id in (%s)" % ','.join([str(x) for x in mprod_ids])
            cr.execute( selection )
            partners = cr.fetchall()
            excluded_partner_ids = [x[0] for x in partners]
        # get all partners concerned
        partner_obj = pooler.get_pool(cr.dbname).get('res.partner')
        partner_ids = partner_obj.search(cr,uid,[('membership_state','in',mstates),('state_id','=',1),('membership_amount','>',0.1),('refuse_membership','=',False),('free_member','=',False),('associate_member','=',False)])
        if data['form']['also_forced']:
            forced_partner_ids = pooler.get_pool(cr.dbname).get('res.partner').search(cr,uid,[('state_id','=',1),('membership_amount','>',0.1),('next_membership_bill_forced','=',True)])
            for partner_id in forced_partner_ids:
                if partner_id not in partner_ids:
                    partner_ids.append(partner_id)
        partners = partner_obj.browse(cr,uid,partner_ids)
        for partner in partners:
            if partner.id not in excluded_partner_ids:
                # search the first good address in the order of sequence_address in partner.address
                # beware : some partner_sequence = false => 99
                good_address = False
                if partner.address:
                    # first, try to find the address containing the membership contact categ
                    if address_type[0:8] == 'contact_':
                        for address in partner.address:
                            if address.job_ids:
                                for job in address.job_ids:
                                    if job.function_code_label and membership_categ in job.function_code_label:
                                        good_address = address
                                        break;
                    #second (of first if contact_ if not asked), try to find the first address of the good type
                    if not good_address:
                        if address_type == 'default' or address_type == 'contact_default':
                            for addr in partner.address:
                                if addr.type == 'default':
                                    good_address = addr
                                    break;
                        else:
                            for addr in partner.address:
                                if addr.type == 'invoice':
                                    good_address = addr
                                    break;
                                else:
                                    if not good_address:
                                        if addr.type == 'default' or not good_address:
                                            good_address = addr
                                            # not break; because we must continue to search after an 'invoice' type
                if good_address:
                    # search for the first job-contact in this address with a good categ
                    lFoundJob = False
                    iCurrent = 0
                    first_seq = 9999
                    while not lFoundJob and iCurrent < len_categs:
                        current_categ = categs[iCurrent]
                        for job in good_address.job_ids:
                            if current_categ in ( job.function_code_label or '' ) and job.contact_id.id > 0:
                                if job.sequence_partner < first_seq:
                                    sel_job = job
                                    sel_contact = job.contact_id
                                    first_seq = job.sequence_partner
                                lFoundJob = True
                                #break;
                        iCurrent += 1
                    # manage special values
                    selected_email = ''
                    if lFoundJob:
                        if sel_job.email:
                            selected_email = sel_job.email
                        elif sel_contact.email:
                            selected_email = sel_contact.email
                    ##
                    ## we now search the same kind of data for the parallel emailing, but perhaps with some other datas
                    ##
                    mailing_good_address = False
                    selected_mailing_email = ''
                    sel_mailing_job = False
                    sel_mailing_contact = False
                    lFoundMailingJob = False
                    complete_mailing_name = 'Cher membre'
                    # first, try to find the address containing the membership contact categ
                    if mailing_address_type[0:8] == 'contact_':
                        for address in partner.address:
                            if address.job_ids:
                                for job in address.job_ids:
                                    if job.function_code_label and mailing_membership_categ in job.function_code_label:
                                        mailing_good_address = address
                                        break;
                    #second (of first if contact_ if not asked), try to find the first address of the good type
                    if not mailing_good_address:
                        if mailing_address_type == 'default' or mailing_address_type == 'contact_default':
                            for addr in partner.address:
                                if addr.type == 'default':
                                    mailing_good_address = addr
                                    break;
                        else:
                            for addr in partner.address:
                                if addr.type == 'invoice':
                                    mailing_good_address = addr
                                    break;
                                else:
                                    if not mailing_good_address:
                                        if addr.type == 'default' or not mailing_good_address:
                                            mailing_good_address = addr
                                            # not break; because we must continue to search after an 'invoice' type
                    if mailing_good_address:
                        # search for the first job-contact in this address with a good categ
                        iCurrent = 0
                        first_seq = 9999
                        while not lFoundMailingJob and iCurrent < len_mailing_categs:
                            current_categ = mailing_categs[iCurrent]
                            for job in mailing_good_address.job_ids:
                                if current_categ in ( job.function_code_label or '' ) and job.contact_id.id > 0 and (job.email or job.contact_id.email):
                                    if job.sequence_partner < first_seq:
                                        sel_mailing_job = job
                                        sel_mailing_contact = job.contact_id
                                        first_seq = job.sequence_partner
                                    lFoundMailingJob = True
                                    #break;
                            iCurrent += 1
                        # manage special values
                        selected_mailing_email = ''
                        if lFoundMailingJob:
                            if sel_mailing_job.email:
                                selected_mailing_email = sel_mailing_job.email
                            elif sel_mailing_contact.email:
                                selected_mailing_email = sel_mailing_contact.email
                            complete_mailing_name = (sel_mailing_contact.title and sel_mailing_contact.title or 'Monsieur') + ' ' + sel_mailing_contact.name.strip()
                        else:
                            complete_mailing_name = 'Cher membre'
                    ##
                    ## this ends the search of data for the emailing
                    ##
                    
                    # Now treat the rest of the necessary data for the call
                    if partner.user_id.id in dSalesman:
                        salesman = dSalesman[partner.user_id.id]
                    else:
                        salesman = False
                    global_name = ''
                    if good_address.name and len(good_address.name) > 0:
                        if good_address.name[0] == '-':
                            global_name = partner.name.strip() + ' ' + good_address.name.strip()
                        else:
                            global_name = good_address.name
                    else:
                        global_name = partner.name
                    if partner.property_account_position:
                        current_tax_rate = 0.0
                    else:
                        current_tax_rate = tax_rate
                    readable_vat_num = ''
                    if partner.vat:
                        if len(partner.vat) > 2:
                            if partner.vat[0:2].upper() == 'BE':
                                if len(partner.vat) == 12:
                                    readable_vat_num = 'BE ' + partner.vat[2:6] + ' ' + partner.vat[6:9] + ' ' + partner.vat[9:]
                                else:
                                    readable_vat_num = 'BE ' + partner.vat[2:]
                            else:
                                readable_vat_num = partner.vat[0:2] + '-' + partner.vat[2:]
                        else:
                            readable_vat_num = partner.vat
                    # record the data
                    record = { 'partner_id':partner.id,
                               'address_id':good_address.id,
                               'job_id':lFoundJob and sel_job.id or 0,
                               'contact_id':lFoundJob and sel_contact.id or 0,
                               'partner_name':global_name,
                               'street':good_address.street or '',
                               'street2':good_address.street2 or '',
                               'zip_code':good_address.zip_id.name or '',
                               'city':good_address.zip_id.city or '',
                               'email_addr':good_address.email or '',
                               'phone_addr':self.convert_phone(good_address.phone or '',PREFIXES),
                               'fax_addr':self.convert_phone(good_address.fax or '',PREFIXES),
                               'name':lFoundJob and sel_contact.name or '',
                               'first_name':lFoundJob and ( sel_contact.first_name or '' ) or '',
                               'courtesy':lFoundJob and (sel_contact.title or '' ) or '',
                               'title':lFoundJob and ( sel_job.function_label or '' ) or '',
                               'title_categs':lFoundJob and sel_job.function_code_label or '',
                               'email_contact':selected_email,
                               'base_amount':partner.membership_amount or 0.0,
                               'count_add_sites':partner.site_membership or 0,
                               'unit_price_site':unit_price,
                               'desc_add_site':partner.desc_add_site or '',
                               'wovat_amount':( partner.membership_amount or 0.0 ) + (( partner.site_membership or 0 ) * unit_price),
                               'wvat_amount':( ( partner.membership_amount or 0.0 ) + (( partner.site_membership or 0 ) * unit_price) ) * ( 1.0 + current_tax_rate ),
                               'char_wovat_amount':' '+float_to_char(( partner.membership_amount or 0.0 ) + (( partner.site_membership or 0 ) * unit_price)),
                               'char_wvat_amount':' '+float_to_char(( ( partner.membership_amount or 0.0 ) + (( partner.site_membership or 0 ) * unit_price) ) * ( 1.0 + current_tax_rate )),
                               'salesman':salesman and salesman.name or '',
                               'salesman_phone':salesman and ( salesman.phone or '' ) or '',
                               'salesman_fax':salesman and ( salesman.fax or '' ) or '',
                               'salesman_mobile':salesman and ( salesman.mobile or '' ) or '',
                               'salesman_email':salesman and ( salesman.email or '' ) or '',
                               'salesman_linkedin':salesman and ( salesman.linkedin or '' ) or '',
                               'salesman_login':salesman and ( salesman.login or '' ) or '',
                               'mailing_email':selected_mailing_email,
                               'mailing_complete_name':complete_mailing_name,
                               'mailing_courtesy':sel_mailing_contact and sel_mailing_contact.title or '',
                               'mailing_name':sel_mailing_contact and sel_mailing_contact.name or '',
                               'mailing_firstname':sel_mailing_contact and sel_mailing_contact.first_name or '',
                               'mailing_job_id':sel_mailing_job and sel_mailing_job.id or 0,
                               'mailing_contact_id':sel_mailing_contact and sel_mailing_contact.id or 0,
                               'vcs':partner.membership_vcs or '',
                               'vat_num':readable_vat_num or 'inconnu',
                             }
                    new_id = call_obj.create(cr, uid, record, context)
        result = {
            'name': _('Membership Call'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'cci_membership.membership_call',
            'context': {},
            'type': 'ir.actions.act_window'
        }
        return result

    states = {
        'init' : {
            'actions' : [_defaults],
            'result' : {'type' : 'form' , 'arch' : form,
                    'fields' : fields,
                    'state' : [('end', 'Cancel'), ('go', 'Go')]}
        },
        'go': {
            'actions': [],
            'result': {'type': 'action', 'action':_open_window_results, 'state':'end'}
        }

    }
membership_calls("membership_calls")
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

