# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
# This wizard select all partners invoiced with a specificed product last year
#
import wizard
import pooler
import datetime
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Select Partners to Invoice">
    <field name="year"/>
    <field name="also_invoiced"/>
    <field name="also_forced"/>
    <newline/>
    <field name="old_product_id" colspan="4" width="800"/>
</form>
"""

fields = {
    'year':{'string':'New year','type':'integer','help':'New year of the called membership...','required':True},
    'also_invoiced':{'string':'Also the invoiced members, else only paid','type':'boolean','default':False},
    'also_forced':{'string':'Also the forced partners','type':'boolean','default':True},
    'old_product_id':{'string':'Old Product to Search','type':'many2one','relation':'product.product','domain':"[('membership','=',True)]",'required':True}
}

class membership_partners_to_invoice(wizard.interface):

    def _defaults(self, cr, uid, data, context):
        data['form']['year'] = datetime.datetime.today().year + 1
        return data['form']

    def _open_window_results(self, cr, uid, data, context):
        # manage parameters
        if not data['form']['year']:
            raise wizard.except_wizard('Warning','You must give the year of membership concerned; between 1900 and the next year')
        if data['form']['year'] and int(data['form']['year']) < 1900:
            raise wizard.except_wizard('Warning','You must give the year of membership concerned; between 1900 and the next year')
        if data['form']['also_invoiced']:
            mstates = ['invoiced','paid']
            smstates = "('invoiced','paid')"
        else:            
            mstates = ['paid']
            smstates = "('paid')"
        # get this list of partners already invoiced for the next year of membership
        # on basis of membership products for this year, active or not
        mprod_obj = pooler.get_pool(cr.dbname).get('product.product')
        mprod_ids = mprod_obj.search(cr,uid,[('membership','=',True),('membership_year','=',int(data['form']['year'])),'|',('active','=',True),('active','=',False)])
        excluded_partner_ids = []
        if len(mprod_ids) > 0:
            selection = "SELECT distinct(partner_id) FROM account_move_line where product_id in (%s)" % ','.join([str(x) for x in mprod_ids])
            cr.execute( selection )
            partners = cr.fetchall()
            excluded_partner_ids = [x[0] for x in partners]
        # get all partners to be invoiced
        selection = "SELECT id FROM res_partner where active AND state_id = 1 AND membership_state in %s AND membership_amount > 0.1 AND id in (SELECT distinct(partner_id) FROM account_move_line where product_id = %s) AND NOT refuse_membership AND NOT free_member AND associate_member IS NULL" % (smstates,str(data['form']['old_product_id']))
        cr.execute( selection )
        partners = cr.fetchall()
        partner_ids = [x[0] for x in partners if x[0] not in excluded_partner_ids]
        if data['form']['also_forced']:
            forced_partner_ids = pooler.get_pool(cr.dbname).get('res.partner').search(cr,uid,[('state_id','=',1),('membership_amount','>',0.1),('next_membership_bill_forced','=',True)])
            for partner_id in forced_partner_ids:
                if partner_id not in partner_ids:
                    partner_ids.append(partner_id)
        result = {
            'name': _('Partners to Invoice'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'res.partner',
            'context': {},
            'type': 'ir.actions.act_window',
            'domain': [('id', 'in', partner_ids)],
        }
        return result

    states = {
        'init' : {
            'actions' : [_defaults],
            'result' : {'type' : 'form' , 'arch' : form,
                    'fields' : fields,
                    'state' : [('end', 'Cancel'), ('go', 'Go')]}
        },
        'go': {
            'actions': [],
            'result': {'type': 'action', 'action':_open_window_results, 'state':'end'}
        }

    }
membership_partners_to_invoice("membership_partners_to_invoice")
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

