# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
import wizard
import time
import datetime
import pooler
import tools
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Send Calls by Email">
    <field name="from" colspan="4" width="700"/>
    <field name="warning1" colspan="4" nolabel="1"/>
    <field name="forced_to" colspan="4"/>
    <field name="warning2" colspan="4" nolabel="1"/>
    <field name="template" colspan="4"/>
    <field name="record_send"/>
    <field name="only_not_sended"/>
</form>"""

fields = {
    'warning1': {'string':'', 'type':'char','size':100,'default':'If empty, use the salesman email address for the sending !','readonly':True},
    'warning2': {'string':'', 'type':'char','size':100,'default':'Only for tests ! If empty, use the email address of the member...','readonly':True},
    'from': {'string': 'Email From', 'type':'char','size':200},
    'forced_to': {'string': 'Email To', 'type':'char','size':200},
    'template': {'string': 'Template', 'type':'many2one','relation':'cci_email_template','required':True},
    'record_send':{'string':'Record date of sending','type':'boolean','default':True},
    'only_not_sended':{'string':'Send only if not already sended','type':'boolean','default':True},
}

final_form = """<?xml version="1.0"?>
<form string="Results">
    <field name="results" colspan="4" width="600"/>
</form>"""

final_fields = {
    'results' : {'string':'Results','type':'text','readonly':True,'value':''}
}

def _personalize_text(base,call_obj):
    # base = text with some fields between {} to get from call_obj
    # call_obj = browse object of a cci_membership.membership_call
    original_text = base
    result = ''
    while '{{' in original_text:
        pos1 = original_text.find('{{')
        pos2 = original_text.find('}}')
        if pos1 < pos2:
            if pos1 > 0:
                result += original_text[0:pos1]
            original_text = original_text[pos1+2:]
            pos2 = original_text.find('}}')
            asked_field = original_text[0:pos2].strip()
            asked_value = getattr(call_obj, asked_field)
            if not asked_value: # if False
                asked_value = ''
            else:
                asked_value = asked_value.strip()
            result += asked_value
            original_text = original_text[pos2+2:]
        else:
            result = base
            original_text = ''
            break # some problems with the brackets, we stop the treatment so far
    result += original_text # the end
    return result

class cci_membership_send_calls_by_email(wizard.interface):
    def _send_emails(self,cr,uid,data,context):
        dt_now = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        template_id = data['form']['template']
        template = pooler.get_pool(cr.dbname).get('cci_email_template').browse(cr,uid,template_id)
        base_body = template.body
        base_subject = template.subject
        sended_calls = 0
        errors = {}
        parameter_obj = pooler.get_pool(cr.dbname).get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['BDDeTest'])
        if param_values.has_key('BDDeTest'):
            bddetest = param_values['BDDeTest']
        else:
            bddetest = True
        call_obj = pooler.get_pool(cr.dbname).get('cci_membership.membership_call')
        calls = call_obj.browse(cr,uid,data['ids'])
        for call in calls:
            if call.mailing_email and not (call.mailing_send_date and data['form']['only_not_sended']):
                if '{{' in base_subject:
                    perso_subject = _personalize_text(base_subject,call)
                else:
                    perso_subject = base_subject
                if '{{' in base_body:
                    perso_body = _personalize_text(base_body,call)
                else:
                    perso_body = base_body
                if data['form']['from']:
                    email_from = data['form']['from']
                else:
                    email_from = call.salesman_email
                if data['form']['forced_to']:
                    email_to = data['form']['forced_to']
                else:
                    email_to = call.mailing_email
                if email_from and email_to:
                    print email_to
                    try:
                        if bddetest:
                            print 'mocked email send'
                            print 'From : \'%s\'' % email_from
                            print 'To : [\'%s\']' % email_to
                            print 'Subject : %s' % perso_subject
                            print 'Body :\n%s' % perso_body
                        else:
                            tools.email_send(email_from, [email_to,], perso_subject, perso_body, subtype='html')
                    except:
                        errors.append('Wrong address : %s' % email_to)
                    if data['form']['record_send']:
                        call_obj.write(cr,uid,[call.id,],{'mailing_send_date':dt_now})
                    sended_calls += 1
        if not errors:
            results = 'Sended emails : %i\nNo errors while sending...' % sended_calls
        else:
            results = 'Sended emails : %i\n\nErrors :\n' % sended_calls
            results += '\n'.join(errors)
        data['form']['results'] = results
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel','gtk-stop'),('send','Send','gtk-find',True)]},
        },
        'send': {
            'actions': [_send_emails],
            'result': {'type':'form', 'arch':final_form, 'fields':final_fields, 'state':[('end','Close')]}
        },
    }

cci_membership_send_calls_by_email('send_calls_by_email')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

