# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
# Version 5.0.1 : Philmer
from osv import fields, osv
import datetime

class cci_metric_sensor(osv.osv):
    _name = 'cci_metric.sensor'
    _description = "Name of the data"
    _columns = {
        'name': fields.char('Name',size=64,required=True),
        'code': fields.char('Internal Code',size=15),
        'type': fields.selection([('integer','Integer'),('text','Text')],'Type',required=True),
        'description': fields.text('Description'),
        'active':fields.boolean('Active'),
    }
    _defaults = {
        'active': lambda *a: True,
    }
cci_metric_sensor()

class cci_metric(osv.osv):
    _name = 'cci_metric'
    _description = "Metrics for the CCI"
    _columns = {
        'capture_date': fields.datetime('Capture Date',required=True),
        'sensor_id': fields.many2one('cci_metric.sensor','Data name',required=True),
        'value_int': fields.integer('Value in Integer'),
        'value_text': fields.text('Value as Text'),
    }
    def _collect(self,cr,uid):
        now = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        partner_obj = self.pool.get('res.partner')
        addr_obj = self.pool.get('res.partner.address')
        metric_obj = self.pool.get('cci_metric')
        metric_sensor_obj = self.pool.get('cci_metric.sensor')
        account_obj = self.pool.get('account.account')
        period_obj = self.pool.get('account.period')
        amline_obj = self.pool.get('account.move.line')
        area_obj = self.pool.get('res.partner.zip.group')
        range_obj = self.pool.get('cci_membership.membership_range')
        zip_obj = self.pool.get('res.partner.zip')
        metrics = {}
        member_ids = partner_obj.search(cr,uid,[('membership_state','in',['free','invoiced','paid'])])
        metrics['#ALLMEMBERS#'] = len(member_ids)
        metrics['#PAIDMEMBERS#'] = 0
        metrics['#INVMEMBERS#'] = 0
        metrics['#FREEMEMBERS#'] = 0
        metrics['#ASSOPMEMBERS#'] = 0
        metrics['#ASSOIMEMBERS#'] = 0
        metrics['#ASSOFMEMBERS#'] = 0
        metrics['#MEMB_TO_10#'] = 0
        metrics['#MEMB_11_25#'] = 0
        metrics['#MEMB_26_50#'] = 0
        metrics['#MEMB_51_100#'] = 0
        metrics['#MEMB_101_250#'] = 0
        metrics['#MEMB_251_500#'] = 0
        metrics['#MEMB_501+#'] = 0
        metrics['#AMOUNTCYMEMB#'] = 0.0
        metrics['#AMOUNTNYMEMB#'] = 0.0
        metrics['#FEDMEMBPAYANT#'] = 0.0
        metrics['#FEDMEMB+5#'] = 0
        metrics['#FEDMEMB+20#'] = 0
        metrics['#FEDMEMB+50#'] = 0
        # get all areas id in 'Anciennes structures' and 'Sous-région'
        area_ids = area_obj.search(cr,uid,[('type_id','in',[4,7])])
        anc_struct_ids = area_obj.search(cr,uid,[('type_id','=',7)])
        if area_ids:
            areas = area_obj.read(cr,uid,area_ids,['name','type_id'])
        range_ids = range_obj.search(cr,uid,[('year','=',2018)])
        if range_ids:
            ranges = range_obj.read(cr,uid,range_ids,['date_from','date_to'])
        areas_list = []
        dZip2AncStruct = {}
        dAreaNames = {}
        if area_ids and range_ids:
            for area_data in areas:
                if area_data['type_id'][0] == 4:
                    area_code = u'SR_'
                else:
                    area_code = u'AS_'
                ranges_list = []
                for range_data in ranges:
                    range_record = {'range_id': range_data['id'],
                                    'range_name': u'%i à %i' % (range_data['date_from'],range_data['date_to']),
                                    'range_from': range_data['date_from'],
                                    'range_to': range_data['date_to'],
                                    'count': 0,
                                   }
                    ranges_list.append(range_record)
                zip_ids = zip_obj.search(cr,uid,[('groups_id','=',area_data['id'])])
                area_record = {'area_id': area_data['id'],
                               'area_name': area_code + area_data['name'].strip(),
                               'area_zip_ids': zip_ids,
                               'ranges': ranges_list,
                              }
                dAreaNames[area_record['area_id']] = area_record['area_name']
                areas_list.append(area_record)
                if area_data['id'] in anc_struct_ids:
                    for zip_id in zip_ids:
                        dZip2AncStruct[zip_id] = area_data['id']        
        members = partner_obj.read(cr,uid,member_ids,['membership_state','employee_nbr_total','associate_member','address'])
        dMembersTypeAreaIDs = {}
        for type_member in ['associated_paid','associated_invoiced','associated_free','full_paid','full_invoiced','full_free']:
            for area_id in anc_struct_ids:
                dMembersTypeAreaIDs[(type_member,area_id)] = []
        for memb in members:
            type_member = False
            if memb['associate_member']:
                if memb['membership_state'] == 'paid':
                    metrics['#ASSOPMEMBERS#'] += 1
                    type_member = 'associated_paid'
                elif memb['membership_state'] == 'invoiced':
                    metrics['#ASSOIMEMBERS#'] += 1
                    type_member = 'associated_invoiced'
                elif memb['membership_state'] == 'free':
                    metrics['#ASSOFMEMBERS#'] += 1
                    type_member = 'associated_free'
            else:
                if memb['membership_state'] == 'paid':
                    metrics['#PAIDMEMBERS#'] += 1
                    metrics['#FEDMEMBPAYANT#'] += 1
                    type_member = 'full_paid'
                elif memb['membership_state'] == 'invoiced':
                    metrics['#INVMEMBERS#'] += 1
                    metrics['#FEDMEMBPAYANT#'] += 1
                    type_member = 'full_invoiced'
                elif memb['membership_state'] == 'free':
                    metrics['#FREEMEMBERS#'] += 1
                    type_member = 'full_free'
            if not memb['employee_nbr_total']:
                metrics['#MEMB_TO_10#'] += 1
            elif memb['employee_nbr_total'] <= 10:
                metrics['#MEMB_TO_10#'] += 1
            elif memb['employee_nbr_total'] <= 25:
                metrics['#MEMB_11_25#'] += 1
            elif memb['employee_nbr_total'] <= 50:
                metrics['#MEMB_26_50#'] += 1
            elif memb['employee_nbr_total'] <= 100:
                metrics['#MEMB_51_100#'] += 1
            elif memb['employee_nbr_total'] <= 250:
                metrics['#MEMB_101_250#'] += 1
            elif memb['employee_nbr_total'] <= 500:
                metrics['#MEMB_251_500#'] += 1
            else:
                metrics['#MEMB_501+#'] += 1
            if memb['employee_nbr_total'] and memb['employee_nbr_total'] > 5:
                metrics['#FEDMEMB+5#'] += 1
                if memb['employee_nbr_total'] > 20:
                    metrics['#FEDMEMB+20#'] += 1
                    if memb['employee_nbr_total'] > 50:
                        metrics['#FEDMEMB+50#'] += 1
            if area_ids and range_ids:
                def_zip_id = False
                if memb['address']:
                    addr_ids = addr_obj.search(cr,uid,[('id','in',memb['address']),('type','=','default')])
                    if addr_ids and len(addr_ids) == 1:
                        address = addr_obj.read(cr,uid,addr_ids,['zip_id'])[0]
                        if address['zip_id']:
                            def_zip_id = address['zip_id'][0]
                if def_zip_id:
                    new_areas = []
                    for area_record in areas_list:
                        if def_zip_id in area_record['area_zip_ids']:
                            old_ranges = area_record['ranges']
                            new_ranges = []
                            for range_record in old_ranges:
                                if (memb['employee_nbr_total'] >= range_record['range_from']) and (memb['employee_nbr_total'] <= range_record['range_to']):
                                    range_record['count'] += 1
                                new_ranges.append(range_record)
                            area_record['ranges'] = new_ranges
                        new_areas.append(area_record)
                    areas_list = new_areas  
                if (def_zip_id in dZip2AncStruct.keys()) and type_member:
                    cumul = dMembersTypeAreaIDs[(type_member,dZip2AncStruct[def_zip_id])]
                    cumul.append(memb['id'])
                    dMembersTypeAreaIDs[(type_member,dZip2AncStruct[def_zip_id])] = cumul
        #
        account_ids = account_obj.search(cr,uid,[('code','in',['707000','493010','493011','493012','493013','493014'])])
        if account_ids:
            beg = now[0:4]+'-01-01'
            end = now[0:4]+'-12-31'
            period_ids = period_obj.search(cr,uid,[('date_start','>=',beg),('date_stop','<=',end)])
            if period_ids and len(period_ids) >= 12 and len(period_ids) <= 13:
                cy_amline_ids = amline_obj.search(cr,uid,[('account_id','in',account_ids),('period_id','in',period_ids)])
                if cy_amline_ids:
                    cy_amlines = amline_obj.read(cr,uid,cy_amline_ids,['credit','debit'])
                    for amline in cy_amlines:
                        metrics['#AMOUNTCYMEMB#'] += ((amline['credit'] or 0.0) - (amline['debit'] or 0.0))
        account_ids = account_obj.search(cr,uid,[('code','=','493100')])
        if account_ids:
            beg = now[0:4]+'-01-01'
            end = now[0:4]+'-12-31'
            period_ids = period_obj.search(cr,uid,[('date_start','>=',beg),('date_stop','<=',end)])
            if period_ids and len(period_ids) >= 12 and len(period_ids) <= 13:
                ny_amline_ids = amline_obj.search(cr,uid,[('account_id','in',account_ids),('period_id','in',period_ids)])
                if ny_amline_ids:
                    ny_amlines = amline_obj.read(cr,uid,ny_amline_ids,['credit','debit'])
                    for amline in ny_amlines:
                        metrics['#AMOUNTNYMEMB#'] += ((amline['credit'] or 0.0) - (amline['debit'] or 0.0))
        for (key,value) in metrics.items():
            sensor_ids = metric_sensor_obj.search(cr,uid,[('code','=',key)])
            if sensor_ids and len(sensor_ids) == 1:
                if key in ['#AMOUNTCYMEMB#','#AMOUNTNYMEMB#']:
                    data = {'capture_date':now,'sensor_id':sensor_ids[0],'value_int':int(value)}
                else:
                    data = {'capture_date':now,'sensor_id':sensor_ids[0],'value_int':value}
                metric_obj.create(cr,uid,data)
        for area_record in areas_list:
            for range_record in area_record['ranges']:
                code = u'#RM_%s_%s#' % (str(area_record['area_id']),str(range_record['range_id']))
                name = u'Répart Membres %s : %s' % (area_record['area_name'],range_record['range_name'])
                count = range_record['count']
                metric_sensor_ids = metric_sensor_obj.search(cr,uid,[('code','=',code)])
                if metric_sensor_ids:
                    metric_sensor_id = metric_sensor_ids[0]
                else:
                    metric_sensor_id = metric_sensor_obj.create(cr,uid,{'name': name,
                                                                        'code': code,
                                                                        'type': 'integer',
                                                                        'description': u'Répartition membres par zones et par tranche de Membership :\nType : %s\nNom zone : %s\nTranche : %s' % (
                                                                                area_record['area_name'][0:3] == 'AS_' and u'Anciennes structures' or u'Sous-régions',
                                                                                area_record['area_name'],
                                                                                range_record['range_name'], )})
                metric_obj.create(cr,uid,{'capture_date':now,'sensor_id':metric_sensor_id,'value_int':count})
        for type_member in ['associated_paid','associated_invoiced','associated_free','full_paid','full_invoiced','full_free']:
            for area_id in anc_struct_ids:
                if type_member == 'associated_paid':
                    code_type = 'AP'
                elif type_member == 'associated_invoiced':
                    code_type = 'AI'
                elif type_member == 'associated_free':
                    code_type = 'AF'
                elif type_member == 'full_paid':
                    code_type = 'FP'
                elif type_member == 'full_invoiced':
                    code_type = 'FI'
                elif type_member == 'full_free':
                    code_type = 'FF'
                code = u'#MID_%s_%s#' % (code_type,str(area_id))
                name = u'IDs Membres %s pour %s' % (type_member,dAreaNames[area_id])
                metric_sensor_ids = metric_sensor_obj.search(cr,uid,[('code','=',code)])
                if metric_sensor_ids:
                    metric_sensor_id = metric_sensor_ids[0]
                else:
                    metric_sensor_id = metric_sensor_obj.create(cr,uid,{'name': name,
                                                                        'code': code,
                                                                        'type': 'text',
                                                                        'description': u'Liste des IDs des membres par zone et par type de membership',
                                                                        })
                if len(dMembersTypeAreaIDs[(type_member,area_id)]) != 1:
                    metric_obj.create(cr,uid,{'capture_date':now,'sensor_id':metric_sensor_id,'value_int':len(dMembersTypeAreaIDs[(type_member,area_id)]),'value_text':'set((%s))' % ','.join([str(x) for x in dMembersTypeAreaIDs[(type_member,area_id)]])})
                else:
                    metric_obj.create(cr,uid,{'capture_date':now,'sensor_id':metric_sensor_id,'value_int':len(dMembersTypeAreaIDs[(type_member,area_id)]),'value_text':'set((%s,))' % ','.join([str(x) for x in dMembersTypeAreaIDs[(type_member,area_id)]])})
        return True
cci_metric()
