# -*- encoding: utf-8 -*-
from osv import fields, osv
import wizard
import pooler
import os
import base64
import time
from tools.translate import _
from pyExcelerator import *
from pyExcelerator.Worksheet import *

form = """<?xml version="1.0"?>
<form string="Extract Membership Metrics">
    <field name="from_period_id" colspan="4" width="600"/>
    <field name="to_period_id" colspan="4"/>
    <field name="steps" colspan="4"/>
    <field name="list_type" colspan="4"/>
</form>"""

fields = {
    'from_period_id': {'string':'From Period', 'type':'many2one', 'relation':'account.period','required':True},
    'to_period_id': {'string':'To Period', 'type':'many2one', 'relation':'account.period','required':True},
    'steps': {'string':'Steps', 'type':'selection', 'selection':[('yearly','Yearly'),('monthly','Monthly'),('weekly','Weekly')],'required':True,'default':'monthly'},
    'list_type': {'string':'List', 'type':'selection', 'selection':[('board','Board'),('all','All')],'required':True,'default':'board'},
}

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File has been created."  colspan="4"/>
     <field name="membership_metrics.xls" colspan="4" />
</form>"""

msg_fields = {
    'membership_metrics.xls':{'string': 'Prepared file', 'type': 'binary', 'readonly': True,},
}

def french_date(sdate):
    return sdate[8:10]+'/'+sdate[5:7]+'/'+sdate[0:4]

def num_to_letter(line_number_in_pyexcel):
    # this method convert an integer to a line label in Excel
    # first line = 0 (beware on pyExcelerator, first line = 0)
    # 0 => 'A'
    # 26 => 'AA'
    # 27 => 'AB'
    first = (line_number_in_pyexcel // 26)
    last = (line_number_in_pyexcel % 26) + 1
    if first > 0:
        result = chr(64+first)+chr(64+last)
    else:
        result = chr(64+last)
    return result
def full_cell(line_number_in_pyexcel,col_number_in_pyexcel,fixed='none'):
    # fixed = none, col, line, both
    result = ''
    if fixed in ('col','both'):
        result += '$'
    result += num_to_letter(col_number_in_pyexcel)
    if fixed in ('line','both'):
        result += '$'
    result += str(line_number_in_pyexcel+1)
    return result
    
class extract_membership_metrics(wizard.interface):
    def _extract_metrics(self,cr,uid,data,context):
        obj_metric = pooler.get_pool(cr.dbname).get('cci_metric')
        obj_metric_sensor = pooler.get_pool(cr.dbname).get('cci_metric.sensor')
        obj_period = pooler.get_pool(cr.dbname).get('account.period')
        obj_partner = pooler.get_pool(cr.dbname).get('res.partner')
        if data['form']['list_type'] == 'board':
            metrics_codes = ['#ALLMEMBERS#','#MEMB_TO_10#','#MEMB_11_25#','#MEMB_26_50#','#MEMB_51_100#','#MEMB_101_250#','#MEMB_251_500#','#MEMB_501+#']
        else:
            metrics_codes = ['#ALLMEMBERS#','#MEMB_TO_10#','#MEMB_11_25#','#MEMB_26_50#','#MEMB_51_100#','#MEMB_101_250#','#MEMB_251_500#','#MEMB_501+#',
                             '#PAIDMEMBERS#','#INVMEMBERS#','#FREEMEMBERS#','#ASSOPMEMBERS#','#ASSOIMEMBERS#','#ASSOFMEMBERS#','#AMOUNTCYMEMB#','#AMOUNTNYMEMB#',
                             '#FEDMEMBPAYANT#','#FEDMEMB+5#','#FEDMEMB+20#','#FEDMEMB+50#']
        dates = []
        if data['form']['steps'] == 'monthly':
            period_from = obj_period.browse(cr,uid,data['form']['from_period_id'])
            period_to = obj_period.browse(cr,uid,data['form']['to_period_id'])
            from_date = period_from.date_start
            to_date = period_to.date_stop
            period_ids = obj_period.search(cr,uid,[('date_start','>=',from_date),('date_stop','<=',to_date),('special','=',False)])
            if period_ids:
                periods = obj_period.read(cr,uid,period_ids,['date_start'])
                for period in periods:
                    dates.append(period['date_start'])
        elif data['form']['steps'] == 'yearly':
            period_from = obj_period.browse(cr,uid,data['form']['from_period_id'])
            period_to = obj_period.browse(cr,uid,data['form']['to_period_id'])
            from_date = period_from.date_start
            to_date = period_to.date_stop
            new_date = from_date
            while new_date <= to_date:
                dates.append(new_date)
                new_date = str(int(new_date[0:4])+1)+new_date[4:]
        if dates:
            wb = Workbook()
            ws1 = wb.add_sheet(_('Raw Data'))
            ws1.write(0,0,_('Raw Data'))
            ws1.write(1,0,_('Date'))
            #
            ws2 = wb.add_sheet(_('Raw Data by Area and Sizes'))
            ws2.write(0,0,_('Raw Data by Area and Sizes'))
            ws2.write(1,1,_('Date->'))
            ws2.write(2,0,_('Area'))
            ws2.write(2,1,_('Size'))
            line2 = 3
            #
            #ws3 = wb.add_sheet(_('Raw Data by Area and Types'))
            #ws3.write(0,0,_('Raw Data'))
            #ws3.write(1,1,_('Date'))
            #ws3.write(2,0,_('Area'))
            #ws3.write(2,1,_('Type'))
            #line3 = 2
            #
            ws4 = wb.add_sheet(_('Lost Members'))
            ws4.write(0,0,_('List of losts members'))
            ws4.write(1,0,'ID')
            ws4.write(1,1,_('Partner Name'))
            ws4.write(1,2,_('Last date as Member'))
            ws4.write(1,3,_('Last Area'))
            ws4.write(1,4,_('Last Type'))
            line4 = 2
            #
            ws5 = wb.add_sheet(_('New Members'))
            ws5.write(0,0,_('List of new members'))
            ws5.write(1,0,'ID')
            ws5.write(1,1,_('Partner Name'))
            ws5.write(1,2,_('First date as Member'))
            ws5.write(1,3,_('Current Area'))
            ws5.write(1,4,_('Current Type'))
            line5 = 2
            #
            dates.sort()
            dates_with_data = []
            for metric_date in dates:
                metric_ids = obj_metric.search(cr,uid,[('capture_date','>=',metric_date+' 00:00:01'),('capture_date','<=',metric_date+' 23:59:59')])
                if metric_ids: ### or len(metric_ids) == len(metrics_codes)
                    dates_with_data.append(metric_date)
            metric_name_ids = obj_metric_sensor.search(cr,uid,[('code','in',metrics_codes)])
            if len(metric_name_ids) == len(metrics_codes):
                metric_names = obj_metric_sensor.read(cr,uid,metric_name_ids,['code','name','type'])
                dMetricName = {}
                for metric_name in metric_names:
                    dMetricName[metric_name['code']] = metric_name
                col = 2
                for mcode in metrics_codes:
                    ws1.write(1,col,dMetricName[mcode]['name'])
                    col += 1
                line = 2
                for metric_date in dates_with_data:
                    ws1.write(line,1,french_date(metric_date))
                    col = 2
                    for mcode in metrics_codes:
                        metric_ids = obj_metric.search(cr,uid,[('capture_date','>=',metric_date+' 00:00:01'),('capture_date','<=',metric_date+' 23:59:59'),('sensor_id','=',dMetricName[mcode]['id'])])
                        if metric_ids:
                            metric = obj_metric.read(cr,uid,metric_ids[0],['value_int'])
                            if metric['value_int']:
                                ws1.write(line,col,metric['value_int'])
                            else:
                                ws1.write(line,col,0)
                        else:    
                            ws1.write(line,col,0)
                        col += 1
                    line += 1
            # by size and area
            dates_with_data = []
            sensor_codes = []
            for metric_date in dates:
                metric_ids = obj_metric.search(cr,uid,[('capture_date','>=',metric_date+' 00:00:01'),('capture_date','<=',metric_date+' 23:59:59'),('sensor_id.name','ilike',u'Répart Membres AS_')])
                if metric_ids: ### or len(metric_ids) == len(metrics_codes)
                    dates_with_data.append(metric_date)
            if dates_with_data:
                dates_with_data.sort(reverse=True)
                dDateCol = {}
                col = 2
                for date_with_data in dates_with_data:
                    ws2.write(1,col,date_with_data)
                    dDateCol[date_with_data] = col
                    col += 1
                metric_ids = obj_metric.search(cr,uid,[('capture_date','>=',dates_with_data[0]+' 00:00:01'),('capture_date','<=',dates_with_data[0]+' 23:59:59'),('sensor_id.name','ilike',u'Répart Membres AS_')])
                metrics = obj_metric.read(cr,uid,metric_ids,['sensor_id'])
                sensor_ids = []
                for metric in metrics:
                    if metric['sensor_id'] and metric['sensor_id'][0] not in sensor_ids:
                        sensor_ids.append(metric['sensor_id'][0])
                sensors = obj_metric_sensor.read(cr,uid,sensor_ids,['code','name'])
                dSensorCode2Name = {}
                sensor_codes = []
                for sensor in sensors:
                    sensor_codes.append(sensor['code'])
                    dSensorCode2Name[sensor['code']] = sensor['name']
                sensor_codes.sort()
                old_area = '-----'
                first_line_area = 0
                dSensor2Line = {}
                subtotal_lines = []
                for sensor_code in sensor_codes:
                    sensor_parts = dSensorCode2Name[sensor_code].split(' ')
                    sensor_area = sensor_parts[2]
                    sensor_size = ' '.join((sensor_parts[4],sensor_parts[5],sensor_parts[6]))
                    if sensor_area != old_area:
                        if first_line_area > 0:
                            # end the current area
                            ws2.write(line2,0,old_area)
                            ws2.write(line2,1,_('Total'))
                            col = 2
                            while col < (2+len(dates_with_data)):
                                ws2.write(line2,col,Formula("SUM(%s:%s)" % (full_cell(first_line_area,col),full_cell(line2-1,col)) ))
                                col += 1
                            subtotal_lines.append(line2)
                            line2 += 2
                        first_line_area = line2
                        old_area = sensor_area
                    ws2.write(line2,0,sensor_area)
                    ws2.write(line2,1,sensor_size)
                    dSensor2Line[sensor_code] = line2
                    line2 += 1
                # end the last area
                ws2.write(line2,0,old_area)
                ws2.write(line2,1,_('Total'))
                col = 2
                while col < (2+len(dates_with_data)):
                    ws2.write(line2,col,Formula("SUM(%s:%s)" % (full_cell(first_line_area,col),full_cell(line2-1,col)) ))
                    col += 1
                line2 += 2

                # end all areas
                ws2.write(line2,0,_('All Areas'))
                ws2.write(line2,1,_('Total'))
                col = 2
                while col < (2+len(dates_with_data)):
                    ws2.write(line2,col,Formula("SUM(%s)" % ';'.join([full_cell(x,col) for x in subtotal_lines])) )
                    col += 1
                line2 += 2
                
                # put all metrics in table
                all_metric_ids = []
                for month_date in dates_with_data:
                    metric_ids = obj_metric.search(cr,uid,[('capture_date','>=',month_date+' 00:00:01'),('capture_date','<=',month_date+' 23:59:59'),('sensor_id.name','ilike',u'Répart Membres AS_')])
                    if metric_ids:
                        all_metric_ids.extend(metric_ids)
                metrics = obj_metric.browse(cr,uid,all_metric_ids)
                for metric in metrics:
                    if dSensor2Line.has_key(metric.sensor_id.code):
                        line_cell = dSensor2Line[metric.sensor_id.code]
                        col_cell = 2+dates_with_data.index(metric.capture_date[0:10])
                        ws2.write(line_cell,col_cell,metric.value_int)
                    else:
                        print 'wrong sensor code ' + metric.sensor_id.code

            # by area and type : NOT NOW ONLY FOR CALCULATING NEW AND OLD
            area_obj = pooler.get_pool(cr.dbname).get('res.partner.zip.group')
            area_ids = area_obj.search(cr,uid,[('type_id','=',7)])
            if area_ids:
                area_ids.sort()
                areas = area_obj.read(cr,uid,area_ids,['name'])
            dAreas = {}
            for area in areas:
                dAreas[area['id']] = {'name':area['name'],'line':0}
            # new and old members
            dates.sort()
            dates_with_data = []
            dDatesMembersSet = {}
            dDatesTypeAreaSet = {}
            for metric_date in dates:
                metric_ids = obj_metric.search(cr,uid,[('capture_date','>=',metric_date+' 00:00:01'),('capture_date','<=',metric_date+' 23:59:59'),('sensor_id.code','ilike','#MID_')])
                if metric_ids:
                    dates_with_data.append(metric_date)
                    metrics = obj_metric.browse(cr,uid,metric_ids)
                    members = set()
                    for metric in metrics:
                        sublist = eval(metric.value_text)
                        members.update(sublist)
                        key_type = False
                        key_area = False
                        if metric.sensor_id.code[5:7] == 'AP':
                            key_type = _('Associated Paid')
                        elif metric.sensor_id.code[5:7] == 'AI':
                            key_type = _('Associated Invoiced')
                        elif metric.sensor_id.code[5:7] == 'AF':
                            key_type = _('Associated Free')
                        elif metric.sensor_id.code[5:7] == 'FP':
                            key_type = _('Full Paid')
                        elif metric.sensor_id.code[5:7] == 'FI':
                            key_type = _('Full Invoiced')
                        elif metric.sensor_id.code[5:7] == 'FF':
                            key_type = _('Full Free')
                        if int(metric.sensor_id.code[8:11]) in area_ids:
                            key_area = dAreas[int(metric.sensor_id.code[8:11])]['name']
                        if key_type and key_area:
                            dDatesTypeAreaSet[(metric_date,key_type,key_area)] = eval(metric.value_text)
                    dDatesMembersSet[metric_date] = members
            dates_with_data.sort(reverse=True)
            index = 0
            while index < (len(dates_with_data)-1):
                new_date = dates_with_data[index]
                old_date = dates_with_data[index+1]
                new_set = dDatesMembersSet[new_date]
                old_set = dDatesMembersSet[old_date]
                diff = new_set.difference(old_set)
                for partner_id in diff:
                    ws5.write(line5,0,partner_id)
                    partner = obj_partner.read(cr,uid,partner_id,['name'])
                    ws5.write(line5,1,partner['name'])
                    ws5.write(line5,2,new_date)
                    last_area = False
                    last_type = False
                    for (key,value_as_set) in dDatesTypeAreaSet.items():
                        if key[0] == new_date and partner_id in value_as_set:
                            last_type = key[1]
                            last_area = key[2]
                    if last_area:
                        ws5.write(line5,3,last_area)
                    if last_type:
                        ws5.write(line5,4,last_type)
                    line5 += 1
                diff_old = old_set.difference(new_set)
                for partner_id in diff_old:
                    ws4.write(line4,0,partner_id)
                    partner = obj_partner.read(cr,uid,partner_id,['name'])
                    ws4.write(line4,1,partner['name'])
                    ws4.write(line4,2,old_date)
                    last_area = False
                    last_type = False
                    for (key,value_as_set) in dDatesTypeAreaSet.items():
                        if key[0] == old_date and partner_id in value_as_set:
                            last_type = key[1]
                            last_area = key[2]
                    if last_area:
                        ws4.write(line4,3,last_area)
                    if last_type:
                        ws4.write(line4,4,last_type)
                    line4 += 1
                index += 1
            wb.save('membership_metrics.xls')
            result_file = open('membership_metrics.xls','rb').read()
            data['form']['msg']='Save the File with '".xls"' extension.'
            data['form']['membership_metrics.xls']=base64.encodestring(result_file)
        else:
            data['form']['msg']='Errors in dates'

        # give the result to the user
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('extract','Extract')]},
        },
        'extract' : {
            'actions': [_extract_metrics],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }
extract_membership_metrics('extract_membership_metrics')
