# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008 CCI Connect ASBL. (http://www.cciconnect.be) All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import time
import datetime
import pooler
from osv import fields, osv
from tools.translate import _

class cci_missions_site(osv.osv):
    _name = 'cci_missions.site'
    _description = 'cci_missions.site'
    _inherit = 'cci_missions.site'
    _columns = {
        'initial' : fields.char('Initial',size=1),
        'active' : fields.boolean('Active'),
    }
    _defaults = {
        'active' : lambda *b : True,
    }
cci_missions_site()

class cci_missions_dossier_type(osv.osv):
    _name = 'cci_missions.dossier_type'
    _description = 'cci_missions.dossier_type'
    _inherit = 'cci_missions.dossier_type'
    _columns = {
        'min_amount_on_create': fields.boolean('Min. on Create',help='Mininum Amount on Create from Certificate'),
        'digital_ref': fields.char('Digital Reference',size=30),
        'active' : fields.boolean('Active'),
    }
    _defaults = {
        'min_amount_on_create': lambda *a: False,
        'active' : lambda *b : True,
    }
cci_missions_dossier_type()

class cci_missions_dossier(osv.osv):
    _name = 'cci_missions.dossier'
    _description = 'cci_missions.dossier'
    _inherit = 'cci_missions.dossier'

    def _amount_subtotal(self, cr, uid, ids, name, args, context=None):
        # rewriting of this method to take taxes into account
        res={}
        tax_obj = self.pool.get('account.tax')
        #prod_obj = self.pool.get('product.product')
        data_dosseir = self.browse(cr,uid,ids)
        for data in data_dosseir:
            sum=0.00
            for product in data.product_ids:
                #product_product = prod_obj.browse(cr,uid,[product.product_id.id])[0]
                #taxes_orig = tax_obj.compute(cr,uid,product_product.taxes_id,product.price_subtotal,1)
                taxes_orig = tax_obj.compute(cr,uid,product.taxes_id,product.price_subtotal,1)
                tax_sum = 0.0
                for tax in taxes_orig:
                    tax_sum += tax['amount']
                sum += ( product.price_subtotal + tax_sum )
            res[data.id]=sum
        return res

    def create(self, cr, uid, vals, *args, **kwargs):
        if not vals.has_key('text_on_invoice') or not vals['text_on_invoice']:
            if vals.has_key('customer_ref') and vals['customer_ref'] and len(vals['customer_ref'].strip()) > 0:
                try:
                    invoice_text = vals['customer_ref'].decode('utf-8')
                except Exception,e:
                    invoice_text = vals['customer_ref']
                if vals.has_key('destination_id') and vals['destination_id']:
                    destination_data = self.pool.get('cci.country').browse(cr,uid,vals['destination_id'])
                    invoice_text += (u' ') # + destination_data.name) # + u' (' + str(vals['quantity_original'])  + u')')
                    #invoice_text += (u' ' + destination_data.name + u' (' + str(vals['quantity_original'])  + u')')
                invoice_text += ' [' + vals['name'] + ']'
                vals.update({'text_on_invoice': invoice_text})
            else:
                invoice_text = vals['name']
                if vals.has_key('destination_id') and vals['destination_id']:
                    destination_data = self.pool.get('cci.country').browse(cr,uid,vals['destination_id'])
                    invoice_text = vals['name'] + ' ' + destination_data.name + ' (' + str(vals['quantity_original'])  + ')'
                vals.update({'text_on_invoice': invoice_text})
        return super(osv.osv,self).create(cr, uid, vals, *args, **kwargs)

    _columns = {
        'sub_total':fields.function(_amount_subtotal, method=True, string='Sub Total for Extra Products', store=True),
        'customer_ref': fields.char('Customer Ref',size=60),
    }
cci_missions_dossier()

class cci_missions_certificate(osv.osv):
    _name = 'cci_missions.certificate'
    _description = 'cci_missions.certificate'
    _inherit = 'cci_missions.certificate'

    def create(self, cr, uid, vals, *args, **kwargs):
        # Overwrite the name fields to set next sequence according to the sequence in the certification type (type_id)
        # this method cumulates code for cci_mission.dossier=>create; cci_mission.certificate.create and new code to handle certificate prefix
        if vals['type_id']:
            data = self.pool.get('cci_missions.dossier_type').browse(cr, uid,vals['type_id'])
            seq = self.pool.get('ir.sequence').get(cr, uid,data.sequence_id.code)
            if seq:
                vals.update({'name': seq})
        # new
        if not vals.has_key('text_on_invoice') or not vals['text_on_invoice']:
            invoice_text = vals['name']
            if vals['destination_id']:
                destination_data = self.pool.get('cci.country').browse(cr,uid,vals['destination_id'])
                invoice_text = vals['name'] + ' ' + destination_data.name
            if not vals.get('digital_number',False) and vals.get('customer_ref',False):
                # vals['customer_ref'].decode('utf-8') permet d'éviter les plantages lors de la créatyion d'un certiicat papier avec une référence client comportant un accent
                invoice_text = vals['customer_ref'].decode('utf-8') + u' ' + invoice_text
            vals.update({'text_on_invoice': invoice_text})
        order_partner = self.pool.get('res.partner').browse(cr,uid,vals['order_partner_id'])
        if order_partner.certificate_prefix:
            if vals['text_on_invoice']:
                if order_partner.certificate_prefix not in vals['text_on_invoice']:
                    vals.update({'text_on_invoice':order_partner.certificate_prefix.strip()+' '+vals['text_on_invoice']})
            else:
                vals.update({'text_on_invoice':order_partner.certificate_prefix.strip()})
        return super(osv.osv,self).create(cr, uid, vals, *args, **kwargs)

    def copy(self, cr, uid, id, default={}, context={}):
        current_certificate = self.browse(cr,uid,id)
        default.update({
            'date': time.strftime('%Y-%m-%d'),
            'goods_value':1.0,
            'invoice_id':False,
            'embassy_folder_id':False,
            'product_ids': False,
            'legalization_ids':False,
            'state':'draft',
            'quantity_copies':0,
            'text_on_invoice':False,
            'invoiced_amount':False,
            'sending_spf':False,
            'internal_comments':False,
            'todo':False,
            'customer_ref':False,
            'exportal':False,
        })
        if current_certificate.type_id.digital:
            default.update({
                'digital_number':'99999999999',
            })
        return super(cci_missions_certificate, self).copy(cr, uid, id, default, context)

    def _amount_total(self, cr, uid, ids, name, args, context=None):
        # rewriting of this method to take taxes into account
        res ={}
        data_dosseir = self.browse(cr,uid,ids)
        tax_obj = self.pool.get('account.tax')
        prod_obj = self.pool.get('product.product')
        for data in data_dosseir:
            if data.state =='draft':
                data_partner = self.pool.get('res.partner').browse(cr,uid,data.order_partner_id.id)
                context.update({'partner_id':data_partner.id})
                context.update({'force_member':False})
                context.update({'force_non_member':False})
                context.update({'date':data.date})
                context.update({'value_goods':data.goods_value})
                context.update({'pricelist':data_partner.property_product_pricelist.id})
                orig_product = prod_obj.browse(cr,uid,[data.type_id.original_product_id.id])[0]
                copy_product = prod_obj.browse(cr,uid,[data.type_id.copy_product_id.id])[0]
                price_orig = self.pool.get('product.product')._product_price(cr, uid, [data.type_id.original_product_id.id], False, False, context)
                price_copy = self.pool.get('product.product')._product_price(cr, uid, [data.type_id.copy_product_id.id], False, False, context)
                cost_orig=price_orig[data.type_id.original_product_id.id]
                cost_copy=price_copy[data.type_id.copy_product_id.id]
                qty_orig = data.quantity_original
                qty_copy = data.quantity_copies
                taxes_orig = tax_obj.compute(cr,uid,orig_product.taxes_id,cost_orig,qty_orig)
                tax_orig = 0.0
                for tax in taxes_orig:
                    tax_orig += tax['amount']
                taxes_copy = tax_obj.compute(cr,uid,copy_product.taxes_id,cost_copy,qty_copy)
                tax_copy = 0.0
                for tax in taxes_copy:
                    tax_copy += tax['amount']
                subtotal = data.sub_total
                if qty_orig < 0 or qty_copy < 0:
                    raise osv.except_osv(_('Input Error!'),_('No. of Copies and Quantity of Originals should be positive.'))
                total = ((cost_orig * qty_orig ) + tax_orig + (cost_copy * qty_copy) + tax_copy + subtotal)
                res[data.id] = total
            else :
                res[data.id]=data.invoiced_amount
        return res

    def single_digital_no(self,cr, uid, ids):
        for data in self.browse(cr, uid, ids):
            if data.type_id.digital and data.digital_number:
                searched_ids = self.search(cr,uid,[('digital_number','=',data.digital_number),('state','in',('invoiced','draft'))])
                if len(searched_ids) > 1:
                    return False
                elif len(searched_ids) == 1 and searched_ids[0] != data.id:
                    return False
        return True

    def max_copies(self,cr, uid, ids):
        for data in self.browse(cr, uid, ids):
            if data.quantity_copies > 99:
                return False
        return True

    def min_value(self,cr, uid, ids):
        for data in self.browse(cr, uid, ids):
            if data.goods_value < 0.01:
                return False
        return True

    _columns = {
        'total':fields.function(_amount_total, method=True, string='Total', store=True),# sum of the price for copies, originals and extra_products, taking taxes into account
        'internal_comments' : fields.text('Internal Comments'),
        'todo': fields.char('ToDo',size=60),
        'exportal':fields.boolean('Exportal'),
    }
    _defaults = {
        'exportal': lambda *a: False,
    }
    _constraints = [(single_digital_no, _('Already used digital number !'), ['digital_number']),
                    (max_copies, _('Maximum number of copies (99) exceeded !'), ['quantity_copies']),
                    (min_value, _('Goods Value must be stricly positive !'), ['goods_value'])]
cci_missions_certificate()

class cci_missions_legalization(osv.osv):
    _name = 'cci_missions.legalization'
    _description = 'cci_missions.legalization'
    _inherit = 'cci_missions.legalization'

    def create(self, cr, uid, vals, *args, **kwargs):
        # Overwrite the name fields to set next sequence according to the sequence in the certification type (type_id)
        # this method cumulates code for cci_mission.dossier=>create; cci_mission.certificate.create and new code to handle certificate prefix
        if vals['type_id']:
            data = self.pool.get('cci_missions.dossier_type').browse(cr, uid,vals['type_id'])
            seq = self.pool.get('ir.sequence').get(cr, uid,data.sequence_id.code)
            if seq:
                vals.update({'name': seq})
        # new
        if not vals.has_key('text_on_invoice') or not vals['text_on_invoice']:
            if vals.has_key('customer_ref') and vals['customer_ref'] and len(vals['customer_ref'].strip()) > 0:
                try:
                    invoice_text = vals['customer_ref'].decode('utf-8')
                except Exception,e:
                    invoice_text = vals['customer_ref']
                invoice_text += ' [' + vals['name'] + ']'
                vals.update({'text_on_invoice': invoice_text})
            else:
                invoice_text = vals['name']
                if vals.has_key('destination_id') and vals['destination_id']:
                    destination_data = self.pool.get('cci.country').browse(cr,uid,vals['destination_id'])
                    invoice_text = vals['name'] + ' ' + destination_data.name + ' (' + str(vals['quantity_original'])  + ')'
                vals.update({'text_on_invoice': invoice_text})
        order_partner = self.pool.get('res.partner').browse(cr,uid,vals['order_partner_id'])
        if order_partner.certificate_prefix:
            if vals['text_on_invoice']:
                if order_partner.certificate_prefix not in vals['text_on_invoice']:
                    vals.update({'text_on_invoice':order_partner.certificate_prefix.strip()+' '+vals['text_on_invoice']})
            else:
                vals.update({'text_on_invoice':order_partner.certificate_prefix.strip()})
        return super(osv.osv,self).create(cr, uid, vals, *args, **kwargs)

    def copy(self, cr, uid, id, default={}, context={}):
        default.update({
            'date': time.strftime('%Y-%m-%d'),
            'goods_value':1.0,
            'invoice_id':False,
            'embassy_folder_id':False,
            'product_ids': False,
            'certificate_id':False,
            'state':'draft',
            'quantity_original':1,
            'quantity_copies':0,
            'text_on_invoice':False,
            'invoiced_amount':False,
            'internal_comments':False,
        })
        return super(cci_missions_legalization, self).copy(cr, uid, id, default, context)

    def max_originals(self,cr, uid, ids):
        for data in self.browse(cr, uid, ids):
            if data.quantity_original > 99:
                return False
        return True

    def max_copies(self,cr, uid, ids):
        for data in self.browse(cr, uid, ids):
            if data.quantity_copies > 99:
                return False
        return True

    def _amount_total(self, cr, uid, ids, name, args, context=None):
        # rewriting of this method to take taxes into account
        res ={}
        visas = self.browse(cr,uid,ids)
        tax_obj = self.pool.get('account.tax')
        prod_obj = self.pool.get('product.product')
        for data in visas:
            if data.state =='draft':
                data_partner = self.pool.get('res.partner').browse(cr,uid,data.order_partner_id.id)

                force_member=force_non_member=False
                if data.member_price==1:
                    force_member=True
                else:
                    force_non_member=True
                context.update({'partner_id':data_partner.id})
                context.update({'force_member':force_member})
                context.update({'force_non_member':force_non_member})
                context.update({'date':data.date})
                context.update({'value_goods':data.goods_value})
                context.update({'pricelist':data_partner.property_product_pricelist.id})
                orig_product = prod_obj.browse(cr,uid,[data.type_id.original_product_id.id])[0]
                copy_product = prod_obj.browse(cr,uid,[data.type_id.copy_product_id.id])[0]
                price_orig = self.pool.get('product.product')._product_price(cr, uid, [data.type_id.original_product_id.id], False, False, context)
                price_copy = self.pool.get('product.product')._product_price(cr, uid, [data.type_id.copy_product_id.id], False, False, context)
                cost_orig=price_orig[data.type_id.original_product_id.id]
                cost_copy=price_copy[data.type_id.copy_product_id.id]
                qty_orig = data.quantity_original
                qty_copy = data.quantity_copies
                taxes_orig = tax_obj.compute(cr,uid,orig_product.taxes_id,cost_orig,qty_orig)
                tax_orig = 0.0
                for tax in taxes_orig:
                    tax_orig += tax['amount']
                taxes_copy = tax_obj.compute(cr,uid,copy_product.taxes_id,cost_copy,qty_copy)
                tax_copy = 0.0
                for tax in taxes_copy:
                    tax_copy += tax['amount']
                subtotal = data.sub_total
                if qty_orig < 0 or qty_copy < 0:
                    raise osv.except_osv(_('Input Error!'),_('No. of Copies and Quantity of Originals should be positive.'))
                total = ((cost_orig * qty_orig ) + tax_orig + (cost_copy * qty_copy) + tax_copy + subtotal)
                res[data.id] = total
            else :
                res[data.id]=data.invoiced_amount
        return res

    _columns = {
        'total':fields.function(_amount_total, method=True, string='Total', store=True),# sum of the price for copies, originals and extra_products with VAT
        'internal_comments' : fields.text('Internal Comments'),
        'digital_number': fields.char('Digital Number', size=12),
    }
    _constraints = [(max_originals, _('Maximum number of originals (99) exceeded !'), ['quantity_original']),
                    (max_copies, _('Maximum number of copies (99) exceeded !'), ['quantity_copies'])]
cci_missions_legalization()

class cci_missions_embassy_folder(osv.osv):
    _name = 'cci_missions.embassy_folder'
    _description = 'cci_missions.embassy_folder'
    _inherit = 'cci_missions.embassy_folder'

    def copy(self, cr, uid, id, default={}, context={}):
        default.update({
            'date': time.strftime('%Y-%m-%d'),
            'invoice_id':False,
            'state':'draft',
            'customer_reference':False,
            'link_ids':False,
            'log_ids':False,
            'internal_note':False,
            'embassy_folder_line_ids':False,
            'invoice_date':False,
            'send_date':False,
            'return_date':False,
            'name':'/',
        })
        return super(cci_missions_embassy_folder, self).copy(cr, uid, id, default, context)

    def _invoice_total(self, cr, uid, ids, name, args, context=None):
        res ={}
        embassies = self.browse(cr,uid,ids)
        tax_obj = self.pool.get('account.tax')
        #prod_obj = self.pool.get('product.product')
        for data in embassies:
            if data.embassy_folder_line_ids:
                total = 0
                for line in data.embassy_folder_line_ids:
                    total += line.customer_amount
                    taxes = tax_obj.compute(cr,uid,[line.tax_rate],line.customer_amount,1)
                    for tax in taxes:
                        total += tax['amount']
                res[data.id] = total
            else :
                res[data.id]=0.0
        return res
    def _margin_total(self, cr, uid, ids, name, args, context=None):
        res ={}
        embassies = self.browse(cr,uid,ids)
        tax_obj = self.pool.get('account.tax')
        #prod_obj = self.pool.get('product.product')
        for data in embassies:
            if data.embassy_folder_line_ids:
                total = 0
                for line in data.embassy_folder_line_ids:
                    total += (line.customer_amount - line.courier_cost)
                res[data.id] = total
            else :
                res[data.id]=0.0
        return res

    _columns = {
        'send_date' : fields.date('Sending Date'), # not visible field on form, but on list
        'return_date' : fields.date('Return Date'), # not visible field on form, but on list
        'contact_id': fields.many2one('res.partner.contact','Contact'),
        'invoice_total':fields.function(_invoice_total, method=True, string='Total Invoice', store=True),# calculation of the total invoiced to the customer with VAT
        'margin_total':fields.function(_margin_total, method=True, string='Margin Total', store=True),# calculation of the margin for the CCI, without VAT
    }

    def _cci_mission_send(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state':'pending','send_date':time.strftime("%Y-%m-%d %H:%M:%S")})
        cases = self.browse(cr, uid, ids)
        self._history(cr, uid, cases, 'Send', history=True,)
        return True

    def _cci_mission_got_back(self,cr,uid,ids,*args):
        self.write(cr, uid, ids, {'state':'open','return_date':time.strftime("%Y-%m-%d %H:%M:%S")})
        cases = self.browse(cr, uid, ids)
        self._history(cr, uid, cases, _('Got Back'), history=True)
        for id in self.browse(cr, uid, ids):
            data = {}
            obj_folder_line = self.pool.get('cci_missions.embassy_folder_line')
            temp = obj_folder_line.search(cr, uid, [('folder_id','=',id.id),('type','=','Translation')])
            if temp:
                translation_line = obj_folder_line.browse(cr, uid, [temp[0]])[0]
                if translation_line.awex_eligible and id.partner_id.awex_eligible == 'yes':
                    #look for an existing credit line in the current time
                    credit_line = self.pool.get('credit.line').search(cr, uid, [('from_date','<=',time.strftime('%Y-%m-%d')), ('to_date', '>=', time.strftime('%Y-%m-%d'))])
                    if credit_line:
                        #if there is one available: get available amount from it
                        amount = self.pool.get('credit.line').get_available_amount(cr, uid, credit_line[0], translation_line.customer_amount, id.partner_id.id)
                        if amount > 0:
                            data['awex_amount'] = amount
                            data['credit_line_id'] =  credit_line[0]
                        else:
                            data['awex_eligible'] = False
                    obj_folder_line.write(cr, uid, [translation_line.id], data)
        return True

    def onchange_partner_id(self, cr, uid, ids, part):
        warning = False
        if not part:
            return {'value':{'partner_address_id':False}}
        part_obj = self.pool.get('res.partner')
        data_partner = part_obj.browse(cr,uid,part)
        if data_partner.alert_legalisations:
                warning = {
                    'title': _("Warning:"),
                    'message': data_partner.alert_explanation or _('Partner is not valid')
                        }
        #addr = part_obj.address_get(cr, uid, [part], ['contact'])
        #data = {'partner_address_id':addr['contact']}
        data = {'member_price':( data_partner['membership_state'] in ['paid','invoiced','free'])}
        if warning:
            return {'value':data, 'warning': warning}
        return {'value':data}
cci_missions_embassy_folder()

class cci_missions_embassy_folder_line(osv.osv):
    _name = 'cci_missions.embassy_folder_line'
    _description = 'cci_missions.embassy_folder_line '
    _inherit = 'cci_missions.embassy_folder_line'

    def create(self, cr, uid, vals, *args, **kwargs):
        t_name = ''
        if vals['folder_id']:
            site_id = self.pool.get('cci_missions.embassy_folder').browse(cr, uid, vals['folder_id']).site_id
            t_name = site_id.initial
        prod_name= 'AMB' + vals['type'] + t_name
        cr.execute('select id from product_product where default_code='"'%s'"''%str(prod_name))
        prod=cr.fetchone()

        if prod:
            product_id=prod[0]
            prod_info = self.pool.get('product.product').browse(cr, uid,product_id)
            account =  prod_info.product_tmpl_id.property_account_income.id
            if not account:
                account = prod_info.categ_id.property_account_income_categ.id
            vals['account_id']=account
            vals['product_id']=product_id
        #print "create"
        #print vals['courier_cost']
        #print vals['customer_amount']
        #if vals['courier_cost'] and ( vals['customer_amount'] and vals['customer_amount'] < 0.001 ) or not vals['customer_amount']:
        #    vals['customer_amount'] = vals['courier_cost']
        #print vals
        return super(osv.osv,self).create(cr, uid, vals, *args, **kwargs)

    def write(self, cr, uid, ids, vals, *args, **kwargs):
        folder_line_id = self.pool.get('cci_missions.embassy_folder_line').browse(cr, uid, ids)
        site_id = folder_line_id and folder_line_id[0].folder_id and folder_line_id[0].folder_id.site_id or False
        if vals.has_key('type'):
            prod_name = 'AMB' + vals['type'] + (site_id and site_id.initial or '')
            cr.execute('select id from product_product where default_code='"'%s'"''%str(prod_name))
            prod=cr.fetchone()
            if prod:
                product_id=prod[0]
                prod_info = self.pool.get('product.product').browse(cr, uid,product_id)
                account =  prod_info.product_tmpl_id.property_account_income.id
                if not account:
                    account = prod_info.categ_id.property_account_income_categ.id
                vals['account_id']=account
                vals['product_id']=product_id
        #print "create"
        #print vals['courier_cost']
        #print vals['customer_amount']
        #if vals['courier_cost'] and ( vals['customer_amount'] and vals['customer_amount'] < 0.001 ) or not vals['customer_amount']:
        #    vals['customer_amount'] = vals['courier_cost']
        #print vals
        return super(osv.osv,self).write( cr, uid, ids,vals, *args, **kwargs)

    def onchange2_line_type(self,cr,uid,ids,type,site_id=False,partner_id=False,member_price=False):
        data={}
        data['courier_cost']=data['customer_amount']=data['account_id']=data['name']=False

        if not type:
            return {'value' : data }

        prod = False
        if site_id:
            site_id = self.pool.get('cci_missions.site').browse(cr, uid, site_id)
            prod_code= 'AMB' + str(type) + site_id.initial
            cr.execute('select pp.id, pt.name, pt.member_price, pt.list_price, pt.standard_price from product_product as pp, product_template as pt where pp.default_code='"'%s'"' AND pp.product_tmpl_id = pt.id'%str(prod_code))
            prod=cr.fetchone()
        if not prod:
            return {'value' : data }

        data['name']=prod[1]
        product_id=prod[0]
        prod_info = self.pool.get('product.product').browse(cr, uid,product_id)
        data['courier_cost']=prod[4] #prod_info.standard_price
        data['customer_amount']=member_price and prod[2] or prod[3] #prod_info.list_price
        data['tax_rate'] = prod_info.taxes_id and prod_info.taxes_id[0].id or False
        account = prod_info.product_tmpl_id.property_account_income.id
        if not account:
            account = prod_info.categ_id.property_account_income_categ.id
        data['account_id']=account
        if partner_id:
            # take fiscal_position into account
            partner = self.pool.get('res.partner').browse(cr,uid, partner_id)
            fiscal_position_id = partner.property_account_position and partner.property_account_position.id or False
            fiscal_position = fiscal_position_id and self.pool.get('account.fiscal.position').browse(cr,uid,fiscal_position_id) or False
            account_id = self.pool.get('account.fiscal.position').map_account(cr,uid,fiscal_position, data['account_id'])
            if account_id:
                data['account_id'] = account_id
            if data['tax_rate']:
                taxes = [self.pool.get('account.tax').browse(cr,uid,data['tax_rate'])]
                tax_id = self.pool.get('account.fiscal.position').map_tax(cr,uid,fiscal_position,taxes)
                if tax_id:
                    data['tax_rate'] = tax_id[0]
            # end fiscal position
        return {'value' : data }

    def onchange_courier_cost(self,cr,uid,ids,courier_cost,customer_amount):
        data={}
        if not customer_amount:
            customer_amount = 0.00
        if courier_cost > 0.001 and customer_amount <= 0.001:
            data['customer_amount'] = courier_cost
        return {'value' : data }

    _columns = {
        'type' : fields.selection([('CBA','CBA'),('MIN','Ministry'),('AMB','Embassy Consulate'),('TRAD','Translation'),('ADM','Administrative'),('TRANS','Travel Costs'),('POST','Postal Costs'),('OTHER','Others')],'Type',required=True),
    }
cci_missions_embassy_folder_line()

class cci_missions_ata_carnet(osv.osv):
    _name = 'cci_missions.ata_carnet'
    _description = 'cci_missions.ata_carnet'
    _inherit = 'cci_missions.ata_carnet'
    #def check_ata_carnet(self,cr, uid, ids):
    #    data_carnet=self.browse(cr, uid, ids)
    #    for data in data_carnet:
    #        if (data.own_risk) or (data.insurer_agreement > 9999999999990L) or (data.insurer_agreement > 0 and data.insurer_agreement < 9999999999990L and data.partner_id.insurer_id > 0):
    #            return True
    #    return False
    def check_goods_value(self,cr, uid, ids):
        data_carnet=self.browse(cr, uid, ids)
        for data in data_carnet:
            if (data.goods_value) < 0.01:
                return False
        return True
    def copy(self, cr, uid, id, default={}, context={}):
        default.update({
            'date':time.strftime('%Y-%m-%d'),
            'creation_date':time.strftime('%Y-%m-%d'),
            'goods_value':1.0,
            'invoice_id':False,
            'state':'draft',
            'duplicata':False,
            'duplicata_details':False,
            'warranty_back_account':False,
            'warranty_back_date':False,
            'insurer_agreement':False,
            'own_risk':True,
            'double_signature':False,
            'initial_pages':0,
            'additional_pages':0,
            'warranty':False,
            'return_date':False,
            'ok_state_date':False,
            'federation_sending_date':False,
            'name':False,
            'product_ids':False,
            'letter_ids':False,
            'internal_comments':False,
        })
        return super(cci_missions_ata_carnet, self).copy(cr, uid, id, default, context)
    def _tot_products(self, cr, uid, ids, name, args, context=None):
        # rewriting of this method to take taxes into account
        res={}
        tax_obj = self.pool.get('account.tax')
        carnets = self.browse(cr,uid,ids)
        for data in carnets:
            sum=0.00
            for product in data.product_ids:
                taxes_orig = tax_obj.compute(cr,uid,product.taxes_id,product.price_subtotal,1)
                tax_sum = 0.0
                for tax in taxes_orig:
                    tax_sum += tax['amount']
                sum += ( product.price_subtotal + tax_sum )
            res[data.id]=sum
        return res
    def _amount_total(self, cr, uid, ids, name, args, context=None):
        res ={}
        tax_obj = self.pool.get('account.tax')
        prod_obj = self.pool.get('product.product')
        carnets = self.browse(cr,uid,ids)
        for data in carnets:
            prod_list = []
            value = []
            final_taxes = []
            taxes_ids = []
            prod_list.append(data.type_id.original_product_id.id)
            taxes_ids.append(data.type_id.original_product_id.taxes_id)
            prod_list.append(data.type_id.copy_product_id.id)
            taxes_ids.append(data.type_id.copy_product_id.taxes_id)
            if data.own_risk:
                prod_list.append(data.type_id.warranty_product_1.id)
                taxes_ids.append(data.type_id.warranty_product_1.taxes_id)
            else:
                prod_list.append(data.type_id.warranty_product_2.id)
                taxes_ids.append(data.type_id.warranty_product_2.taxes_id)
            context.update({'date':data.creation_date})
            count=0
            qty_copy=data.initial_pages
            if qty_copy<0:
               qty_copy=0
            force_member = data.member_price
            force_non_member = not data.member_price
            if force_member:
                ptype = 'member_price'
            else:
                ptype = 'list_price'
            context.update({'partner_id':False})
            context.update({'force_member':force_member})
            context.update({'force_non_member':force_non_member})
            context.update({'value_goods':data.goods_value})
            context.update({'double_signature':data.double_signature})
            context.update({'date':data.creation_date})
            context.update({'emission_date':data.creation_date})
            index = 0
            for prod_id in prod_list:
                count +=1
                price=self.pool.get('product.product').price_get(cr, uid, [prod_id], ptype, context)
                value.append(price[prod_id])
                taxes = tax_obj.compute(cr,uid,taxes_ids[index],price[prod_id],index==1 and qty_copy or 1)
                tax_sum = 0.0
                for tax in taxes:
                    tax_sum += tax['amount']
                final_taxes.append(tax_sum)
                index += 1
            sum = value[0]+round(value[1]*qty_copy,2)+final_taxes[0]+final_taxes[1]
            sum += data.warranty + final_taxes[2] + data.sub_total
            res[data.id] = sum
        return res
    _columns = {
        'sub_total': fields.function(_tot_products, method=True, string='Subtotal of Extra Products',type="float", store=True),
        'total':fields.function(_amount_total, method=True, string='Total', store=True),# calculation of the price of the product and extra_products
        'duplicata' : fields.boolean('Duplicata'), # indicates if this carnet has a duplicate created
        'duplicata_details' : fields.char('Duplicata details',size=50), # give the reason for duplication
        'warranty_back_account': fields.char('Account for Cash Back',size=32),
        'warranty_back_date': fields.date('Date of Cash Back'),
        'internal_comments': fields.text('Internal comments'),
        'digital_number': fields.char('Digital Number',size=20),
        'cci_country_ids': fields.many2many('cci.country','ata_carnet_cci_country_rel','ata_carnet_id','cci_country_id','Countries'),
    }
    _defaults = {
        'duplicata' : lambda *b : False,
    }
    _constraints = [
            #(check_ata_carnet, _('Error: Please Select (Own Risk) OR (Insurer Agreement = 9999999999999) OR ("Insurer Agreement" and "Partners Insure id" should be greater than Zero)'), ['own_risk','insurer_agreement','partner_insurer_id']),
            (check_goods_value, _('Error: The Value of the Goods must be greater than zero.'), ['goods_value',]),
    ]
cci_missions_ata_carnet()

class cci_missions_ata_usage(osv.osv):
    _name = 'cci_missions.ata_usage'
    _description = 'cci_missions.ata_usage'
    _inherit = 'cci_missions.ata_usage'
    _columns = {
        'digital_name': fields.char('Digital Name',size=50),
        'active' : fields.boolean('Active'),
    }
    _defaults = {
        'active' : lambda *b : True,
    }
cci_missions_ata_usage()

class cci_missions_operation(osv.osv):
    _name = 'cci_missions.operation'
    _description = 'cci_missions.operation'
    _columns = {
        'type': fields.char('Type',size=40),
        'name' : fields.char('Name',size=50),
        'amount': fields.float('Amount', digits=(16,2)),

    }
cci_missions_operation()

class cci_country(osv.osv):
    _name = "cci.country"
    _description = "Country or Area for CCI"
    _inherit = "cci.country"
    _columns = {
        'isocode3' : fields.char('ISO Code 3',size=3,help="ISO Code in 3 characters for countries and only countries"),
    }
cci_country()

class cci_missions_document(osv.osv):
    _name = "cci_missions.document"
    _description = "Legal document for proof of validity of certificates"
    _columns = {
        'partner_id': fields.many2one('res.partner','Partner'),
        'type': fields.selection([('subsidiary','Subsidiary'),('other','Contract')],'Type'),
        'name': fields.char('Name',size=100,required=True),
        'date': fields.date('Emission date',required=True),
        'active': fields.boolean('Active'),
        'filename': fields.binary('File'),
    }
    _defaults = {
        'type': lambda *a : 'subsidiary',
        'date' : lambda *a : datetime.date.today().strftime('%Y-%m-%d'),
        'active': lambda *a : True,
    }
cci_missions_document()

class res_partner(osv.osv):
    _inherit = 'res.partner'
    _description = 'res.partner'

    _columns = {
        'certificate_prefix': fields.char('Prefix on Certificates',size=30),
        'mission_doc_ids': fields.one2many('cci_missions.document','partner_id','Proofs for Certificates'),
    }
res_partner()

