# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import datetime
import wizard
import pooler
from tools.translate import _
from pyExcelerator import *
import base64

form = """<?xml version="1.0"?>
<form string="Extract Customers'Contacts (L, C, G)">
    <field name="date_from"/>
    <field name="date_to"/>
    <newline/>
    <separator string="Customer's category" colspan="4"/>
    <field name="cust_cert_classic" colspan="4"/>
    <field name="cust_cert_digital" colspan="4"/>
    <field name="cust_visa" colspan="4"/>
    <field name="cust_amb" colspan="4"/>
    <field name="cust_ata" colspan="4"/>
    <field name="cust_trad" colspan="4"/>
</form>"""

fields = {
      'date_from': {'string':'Between', 'type':'date', 'required':True},
      'date_to': {'string':'and', 'type':'date', 'required':True},
      'cust_cert_classic': {'string':'Classical Certificates','type':'boolean','default':True},
      'cust_cert_digital': {'string':'Digital Certificates','type':'boolean','default':True},
      'cust_visa': {'string':'Visas','type':'boolean','default':True},
      'cust_amb': {'string':'Embassies','type':'boolean','default':True},
      'cust_ata': {'string':'ATA Carnets','type':'boolean','default':True},
      'cust_trad': {'string':'Translations','type':'boolean','default':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Result">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="customer_contacts.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'Files created', 'type':'text', 'size':'100','readonly':True},
    'customer_contacts.xls':{'string': 'Fichier','type': 'binary','readonly': True,},
}

def _get_partner_name(partner_name,address_name):
    if address_name:
        if address_name.strip()[0:2] == '- ':
            result = partner_name.strip() + ' ' + address_name.strip()
        else:
            result = address_name.strip()
    else:
        result = partner_name.strip()
    return result
    
def _get_street(street1,street2):
    if street1:
        if street2:
            result = street1.strip() + ' - ' + street2.strip()
        else:
            result = street1.strip()
    else:
        result = ''
    return result

def _get_phone(phone):
    if phone:
        phone = phone.strip()
        if len(phone) > 0:
            if len(phone) == 9:
                if phone[0:2] in ['02','03','04','09']:
                    result = phone[0:2] + "/" + phone[2:5] + "." + phone[5:7] + "." + phone[7:]
                else:
                    result = phone[0:3] + "/" + phone[3:5] + "." + phone[5:7] + "." + phone[7:]
            elif len(phone) == 10:
                result = phone[0:4] + "/" + phone[4:6] + "." + phone[6:8] + "." + phone[8:]
            else:
                result = phone
        else:
            result = ''
    else:
        result = ''
    return result
    
class wizard_extract_customer_contacts(wizard.interface):
    def _extract(self, cr, uid, data, context):
        partner_ids = []
        cert_classic_partner_ids = []
        cert_digital_partner_ids = []
        visa_partner_ids = []
        amb_partner_ids = []
        ata_partner_ids = []
        trad_partner_ids = []
        
        # dates
        date_from = data['form']['date_from']
        date_to = data['form']['date_to']
        if date_to < date_from:
            (dateto,date_from), = (date_from,date_to)

        # selection by types
        if data['form']['cust_cert_classic'] or data['form']['cust_cert_digital']:
            obj_cert = pooler.get_pool(cr.dbname).get('cci_missions.certificate')
            cert_ids = obj_cert.search(cr,uid,[('date','>=',date_from),('date','<=',date_to)])
            certs = obj_cert.browse(cr,uid,cert_ids)
        if data['form']['cust_cert_classic']:
            for cert in certs:
                if not cert.dossier_id.type_id.digital:
                    if cert.dossier_id and cert.dossier_id.order_partner_id and cert.dossier_id.order_partner_id and cert.dossier_id.order_partner_id.id not in cert_classic_partner_ids:
                        cert_classic_partner_ids.append(cert.dossier_id.order_partner_id.id)
                    if cert.dossier_id and cert.dossier_id.order_partner_id and cert.dossier_id.order_partner_id.id not in partner_ids:
                        partner_ids.append(cert.dossier_id.order_partner_id.id)
        if data['form']['cust_cert_digital']:
            for cert in certs:
                if cert.dossier_id.type_id.digital:
                    if cert.dossier_id and cert.dossier_id.order_partner_id and cert.dossier_id.order_partner_id and cert.dossier_id.order_partner_id.id not in cert_digital_partner_ids:
                        cert_digital_partner_ids.append(cert.dossier_id.order_partner_id.id)
                    if cert.dossier_id and cert.dossier_id.order_partner_id and cert.dossier_id.order_partner_id.id not in partner_ids:
                        partner_ids.append(cert.dossier_id.order_partner_id.id)
        if data['form']['cust_visa']:
            obj_visa = pooler.get_pool(cr.dbname).get('cci_missions.legalization')
            visa_ids = obj_visa.search(cr,uid,[('date','>=',date_from),('date','<=',date_to)])
            visas = obj_visa.browse(cr,uid,visa_ids)
            for visa in visas:
                if visa.dossier_id and visa.dossier_id.order_partner_id and visa.dossier_id.order_partner_id and visa.dossier_id.order_partner_id.id not in visa_partner_ids:
                    visa_partner_ids.append(visa.dossier_id.order_partner_id.id)
                if visa.dossier_id and visa.dossier_id.order_partner_id and visa.dossier_id.order_partner_id.id not in partner_ids:
                    partner_ids.append(visa.dossier_id.order_partner_id.id)
        if data['form']['cust_amb']:
            obj_amb = pooler.get_pool(cr.dbname).get('cci_missions.embassy_folder')
            amb_ids = obj_amb.search(cr,uid,[('date','>=',date_from),('date','<=',date_to)])
            ambs = obj_amb.browse(cr,uid,amb_ids)
            for amb in ambs:
                if amb.crm_case_id and amb.crm_case_id.partner_id and amb.crm_case_id.partner_id and amb.crm_case_id.partner_id.id not in amb_partner_ids:
                    amb_partner_ids.append(amb.crm_case_id.partner_id.id)
                if amb.crm_case_id and amb.crm_case_id.partner_id and amb.crm_case_id.partner_id.id not in partner_ids:
                    partner_ids.append(amb.crm_case_id.partner_id.id)
        if data['form']['cust_ata']:
            obj_ata = pooler.get_pool(cr.dbname).get('cci_missions.ata_carnet')
            ata_ids = obj_ata.search(cr,uid,[('creation_date','>=',date_from),('creation_date','<=',date_to)])
            atas = obj_ata.browse(cr,uid,ata_ids)
            for ata in atas:
                if ata.partner_id and ata.partner_id.id not in ata_partner_ids:
                    ata_partner_ids.append(ata.partner_id.id)
                if ata.partner_id and ata.partner_id.id not in partner_ids:
                    partner_ids.append(ata.partner_id.id)
        if data['form']['cust_trad']:
            obj_trad = pooler.get_pool(cr.dbname).get('translation.folder')
            trad_ids = obj_trad.search(cr,uid,[('order_date','>=',date_from),('order_date','<=',date_to)])
            trads = obj_trad.browse(cr,uid,trad_ids)
            for trad in trads:
                if trad.partner_id and trad.partner_id.id not in trad_partner_ids:
                    trad_partner_ids.append(trad.partner_id.id)
                if trad.partner_id and trad.partner_id.id not in partner_ids:
                    partner_ids.append(trad.partner_id.id)        
        if partner_ids:
            wb1 = Workbook()
            ws1 = wb1.add_sheet('Customers')
            ws1.write(0,0,'partner_id')
            ws1.write(0,1,'contact_id')
            ws1.write(0,2,_('Partner Name'))
            ws1.write(0,3,_('Street'))
            ws1.write(0,4,_('Zip Code'))
            ws1.write(0,5,_('Locality'))
            ws1.write(0,6,_('Address Phone'))
            ws1.write(0,7,_('Address Fax'))
            ws1.write(0,8,_('Courtesy'))
            ws1.write(0,9,_('Name'))
            ws1.write(0,10,_('First Name'))
            ws1.write(0,11,_('Professional Phone'))
            ws1.write(0,12,_('Mobile'))
            ws1.write(0,13,_('EMail'))
            ws1.write(0,14,_('Title'))
            ws1.write(0,15,_('Codes'))
            ws1.write(0,16,_('Classical Certificates Customer'))
            ws1.write(0,17,_('Digital Certificates Customer'))
            ws1.write(0,18,_('Visas Customer'))
            ws1.write(0,19,_('Embassies Customer'))
            ws1.write(0,20,_('ATA Customer'))
            ws1.write(0,21,_('Translations Customer'))
            ws1.write(0,22,u'Membership')
            line = 1
            # Get each partner in excel file
            # 1. first get all addresse with at least 'L' et get each 'L'
            # 2. If an address  has no 'L', searche for the first 'C', else for the first 'G'
            obj_partner = pooler.get_pool(cr.dbname).get('res.partner')
            partners = obj_partner.browse(cr,uid,partner_ids)
            for partner in partners:
                if partner.state_id and partner.state_id.id == 1:
                    for addr in partner.address:
                        # 'L' present
                        special_found = False
                        for job in addr.job_ids:
                            if job.function_code_label and 'L' in job.function_code_label and job.active and job.contact_id and job.contact_id.active:
                                special_found = True
                                ws1.write(line,0,partner.id)
                                ws1.write(line,1,job.contact_id.id)
                                ws1.write(line,2,_get_partner_name(partner.name,addr.name))
                                ws1.write(line,3,_get_street(addr.street,addr.street2))
                                ws1.write(line,4,addr.zip_id and addr.zip_id.name or '')
                                ws1.write(line,5,addr.zip_id and addr.zip_id.city or '')
                                ws1.write(line,6,_get_phone(addr.phone))
                                ws1.write(line,7,_get_phone(addr.fax))
                                ws1.write(line,8,job.contact_id.title or '')
                                ws1.write(line,9,job.contact_id.name or '')
                                ws1.write(line,10,job.contact_id.first_name or '')
                                ws1.write(line,11,_get_phone(job.phone))
                                ws1.write(line,12,_get_phone(job.contact_id.mobile))
                                ws1.write(line,13,job.email or job.contact_id.email or '')
                                ws1.write(line,14,job.function_label or '')
                                ws1.write(line,15,job.function_code_label or '')
                                ws1.write(line,16,(partner.id in cert_classic_partner_ids) and 'OUI' or '')
                                ws1.write(line,17,(partner.id in cert_digital_partner_ids) and 'OUI' or '')
                                ws1.write(line,18,(partner.id in visa_partner_ids) and 'OUI' or '')
                                ws1.write(line,19,(partner.id in amb_partner_ids) and 'OUI' or '')
                                ws1.write(line,20,(partner.id in ata_partner_ids) and 'OUI' or '')
                                ws1.write(line,21,(partner.id in trad_partner_ids) and 'OUI' or '')
                                if partner.membership_state and partner.membership_state in ['paid','invoiced','free']:
                                    ws1.write(line,22,u'OUI')
                                else:
                                    ws1.write(line,22,u'Non')
                                line += 1
                        if not special_found:
                            current_job = False
                            current_contact = False
                            seq_minimum = 99999
                            for job in addr.job_ids:
                                if job.function_code_label and 'C' in job.function_code_label and job.active and job.contact_id and job.contact_id.active and job.sequence_partner and job.sequence_partner < seq_minimum:
                                    current_job = job
                                    current_contact = job.contact_id
                                    seq_minimum = job.sequence_partner
                                    special_found = True
                            if special_found:
                                ws1.write(line,0,partner.id)
                                ws1.write(line,1,current_job.contact_id.id)
                                ws1.write(line,2,_get_partner_name(partner.name,addr.name))
                                ws1.write(line,3,_get_street(addr.street,addr.street2))
                                ws1.write(line,4,addr.zip_id and addr.zip_id.name or '')
                                ws1.write(line,5,addr.zip_id and addr.zip_id.city or '')
                                ws1.write(line,6,_get_phone(addr.phone))
                                ws1.write(line,7,_get_phone(addr.fax))
                                ws1.write(line,8,current_contact.title or '')
                                ws1.write(line,9,current_contact.name or '')
                                ws1.write(line,10,current_contact.first_name or '')
                                ws1.write(line,11,_get_phone(current_job.phone))
                                ws1.write(line,12,_get_phone(current_contact.mobile))
                                ws1.write(line,13,current_job.email or current_contact.email or '')
                                ws1.write(line,14,current_job.function_label or '')
                                ws1.write(line,15,current_job.function_code_label or '')
                                ws1.write(line,16,(partner.id in cert_classic_partner_ids) and 'OUI' or '')
                                ws1.write(line,17,(partner.id in cert_digital_partner_ids) and 'OUI' or '')
                                ws1.write(line,18,(partner.id in visa_partner_ids) and 'OUI' or '')
                                ws1.write(line,19,(partner.id in amb_partner_ids) and 'OUI' or '')
                                ws1.write(line,20,(partner.id in ata_partner_ids) and 'OUI' or '')
                                ws1.write(line,21,(partner.id in trad_partner_ids) and 'OUI' or '')
                                if partner.membership_state and partner.membership_state in ['paid','invoiced','free']:
                                    ws1.write(line,22,u'OUI')
                                else:
                                    ws1.write(line,22,u'Non')
                                line += 1
                        if not special_found:
                            current_job = False
                            current_contact = False
                            seq_minimum = 99999
                            for job in addr.job_ids:
                                if job.function_code_label and 'G' in job.function_code_label and job.active and job.contact_id and job.contact_id.active and job.sequence_partner and job.sequence_partner < seq_minimum:
                                    current_job = job
                                    current_contact = job.contact_id
                                    seq_minimum = job.sequence_partner
                                    special_found = True
                            if special_found:
                                ws1.write(line,0,partner.id)
                                ws1.write(line,1,current_job.contact_id.id)
                                ws1.write(line,2,_get_partner_name(partner.name,addr.name))
                                ws1.write(line,3,_get_street(addr.street,addr.street2))
                                ws1.write(line,4,addr.zip_id and addr.zip_id.name or '')
                                ws1.write(line,5,addr.zip_id and addr.zip_id.city or '')
                                ws1.write(line,6,_get_phone(addr.phone))
                                ws1.write(line,7,_get_phone(addr.fax))
                                ws1.write(line,8,current_contact.title or '')
                                ws1.write(line,9,current_contact.name or '')
                                ws1.write(line,10,current_contact.first_name or '')
                                ws1.write(line,11,_get_phone(current_job.phone))
                                ws1.write(line,12,_get_phone(current_contact.mobile))
                                ws1.write(line,13,current_job.email or current_contact.email or '')
                                ws1.write(line,14,current_job.function_label or '')
                                ws1.write(line,15,current_job.function_code_label or '')
                                ws1.write(line,16,(partner.id in cert_classic_partner_ids) and 'OUI' or '')
                                ws1.write(line,17,(partner.id in cert_digital_partner_ids) and 'OUI' or '')
                                ws1.write(line,18,(partner.id in visa_partner_ids) and 'OUI' or '')
                                ws1.write(line,19,(partner.id in amb_partner_ids) and 'OUI' or '')
                                ws1.write(line,20,(partner.id in ata_partner_ids) and 'OUI' or '')
                                ws1.write(line,21,(partner.id in trad_partner_ids) and 'OUI' or '')
                                if partner.membership_state and partner.membership_state in ['paid','invoiced','free']:
                                    ws1.write(line,22,u'OUI')
                                else:
                                    ws1.write(line,22,u'Non')
                                line += 1
            wb1.save('customer_contacts.xls')
            result_file = open('customer_contacts.xls','rb').read()
            # give the result to the user
            data['form']['msg']=_('Save the file with '".xls"' extension.')
            data['form']['customer_contacts.xls']=base64.encodestring(result_file)
        else:
            data['form']['msg'] = _('No customers between these two dates.')
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','_Cancel'),('extract','Extract')]},
        },
        'extract': {
            'actions': [_extract],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }
wizard_extract_customer_contacts('wizard_extract_customer_contacts')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
