# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import datetime
import wizard
import pooler
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Extract Expired Doc">
    <field name="date_max"/>
    <newline/>
    <label string="Maximum date of emission for expired doc (usually a year ago)." colspan="4"/>
</form>"""

fields = {
      'date_max': {'string':'Date max', 'type':'date', 'required':True},
   }

class wizard_extract_expired_doc(wizard.interface):

    found_ids = []
    
    def _get_defaults(self, cr, uid, data, context=None):
        data['form']['date_max'] = (datetime.date.today() - datetime.timedelta(days=366)).strftime('%Y-%m-%d')
        return data['form']
        
    def _extract(self, cr, uid, data, context):
        obj_doc = pooler.get_pool(cr.dbname).get('cci_missions.document')
        self.found_ids = obj_doc.search(cr,uid,[('date','<=',data['form']['date_max'])])
        return data['form']

    def _open_window_results(self, cr, uid, data, context):
        if self.found_ids:
            found_domain = "[('id','in', ['" + "','".join([str(x) for x in self.found_ids]) + "'])]"
        else:
            found_domain = "[('id','=',0)]"
        result = {
            'name': _('Expired Proofs'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'cci_missions.document',
            'domain': found_domain,
            'context': {},
            'type': 'ir.actions.act_window'
        }
        return result
    states = {
        'init': {
            'actions': [_get_defaults],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','_Cancel'),('extract','extract')]},
        },
        'extract': {
            'actions': [_extract],
            'result': {'type': 'action', 'action':_open_window_results, 'state':'end'},
        },
    }
wizard_extract_expired_doc('wizard_extract_expired_doc')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
