# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
# Version 1.1 2011-07-25 Philmer export in two excel files rather than seding a mail with a csv file
#             2013-05-27 Si personne privée, no euler ID; si 'own risk/risque propre' no euler ID
import wizard
import time
import datetime
import tools
import pooler
import base64
from pyExcelerator import *
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Extract Janssen file">
    <field name="partner_id" colspan="4"/>    
    <field name="from"/>
    <field name="to"/>
</form>"""

fields = {
    'partner_id': {'string':'Customer','type':'many2one','relation':'res.partner','required':True},
    'from': {'string':'From', 'type':'date', 'required':True},
    'to': {'string':'To', 'type':'date', 'required':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Result">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="janssen.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'Files created', 'type':'text', 'size':'100','readonly':True},
    'janssen.xls':{'string': 'Fichier','type': 'binary','readonly': True,},
}

class wizard_extract_janssen(wizard.interface):
    def _get_file(self,cr,uid,data,context):
        # first, create the file with all certificates of the month, associated with their visa and embassy folders
        obj_cert = pooler.get_pool(cr.dbname).get('cci_missions.certificate')
        cert_ids = obj_cert.search(cr,uid,[('date_certificate','>=',data['form']['from']),('date_certificate','<=',data['form']['to']),('order_partner_id','=',data['form']['partner_id']),('state','in',['draft','invoiced'])])
        obj_leg = pooler.get_pool(cr.dbname).get('cci_missions.legalization')
        leg_ids = obj_leg.search(cr,uid,[('date_legalization','>=',data['form']['from']),('date_legalization','<=',data['form']['to']),('order_partner_id','=',data['form']['partner_id']),('state','in',['draft','invoiced']),('certificate_id','=',False)])
        if cert_ids or leg_ids:
            wb1 = Workbook()
            ws1 = wb1.add_sheet('')
            ws1.write(0,0,_('From'))
            ws1.write(0,1,data['form']['from'])
            ws1.write(1,0,_('To'))
            ws1.write(1,1,data['form']['to'])
            ws1.write(3,0,_('Date'))
            ws1.write(3,1,_('Customer Ref.'))
            ws1.write(3,2,_('Amount'))
            ws1.write(3,3,_('Destination country'))
            ws1.write(3,4,_('Nbr. linked Visas'))
            ws1.write(3,5,_('Amount Visas'))
            ws1.write(3,6,_('Embassy Folder Ref'))
            ws1.write(3,7,_('Amount Embassy Folder'))
            ws1.write(3,8,_('CCI Ref.'))
            line = 4
            if cert_ids:
                certificates = obj_cert.browse(cr,uid,cert_ids)
                for cert in certificates:
                    ws1.write(line,0,cert.date_certificate)
                    ws1.write(line,1,cert.customer_ref or cert.name)
                    ws1.write(line,2,cert.total)
                    ws1.write(line,3,cert.destination_id and cert.destination_id.name or _('-none-'))
                    ws1.write(line,4,len(cert.legalization_ids))
                    total_visas = 0.0
                    for leg in cert.legalization_ids:
                        total_visas += leg.total
                    ws1.write(line,5,total_visas)
                    if cert.embassy_folder_id and cert.embassy_folder_id.state <> 'cancel':
                        ws1.write(line,6,cert.embassy_folder_id.name)
                        total_embassy = 0.0
                        for emb_line in cert.embassy_folder_id.embassy_folder_line_ids:
                            total_embassy += emb_line.customer_amount
                        ws1.write(line,7,total_embassy)
                    else:
                        ws1.write(line,7,0.0)
                    ws1.write(line,8,cert.name)
                    line += 1
            if leg_ids:
                legs = obj_leg.browse(cr,uid,leg_ids)
                for visa in legs:
                    ws1.write(line,0,visa.date_legalization)
                    ws1.write(line,1,visa.customer_ref or visa.name)
                    ws1.write(line,2,0.0)
                    ws1.write(line,3,visa.destination_id and visa.destination_id.name or _('-none-'))
                    ws1.write(line,4,1)
                    ws1.write(line,5,visa.total)
                    if visa.embassy_folder_id and visa.embassy_folder_id.state <> 'cancel':
                        ws1.write(line,6,visa.embassy_folder_id.name)
                        total_embassy = 0.0
                        for emb_line in visa.embassy_folder_id.embassy_folder_line_ids:
                            total_embassy += emb_line.customer_amount
                        ws1.write(line,7,total_embassy)
                    else:
                        ws1.write(line,7,0.0)
                    ws1.write(line,8,visa.name)
                    line += 1
            wb1.save('janssen.xls')
            result_file = open('janssen.xls','rb').read()

            # give the result to the user
            data['form']['msg']=_('Save the file with '".xls"' extension.')
            data['form']['janssen.xls']=base64.encodestring(result_file)
        else:
            data['form']['msg'] = _('No certificates or legalizations between these two dates for this customer.')
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('getfile','Get Excel File')]},
        },
        'getfile': {
            'actions': [_get_file],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }

wizard_extract_janssen('extract_janssen')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

