# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import datetime
import wizard
import pooler
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Extract ATA Carnet without answer">
    <field name="date_max"/>
    <newline/>
    <label string="Maximum date of sending of second call to action." colspan="4"/>
</form>"""

fields = {
      'date_max': {'string':'Date max', 'type':'date', 'required':True},
   }

class wizard_extract_wo_answer_ata(wizard.interface):

    found_ids = []
    
    def _get_defaults(self, cr, uid, data, context=None):
        data['form']['date_max'] = (datetime.date.today() - datetime.timedelta(days=14)).strftime('%Y-%m-%d')
        return data['form']
        
    def _extract(self, cr, uid, data, context):
        date_max = data['form']['date_max']
        obj_ata = pooler.get_pool(cr.dbname).get('cci_missions.ata_carnet')
        ata_ids = obj_ata.search(cr,uid,[('state','in',['draft','created','pending'])])
        if ata_ids:
            atas = obj_ata.browse(cr,uid,ata_ids)
            for ata in atas:
                if ata.letter_ids:
                    last_rappel = '1980-01-01'
                    for letter in ata.letter_ids:
                        if (letter.letter_type in ['Rappel avant echeance','Rappel apres echeance']) and (letter.date > last_rappel):
                            last_rappel = letter.date
                    if last_rappel > '1980-01-01' and last_rappel <= date_max:
                        self.found_ids.append(ata.id)
        return data['form']

    def _open_window_results(self, cr, uid, data, context):
        if self.found_ids:
            found_domain = "[('id','in', ['" + "','".join([str(x) for x in self.found_ids]) + "'])]"
        else:
            found_domain = "[('id','=',0)]"
        result = {
            'name': _('ATA without Answer'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'cci_missions.ata_carnet',
            'domain': found_domain,
            'context': {},
            'type': 'ir.actions.act_window'
        }
        return result
    states = {
        'init': {
            'actions': [_get_defaults],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','_Cancel'),('extract','extract')]},
        },
        'extract': {
            'actions': [_extract],
            'result': {'type': 'action', 'action':_open_window_results, 'state':'end'},
        },
    }
wizard_extract_wo_answer_ata('wizard_extract_wo_answer_ata')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
