# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
# Version 1.0 Philmer
import wizard
import time
import datetime
import tools
import pooler
import base64
from osv import fields
from osv import osv
from lxml import objectify        
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Import eATAs">
    <field name="type_id" colspan="4" width="600"/>
    <field name="input" colspan="4"/>
</form>"""

fields = {
    'type_id': {'type':'many2one','relation':'cci_missions.dossier_type','domain':"[('section','=','ATA')]",'string':'ATA Type','required':True},
    'input': {'type':'binary','string':'eATA file','required':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="Importation result" colspan="4"/>
     <field name="msg" colspan="4" nolabel="1" width="800" heigth="600"/>
</form>"""

msg_fields = {
    'msg': {'string':'File imported', 'type':'text', 'readonly':True},
}

class wizard_import_digital_atas(wizard.interface):

    new_ids = []
    
    def _import_data(self,cr,uid,data,context):
        inputfile = data['form']['input']
        inputdata = base64.decodestring(inputfile)
        inputdata = inputdata.decode('utf-8') # 2015-03-04 : no more utf-16 but utf-8 as at the beginning
        # store the results
        rejected = 0
        imported = 0
        errors = []
        created_codes = []
        # objects necessary for the import
        parameter_obj = pooler.get_pool(cr.dbname).get('cci_parameters.cci_value')
        parameter = parameter_obj.get_value_from_names(cr,uid,['DigitalChamberNumber'])
        obj_partner = pooler.get_pool(cr.dbname).get('res.partner')
        obj_carnet = pooler.get_pool(cr.dbname).get('cci_missions.ata_carnet')
        obj_type = pooler.get_pool(cr.dbname).get('cci_missions.dossier_type')
        obj_usage = pooler.get_pool(cr.dbname).get('cci_missions.ata_usage')
        obj_country = pooler.get_pool(cr.dbname).get('cci.country')
        # decoding the xml file
        root = objectify.fromstring(inputdata)
        for carnet in root.CARNET:
            internal_id = str(carnet.ID)
            if True:
            #try:
                if len(internal_id) > 0 and not (internal_id[-1:] == 'D'):
                    if str(carnet.ChamberNumber) == parameter['DigitalChamberNumber']:
                        full_customer_number = str(carnet.CustomerNumber)
                        if full_customer_number[0:4] == 'cci_':
                            # searching partner by CustomerNumber
                            partner_ids = obj_partner.search(cr,uid,[('id','=',int(full_customer_number[4:])),('active','=',True)])
                            if len(partner_ids) == 1:
                                partner_id = partner_ids[0]
                                found_partner = obj_partner.read(cr,uid,partner_id,['name','membership_state','insurer_id'])
                                existing_ata_ids = obj_carnet.search(cr,uid,[('digital_number','=',internal_id)]) # the search on digital_number is not enough because wa have a serie of ATA Carnet without digital_number
                                if not existing_ata_ids:
                                    existing_ata_ids = obj_carnet.search(cr,uid,[('name','=',carnet.CarnetNumber or '')]) # so we search also on 'name'
                                    if not existing_ata_ids:
                                        # search for usage_id, of not, create it
                                        if str(carnet.IntendedUseList.IntendedUse) == 'Other':
                                            usage_name = unicode(carnet.IntendedUseList.IntendedUseOther)
                                        else:
                                            usage_name = unicode(carnet.IntendedUseList.IntendedUse)
                                        usage_ids = obj_usage.search(cr,uid,[('digital_name','=',usage_name)])
                                        if usage_ids:
                                            usage_id = usage_ids[0]
                                        else:
                                            usage_id = obj_usage.create(cr,uid,{'name':usage_name,'digital_name':usage_name})
                                        country_ids = []
                                        if hasattr(carnet.DestinationList,'Destination'):
                                            for destination in carnet.DestinationList.Destination:
                                                isocode3 = str(destination)
                                                if len(isocode3) == 3:
                                                    search_country_ids = obj_country.search(cr,uid,[('isocode3','=',isocode3)])
                                                    if len(search_country_ids) == 1:
                                                        country_ids.append(search_country_ids[0])
                                        # extraction of countries
                                        # extract fields
                                        ata_data = {}
                                        ata_data['name'] = carnet.CarnetNumber or False
                                        ata_data['digital_number'] = internal_id
                                        ata_data['type_id'] = data['form']['type_id']
                                        ata_data['partner_id'] = partner_id
                                        ata_data['partner_insurer_id'] = found_partner['insurer_id']
                                        ata_data['creation_date'] = str(carnet.AcceptedDateTime)[6:10]+"-"+str(carnet.AcceptedDateTime)[3:5]+"-"+str(carnet.AcceptedDateTime)[0:2]
                                        ata_data['validity_date'] = str(carnet.ExpiryDateTime)[6:10]+'-'+str(carnet.ExpiryDateTime)[3:5]+'-'+str(carnet.ExpiryDateTime)[0:2]
                                        ata_data['holder_name'] = unicode(carnet.Holder.Applicant)
                                        ata_data['holder_address'] = ''
                                        ata_data['holder_city'] = ''
                                        ata_data['representer_name'] = ''
                                        ata_data['representer_address'] = ''
                                        ata_data['representer_city'] = ''
                                        ata_data['usage_id'] = usage_id
                                        ata_data['area_id'] = obj_country.search(cr,uid,[('code','=','ALL')])[0]
                                        ata_data['cci_country_ids'] = [(6,0,country_ids)]
                                        if hasattr(carnet.Deposit,'EulerID'):
                                            if carnet.Deposit.EulerID and int(carnet.Deposit.EulerID) > 0 and not found_partner['insurer_id']:
                                                partner_data = {}
                                                partner_data['insurer_id'] = int(carnet.Deposit.EulerID)
                                                obj_partner.write(cr,uid,[partner_id,],partner_data)
                                        ata_data['goods_value'] = float(str(carnet.TotalValue).replace(',','.'))
                                        ata_data['double_signature'] = True
                                        ata_data['initial_pages'] = int(carnet.TotalNumberOfPages or '0')
                                        ata_data['additional_pages'] = int(carnet.NumberOfExtraPages or '0')
                                        ata_data['own_risk'] = False
                                        ata_data['warranty'] = 50.00
                                        if hasattr(carnet.Deposit,'DepositAmount'):
                                            if carnet.Deposit.DepositAmount:
                                                ata_data['warranty'] = 0.0
                                                ata_data['own_risk'] = True
                                                ata_data['internal_comments'] = u'Caution de %.2f euros' % float(str(carnet.Deposit.DepositAmount or '0,0').replace(',','.'))
                                        ata_data['insurer_agreement'] = 9999999999
                                        ata_data['return_date'] = False
                                        ata_data['state'] = 'created'
                                        ata_data['name'] = str(carnet.CarnetNumber).replace(' ','')
                                        ata_data['partner_member_state'] = found_partner['membership_state']
                                        ata_data['member_price'] = (found_partner['membership_state'] in ['free','invoiced','paid'])
                                        final_name = ata_data['name']
                                        new_id = obj_carnet.create(cr,uid,ata_data)
                                        if new_id:
                                            obj_carnet.write(cr,uid,[new_id,],{'name':final_name})
                                            imported += 1
                                            self.new_ids.append(str(new_id))
                                    else:
                                        rejected += 1
                                        errors.append( 'Name : %s - Already used name-number ' % (carnet.CarnetNumber or '--none--') )
                                else:
                                    rejected += 1
                                    errors.append( 'InternalID : %s - Already used digital number ' % internal_id )
                            else:
                                rejected += 1
                                errors.append( 'InternalID : %s - Wrong Customer ID : \'%s\'' % (str(internal_id),full_customer_number) )
                        else:
                            rejected += 1
                            errors.append( 'InternalID : %s - Unknown Customer Number : \'%s\'' % (str(internal_id),full_customer_number) )
                    else:
                        rejected += 1
                        errors.append( 'InternalID : %s - Wrong ChamberNumber : \'%s\'' % (str(internal_id),str(carnet.ChamberNumber)) )
                else:
                    # we search for the original and indicate that it has a duplicata
                    if str(carnet.ChamberNumber) == parameter['DigitalChamberNumber']:
                        internal_id = internal_id[0:-1]
                        while internal_id[0:1] == '0':
                            internal_id = internal_id[1:]
                        if len(internal_id) > 0 and not (internal_id[-1:] == 'D'):
                            existing_ata_ids = obj_carnet.search(cr,uid,[('digital_number','=',internal_id)])
                            if existing_ata_ids and len(existing_ata_ids) == 1:
                                obj_carnet.write(cr,uid,existing_ata_ids,{'duplicata':True})
                    # we indicate this import as in error to force the user ta validate this duplicata and invoice it manually
                    rejected += 1
                    errors.append( 'InternalID : %s - Duplicata' % str(internal_id) )
            #except Exception, msg_error:
            #    rejected += 1
            #    errors.append( 'InternalID : %s - GENERAL UNDEFINED ERROR IN XML FILE : \'%s\'' % (str(internal_id),str(msg_error)) )
        # give the result to the user
        data['form']['msg']='Number of imported digital eATAs : %s\nNon critical errors : \nNumber of rejected eATAs : %s\nReasons :\n%s' % (str(imported),str(rejected),'\n'.join(errors))
        return data['form']
    def _open_window_results(self, cr, uid, data, context):
        result = {
            'name': _('Added ATAs'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'cci_missions.ata_carnet',
            'domain': "[('id','in', [" + ",".join(self.new_ids) + "])]",
            'context': {},
            'type': 'ir.actions.act_window'
        }
        return result
    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('import','Import File')]},
        },
        'import': {
            'actions': [_import_data],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Close'),('show','Show Result')]}
        },
        'show': {
            'actions': [],
            'result': {'type': 'action', 'action':_open_window_results, 'state':'end'}
        },
    }

wizard_import_digital_atas('cci_mission_extend.import_digital_atas')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

