# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
# Version 1.0 Philmer
import wizard
import time
import datetime
import tools
import pooler
import base64
from osv import fields
from osv import osv
from lxml import objectify        
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Import Digital Visas">
    <field name="input" colspan="4" width="600"/>
</form>"""

fields = {
    'input': {'type':'binary','string':'Digichambers file','required':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="Importation result" colspan="4"/>
     <field name="msg" colspan="4" nolabel="1" width="800" heigth="600"/>
</form>"""

msg_fields = {
    'msg': {'string':'File imported', 'type':'text', 'readonly':True},
}

class wizard_import_digital_visas(wizard.interface):

    new_nums = []
    
    def _import_data(self,cr,uid,data,context):
        inputfile = data['form']['input']
        inputdata = base64.decodestring(inputfile)
        # store the results
        rejected = 0
        imported = 0
        errors = []
        created_codes = []
        # objects necessary for the import
        parameter_obj = pooler.get_pool(cr.dbname).get('cci_parameters.cci_value')
        parameter = parameter_obj.get_value_from_names(cr,uid,['DigitalChamberNumber'])
        obj_partner = pooler.get_pool(cr.dbname).get('res.partner')
        obj_leg = pooler.get_pool(cr.dbname).get('cci_missions.legalization')
        obj_type = pooler.get_pool(cr.dbname).get('cci_missions.dossier_type')
        # decoding the xml file
        root = objectify.fromstring(inputdata)
        for visa in root.VISA:
            internal_id = int(visa.ID)
            try:
                if str(visa.ChamberNumber) == parameter['DigitalChamberNumber']:
                    # searching partner by VAT Number
                    custVAT = str(visa.CompanyVAT).replace(' ','').replace('.','')
                    if (len(custVAT) > 2) and (custVAT[0:2] in ('SE','LU')):
                        pass
                    else:
                        if len(custVAT)==9:
                            custVAT = 'BE0' + custVAT
                        elif len(custVAT)==10:
                            custVAT = 'BE' + custVAT
                    partner_ids = obj_partner.search(cr,uid,[('vat','=',custVAT),('active','=',True)])
                    if len(partner_ids) == 1:
                        partner_id = partner_ids[0]
                        found_partner = obj_partner.read(cr,uid,partner_id,['name','membership_state'])
                        # extract others mandatory fields
                        accepteddatetime = str(visa.AcceptedDateTime)
                        visa_date = accepteddatetime[6:10]+'-'+accepteddatetime[3:5]+'-'+accepteddatetime[0:2]
                        #
                        digital_number = str(visa.VisaNumber).replace(' ','')
                        visa_ids = obj_leg.search(cr,uid,[('state','in',('draft','invoiced')),('digital_number','=',digital_number)])
                        if len(visa_ids) == 0:
                            customerRef = unicode(visa.CustomerRef)
                            goods_value = 0.01
                            type_id = False
                            origs = 1
                            copies = 0
                            # copies default to 1 until 2016-10-25 : decision of SH and AP to set it to 0 again
                            # try to find attachments
                            if hasattr(visa.ReviewedAttachments,'Attachment'):
                                for attach in visa.ReviewedAttachments.Attachment:
                                    if 'Duplicate' in attach.Type:
                                        copies = int(attach.Number)
                                    else:
                                        if parameter['DigitalChamberNumber'] == '88':
                                            # For CCIH, always 'CT'
                                            type_ids = obj_type.search(cr,uid,[('code','=','CT')])
                                            if type_ids and len(type_ids) == 1:
                                                type_id = type_ids[0]
                                        else:
                                            type_ids = obj_type.search(cr,uid,[('digital_ref','=',attach.Type)])
                                            if type_ids and len(type_ids) == 1:
                                                type_id = type_ids[0]
                                            # demande de Pascale Renson du 7-6-2018 : si number > 1, mettre ce nombre en nombre d'originaux
                                            if hasattr(attach,'Number'):
                                                origs = int(attach.Number)
                            if type_id:
                                # try to add it to the legalizations table
                                leg_data = {}
                                leg_data['type_id'] = type_id
                                leg_data['date'] = visa_date
                                leg_data['order_partner_id'] = partner_id
                                leg_data['asker_name'] = found_partner['name']
                                leg_data['sender_name'] = found_partner['name']
                                leg_data['state'] = 'draft'
                                leg_data['goods_value'] = goods_value
                                leg_data['quantity_copies'] = copies
                                leg_data['quantity_original'] = origs
                                leg_data['digital_number'] = digital_number
                                leg_data['customer_ref'] = customerRef
                                leg_data['member_price'] = ( found_partner['membership_state'] in ['free','invoiced','paid'] )
                                new_id = obj_leg.create(cr,uid,leg_data)
                                self.new_nums.append(digital_number)
                                imported += 1
                            else:
                                rejected += 1
                                errors.append( 'InternalID : %s - Undetermined type of visa' % str(internal_id) )
                        else:
                            rejected += 1
                            errors.append( 'InternalID : %s - Already used digital number : %s' % (str(internal_id),digital_number) )
                    else:
                        rejected += 1
                        errors.append( 'InternalID : %s - Unknown VAT Number : \'%s\'' % (str(internal_id),custVAT) )
                else:
                    rejected += 1
                    errors.append( 'InternalID : %s - Wrong ChamberNumber : \'%s\'' % (str(internal_id),str(visa.ChamberNumber)) )
            except Exception, msg_error:
                rejected += 1
                errors.append( 'InternalID : %s - GENERAL UNDEFINED ERROR IN XML FILE : \'%s\'' % (str(internal_id),str(msg_error)) )
        # give the result to the user
        data['form']['msg']='Number of imported digital visas : %s\nNumber of rejected visas : %s\nReasons :\n%s' % (str(imported),str(rejected),'\n'.join(errors))
        return data['form']
    def _open_window_results(self, cr, uid, data, context):
        result = {
            'name': _('Added Visas'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'cci_missions.legalization',
            'domain': "[('digital_number','in', ['" + "','".join(self.new_nums) + "'])]",
            'context': {},
            'type': 'ir.actions.act_window'
        }
        return result
    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('import','Import File')]},
        },
        'import': {
            'actions': [_import_data],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Close'),('show','Show Result')]}
        },
        'show': {
            'actions': [],
            'result': {'type': 'action', 'action':_open_window_results, 'state':'end'}
        },
    }

wizard_import_digital_visas('cci_mission_extend.import_digital_visas')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

