# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
# Version 1.0 Philmer
import wizard
import time
import datetime
import tools
import pooler
import base64
from osv import fields
from osv import osv
import csv
import re

form = """<?xml version="1.0"?>
<form string="Options">
    <field name="explanation" colspan="4" nolabel="1"/>
    <field name="source_id" colspan="4"/>
    <field name="input" colspan="4" width="600"/>
    <field name="treatment"/>
</form>"""

fields = {
    'source_id': {'string': 'Source', 'type':'many2one','relation': 'cci_newsletter.source','required': True},
    'input': {'type':'binary','string':'WAPI CSV file','required':True},
    'treatment': {'type':'selection','selection':[('overwrite','Overwrite')],'string':'Method','required':True},
    'explanation': {'string':'Explanation', 'type':'text', 'readonly':True, 'default':u"You can import a CSV file with 4 and only 4 columns:\n- Sociétés.Nom\n- Contacts.Nom\n- Contacts.Prénom\nContacts.E-mail1\nin that precise order !"},
}

msg_form = """<?xml version="1.0"?>
<form string="Results">
     <field name="msg" colspan="4" nolabel="1" width="600" heigth="400"/>
</form>"""

msg_fields = {
    'msg': {'string':'File imported', 'type':'text', 'readonly':True},
}

class wizard_import_rdp_wapi(wizard.interface):
    def _import_data(self,cr,uid,data,context):
        inputfile = data['form']['input']
        inputdata = base64.decodestring(inputfile)
        obj_subscriber = pooler.get_pool(cr.dbname).get('cci_newsletter.subscriber')
        obj_source = pooler.get_pool(cr.dbname).get('cci_newsletter.source')
        source = obj_source.browse(cr,uid,[data['form']['source_id'],])[0]
        if source.default_area:
            default_area_of_source = source.default_area
        else:
            default_area_of_source = 9999
        good_struct = False
        imported = 0
        errors = 0
        deleted = 0
        error_desc = []
        if data['form']['treatment'] == 'overwrite':
            sub_ids = obj_subscriber.search(cr,uid,[('source_id','=',data['form']['source_id'])])
            obj_subscriber.unlink(cr,uid,sub_ids)
            deleted = len(sub_ids)
        EMAIL_REGEX = re.compile(r"^[A-Za-z0-9\.\+_-]+@[A-Za-z0-9\._-]+\.[a-zA-Z]*$")
        for line in inputdata.splitlines():
            if imported > 0:
                parts = line.split(";")
                if len(parts) == 4:
                    (company_name,contact_name,contact_first,email) = parts
                    email = email.decode('cp1252').strip().lower()
                    if (' ' not in email) and EMAIL_REGEX.match(email):
                        new_data = {'name':(contact_name or '.').decode('cp1252').encode('utf-8'),
                                    'first_name':(contact_first or '.').decode('cp1252').encode('utf-8'),
                                    'email':email,
                                    'company_name':(company_name or '.').decode('cp1252').encode('utf-8'),
                                    'source_id':data['form']['source_id'],
                                   }
                        # check if email already exist from the same source and with the same forced_area without expiration date
                        already_ids = obj_subscriber.search(cr,uid,[('email','=',email),('source_id','=',data['form']['source_id']),'|',('forced_area','=',False),('forced_area','=',default_area_of_source)])
                        if already_ids:
                            errors += 1
                            error_desc.append(u'double email : ' + email)
                        else:
                            obj_subscriber.create(cr,uid,new_data)
                    else:
                        errors += 1
                        error_desc.append(u'email : ' + email)
                else:
                    errors += 1
                    error_desc.append(u'line : ' + line)
            else:
                # check if first line is correct
                parts = line.split(";")
                if len(parts) == 4:
                    if parts[0].decode('cp1252') != u'Sociétés.Nom' or parts[1].decode('cp1252') != u'Contacts.Nom' or \
                       parts[2].decode('cp1252') != u'Contacts.Prénom' or parts[3].decode('cp1252') != u'Contacts.E-mail1':
                        good_struct = True
                    else:
                        data['form']['msg'] = u"Wrong structure of CSV file : The 4 columns must be named 'Sociétés.Nom', 'Contacts.Nom', 'Contacts.Prénom', 'Contacts.E-mail1' in that order"
                else:
                        data['form']['msg'] = u"Wrong structure of CSV file : You must have 4 and only 4 columns"
            imported += 1
        data['form']['msg']=u'Number of imported subscribers : %s\nDeleted : %s\nErrors (probably ; in names) : %s\n\n- %s' % (str(imported-1),str(deleted),str(errors),u'\n - '.join(error_desc))
        return data['form']
    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('import','Import File')]},
        },
        'import': {
            'actions': [_import_data],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }
wizard_import_rdp_wapi('import_rdp_wapi')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
