# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008 CCI Connect ASBL. (http://www.cciconnect.be) All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
from osv import fields, osv
import datetime
import tools
import string

def _check_digit_1_digit(digital_string):
    digit_sum = 0
    for carac in digital_string:
        if carac.isdigit():
            digit_sum += int(carac)
    return str(digit_sum)[-1:]
def _link_id(record_id):
    partner = str(record_id).rjust(5,'0')
    inv_partner = ''
    for carac in partner:
        inv_partner = carac + inv_partner
    year = (7 * int(inv_partner[3:4])) + int(inv_partner[2:3]) + 13 ## year was the last two caracters of the current year
    link_id = inv_partner[0:3] + str(year) + inv_partner[3:4]
    link_id += _check_digit_1_digit(link_id)
    link_id += inv_partner[4:5]
    rest = int(link_id) % 93
    link_id += _check_digit_1_digit(link_id)
    link_id = link_id + str(rest).rjust(2,'0')
    return link_id
class partner_subscription_type(osv.osv):
    _name = "partner_subscription.type"
    _description = "Type/Group of Subscriptions"
    _columns = {
        'name':fields.char('Name',size=100,required=True),
        'code':fields.char('Code',size=10),
        'product_ids':fields.one2many('product.product','partner_subscription_type_id','Linked Products'),
    }
partner_subscription_type()

class product_product(osv.osv):
    _name = 'product.product'
    _inherit = 'product.product'
    _columns = {
        'partner_subscription_type_id': fields.many2one('partner_subscription.type','Partner Subscription Type'),
    }
product_product()

class partner_subscription(osv.osv):
    _name = "partner_subscription"
    _description = "Subscriptions for Partners"
    _columns = {
        'name':fields.char('Description',size=100),
        'date':fields.date('Date of subscription'),
        'begin':fields.date('Begin of Subscription'),
        'end':fields.date('End of Subscription'),
        'invoice_id':fields.many2one('account.invoice','Invoice'),
        'partner_id':fields.many2one('res.partner','Partner'),
        'contact_id':fields.many2one('res.partner.contact','Contact'),
        'type_id':fields.many2one('partner_subscription.type','Type'),
        'product_id':fields.many2one('product.product','Product'),
        'price':fields.float('Price WoVAT',digits=(16, 2)),
        'source':fields.char('Source',size=30),
        'state':fields.selection([('draft','Draft'),('current','Current'),('closed','Closed'),('cancel','Canceled')],'State'),
        'close_date':fields.date('Close Date'),
        'close_user_id':fields.many2one('res.users','Close User'),
        'close_source':fields.char('Source of Close',size=30),
        'active':fields.boolean('Active'),
    }
    _defaults = {
        'state': lambda *a: 'draft',
        'active': lambda *a: True,
        'date': lambda *a: datetime.datetime.today().strftime('%Y-%m-%d'),
        'begin': lambda *a: datetime.datetime.today().strftime('%Y-%m-%d'),
    }
    _order = 'date desc'
    def _onchange_product_id(self, cr, uid, ids, product_id,context={}):
        if not product_id:
            return {'value':{'price' : False ,'name' : False }}
        data_product = self.pool.get('product.product').browse(cr,uid,product_id)
        data = {}
        if data_product:
            data['name'] = data_product.name
            data['price'] = data_product.list_price
        else:
            data['name'] = False
            data['price'] = False
        return {'value':data}
    def button_draft(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state':'draft','close_date':False,'close_user_id':False,'close_source':''})
        return True
    def button_current(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state':'current','close_date':False,'close_user_id':False,'close_source':''})
        return True
    def button_close(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state':'closed','close_date':datetime.datetime.today().strftime('%Y-%m-%d'),'close_user_id':uid,'close_source':'Manual'})
        return True
    def button_cancel(self, cr, uid, ids, *args):
        self.write(cr, uid, ids, {'state':'cancel','close_date':False,'close_user_id':False,'close_source':''})
        return True
    def _send_url(self,cr,uid,subscription_id,template_id,forced_address):
        template_obj = self.pool.get('cci_email_template')
        template = template_obj.browse(cr,uid,template_id)
        sub_obj = self.pool.get('partner_subscription')
        subscription = sub_obj.browse(cr,uid,subscription_id)
        email_job = False
        final_email = False
        # pour l'adresse d'envoi, on va rechercher soit le contact avec la même adresse email que l'adresse forcée, sinon le contact_id de la subscription,
        # sinon le premier G
        # si on trouve, on aura un object res.partner.job en mode browse
        if forced_address:
            final_email = forced_address
            if subscription.partner_id:
                for addr in subscription.partner_id.address:
                    for job in addr.job_ids:
                        if job.email and job.email == forced_address:
                            email_job = job
                            break
                        elif job.contact_id and job.contact_id.email and job.contact_id.email == forced_address:
                            email_job = job
                            break
        elif subscription.contact_id:
            if subscription.contact_id.job_ids:
                for job in subscription.contact_id.job_ids:
                    if job.address_id and job.address_id.partner_id and job.address_id.partner_id.id == subscription.partner_id.id:
                        if job.email:
                            final_email = job.email
                            email_job = job
                            break
                if not final_email and subscription.contact_id.email:
                    final_email = subscription.contact_id.email
                    if subscription.contact_id.job_ids:
                        for job in subscription.contact_id.job_ids:
                            if job.address_id and job.address_id.partner_id and job.address_id.partner_id.id == subscription.partner_id.id:
                                email_job = job
                                break
        else:
            # recherche du premier G si possible de l'adresse par défaut, sinon de la première adresse autre que invoice, sinon invoice...
            min_seq = 9999
            if subscription.partner_id:
                for addr in subscription.partner_id.address:
                    if addr.type == 'default':
                        for job in addr.job_ids:
                            if job.email and job.function_code_label and 'G' in job.function_code_label and job.sequence_partner < min_seq:
                                email_job = job
                                final_email = job.email
                                min_seq = job.sequence_partner
                if not email_job:
                    min_seq_addr = 9999
                    min_seq = 9999
                    for addr in subscription.partner_id.address:
                        if addr.type == 'other':
                            for job in addr.job_ids:
                                if job.email and job.function_code_label and 'G' in job.function_code_label and job.sequence_partner < min_seq and addr.sequence_partner < min_seq_addr:
                                    email_job = job
                                    final_email = job.email
                                    min_seq = job.sequence_partner
                                    min_seq_addr = addr.sequence_partner
                if not email_job:
                    min_seq = 9999
                    for addr in subscription.partner_id.address:
                        if addr.type == 'invoice':
                            for job in addr.job_ids:
                                if job.email and job.function_code_label and 'G' in job.function_code_label and job.sequence_partner < min_seq and addr.sequence_partner < min_seq_addr:
                                    email_job = job
                                    final_email = job.email
                                    min_seq = job.sequence_partner
        if final_email:
            if email_job and email_job.contact_id:
                courtesy = email_job.contact_id.title
                name = email_job.contact_id.name
            else:
                courtesy = 'cher'
                name = 'membre'
            email_content = template.body.replace('[courtesy]',courtesy).replace('[name]',name)
            if subscription.begin:
                email_content = email_content.replace('[from]',subscription.begin[8:10]+'/'+subscription.begin[5:7]+'/'+subscription.begin[0:4])
            else:
                email_content = email_content.replace('[from]','')
            if subscription.end:
                email_content = email_content.replace('[to]',subscription.end[8:10]+'/'+subscription.end[5:7]+'/'+subscription.end[0:4])
            else:
                email_content = email_content.replace('[to]','')
            if subscription.partner_id:
                if subscription.partner_id.address:
                    links = []
                    for addr in subscription.partner_id.address:
                        if not addr.dir_exclude:
                            links.append( _link_id( addr.id ) )
                    if not links:
                        final_email = ''
                else:
                    final_email = ''
            else:
                final_email = ''
            if final_email:
                full_links = []
                for link in links:
                    full_links.append( '<a href="http://www.ccilvn.be/mise-a-jour-repertoire/?key=%s">http://www.ccilvn.be/mise-a-jour-repertoire/?key=%s</a>' % (link,link) )
                inside = u'</li><li>'.join(full_links)
                email_content = email_content.replace('[links]',u'<ul><li>' + inside + u'</li></ul>')
                errors = []
                try:
                    tools.email_send('fb@ccilvn.be', [final_email,], template.subject or '', email_content, subtype=template.type)
                except:
                    errors.append(email_address)
                if errors:
                    final_email = ''
        return (final_email)
    def _create_new(self,cr,uid,partner_id,contact_id,product_id,begin,end,to_invoice):
        msg_final = ''
        new_id_sub = False
        new_id_invoice = False

        obj_sub = self.pool.get('partner_subscription')
        obj_product = self.pool.get('product.product')
        product = obj_product.browse(cr,uid,product_id)
        data_sub = {}
        data_sub['name'] = product.name
        data_sub['date'] = datetime.datetime.today().strftime('%Y-%m-%d')
        data_sub['begin'] = begin
        data_sub['end'] = end
        data_sub['contact_id'] = contact_id
        data_sub['partner_id'] = partner_id
        data_sub['type_id'] = False
        data_sub['product_id'] = product.id
        data_sub['price'] = product.list_price
        data_sub['source'] = 'Manual wizard'
        data_sub['type_id'] = product.partner_subscription_type_id.id
        data_sub['state'] = 'current'
        if to_invoice and partner_id and data_sub['price'] > 0.0:
            data_sub['invoice_id'] = False
            obj_lines = self.pool.get('account.invoice.line')
            tax_ids=[]
            val_invoice = self.pool.get('account.invoice').onchange_partner_id(cr, uid, [], 'out_invoice', partner_id, False, False)
            val_invoice['value'].update({'partner_id': partner_id})
            partner_address_id = val_invoice['value']['address_invoice_id']
            if not partner_address_id:
                msg_final = "Given partner doesn't have an address to make the invoice.\nInvoice not generated but subscription created"
            else:
                obj_partner = self.pool.get('res.partner')
                partner = obj_partner.browse(cr,uid,partner_id)
                value = obj_lines.product_id_change(cr, uid, [], product_id,uom =False, partner_id=partner_id, fposition_id=partner.property_account_position.id)
                for tax in product.taxes_id:
                    tax_ids.append(tax.id)

                vals = value['value']
                c_name = product.name or ''
                vals.update({
                    'name': c_name,
                    'price_unit': product.list_price,
                    'quantity': 1,
                    'product_id': product.id,
                    'invoice_line_tax_id': [(6,0,tax_ids)],
                })
                inv_line_ids = obj_lines.create(cr, uid,vals)
                val_invoice['value'].update({
                    'origin': '',
                    'reference': False,
                    'invoice_line': [(6,0,[inv_line_ids])],
                    'comment': "",
                })

                inv_obj = self.pool.get('account.invoice')
                new_id_invoice = inv_obj.create(cr, uid, val_invoice['value'])
                data_sub['invoice_id'] = new_id_invoice
        else:
            data_sub['invoice_id'] = False
        new_id_sub = obj_sub.create(cr,uid,data_sub)
        return (new_id_sub,new_id_invoice,msg_final)
partner_subscription()

class res_partner(osv.osv):
    _inherit = "res.partner"
    _description = "res.partner"
    _columns = {
        'subscription_ids': fields.one2many('partner_subscription','partner_id','Subscriptions'),
    }
res_partner()

