# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008 CCI Connect ASBL. (http://www.cciconnect.be) All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import datetime
import time
from osv import fields, osv
from tools.translate import _
import tools
import pymysql
import json
import ftplib
import random

def correct_phone(string):
    if 'conf' not in string:
        result = string.replace(' ','').replace('/','').replace('.','').replace('-','').replace('+','00')
    else:
        result = ''
    return result
def get_phone(phone_num):
    result = ''
    if phone_num and 'conf' not in phone_num:
        result = phone_num.replace(' ','').replace('/','').replace('.','').replace('-','').replace('+','00')
    else:
        result = ''
    return result
def _correct_vat(vat):
    corrected_vat = vat.strip().replace(' ','').replace('/','').replace('-','').replace('.','').upper()
    if len(corrected_vat ) >= 8:
        if corrected_vat[0:2] == 'BE':
            if len(corrected_vat) == 12:
                pass
            elif len(corrected_vat) == 11:
                corrected_vat = 'BE0'+corrected_vat[2:]
            else:
                corrected_vat = ''
        else:
            if corrected_vat[0:2].isdigit():
                if len(corrected_vat) == 10:
                    corrected_vat = 'BE' + corrected_vat
                elif corrected_vat == 9:
                    corrected_vat = 'BE0' + corrected_vat
                else:
                    corrected_vat = ''
            else:
                # suppose stranger VAT number : let it untouched
                corrected_vat = corrected_vat
        if len(corrected_vat) == 12 and corrected_vat[0:2] == 'BE':
            # belgian complete vat number => check digit
            try:
                fullnum = int(corrected_vat[2:])
            except:
                corrected_vat = ''
                fullnum = 0
            if fullnum:
                if int(corrected_vat[-2:]) != (97 - (int(corrected_vat[2:10]) % 97)):
                    corrected_vat = ''
    else:
        corrected_vat = ''
    return corrected_vat

class cci_popup_stand_type(osv.osv):
    _name = "cci.popup.stand_type"
    _description = "type of stand for popup"
    _columns = {
        'name': fields.char('Name',size=30,required=True),
        'default_price': fields.float('Default Price',digits=(15,2)),
        'default_exhibitors': fields.integer('Default Free Exhibitors'),
        'default_guests': fields.integer('Default Free Guests'),
        'default_champagne': fields.integer('Default Champagne Bottles'),
        'default_night_tickets': fields.integer('Default Night Tickets'),
        'web_type': fields.integer('Type on Popup Plateform',required=True),
        'event_id': fields.many2one('event.event','Event for Registrations'),
        'product_id': fields.many2one('product.product','Product to Invoice stand'),
        'night_product_id': fields.many2one('product.product','Product to Invoice Night Bracelets'),
        'exhibitor_product_id': fields.many2one('product.product','Product to Invoice Additional Exhibitor Tickets'),
        'guest_product_id': fields.many2one('product.product','Product to Invoice Additional Guest Tickets'),
        'default_screen': fields.integer('Default Screen'),
        'active': fields.boolean('Active'),
    }
    _defaults = {
        'active': lambda *a: True,
    }
cci_popup_stand_type()

class cci_popup_stand(osv.osv):
    _name = "cci.popup.stand"
    _description = "folder to manage a stand for an edition of popup"
    def _get_salesman(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for stand in self.browse(cr,uid,ids):
            if stand.partner_id and stand.partner_id.user_id:
                res[stand.id] = stand.partner_id.user_id.name + ' [' + stand.partner_id.user_id.login +']'
            else:
                res[stand.id] = ''
        return res
    def _get_invoices_amount(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for stand in self.browse(cr,uid,ids):
            res[stand.id] = 0.0
            multiplier = 1
            if stand.invoice_id and stand.invoice_id.journal_id.refund_journal:
                multiplier = -1
            res[stand.id] += ( stand.invoice_id.amount_untaxed * multiplier )
        return res
    def _get_email_order(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for stand in self.browse(cr,uid,ids):
            email = ''
            if stand.partner_id and stand.order_contact_id:
                if stand.partner_id.address:
                    for addr in stand.partner_id.address:
                        if addr.job_ids:
                            for job in addr.job_ids:
                                if job.contact_id and job.contact_id.id == stand.order_contact_id.id:
                                    email = job.email or job.contact_id.email or ''
                                    if email:
                                        break
                        if email:
                            break
            if not email:
                if stand.order_contact_id.email:
                    email = stand.order_contact_id.email
            res[stand.id] = email
        return res
    def _get_email_material(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for stand in self.browse(cr,uid,ids):
            email = ''
            if stand.partner_id and stand.material_contact_id:
                if stand.partner_id.address:
                    for addr in stand.partner_id.address:
                        if addr.job_ids:
                            for job in addr.job_ids:
                                if job.contact_id and job.contact_id.id == stand.material_contact_id.id:
                                    email = job.email or job.contact_id.email or ''
                                    if email:
                                        break
                        if email:
                            break
            if not email:
                if stand.material_contact_id.email:
                    email = stand.material_contact_id.email
            res[stand.id] = email
        return res
    def _get_email_stand(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for stand in self.browse(cr,uid,ids):
            email = ''
            if stand.partner_id and stand.stand_contact_id:
                if stand.partner_id.address:
                    for addr in stand.partner_id.address:
                        if addr.job_ids:
                            for job in addr.job_ids:
                                if job.contact_id and job.contact_id.id == stand.stand_contact_id.id:
                                    email = job.email or job.contact_id.email or ''
                                    if email:
                                        break
                        if email:
                            break
            if not email:
                if stand.stand_contact_id.email:
                    email = stand.stand_contact_id.email
            res[stand.id] = email
        return res
    def onchange_type_id(self, cr, uid, ids, type_id):
        if type_id:
            stand_type = self.pool.get('cci.popup.stand_type').browse(cr, uid, type_id)
            if stand_type.default_price:
                result = {'value': {'final_stand_price': stand_type.default_price}}
            else:
                result = {'value': {'final_stand_price': 0.0}}
            if stand_type.default_night_tickets:
                result['value']['order_night'] = stand_type.default_night_tickets
                result['value']['price_night'] = 0.0
            else:
                result['value']['order_night'] = 0
                result['value']['price_night'] = 20.0
        else:
            result = {'value':{'final_stand_price':0.0,'order_night':0,'price_night':20.0}}
        return result
    def onchange_partner_id(self, cr, uid, ids, partner_id):
        data={}
        #raise an error if the partner has a warning
        warning = False
        if partner_id:
            data_partner = self.pool.get('res.partner').browse(cr,uid,partner_id)
            if data_partner.alert_events:
                warning = {
                    'title': "Warning:",
                    'message': data_partner.alert_explanation or 'Partner is not valid but there is no explanation on partner, only the \'event warning\' selected'
                        }
        #data = {'value':data}
        if warning:
            data['warning'] =  warning
        return data
    def onchange_order_contact_id(self, cr, uid, ids, contact_id, stand_contact_id, material_contact_id):
        if contact_id and ((not stand_contact_id) or (not material_contact_id)):
            new_contacts = {}
            if not stand_contact_id:
                new_contacts['stand_contact_id'] = contact_id
            if not material_contact_id:
                new_contacts['material_contact_id'] = contact_id
            if new_contacts:
                result = {'value': new_contacts}
            else:
                result = {'value':{}}
        else:
            result = {'value':{}}
        return result
    def create(self,cr,uid,vals,context=None):
        if vals and vals.has_key('type_id'):
            stand_type = self.pool.get('cci.popup.stand_type').browse(cr,uid,vals['type_id'])
            if stand_type.event_id:
                contact_name = 'unknown'
                if vals['order_contact_id']:
                    contact = self.pool.get('res.partner.contact').browse(cr,uid,vals['order_contact_id'])
                    contact_name = (contact.name+' '+(contact.first_name or '')).strip()
                reg_obj = self.pool.get('event.registration')
                reg_data = {'event_id':stand_type.event_id.id,
                            'partner_invoice_id':vals['partner_id'],
                            'partner_id':vals['partner_id'],
                            'contact_id':vals['order_contact_id'],
                            'unit_price':vals['final_stand_price'],
                            'tobe_invoiced':False,
                            'attendance':True,
                            'nb_register':1,
                            'name':'Registration: '+contact_name,
                            'invoice_label': stand_type.event_id.product_id and stand_type.event_id.product_id.name or 'Popup',
                           }
                reg_id = reg_obj.create(cr,uid,reg_data,context)
                if reg_id:
                    new_reg_data = {'state':'done',}
                    new_values = reg_obj.onchange_contact_id(cr,uid,[reg_id,], vals['order_contact_id'], vals['partner_id'])
                    new_reg_data['badge_name'] = new_values.has_key('badge_name') and new_values['value']['badge_name'] or ''
                    new_reg_data['email_from'] = new_values.has_key('email_from') and new_values['value']['email_from'] or ''
                    new_reg_data['badge_title'] = new_values.has_key('badge_title') and new_values['value']['badge_title'] or ''
                    new_values = reg_obj.onchange_partner_id(cr,uid,[reg_id,], vals['partner_id'],stand_type.event_id.id)
                    new_reg_data['badge_partner'] = new_values.has_key('badge_partner') and new_values['value']['badge_partner'] or ''
                    reg_obj.write(cr,uid,[reg_id,],new_reg_data)
                    vals['stand_reg_id'] = reg_id
        result = super(cci_popup_stand, self).create(cr, uid, vals, context)
        return result
    def write(self,cr,uid,ids,vals,context=None):
        for stand_id in ids:
            if ('type_id' in vals) or ('partner_id' in vals) or ('order_contact_id' in vals) or ('final_stand_price' in vals):
                # changes to write down to linked event.registration
                stand = self.browse(cr,uid,stand_id)
                if stand.stand_reg_id:
                    reg_data = {}
                    if 'type_id' in vals:
                        stand_type = self.pool.get('cci.popup.stand_type').browse(cr,uid,vals['type_id'])
                        if stand_type.event_id:
                            reg_data['event_id'] = stand_type.event_id.id
                            reg_data['invoice_label'] = stand_type.event_id.product_id and stand_type.event_id.product_id.name or 'Popup'
                    if 'partner_id' in vals:
                        reg_data['partner_invoice_id'] = vals['partner_id']
                        reg_data['partner_id'] = vals['partner_id']
                        event_id = 0
                        if 'type_id' in vals:
                            stand_type = self.pool.get('cci.popup.stand_type').browse(cr,uid,vals['type_id'])
                            event_id = stand_type.event_id.id
                        else:
                            event_id = stand.type_id.event_id.id
                        new_values = self.pool.get('event.registration').onchange_partner_id(cr,uid,[stand.stand_reg_id,],vals['partner_id'],event_id)
                        reg_data['badge_partner'] = new_values.has_key('badge_partner') and new_values['value']['badge_partner'] or ''
                    if 'order_contact_id' in vals:
                        contact = self.pool.get('res.partner.contact').browse(cr,uid,vals['order_contact_id'])
                        contact_name = (contact.name+' '+(contact.first_name or '')).strip()
                        reg_data['contact_id'] = vals['order_contact_id']
                        reg_data['name'] = 'Registration: '+contact_name
                        if 'partner_id' in vals:
                            partner_id = vals['partner_id']
                        else:
                            partner_id = stand.partner_id.id
                        new_values = self.pool.get('event.registration').onchange_contact_id(cr,uid,[stand.stand_reg_id,], vals['order_contact_id'], partner_id)
                        reg_data['badge_name'] = new_values.has_key('badge_name') and new_values['value']['badge_name'] or ''
                        reg_data['email_from'] = new_values.has_key('email_from') and new_values['value']['email_from'] or ''
                        reg_data['badge_title'] = new_values.has_key('badge_title') and new_values['value']['badge_title'] or ''
                    if 'final_stand_price' in vals:
                        reg_data['unit_price'] = vals['final_stand_price']
                    self.pool.get('event.registration').write(cr,uid,[stand.stand_reg_id.id,],reg_data,context)
        return super(cci_popup_stand,self).write(cr,uid,ids,vals,context)
    def copy(self, cr, uid, id, default=None, context={}):
        stand = self.read(cr,uid,id,['name'])
        if not default:
            default = {}
        default.update({
            'name':(stand['name'] and stand['name'].strip() or '')+' (copy)',
            'edition':False,
            'invoice_id': False,
            'stand_reg_id':False,
            'external_name':False,
            'video_status':'waiting',
            'canvas_status':'waiting',
            'web_id':False,
            'confirm_sending':False,
        })
        return super(cci_popup_stand, self).copy(cr, uid, id, default, context)
    _columns = {
        'edition': fields.integer('Edition',required=True),
        'type_id': fields.many2one('cci.popup.stand_type','Type',required=True),
        'partner_id': fields.many2one('res.partner','Partner',required=True),
        'salesman': fields.function(_get_salesman,type='string',size=254,method=True,string='Salesman'),
        'order_contact_id': fields.many2one('res.partner.contact','Order Contact'),
        'stand_contact_id': fields.many2one('res.partner.contact','Stand Contact'),
        'name': fields.char('Stand Name',size=90),
        'final_stand_price': fields.float('Stand Price',digits=(15,2)),
        'order_night': fields.integer('Entries for the night'),
        'price_night': fields.float('Night entries Price',digits=(15,2)),
        'order_add_exhibitor': fields.integer('Additional Exhibitors'),
        'price_add_exhibitor': fields.float('Add. Exhibitors Price',digits=(15,2)),
        'order_add_guest': fields.integer('Additional Guests'),
        'price_add_guest': fields.float('Add. Guests Price',digits=(15,2)),
        'stand_reg_id': fields.many2one('event.registration','Registration'),
        'invoice_id': fields.many2one('account.invoice','Invoice'),
        'comments': fields.text('Comments'),
        'other_partner_ids': fields.text('Other Partners ID',help='List of id, separated by comma'),
        'video_status': fields.selection([('waiting','En attente'),('waiting_convert','A Convertir'),('ready','Ready')],'Video Status'),
        'video_name': fields.char('Video Name',size=1024),
        'video_price': fields.float('Supplement for Video Works',digits=(15,2)),
        'material_contact_id': fields.many2one('res.partner.contact','Material Contact'),
        'canvas_status': fields.selection([('waiting','En attente'),('waiting_convert','A retravailler'),('ready','Ready')],'Canvas Status'),
        'commissioned': fields.selection([('Thomas Schloesser','Thomas Schloesser'),('mailing','Mailings'),('other','Autre')],'Commission',required=True),
        'web_id': fields.integer('ID on Popup plateform'),
        'active': fields.boolean('Active'),
        'email_order': fields.function(_get_email_order,type='string',size=254,method=True,string='Email Contact Order'),
        'email_material': fields.function(_get_email_material,type='string',size=254,method=True,string='Email Contact Material'),
        'email_stand': fields.function(_get_email_stand,type='string',size=254,method=True,string='Email Stand Responsible'),
        'confirm_sending': fields.datetime('Confirmation Sending'),
        'count_value': fields.integer('Count as'),
        
        #'history_ids': fields.one2many('fullpage.history','fullpage_id','History'),
    }
    _defaults = {
        'video_status': lambda *a: 'waiting',
        'canvas_status' : lambda *a : 'waiting',
        'active': lambda *a: True,
        'price_add_exhibitor': lambda *a: 25.0,
        'price_add_guest': lambda *a: 25.0,
        'count_value': lambda *a: 1,
    }
    def convert_old_sector_ids_to_new_ids(self,cr,uid,dSectCode2ID):
        dResult = {}
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['ActivityCodeRootID'])
        if param_values.has_key('ActivityCodeRootID'):
            activityCodeRootID = param_values['ActivityCodeRootID']
            obj_categ = self.pool.get('res.partner.category')
            old_len = 0
            categ_ids = [ activityCodeRootID ]
            while len(categ_ids) > old_len:
                new_ids = categ_ids[old_len:] # ids of categories added last time
                old_len = len(categ_ids) # the new frontier ...
                new_categs = obj_categ.read(cr,uid,new_ids,['child_ids'])
                for categ in new_categs:
                    if categ['child_ids']:
                        categ_ids.extend(categ['child_ids'])
            categs = obj_categ.read(cr,uid,categ_ids,['name'],{'lang':'fr_FR'})
            dCategs = {}
            for categ in categs:
                formated_name = categ['name']
                posA = formated_name.rfind('[')
                posB = formated_name.rfind(']')
                if posA >= 0 and posB > 0 and posA < posB:
                    code = formated_name[posA+1:posB]
                    if dSectCode2ID.has_key(code):
                        dResult[categ['id']] = dSectCode2ID[code]
        return (dResult,categ_ids)
    def _json_on_site(self,cr,uid,source,edition):
        # Start log
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'Popup JSON On site')
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Source : %s - Edition : %i\n' % (source,edition))
        def get_phone_country_prefix(self,cr,uid):
            result = []
            obj_country = self.pool.get('cci.country')
            country_ids = obj_country.search(cr,uid,[('phoneprefix','!=',False),('phoneprefix','!=',0)])
            if country_ids:
                countries = obj_country.read(cr,uid,country_ids,['phoneprefix'])
                result = [str(x['phoneprefix']) for x in countries]
            return result
        PHONE_COUNTRY_PREFIX = get_phone_country_prefix(self,cr,uid)
        def convert_phone(string,PHONE_COUNTRY_PREFIX):
            def only_digits(string):
                cleaned = ''
                for carac in string:
                    if carac in '0123456789':
                        cleaned += carac
                return cleaned
            result = ''
            string = only_digits(string or '')
            if len(string) > 0:
                if len(string) == 9:
                    if string[0:2] in ['02','03','04','09']:
                        result = string[0:2] + "/" + string[2:5] + "." + string[5:7] + "." + string[7:]
                    else:
                        result = string[0:3] + "/" + string[3:5] + "." + string[5:7] + "." + string[7:]
                elif len(string) == 10:
                    result = string[0:4] + "/" + string[4:6] + "." + string[6:8] + "." + string[8:]
                else:
                    # international number
                    #print string
                    if string[0:2] == '00':
                        # search after a country with this prefix
                        prefix = string[2:4]
                        if prefix not in PHONE_COUNTRY_PREFIX:
                            prefix = string[2:5]
                            if prefix not in PHONE_COUNTRY_PREFIX:
                                prefix = string[2:6]
                                if prefix not in PHONE_COUNTRY_PREFIX:
                                    prefix = ''
                        if prefix:
                            result = '+' + string[2:2+len(prefix)] + ' ' + string[2+len(prefix):4+len(prefix)]
                            rest = string[4+len(prefix):]
                            while len(rest) > 3:
                                result += '.' + rest[0:2]
                                rest = rest[2:]
                            result += '.' + rest
                        else:
                            result = string
            return result
        json_string = ''
        if source == 'ERP':
            
            # Extract Data from this ERP
            
            stand_obj = self.pool.get('cci.popup.stand')
            stand_ids = stand_obj.search(cr,uid,[('edition','=',edition)])
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Count stands from ERP : %i\n' % len(stand_ids) )
            if stand_ids:
                stands = stand_obj.browse(cr,uid,stand_ids,{'lang':'fr_FR'})
                dStands = {}
                for stand in stands:
                    if stand.partner_id:
                        record = {}
                        record['name'] = (stand.name or stand.partner_id.name).strip().encode('utf-8')
                        address = False
                        for addr in stand.partner_id.address:
                            if addr.type == 'default':
                                address = addr
                        if address:
                            record['street'] = (address.street or '').strip().encode('utf-8')
                            record['street2'] = (address.street2 or '').strip().encode('utf-8')
                            record['zip_city'] = (address.zip_id.name+' '+address.zip_id.city).strip().encode('utf-8')
                            record['phone'] = convert_phone(address.phone,PHONE_COUNTRY_PREFIX).encode('utf-8')
                            record['website'] = (stand.partner_id.website or '').strip().encode('utf-8')
                            record['activ_desc'] = (stand.partner_id.activity_description or '').replace('\n','<br/>').replace('\r','').strip().encode('utf-8')
                            record['erp_id'] = stand.id
                            record['webapp_id'] = stand.web_id
                    if not dStands.has_key(record['name']):
                        dStands[record['name']] = record
            # sort randomly list of stands
            lStands = []
            to_pub_names = [x['name'] for x in dStands.values()]
            random.seed()
            while to_pub_names:
                name_to_pub = random.choice(to_pub_names)
                lStands.append(dStands[name_to_pub])
                to_pub_names.remove(name_to_pub)
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Count published stands : %i\n' % len(dStands.keys()) )
            json_string = json.dumps(lStands,indent=4)
        else:
            
            # EXTRACT UPDATED DATA FROM POPUP PLATFORM
            parameter_obj = self.pool.get('cci_parameters.cci_value')
            param_values = parameter_obj.get_value_from_names(cr,uid,['PopupExportSupervisors','PopupConnectionDict','CompanyFacebookURL','CompanyLinkedInURL','CompanyTwitterURL','PopupTest','BDDeTest'])
            if param_values.has_key('PopupExportSupervisors') and param_values.has_key('PopupConnectionDict'):
                supervisor_emails = param_values['PopupExportSupervisors'] # list of emails
                connection = param_values['PopupConnectionDict']
                final_result = True
                # Connection to the external db
                try:
                    con = pymysql.connect(host=connection['host'], port=connection['port'], user=connection['user'], passwd=connection['pw'], db=connection['db'], charset='utf8');
                    cur = con.cursor()
                except pymysql.err.Error, e:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Connection Error %d: %s' % (e.args[0],e.args[1]))
                    final_result = False
                    if cur:
                        cur.close()
                    if con:
                        con.close()
                if final_result:
                    try:
                        sentence = "SELECT part_nom, part_adresse, part_adresse_plus, part_adresse_cp, part_adresse_loca, part_description, part_tel, id_stand, part_site, erp_reg_id FROM `stand`"
                        cur.execute(sentence)
                    except pymysql.err.Error, e:
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Connection Error %d: %s' % (e.args[0],e.args[1]))
                        final_result = False
                    if final_result:
                        data = cur.fetchall()
                        dStands = {}
                        for line in data:
                            record = {}
                            record['name'] = line[0].strip()
                            record['street'] = line[1].strip()
                            record['street2'] = line[2].strip()
                            record['zip_city'] = (line[3]+u' '+line[4]).strip()
                            record['activ_desc'] = line[5].strip().replace('\n','<br/>').replace('\r','')
                            record['phone'] = convert_phone(line[6],PHONE_COUNTRY_PREFIX)
                            record['webapp_id'] = line[7]
                            record['website'] = line[8].strip()
                            record['erp_id'] = line[9]
                            if not dStands.has_key(record['name']):
                                dStands[record['name']] = record
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Count published stands : %i\n' % len(dStands.keys()) )
                        # sort randomly list of stands
                        lStands = []
                        to_pub_names = [x['name'] for x in dStands.values()]
                        random.seed()
                        while to_pub_names:
                            name_to_pub = random.choice(to_pub_names)
                            lStands.append(dStands[name_to_pub])
                            to_pub_names.remove(name_to_pub)
                        json_string = json.dumps(lStands,indent=4)
        # RECORD THE FILE THRU FTP ON POPUP SITE
        if json_string:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Creating File\n')
            hfFile = open("exposants.json","w+")
            json.dump(dStands.values(),hfFile,indent=4)
            hfFile.close()
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Opening FTP\n')
            ftp = ftplib.FTP()
            ftp.connect('popup.ccilvn.be','20023')
            ftp.login('ccilvn','U6qcTby5E7$8ZePv')
            ftp.cwd( '/var/www/html/files')
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Store File\n')
            hfIn = open('exposants.json','r')
            ftp.storlines('STOR exposants.json',hfIn)
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Close Connexion\n')
            ftp.close()
        else:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Empty List\n')
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,True,'\n---end---')
        return json_string
    def _export2platform(self,cr,uid):
        # Start log
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'ExportPopup2Platform')
        final_result = True
        test = False
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['PopupExportSupervisors','PopupConnectionDict','CompanyFacebookURL','CompanyLinkedInURL','CompanyTwitterURL','PopupTest','BDDeTest'])
        if param_values.has_key('PopupExportSupervisors') and param_values.has_key('PopupConnectionDict'):
            supervisor_emails = param_values['PopupExportSupervisors'] # list of emails
            connection = param_values['PopupConnectionDict']
            BD_DE_TEST = False
            if param_values.has_key('BDDeTest'):
                if param_values['BDDeTest'] == 1:
                    BD_DE_TEST = True
            BD_DE_TEST = False
            if param_values.has_key('CompanyFacebookURL'):
                question_fb_id = param_values['CompanyFacebookURL']
            else:
                question_fb_id = False
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nATTENTION pas de question ID sur Facebook définie\n' )
            if param_values.has_key('CompanyLinkedInURL'):
                question_li_id = param_values['CompanyLinkedInURL']
            else:
                question_li_id = False
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nATTENTION pas de question ID sur LinkedIn définie\n' )
            if param_values.has_key('CompanyTwitterURL'):
                question_tw_id = param_values['CompanyTwitterURL']
            else:
                question_tw_id = False
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nATTENTION pas de question ID sur Twitter définie\n' )
            if not BD_DE_TEST:
                if param_values.has_key('PopupTest'):
                    test = param_values['PopupTest']
            nom_table_secteur = test and 'secteurs_test' or 'secteurs'
            nom_table_stand = test and 'stand_test' or 'stand'
            #
            function_categs = [('G',u'DIRECTION GÉNÉRALE'),('$',u'DIRECTION GÉNÉRALE'),('K',u'DIRECTION'),('D',u'DIRECTION'),
                               ('B',u'ADMINISTRATIF ET FINANCIER'),('F',u'ADMINISTRATIF ET FINANCIER'),('J',u'ADMINISTRATIF ET FINANCIER'),
                               ('A',u'ACHAT'),('M',u'MARKETING ET COMMERCIAL'),('C',u'MARKETING ET COMMERCIAL'),
                               ('U',u'COMMUNICATION'),('P',u'COMMUNICATION'),('R',u'COMMUNICATION'),('E',u'RH'),
                               ('I',u'IT'),('O',u'TECHNIQUE'),('Q',u'TECHNIQUE'),('T',u'TECHNIQUE'),
                               ('S',u'SECRETARIAT'),('Z',u'LOGISTIQUE')]
            undefined_categ = u'Autres'
            if not BD_DE_TEST:
                # Connection to the external db
                try:
                    con = pymysql.connect(host=connection['host'], port=connection['port'], user=connection['user'], passwd=connection['pw'], db=connection['db']);
                    cur = con.cursor()
                except pymysql.err.Error, e:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Connection Error %d: %s' % (e.args[0],e.args[1]))
                    final_result = False
                    if cur:
                        cur.close()
                    if con:
                        con.close()
            if final_result:
                changes = False
                email_problems = []
                # 1. Synchronise the secteur table and keep mysql-ids
                sector_obj = self.pool.get('res.partner.activsector')
                sector_ids = sector_obj.search(cr,uid,[])
                sectors = sector_obj.read(cr,uid,sector_ids,['id','name','code'],{'lang':'fr_FR'})
                try:
                    sentence = "SELECT code, denomination, id FROM `%s`" % nom_table_secteur
                    if BD_DE_TEST:
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,sentence+'\n')
                    else:
                        cur.execute(sentence)
                except pymysql.err.Error, e:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Connection Error %d: %s' % (e.args[0],e.args[1]))
                    final_result = False
                if BD_DE_TEST:
                    data = []
                else:
                    data = cur.fetchall()
                dSectors = {}
                dSectCode2ID = {} # usefull to res.partner.category->id to res.partner.activsector->id
                for line in data:
                    dSectors[line[0]] = {'id':line[2],'denomination':line[1]}
                    dSectCode2ID[line[0]] = line[2]
                # Table to convert res.partner.category->id to res.partner.activsector->id
                (dOldSect2New,old_categ_ids) = self.convert_old_sector_ids_to_new_ids(cr,uid,dSectCode2ID) # table to convert id from res.partner_category to id in res.partner.activsector AND list of valid res.partner.category ids
                #
                dSectID_ERP_MYSQL = {}
                dSectCode_MYSQL = {}
                for sector in sectors:
                    if not dSectors.has_key(sector['code']):
                        sentence = "INSERT INTO `%s` ( code, denomination ) VALUES ( '%s', '%s')" % (nom_table_secteur,sector['code'],sector['name'].replace("'","''").replace("\\","\\\\"))
                        if BD_DE_TEST:
                            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,sentence+'\n')
                            new_id = 9999
                        else:
                            cur.execute(sentence)
                            new_id = cur.lastrowid
                        if new_id > 0:
                            changes = True
                            if not BD_DE_TEST:
                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion du secteur %s reussie -> %s' % (str(sector['id']),str(new_id)) )
                            dSectID_ERP_MYSQL[sector['id']] = new_id
                        else:
                            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion du secteur %s ratée' % str(sector['id']) )
                    else:
                        dSectID_ERP_MYSQL[sector['id']] = dSectors[sector['code']]['id']
                        if dSectors[sector['code']]['denomination'].decode('iso-8859-1') != sector['name']:
                            sentence = u"UPDATE `%s` SET denomination = '%s' WHERE code = '%s'" % (nom_table_secteur,sector['name'].replace("'","''").replace("\\","\\\\"),sector['code'])
                            if BD_DE_TEST:
                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,sentence+'\n')
                            else:
                                cur.execute(sentence)
                            changes = True
                            #self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nMise à jour du secteur [%s] reussie -> %s' % (str(dSectors[sector['code']]['id']),sector['name']) )
                if not BD_DE_TEST:
                    if cur:
                        con.commit()
                        cur.close()
                # 2. Synchronise to MYSQL the events definies as popup recordings
                #TEMP
                #updates = []
                mySQL_reg_ids =  []
                if not BD_DE_TEST:
                    cur = con.cursor()
                    try:
                        sentence = "SELECT id_stand, erp_reg_id FROM `%s`" % nom_table_stand
                        cur.execute(sentence)
                    except pymysql.err.Error, e:
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Connection Error %d: %s' % (e.args[0],e.args[1]))
                        final_result = False
                    data = cur.fetchall()
                else:
                    data = []
                dExistingStands = {}
                for line in data:
                    dExistingStands[line[1]] = line[0]
                    if line[1] and line[1] > 0:
                        mySQL_reg_ids.append(line[1])
                stand_obj = self.pool.get('cci.popup.stand')
                stand_ids = stand_obj.search(cr,uid,[])
                all_stand_ids = [x for x in stand_ids]
                stand_ids = stand_obj.search(cr,uid,[('web_id','=',False)])
                current_stand_ids = [x for x in stand_ids]
                if stand_ids:
                    stands = stand_obj.browse(cr,uid,stand_ids,{'lang':'fr_FR'})
                    for stand in stands:
                        if not dExistingStands.has_key(stand.id):
                            # new stand to insert into mysql
                            if stand.stand_contact_id and stand.partner_id:
                                # search for the address and the job that links the stand.partner_id and stand.order_contact_id
                                job = False
                                current_sequence_contact = 999
                                if stand.stand_contact_id.job_ids:
                                    for possible_job in stand.stand_contact_id.job_ids:
                                        if possible_job.address_id and possible_job.address_id.partner_id and possible_job.address_id.partner_id.id == stand.partner_id.id:
                                            if possible_job.sequence_contact < current_sequence_contact:
                                                job = possible_job
                                to_continue = False
                                if job:
                                    address = job.address_id
                                    partner = address.partner_id
                                    contact = stand.stand_contact_id
                                    to_continue = True
                                else:
                                    # not job found, we extract default address from the partner and data from the contact
                                    address = False
                                    for addr in stand.partner_id.address:
                                        if addr.type == 'default':
                                            address = addr
                                            break
                                    if address:
                                        partner = stand.partner_id
                                        contact = stand.stand_contact_id
                                        to_continue = True
                                if to_continue:
                                    if address.name:
                                        if address.name[0:3] == ' - ':
                                            partner_full_name = partner.name + address.name
                                        elif address.name[0:2] == '- ':
                                            partner_full_name = partner.name + ' ' + address.name
                                        else:
                                            partner_full_name = address.name
                                    else:
                                        partner_full_name = partner.name
                                    addr_fb = ''
                                    if question_fb_id:
                                        for answer in partner.answers_ids:
                                            if answer.question_id.id == question_fb_id:
                                                addr_fb = answer.text or ''
                                    addr_li = ''
                                    if question_li_id:
                                        for answer in partner.answers_ids:
                                            if answer.question_id.id == question_li_id:
                                                addr_li = answer.text or ''
                                    addr_tw = ''
                                    if question_tw_id:
                                        for answer in partner.answers_ids:
                                            if answer.question_id.id == question_tw_id:
                                                addr_tw = answer.text or ''
                                    sector1 = 0
                                    sector2 = 0
                                    sector3 = 0
                                    if address.sector1:
                                        # we take the sectors from the address
                                        if dSectID_ERP_MYSQL.has_key(address.sector1.id):
                                            sector1 = dSectID_ERP_MYSQL[address.sector1.id]
                                        if address.sector2 and dSectID_ERP_MYSQL.has_key(address.sector2.id):
                                            sector2 = dSectID_ERP_MYSQL[address.sector2.id]
                                        if address.sector3 and dSectID_ERP_MYSQL.has_key(address.sector3.id):
                                            sector3 = dSectID_ERP_MYSQL[address.sector3.id]
                                    else:
                                        # we get them from the partner's categories
                                        new_sector_ids = []
                                        for category in partner.category_id:
                                            categ_id = category.id
                                            if categ_id in old_categ_ids and dOldSect2New.has_key(categ_id):
                                                new_sector_ids.append(dOldSect2New[categ_id])
                                        if len(new_sector_ids) > 0:
                                            sector1 = new_sector_ids[0]
                                            if len(new_sector_ids) > 1:
                                                sector2 = new_sector_ids[1]
                                                if len(new_sector_ids) > 2:
                                                    sector3 = new_sector_ids[2]
                                    zip_code = address.zip_id and address.zip_id.name or ''
                                    if len(zip_code) > 12:
                                        zip_code = '0'
                                    user_phone = ''
                                    user_mobile = ''
                                    if job and job.phone and not 'conf' in job.phone:
                                        user_phone = job.phone
                                    if contact.mobile and not 'conf' in contact.mobile:
                                        user_mobile = contact.mobile
                                    function_categ = ''
                                    if job:
                                        this_job_code_labels = (job.function_code_label or '')
                                        for (letter,fcateg) in function_categs:
                                            if letter in this_job_code_labels:
                                                function_categ = fcateg
                                                break
                                    if not function_categ:
                                        function_categ = undefined_categ
                                    if job:
                                        final_badge_title = job.function_label or ''
                                    else:
                                        final_badge_title = ''
                                    if stand.stand_reg_id and stand.stand_reg_id.badge_title:
                                        final_badge_title = stand.stand_reg_id.badge_title
                                    sentence = """INSERT INTO `%s` ( user_level, guichet, user_mail, user_mdp, user_civ, user_nom, user_prenom, user_fonction, user_fonction_plus, user_tel, user_gsm, 
                                                      part_tva, part_nom, part_adresse, part_adresse_plus, part_adresse_cp, part_adresse_loca, part_adresse_pays, part_tel, part_email, 
                                                      part_site, part_linkedin, part_facebook, part_twitter, part_description, part_logo, part_s1, part_s2, part_s3, erp_partner_id, erp_reg_id, erp_type_stand, erp_adresse_id, 
                                                      erp_effectif_tot, max_part_expo, max_part_invit, last_action, stand_pos1, stand_pos2, stand_pos3, actif, screen, video, bache, notif, to_ignore ) 
                                                  VALUES ( 1, 0, '%s', '', '%s', '%s', '%s', '%s', '%s', '%s', '%s',
                                                           '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s',
                                                           '%s', '%s', '%s', '%s', '%s', '%s', %i, %i, %i, %i, %i, %i, %i, 
                                                           %s, %i, %i, 0, '', '', '', 0, %i, 0, 0, 0, 0 )""" % ( nom_table_stand, 
                                                                                 stand.email_stand or '',
                                                                                 (contact.title or '').replace("'","''").replace("\\","\\\\"),
                                                                                 (contact.name or '').replace("'","''").replace("\\","\\\\"),
                                                                                 (contact.first_name or '').replace("'","''").replace("\\","\\\\"),
                                                                                 (function_categ).replace("'","''").replace("\\","\\\\"),
                                                                                 final_badge_title.replace("'","''").replace("\\","\\\\"),
                                                                                 correct_phone(user_phone),
                                                                                 correct_phone(user_mobile),
                                                                                 partner.vat or '',
                                                                                 (stand.name or partner_full_name).replace("'","''").replace("\\","\\\\"),
                                                                                 (address.street or '').replace("'","''").replace("\\","\\\\"),
                                                                                 (address.street2 or '').replace("'","''").replace("\\","\\\\"),
                                                                                 zip_code,
                                                                                 (address.zip_id and address.zip_id.city or '').replace("'","''").replace("\\","\\\\"),
                                                                                 (address.country_id and address.country_id.name or 'Belgique'),
                                                                                 (correct_phone(address.phone or '')).replace("'","''").replace("\\","\\\\"),
                                                                                 (address.email or '').replace("'","''").replace("\\","\\\\"),
                                                                                 (partner.website or '').replace("'","''").replace("\\","\\\\"),
                                                                                 (addr_li).replace("'","''").replace("\\","\\\\"),
                                                                                 (addr_fb).replace("'","''").replace("\\","\\\\"),
                                                                                 (addr_tw).replace("'","''").replace("\\","\\\\"),
                                                                                 (address.activity_description or partner.activity_description or '').replace(u"’","'").replace("'","''").replace("\\","\\\\").replace(u'€','euros'),
                                                                                 '',
                                                                                 sector1,
                                                                                 sector2,
                                                                                 sector3,
                                                                                 partner.id,
                                                                                 stand.id,
                                                                                 stand.type_id.web_type,
                                                                                 address.id,
                                                                                 str(partner.employee_nbr_total or 0),
                                                                                 stand.type_id.default_exhibitors or 0,
                                                                                 stand.type_id.default_guests or 0,
                                                                                 stand.type_id.default_screen or 0)
                                                                                 # (address.activity_description or partner.activity_description or '').replace(u"’","'").replace("'","''").replace("\\","\\\\").replace(u'€','euros'),
                                    final_result = True
                                    if BD_DE_TEST:
                                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,sentence+'\n')
                                    else:
                                        try:
                                            cur.execute(sentence)
                                            changes = True
                                        except pymysql.err.Error, e:
                                            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nConnection Error %d: %s' % (e.args[0],e.args[1]))
                                            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n' + sentence )
                                            final_result = False
                                        except:
                                            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nSouci lors de l''ajout de cette phrase')
                                            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n' + sentence )
                                            final_result = False
                                    if final_result:
                                        if BD_DE_TEST:
                                            new_id = 99999
                                        else:
                                            new_id = cur.lastrowid
                                        if new_id > 0:
                                            if not BD_DE_TEST:
                                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion stand [%s] réussie -> %s' % (str(stand.id),str(new_id)) )
                                                stand_obj.write(cr,uid,[stand.id,],{'web_id':new_id})
                                        else:
                                            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion stand [%s] ratée' % str(stand.id) )
                                            email_problems.append('<li>Insertion stand [%s] echoue dans MySQL.</li>' % str(stand.id) )
                                        # Injection of additional exhibitors and/or visitors
                                        if stand.order_add_exhibitor:
                                            ctr = 1
                                            while ctr  <= stand.order_add_exhibitor:
                                                sentence = """INSERT INTO `exposants` ( id_exposant, id_stand, cle, used, civilite, nom, prenom, mail, fonction, fonction_plus, telephone, gsm, 
                                                                                       sent, print, direct, login, mdp, app, send_to_print, scan_in, facture, access ) 
                                                              VALUES ( 0, '%i', '', '0', '', '', '', '', '', '', '', '', 
                                                                       '0', '0', '0', '', '', 0, 0, 0, '%s', '' )""" % ( new_id,stand.price_add_exhibitor and 'sent' or 'free' )
                                                final_result = True
                                                if BD_DE_TEST:
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,sentence+'\n')
                                                else:
                                                    try:
                                                        cur.execute(sentence)
                                                        changes = True
                                                    except pymysql.err.Error, e:
                                                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nConnection Error %d: %s' % (e.args[0],e.args[1]))
                                                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n' + sentence )
                                                        final_result = False
                                                    except:
                                                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nSouci lors de l''ajout de cette phrase')
                                                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n' + sentence )
                                                        final_result = False
                                                if final_result and  not BD_DE_TEST:
                                                    if new_id > 0:
                                                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion exposant vierge réussie')
                                                    else:
                                                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion exposant vierge ratée')
                                                        email_problems.append(u'<li>Insertion exposant vierge stand [%s] échoué dans MySQL.</li>' % str(stand.id) )
                                                ctr += 1
                                        if stand.order_add_guest:
                                            ctr = 1
                                            while ctr  <= stand.order_add_guest:
                                                sentence = """INSERT INTO `visiteurs` ( id_visiteur, id_stand, cle, used, civilite, nom, prenom, mail_perso, fonction, fonction_plus, tel, gsm, 
                                                                                        send, print, direct, send_to_print, scan_in, facture, access, type ) 
                                                              VALUES ( 0, '%i', '', '0', '', '', '', '', '', '', '', '', 
                                                                       '0', '0', '0', 0, 0, '%s', '', 'invite' )""" % ( new_id,stand.price_add_guest and 'sent' or 'free')
                                                final_result = True
                                                if BD_DE_TEST:
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,sentence+'\n')
                                                else:
                                                    try:
                                                        cur.execute(sentence)
                                                        changes = True
                                                    except pymysql.err.Error, e:
                                                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nConnection Error %d: %s' % (e.args[0],e.args[1]))
                                                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n' + sentence )
                                                        final_result = False
                                                    except:
                                                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nSouci lors de l''ajout de cette phrase')
                                                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n' + sentence )
                                                        final_result = False
                                                if final_result and  not BD_DE_TEST:
                                                    if new_id > 0:
                                                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion visiteur vierge réussie')
                                                    else:
                                                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion visiteur vierge ratée')

                                                        email_problems.append(u'<li>Insertion visiteur vierge stand [%s] échoué dans MySQL.</li>' % str(stand.id) )
                                                ctr += 1
                                        if stand.order_night:
                                            sprice = '%.2f' % (stand.price_night or 0.0)
                                            sentence = u"""INSERT INTO `tickets` ( id_facture, id_stand, libelle, prix, quantite, date, facture ) 
                                                          VALUES ( 0, %i, 'Soirée', %s, %i, %i, '%s' )""" % ( new_id, 
                                                                                                                sprice,
                                                                                                                stand.order_night,
                                                                                                                time.mktime(datetime.datetime.now().timetuple()),
                                                                                                                stand.price_night and 'sent' or 'free' )
                                            final_result = True
                                            if BD_DE_TEST:
                                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,sentence+'\n')
                                            else:
                                                try:
                                                    cur.execute(sentence)
                                                    changes = True
                                                except pymysql.err.Error, e:
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nConnection Error %d: %s' % (e.args[0],e.args[1]))
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n' + sentence )
                                                    final_result = False
                                                except:
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nSouci lors de l''ajout de cette phrase')
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n' + sentence )
                                                    final_result = False
                                            if final_result and  not BD_DE_TEST:
                                                if new_id > 0:
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion badge soirée réussie')
                                                else:
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion badge soirée ratée')
                                                    email_problems.append(u'<li>Insertion badges soirée stand [%s] échoué dans MySQL.</li>' % str(stand.id) )
                                        if stand.type_id and stand.type_id.default_night_tickets:
                                            sentence = u"""INSERT INTO `tickets` ( id_facture, id_stand, libelle, prix, quantite, date, facture ) 
                                                          VALUES ( 0, %i, 'Soirée', %s, %i, %i, 'free' )""" % ( new_id, 
                                                                                                                '0.00',
                                                                                                                stand.type_id.default_night_tickets,
                                                                                                                time.mktime(datetime.datetime.now().timetuple()) )
                                            final_result = True
                                            if BD_DE_TEST:
                                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,sentence+'\n')
                                            else:
                                                try:
                                                    cur.execute(sentence)
                                                    changes = True
                                                except pymysql.err.Error, e:
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nConnection Error %d: %s' % (e.args[0],e.args[1]))
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n' + sentence )

                                                    final_result = False
                                                except:
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nSouci lors de l''ajout de cette phrase')
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n' + sentence )
                                                    final_result = False
                                            if final_result and  not BD_DE_TEST:
                                                if new_id > 0:
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion badge soirée réussie')
                                                else:
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion badge soirée ratée')
                                                    email_problems.append(u'<li>Insertion badges soirée gratuits stand [%s] échoué dans MySQL.</li>' % str(stand.id) )
                                        if stand.type_id and stand.type_id.default_champagne > 0:
                                            sentence = u"""INSERT INTO `tickets` ( id_facture, id_stand, libelle, prix, quantite, date, facture ) 
                                                          VALUES ( 0, %i, 'De Castellane', %s, %i, %i, 'free' )""" % ( new_id, 
                                                                                                                       '60.50',
                                                                                                                       stand.type_id.default_champagne,
                                                                                                                       time.mktime(datetime.datetime.now().timetuple()) )
                                            final_result = True
                                            if BD_DE_TEST:
                                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,sentence+'\n')
                                            else:
                                                try:
                                                    cur.execute(sentence)
                                                    changes = True
                                                except pymysql.err.Error, e:
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nConnection Error %d: %s' % (e.args[0],e.args[1]))
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n' + sentence )
                                                    final_result = False
                                                except:
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nSouci lors de l''ajout de cette phrase')
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\n' + sentence )
                                                    final_result = False
                                            if final_result and  not BD_DE_TEST:
                                                if new_id > 0:
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion badge soirée réussie')
                                                else:
                                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion badge soirée ratée')
                                                    email_problems.append(u'<li>Insertion Champagne stand [%s] échoué dans MySQL.</li>' % str(stand.id) )
                                else:
                                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nStand [%s] impossible to send to Mysql : no link between contact and partner' % str(stand.id) )
                                    email_problems.append('<li>Impossible d\'envoyer le stand [%s] dans Mysql : pas de lien entre le partenaire et le contact.</li>' % str(stand.id) )
                            else:
                                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nStand [%s] impossible to send to Mysql : no partner or no contact' % str(stand.id) )
                                email_problems.append('<li>Impossible d\'envoyer le stand [%s] dans Mysql : le partenaire ou le contact manque.</li>' % str(stand.id) )
                        else:
                            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nStand [%s] already existing on platform but link not registered' % str(stand.id) )
                else:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nNo new stands to synchronize')
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nEnd of main loop')
                if not BD_DE_TEST:
                    if final_result and changes:
                        con.commit()
                    if cur:
                        cur.close()
                    if con:
                        con.close()
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nChecking deleted')
                # check if deleted or canceled registrations not still in MySQL
                for mySQL_id in mySQL_reg_ids:
                    if mySQL_id not in all_stand_ids:
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nStand ID [%s] still existing in MySQL while erased or canceled in ERP' % str(mySQL_id) )
                        email_problems.append(u'<li>Le stand ID [%s] existe encore dans MySQL alors qu\'elle a été éffacé.</li>' % str(mySQL_id) )
                # envoi de l'email au superviseur
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nCheck if we need to send email')
                if email_problems and supervisor_emails:
                    if test:
                        supervisor_emails = ['philmer.vdm@gmail.com']
                    #if BD_DE_TEST:
                    if True:
                        print 'From : no-reply@ccilvn.be'
                        print 'To : ' + str(supervisor_emails)
                        print 'Content:'
                        print '\n'.join(email_problems)
                        #print u"<p>La mise à jour de ce matin a eu quelques problèmes :</p><p><ul>%s</ul></p>" % ('\n'.join(email_problems))
                    else:
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nTrying to send mail')
                        tools.email_send('no-reply@ccilvn.be', 
                                         supervisor_emails, 
                                         u'Mise à jour exposants Pop Up', 
                                         u'<p>La mise à jour de ce matin a eu quelques problèmes :</p><p><ul>%s</ul></p>' % ('\n'.join(email_problems)), 
                                         subtype='html' )
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nEmail sended')
        else:
            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nNothing done - some parameters were missing')
            final_result = False
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return True
cci_popup_stand()

