# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import datetime
import wizard
import netsvc
import pooler
import pymysql
import base64
from pyExcelerator import *
from osv import fields, osv
import csv

init_form = """<?xml version="1.0"?>
<form string="Create Invoices from Popup Website" height="400">
    <field name="invoice_date" colspan="4" width="600"/>
    <field name="invoice_options" colspan="4"/>
    <field name="invoice_expo_invit" colspan="4"/>
    <field name="extra_invit" colspan="4"/>
    <field name="mark_as_invoiced" colspan="4"/>
</form>
"""
init_fields = {
    'invoice_date': {'string':u'Invoice Date', 'type':'date','required':True},
    'invoice_options': {'string':u'Invoice Options (tickets, champagne)', 'type':'boolean','default':False},
    'invoice_expo_invit': {'string':u'Invoice Persons (exposants et invités)', 'type':'boolean', 'default':False},
    'extra_invit': {'string':u'Get Extra Persons for invoicing', 'type':'boolean', 'default':False,'help':'The persons not linked to a stand but to bill one by one, manually.'},
    'mark_as_invoiced': {'string':u'Mark as Invoiced', 'type':'boolean', 'default':False,'help':'Don\'t check it while testing.'},
}

form = """<?xml version="1.0"?>
<form string="Invoices created from Popup Website">
    <newline />
    <field name="inv_created"/>
    <newline />
    <field name="inv_rejected"/>
    <newline />
    <field name="invit_popup_2_inv.xls"/>
    <newline />
    <field name="inv_rej_reason" width="400"/>
</form>
"""

fields = {
    'inv_created': {'string':'Invoice Created', 'type':'char', 'readonly':True},
    'inv_rejected': {'string':'Invoice Rejected', 'type':'char', 'readonly':True},
    'inv_rej_reason': {'string':'Error Messages', 'type':'text', 'readonly':True},
    'invit_popup_2_inv.xls':{'string': 'Prepared file','type': 'binary','readonly': True,},
}

def _correct_vat(cr, uid, vat):
    corrected_vat = vat.strip().replace(' ','').replace('/','').replace('-','').replace('.','').upper()
    if len(corrected_vat ) >= 8:
        if corrected_vat[0:2] == 'BE':
            if len(corrected_vat) == 12:
                pass
            elif len(corrected_vat) == 11:
                corrected_vat = 'BE0'+corrected_vat[2:]
            else:
                corrected_vat = ''
        else:
            if corrected_vat[0:2].isdigit():
                if len(corrected_vat) == 10:
                    corrected_vat = 'BE' + corrected_vat
                elif corrected_vat == 9:
                    corrected_vat = 'BE0' + corrected_vat
                else:
                    corrected_vat = ''
            else:
                # suppose stranger VAT number : let it untouched
                corrected_vat = corrected_vat
        if len(corrected_vat) == 12 and corrected_vat[0:2] == 'BE':
            # belgian complete vat number => check digit
            try:
                fullnum = int(corrected_vat[2:])
            except:
                corrected_vat = ''
                fullnum = 0
            if fullnum:
                if int(corrected_vat[-2:]) != (97 - (int(corrected_vat[2:10]) % 97)):
                    corrected_vat = ''
    else:
        corrected_vat = ''
    return corrected_vat
    
def _make_invoices(self, cr, uid, data, context):
    inv_create = 0
    inv_reject = 0
    inv_rej_reason = ""
    list_inv = []
    final_result = True
    pool_obj = pooler.get_pool(cr.dbname)
    parameter_obj = pool_obj.get('cci_parameters.cci_value')
    param_values = parameter_obj.get_value_from_names(cr,uid,['PopupConnectionDict']) #     param_values = parameter_obj.get_value_from_names(cr,uid,['PopupConnectionDict','PopupInvoicedIDS'])
    if param_values.has_key('PopupConnectionDict'): #     if param_values.has_key('PopupConnectionDict') and param_values.has_key('PopupInvoicedIDS'):
        connection = param_values['PopupConnectionDict']
        #invoiced_ids = param_values['PopupInvoicedIDS']
        # Connection to the external db
        cur = False
        con = False
        try:
            con = pymysql.connect(host=connection['host'], port=connection['port'], user=connection['user'], passwd=connection['pw'], db=connection['db']);
            cur = con.cursor()
        except pymysql.err.Error, e:
            inv_rej_reason = 'Connection Error %d: %s\n' % (e.args[0],e.args[1])
            final_result = False
            if cur:
                cur.close()
            if con:
                con.close()
        if final_result:
            lines_to_inv = []
            partner_ids = []
            if data['form']['invoice_options']:
                db_tables = [('tickets','TVAC21','T'),]
                for table in db_tables:
                    nom_table = table[0]
                    type_prix = table[1]
                    initial = table[2]
                    # 1. Read the Table
                    try:
                        sentence = "select st.erp_partner_id, so.id_facture, so.id_stand, so.libelle, so.prix, so.quantite, so.facture FROM %s as so, stand as st WHERE so.id_stand = st.id_stand AND so.facture = 'topay' AND st.to_ignore IS NOT NULL" % nom_table
                        cur.execute(sentence)
                    except pymysql.err.Error, e:
                        inv_rej_reason += 'Reading %s Error %d: %s\n' % (nom_table,e.args[0],e.args[1])
                        final_result = False
                    if final_result:
                        lines = cur.fetchall()
                        for line in lines:
                            #if initial+str(line[1]) not in invoiced_ids:
                            if line[0] > 0 and float(line[4]) > 0.01 and int(line[5]) > 0:
                                new_line = {'id':initial+str(line[1]), 'partner_id':line[0], 'stand_id':line[2], 'product_key':line[3], 'price': float(line[4]), 'quantity': line[5], 'to_invoice': line[6]}
                                lines_to_inv.append(new_line)
                                if line[0] not in partner_ids:
                                    partner_ids.append(line[0])
                try:
                    sentence = "SELECT st.erp_partner_id, tp.id_stand, tp.facture, COUNT(tp.id_stand) FROM `tickets_parking` as tp, stand as st WHERE tp.id_stand = st.id_stand AND tp.facture = 'topay' GROUP BY id_stand"
                    cur.execute(sentence)
                except pymysql.err.Error, e:
                    inv_rej_reason += 'Reading %s Error %d: %s\n' % (nom_table,e.args[0],e.args[1])
                    final_result = False
                if final_result:
                    lines = cur.fetchall()
                    for line in lines:
                        if line[0] > 0:
                            new_line = {'id':'P'+str(line[1]), 'partner_id':line[0], 'stand_id':line[1], 'product_key':u'Parking', 'price': 10.0, 'quantity': line[3], 'to_invoice': line[2]}
                            lines_to_inv.append(new_line)
                            if line[0] not in partner_ids:
                                partner_ids.append(line[0])
                try:
                    sentence = "SELECT st.erp_partner_id, wifi.id_stand, wifi.facture, COUNT(wifi.id_stand) FROM `wifi` as wifi, stand as st WHERE wifi.id_stand = st.id_stand AND wifi.facture = 'topay' GROUP BY id_stand"
                    cur.execute(sentence)
                except pymysql.err.Error, e:
                    inv_rej_reason += 'Reading %s Error %d: %s\n' % (nom_table,e.args[0],e.args[1])
                    final_result = False
                if final_result:
                    lines = cur.fetchall()
                    for line in lines:
                        if line[0] > 0:
                            new_line = {'id':'W'+str(line[1]), 'partner_id':line[0], 'stand_id':line[1], 'product_key':u'Wifi', 'price': 10.0, 'quantity': line[3], 'to_invoice': line[2]}
                            lines_to_inv.append(new_line)
                            if line[0] not in partner_ids:
                                partner_ids.append(line[0])
            if (not inv_rej_reason) and data['form']['invoice_expo_invit']:
                # Read the max_part_expo and max_part_invit from the stands
                try:
                    sentence = "select id_stand, max_part_expo, max_part_invit, erp_partner_id FROM stand WHERE to_ignore = 0"
                    cur.execute(sentence)
                    dStands = {}
                    lines = cur.fetchall()
                    for line in lines:
                        dStands[line[0]] = {'max_part_expo':0,'max_part_invit':0,'used_expo':0,'used_invit':0,'invoice_expo':0,'invoice_invit':0,'erp_partner_id':line[3],'list_ids':[]}
                except pymysql.err.Error, e:
                    inv_rej_reason += 'Reading Stands Error %d: %s\n' % (e.args[0],e.args[1])
                    final_result = False
            if (not inv_rej_reason) and data['form']['invoice_expo_invit']:
                hfOutE = csv.writer(open('new_invoiced_expo.csv','w'),delimiter=";",quotechar='"',quoting=csv.QUOTE_NONNUMERIC,lineterminator='\n')
                hfOutE.writerow(['stand_id','count_expo'])
                hfOutI = csv.writer(open('new_invoiced_invit.csv','w'),delimiter=";",quotechar='"',quoting=csv.QUOTE_NONNUMERIC,lineterminator='\n')
                hfOutI.writerow(['stand_id','count_visit'])
                db_tables2 = [('exposants','Badge exposant supplementaire','E','id_exposant',25.0,'expo'),('visiteurs','Badge invite supplementaire','I','id_visiteur',25.0,'invit')]
                for table in db_tables2:
                    nom_table = table[0]
                    type_prod = table[1]
                    initial = table[2]
                    key_name = table[3]
                    unit_price = table[4]
                    count_name = table[5]
                    # this first version use max_part_expo but this isn't the good choice :
                    # we have to calculate the 'possible' with the sum of 'in' and 'free' records
                    # then substract 'used' ones
                    # 1. Read the Table
                    #try:
                    #    sentence = "select st.erp_partner_id, so.%s, so.id_stand, so.facture FROM %s as so, stand as st WHERE so.id_stand = st.id_stand AND so.facture in ('topay','in') AND so.used > 10000 AND st.to_ignore IS NULL" % (key_name, nom_table)
                    #    cur.execute(sentence)
                    #except pymysql.err.Error, e:
                    #    inv_rej_reason += 'Reading %s Error %d: %s\n' % (nom_table,e.args[0],e.args[1])
                    #    final_result = False
                    #if final_result:
                    #    lines = cur.fetchall()
                    #    for line in lines:
                    #        if line[2] and line[2] in dStands.keys():
                    #            field_count_name = 'count_' + line[3] + '_' + count_name
                    #            dStands[line[2]][field_count_name] += 1
                    #            dStands[line[2]]['list_ids'].append(initial+str(line[1]))
                    # 1. Read the Table
                    #    Count the number of expo/invits authorized
                    final_result = True
                    try:
                        sentence = "select st.erp_partner_id, so.%s, so.id_stand, so.facture FROM %s as so, stand as st WHERE so.id_stand = st.id_stand AND so.facture in ('free','in') AND st.to_ignore = 0" % (key_name, nom_table)
                        cur.execute(sentence)
                    except pymysql.err.Error, e:
                        inv_rej_reason += 'Reading %s Error %d: %s\n' % (nom_table,e.args[0],e.args[1])
                        final_result = False
                    if final_result:
                        lines = cur.fetchall()
                        for line in lines:
                            if line[2] and line[2] in dStands.keys():
                                field_count_name = 'max_part_' + table[5]
                                dStands[line[2]][field_count_name] += 1
                                #dStands[line[2]]['list_ids'].append(initial+str(line[1]))
                    #    Count the number of used expo/invits
                    final_result = True
                    try:
                        sentence = "select st.erp_partner_id, so.%s, so.id_stand, so.facture FROM %s as so, stand as st WHERE so.id_stand = st.id_stand AND so.used > 10000 AND st.to_ignore = 0" % (key_name, nom_table)
                        cur.execute(sentence)
                    except pymysql.err.Error, e:
                        inv_rej_reason += 'Reading %s Error %d: %s\n' % (nom_table,e.args[0],e.args[1])
                        final_result = False
                    if final_result:
                        lines = cur.fetchall()
                        for line in lines:
                            if line[2] and line[2] in dStands.keys():
                                field_count_name = 'used_' + table[5]
                                dStands[line[2]][field_count_name] += 1
                                dStands[line[2]]['list_ids'].append(initial+str(line[1]))
                for (id_stand,stand) in dStands.items():
                    # TO DO : way to calculate paid persons
                    if stand['used_expo'] > 0 or stand['used_invit'] > 0:
                        stand['invoice_expo'] = max(0,stand['used_expo']-stand['max_part_expo'])
                        stand['invoice_invit'] = max(0,stand['used_invit']-stand['max_part_invit'])
                    if stand['invoice_expo'] > 0:
                        hfOutE.writerow([id_stand,stand['invoice_expo']])
                        # 'S' for stand id
                        new_line = {'id':'S'+str(id_stand), 'partner_id':stand['erp_partner_id'], 'stand_id':id_stand, 'product_key':u'Badge exposant supplementaire', 'price': 25.0, 'quantity': stand['invoice_expo'], 'to_invoice': 'topay'}
                        lines_to_inv.append(new_line)
                        if stand['erp_partner_id'] not in partner_ids:
                            partner_ids.append(stand['erp_partner_id'])
                    if stand['invoice_invit'] > 0:
                        hfOutI.writerow([id_stand,stand['invoice_invit']])
                        # 'S' for stand id
                        new_line = {'id':'S'+str(id_stand), 'partner_id':stand['erp_partner_id'], 'stand_id':id_stand, 'product_key':u'Badge invite supplementaire', 'price': 25.0, 'quantity': stand['invoice_invit'], 'to_invoice': 'topay'}
                        lines_to_inv.append(new_line)
                        if stand['erp_partner_id'] not in partner_ids:
                            partner_ids.append(stand['erp_partner_id'])
        invoiced_ids = []
        if final_result and (not inv_rej_reason) and lines_to_inv:
            dProds = {}
            dProds[u'Badge exposant supplementaire'] = {'product_id':1292,'type':'HTVA','lib':u'POP UP Marina Bay - Entrées exposant supplémentaires 2019'}
            dProds[u'Badge invite supplementaire'] = {'product_id':1293,'type':'HTVA','lib':u'POP UP Marina Bay - Entrées invité supplémentaires 2019'}
            dProds[u'Tickets'] = {'product_id':1294,'type':'TVAC21','lib':u'POP UP Marina Bay - Tickets boissons 2019'}
            dProds[u'De Castellane'] = {'product_id':1338,'type':'TVAC21','lib':u'POP UP Marina Bay - Bouteille Champagne De Castellane 2019'}
            dProds[u'La Cuvée'] = {'product_id':1339,'type':'TVAC21','lib':u'POP UP Marina Bay - Bouteille Champagne \'La Cuvée\''}
            dProds[u'Cuvée Rosée Brut'] = {'product_id':1340,'type':'TVAC21','lib':u'POP UP Marina Bay - Bouteille \'Cuvée Rosée Brut\' 2019'}
            dProds[u'Parking'] = {'product_id':1294,'type':'HTVA','lib':u'POP UP Marina Bay - Entrées Parking'}
            dProds[u'Wifi'] = {'product_id':1294,'type':'HTVA','lib':u'POP UP Marina Bay - Code accès Wi-Fi'}
            dProds[u'Soirée'] = {'product_id':1291,'type':'TVAC21','lib':u'POP UP Marina Bay - Bracelets entrée Soirée'}
            
            invoices = {}
            create_ids=[]
            pool_obj=pooler.get_pool(cr.dbname)

            obj_lines=pool_obj.get('account.invoice.line')
            obj_partner = pool_obj.get('res.partner')
            obj_inv = pool_obj.get('account.invoice')
            obj_product = pool_obj.get('product.product')
            obj_period = pool_obj.get('account.period')
            invoice_period_id = obj_period.search(cr,uid,[('date_start','<=',data['form']['invoice_date']),('date_stop','>=',data['form']['invoice_date']),('special','=',False)])[0]
            for partner_id in partner_ids:
                partner_ids = obj_partner.search(cr,uid,[('id','=',partner_id)])
                if partner_ids:
                    partner = obj_partner.browse(cr,uid,partner_id)
                    val_invoice = pool_obj.get('account.invoice').onchange_partner_id(cr, uid, [], 'out_invoice', partner_id, False, False)
                    val_invoice['value'].update({'partner_id': partner_id})
                    partner_address_id = val_invoice['value']['address_invoice_id']
                    if not partner_address_id:
                        inv_rej_reason += u"Given partner doesn't have an address to make the invoice.\nInvoice not generated for the partner id %i \n" % partner_id
                        inv_reject += 1
                    else:
                        inv_line_ids = []
                        for line in lines_to_inv:
                            if line['partner_id'] == partner_id and line['to_invoice'] == 'topay':
                                if dProds.has_key(line['product_key']):
                                    conv_prod = dProds[line['product_key']]
                                    product_id = conv_prod['product_id']
                                    type_prix = conv_prod['type']
                                    libelle = conv_prod['lib']
                                    
                                    product = obj_product.browse(cr,uid,product_id)
                                    value = obj_lines.product_id_change(cr, uid, [], product_id,uom =False, partner_id=partner_id, fposition_id=partner.property_account_position.id)
                                    tax_ids=[]
                                    for tax in product.taxes_id:
                                        tax_ids.append(tax.id)
                                    
                                    pack_name = '' ## in case of TVAC prices, we must change the quantity to 'one' to have good results in re-calculation of TVA
                                    
                                    vals = value['value']
                                    new_price_unit = line['price']
                                    if type_prix == 'TVAC21':
                                        new_price_unit = ( new_price_unit * line['quantity'] ) / 1.21
                                        pack_name = ' (Pack %i)' % line['quantity']
                                        line['quantity'] = 1
                                    elif type_prix == 'TVAC6':
                                        new_price_unit = ( new_price_unit * line['quantity'] ) / 1.06
                                        pack_name = ' (Pack %i)' % line['quantity']
                                        line['quantity'] = 1
                                    vals.update({
                                        'name': libelle + pack_name + ' [st:' + str(line['stand_id']) + ']',
                                        'price_unit': new_price_unit,
                                        'quantity': line['quantity'],
                                        'product_id': product_id,
                                        'invoice_line_tax_id': [(6,0,tax_ids)],
                                    })
                                    inv_line_ids.append(obj_lines.create(cr, uid,vals))
                                    invoiced_ids.append(line['id'])
                                else:
                                    inv_rej_reason += "Unknow product : %s\n" % line['product_key']
                        val_invoice['value'].update({
                            'date_invoice':data['form']['invoice_date'],
                            'period_id':invoice_period_id,
                            'origin': '',
                            'reference': False,
                            'invoice_line': [(6,0,inv_line_ids)],
                            'comment': "",
                        })
                        new_id_invoice = obj_inv.create(cr, uid, val_invoice['value'])
                        inv_create += 1
                        list_inv.append(new_id_invoice)
                else:
                    inv_rej_reason += 'The partner %i is not more in the ERP database\n' % partner_id
                    inv_reject += 1
        #new_value = '[' + ','.join("'"+x+"'" for x in invoiced_ids) + ']'
        #parameter_ids = parameter_obj.search(cr,uid,[('name','=','PopupInvoicedIDS')])
        #parameter_obj.write(cr,uid,parameter_ids,{'value':new_value})
        # TO DO set 'sent' in 'facture' of each table
        ticket_ids = []
        parking_ids = []
        wifi_ids = []
        expo_stand_ids = []
        invit_stand_ids = []
        for invoiced_id in invoiced_ids:
            line_type = invoiced_id[0]
            line_id = int(invoiced_id[1:])
            if line_type == 'T':
                ticket_ids.append(line_id)
            elif line_type == 'P':
                parking_ids.append(line_id)
            elif line_type == 'W':
                wifi_ids.append(line_id)
            elif line_type == 'S':
                if dStands[line_id]['invoice_expo'] > 0:
                    expo_stand_ids.append(line_id)
                if dStands[line_id]['invoice_invit'] > 0:
                    invit_stand_ids.append(line_id)
            else:
                inv_rej_reason += 'Strange ID to set sent in DB [%s]\n' % invoiced_id
        if final_result and data['form']['mark_as_invoiced']:
            if ticket_ids:
                sentence_tickets = """UPDATE tickets SET facture = 'sent' WHERE id_facture in (%s) AND facture = 'topay'""" % ','.join(str(x) for x in ticket_ids)
                cur.execute(sentence_tickets)
            if parking_ids:
                sentence_tickets = """UPDATE tickets_parking SET facture = 'sent' WHERE id_facture in (%s) AND facture = 'topay'""" % ','.join(str(x) for x in ticket_ids)
                cur.execute(sentence_tickets)
            if wifi_ids:
                sentence_tickets = """UPDATE wifi SET facture = 'sent' WHERE id_facture in (%s) AND facture = 'topay'""" % ','.join(str(x) for x in ticket_ids)
                cur.execute(sentence_tickets)
            if expo_stand_ids:
                sentence_exposants = """UPDATE exposants SET facture = 'sent' WHERE id_stand in (%s) AND facture = 'topay'""" % ','.join(str(x) for x in expo_stand_ids)
                cur.execute(sentence_exposants)
            if invit_stand_ids:
                sentence_visiteurs = """UPDATE visiteurs SET facture = 'sent' WHERE id_stand in (%s) AND facture = 'topay'""" % ','.join(str(x) for x in invit_stand_ids)
                cur.execute(sentence_visiteurs)
        if final_result and data['form']['extra_invit']:
            wb1 = Workbook()
            ws1 = wb1.add_sheet('Invites isoles pas factures')
            ws1.write(0,0,'ID in popup')
            ws1.write(0,1,'Corrected VAT')
            ws1.write(0,2,'Partner ID OpenERP')
            ws1.write(0,3,'Contact ID OpenERP')
            ws1.write(0,4,'Partner Name')
            ws1.write(0,5,'Partner_VAT')
            ws1.write(0,6,'Partner Address')
            ws1.write(0,7,'Partner Address 2')
            ws1.write(0,8,'Partner Zip Code')
            ws1.write(0,9,'Partner City')
            ws1.write(0,10,'Partner Country')
            ws1.write(0,11,'Partner Phone')
            ws1.write(0,12,'Partner Email')
            ws1.write(0,13,'Contact Courtesy')
            ws1.write(0,14,'Contact Name')
            ws1.write(0,15,'Contact First Name')
            ws1.write(0,16,'Contact Phone')
            ws1.write(0,17,'Contact Mobile')
            ws1.write(0,18,'Contact Email')
            ws1.write(0,19,'Contact Function')
            xl_line = 1
            obj_partner = pool_obj.get('res.partner')
            db_tables3 = [('visiteurs','Visiteur Pop\'Up 2017 Speakeasy 27/4/2017','X','id_visiteur',25.0)]
            for table in db_tables3:
                nom_table = table[0]
                type_prod = table[1]
                initial = table[2]
                key_name = table[3]
                unit_price = table[4]
                # 1. Read the Table
                try:
                    sentence = "select so.%s, so.part_nom, so.part_tva, so.part_adresse, so.part_adresse_plus, so.part_adresse_cp, so.part_adresse_localite, so.part_adresse_pays, so.part_tel, so.part_mail, so.civilite, so.nom, so.prenom, so.tel, so.gsm, so.mail_perso, so.fonction FROM %s as so WHERE so.id_stand IS NULL AND so.facture = 'topay' AND so.used > 10000 AND so.send > 10000" % (key_name, nom_table)
                    cur.execute(sentence)
                except pymysql.err.Error, e:
                    inv_rej_reason += 'Reading %s Error %d: %s\n' % (nom_table,e.args[0],e.args[1])
                    final_result = False
                if final_result:
                    lines = cur.fetchall()
                    for line in lines:
                        record_id = line[0]
                        partner_name = line[1]
                        vat = line[2]
                        street = line[3]
                        street2 = line[4]
                        zip_code = str(line[5])
                        zip_loc = line[6]
                        country = line[7]
                        partner_phone = line[8]
                        partner_email = line[9]
                        courtesy = line[10]
                        contact_name = line[11]
                        contact_fname = line[12]
                        contact_phone = line[13]
                        contact_mobile = line[14]
                        contact_email = line[15]
                        contact_function = line[16]
                        corrected_vat = _correct_vat(vat)
                        partner_id = 0
                        contact_id = 0
                        to_invoice = False
                        if len(correct_vat) >= 8:
                            partner_ids = obj_partner.search(cr,uid,[('vat','=',vat)])
                            if partner_ids and len(partner_ids) == 1:
                                partner_id = partner_ids[0]
                                partner = obj_partner.browse(cr,uid,[partner_id,])[0]
                                for addr in partner.address:
                                    for job in addr.jobs:
                                        if job.contact_id and job.contact_id.name == contact_name.upper() and job.contact_id.first_name.lower() == contact_fname.lower():
                                            contact_id = job.contact_id.id
                        ws1.write(xl_line,0,record_id)
                        ws1.write(xl_line,1,corrected_vat)
                        ws1.write(xl_line,2,partner_id)
                        ws1.write(xl_line,3,contact_id)
                        ws1.write(xl_line,4,partner_name)
                        ws1.write(xl_line,5,vat + " => " + corrected_vat)
                        ws1.write(xl_line,6,street)
                        ws1.write(xl_line,7,street2)
                        ws1.write(xl_line,8,zip_code)
                        ws1.write(xl_line,9,zip_loc)
                        ws1.write(xl_line,10,country)
                        ws1.write(xl_line,11,partner_phone)
                        ws1.write(xl_line,12,partner_email)
                        ws1.write(xl_line,13,courtesy)
                        ws1.write(xl_line,14,contact_name)
                        ws1.write(xl_line,15,contact_fname)
                        ws1.write(xl_line,16,contact_phone)
                        ws1.write(xl_line,17,contact_mobile)
                        ws1.write(xl_line,18,contact_email)
                        ws1.write(xl_line,19,contact_function)
                        xl_line += 1
            wb1.save('invit_popup_2_inv.xls')
            result_file = open('invit_popup_2_inv.xls','rb').read()

            # give the result tos the user
            data['form']['invit_popup_2_inv.xls']=base64.encodestring(result_file)
        if cur:
            con.commit()
            cur.close()
    else:
        inv_rej_reason = u'Manque de valeurs pour les paramètres PopupConnectionDict'
    data['form']['inv_created'] = str(inv_create)
    data['form']['inv_rejected'] = str(inv_reject)
    data['form']['invoice_ids'] = list_inv
    data['form']['inv_rej_reason'] = inv_rej_reason
    return data['form']

class invoice_from_site(wizard.interface):

    def _get_defaults(self, cr, uid, data, context):
        today = datetime.datetime.today()
        data['form']['invoice_date'] = today.strftime('%Y-%m-%d')
        return data['form']
    def _list_invoice(self, cr, uid, data, context):
        pool_obj = pooler.get_pool(cr.dbname)
        model_data_ids = pool_obj.get('ir.model.data').search(cr,uid,[('model','=','ir.ui.view'),('name','=','invoice_form')])
        resource_id = pool_obj.get('ir.model.data').read(cr,uid,model_data_ids,fields=['res_id'])[0]['res_id']
        return {
            'domain': "[('id','in', ["+','.join(map(str,data['form']['invoice_ids']))+"])]",
            'name': 'Invoices',
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'account.invoice',
            'views': [(False,'tree'),(resource_id,'form')],
            'context': "{'type':'out_invoice'}",
            'type': 'ir.actions.act_window'
        }

    states = {
        'init' : {
               'actions' : [_get_defaults],
               'result': {'type': 'form', 'arch': init_form, 'fields': init_fields, 'state':[('end','Cancel'),('invoice','Invoice')]}
            },
        'invoice' : {
               'actions' : [_make_invoices],
               'result': {'type': 'form', 'arch': form, 'fields': fields, 'state':[('end','Ok'),('open','Open')]}
            },
        'open': {
            'actions': [],
            'result': {'type':'action', 'action':_list_invoice, 'state':'end'}
        }

    }

invoice_from_site("invoice_from_site")

