# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import pooler
import time
from tools.translate import _

form = '''<?xml version="1.0"?>
<form string="Sales/Purchases">
    <field name="period_from" colspan="4"/>
    <field name="period_to" colspan="4"/>
    <field name="type" colspan="4"/> 
</form>'''

fields={
        'period_from': {'string':'From', 'type':'many2one', 'relation':'account.period', 'required':True},
        'period_to': {'string':'To', 'type':'many2one', 'relation':'account.period', 'required':True},
        'type':{'string':'Selection', 'type':'selection','selection':[('sales','Sales Only'),('purchases','Purchases Only'),('all','Sales and Purchases')],'required':True}
}

class wizard_sales_purchases(wizard.interface):
    def _get_period_ids(self,cr,uid,period_from_id,period_to_id):
        period_obj = pooler.get_pool(cr.dbname).get('account.period')
        per_from = period_obj.read(cr,uid,period_from_id,['id','date_start','date_stop'])
        per_to = period_obj.read(cr,uid,period_to_id,['id','date_start','date_stop'])
        if per_from['date_start'] > per_to['date_start']:
            perto,per_from = per_from,per_to
        period_ids = period_obj.search(cr,uid,[('date_start','>=',per_from['date_start']),('date_stop','<=',per_to['date_stop']),('special','=',False)])
        return period_ids
    def _open_window_selected_move_lines(self, cr, uid, data, context):
        period_ids = self._get_period_ids(cr,uid,data['form']['period_from'],data['form']['period_to'])
        product_ids = data['ids']
        if data['form']['type'] == 'sales':
            journal_ids = pooler.get_pool(cr.dbname).get('account.journal').search(cr,uid,[('type','=','sale')])
        elif data['form']['type'] == 'purchases':
            journal_ids = pooler.get_pool(cr.dbname).get('account.journal').search(cr,uid,[('type','=','purchase')])
        elif data['form']['type'] == 'all':
            journal_ids = pooler.get_pool(cr.dbname).get('account.journal').search(cr,uid,['|',('type','=','sale'),('type','=','purchase')])
        selection = """SELECT aml.id 
                              FROM account_move_line as aml, account_move as am
                              WHERE aml.move_id = am.id AND am.period_id in (%s) AND aml.product_id in (%s)
                              AND aml.journal_id in (%s) AND aml.state = 'valid' AND am.state = 'posted'
                    """ % (','.join(map(str,period_ids)),','.join(map(str,product_ids)),','.join(map(str,journal_ids)))
        cr.execute(selection)
        res = cr.fetchall()
        mline_ids = []
        for lid in res:
            mline_ids.append(lid[0])
        result = {
            'domain': [('id', 'in', mline_ids)],
            'name': _('Account Move Lines'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'account.move.line',
            'context': {},
            'type': 'ir.actions.act_window'
        }
        return result
    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('view','View Data')]}
        },
        'view': {
            'actions': [],
            'result': {'type': 'action', 'action':_open_window_selected_move_lines, 'state':'end'}
        },
    }
wizard_sales_purchases('cci_product_report.sales_purchases')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
