# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import pooler
from tools.translate import _
from osv import fields,osv
import time

_analytic_form = '''<?xml version="1.0"?>
<form string="make your choice">
    <field name="period_from"/>
    <field name="period_to"/>
    <field name="account_id"/>
    <field name="expand_account_id"/>
    <field name="analytic_account_id1"/>
    <field name="expand_analytic1"/>
    <field name="analytic_account_id2"/>
    <field name="expand_analytic2"/>
    <field name="analytic_account_id3"/>
    <field name="expand_analytic3"/>
</form>'''


_journal_fields = {
    'period_from': {'string':'Period From','type':'many2one','relation':'account.period','required':True},
    'period_to': {'string':'Period To','type':'many2one','relation':'account.period','required':True},
    'account_id': {'string':'General Account','type':'many2one','relation':'account.account','required':False},
    'expand_account_id': {'string':'with sub-accounts','type':'boolean'},
    'analytic_account_id1': {'string':'Analytic Account 1','type':'many2one','relation':'account.analytic.account','required':False},
    'expand_analytic1': {'string':'with sub-accounts','type':'boolean'},
    'analytic_account_id2': {'string':'Analytic Account 2','type':'many2one','relation':'account.analytic.account','required':False},
    'expand_analytic2': {'string':'with sub-accounts','type':'boolean'},
    'analytic_account_id3': {'string':'Analytic Account 3','type':'many2one','relation':'account.analytic.account','required':False},
    'expand_analytic3': {'string':'with sub-accounts','type':'boolean'},
}
class cci_analytic_cross_details3(osv.osv):

    _name='cci.analytic.cross.details3'
    _description = "CCI Account Analytic Line Report Before 2012"

    _columns = {
        'name': fields.char('Name', size=64, readonly=True, select=True),
        'product_id': fields.many2one('product.product','Product', readonly=True, select=True),
        'gl_account_id': fields.many2one('account.account','Account', readonly=True, select=True),
        'move_id': fields.many2one('account.move','Move', readonly=True, select=True),
        'partner_id': fields.many2one('res.partner','Partner', readonly=True, select=True),
        'period_id': fields.many2one('account.period','Period', readonly=True, select=True),
        'account1_id' : fields.many2one('account.analytic.account','Account1'),
        'account2_id' : fields.many2one('account.analytic.account','Account2'),
        'account3_id' : fields.many2one('account.analytic.account','Account3'),
        'amount': fields.float('Amount'),
    }
cci_analytic_cross_details3()

class cci_analytic_cross3(osv.osv_memory):

    _name='cci.analytic.cross3'
    _columns = {
        'period_from':fields.many2one('account.period','Period From',),
        'period_to':fields.many2one('account.period','Period To',),
        'account_id': fields.many2one('account.account','General Account'),
        'expand_account_id': fields.boolean('with sub-accounts'),
        'analytic_plan_id1': fields.many2one('account.analytic.account','Analytic Plan 1'),
        'analytic_account_id1': fields.many2one('account.analytic.account','Analytic Account 1'),
        'expand_analytic1': fields.boolean('with analytic sub-accounts on plan 1'),
        'analytic_plan_id2': fields.many2one('account.analytic.account','Analytic Plan 2'),
        'analytic_account_id2': fields.many2one('account.analytic.account','Analytic Account 2'),
        'expand_analytic2': fields.boolean('with analytic sub-accounts on plan 2'),
        'analytic_plan_id3': fields.many2one('account.analytic.account','Analytic Plan 3'),
        'analytic_account_id3': fields.many2one('account.analytic.account','Analytic Account 3'),
        'expand_analytic3': fields.boolean('with analytic sub-accounts on plan 3'),
    }

    def action_create(self, cr, uid, ids, context=None):
        def get_matching_vals(d, mk):
            return sum([k[1] for k, v in d.items() if (k[0], k[2]) == mk])

        def parent_account_id(account_id):
            parent_account = self.pool.get("account.analytic.account").browse(cr,uid,[account_id])[0]
            while parent_account.parent_id.id != False:
                parent_account = self.pool.get("account.analytic.account").browse(cr,uid,[parent_account.parent_id.id])[0]
            return parent_account

        def get_account_descendants(account_id,prev_list):
            account_obj = self.pool.get("account.account")
            prev_list.append(account_id)
            father = account_obj.browse(cr,uid,account_id)
            if father.child_id:
                for sub_account in father.child_id:
                    prev_list = get_account_descendants(sub_account.id, prev_list )
            return prev_list

        def get_analytic_account_descendants(aaccount_id,prev_list):
            account_obj = self.pool.get("account.analytic.account")
            prev_list.append(aaccount_id)
            father = account_obj.browse(cr,uid,aaccount_id)
            if father.child_ids:
                for sub_account in father.child_ids:
                    prev_list = get_analytic_account_descendants(sub_account.id, prev_list )
            return prev_list

        def parent_account_id_id(account_id):
            res = parent_account_id(account_id)
            return res and res.id

        data_ids = []
        period_from_id = self.browse(cr,uid,ids,context)[0].period_from.id
        period_to_id = self.browse(cr,uid,ids,context)[0].period_to.id
        general_account_id = self.browse(cr,uid,ids,context)[0].account_id.id
        aplan_id1 = self.browse(cr,uid,ids,context)[0].analytic_plan_id1.id
        aplan_id2 = self.browse(cr,uid,ids,context)[0].analytic_plan_id2.id
        aplan_id3 = self.browse(cr,uid,ids,context)[0].analytic_plan_id3.id
        aaccount_id1 = self.browse(cr,uid,ids,context)[0].analytic_account_id1.id
        aaccount_id2 = self.browse(cr,uid,ids,context)[0].analytic_account_id2.id
        aaccount_id3 = self.browse(cr,uid,ids,context)[0].analytic_account_id3.id
        expand_general = self.browse(cr,uid,ids,context)[0].expand_account_id
        expand_analytic1 = self.browse(cr,uid,ids,context)[0].expand_analytic1
        expand_analytic2 = self.browse(cr,uid,ids,context)[0].expand_analytic2
        expand_analytic3 = self.browse(cr,uid,ids,context)[0].expand_analytic3
        details_obj = self.pool.get('cci.analytic.cross.details3')
        move_l_obj = self.pool.get('account.move.line')

        # selection of all account move line id with the two periods and with selected general account if asked
        period_obj = self.pool.get('account.period')
        per_from = period_obj.read(cr,uid,period_from_id,['id','date_start','date_stop'])
        per_to = period_obj.read(cr,uid,period_to_id,['id','date_start','date_stop'])
        if per_from['date_start'] > per_to['date_start']:
            perto,per_from = per_from,per_to
        period_ids = period_obj.search(cr,uid,[('date_start','>=',per_from['date_start']),('date_stop','<=',per_to['date_stop']),('special','=',False)])
        if general_account_id:
            if expand_general:
                account_ids = get_account_descendants(general_account_id,[])
            else:
                account_ids = [general_account_id]
            cr.execute("select id from account_move_line where period_id in (%s) and account_id in (%s)" % (','.join([str(x) for x in period_ids]),','.join([str(x) for x in account_ids])) )
            lines = cr.fetchall()
            general_ids = [x[0] for x in lines]
        else:
            cr.execute("select id from account_move_line where period_id in (%s)" % ','.join([str(x) for x in period_ids]) )
            lines = cr.fetchall()
            general_ids = [x[0] for x in lines]

        # selection of all account move line id linked to the first analytic account
        if aaccount_id1:
            if expand_analytic1:
                aaccount1_ids = get_analytic_account_descendants(aaccount_id1,[])
            else:
                aaccount1_ids = [aaccount_id1]
            cr.execute("select distinct(move_id) from account_analytic_line where account_id in (%s)" % ','.join([str(x) for x in aaccount1_ids]) )
            lines = cr.fetchall()
            analytic1_ids = [x[0] for x in lines]
        else:
            analytic1_ids = []

        # selection of all account move line id linked to the second analytic account
        if aaccount_id2:
            if expand_analytic2:
                aaccount2_ids = get_analytic_account_descendants(aaccount_id2,[])
            else:
                aaccount2_ids = [aaccount_id2]
            cr.execute("select distinct(move_id) from account_analytic_line where account_id in (%s)" % ','.join([str(x) for x in aaccount2_ids]) )
            lines = cr.fetchall()
            analytic2_ids = [x[0] for x in lines]
        else:
            analytic2_ids = []

        # selection of all account move line id linked to the third analytic account
        if aaccount_id3:
            if expand_analytic3:
                aaccount3_ids = get_analytic_account_descendants(aaccount_id3,[])
            else:
                aaccount3_ids = [aaccount_id3]
            cr.execute("select distinct(move_id) from account_analytic_line where account_id in (%s)" % ','.join([str(x) for x in aaccount3_ids]) )
            lines = cr.fetchall()
            analytic3_ids = [x[0] for x in lines]
        else:
            analytic3_ids = []

        # matching between these four possible result sets into a list of unique account_move_line ids
        if aaccount_id1:
            mline_ids = [x for x in set(general_ids).intersection(set(analytic1_ids))]
        else:
            mline_ids = general_ids
        if aaccount_id2:
            mline_ids = [x for x in set(mline_ids).intersection(set(analytic2_ids))]
        if aaccount_id3:
            mline_ids = [x for x in set(mline_ids).intersection(set(analytic3_ids))]

        if mline_ids:
            account_plan1_ids = get_analytic_account_descendants(aplan_id1,[])
            account_plan2_ids = get_analytic_account_descendants(aplan_id2,[])
            account_plan3_ids = get_analytic_account_descendants(aplan_id3,[])
            mlines = move_l_obj.browse(cr,uid,mline_ids)
            for mline in mlines:
                detail_rec = {}
                detail_rec['name'] = mline.name or ''
                detail_rec['product_id'] = mline.product_id and mline.product_id.id or False
                detail_rec['gl_account_id'] = mline.account_id.id
                detail_rec['move_id'] = mline.move_id.id
                detail_rec['partner_id'] = mline.partner_id.id
                detail_rec['period_id'] = mline.period_id.id
                # calculation of the repartition of the amount between analytics
                # because the rate of repartition can be different from 100%
                if len(mline['analytic_lines']) == 3:
                    # the rate must be 100% for all lines
                    detail_rec['amount'] = mline['analytic_lines'][0].amount
                    for aline in mline['analytic_lines']:
                        if aline.account_id.id in account_plan1_ids:
                            detail_rec['account1_id'] = aline.account_id.id
                        elif aline.account_id.id in account_plan2_ids:
                            detail_rec['account2_id'] = aline.account_id.id
                        elif aline.account_id.id in account_plan3_ids:
                            detail_rec['account3_id'] = aline.account_id.id
                    detail_id = details_obj.create(cr, uid, detail_rec)
                    data_ids.append(detail_id)
                else:
                    # there is a rate of repartition between each plans
                    # we create only the lines that correspond to the filter asked by the user
                    count1 = count2 = count3 = 0
                    amount1 = amount2 = amount3 = 0
                    adetails1 = []
                    adetails2 = []
                    adetails3 = []
                    for aline in mline['analytic_lines']:
                        if aline.account_id.id in account_plan1_ids:
                            count1 += 1
                            amount1 += aline.amount 
                            adetails1.append([aline.amount,aline.account_id.id])
                        elif aline.account_id.id in account_plan2_ids:
                            count2 += 1
                            amount2 += aline.amount 
                            adetails2.append([aline.amount,aline.account_id.id])
                        elif aline.account_id.id in account_plan3_ids:
                            count3 += 1
                            amount3 += aline.amount 
                            adetails3.append([aline.amount,aline.account_id.id])
                    # first, check if there is a split only on 1 plan
                    if count1 > 1 and count2 == 1 and count3 == 1:
                        # the first plan is split
                        for detail in adetails1:
                            if ( aaccount_id1 and adetails1[0][1] in aaccount1_ids ) or not aaccount_id1:
                                line_rec = detail_rec
                                line_rec['amount'] = detail[0]
                                line_rec['account1_id'] = detail[1]
                                line_rec['account2_id'] = adetails2[0][1]
                                line_rec['account3_id'] = adetails3[0][1]
                                detail_id = details_obj.create(cr, uid, line_rec)
                                data_ids.append(detail_id)
                    elif count1 == 1 and count2 > 1 and count3 == 1:
                        # the second plan is split
                        for detail in adetails2:
                            if ( aaccount_id2 and adetails2[0][1] in aaccount2_ids ) or not aaccount_id2:
                                line_rec = detail_rec
                                line_rec['amount'] = detail[0]
                                line_rec['account2_id'] = detail[1]
                                line_rec['account1_id'] = adetails1[0][1]
                                line_rec['account3_id'] = adetails3[0][1]
                                detail_id = details_obj.create(cr, uid, line_rec)
                                data_ids.append(detail_id)
                    elif count1 == 1 and count2 == 1 and count3 > 1:
                        # the third plan is split
                        for detail in adetails3:
                            if ( aaccount_id3 and adetails3[0][1] in aaccount3_ids ) or not aaccount_id3:
                                line_rec = detail_rec
                                line_rec['amount'] = detail[0]
                                line_rec['account3_id'] = detail[1]
                                line_rec['account1_id'] = adetails1[0][1]
                                line_rec['account2_id'] = adetails2[0][1]
                                detail_id = details_obj.create(cr, uid, line_rec)
                                data_ids.append(detail_id)
                    elif count1 > 1 and count2 > 1 and count3 == 1:
                        # there is a split on two first plans
                        amount_total = amount3
                        cols = []
                        lines = []
                        for detail in adetails1:
                            if amount_total <> 0.0:
                                cols.append( [detail[0]/amount_total,detail[1]] )
                            else:
                                cols.append( [0.0,detail[1]] )
                        for detail in adetails2:
                            if amount_total <> 0.0:
                                lines.append([detail[0]/amount_total,detail[1]] )
                            else:
                                lines.append([0.0,detail[1]] )
                        for line in lines:
                            for col in cols:
                                if ( ( aaccount_id1 and col[1] in aaccount1_ids ) or not aaccount_id1 ) and ( ( aaccount_id2 and line[1] in aaccount2_ids ) or not aaccount_id2 ):
                                    line_rec = detail_rec
                                    line_rec['amount'] = amount_total * col[0] * line[0]
                                    line_rec['account1_id'] = col[1]
                                    line_rec['account2_id'] = line[1]
                                    line_rec['account3_id'] = adetails3[0][1]
                                    detail_id = details_obj.create(cr, uid, line_rec)
                                    data_ids.append(detail_id)
                    elif count1 > 1 and count2 == 1 and count3 > 1:
                        # there is a split on first and third plans
                        amount_total = amount2
                        cols = []
                        lines = []
                        for detail in adetails1:
                            if amount_total <> 0.0:
                                cols.append( [detail[0]/amount_total,detail[1]] )
                            else:
                                cols.append( [0.0,detail[1]] )
                        for detail in adetails3:
                            if amount_total <> 0.0:
                                lines.append([detail[0]/amount_total,detail[1]] )
                            else:
                                lines.append([0.0,detail[1]] )
                        for line in lines:
                            for col in cols:
                                if ( ( aaccount_id1 and col[1] in aaccount1_ids ) or not aaccount_id1 ) and ( ( aaccount_id3 and line[1] in aaccount3_ids ) or not aaccount_id3 ):
                                    line_rec = detail_rec
                                    line_rec['amount'] = amount_total * col[0] * line[0]
                                    line_rec['account1_id'] = col[1]
                                    line_rec['account3_id'] = line[1]
                                    line_rec['account2_id'] = adetails2[0][1]
                                    detail_id = details_obj.create(cr, uid, line_rec)
                                    data_ids.append(detail_id)
                    elif count1 == 1 and count2 > 1 and count3 > 1:
                        # there is a split on two last plans
                        amount_total = amount1
                        cols = []
                        lines = []
                        for detail in adetails2:
                            if amount_total <> 0.0:
                                cols.append( [detail[0]/amount_total,detail[1]] )
                            else:
                                cols.append( [0.0,detail[1]] )
                        for detail in adetails3:
                            if amount_total <> 0.0:
                                lines.append([detail[0]/amount_total,detail[1]] )
                            else:
                                lines.append([0.0,detail[1]] )
                        for line in lines:
                            for col in cols:
                                if ( ( aaccount_id2 and col[1] in aaccount2_ids ) or not aaccount_id2 ) and ( ( aaccount_id3 and line[1] in aaccount3_ids ) or not aaccount_id3 ):
                                    line_rec = detail_rec
                                    line_rec['amount'] = amount_total * col[0] * line[0]
                                    line_rec['account2_id'] = col[1]
                                    line_rec['account1_id'] = line[1]
                                    line_rec['account1_id'] = adetails1[0][1]
                                    detail_id = details_obj.create(cr, uid, line_rec)
                                    data_ids.append(detail_id)
                    else:
                        # there is a split on the three plans
                        amount_total = amount1
                        cols = []
                        lines = []
                        ups = []
                        for detail in adetails1:
                            if amount_total <> 0.0:
                                cols.append( [detail[0]/amount_total,detail[1]] )
                            else:
                                cols.append( [0.0,detail[1]] )
                        for detail in adetails2:
                            if amount_total <> 0.0:
                                lines.append([detail[0]/amount_total,detail[1]] )
                            else:
                                lines.append([0.0,detail[1]] )
                        for detail in adetails3:
                            if amount_total <> 0.0:
                                ups.append([detail[0]/amount_total,detail[1]] )
                            else:
                                ups.append([0.0,detail[1]] )
                        for up in ups:
                            for line in lines:
                                for col in cols:
                                    if ( ( aaccount_id1 and col[1] in aaccount1_ids ) or not aaccount_id1 ) and ( ( aaccount_id2 and line[1] in aaccount2_ids ) or not aaccount_id2 ) and ( ( aaccount_id3 and up[1] in aaccount3_ids ) or not aaccount_id3 ):
                                        line_rec = detail_rec
                                        line_rec['amount'] = amount_total * col[0] * line[0] * up[0]
                                        line_rec['account1_id'] = col[1]
                                        line_rec['account2_id'] = line[1]
                                        line_rec['account3_id'] = up[1]
                                        detail_id = details_obj.create(cr, uid, line_rec)
                                        data_ids.append(detail_id)
        return {
                'view_type': 'form',
                'domain': "[('id', 'in', %s)]" % str(data_ids),
                "view_mode": 'tree',
                'res_model': 'cci.analytic.cross.details3',
                'type': 'ir.actions.act_window',
        }
    def action_cancel(self, cr, uid, ids, context=None):
        return {
                'view_type': 'form',
                'domain': "[('id', 'in', [])]",
                "view_mode": 'tree',
                'res_model': 'cci.analytic.cross.details3',
                'type': 'ir.actions.act_window',
        }
cci_analytic_cross3()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
