# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008 CCI Connect ASBL. (http://www.cciconnect.be) All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import time
import datetime
from osv import fields, osv
from wordpress_xmlrpc import Client, WordPressPost
from wordpress_xmlrpc.methods.posts import NewPost, GetPost
import tools

class riplay_prohibited_nace(osv.osv):
    _name = 'riplay.prohibited_nace'
    _description = 'NACE Code not admitted in database if alone'
    _columns = {
        'name': fields.char('Name',size=5,required=True),
        'meaning': fields.char('Meaning',size=42),
        'prohibited': fields.boolean('Prohibited if alone'),
        'import_date': fields.date('First Import Date'),
    }
    _order = "name"
    _defaults = {
        'prohibited': lambda *a: False,
        'import_date': lambda *a: datetime.date.today().strftime('%y-%m-%d')
    }
riplay_prohibited_nace()

class riplay_creation(osv.osv):
    _name = "riplay.creation"
    _description = "New creations of companies"
    _columns = {
        'name': fields.char('Name',size=250,required=True),
        'street': fields.char('Street',size=250),
        'zip_code': fields.char('Zip Code',size=15),
        'city': fields.char('Locality',size=80),
        'vat_num': fields.char('VAT Num',size=20),
        'legal_status': fields.char('Legal Status',size=20),
        'admin_type': fields.char('Administrator\'s Type',size=20),
        'admin_courtesy': fields.char('Administrator\'s Courtesy',size=30),
        'admin_name': fields.char('Administrator\'s Name',size=80),
        'admin_first_name': fields.char('Administrator\'s First Name',size=80),
        'admin_title': fields.char('Administrator\'s Title',size=80),
        'admin_codes': fields.char('Administrator\'s Codes',size=20),
        'activ_desc': fields.text('Activity Description'),
        'capital': fields.integer('Capital'),
        'nace_ids': fields.many2many('riplay.prohibited_nace','creation_nace_rel','creation_id','prohibited_nace_id','NACE Codes'),
        'creation_date': fields.date('Creation Date'),
        'keep_aside': fields.boolean('Don\'t send to Partners'),
        'import_date': fields.date('Import Date'),
        'inject_date': fields.date('Inject Date'),
        'final_partner_id': fields.many2one('res.partner','Final Partner'),
        'final_zip_id': fields.many2one('res.partner.zip','Final Zip Code'),
        'output_date': fields.date('Date of output',help='Date of output on CCI\' Mag Website'),
    }

    def check_vat_num(self, cr, uid, ids):
        for creation in self.browse(cr, uid, ids):
            if not creation.vat_num:
                continue
            vat_country, vat_number = creation.vat_num[:2], creation.vat_num[2:]
            if vat_country != 'BE':
                return False
            if (' ' in vat_number) or ('.' in vat_number):
                return False
            if len(vat_number) != 10:
                return False
            try:
                int(vat_number)
            except:
                return False
            if int(vat_number[-2:]) != 97 - (int(vat_number[:8]) % 97):
                return False
        return True

    _order = "creation_date desc" # this order is unsed (in reverse order) for the publication => if changed, change also the code in export_week.py wizard
    _constraints = [(check_vat_num, "\n\nThe VAT doesn't seem to be correct.\n\nIt must be in the format 'BE0123456789' or empty.", ["vat_num"])]
    _defaults = {
        'keep_aside': lambda *a: False,
        'import_date': lambda *a: datetime.date.today().strftime('%y-%m-%d'),
    }
    def count_new_month(self,cr,uid,month):
        count_lg = 0
        count_nam = 0
        creation_ids = self.search(cr,uid,[('output_date','=',False)])
        if creation_ids:
            creations = self.browse(cr,uid,creation_ids)
            for creat in creations:
                if creat.final_zip_id:
                    if creat.final_zip_id.name[0] == '4':
                        count_lg += 1
                    elif creat.final_zip_id.name[0] == '5':
                        count_nam += 1
                elif creat.zip_code:
                    if creat.zip_code[0] == '4':
                        count_lg += 1
                    elif creat.zip_code[0] == '5':
                        count_nam += 1
        count_by_month_obj = self.pool.get('riplay.count_by_month')
        new_id = count_by_month_obj.create(cr,uid,{'name':month,'calculation_date':datetime.date.today().strftime('%Y-%m-%d'),
                                                   'count_month_prov_lg':count_lg,'count_month_prov_nam':count_nam,
                                                   'count_week_prov_lg':int(count_lg/5),'count_week_prov_nam':int(count_nam/5)})
        return new_id
    def publish_bankr_and_creations(self,cr,uid,usage,status): # usage can be 'manual' or 'automatic'
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        followup_emails = []
        if usage == 'automatic':
            param_values = parameter_obj.get_value_from_names(cr,uid,['failcons_email','BDDeTest'])
            BD_DE_TEST = True
            if param_values.has_key('BDDeTest'):
                if param_values['BDDeTest'] == 0:
                    BD_DE_TEST = False
            if not BD_DE_TEST:
                if param_values.has_key('failcons_email'):
                    followup_emails = param_values['failcons_email']
        final_slug = ''
        result = ''
        holiday_obj = self.pool.get('riplay.holiday')
        today = datetime.date.today().strftime('%Y-%m-%d')
        holiday_ids = holiday_obj.search(cr,uid,[('date_from','<=',today),('date_to','>=',today),('type','=','rdp')])
        if not holiday_ids:
            # orm objects
            creation_obj = self.pool.get('riplay.creation')
            bankr_obj = self.pool.get('riplay.bankruptcy')
            count_obj = self.pool.get('riplay.count_by_month')
            param_values = parameter_obj.get_value_from_names(cr,uid,['failcons_template_article','failcons_template_bankr_prov','failcons_template_bankr','failcons_template_creation_prov','failcons_template_creation','failcons_wp_addr','failcons_wp_login','failcons_wp_access','failcons_wp_thumb_id'])
            if param_values.has_key('failcons_template_article') and \
                    param_values.has_key('failcons_template_bankr') and \
                    param_values.has_key('failcons_template_bankr_prov') and \
                    param_values.has_key('failcons_template_creation') and \
                    param_values.has_key('failcons_template_creation_prov') and \
                    param_values.has_key('failcons_wp_addr') and \
                    param_values.has_key('failcons_wp_login') and \
                    param_values.has_key('failcons_wp_access') and \
                    param_values.has_key('failcons_wp_thumb_id'):

                template_creation = param_values['failcons_template_creation']
                template_creation_province = param_values['failcons_template_creation_prov']
                template_bankr = param_values['failcons_template_bankr']
                template_bankr_province = param_values['failcons_template_bankr_prov']
                template_creation = param_values['failcons_template_creation']
                template_main_content = param_values['failcons_template_article']
                
                # extraction of max to publish by month
                max_lg = 0
                max_nam = 0
                creation_ids_lg = []
                creation_ids_nam = []
                
                # search for the max to publish for Liege and Namur
                today = datetime.datetime.today()
                s_today = today.strftime('%Y%m%d')
                if s_today[:-2] < '07':
                    # first week of the next month => we extract the count of the past month
                    count_month = (today + datetime.timedelta(days=-7)).strftime('%y%m')
                else:
                    count_month = s_today[0:6]
                count_ids = count_obj.search(cr,uid,[('name','=',count_month)])
                if not count_ids:
                    count_id = creation_obj.count_new_month(cr,uid,count_month)
                else:
                    count_id = count_ids[0]
                if count_id:
                    count = count_obj.browse(cr,uid,count_id)
                    max_lg = count.count_week_prov_lg
                    max_nam = count.count_week_prov_nam
                
                # searching real ID of creation
                creation_ids_lg = []
                creation_ids_nam = []
                creation_text_lg = ''
                creation_text_nam = ''
                creation_ids = creation_obj.search(cr,uid,[('output_date','=',False)])
                if creation_ids:
                    creations = creation_obj.browse(cr,uid,creation_ids)
                    creations.reverse()
                    for creat in creations:
                        prov = 'none'
                        if creat.final_zip_id:
                            if creat.final_zip_id.name[0] == '4':
                                prov = 'lg'
                            elif creat.final_zip_id.name[0] == '5':
                                prov = 'nam'
                        else:
                            if creat.zip_code:
                                if creat.zip_code[0] == '4':
                                    prov = 'lg'
                                elif creat.zip_code[0] == '5':
                                    prov = 'nam'
                        if prov == 'lg' and len(creation_ids_lg) < max_lg:
                            creation_ids_lg.append(creat.id)
                            creation_zip_city = u''
                            if creat.final_zip_id:
                                creation_zip_city = creat.final_zip_id.name + u' ' + creat.final_zip_id.city
                            else:
                                creation_zip_city = creat.zip_code + u' ' + creat.city
                            creation_text_lg += template_creation.format(company_name=creat.name or '-',
                                                                         company_legal_status=creat.legal_status or '',
                                                                         street=creat.street or '-',
                                                                         zip_city=creation_zip_city,
                                                                         vat_number_only=creat.vat_num and creat.vat_num[2:] or '',
                                                                         formatted_vat_number=creat.vat_num and (creat.vat_num[0:2]+'-'+creat.vat_num[2:6]+'.'+creat.vat_num[6:9]+'.'+creat.vat_num[9:]) or '',
                                                                         activ_desc=creat.activ_desc or '',
                                                                         contact_courtesy=(creat.admin_type == 'person') and creat.admin_courtesy or '',
                                                                         contact_name=(creat.admin_type == 'person') and creat.admin_name or '',
                                                                         contact_first_name=(creat.admin_type == 'person') and creat.admin_first_name or '',
                                                                         contact_title=(creat.admin_type == 'person') and creat.admin_title or ''
                                                                        ) 
                        elif prov == 'nam' and len(creation_ids_nam) < max_nam:
                            creation_ids_nam.append(creat.id)
                            creation_zip_city = u''
                            if creat.final_zip_id:
                                creation_zip_city = creat.final_zip_id.name + u' ' + creat.final_zip_id.city
                            else:
                                creation_zip_city = creat.zip_code + u' ' + creat.city
                            creation_text_nam += template_creation.format(company_name=creat.name or '-',
                                                                          company_legal_status=creat.legal_status or '',
                                                                          street=creat.street or '-',
                                                                          zip_city=creation_zip_city,
                                                                          vat_number_only=creat.vat_num and creat.vat_num[2:] or '',
                                                                          formatted_vat_number=creat.vat_num and (creat.vat_num[0:2]+'-'+creat.vat_num[2:6]+'.'+creat.vat_num[6:9]+'.'+creat.vat_num[9:]) or '',
                                                                          activ_desc=creat.activ_desc or '',
                                                                          contact_courtesy=(creat.admin_type == 'person') and creat.admin_courtesy or '',
                                                                          contact_name=(creat.admin_type == 'person') and creat.admin_name or '',
                                                                          contact_first_name=(creat.admin_type == 'person') and creat.admin_first_name or '',
                                                                          contact_title=(creat.admin_type == 'person') and creat.admin_title or ''
                                                                         )
                        if len(creation_ids_lg) >= max_lg and len(creation_ids_nam) >= max_nam:
                            break
                    if creation_ids_lg and creation_ids_nam:
                        creation_text = template_creation_province.format(province_code=u'liege',
                                                                          province_name=u'Province de Liège',
                                                                          inside_content=creation_text_lg \
                                                                         ) + \
                                        template_creation_province.format(province_code=u'namur',
                                                                          province_name=u'Province de Namur',
                                                                          inside_content=creation_text_nam \
                                                                         )
                    elif creation_ids_lg and not creation_ids_nam:
                        creation_text = template_creation_province.format(province_code=u'liege',
                                                                          province_name=u'Province de Liège',
                                                                          inside_content=creation_text_lg 
                                                                         )
                    elif creation_ids_nam and not creation_ids_lg:
                        creation_text = template_creation_province.format(province_code=u'namur',
                                                                          province_name=u'Province de Namur',
                                                                          inside_content=bankr_text_nam
                                                                         )
                    else:
                        creation_text = ''
                    if creation_ids_lg and status == 'publish':
                        creation_obj.write(cr,uid,creation_ids_lg,{'output_date':datetime.datetime.today().strftime('%Y-%m-%d')})
                    if creation_ids_nam and status == 'publish':
                        creation_obj.write(cr,uid,creation_ids_nam,{'output_date':datetime.datetime.today().strftime('%Y-%m-%d')})
                # searching for bankruptcies to publish
                bankr_ids_lg = []
                bankr_ids_nam = []
                nearest_bankr_date = '1980-01-01'
                bankr_text = ''
                bankr_text_lg = ''
                bankr_text_nam = ''
                bankr_ids = bankr_obj.search(cr,uid,[('output_date','=',False)])
                if bankr_ids:
                    bankrs = bankr_obj.browse(cr,uid,bankr_ids)
                    for bankr in bankrs:
                        curators = [x.name.strip() for x in bankr.curator_ids]
                        if len(curators) == 0:
                            curator_header = ''
                            curators = ''
                        elif len(curators) == 1:
                            curator_header= '<strong>Curateur :</strong> '
                            curators_name = curators[0]
                        else:
                            curator_header = '<strong>Curateurs :</strong> '
                            curators_name = ', '.join(curators)
                        commissioner_name = bankr.commissioner_id and bankr.commissioner_id.name.strip() or ''
                        if curator_header and commissioner_name:
                            curators_and_commissioner = curator_header + curators_name + '<br/><strong>Commissaire :</strong> ' + commissioner_name
                        elif curator_header and not commissioner_name:
                            curators_and_commissioner = curator_header + curators_name
                        elif commissioner_name and not curator_header:
                            curators_and_commissioner = '<strong>Commissaire :</strong> ' + commissioner_name
                        else:
                            curators_and_commissioner = ''
                        if bankr.zip_id.name[0] == '4':
                            bankr_ids_lg.append(bankr.id)
                            bankr_text_lg += template_bankr.format(company_name=bankr.name or '-',
                                                                   company_legal_status=bankr.legal_status or '',
                                                                   street=bankr.street or '-',
                                                                   zip_city=bankr.zip_id and (bankr.zip_id.name + ' ' + bankr.zip_id.city) or '',
                                                                   vat_number_only=bankr.vat_num and bankr.vat_num[2:] or '',
                                                                   formatted_vat_number=bankr.vat_num and (bankr.vat_num[0:2]+'-'+bankr.vat_num[2:6]+'.'+bankr.vat_num[6:9]+'.'+bankr.vat_num[9:]) or '',
                                                                   activ_desc=bankr.activ_desc or '',
                                                                   curators_and_commissioner=curators_and_commissioner,
                                                                   bankr_date=bankr.bankruptcy_date or '',
                                                                   br_type=bankr.type_id and bankr.type_id.name or ''
                                                                  ) 
                        elif bankr.zip_id.name[0] == '5':
                            bankr_ids_nam.append(bankr.id)
                            bankr_text_nam += template_bankr.format(company_name=bankr.name or '-',
                                                                    company_legal_status=bankr.legal_status or '',
                                                                    street=bankr.street or '-',
                                                                    zip_city=bankr.zip_id and (bankr.zip_id.name + ' ' + bankr.zip_id.city) or '',
                                                                    vat_number_only=bankr.vat_num and bankr.vat_num[2:] or '',
                                                                    formatted_vat_number=bankr.vat_num and (bankr.vat_num[0:2]+'-'+bankr.vat_num[2:6]+'.'+bankr.vat_num[6:9]+'.'+bankr.vat_num[9:]) or '',
                                                                    activ_desc=bankr.activ_desc or '',
                                                                    curators_and_commissioner=curators_and_commissioner,
                                                                    bankr_date=bankr.bankruptcy_date or '',
                                                                    br_type=bankr.type_id and bankr.type_id.name or ''
                                                                   ) 
                        if bankr.bankruptcy_date and bankr.bankruptcy_date > nearest_bankr_date:
                            nearest_bankr_date = bankr.bankruptcy_date
                    if bankr_ids_lg and bankr_ids_nam:
                        bankr_text = template_bankr_province.format(province_code=u'liege',
                                                                    province_name=u'Province de Liège',
                                                                    inside_content=bankr_text_lg \
                                                                   ) + \
                                     template_bankr_province.format(province_code=u'namur',
                                                                    province_name=u'Province de Namur',
                                                                    inside_content=bankr_text_nam \
                                                                   )
                    elif bankr_ids_lg and not bankr_ids_nam:
                        bankr_text = template_bankr_province.format(province_code=u'liege',
                                                                    province_name=u'Province de Liège',
                                                                    inside_content=bankr_text_lg 
                                                                   )
                    elif bankr_ids_nam and not bankr_ids_lg:
                        bankr_text = template_bankr_province.format(province_code=u'namur',
                                                                    province_name=u'Province de Namur',
                                                                    inside_content=bankr_text_nam
                                                                   )
                    else:
                        bankr_text = ''
                    if bankr_ids_lg and status == 'publish':
                        bankr_obj.write(cr,uid,bankr_ids_lg,{'output_date':datetime.datetime.today().strftime('%Y-%m-%d')})
                    if bankr_ids_nam and status == 'publish':
                        bankr_obj.write(cr,uid,bankr_ids_nam,{'output_date':datetime.datetime.today().strftime('%Y-%m-%d')})
                if creation_text and bankr_text:
                    separator = u'<hr/><hr/>'
                else:
                    separator = ''
                links_lg = ''
                if creation_ids_lg and bankr_ids_lg:
                    links_lg = u'<a href="#constitutions-liege">Constitutions Liège</a> &#8211; <a href="#faillites-liege">Faillites Liège</a>'
                elif creation_ids_lg and not bankr_ids_lg:
                    links_lg = u'<a href="#constitutions-liege">Constitutions Liège</a>'
                if bankr_ids_lg and not creation_ids_lg:
                    links_lg = u'<a href="#faillites-liege">Faillites Liège</a>'
                links_nam = ''
                if creation_ids_nam and bankr_ids_nam:
                    links_nam = u'<a href="#constitutions-namur">Constitutions Namur</a> &#8211; <a href="#faillites-namur">Faillites Namur</a>'
                elif creation_ids_nam and not bankr_ids_nam:
                    links_nam = u'<a href="#constitutions-namur">Constitutions Namur</a>'
                if bankr_ids_nam and not creation_ids_nam:
                    links_nam = u'<a href="#faillites-namur\">Faillites Namur</a>'
                links = ''
                if links_lg and links_nam:
                    links = links_lg + u'<br/>' + links_nam
                elif links_lg and not links_nam:
                    links = links_lg
                elif links_nam and not links_lg:
                    links = links_nam
                final_article = template_main_content.format(links=links,
                                                             content_constitutions=creation_text,
                                                             separator=separator,
                                                             content_faillites=bankr_text
                                                            )
                cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'HTML Content Riplay ' + datetime.date.today().strftime('%Y%m%d'))
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,final_article)
                self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,True,'')
                try:
                    print('try connect')
                    print(param_values['failcons_wp_addr'])
                    wp = Client(param_values['failcons_wp_addr'], param_values['failcons_wp_login'], param_values['failcons_wp_access'])
                    post = WordPressPost()
                    print('give data')
                    post.title = 'Faillites & Constitutions'
                    post.content = final_article
                    post.terms_names = {'post_tag':['Faillites','Constitutions'],
                                        'category':['Infos entreprises',],
                                       }
                    post.post_status = status
                    post.thumbnail = param_values['failcons_wp_thumb_id']
                    print('call')
                    new_id = wp.call(NewPost(post))
                    new_post = wp.call(GetPost(new_id))
                    print('new_id')
                    print(new_post)
                    final_slug = new_post.slug or 'not yet published'
                    result = final_article
                    if followup_emails:
                        tools.email_send('philmer@ccilvn.be', followup_emails, 'Publication Faillites et Constititutions automatique', '<p>Publication automatique des faillites et constitutions sur le site du CCIMag : réussite</p><p>Permalien : %s</p>' % final_slug, subtype='html')
                except Exception,e:
                    result = str(e)
                    if followup_emails:
                        tools.email_send('philmer@ccilvn.be', followup_emails, 'ERREUR - Publication Faillites et Constititutions automatique', '<p>Publication automatique des faillites et constitutions sur le site du CCIMag : echec</p><p>Erreur : %s</p>' % result, subtype='html')
            else:
                result = u'ERROR :\nParamters missing !'
                if followup_emails:
                    tools.email_send('philmer@ccilvn.be', followup_emails, 'ERREUR - Publication Faillites et Constititutions automatique', '<p>Publication automatique des faillites et constitutions sur le site du CCIMag : echec</p><p>%s</p>' % result, subtype='html')
        else:
            holidays = holiday_obj.read(cr,uid,holiday_ids,['date_from','date_to','name'])
            result = u'ERROR : \nToday is an holiday : %s [%s -> %s]\n\nNo publications.' % (holidays[0]['name'],holidays[0]['date_from'],holidays[0]['date_to'])
            if followup_emails:
                tools.email_send('philmer@ccilvn.be', followup_emails, 'WARNING - Publication Faillites et Constititutions automatique', '<p>Publication automatique des faillites et constitutions sur le site du CCIMag : echec</p><p>%s</p>' % result, subtype='html')
        return (result,final_slug)
riplay_creation()

class riplay_bankruptcy_type(osv.osv):
    _name = 'riplay.bankruptcy_type'
    _description = 'Type of bankruptcy and way of dealing it'
    _columns = {
        'name': fields.char('Name',size=100,required=True),
        'description': fields.text('Description'),
        'partner_state_id': fields.many2one('res.partner.state','Corresponding State in Partners',required=True),
        'active': fields.boolean('Active'),
    }
    _order = "name"
    _defaults = {
        'active': lambda *a: True,
    }
riplay_bankruptcy_type()

class riplay_curator(osv.osv):
    _name = "riplay.curator"
    _description = "Curator Coordinates"
    _columns = {
        'name': fields.char('Complete name',size=80,required=True),
        'street': fields.char('Street',size=250),
        'zip_city': fields.char('Zip Code and Locality',size=80),
        'office_name': fields.char('Office Name',size=80),
        'email': fields.char('EMail',size=200),
        'phone': fields.char('Phone',size=80),
        'fax': fields.char('Fax',size=80),
        'active': fields.boolean('Active'),
    }
    _order = "name"
    _defaults = {
        'active': lambda *a: True,
    }
riplay_curator()

class riplay_commissioner(osv.osv):
    _name = "riplay.commissioner"
    _description = "Commissioner Coordinates"
    _columns = {
        'name': fields.char('Complete name',size=80,required=True),
        'street': fields.char('Street',size=250),
        'zip_city': fields.char('Zip Code and Locality',size=80),
        'office_name': fields.char('Office Name',size=80),
        'email': fields.char('EMail',size=200),
        'phone': fields.char('Phone',size=80),
        'fax': fields.char('Fax',size=80),
        'active': fields.boolean('Active'),
    }
    _order = "name"
    _defaults = {
        'active': lambda *a: True,
    }
riplay_commissioner()

class riplay_bankruptcy(osv.osv):
    _name = "riplay.bankruptcy"
    _description = "Bankruptcies"
    _columns = {
        'name' : fields.char('Name',size=100,required=True),
        'bankruptcy_date': fields.date('Bankruptcy Date'),
        'type_id': fields.many2one('riplay.bankruptcy_type','Type',required=True),
        'street': fields.char('Street',size=250),
        'zip_id': fields.many2one('res.partner.zip','Zip',required=True), # required to split them by province
        'vat_num': fields.char('VAT Num',size=20),
        'legal_status': fields.char('Legal Status',size=20),
        'activ_desc': fields.text('Activity Description'),
        'commissioner_id': fields.many2one('riplay.commissioner','Justice Commisioner'),
        'curator_ids': fields.many2many('riplay.curator','bankruptcy_curator_rel','bankruptcy_id','curator_id','Curators'),
        'partner_id': fields.many2one('res.partner','Found Partner'),
        'output_date': fields.date('Date of output',help='Date of output on CCI\' Mag Website'),
    }

    def check_vat_num(self, cr, uid, ids):
        for bankr in self.browse(cr, uid, ids):
            if not bankr.vat_num:
                continue
            vat_country, vat_number = bankr.vat_num[:2], bankr.vat_num[2:]
            if vat_country != 'BE':
                return False
            if (' ' in vat_number) or ('.' in vat_number):
                return False
            if len(vat_number) != 10:
                return False
            try:
                int(vat_number)
            except:
                return False
            if int(vat_number[-2:]) != 97 - (int(vat_number[:8]) % 97):
                return False
        return True

    _order = "bankruptcy_date desc" # also publisinhing order => if changed, manage the order in the publising wizard 'export_week'
    _constraints = [(check_vat_num, "\n\nThe VAT doesn't seem to be correct.\n\nIt must be in the format 'BE0123456789' or empty.", ["vat_num"])]
riplay_bankruptcy()

class riplay_count_by_month(osv.osv):
    _name = "riplay.count_by_month"
    _description = "Count Creations to be published every week of a month"
    _columns = {
        'name' : fields.char('Name',size=6,required=True),
        'calculation_date': fields.date('Calculation Date'),
        'count_month_prov_lg': fields.integer(u'Count for a month for Liege'), 
        'count_month_prov_nam': fields.integer(u'Count for a month for Namur'),
        'count_week_prov_lg': fields.integer(u'Count for a week for Liege'), # 1/5 of the month because we can have 5 publications in a given month
        'count_week_prov_nam': fields.integer(u'Count for a week for Namur'), # 1/5 of the month because we can have 5 publications in a given month
    }
    _order = "name desc"
riplay_count_by_month()

class riplay_holiday(osv.osv):
    _name = "riplay.holiday"
    _description = "Holidays for multiples usages not Clubs"
    _columns = {
        'name' : fields.char('Name',size=32,required=True),
        'date_from' : fields.date('From',required=True),
        'date_to' : fields.date('To',required=True),
        'description' : fields.text('Description'),
        'type': fields.selection([('rdp','Revue de Presse')],'Usage',required=True),
    }
    _order = "date_from desc"
    _defaults = {
        'type': lambda *a: 'rdp',
    }
riplay_holiday()

