# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
# Version 1.0 Philmer
import wizard
import time
import datetime
import tools
import pooler
import base64
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Options">
    <field name="input" colspan="4" width="600"/>
</form>"""

fields = {
    'input': {'type':'binary','string':'Constitutions file','required':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="Importation result" colspan="4"/>
     <field name="msg" colspan="4" nolabel="1" width="800" heigth="600"/>
</form>"""

msg_fields = {
    'msg': {'string':'File imported', 'type':'text', 'readonly':True},
}

def _is_only_digits(string):
    result = True
    for car in string:
        if car not in ['0','1','2','3','4','5','6','7','8','9']:
            result = False
            break
    return result
    
def _remove_accentuation(name):
    return name.strip().replace(u'é','e').replace(u'è','e').replace(u'ê','e').replace(u'ë','e').replace(u'ù','u').replace(u'û','u').replace(u'ü','u').replace(u'à','a').replace(u'â','a').replace(u'ä','a').replace(u'ç','c').replace(u'î','i').replace(u'ï','i').replace(u'ô','o').replace(u'ö','o')

class import_constitutions(wizard.interface):

    def _import_data(self,cr,uid,data,context):
        imported = 0
        rejected = 0
        created = 0

        # orm objects
        creation_obj = pooler.get_pool(cr.dbname).get('riplay.creation')
        nace_obj = pooler.get_pool(cr.dbname).get('riplay.prohibited_nace')
        partner_obj = pooler.get_pool(cr.dbname).get('res.partner')
        zip_obj = pooler.get_pool(cr.dbname).get('res.partner.zip')
        title_obj = pooler.get_pool(cr.dbname).get('res.partner.title')
        #
        inputfile = data['form']['input']
        inputdata = base64.decodestring(inputfile).split('\n')
        #
        injected_vat_nums = [] # we inject only ONCE the same vat_num FROM THE SAME INPUT FILE
                               # if the vat_num already exist but from a different input_file, this is probably a correction so we keep it
        for line in inputdata:
            vat_num = line[0:10] # Format 0123456789
            official_name = line[10:10+160].strip().decode('cp1252') # Format COMPANY NAME
            if vat_num or official_name:
                # format the data in the CCI format
                if vat_num.strip():
                    vat_num = 'BE'+vat_num
                if  vat_num and vat_num not in injected_vat_nums:
                    street = line[271:271+76].decode('cp1252').strip() # Format 'Féronstrée 149 bte 4 F' ou 'Avenue Lous Maire 4 C bte 2'
                    zip_code = line[347:347+4].strip() # FOrmat '4400      '
                    locality = line[357:357+40].strip() # format 'MONS-LEZ-LIEGE'
                    legal_status = line[173:173+12].strip().decode('cp1252') # format S.P.R.L.
                    creation_date = line[185:185+10] # format 2018-02-11
                    nom = line[1044:1044+48].decode('cp1252').strip()
                    prenom = line[1092:1092+24].decode('cp1252').strip()
                    sexe = line[1178:1178+1] # sexe = 1 = Homme; 2 = femme
                    title = line[1118:1118+50].decode('cp1252').strip()
                    capital = 0
                    try:
                        capital = int(line[195:195+13].strip())  # can be empty
                    except Exception,e:
                        pass
                    nace1 = line[759:759+5].strip()
                    libnace1 = line[764:764+42].decode('cp1252').strip() # format sans point final
                    nace2 = line[816:816+5].strip()
                    libnace2 = line[821:821+42].decode('cp1252').strip()
                    nace3 = line[873:873+5].strip()
                    libnace3 = line[878:878+42].decode('cp1252').strip()
                    nace4 = line[930:930+5].strip()
                    libnace4 = line[935:935+42].decode('cp1252').strip()
                    nace5 = line[987:987+5].strip()
                    libnace5 = line[992:992+42].decode('cp1252').strip()
                    #
                    current_name = official_name
                    if current_name[0:14] == 'ETABLISSEMENT ':
                        current_name = 'ETS ' + current_name[14:]
                    if current_name[0:15] == 'ETABLISSEMENTS ':
                        current_name = 'ETS ' + current_name[15:]
                    #
                    current_street = _remove_accentuation(street).upper()
                    parts = current_street.split(' ')
                    last_num = 999
                    last_initial = 999
                    last_bte = 999
                    index = len(parts)-1
                    while index >= 0:
                        if parts[index] == 'BTE':
                            last_bte = index
                        elif len(parts[index]) == 1 and parts[index] in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ':
                            last_initial = index
                        elif _is_only_digits(parts[index]):
                            last_num = index
                        index -= 1
                    if last_num != 999 and last_initial != 999:
                        if last_initial == last_num + 1:
                            new_parts = parts[0:last_num]
                            new_parts.append(parts[last_num]+parts[last_initial])
                            if last_initial < len(parts)-1:
                                new_parts.extend(parts[last_initial+1:])
                            if last_bte != 999 and last_bte > last_initial:
                                last_bte -= 1
                            parts = new_parts[0:len(new_parts)]
                    if last_num != 999 and last_num < last_bte:
                        if last_num > 0:
                            current_street = ' '.join(parts[0:last_num]) + ', '
                        else:
                            current_street = ''
                        current_street = current_street + ' '.join(parts[last_num:])
                    else: # unable to find number in street
                        current_street = ' '.join(parts)
                    for beg in [('BOULEVARD ','BLD '),('AVENUE ','AVE '),('PLACE ','PL '),('CHAUSSEE ','CHSEE '),('ROUTE ','RTE ')]:
                        if current_street[0:len(beg[0])] == beg[0]:
                            current_street = beg[1] + current_street[len(beg[0]):]
                    #
                    final_zip_id = False
                    zip_ids = zip_obj.search(cr,uid,[('name','=',zip_code)])
                    if zip_ids >= 1:
                        zips = zip_obj.read(cr,uid,zip_ids,['id','name','city','other_names'])
                        for czip in zips:
                            if czip['city'].upper() == locality:
                                final_zip_id = czip['id']
                            else:
                                if czip['other_names'] and ( locality in czip['other_names'] ):
                                    final_zip_id = czip['id']
                    #
                    title_ids = title_obj.search(cr,uid,['|',('other1','=',legal_status),('other2','=',legal_status)])
                    if title_ids:
                        if len(title_ids) == 1:
                            found_title = title_obj.read(cr,uid,title_ids,['shortcut'])[0]
                            legal_status = found_title['shortcut'] and found_title['shortcut'] or legal_status
                        else:
                            # only if same length
                            titles = title_obj.read(cr,uid,title_ids,['shortcut','other1','other2'])
                            for found_title in titles:
                                if (found_title['other1'] == lgeal_status) or (found_title['other2'] == legal_status):
                                    legal_status = found_title['shortcut']
                                    break
                    else:
                        official_name = official_name + u'[ ' + title + u']'
                    #
                    if sexe == '2':
                        gender = 'women'
                        courtesy = 'Madame'
                    else:
                        gender = 'man'
                        courtesy = 'Monsieur'
                    codes = 'G'
                    #
                    rejected = True
                    current_nace_ids = []
                    activ_desc = ''
                    if nace1:
                        nace_ids = nace_obj.search(cr,uid,[('name','=',nace1)])
                        if nace_ids:
                            nace = nace_obj.read(cr,uid,nace_ids,['prohibited'])[0]
                            if not nace['prohibited']:
                                rejected = False
                            current_nace_ids.append(nace['id'])
                        else:
                            new_nace_id = nace_obj.create(cr,uid,{'name':nace1,'meaning':libnace1 or ''})
                            current_nace_ids.append(new_nace_id)
                            rejected = False
                    else:
                        # NO NACE AT ALL => not rejected
                        rejected = False
                    if libnace1:
                        activ_desc += (libnace1+'. ')
                    if nace2:
                        nace_ids = nace_obj.search(cr,uid,[('name','=',nace2)])
                        if nace_ids:
                            nace = nace_obj.read(cr,uid,nace_ids,['prohibited'])[0]
                            if not nace['prohibited']:
                                rejected = False
                            current_nace_ids.append(nace['id'])
                        else:
                            new_nace_id = nace_obj.create(cr,uid,{'name':nace2,'meaning':libnace2 or ''})
                            current_nace_ids.append(new_nace_id)
                            rejected = False
                    else:
                        # NO NACE AT ALL => not rejected
                        rejected = False
                    if libnace2:
                        activ_desc += (libnace2+'. ')
                    if nace3:
                        nace_ids = nace_obj.search(cr,uid,[('name','=',nace3)])
                        if nace_ids:
                            nace = nace_obj.read(cr,uid,nace_ids,['prohibited'])[0]
                            if not nace['prohibited']:
                                rejected = False
                            current_nace_ids.append(nace['id'])
                        else:
                            new_nace_id = nace_obj.create(cr,uid,{'name':nace3,'meaning':libnace3 or ''})
                            current_nace_ids.append(new_nace_id)
                            rejected = False
                    else:
                        # NO NACE AT ALL => not rejected
                        rejected = False
                    if libnace3:
                        activ_desc += (libnace3+'. ')
                    if nace4:
                        nace_ids = nace_obj.search(cr,uid,[('name','=',nace4)])
                        if nace_ids:
                            nace = nace_obj.read(cr,uid,nace_ids,['prohibited'])[0]
                            if not nace['prohibited']:
                                rejected = False
                            current_nace_ids.append(nace['id'])
                        else:
                            new_nace_id = nace_obj.create(cr,uid,{'name':nace4,'meaning':libnace4 or ''})
                            current_nace_ids.append(new_nace_id)
                            rejected = False
                    else:
                        # NO NACE AT ALL => not rejected
                        rejected = False
                    if libnace4:
                        activ_desc += (libnace4+'. ')
                    if nace5:
                        nace_ids = nace_obj.search(cr,uid,[('name','=',nace5)])
                        if nace_ids:
                            nace = nace_obj.read(cr,uid,nace_ids,['prohibited'])[0]
                            if not nace['prohibited']:
                                rejected = False
                            current_nace_ids.append(nace['id'])
                        else:
                            new_nace_id = nace_obj.create(cr,uid,{'name':nace5,'meaning':libnace5 or ''})
                            current_nace_ids.append(new_nace_id)
                            rejected = False
                    else:
                        # NO NACE AT ALL => not rejected
                        rejected = False
                    if libnace5:
                        activ_desc += (libnace5+'. ')
                    activ_desc = activ_desc.strip()
                    creation_obj.create(cr,uid,{'name': current_name,
                                                'street': current_street,
                                                'zip_code': zip_code,
                                                'city': locality,
                                                'vat_num': vat_num,
                                                'legal_status': legal_status,
                                                'admin_type': 'person',
                                                'admin_courtesy': courtesy,
                                                'admin_name': nom,
                                                'admin_first_name': prenom,
                                                'admin_title': title,
                                                'admin_codes': codes,
                                                'activ_desc': activ_desc,
                                                'creation_date': creation_date,
                                                'capital': capital,
                                                'nace_ids': [(6, 0, current_nace_ids)],
                                                'keep_aside': rejected,
                                                'final_zip_id': final_zip_id,
                                               })
                    injected_vat_nums.append(vat_num)
                    imported += 1
        data['form']['msg']=_('Number of imported creations : %s') % (imported)
        return data['form']
    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('import','Import File')]},
        },
        'import': {
            'actions': [_import_data],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }

import_constitutions('import_constitutions')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

