# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import time
import netsvc
import tools
from osv import fields, osv

class sale_order(osv.osv):
    _name = "sale.order"
    _inherit = "sale.order"
    _columns= {
        'job_id': fields.many2one('res.partner.job',"Contact (Signator)",help='the contact that has signed the sale order and must receive a confirmation'),
    }
    _defaults = {
        'deadline' : lambda *a: (time.strftime('%m-%d') < '12-01') and time.strftime('%Y-12-31') or str(int(time.strftime('%Y'))+1)+'-12-31',
    }
    def htmlify(self,text):
        return text.replace('\n','<br/>')
    def action_wait(self, cr, uid, ids, *args):
        # this method replace the old one to add a new feature :
        # sending an email to the salesman associated with the
        # sale order or the partner of the sale order
        # and also an email to the contact associated with the sale_order if any
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        parameters = parameter_obj.get_value_from_names(cr,uid,['confirm_sale_from','confirm_sale_html'])
        #
        event_p = self.pool.get('res.partner.event.type').check(cr, uid, 'sale_open')
        event_obj = self.pool.get('res.partner.event')
        for o in self.browse(cr, uid, ids):
            if event_p:
                event_obj.create(cr, uid, {'name': 'Sale Order: '+ o.name,\
                        'partner_id': o.partner_id.id,\
                        'date': time.strftime('%Y-%m-%d %H:%M:%S'),\
                        'user_id': (o.user_id and o.user_id.id) or uid,\
                        'partner_type': 'customer', 'probability': 1.0,\
                        'planned_revenue': o.amount_untaxed})
            if (o.order_policy == 'manual'):
                self.write(cr, uid, [o.id], {'state': 'manual'})
            else:
                self.write(cr, uid, [o.id], {'state': 'progress'})
            if parameters.has_key('confirm_sale_from') and parameters.has_key('confirm_sale_html') and parameters['confirm_sale_from']:
                # sending of the sale_order as email to cci salesman and customer contact
                # and to additional emails givent in each product definition
                emails_to = []
                if o.user_id and o.user_id.email:
                    emails_to.append(o.user_id.email)
                else:
                    if o.partner_id and o.partner_id.id and o.partner_id.user_id.email:
                        emails_to.append(o.partner_id.user_id.email)
                if o.job_id and ( o.job_id.email or ( o.job_id.contact_id and o.job_id.contact_id.email )):
                    emails_to.append(o.job_id.email or o.job_id.contact_id.email)
                if o.order_line:
                    for line in o.order_line:
                        if line.product_id and line.product_id.cci_add_sale_emails:
                            emails = line.product_id.cci_add_sale_emails.split(';')
                            for addr_email in emails:
                                addr_email = addr_email.strip()
                                if addr_email not in emails_to:
                                    emails_to.append( addr_email )
                #
                if emails_to:
                    # construction of the email
                    proof_obj = self.pool.get('sale.advertising.proof')
                    proof_ids = proof_obj.search(cr,uid,[('target_id','=',o.id)])
                    if proof_ids:
                        proofs = proof_obj.browse(cr,uid,proof_ids)
                        list_of_proofs = u'<p><b>Preuves de parution à envoyer </b>:'
                        for proof in proofs:
                            proof_address = proof.address_id.street
                            if proof.address_id.street2:
                                proof_address += u' ' + proof.address_id.street2
                            if proof.address_id.zip_id:
                                proof_address += ( u' - ' + proof.address_id.zip_id.name + u' ' + proof.address_id.zip_id.city )
                            list_of_proofs += ( u'<br/>\n' + unicode( proof.number and proof.number or 1 ) + u' copies pour : ' + proof.name + u' à ' + proof_address )
                        list_of_proofs += u'</p>'
                    else:
                        list_of_proofs = u''
                    #    
                    details_table = u''
                    for line in o.order_line:
                        ltaxes = []
                        for tax in line.tax_id:
                            ltaxes.append(tax.name)
                        taxes = ','.join(ltaxes)
                        if line.expected_invoice_date:
                            invoice_date = line.expected_invoice_date[8:10]+'/'+line.expected_invoice_date[5:7]+'/'+line.expected_invoice_date[0:4]
                        else:
                            invoice_date = ''
                        notes_on_line = ''
                        if line.notes:
                            notes_on_line = '<br/><span style="font-size: 75%; font-style: italic;">' + self.htmlify(line.notes) + '</span>'
                        if line.product_id and line.product_id.cci_confirm_msg:
                            notes_on_line += '<br/><span style="font-size: 75%; font-style: italic;">' + self.htmlify(line.product_id.cci_confirm_msg) + '</span>'
                        details_table += u'<tr><td style="border: 1px dotted gray;">%s %s</td><td style="border: 1px dotted gray;">%10.2f &euro;</td><td style="border: 1px dotted gray;">%s</td><td style="border: 1px dotted gray;">%s</td></tr>' % (line.name,notes_on_line,line.price_subtotal,invoice_date,taxes)
                    if details_table:
                        details_table = u'<tr><th style="border: 1px dotted gray;">Desc.</th><th style="border: 1px dotted gray;">Montant</th><th style="border: 1px dotted gray;">Date fact. prévue</th><th style="border: 1px dotted gray;">TVA</th></tr>' + details_table
                    #
                    vat_num = '<br/>TVA : Inconnu'
                    if o.partner_id and o.partner_id.vat:
                        vat_num = '<br/>TVA : ' + o.partner_id.vat
                    #
                    invoice_address = o.partner_invoice_id.street
                    if o.partner_invoice_id.street2:
                        invoice_address += u' ' + o.partner_invoice_id.street2
                    if o.partner_invoice_id.zip_id:
                        invoice_address += ( u' - ' + o.partner_invoice_id.zip_id.name + u' ' + o.partner_invoice_id.zip_id.city )
                    #
                    if o.deadline:
                        validity = u'<p>Date limite de validité de l\'offre : %s </p>' % (o.deadline[8:10]+'/'+o.deadline[5:7]+'/'+o.deadline[0:4])
                    else:
                        validity = ''
                    #
                    salesman_data = u'<p>Votre Chargé de Relation</p>\n<p>'
                    if o.user_id:
                        salesman = o.user_id
                    else:
                        if o.partner_id and o.partner_id.user_id:
                            salesman = o.partner_id.user_id
                    salesman_data += salesman.name
                    if salesman.mobile:
                        salesman_data += ( '<br/>' + salesman.mobile )
                    if salesman.email:
                        salesman_data += ( '<br/><a href="mailto:%s">%s</a>' % (salesman.email,salesman.email) )
                    salesman_data += '</p>'
                    #
                    mail_subject = u'Confirmation de commande - %s' % o.name
                    mail_body = parameters['confirm_sale_html'] % (
                        o.name,
                        (o.client_order_ref and ( u" - Votre référence : " + o.client_order_ref ) or u''),
                        o.partner_id.name,
                        invoice_address,
                        vat_num,
                        (o.job_id and ( u'<br/>\nContact : ' + (o.job_id.contact_id.title or '') + u' ' + o.job_id.contact_id.name + u' ' + (o.job_id.contact_id.first_name or '') ) or u'' ),
                        '%.2f' % o.amount_untaxed,
                        '%.2f' % o.amount_total,
                        details_table,
                        list_of_proofs,
                        (o.note and ( u'<p><b>Divers </b>: ' + o.note + u'</p>') or u'' ),
                        validity,
                        salesman_data,
                        )
                    src = parameters['confirm_sale_from']
                    param_values = parameter_obj.get_value_from_names(cr,uid,['BDDeTest'])
                    if param_values.has_key('BDDeTest'):
                        bd_test = param_values['BDDeTest']
                    else:
                        bd_test = False
                    if bd_test:
                        print "------------ Msg to send --------------------"
                        print "From : " + src
                        print "To : " + ','.join(emails_to)
                        print "Subject : " + mail_subject
                        print "Body :\n" + mail_body
                    else:
                        tools.email_send(src, emails_to, mail_subject, mail_body, subtype='html')
                    self.pool.get('cci_email_sended').create(cr,uid,{'name':u'Confirmation Bon de Commande',
                                                                     'subject':mail_subject,
                                                                     'body':mail_body,
                                                                     'ressource':'sale.order:'+str(o.id)+':',
                                                                     'addr_from':src,
                                                                     'addrs_to':'['+','.join(emails_to)+']',
                                                                    })
                    if event_p:
                        event_obj.create(cr, uid, {'name': 'Email Confirm Order: '+ o.name,\
                                'partner_id': o.partner_id.id,\
                                'date': time.strftime('%Y-%m-%d %H:%M:%S'),\
                                'user_id': (o.user_id and o.user_id.id) or uid,\
                                'partner_type': 'customer',
                                'probability': 1.0,\
                                'planned_revenue': o.amount_untaxed,
                                'description':'To:' + ','.join(emails_to) + '\n\n' + mail_body})
        # confirm the attached lines
        self.pool.get('sale.order.line').button_confirm(cr, uid, [x.id for x in o.order_line])
sale_order()

class Product(osv.osv):
    '''Product'''
    _inherit = 'product.product'
    _description = 'product.product'
    _columns = {
            'cci_add_sale_emails': fields.char('Add. Emails Sale Orders', size=250, help='List of additionnal emails to send for sales orders containing this product. Separated by \';\'.'),
            'cci_confirm_msg':fields.text('Message on confirmation mail',help='This message will be added to the mail confirming the orders or this product'),
            }
Product()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

