# -*- encoding: utf-8 -*-

from osv import fields, osv
import wizard
import pooler
import os
import base64
import time
from pyExcelerator import *
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Extract Advertising Customers">
    <field name="medium" colspan="4" width="650"/>
    <field name="date_from"/>
    <field name="date_to"/>
    <field name="type" colspan="4"/>
</form>"""

fields = {
    'medium':{'string':'Medium','type':'many2one','relation':'product.category','required':True,'help':'This medium ant all his sub-medium will be considered'},
    'date_from':{'string':'From','type':'date','required':True},
    'date_to':{'string':'To','type':'date','required':True},
    'type':{'string':'Type','type':'selection','selection':[('all','All (Agencies,Published Customers,Invoiced customers)'),('agency','Only Agencies'),('published','Only Published Customers'),('invoiced','Only invoiced customers')],'required':True},
}

msg_form = """<?xml version="1.0"?>
<form string="Results">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="advertising_customers.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'advertising_customers.xls':{'string': 'Prepared file','type': 'binary','readonly': True,},
}

class advertising_extract_customers(wizard.interface):
    def _get_pcateg_ids(self,cr,uid,pcateg_parent_id):
        # extract all ids of product categories
        obj_pcateg = pooler.get_pool(cr.dbname).get('product.category')
        old_len = 0
        pcateg_ids = [ pcateg_parent_id ]
        while len(pcateg_ids) > old_len:
            new_ids = pcateg_ids[old_len:] # ids of categories added last time
            old_len = len(pcateg_ids) # the new frontier ...
            new_categs = obj_pcateg.read(cr,uid,new_ids,['child_id'])
            for categ in new_categs:
                if categ['child_id']:
                    pcateg_ids.extend(categ['child_id'])
        return pcateg_ids
    def _extract_file(self,cr,uid,data,context):
        # create list of concerned periods
        from_date = data['form']['date_from']
        to_date = data['form']['date_to']
        obj_issue = pooler.get_pool(cr.dbname).get('sale.advertising.issue')
        obj_pcat = pooler.get_pool(cr.dbname).get('product_category')
        medium_ids = self._get_pcateg_ids(cr,uid,data['form']['medium'])
        #
        date_start = min(from_date,to_date)
        date_stop = max(from_date,to_date)
        issue_ids = obj_issue.search(cr,uid,[('issue_date','>=',date_start),('issue_date','<=',date_stop),('medium','in',medium_ids)])
        
        # create empty excel file
        wb = Workbook()
        ws1 = wb.add_sheet(_('Customers'))
        ws1.write(0,0,_('partner_id'))
        ws1.write(0,1,_('address_id'))
        ws1.write(0,2,_('job_id'))
        ws1.write(0,3,_('contact_id'))
        ws1.write(0,4,_('Partner Name'))
        ws1.write(0,5,_('Street'))
        ws1.write(0,6,_('Street2'))
        ws1.write(0,7,_('Zip code'))
        ws1.write(0,8,_('City'))
        ws1.write(0,9,_('Company Phone'))
        ws1.write(0,10,_('Company EMail'))
        ws1.write(0,11,_('Last Name'))
        ws1.write(0,12,_('First Name'))
        ws1.write(0,13,_('Courtesy'))
        ws1.write(0,14,_('Mobile'))
        ws1.write(0,15,_('Function'))
        ws1.write(0,16,_('Prof Phone'))
        ws1.write(0,17,_('Prof EMail'))
        ws1.write(0,18,_('Private EMail'))
        ws1.write(0,19,_('Years'))
        ws1.write(0,20,_('Amount WoVAT'))
        # we will extract all sale order linked to advertising using the same selection as others menus in this section
        # while extracting, we construct pivot table for salesman and customers
        dPartnerJobs = {}
        order_obj = pooler.get_pool(cr.dbname).get('sale.order')
        order_ids = order_obj.search(cr,uid,[('published_customer','<>',False),('state','in',['manual','progress','shipping_except','invoice_except','done'])])
        if order_ids:
            orders = order_obj.browse(cr,uid,order_ids)
            for sorder in orders:
                current_amount = 0.0
                years = []
                for line in sorder.order_line:
                    if line.adv_issue and line.adv_issue.id in issue_ids:
                        current_year = line.adv_issue.issue_date[0:4]
                        if current_year not in years:
                            years.append(current_year)
                        current_amount += line.price_subtotal or 0.0
                if years:
                    partner_ids = []
                    if (data['form']['type'] == 'all' or data['form']['type'] == 'invoiced') and sorder.partner_id:
                        partner_ids.append(sorder.partner_id.id)
                    if (data['form']['type'] == 'all' or data['form']['type'] == 'agency') and sorder.advertising_agency and sorder.advertising_agency.id not in partner_ids:
                        partner_ids.append(sorder.advertising_agency.id)
                    if (data['form']['type'] == 'all' or data['form']['type'] == 'published') and sorder.published_customer and sorder.published_customer.id not in partner_ids:
                        partner_ids.append(sorder.published_customer.id)
                    for partner_id in partner_ids:
                        if partner_id:
                            if partner_id and sorder.job_id and sorder.partner_id.id == partner_id and sorder.job_id.active and sorder.job_id.contact_id and sorder.job_id.contact_id.active and sorder.job_id.address_id and sorder.job_id.address_id.active:
                                # we take the job_id from the sale_order if this job_id is still active
                                key = (partner_id,sorder.job_id.id)
                            else:
                                # we search for the first G else first M on default address of this partner
                                job_id = 0
                                partner = pooler.get_pool(cr.dbname).get('res.partner').browse(cr,uid,partner_id)
                                if partner.address:
                                    for addr in partner.address:
                                        if addr.type == 'default':
                                            if addr.job_ids:
                                                first_sequence = 99999
                                                for job in addr.job_ids:
                                                    if job.sequence_partner < first_sequence and job.contact_id and job.function_code_label and 'G' in job.function_code_label:
                                                        job_id = job.id
                                                        first_sequence = job.sequence_partner
                                                if not job_id:
                                                    for job in addr.job_ids:
                                                        if job.sequence_partner < first_sequence and job.contact_id and job.function_code_label and 'M' in job.function_code_label:
                                                            job_id = job.id
                                                            first_sequence = job.sequence_partner
                                key = (partner_id,job_id)
                            if dPartnerJobs.has_key(key):
                                cumul = dPartnerJobs[key]
                                cumul['amount'] += current_amount
                                for current_year in years:
                                    if current_year not in cumul['years']:
                                        cumul['years'].append(current_year)
                            else:
                                cumul = {'years':years,'amount':current_amount}
                            dPartnerJobs[key] = cumul
        linenum = 1
        obj_partner = pooler.get_pool(cr.dbname).get('res.partner')
        for (key,value) in dPartnerJobs.items():
            # couple partner_id, job_id
            ws1.write(linenum,0,key[0])
            ws1.write(linenum,2,key[1])
            ws1.write(linenum,19,','.join(value['years']))
            ws1.write(linenum,20,value['amount'])
            partner = obj_partner.browse(cr,uid,key[0])
            if partner.address:
                current_job = False
                current_addr = False
                if key[1]:
                    for addr in partner.address:
                        if addr.job_ids:
                            for job in addr.job_ids:
                                if job.id == key[1]:
                                    current_job = job
                                    current_addr = addr
                                    break
                else:
                    for addr in partner.address:
                        if addr.type == 'default':
                            current_addr = addr
                            break
                if current_addr:
                    ws1.write(linenum,1,current_addr.id)
                    ws1.write(linenum,3,(current_job and current_job.contact_id) and current_job.contact_id.id or 0)
                    if current_addr.name:
                        if current_addr.name[0:2] == '- ' or current_addr.name[0:3] == ' - ':
                            ws1.write(linenum,4,partner.name.strip() + ' ' + current_addr.name.strip() )
                        else:
                            ws1.write(linenum,4,current_addr.name.strip() )
                    else:
                         ws1.write(linenum,4,partner.name.strip())
                    ws1.write(linenum,5,current_addr.street or '')
                    ws1.write(linenum,6,current_addr.street2 or '')
                    ws1.write(linenum,7,current_addr.zip_id and current_addr.zip_id.name or '')
                    ws1.write(linenum,8,current_addr.zip_id and current_addr.zip_id.city or '')
                    ws1.write(linenum,9,current_addr.phone or '')
                    ws1.write(linenum,10,current_addr.email or '')
                    if current_job:
                        ws1.write(linenum,11,current_job.contact_id and current_job.contact_id.name or '')
                        ws1.write(linenum,12,(current_job.contact_id and current_job.contact_id.first_name) and current_job.contact_id.first_name or '')
                        ws1.write(linenum,13,(current_job.contact_id and current_job.contact_id.title) and current_job.contact_id.title or '')
                        ws1.write(linenum,14,(current_job.contact_id and current_job.contact_id.mobile) and current_job.contact_id.mobile or '')
                        ws1.write(linenum,14,(current_job.contact_id and current_job.contact_id.mobile) and current_job.contact_id.mobile or '')
                        ws1.write(linenum,15,current_job.function_label and current_job.function_label or '')
                        ws1.write(linenum,16,current_job.phone and current_job.phone or '')
                        ws1.write(linenum,17,current_job.email and current_job.email or '')
                        ws1.write(linenum,18,(current_job.contact_id and current_job.contact_id.email) and current_job.contact_id.email or '')
            linenum += 1
        
        wb.save('advertising_customers.xls')
        result_file = open('advertising_customers.xls','rb').read()

        # give the result to the user
        data['form']['msg']=_('Save the File with '".xls"' extension.')
        data['form']['advertising_customers.xls']=base64.encodestring(result_file)
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('extract','Extract')]},
        },
        'extract': {
            'actions': [_extract_file],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }
advertising_extract_customers('advertising_extract_customers')
