# -*- encoding: utf-8 -*-

from osv import fields, osv
import wizard
import pooler
import os
import base64
import time
from pyExcelerator import *
from tools.translate import _

form = """<?xml version="1.0"?>
<form string="Extract Advertising Sales and Orders">
    <field name="period_from" colspan="4" width="650"/>
    <field name="period_to" colspan="4"/>
    <field name="sales"/>
    <field name="orders"/>
</form>"""

fields = {
    'period_from':{'string':'From Period','type':'many2one','relation':'account.period','required':True},
    'period_to':{'string':'To Period','type':'many2one','relation':'account.period','required':True},
    'sales':{'string':'Lines With Sales','type':'boolean'},
    'orders':{'string':'Lines still in Orders','type':'boolean'},
}

msg_form = """<?xml version="1.0"?>
<form string="Results">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="sales_and_orders.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'sales_and_orders.xls':{'string': 'Prepared file','type': 'binary','readonly': True,},
}

class extract_sales_and_orders(wizard.interface):
    def _extract_file(self,cr,uid,data,context):
        # create list of concerned periods
        from_id = data['form']['period_from']
        to_id = data['form']['period_to']
        obj_period = pooler.get_pool(cr.dbname).get('account.period')
        period_from = obj_period.read(cr,uid,from_id,['date_start','date_stop'])
        period_to = obj_period.read(cr,uid,to_id,['date_start','date_stop'])
        date_start = min(period_from['date_start'],period_to['date_start'])
        date_stop = max(period_from['date_stop'],period_to['date_stop'])
        period_ids = obj_period.search(cr,uid,[('date_start','>=',date_start),('date_stop','<=',date_stop),('special','=',False)])
        periods = obj_period.read(cr,uid,period_ids,['name'])
        lPeriods = []
        for period in periods:
            lPeriods.append(period['name'][3:]+period['name'][0:2])
        lPeriods = sorted(lPeriods)
        # create empty excel file
        wb = Workbook()
        ws1 = wb.add_sheet(_('Details'))
        ws1.write(0,0,_('Agency'))
        ws1.write(0,1,_('Description'))
        ws1.write(0,2,_('Asked Invoice Date'))
        ws1.write(0,3,_('Invoice Period'))
        ws1.write(0,4,_('Invoice Number'))
        ws1.write(0,5,_('Origin'))
        ws1.write(0,6,_('Order Date'))
        ws1.write(0,7,_('Salesman'))
        ws1.write(0,8,_('Published Customer'))
        ws1.write(0,9,_('Amount'))
        ws1.write(0,10,_('Order state'))
        ws1.write(0,11,_('Parution Date'))
        ws1.write(0,12,_('Period for sorting'))
        ws1.write(0,13,_('Year for sorting'))
        # we will extract all sale order linked to advertising using the same selection as others menus in this section
        # while extracting, we construct pivot table for salesman and customers
        order_obj = pooler.get_pool(cr.dbname).get('sale.order')
        order_ids = order_obj.search(cr,uid,[('published_customer','<>',False),('state','in',['manual','progress','shipping_except','invoice_except','done'])])
        if order_ids:
            orders = order_obj.browse(cr,uid,order_ids)
            linenum = 1
            dSalesmenYears = {}
            dSalesmenPeriods = {}
            dCustYears = {}
            dCustPeriods = {}
            lYears = []
            #lPeriods = []
            for sorder in orders:
                for line in sorder.order_line:
                    if ( data['form']['sales'] and line.invoice_lines ) or ( data['form']['orders'] and not line.invoice_lines ):
                        # for adding this line to the pivot table : the year and/or month used depends if invoiced or not
                        if line.invoice_lines:
                            period = line.invoice_lines[0].invoice_id.period_id.name or ''
                            if not period:
                                period = 'undetermined'
                                year = 'undetermined'
                            else:
                                period = period[3:]+period[0:2]
                                year = period[0:4]
                        else:
                            if line.expected_invoice_date:
                                period = line.expected_invoice_date[0:4] + line.expected_invoice_date[5:7]
                                year = line.expected_invoice_date[0:4]
                            else:
                                period = 'undetermined'
                                year = 'undetermined'
                        if period in lPeriods:
                            # line to extract => also to add to the pivot tables
                            ws1.write(linenum,0,sorder.advertising_agency and sorder.advertising_agency.name or '')
                            ws1.write(linenum,1,line.name or '')
                            ws1.write(linenum,2,(line.expected_invoice_date or '')[0:10] )
                            if line.invoice_lines:
                                ws1.write(linenum,3,line.invoice_lines[0].invoice_id.period_id.name)
                                ws1.write(linenum,4,line.invoice_lines[0].invoice_id.number or '')
                            ws1.write(linenum,5,sorder.name or '')
                            ws1.write(linenum,6,sorder.date_order or '')
                            ws1.write(linenum,7,sorder.user_id.name or sorder.partner_id.user_id.name or '')
                            ws1.write(linenum,8,sorder.published_customer.name or '')
                            ws1.write(linenum,9,line.price_subtotal or 0.0)
                            ws1.write(linenum,10,sorder.state)
                            ws1.write(linenum,11,line.adv_issue and line.adv_issue.issue_date or '' )
                            ws1.write(linenum, 12, period )
                            ws1.write(linenum, 13, year )

                            if not year in lYears:
                                lYears.append(year)
                            #if not period in lPeriods:
                            #    lPeriods.append(period)
                            
                            salesman = ( sorder.user_id.name or sorder.partner_id.user_id.name or 'undetermined' )    
                            customer = ( sorder.published_customer.name or '' )
                            if dSalesmenYears.has_key( salesman ):
                                tot_years = dSalesmenYears[salesman]
                                if tot_years.has_key( year ):
                                    tot_years[year] += ( line.price_subtotal or 0.0 )
                                else:
                                    tot_years[year] = ( line.price_subtotal or 0.0 )
                                dSalesmenYears[salesman] = tot_years
                            else:
                                dSalesmenYears[salesman] = {year:(line.price_subtotal or 0.0)}
                            if dSalesmenPeriods.has_key( salesman ):
                                tot_periods = dSalesmenPeriods[salesman]
                                if tot_periods.has_key( period ):
                                    tot_periods[period] += ( line.price_subtotal or 0.0 )
                                else:
                                    tot_periods[period] = ( line.price_subtotal or 0.0 )
                                dSalesmenPeriods[salesman] = tot_periods
                            else:
                                dSalesmenPeriods[salesman] = {period:(line.price_subtotal or 0.0)}
                            if dCustYears.has_key( customer ):
                                tot_years = dCustYears[customer]
                                if tot_years.has_key( year ):
                                    tot_years[year] += ( line.price_subtotal or 0.0 )
                                else:
                                    tot_years[year] = ( line.price_subtotal or 0.0 )
                                dCustYears[customer] = tot_years
                            else:
                                dCustYears[customer] = {year:(line.price_subtotal or 0.0)}
                            if dCustPeriods.has_key( customer ):
                                tot_periods = dCustPeriods[customer]
                                if tot_periods.has_key( period ):
                                    tot_periods[period] += ( line.price_subtotal or 0.0 )
                                else:
                                    tot_periods[period] = ( line.price_subtotal or 0.0 )
                                dCustPeriods[customer] = tot_periods
                            else:
                                dCustPeriods[customer] = {period:(line.price_subtotal or 0.0)}
                            
                            # end of management of line
                            linenum += 1

        # sort of years, periods, salesmen and customers
        lYears = sorted(lYears)
        #lPeriods = sorted(lPeriods)
        lSalesmen = []
        for salesman in dSalesmenYears.keys():
            lSalesmen.append(salesman)
        lSalesmen = sorted(lSalesmen)
        lCustomers = []
        for customer in dCustYears.keys():
            lCustomers.append(customer)
        lCustomers = sorted(lCustomers)
        # recording of the pivot table by salesmen in sheet 2
        ws2 = wb.add_sheet(_('By Salesmen'))
        ws2.write(1,0,_('Salesman'))
        ws2.write(0,1,_('All'))
        ws2.write(0,2,_('Years'))
        coltable = 2
        dYears = {}
        dPeriods = {}
        for year in lYears:
            ws2.write(1,coltable,year)
            dYears[year] = 0.0
            coltable += 1
        for period in lPeriods:
            ws2.write(1,coltable,period[4:]+'/'+period[0:4])
            dPeriods[period] = 0.0
            coltable += 1
        ws2.write(0,2+len(lYears),_('Periods'))
        linetable = 2
        all_in_all = 0.0
        for salesman in lSalesmen:
            total = dSalesmenYears[salesman]
            ws2.write(linetable,0,salesman)
            coltable = 2
            all_years = 0.0
            for year in lYears:
                if total.has_key(year):
                    ws2.write(linetable,coltable,total[year])
                    dYears[year] += total[year]
                    all_years += total[year]
                else:
                    ws2.write(linetable,coltable,0.0)
                coltable += 1
            ws2.write(linetable,1,all_years)
            all_in_all += all_years
            linetable += 1
        ws2.write(linetable,1,all_in_all)
        linetable = 2
        for salesman in lSalesmen:
            total = dSalesmenPeriods[salesman]
            coltable = 2+len(lYears)
            for period in lPeriods:
                if total.has_key(period):
                    ws2.write(linetable,coltable,total[period])
                    dPeriods[period] += total[period]
                else:
                    ws2.write(linetable,coltable,0.0)
                coltable += 1
            linetable += 1
        linetable = 2 + len(lSalesmen)
        coltable = 2
        for year in lYears:
            ws2.write(linetable,coltable,dYears[year])
            coltable += 1
        for period in lPeriods:
            ws2.write(linetable,coltable,dPeriods[period])
            coltable += 1
        # recording of the pivot table by customers in sheet 3
        ws3 = wb.add_sheet(_('By Customers'))
        ws3.write(1,0,_('Customer'))
        ws3.write(0,1,_('All'))
        ws3.write(0,2,_('Years'))
        dYears = {}
        dPeriods = {}
        coltable = 2
        for year in lYears:
            ws3.write(1,coltable,year)
            dYears[year] = 0.0
            coltable += 1
        for period in lPeriods:
            ws3.write(1,coltable,period[4:]+'/'+period[0:4])
            dPeriods[period] = 0.0
            coltable += 1
        ws3.write(0,2+len(lYears),_('Periods'))
        linetable = 2
        all_in_all = 0.0
        for customer in lCustomers:
            total = dCustYears[customer]
            ws3.write(linetable,0,customer)
            coltable = 2
            all_years = 0.0
            for year in lYears:
                if total.has_key(year):
                    ws3.write(linetable,coltable,total[year])
                    dYears[year] += total[year]
                    all_years += total[year]
                else:
                    ws3.write(linetable,coltable,0.0)
                coltable += 1
            ws3.write(linetable,1,all_years)
            all_in_all += all_years
            linetable += 1
        ws3.write(linetable,1,all_in_all)
        linetable = 2
        for customer in lCustomers:
            total = dCustPeriods[customer]
            coltable = 2+len(lYears)
            for period in lPeriods:
                if total.has_key(period):
                    ws3.write(linetable,coltable,total[period])
                    dPeriods[period] += total[period]
                else:
                    ws3.write(linetable,coltable,0.0)
                coltable += 1
            linetable += 1
        linetable = 2 + len(lCustomers)
        coltable = 2
        for year in lYears:
            ws3.write(linetable,coltable,dYears[year])
            coltable += 1
        for period in lPeriods:
            ws3.write(linetable,coltable,dPeriods[period])
            coltable += 1
        
        wb.save('sales_and_orders.xls')
        result_file = open('sales_and_orders.xls','rb').read()

        # give the result to the user
        data['form']['msg']=_('Save the File with '".xls"' extension.')
        data['form']['sales_and_orders.xls']=base64.encodestring(result_file)
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('extract','Extract')]},
        },
        'extract': {
            'actions': [_extract_file],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }
extract_sales_and_orders('extract_sales_and_orders')
