# -*- coding: utf-8 -*-

from osv import osv
from osv import fields
from datetime import date
import time

class product_product(osv.osv):
    _inherit = 'product.product'
    _columns = {
        'cci_product_id' : fields.many2one('cci.product', 'Produit commercial associe'),
    }
product_product()

class res_partner_job(osv.osv):
    _inherit = 'res.partner.job'

    _columns = {
        'name': fields.related('address_id', 'partner_id', type='many2one',
                               relation='res.partner', string='Partenaire',
                               store=True),
    }
res_partner_job()

class cci_product(osv.osv):
    _name = 'cci.product'

    _columns = {
        'name': fields.char('Produit', size=64, required=True),
        'categories': fields.one2many('cci.product.category', 'cci_product',
                                      'Categories'),
        'commissioned': fields.boolean('Commissionned'),
        'order_type': fields.selection([('participation', 'Participation Club'),
                                        ('order', 'Devis'),
                                        ('membership', 'Adhesion')],
                                       'Type de BDC'),
        'product_ids': fields.one2many('product.product', 'cci_product_id', 'Produits comptables associes'),
        'code': fields.char('Code',size=12, required=True),
        'active':fields.boolean('Active'),
    }
    _defaults = {
        'active': lambda *a: True,
    }
cci_product()

class cci_product_category(osv.osv):
    _name = 'cci.product.category'

    _columns = {
        'name': fields.char('Categorie', size=64, required=True),
        'cci_product': fields.many2one('cci.product', 'Produit', 
                                       required=True),
        'active':fields.boolean('Active'),
        #'product': fields.many2one('product.product', 'Intitule comptable', 
        #                           required=True),
    }
    _defaults = {
        'active': lambda *a: True,
    }
cci_product_category()

class crm_case(osv.osv):
    _inherit = 'crm.case'
    
    _columns = {
        'product': fields.many2one('cci.product', 'Produit'),
        'product_category' : fields.many2one('cci.product.category',
                                              'Categorie'),
        'partner_contact_id': fields.many2one('res.partner.contact','Contact Person'),
    }    
crm_case()

class cci_contact(osv.osv):
    _name = 'cci.contact'

    def _get_year(self,cr,uid,context):
        today = date.today()
        currentYear = today.strftime("%Y")
        yearTab = []
        
        for i in xrange(int(currentYear) + 2, int(currentYear) - 2, -1):
            yearTab.append((str(i), str(i)))

        return tuple(yearTab)
    
    def _get_commission(self,cr, uid, ids, year, context):
        res= {}
        #effacement de la table
        commission_obj = self.pool.get('res.commission')
        all_com_ids = commission_obj.search(cr, uid, [])
        commission_obj.unlink(cr, uid, all_com_ids)

        ## prendre tout les product_obj commissionne
        cci_product_obj = self.pool.get('cci.product')
        cci_product_ids = cci_product_obj.search(cr, uid, [('commissioned', '=', True)])
        
        lines = []
        real_tot = obj_tot = com1_tot = com2_tot = 0

        #if view tree
        if  len(ids) != 1: 
            return res
        
        #obtenir le user_id
        cci_contact_obj = self.pool.get('cci.contact')
        cci_contact = cci_contact_obj.read(cr, uid, ids[0], ['id','user'])
        searched_user_id = cci_contact['user'][0]

        commissions = []
        for cci_product_id in cci_product_ids:
            objective = com1 = com2 = 0

            #calcul du realise
            #cr.execute("""
            #    SELECT SUM(ail.price_subtotal) as amount
            #    FROM account_invoice_line ail,
            #        account_invoice ai,
            #        cci_product_category cpc,
            #        cci_product cp
            #    WHERE ail.invoice_id = ai.id
            #        AND ail.product_id = cpc.product
            #        AND cpc.cci_product = cp.id
            #        AND cp.commissioned is true
            #        AND ai.user_id = %d
            #        AND cp.id = %d
            #        AND date_part('year', ai.date_invoice) = %d
            #    """ % (searched_user_id, cci_product_id, int(year)))
            cr.execute("""
                SELECT SUM(ail.price_subtotal) as amount
                FROM account_invoice_line ail,
                    account_invoice ai,
                    product_product pp,
                    cci_product cp
                WHERE ail.invoice_id = ai.id
                    AND ail.product_id = pp.id
                    AND pp.cci_product_id = cp.id
                    AND cp.commissioned is true
                    AND ai.user_id = %d
                    AND cp.id = %d
                    AND date_part('year', ai.date_invoice) = %d
                """ % (searched_user_id, cci_product_id, int(year)))
            lines = cr.fetchall()

            if None in lines[0]:
                del lines[0]

            realised = 0
            if len(lines) > 0:  
                realised = int(lines[0][0] or 0)
                
            #find objective
            objective_obj = self.pool.get('cci.objectif')
            obj_id = objective_obj.search(cr, uid, [('cci_contact_id', '=', ids[0]),
                                                    ('year', '=', year),
                                                    ('cci_product', '=', cci_product_id)])
            if len(obj_id) == 1:
                objective = objective_obj.browse(cr, uid, obj_id, context)[0]['objective']
                obj_id = obj_id[0]
            else:
                data_objectif = {'cci_contact_id': ids[0],
                        'year': year,
                        'cci_product': cci_product_id,
                        'objective' : 0}
                obj_id = objective_obj.create(cr, uid, data_objectif, context) 
            
            #calcul des totaux et des com
            real_tot += realised
            obj_tot += objective
            
            if realised <= objective:
                com1 = realised * 0.015
                com1_tot += realised * 0.015
            else: 
                com2 = realised * 0.025
                com2_tot += realised * 0.025
            
            data = {'product': cci_product_id,
                    'realised': realised,
                    'objectif': objective,
                    'objective_id': obj_id,
                    'com1' : com1,
                    'com2' : com2,
                    }
            context['read'] = True
            id = commission_obj.create(cr, uid, data, context)

            commissions.append(int(id))
            
        res.update({'commission': commissions,
                    'realised_total': real_tot,
                    'objective_total': obj_tot,
                    'com1_total': com1_tot,
                    'com2_total': com2_tot
                  })

        return res
    
    def onchange_year(self, cr, uid, ids, years, context=None):
        if context is None:
            context = {}
        return {'value':self._get_commission(cr, uid, ids, years, context)}
    
    def read(self, cr, uid, ids, fields, context=None, load='_classic_read'):
        res = {}
        res = super(cci_contact, self).read(cr, uid, ids, fields, context, load)
        
        if 'commission' in fields:
            coms = self.browse(cr, uid, ids, context)
            for com in coms:
                year = com.years
                if not year:
                    today = date.today()
                    year = today.strftime("%Y")
            res[0].update(self._get_commission(cr, uid, ids, year, context))

        return res
    
    #def create(self, cr, uid, vals, context=None):
    #    if context is None:
    #        context = {}
            
        #res = {}
        #if 'groups_id' in vals and vals['groups_id'][0] and \
            #vals['groups_id'][0] > 2:
            #for group in self.pool.get('res.groups').browse(cr, uid, 
            #                                        vals['groups_id'][0][2]):
                #if group.name == 'Sale / Salesman':
                #    vals['salesman'] = True
        
        #res = super(res_users, self).create(cr, uid, vals, context)
        
        #return res
    
    def write(self, cr, uid, ids, data, context=None):
        if 'years' in data:
            today = date.today()
            year = today.strftime("%Y")
            data['years'] = year
        
        #if 'groups_id' in data and data['groups_id'][0] and \
        #    len(data['groups_id'][0]) > 2:
            #for group in self.pool.get('res.groups').browse(cr, uid, 
            #                                        data['groups_id'][0][2]):
                #if group.name == 'Sale / Salesman':
                #    data['salesman'] = True
                    
            #if not 'salesman' in data:
            #    data['salesman'] = False
                   
        return super(cci_contact,self).write(cr, uid, ids, data, context)

    _columns = {
        'name': fields.char('Nom', size=64, required=True),
        'user': fields.many2one('res.users', 'Utilisateur', required=True),
        'years':fields.selection(_get_year,'Years'),
        'commission': fields.one2many('res.commission', 'product', 'Commission'),
        'realised_total': fields.float('Realise', readonly=True),
        'objective_total':fields.float('Objectif',readonly=True),
        'com1_total':fields.float('Com (1,5pourcent)', readonly=True),
        'com2_total':fields.float('Com (2,5pourcent)', readonly=True),
        'commissioned': fields.boolean('Commissionned'),
        'active': fields.boolean('Active'),
    }
    _defaults = {
        'active': lambda *a: True,
        'commissioned': lambda *a: False,
    }
cci_contact()

class res_partner_interest(osv.osv):
    _name = 'res.partner.interest'
    
    #def _get_orders(self, cr, uid, ids, name, args, context=None):
    #    if context is None:
    #        context = {}
    #    res = {}
        
        #for interest in self.browse(cr, uid, ids, context=context):
        #    if interest.category:
        #        product_id = interest.category.product.id
        #        partner_id = interest.partner.id
        #        year = interest.partner.year
        #        
        #        cr.execute("SELECT sum(ail.price_subtotal) \
        #                    FROM account_invoice_line ail, account_invoice ai \
        #                    WHERE ail.invoice_id = ai.id \
        #                    AND ai.partner_id = " + str(partner_id) + 
        #                    " AND ail.product_id = " + str(product_id) + 
        #                    " AND ai.state in ('open', 'paid')")
        #        value = cr.fetchone()
        #        if value[0]:
        #            res[interest.id] = value[0]
        #        else:
        #            res[interest.id] = False
        #    else:
        #        res[interest.id] = False 
        #
   #     return res
    
    _columns = {
        'partner': fields.many2one('res.partner', 'Partenaire'),
        'date': fields.date('Date'),
        'product': fields.many2one('cci.product', 'Produit'),
        'cci_contact': fields.many2one('cci.contact', 'Contact CCI'),
        'contact': fields.many2one('res.partner.job', 'Contact client'),
        'category': fields.many2one('cci.product.category', 'Categorie'),
        'turnover_hoped': fields.integer('CA espere'),
        #'orders': fields.function(_get_orders, method=True, type='float', 
        #                          digits=(16, 2), string='BDC'), 
        'next_action': fields.date('Prochaine action'),
        'cci_contact_follow': fields.many2one('res.users', 
                                              'Contact CCI Suivi'),
        'description': fields.char('Description/commentaire', size=1024),
        #'case_id': fields.many2one('crm.case','CRM Case'),
        'year': fields.integer('Prospection Year'),
        'positive': fields.boolean('Positive Conclusion'),
        'active': fields.boolean('Active'),
    }
    _defaults = {
        'active': lambda *a: True,
        'positive': lambda *a: False,
        'date': lambda *a: time.strftime('%Y-%m-%d'),
    }

#    def create(self, cr, uid, vals, context=None):
#        if context is None:
#            context = {}
#        
#        crm_section_obj = self.pool.get('crm.case.section')
#        partner = False
#        description = ''
#        next_action = False
#            
#        if 'partner' in vals:
#            partner = vals['partner']
#        
#        if 'description' in vals and vals['description']:
#            description = vals['description']
#            
#        if 'next_action' in vals:
#                next_action = vals['next_action']
#
#        section_id = crm_section_obj.search(cr, uid, 
#                                            [('code', '=', 'CRMInt')])[0]
#        new_case_id = 0
#        if 'cci_contact_follow' in vals and section_id and next_action:
#            linked_contact_id = False
#            if 'contact' in vals:
#                job_obj = self.pool.get('res.partner.job')
#                linked_job = job_obj.read(cr,uid,vals['contact'],['id','contact_id'])
#                if linked_job['contact_id']:
#                    linked_contact_id = linked_job['contact_id'][0]
#            new_case_id = self.pool.get('crm.case').create(cr, uid, 
#                                    {'partner_id': partner,
#                                     'date': time.strftime('%Y-%m-%d'),
#                                     'user_id': vals['cci_contact_follow'],
#                                     'name': description,
#                                     'section_id': section_id,
#                                     'date_deadline': next_action,
#                                     'product': vals['product'],
#                                     'product_category': vals['category'],
#                                     'planned_revenue':vals['turnover_hoped'],
#                                     'partner_contact_id':linked_contact_id,
#                                     }, context),
#            new_case_id = int(new_case_id[0])
#            vals['case_id'] = new_case_id
#        res_id = super(res_partner_interest, self).create(cr, uid, 
#                                                         vals, context)
#        if new_case_id > 0:
#            result = self.pool.get('crm.case').write(cr, uid, [new_case_id], {'partner_interest_id':res_id} )
#        return res_id
    
#    def write(self, cr, uid, ids, vals, context=None):
#        if context is None:
#            context = {}
#        
#        crm_section_obj = self.pool.get('crm.case.section')
#           
#        interest = self.browse(cr, uid, ids, context)[0]
#        
#        partner = interest.partner.id
#        
#        if 'description' in vals and vals['description']:
#            description = vals['description']
#        elif interest.description:
#            description = interest.description
#        else:
#            description = ''
#            
#        if 'next_action' in vals:
#            next_action = vals['next_action']
#        else:
#            next_action = interest.next_action
#            
#        section_id = crm_section_obj.search(cr, uid, [('code', '=', 'CRMInt')])[0]
#
#        if 'cci_contact_follow' in vals:
#            cci_contact_follow = vals['cci_contact_follow']
#        else:
#            cci_contact_follow = self.browse(cr, uid, ids[0]).cci_contact_follow.id
#
#        # management of linked crm_case
#        # if exists but not more next_action date => close crm_case
#        # if not exist but cci_contact_follow and next_action date => create draft
#        # otherwise, if exist update deadline date, description, product, cci_contact_follow, turnover
#        if interest.case_id.id and 'next_action' in vals and not vals['next_action']:
#            # close the linked crm_case
#            self.pool.get('crm.case').case_close(cr, uid, [interest.case_id.id] )
#        elif not interest.case_id.id and next_action and cci_contact_follow and section_id:
#            # create a new case linked to this interest
#            linked_contact_id = False
#            if 'contact' in vals:
#                job_obj = self.pool.get('res.partner.job')
#                linked_job = job_obj.read(cr,uid,vals['contact'],['id','contact_id'])
#                if linked_job['contact_id']:
#                    linked_contact_id = linked_job['contact_id'][0]
#            new_case_id = self.pool.get('crm.case').create(cr, uid, 
#                                    {'partner_id': partner,
#                                     'date': time.strftime('%Y-%m-%d'),
#                                     'user_id': cci_contact_follow,
#                                     'name': description,
#                                     'section_id': section_id,
#                                     'date_deadline': next_action,
#                                     'product': vals['product'],
#                                     'product_category': vals['category'],
#                                     'planned_revenue':vals['turnover_hoped'],
#                                     'partner_interest_id':ids[0],
#                                     'partner_contact_id':linked_contact_id,
#                                     }, context),
#            new_case_id = int(new_case_id[0])
#            vals['case_id'] = new_case_id
#        elif interest.case_id.id and not ('no_update_case' in context ):
#            # write back to crm_case values found here
#            linked_contact_id = False
#            if 'contact' in vals:
#                job_obj = self.pool.get('res.partner.job')
#                linked_job = job_obj.read(cr,uid,vals['contact'],['id','contact_id'])
#                if linked_job['contact_id']:
#                    linked_contact_id = linked_job['contact_id'][0]
#            if 'cci_contact_follow' in vals:
#                user_id = vals['cci_contact_follow']
#            else:
#                user_id = False
#            self.pool.get('crm.case').write(cr, uid,[interest.case_id.id], 
#                                    {'partner_id': partner,
#                                     'date': time.strftime('%Y-%m-%d'),
#                                     'user_id': user_id,
#                                     'name': description,
#                                     'section_id': section_id,
#                                     'date_deadline': next_action,
#                                     'product': vals['product'],
#                                     'product_category': vals['category'],
#                                     'planned_revenue':vals['turnover_hoped'],
#                                     'partner_contact_id':linked_contact_id,
#                                     }, context)
#        res = super(res_partner_interest, self).write(cr, uid, ids, vals, context)
#        return res
res_partner_interest()

class res_partner_history(osv.osv):
    _name = 'res.partner.history'
    _rec_name = 'description'

    _columns = {
        'partner': fields.many2one('res.partner', 'Partenaire'),
        'date': fields.date('Date'),
        'cci_contact': fields.many2one('cci.contact', 'Contact CCI'),
        'contact': fields.many2one('res.partner.job', 'Contact client'),
        'action': fields.selection([
            ('appel_sortant','Appel sortant'),
            ('appel_entrant','Appel entrant'),
            ('commando','Commando'),
            ('mail','Mail'),
            ('site','Site Internet'),
            ('meeting_cci','Meeting CCI'),
            ('meeting_externe','Meeting externe'),
            ('midi','Midi'),
            ('rdv','RDV'),
            ('publication','Publication'),], 'Action'),
        # be careful : if you change the definitions here, we must also adapt wizard/extract_actions.py
        # and also in wizard/create_interests.py
        'product': fields.many2one('cci.product', 'Produit'),
        'category': fields.many2one('cci.product.category', 'Categorie'),
        'description': fields.char('Description/commentaire', size=1024),
        'next_action': fields.date('Prochaine action'),
        'cci_contact_follow': fields.many2one('res.users', 
                                              'Contact CCI Suivi'),
        #'case_id': fields.many2one('crm.case','CRM Case'),
        'state': fields.selection([('open','Open'),('closed','Closed')],'State'),
    }

    _defaults = {
        'state': lambda *a: 'open',
        'date': lambda *a: time.strftime('%Y-%m-%d'),
    }
#    def create(self, cr, uid, vals, context=None):
#        if context is None:
#            context = {}
#        
#        crm_section_obj = self.pool.get('crm.case.section')
#        partner = False
#        description = ''
#        next_action = False
#            
#        if 'partner' in vals:
#            partner = vals['partner']
#        
#        if 'description' in vals and vals['description']:
#            description = vals['description']
#            
#        if 'next_action' in vals:
#                next_action = vals['next_action']
#
#        section_id = crm_section_obj.search(cr, uid, 
#                                            [('code', '=', 'CRMHistoPartner')])[0]
#        new_case_id = 0
#        if 'cci_contact_follow' in vals and section_id and next_action:
#            linked_contact_id = False
#            if 'contact' in vals and vals['contact'] and vals['contact'] > 0:
#                job_obj = self.pool.get('res.partner.job')
#                linked_job = job_obj.read(cr,uid,vals['contact'],['id','contact_id'])
#                if linked_job['contact_id']:
#                    linked_contact_id = linked_job['contact_id'][0]
#            new_case_id = self.pool.get('crm.case').create(cr, uid,
#                                    {'partner_id': partner,
#                                     'date': time.strftime('%Y-%m-%d'),
#                                     'user_id': vals['cci_contact_follow'],
#                                     'name': description,
#                                     'section_id': section_id,
#                                     'date_deadline': next_action,
#                                     'product': vals['product'],
#                                     'product_category': vals['category'],
#                                     'partner_contact_id':linked_contact_id,
#                                     }, context),
#            new_case_id = int(new_case_id[0])
#            vals['case_id'] = new_case_id
#        res_id = super(res_partner_history, self).create(cr, uid, 
#                                                          vals, context)
#        if new_case_id > 0:
#            result = self.pool.get('crm.case').write(cr, uid, [new_case_id], {'partner_history_id':res_id} )
#        return res_id
    
#    def write(self, cr, uid, ids, vals, context=None):
#        if context is None:
#            context = {}
#        
#        crm_section_obj = self.pool.get('crm.case.section')
#            
#        history = self.browse(cr, uid, ids, context)[0]
#        
#        partner = history.partner.id
#        
#        if 'description' in vals and vals['description']:
#            description = vals['description']
#        elif history.description:
#            description = history.description
#        else:
#            description = ''
#            
#        if 'next_action' in vals:
#            next_action = vals['next_action']
#        else:
#            next_action = history.next_action
#            
#        section_id = crm_section_obj.search(cr, uid, [('code', '=', 'CRMHistoPartner')])[0]
#
#        if 'cci_contact_follow' in vals:
#            cci_contact_follow = vals['cci_contact_follow']
#        else:
#            cci_contact_follow = self.browse(cr, uid, ids[0]).cci_contact_follow.id
#
#        # management of linked crm_case
#        # if exists but not more next_action date => close crm_case
#        # if not exist but cci_contact_follow and next_action date => create draft
#        # otherwise, if exist update deadline date, description, product, cci_contact_follow, turnover
#        if history.case_id.id and 'next_action' in vals and not vals['next_action']:
#            # close the linked crm_case
#            self.pool.get('crm.case').case_close(cr, uid, [interest.case_id.id] )
#        elif not history.case_id.id and next_action and cci_contact_follow and section_id:
#            # create a new case linked to this interest
#            linked_contact_id = False
#            if 'contact' in vals and vals['contact'] and vals['contact']>0:
#                job_obj = self.pool.get('res.partner.job')
#                linked_job = job_obj.read(cr,uid,vals['contact'],['id','contact_id'])
#                if linked_job['contact_id']:
#                    linked_contact_id = linked_job['contact_id'][0]
#            new_case_id = self.pool.get('crm.case').create(cr, uid, 
#                                    {'partner_id': partner,
#                                     'date': time.strftime('%Y-%m-%d'),
#                                     'user_id': cci_contact_follow,
#                                     'name': description,
#                                     'section_id': section_id,
#                                     'date_deadline': next_action,
#                                     'product': vals['product'],
#                                     'product_category': vals['category'],
#                                     'partner_history_id':ids[0],
#                                     'partner_contact_id':linked_contact_id,
#                                     }, context),
#            new_case_id = int(new_case_id[0])
#            vals['case_id'] = new_case_id
#        elif history.case_id.id and not ('no_update_case' in context ):
#            # write back to crm_case values found here
#            linked_contact_id = False
#            if 'contact' in vals and vals['contact'] and vals['contact'] > 0:
#                job_obj = self.pool.get('res.partner.job')
#                linked_job = job_obj.read(cr,uid,vals['contact'],['id','contact_id'])
#                if linked_job['contact_id']:
#                    linked_contact_id = linked_job['contact_id'][0]
#            self.pool.get('crm.case').write(cr, uid,[history.case_id.id], 
#                                    {'partner_id': partner,
#                                     'date': time.strftime('%Y-%m-%d'),
#                                     'user_id': vals['cci_contact_follow'],
#                                     'name': description,
#                                     'section_id': section_id,
#                                     'date_deadline': next_action,
#                                     'product': vals['product'],
#                                     'product_category': vals['category'],
#                                     'partner_contact_id':linked_contact_id,
#                                     }, context)
#        res = super(res_partner_history, self).write(cr, uid, ids, vals, context)
#        return res
#    
res_partner_history()

class res_partner(osv.osv):
    _inherit = 'res.partner'
    
    #def _get_year(self,cr,uid,context):
    #    today = date.today()
    #    currentYear = today.strftime("%Y")
    #    
    #    yearTab = []
    #    for i in xrange(int(currentYear), 2007, -1):
    #        yearTab.append((str(i),str(i)))

    #    return tuple(yearTab)

    def _get_turnover(self, cr, uid, ids, name, args, context=None):
        if context is None:
            context = {}
        res = {}
       
        for partner in self.browse(cr, uid, ids, context):
            turnover = 0
            for interest in partner.interest_year:
                turnover += interest.turnover_hoped
            
            res[partner.id] = turnover
        return res
        
    #def _get_turnover_next(self, cr, uid, ids, name, args, context=None):
    #    if context is None:
    #        context = {}
    #    res = {}
        
    #    for partner in self.browse(cr, uid, ids, context):
    #        turnover = 0
    #        for interest in partner.interest_next_year:
    #            turnover += interest.turnover_hoped
            
    #        res[partner.id] = turnover
    #    return res
        
    #    return res
    
    #def _get_orders(self, cr, uid, ids, name, args, context=None):
    #    if context is None:
    #        context = {}
    #    res = {}
        
    #    for partner in self.browse(cr, uid, ids, context):
            
    #        if partner.year == False:
    #            year = str(time.strftime('%Y'))
    #        else:
    #            year = str(partner.year)
            
    #        cr.execute("SELECT sum(ail.price_subtotal) \
    #                    FROM account_invoice_line ail, account_invoice ai \
    #                    WHERE ail.invoice_id = ai.id \
    #                    AND ai.partner_id = " + str(partner.id) + 
    #                    " AND ai.date_invoice BETWEEN '" + year + 
    #                    "-01-01' AND '" + year + "-12-31' " + 
    #                    "AND ai.state in ('open', 'paid')")
    #        value = cr.fetchone()
            
    #        if value[0]:
    #            res[partner.id] = value[0]
    #        else:
    #            res[partner.id] = False
        
    #    return res
        
    #def _get_orders_next(self, cr, uid, ids, name, args, context=None):
    #    if context is None:
    #        context = {}
    #    res = {}
        
    #    for partner in self.browse(cr, uid, ids, context):
    #        
    #        if partner.year == False:
    #            year = str(time.strftime('%Y'))
    #        else:
    #            year =str(partner.year + 1)
    #        
    #        cr.execute("SELECT sum(ail.price_subtotal) \
    #                    FROM account_invoice_line ail, account_invoice ai \
    #                    WHERE ail.invoice_id = ai.id \
    #                    AND ai.partner_id = " + str(partner.id) + 
    #                    " AND ai.date_invoice BETWEEN '" + year + 
    #                    "-01-01' AND '" + year + "-12-31' " + 
    #                    "AND ai.state in ('open', 'paid')")
    #        value = cr.fetchone()
            
    #        if value[0]:
    #            res[partner.id] = value[0]
    #        else:
    #            res[partner.id] = False
        
    #    return res

    _columns = {
        #'year': fields.integer('Year'),
        'tunover_hoped': fields.function(_get_turnover, method=True, 
                                 type='float', 
                                 digits=(16, 2), string='Total CA espere'),
        #'tunover_hoped_next': fields.function(_get_turnover_next, method=True, 
        #                         type='float', 
        #                         digits=(16, 2), string='Total CA espéré'),
        #'orders': fields.function(_get_orders, method=True, 
        #                 type='float', 
        #                 digits=(16, 2), string='Total BDC saison en cours'),
        #'orders_next': fields.function(_get_orders_next, method=True, 
        #                 type='float', 
        #                 digits=(16, 2), string='Total BDC saison prochaine'),
        'history': fields.one2many('res.partner.history', 'partner',
                                  'Historique'),
        'interest_year': fields.one2many('res.partner.interest', 'partner', 
                                    'Marque d\'interet'),
        #'cci_turnovers': fields.one2many('cci.turnover', 'partner', 
        #                                 'Turnovers'),
        #'interest_next_year': fields.one2many('res.partner.interest.next', 
        #                                      'partner', 
        #                            'Marque d\'intéret (saison prochaine)'),
        #'years': fields.selection(_get_year,'Years'),
        #'total': fields.integer('Total'),
    }

    #_defaults = {
    #    'year': lambda *a: int(time.strftime('%Y')),
    #}
    
    #def year_changed(self, cr, uid, ids, years):
    #    res = self.read(cr, uid, ids, ['cci_turnovers', 'years'], 
    #                    context={'years': int(years)})
    #    return {'value': res[0]}
                                                                     
    #def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
    #    if fields is None:
    #        fields = []

    #    res = {}
    #    res = super(res_partner, self).read(cr, uid, ids, fields, context, load)
    #    cci_product_obj = self.pool.get('cci.product')

    #    if 'cci_turnovers' in fields:
    #        turnover_obj = self.pool.get('cci.turnover')
            
            # remove all turnovers
    #        all_turnovers = turnover_obj.search(cr, uid, [])
    #        turnover_obj.unlink(cr, uid, all_turnovers)
            
    #        cr.execute("""
    #            SELECT cp.id AS product_name,
    #                ai.partner_id,
    #                date_part('year', ai.date_invoice) AS date_year,
    #                SUM(ail.price_subtotal) AS total
    #            FROM account_invoice ai
    #                INNER JOIN account_invoice_line ail
    #                    ON ai.id = ail.invoice_id
    #                INNER JOIN product_product p
    #                    ON ail.product_id = p.id
    #                INNER JOIN cci_product_category cpc
    #                    ON ail.product_id = cpc.product
    #                INNER JOIN cci_product cp
    #                    ON cp.id = cpc.cci_product 
    #            WHERE ai.state in ('open', 'paid') 
    #            GROUP BY cp.id, ai.partner_id, date_year
    #        """)
    #        lines = cr.fetchall()
            
    #        # relink new turnover
    #        res[0]['cci_turnovers'] = []
            
    #        total = 0
    #        for line in lines:
    #            # computes new turnovers
    #            if 'years' in res[0] and not res[0]['years']:
    #                res[0]['years'] = int(time.strftime('%Y'))
    #            
    #            if context and context.has_key('years'):
    #                res[0]['years'] = context['years']

    #            if 'years' in res[0] and ids[0] == line[1] and \
    #                int(res[0]['years']) == int(line[2]):
    #                data = {
    #                    'cci_product': line[0],
    #                    'partner': line[1],
    #                    'sum_price': line[3],
    #                    'years': res[0]['years']
    #                }
    #                turnover_id = turnover_obj.create(cr, uid, data, context)
    #                res[0]['cci_turnovers'].append(turnover_id)
    #                total += line[3]
                    
    #        cci_product_ids = cci_product_obj.search(cr, uid, [])
            
    #        if context and context.has_key('years'):
    #            years = context['years']
    #        else:
    #            years = int(time.strftime('%Y'))
            
    #        for partner_id in ids:
    #            for cci_product_id in cci_product_ids:
    #                turnover_ids = turnover_obj.search(cr, uid, 
    #                                    [('partner', '=', partner_id), 
    #                                     ('cci_product', '=', cci_product_id)])
                    
    #                if not turnover_ids:
    #                    data = {
    #                        'cci_product': cci_product_id,
    #                        'partner': partner_id,
    #                        'sum_price': 0,
    #                        'years': years,
    #                    }
    #                    turnover_id = turnover_obj.create(cr, uid, data, 
    #                                                      context)
    #                    res[0]['cci_turnovers'].append(turnover_id)
                
    #        res[0]['total'] = total
                            
    #    return res
res_partner()

class cci_turnover(osv.osv):
    _name = 'cci.turnover'
    _rec_name = 'cci_product'
    
    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        if fields is None:
            fields = []

        res = super(cci_turnover, self).read(cr, uid, ids, fields, context, load)
        if 'details' in fields:
            turnover = self.browse(cr, uid, ids[0], context)
            detail_obj = self.pool.get('cci.turnover.details')
            
            # remove all details
            all_details = detail_obj.search(cr, uid, [])
            detail_obj.unlink(cr, uid, all_details)
            
            cr.execute("""
                SELECT ai.number, pt.name AS product, ail.price_subtotal, ail.name
                FROM account_invoice ai
                    INNER JOIN account_invoice_line ail
                        ON ai.id = ail.invoice_id
                    INNER JOIN product_product p
                        ON ail.product_id = p.id
                    INNER JOIN cci_product cp
                        ON p.cci_product_id = cp.id
                    INNER JOIN product_template as pt
                        ON p.product_tmpl_id = pt.id
               WHERE cp.id = %s
                    AND ai.partner_id = %s
                    AND date_part('year', ai.date_invoice) = %d 
                    AND ai.state in ('open', 'paid') AND ai.type in ('out_invoice','out_refund')
                """ % (turnover.cci_product.id, 
                       turnover.partner.id,
                       int(turnover.years)))
            lines = cr.fetchall()
            for line in lines:
                data = {
                        'invoice_number': line[0],
                        'product_name': line[1],
                        'price': line[2],
                        'invoice_line_desc':line[3],
                    }
                detail_id = detail_obj.create(cr, uid, data, context)
                res[0]['details'].append(detail_id)
        
        return res
    
    _columns = {
        'cci_product': fields.many2one('cci.product', 'Product', readonly=True),
        'sum_price': fields.float('Amount',digits=(16,2), readonly=True),
        'partner': fields.many2one('res.partner', 'Partner', readonly=True),
        'details': fields.one2many('cci.turnover.details', 'turnover', 'Details', readonly=True),
        'years': fields.char('Year', size=4, readonly=True),
        'total': fields.integer('Total', readonly=True),
    }

cci_turnover()

class cci_turnover_details(osv.osv):
    _name = 'cci.turnover.details'
    _rec_name = 'product_name'
    
    _columns = {
        'invoice_number': fields.char('Invoice Number', size=64, readonly=True),
        'product_name': fields.char('Account product', size=64, readonly=True),
        'price': fields.float('Amount',digits=(16,2), readonly=True),
        'turnover': fields.many2one('cci.turnover', 'Turnover', readonly=True),
        'invoice_line_desc': fields.char('Invoice Line Description', size=120, readonly=True),
    }

cci_turnover_details()

class crm_case(osv.osv):
    _inherit = 'crm.case'

#    def write(self, cr, uid, ids, vals, context=None):
#        if context is None:
#            context = {}
#        cases = self.browse(cr, uid, ids)
#        for case in cases:
#            if case.partner_interest_id and case.partner_interest_id.id:
#                if 'planned_revenue' in vals or 'product' in vals or 'product_category' in vals or 'date_deadline' in vals or 'name' in vals \
#                    or 'user_id' in vals or 'partner_id' in vals or 'state' in vals:
#                    obj_partner_interest = self.pool.get('res.partner.interest')
#                    interest = obj_partner_interest.read(cr,uid,case.partner_interest_id.id,['next_action','description','cci_contact_follow'])
#                    if 'planned_revenue' in vals:
#                        new_revenue = vals['planned_revenue']
#                    else:
#                        new_revenue = case.planned_revenue
#                    if 'product' in vals:
#                        new_product = vals['product']
#                    else:
#                        new_product = case.product.id
#                    if 'product_category' in vals:
#                        new_category = vals['product_category']
#                    else:
#                        new_category = case.product_category.id
#                    if 'date_deadline' in vals:
#                        new_next = vals['date_deadline']
#                    else:
#                        new_next = case.date_deadline
#                    if 'name' in vals:
#                        new_desc = vals['name']
#                    else:
#                        new_desc = case.name
#                    if 'user_id' in vals:
#                        new_cci_contact = vals['user_id']
#                    else:
#                        new_cci_contact = case.user_id.id
#                    if 'partner_id' in vals:
#                        new_partner = vals['partner_id']
#                    else:
#                        new_partner = case.partner_id.id
#                    if 'state' in vals:
#                        new_state = vals['state']
#                    else:
#                        new_state = case.state
#                    new_active = ( new_state in ['draft','open','pending'] )
#                    newvalues = {'active':new_active,'turnover_hoped':new_revenue,'product':new_product,'category':new_category,
#                                 'next_action':new_next[0:10],'description':new_desc,'cci_contact_follow':new_cci_contact,'partner':new_partner}
#                    obj_partner_interest.write(cr,uid, [case.partner_interest_id.id], newvalues, {'no_update_case':True} )
#        res = super(crm_case, self).write(cr, uid, ids, vals, context)
#        return res
    
    _columns = {
        'name': fields.char('Description',size=1024,required=True),
        #'partner_interest_id': fields.many2one('res.partner.interest','Interest Mark'),
        #'partner_history_id': fields.many2one('res.partner.history','Partner History'),
    }    
crm_case()

class crm_cci_todo(osv.osv):
    _name = 'crm_cci.todo'
    _columns = {
        'model': fields.char('Model',size=16), # 'Marque d'intérêt' or 'Historique'
        'res_id': fields.integer('Internal Source ID'),
        'partner': fields.many2one('res.partner', 'Partenaire'),
        'date': fields.date('Date'),
        'product': fields.many2one('cci.product', 'Produit'),
        'cci_contact': fields.many2one('cci.contact', 'Contact CCI'),
        'contact': fields.many2one('res.partner.job', 'Contact client'),
        'category': fields.many2one('cci.product.category', 'Categorie'),
        'turnover_hoped': fields.integer('CA espere'),
        'next_action': fields.date('Prochaine action'),
        'cci_contact_follow': fields.many2one('res.users', 'Contact CCI Suivi'),
        'description': fields.char('Description/commentaire', size=1024),
        'year': fields.integer('Prospection Year'),
        'action': fields.selection([
            ('appel_sortant','Appel sortant'),
            ('appel_entrant','Appel entrant'),
            ('commando','Commando'),
            ('mail','Mail'),
            ('site','Site Internet'),
            ('meeting_cci','Meeting CCI'),
            ('meeting_externe','Meeting externe'),
            ('midi','Midi'),
            ('rdv','RDV'),
            ('publication','Publication'),
            ('/','Aucune')], 'Action'),
        'state': fields.selection([('open','Open'),('closed','Closed')],'State'),
        'company_phone': fields.char('Company Phone', size=64),
        'prof_phone': fields.char('Prof. Phone', size=64),
        'mobile': fields.char('Mobile', size=64),
        'company_email': fields.char('Company EMail', size=240),
        'prof_email': fields.char('Prof. Email', size=240),
        'positive': fields.boolean('Positive Conclusion'),
    }
    _order = 'next_action'
    
    def get_phone_country_prefix(self,cr,uid):
        result = []
        obj_country = self.pool.get('cci.country')
        country_ids = obj_country.search(cr,uid,[('phoneprefix','!=',False),('phoneprefix','!=',0)])
        if country_ids:
            countries = obj_country.read(cr,uid,country_ids,['phoneprefix'])
            result = [str(x['phoneprefix']) for x in countries]
        return result

    def convert_phone(self,cr,uid,string):
        PHONE_COUNTRY_PREFIX = self.get_phone_country_prefix(cr,uid)
        def only_digits(string):
            cleaned = ''
            for carac in string:
                if carac in '0123456789':
                    cleaned += carac
            return cleaned
        result = ''
        string = string and only_digits(string) or ''
        if len(string) > 0:
            if len(string) == 9:
                if string[0:2] in ['02','03','04','09']:
                    result = string[0:2] + "/" + string[2:5] + "." + string[5:7] + "." + string[7:]
                else:
                    result = string[0:3] + "/" + string[3:5] + "." + string[5:7] + "." + string[7:]
            elif len(string) == 10:
                result = string[0:4] + "/" + string[4:6] + "." + string[6:8] + "." + string[8:]
            else:
                # international number
                #print string
                if string[0:2] == '00':
                    # search after a country with this prefix
                    prefix = string[2:4]
                    if prefix not in PHONE_COUNTRY_PREFIX:
                        prefix = string[2:5]
                        if prefix not in PHONE_COUNTRY_PREFIX:
                            prefix = string[2:6]
                            if prefix not in PHONE_COUNTRY_PREFIX:
                                prefix = ''
                    if prefix:
                        result = '+' + string[2:2+len(prefix)] + ' ' + string[2+len(prefix):4+len(prefix)]
                        rest = string[4+len(prefix):]
                        while len(rest) > 3:
                            result += '.' + rest[0:2]
                            rest = rest[2:]
                        result += '.' + rest
                    else:
                        result = 'International:'+string
        return result

    def onchange_contact(self, cr, uid, ids, contact, context=None):
        if contact:
            job_obj = self.pool.get('res.partner.job')
            job = job_obj.browse(cr,uid,[contact])[0]
            pphone = self.convert_phone(cr,uid,job.phone )
            if job.contact_id:
                pemail = job.email or job.contact_id.email or ''
            else:
                pemail = ''
            if job.address_id:
                cphone = self.convert_phone(cr,uid,job.address_id.phone )
                cemail = job.address_id.email or ''
            else:
                cphone = ''
                cemail = ''
            if job.contact_id:
                pmobile = self.convert_phone(cr,uid,job.contact_id.mobile )
            else:
                pmobile = ''
        else:
            cphone = ''
            pphone = ''
            cemail = ''
            pemail = ''
            pmobile = ''
        return {'value':{'company_phone': cphone,
                         'prof_phone': pphone,
                         'mobile': pmobile,
                         'company_email': cemail,
                         'prof_email': pemail,
                }}

    def _onlydigits(self,string):
        result = ''
        string = string or ''
        for carac in string:
            if carac in '0123456789':
                result += carac
        return result

    def action_dial_company_phone(self, cr, uid, ids, context=None):
        '''Function called by the button 'Dial' next to the 'company_phone' field'''
        erp_number = self._onlydigits(self.read(cr, uid, ids, ['company_phone'], context=context)[0]['company_phone'])
        self.pool.get('escaux.server').dial(cr, uid, ids, erp_number, context=context)
    def action_dial_prof_phone(self, cr, uid, ids, context=None):
        '''Function called by the button 'Dial' next to the 'prof_phone' field'''
        erp_number = self._onlydigits(self.read(cr, uid, ids, ['prof_phone'], context=context)[0]['prof_phone'])
        self.pool.get('escaux.server').dial(cr, uid, ids, erp_number, context=context)
    def action_dial_mobile(self, cr, uid, ids, context=None):
        '''Function called by the button 'Dial' next to the 'mobile' field'''
        erp_number = self._onlydigits(self.read(cr, uid, ids, ['mobile'], context=context)[0]['mobile'])
        self.pool.get('escaux.server').dial(cr, uid, ids, erp_number, context=context)

    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}

        # first, we record all changes other than changes of state
        res_id = False
        
        todo_obj = self.pool.get('crm_cci.todo')
        todo = todo_obj.browse(cr, uid, ids[0])

        # for the write, sometimes, we must give the same data because these data are mandatory to record the changes
        if todo.model == u"Marque d'intérêt":
            interest_obj = self.pool.get('res.partner.interest')
            interest = interest_obj.browse(cr,uid, todo.res_id )
            new_vals = vals.copy()
            if 'state' in vals and vals['state'] == 'closed':
                new_vals['active'] = False
            else:
                new_vals['active'] = True
            if 'product' not in new_vals and interest.product:
                new_vals['product'] = interest.product.id
            if 'category' not in new_vals and interest.category:
                new_vals['category'] = interest.category.id
            if 'turnover_hoped' not in new_vals and interest.turnover_hoped:
                new_vals['turnover_hoped'] = interest.turnover_hoped
            if 'cci_contact_follow' not in new_vals and interest.cci_contact_follow:
                new_vals['cci_contact_follow'] = interest.cci_contact_follow.id
            if 'company_phone' in new_vals:
                del new_vals['company_phone']
            if 'prof_phone' in new_vals:
                del new_vals['prof_phone']
            if 'company_email' in new_vals:
                del new_vals['company_email']
            if 'prof_email' in new_vals:
                del new_vals['prof_email']
            if 'mobile' in new_vals:
                del new_vals['mobile']
            if 'state' in new_vals:
                del new_vals['state']
            if 'action' in new_vals:
                del new_vals['action']
            result = interest_obj.write(cr,uid,[todo.res_id],new_vals,context)
        elif todo.model == u"Historique":
            histo_obj = self.pool.get('res.partner.history')
            histo = histo_obj.browse(cr,uid, todo.res_id )
            new_vals = vals.copy()
            if 'action' in vals and vals['action'] == '/':
                new_vals['action'] = 'appel_sortant'
            if 'product' not in new_vals and histo.product:
                new_vals['product'] = histo.product.id
            if 'category' not in new_vals and histo.category:
                new_vals['category'] = histo.category.id
            if 'cci_contact_follow' not in new_vals and histo.cci_contact_follow:
                new_vals['cci_contact_follow'] = histo.cci_contact_follow.id
            if 'company_phone' in new_vals:
                del new_vals['company_phone']
            if 'prof_phone' in new_vals:
                del new_vals['prof_phone']
            if 'mobile' in new_vals:
                del new_vals['mobile']
            if 'prof_email' in new_vals:
                del new_vals['prof_email']
            if 'company_email' in new_vals:
                del new_vals['company_email']
            if 'year' in new_vals:
                del new_vals['year']
            if 'turnover_hoped' in new_vals:
                del new_vals['turnover_hoped']
            if 'positive' in new_vals:
                del new_vals['positive']
            result = histo_obj.write(cr,uid,[todo.res_id],new_vals,context)
        # record the changes in the todo table
        res = super(crm_cci_todo, self).write(cr, uid, ids, vals, context)
        # perhaps changes in phone or emails to save on rights objects
        # we record this AFTER the possible change of job
        todo = todo_obj.browse(cr, uid, ids[0])
        if 'prof_phone' in vals:
            if todo.contact and todo.contact.id > 0:
                job_obj = self.pool.get('res.partner.job')
                result = job_obj.write(cr,uid,[todo.contact.id],{'phone':self._onlydigits(vals['prof_phone'])})
        if 'mobile' in vals:
            if todo.contact and todo.contact.contact_id and todo.contact.contact_id.id > 0:
                contact_obj = self.pool.get('res.partner.contact')
                result = contact_obj.write(cr,uid,[todo.contact.contact_id.id],{'mobile':self._onlydigits(vals['mobile'])})
        if 'prof_email' in vals:
            if todo.contact and todo.contact.id > 0:
                job_obj = self.pool.get('res.partner.job')
                result = job_obj.write(cr,uid,[todo.contact.id],{'email':vals['prof_email']})
        if 'company_phone' in vals:
            if todo.contact and todo.contact.address_id and todo.contact.address_id.id > 0:
                addr_obj = self.pool.get('res.partner.address')
                result = addr_obj.write(cr,uid,[todo.contact.address_id.id],{'phone':self._onlydigits(vals['company_phone'])})
        if 'company_email' in vals:
            if todo.contact and todo.contact.address_id and todo.contact.address_id.id > 0:
                addr_obj = self.pool.get('res.partner.address')
                result = addr_obj.write(cr,uid,[todo.contact.address_id.id],{'email':vals['company_email']})
        return res
crm_cci_todo()

class account_account(osv.osv):
    _inherit = 'account.account'

    _columns = {
        'cci_commission': fields.boolean('CCI Commission'),
    }
    _defaults = {
        'cci_commission': lambda *a: True,
    }
account_account()

