# -*- coding: utf-8 -*-
# 27/11/2012 : Philmer - Change in the way to calculate yearly and monthly commissions

from osv import osv
from osv import fields
from datetime import date
from tools.translate import _

class commission_slice(osv.osv):
    _name = 'cci_salesman.commission_slice'
    _columns = {
        'user_id': fields.many2one('res.users', 'User',required=True),
        'year': fields.integer('Year',required=True),
        'amount_from': fields.float('From Amount',digit=(15,2)),
        'rate100': fields.float('Rate (pourcents)',digit=(4,2),required=True),
    }
commission_slice()

class res_users(osv.osv):
    _inherit = 'res.users'
    _columns = {
        'salesman': fields.boolean('Salesman'),
        'cci_crm_follower': fields.boolean('CRM Follower',help='Select only if this person will use his/her CCI CRM Todo list'),
        'slice_ids':fields.one2many('cci_salesman.commission_slice', 'user_id','Slices'),
    }
    _defaults = {
        'salesman': lambda *a: False,
        'cci_crm_follower': lambda *a: False,
    }
res_users()

class cci_objectif(osv.osv):
    _name = 'cci.objectif'
    _columns = {
        'objective': fields.integer('Objectif'),
        'cci_product': fields.many2one('cci.product', 'Product'),
        'user' : fields.many2one('res.users', 'User'),
        'year': fields.char('Year', size=32),
        'cci_contact_id': fields.many2one('cci.contact', 'Salesman'),
    }
    
cci_objectif()

class res_commission(osv.osv):
    _name = 'res.commission'
    
    def _get_com1(self, cr, uid, ids, name, args, context=None):
        res = {}
        lines =  self.browse(cr, uid, ids, context=context)
        for line in lines:
            if not line.objectif or line.realised >= line.objectif:
                res[line.id] = 0
            else:
                res[line.id] = line.realised*0.015
        return res
    
    def _get_com2(self, cr, uid, ids, name, args, context=None):
        res = {}
        lines =  self.browse(cr, uid, ids, context=context)
        for line in lines:
            if not line.objectif or line.realised < line.objectif:
                res[line.id] = 0
            else:
                res[line.id] = line.realised*0.025
        return res
    
    _columns = {
        'product': fields.many2one('cci.product', 'Product', domain=[('commissioned', '=', True)], readonly=True),
        'realised': fields.integer('Realised', readonly=True),
        'objectif': fields.integer('Objectif'),
        'com1': fields.integer('Com (1.5pourcent)'),
        'com2': fields.integer('Com (2.5pourcent)'),
        'objective_id': fields.many2one('cci.objectif', 'Objective_id'),
    }
    
    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}
        res = {}
        if 'read' in context and context.get('read') == True:
            res = super(res_commission, self).create(cr, uid, vals, context)
        else:
            raise osv.except_osv('Action non permise!', 
                             'Vous ne pouvez pas ajouter de lignes')
        
        return res
    
    def write(self, cr, uid, ids, data, context=None):
        id =  self.browse(cr, uid, ids,context)[0]['objective_id']['id']
        self.pool.get('cci.objectif').write(cr, uid, id, {'objective':data['objectif']})
        return super(res_commission,self).write(cr,uid,ids,data,context)

res_commission()

class commission_exception(osv.osv):
    _name = 'cci_salesman.commission_exception'
    def check_only_one_value(self,cr, uid, ids):
        exceptions=self.browse(cr, uid, ids)
        for data in exceptions:
            if data.amount and data.commission:
                return False
        return True
    def check_global_comm_change(self,cr, uid, ids):
        exceptions=self.browse(cr, uid, ids)
        for data in exceptions:
            if not data.cci_product_id and data.amount:
                return False
        return True
    _columns = {
        'name': fields.char('Name',size=60,required=True),
        'period_id': fields.many2one('account.period', 'Period', domain=[('special', '=', False)],required=True),
        'user_id': fields.many2one('res.users', 'User',required=True),
        'cci_product_id': fields.many2one('cci.product', 'CCI Product'), ### not mandatory, because it can be global in case of a diminution of commission on a given period
        'amount': fields.float('Amount correction',digit=(15,2)),
        'commission': fields.float('Commission correction',digit=(15,2)),
        'comments': fields.text('Comments'),
    }
    _constraints = [
            (check_only_one_value, _("Error: You can define a change on amount or on final commission, not both."), ['amount','commission']),
            (check_global_comm_change, _("Error: You can let the 'CCI product' undefined but only for global commission change."), ['cci_product_id','amount','commission']),
    ]
commission_exception()

class commission_payment(osv.osv):
    _name = 'cci_salesman.commission_payment'
    _columns = {
        'period_id': fields.many2one('account.period', 'Period', domain=[('special', '=', False)],required=True),
        'user_id': fields.many2one('res.users', 'User',required=True),
        'amount': fields.float('Paid Amount',digit=(15,2)),
        'date_payment': fields.date('Real Date of payment'),
        'comments': fields.text('Comments'),
    }
commission_payment()

