# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import wizard
import pooler
import datetime

form = """<?xml version="1.0"?>
<form string="Correct Date on Move">
    <field name="move_id" required="1"/>
    <field name="new_date"/>
</form>"""

fields = {
    'move_id': {'string':'Move ID','type':'integer','help':'Numeric ID of the move to correct. The move and the statement OR the invoice linked will be corrected.'},
    'new_date' : {'string':'New Date','type':'date'},
   }

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="Results" colspan="4"/>
     <field name="msg" colspan="4" nolabel="1" width="400"/>
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
}

class correct_date_on_move(wizard.interface):
    def _correct_move(self, cr, uid, data, context):
        pool_obj = pooler.get_pool(cr.dbname)
        if pool_obj.get('account.move').search(cr,uid,[('id','=',data['form']['move_id'])]):
            move = pool_obj.get('account.move').browse(cr,uid,data['form']['move_id'])
            if move:
                if move.date < move.period_id.date_start or move.date > move.period_id.date_stop:
                    new_date = data['form']['new_date']
                    if not new_date:
                        if move.date < move.period_id.date_start:
                            new_date = move.period_id.date_start
                        elif move.date > move.period_id.date_stop:
                            new_date = move.period_id.date_stop
                    if new_date:
                        if new_date >= move.period_id.date_start and new_date <= move.period_id.date_stop:
                            # correct in move and move lines
                            old_date = move.date
                            line_ids = []
                            statement_ids = []
                            analytic_line_ids = []
                            for line in move.line_id:
                                if line.date == old_date:
                                    line_ids.append(line.id)
                                    if line.statement_id:
                                        if line.statement_id.id not in statement_ids:
                                            statement_ids.append(line.statement_id.id)
                                    if line.analytic_lines:
                                        for aline in line.analytic_lines:
                                            if aline.date == old_date:
                                                analytic_line_ids.append(aline.id)
                            cr.execute("UPDATE account_move SET date='%s' WHERE id=%i" % (new_date,move.id))
                            if line_ids:
                                cr.execute("UPDATE account_move_line SET date='%s' WHERE id in (%s)" % (new_date,','.join([str(x) for x in line_ids])))
                            if analytic_line_ids:
                                cr.execute("UPDATE account_analytic_line SET date='%s' WHERE id in (%s)" % (new_date,','.join([str(x) for x in analytic_line_ids])))
                            data['form']['msg'] = "Correct in move ID %i : date %s -> %s\nand also in line ids %s\nand also in analytic line ids %s" % (move.id,
                                        old_date,
                                        new_date,
                                        ','.join([str(x) for x in line_ids]),
                                        ','.join([str(x) for x in analytic_line_ids]) )
                            
                            # check if we must also correct in statement and invoice
                            if statement_ids:
                                statements = pool_obj.get('account.bank.statement').browse(cr,uid,statement_ids)
                                to_correct_statement_ids = []
                                to_correct_statement_line_ids = []
                                for statement in statements:
                                    if statement.date == old_date:
                                        to_correct_statement_ids.append(statement.id)
                                    for sline in statement.line_ids:
                                        if sline.date == old_date:
                                            to_correct_statement_line_ids.append(sline.id)
                                if to_correct_statement_ids:
                                    cr.execute("UPDATE account_bank_statement SET date='%s' WHERE id in (%s)" % (new_date,','.join([str(x) for x in to_correct_statement_ids])))
                                if to_correct_statement_line_ids:
                                    cr.execute("UPDATE account_bank_statement_line SET date='%s' WHERE id in (%s)" % (new_date,','.join([str(x) for x in to_correct_statement_line_ids])))
                                data['form']['msg'] += "\n\nAlso corrected Statement IDs %s and Statement Lines IDs %s" % (','.join([str(x) for x in to_correct_statement_ids]) or '--',
                                                                                                                           ','.join([str(x) for x in to_correct_statement_line_ids]) or '--')
                            else:
                                if move.journal_id.type in ['purchase','sale']:
                                    invoice_ids = pool_obj.get('account.invoice').search(cr,uid,[('move_id','=',move.id)])
                                    if invoice_ids:
                                        cr.execute("UPDATE account_invoice SET date_invoice='%s' WHERE id in (%s)" % (new_date,','.join([str(x) for x in invoice_ids])))
                                        data['form']['msg'] += "\n\nAlso corrected Invoice IDs %s" % ','.join([str(x) for x in invoice_ids])
                                else:
                                    data['form']['msg'] += "\n\nAnd that's all folks !"
                        else:
                            data['form']['msg'] = "The new date is ALSO outside the period. Please retry with a correct new date !"
                    else:
                        data['form']['msg'] = "Unable to determine the correct new date. Can you force it ?"
                else:
                    data['form']['msg'] = "The move date seems to be inside the period. Sure ?"
            else:
                data['form']['msg'] = "Move ID doesn\'t exists. Sure it\'s a move ID, not a line ID ?"
        else:
            data['form']['msg'] = "Move ID doesn\'t exists. Sure it\'s a move ID, not a line ID ?"
        return data['form']
    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Cancel'),('correct','Correct','',True)]},
        },
        'correct': {
            'actions': [_correct_move],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }
correct_date_on_move("correct_date_on_move")
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

