# -*- encoding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################
# Version 1.0 Philmer
import wizard
import time
import datetime
import dateutil
#import re
import tools
import pooler
import base64
from pyExcelerator import *
from pyExcelerator.Worksheet import *
from tools.translate import _

wizard_arch= """<?xml version="1.0"?>
<form string="Extract Sales on New Membership, Alter Ego and CNN">
    <field name="year"/>
    <newline/>
    <field name="period_from"/>
    <field name="period_to"/>
</form>"""

wizard_fields = {
    'year': {'string': 'Year', 'type':'integer','required':True},
    'period_from': {'string':'Period From','type':'many2one','relation':'account.period','required':True},
    'period_to': {'string':'Period To','type':'many2one','relation':'account.period'},
}

msg_form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="File has been created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="followup_yearly_products.xls" colspan="4" />
</form>"""

msg_fields = {
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True},
    'followup_yearly_products.xls':{'string': 'Prepared file', 'type': 'binary', 'readonly': True,},
}

pyexcel_colours = {
        'aqua'    : 0x0F,
        'cyan'    : 0x0F,
        'black'   : 0x08,
        'blue'    : 0x0C,
        'brown'   : 0x10,
        'magenta' : 0x0E,
        'fuchsia' : 0x0E,
        'gray'    : 0x17,
        'grey'    : 0x17,
        'green'   : 0x11,
        'lime'    : 0x0B,
        'navy'    : 0x12,
        'orange'  : 0x35,
        'pink'    : 0x21,
        'purple'  : 0x14,
        'red'     : 0x0A,
        'silver'  : 0x16,
        'white'   : 0x09,
        'yellow'  : 0x0D,
}

def get_contact_club(cr,uid,ailine): # ailine = browse object of an account invoice line
    ''' this method extract the contact_id of the participation linked to an invoice line
    '''
    sentence = """SELECT ccp.id, ccp.contact_id FROM sale_order_line_invoice_rel as solir, sale_order_line as sol, sale_order as so, cci_club_participation as ccp
                      WHERE solir.invoice_id = %i AND solir.order_line_id = sol.id AND sol.order_id = so.id AND ccp.order_id = so.id
	""" % ailine.id
    cr.execute(sentence)
    res = cr.fetchall()
    if len(res) == 1:
        result = res[0][1]
    else:
        result = False
    return result
def get_participation_id(cr,uid,ailine): # ailine = browse object of an account invoice line
    sentence = """SELECT ccp.id, ccp.contact_id FROM sale_order_line_invoice_rel as solir, sale_order_line as sol, sale_order as so, cci_club_participation as ccp
                      WHERE solir.invoice_id = %i AND solir.order_line_id = sol.id AND sol.order_id = so.id AND ccp.order_id = so.id
	""" % ailine.id
    cr.execute(sentence)
    res = cr.fetchall()
    if len(res) == 1:
        result = res[0][0]
    else:
        result = False
    return result
def french_date(date):
    return date[8:10]+'/'+date[5:7]+'/'+date[0:4]
def _get_period(alpha_period):
    return '%s/%s' % (alpha_period[6:8],alpha_period[1:5])
def _get_alpha_period(date_as_string): # added a prefix of one space to force excel to treat it as a string
    result = date_as_string[0:7]
    return ' '+result
def _french_date(date_as_string):
    return '%s/%s/%s' % (date_as_string[8:10],date_as_string[5:7],date_as_string[0:4])
def _get_last_histo(partner,valid_user_ids):
    last_date = '1980-01-01'
    last_histo = ''
    if partner.history:
        for histo in partner.history:
            if histo.cci_contact and histo.cci_contact.user and (histo.cci_contact.user.id in valid_user_ids) and histo.date > last_date:
                last_date = histo.date
                last_histo = _french_date(histo.date) + ' - ' + histo.cci_contact.name + ' - ' + (histo.action or '?') + ' - ' + (histo.description or '')
    return clean_description(last_histo)
def num_to_letter(line_number_in_pyexcel):
    # this method convert an integer to a line label in Excel
    # first line = 0 (beware on pyExcelerator, first line = 0)
    # 0 => 'A'
    # 26 => 'AA'
    # 27 => 'AB'
    first = (line_number_in_pyexcel // 26)
    last = (line_number_in_pyexcel % 26) + 1
    if first > 0:
        result = chr(64+first)+chr(64+last)
    else:
        result = chr(64+last)
    return result
def full_cell(line_number_in_pyexcel,col_number_in_pyexcel,fixed='none'):
    # fixed = none, col, line, both
    result = ''
    if fixed in ('col','both'):
        result += '$'
    result += num_to_letter(col_number_in_pyexcel)
    if fixed in ('line','both'):
        result += '$'
    result += str(line_number_in_pyexcel+1)
    return result
def clean_description(full_description): # this function remove carraige returns and cut the texte to a max mength of 200
    result = full_description.replace('\t',' ').replace('\n',' ').replace('\r',' ')
    if len(result) > 200:
        result = result[0:296] + ' ...'
    return result
class follow_yearly_products(wizard.interface):
    def _get_defaults(self,cr,uid,data,context):
        data['form']['year'] = int(datetime.date.today().strftime('%Y'))
        return data['form']
    def _get_file(self,cr,uid,data,context):
        obj_period = pooler.get_pool(cr.dbname).get('account.period')
        obj_ailine = pooler.get_pool(cr.dbname).get('account.invoice.line')
        obj_prod = pooler.get_pool(cr.dbname).get('product.product')
        obj_journal = pooler.get_pool(cr.dbname).get('account.journal')
        obj_mline = pooler.get_pool(cr.dbname).get('membership.membership_line')
        obj_participation = pooler.get_pool(cr.dbname).get('cci_club.participation')
        obj_stand = pooler.get_pool(cr.dbname).get('cci.popup.stand')
        
        sales_journal_ids = obj_journal.search(cr,uid,[('type','=','sale'),('refund_journal','=',False)])
        refund_sales_journal_ids = obj_journal.search(cr,uid,[('type','=','sale'),('refund_journal','=',True)])

        sentence = """SELECT distinct(user_id) FROM res_partner WHERE active"""
        cr.execute(sentence)
        res = cr.fetchall()
        commercial_ids = [x[0] for x in res if x[0]]
        
        from_period_id = data['form']['period_from']
        to_period_id = data['form']['period_to']
        if not to_period_id:
            to_period_id = from_period_id
        from_period = obj_period.read(cr,uid,[from_period_id,],['date_start','date_stop'])[0]
        from_date = from_period['date_start']
        if from_period_id == to_period_id:
            to_date = from_period['date_stop']
        else:
            to_period = obj_period.read(cr,uid,[to_period_id,],['date_stop'])[0]
            to_date = to_period['date_stop']
        prod_cotis_year_ids = obj_prod.search(cr,uid,[('membership_year','=',data['form']['year'])])
        prod_cotis_pyear_ids = obj_prod.search(cr,uid,[('membership_year','=',data['form']['year']-1)])
        if prod_cotis_year_ids and prod_cotis_pyear_ids:
            
            dResByPartner = {}
            ailine_ids = obj_ailine.search(cr,uid,[('product_id','in',prod_cotis_year_ids+prod_cotis_pyear_ids)])
            if ailine_ids:
                ailines = obj_ailine.browse(cr,uid,ailine_ids)
                for ailine in ailines:
                    if ailine.invoice_id and ailine.invoice_id.state not in ('draft','cancel') and ailine.invoice_id.partner_id:
                        mult = 1
                        if ailine.invoice_id.journal_id.id in refund_sales_journal_ids:
                            mult = -1
                        if dResByPartner.has_key(ailine.invoice_id.partner_id.id):
                            partner_data = dResByPartner[ailine.invoice_id.partner_id.id]
                            if ailine.product_id.id in prod_cotis_year_ids:
                                partner_data['sum_year'] += ailine.price_subtotal * mult
                                partner_data['year_invoices'] += (', '+ailine.invoice_id.number)
                                if (ailine.invoice_id.date_invoice < partner_data['date_first_invoice']) and (ailine.invoice_id.journal_id.id in sales_journal_ids):
                                    partner_data['date_first_invoice'] = ailine.invoice_id.date_invoice
                            elif ailine.product_id.id in prod_cotis_pyear_ids:
                                partner_data['sum_pyear'] += ailine.price_subtotal * mult
                                partner_data['pyear_invoices'] += (', '+ailine.invoice_id.number)
                        else:
                            partner_data = {'partner_id': ailine.invoice_id.partner_id.id,
                                            'partner_name': ailine.invoice_id.partner_id.name,
                                            'partner_membership_state': ailine.invoice_id.partner_id.membership_state,
                                            'partner_salesman': ailine.invoice_id.partner_id.user_id and ailine.invoice_id.partner_id.user_id.name or '',
                                            'year_invoices': ailine.product_id.id in prod_cotis_year_ids and ailine.invoice_id.number or '',
                                            'pyear_invoices': ailine.product_id.id in prod_cotis_pyear_ids and ailine.invoice_id.number or '',
                                            'sum_year': ailine.product_id.id in prod_cotis_year_ids and (ailine.price_subtotal * mult) or 0.0,
                                            'sum_pyear':  ailine.product_id.id in prod_cotis_pyear_ids and (ailine.price_subtotal * mult) or 0.0,
                                            'date_first_invoice':(ailine.product_id.id in prod_cotis_year_ids and ailine.invoice_id.journal_id.id in sales_journal_ids) and ailine.invoice_id.date_invoice or '2999-12-31',
                                            'last_histo': _get_last_histo(ailine.invoice_id.partner_id, commercial_ids)
                                           }
                        dResByPartner[ailine.invoice_id.partner_id.id] = partner_data
                   
        wb1 = Workbook()

        # styles
        pattern_red = Pattern()
        pattern_red.pattern = 0x01 # SOLID
        pattern_red.set_pattern_fore_colour(pyexcel_colours['red'])
        #
        pattern_green = Pattern()
        pattern_green.pattern = 0x01 # SOLID
        pattern_green.set_pattern_fore_colour(pyexcel_colours['lime'])
        #
        pattern_fuchsia = Pattern()
        pattern_fuchsia.pattern = 0x01 # SOLID
        pattern_fuchsia.set_pattern_fore_colour(pyexcel_colours['fuchsia'])
        #
        pattern_yellow = Pattern()
        pattern_yellow.pattern = 0x01 # SOLID
        pattern_yellow.set_pattern_fore_colour(pyexcel_colours['yellow'])
        #
        pattern_orange = Pattern()
        pattern_orange.pattern = 0x01 # SOLID
        pattern_orange.set_pattern_fore_colour(pyexcel_colours['orange'])
        #
        style_red = XFStyle()
        style_red.font.bold = True
        style_red.pattern = pattern_red
        #
        style_fuchsia = XFStyle()
        style_fuchsia.font.bold = True
        style_fuchsia.pattern = pattern_fuchsia
        #
        style_yellow = XFStyle()
        style_yellow.pattern = pattern_yellow
        #
        style_orange = XFStyle()
        style_orange.pattern = pattern_orange
        #
        style_green = XFStyle()
        style_green.pattern = pattern_green
        #
        #ws1 = wb1.add_sheet('New Membership')
        #ws1.write(0,0,_('New Membership between %s and %s') % (str(data['form']['year']-1),str(data['form']['year'])))
        #ws1.write(2,0,'Date : ' + datetime.datetime.now().strftime('%d/%m/%Y %H:%M:%S'))
        #ws1.write(4,0,u'Cette liste reprend toutes les ventes de cotisation %s en sa basant sur les factures et les notes de crédit.' % (str(data['form']['year'])))
        #ws1.write(5,0,u'Même les factures pas encore payées sont reprises dans cette liste.')
        #ws1.write(6,0,u'Sont reprises aussi dans cette liste, les membres pour lesquels on a une cotisation payée pour %s mais aucun montant payé en %s.' % (str(data['form']['year']),str(data['form']['year']-1)))
        #ws1.write(7,0,u'Ils ne sont cependant pas comptabilisés dans la tableau en fin de liste (colonne K).')
        #ws1.write(8,0,u'Cela reprend les cas des membres avec une facture de cotisation de 0 en %s OU de transfert de cotisation depuis un autre membre facturé pour %s.' % (str(data['form']['year']-1),str(data['form']['year']-1)))
        #ws1.write(9,0,u'Attention, le tableau récapitulatif en dessous ressemble à un tableau croisé dynamique, mais il n\'est pas du tout dynamique !')
        #ws1.write(11,0,'partner_id')
        #ws1.write(11,1,_('Partner Name'))
        #ws1.write(11,2,_('Partner Salesman'))
        #ws1.write(11,3,_('Partner Membership State'))
        #ws1.write(11,4,_('Date First Invoice'))
        #ws1.write(11,5,_('Month First Invoice'))
        #ws1.write(11,6,_('Amount %i') % data['form']['year'])
        #ws1.write(11,7,_('Invoices %i') % data['form']['year'])
        #ws1.write(11,8,_('Amount %i') % (data['form']['year']-1))
        #ws1.write(11,9,_('Invoices %i') % (data['form']['year']-1))
        #ws1.write(11,10,_('In final count'))
        #ws1.write(11,11,_('Last Histo'))
        #line1 = 12
        #partners_data = [x for x in dResByPartner.values()]
        #spartners_data = sorted(partners_data, key=lambda x: x['date_first_invoice'])
        #dTotalByMonthAndSalesman = {}
        #months = []
        #salesmen = []
        #for partner_data in spartners_data:
        #    if (partner_data['date_first_invoice'] != '2999-12-31') and (partner_data['sum_year'] >= 0.01) and (partner_data['sum_pyear'] < 0.01):
        #        # check membership lines only for theses partners
        #        past_year_membership_line = False
        #        mline_ids = obj_mline.search(cr,uid,[('partner','=',partner_data['partner_id'])])
        #        if mline_ids:
        #            mlines = obj_mline.browse(cr,uid,mline_ids)
        #            for mline in mlines:
        #                if mline.account_invoice_line and mline.account_invoice_line.product_id and mline.account_invoice_line.product_id.id in prod_cotis_pyear_ids:
        #                    past_year_membership_line = True
        #        #
        #        ws1.write(line1,0,partner_data['partner_id'])
        #        ws1.write(line1,1,partner_data['partner_name'])
        #        ws1.write(line1,2,partner_data['partner_salesman'])
        #        ws1.write(line1,3,partner_data['partner_membership_state'])
        #        ws1.write(line1,4,_french_date(partner_data['date_first_invoice']))
        #        ws1.write(line1,5,_get_alpha_period(partner_data['date_first_invoice']))
        #        ws1.write(line1,6,partner_data['sum_year'])
        #        ws1.write(line1,7,partner_data['year_invoices'])
        #        ws1.write(line1,8,partner_data['sum_pyear'])
        #        ws1.write(line1,9,partner_data['pyear_invoices'])
        #        ws1.write(line1,10,past_year_membership_line and 'NON' or '')
        #        ws1.write(line1,11,partner_data['last_histo'])
        #        line1 += 1
        #        #
        #        if not past_year_membership_line:
        #            month = _get_alpha_period(partner_data['date_first_invoice'])
        #            salesman = partner_data['partner_salesman']
        #            if month not in months:
        #                months.append(month)
        #            if salesman not in salesmen:
        #                salesmen.append(salesman)
        #            key_total = (month,salesman)
        #            if dTotalByMonthAndSalesman.has_key(key_total):
        #                subtotal = dTotalByMonthAndSalesman[key_total]
        #            else:
        #                subtotal = {'amount':0.0,
        #                            'count':0,
        #                           }
        #            subtotal['amount'] += partner_data['sum_year']
        #            subtotal['count'] += 1
        #            dTotalByMonthAndSalesman[key_total] = subtotal
        #
        #smonths = sorted(months)
        #ssalesmen = sorted(salesmen)
        #line1 += 1
        #ws1.write(line1,0,'Month')
        #col = 1
        #for salesman in ssalesmen:
        #    ws1.write(line1,col,salesman)
        #    ws1.write(line1+1,col,_('Monthly Amount'))
        #    ws1.write(line1+1,col+1,_('Monthly Count'))
        #    col += 2
        #line1 += 2
        #for month in smonths:
        #    ws1.write(line1,0,_get_period(month))
        #    col = 1
        #    for salesman in ssalesmen:
        #        if dTotalByMonthAndSalesman.has_key((month,salesman)):
        #            ws1.write(line1,col,dTotalByMonthAndSalesman[(month,salesman)]['amount'])
        #            ws1.write(line1,col+1,dTotalByMonthAndSalesman[(month,salesman)]['count'])
        #        else:
        #            ws1.write(line1,col,0.0)
        #            ws1.write(line1,col+1,0)
        #        col += 2
        #    line1 += 1

        period_ids = obj_period.search(cr,uid,[('date_start','>=',from_date),('date_stop','<=',to_date),('special','=',False)])
        others = [{'name':'CNN30+','product_codes':[u'CNN30+',],'special_user_ids':[62,]},{'name':u'CNN-30','product_codes':['AEN1NFINLG','AEN1NFINNAM','CNN-30'],'special_user_ids':[20,49,]}]
        for wsheet in others:
            key = wsheet['name']
            wso = wb1.add_sheet(key)
            wso.write(0,0,key + ' ' + french_date(from_date) + ' -> ' + french_date(to_date))
            wso.write(2,0,'Date : ' + datetime.datetime.now().strftime('%d/%m/%Y %H:%M:%S'))
            wso.write(3,0,_('Product codes : %s') % ','.join(wsheet['product_codes']))
            wso.write(5,0,'ailine_id')
            wso.write(5,1,'partner_id')
            wso.write(5,2,'participation_id')
            wso.write(5,3,_('Partner Name'))
            wso.write(5,4,_('Partner Salesman'))
            wso.write(5,5,_('Date Invoice'))
            wso.write(5,6,_('Month Invoice'))
            wso.write(5,7,_('Invoice'))
            wso.write(5,8,_('Amount %i') % data['form']['year'])
            wso.write(5,9,_('New (18 months)'),style_yellow)
            wso.write(5,10,_('Not Continuation'),style_orange)
            wso.write(5,11,_('Assignment'),style_yellow)
            wso.write(5,12,_('Details'))
            wso.write(5,13,_('Club'))
            wso.write(5,14,_('Participer'))
            wso.write(5,15,_('Last Histo'))
            lineo = 6
            start_lineo = lineo+1
            prod_ids = obj_prod.search(cr,uid,[('default_code','in',wsheet['product_codes'])])
            if prod_ids:
                ailine_ids = obj_ailine.search(cr,uid,[('product_id','in',prod_ids),('invoice_id.period_id','in',period_ids)])
                if ailine_ids:
                    dTotalByMonthAndSalesman = {}
                    months = []
                    salesmen = []
                    ailines = obj_ailine.browse(cr,uid,ailine_ids)
                    for ailine in ailines:
                        if ailine.invoice_id and ailine.invoice_id.state not in ('draft','cancel'):
                            mult = 1
                            if ailine.invoice_id.journal_id.id in refund_sales_journal_ids:
                                mult = -1
                            month = _get_alpha_period(ailine.invoice_id.date_invoice)
                            salesman = (ailine.invoice_id.partner_id and ailine.invoice_id.partner_id.user_id) and ailine.invoice_id.partner_id.user_id.name or ''
                            last_histo = _get_last_histo(ailine.invoice_id.partner_id, commercial_ids + wsheet['special_user_ids'])
                            part_id = False
                            participation = False
                            part_id = get_participation_id(cr,uid,ailine)
                            found_same = False
                            if part_id:
                                participation = obj_participation.browse(cr,uid,part_id)
                                if participation.date_in:
                                    current_date_start = datetime.datetime.strptime(participation.date_in, '%Y-%m-%d')
                                    before18months = current_date_start + dateutil.relativedelta.relativedelta(months=-18)
                                    current_contact_id = get_contact_club(cr,uid,ailine)
                                    same_contact_ids = obj_participation.search(cr,uid,[('contact_id','=',current_contact_id),('date_in','<',current_date_start.strftime('%Y-%m-%d')),('date_in','>=',before18months.strftime('%Y-%m-%d'))])
                                    if same_contact_ids:
                                        # check if one of these other participations is in the same type of club
                                        same_contact_participations = obj_participation.browse(cr,uid,same_contact_ids)
                                        for same_contact_part in same_contact_participations:
                                            if same_contact_part.group_id and same_contact_part.group_id.product_id and same_contact_part.group_id.product_id.code and same_contact_part.group_id.product_id.code in wsheet['product_codes']:
                                                found_same = True
                                                break
                                            # search in past 18 months
                                            # if same person billed for same product for CNN and AE
                                            # if same partner billed for same product for RdM
                                            #current_date = datetime.datetime.strptime(ailine.invoice_id.period_id.date_start, "%Y-%m-%d")
                                            #months18_date = current_date + dateutil.relativedelta.relativedelta(months=-18)
                                            #yesterday = current_date + dateutil.relativedelta.relativedelta(days=-1)
                                            #first_date = months18_date.strftime('%Y-%m-%d')
                                            #last_date = yesterday.strftime('%Y-%m-%d')
                                            #retro_period_ids = obj_period.search(cr,uid,[('date_start','>=',first_date),('date_stop','<=',last_date),('special','=',False)])
                                            #if current_contact_id:
                                            #    same_partner_ailine_ids = obj_ailine.search(cr,uid,[('product_id','in',prod_ids),('invoice_id.partner_id','=',ailine.invoice_id.partner_id.id),('invoice_id.period_id','in',retro_period_ids)])
                                            #    if same_partner_ailine_ids:
                                            #        for same_partner_ailine_id in same_partner_ailine_ids:
                                            #            ailine = obj_ailine.browse(cr,uid,same_partner_ailine_id)
                                            #            if get_contact_club(cr,uid,ailine) == current_contact_id:
                                            #                found_same = True
                                wso.write(lineo,13,participation.group_id.name)
                                wso.write(lineo,14,(participation.contact_id.name + ' ' + (participation.contact_id.first_name or '')).strip())
                            #
                            wso.write(lineo,0,ailine.id)
                            wso.write(lineo,1,ailine.invoice_id.partner_id and ailine.invoice_id.partner_id.id or '')
                            wso.write(lineo,2,part_id and part_id or '')
                            wso.write(lineo,3,ailine.invoice_id.partner_id and ailine.invoice_id.partner_id.name)
                            wso.write(lineo,4,salesman)
                            wso.write(lineo,5,_french_date(ailine.invoice_id.date_invoice))
                            wso.write(lineo,6,month) # add a space to treat period as a string in Excel
                            wso.write(lineo,7,ailine.invoice_id.number)
                            wso.write(lineo,8,ailine.price_subtotal * mult)
                            wso.write(lineo,9,found_same and 0 or 1,style_yellow)
                            if found_same:
                                wso.write(lineo,9,0,style_yellow)
                            else:
                                wso.write(lineo,9,1,style_yellow)
                            if part_id and participation and participation.continuation:
                                wso.write(lineo,10,0,style_orange)
                            else:
                                wso.write(lineo,10,1,style_orange)
                            wso.write(lineo,11,salesman,style_yellow)
                            wso.write(lineo,12,ailine.name or '')
                            
                            wso.write(lineo,15,last_histo)
                            lineo += 1
                            #
                            if month not in months:
                                months.append(month)
                            if salesman not in salesmen:
                                salesmen.append(salesman)
                            key_total = (month,salesman)
                            if dTotalByMonthAndSalesman.has_key(key_total):
                                subtotal = dTotalByMonthAndSalesman[key_total]
                            else:
                                subtotal = {'amount':0.0,
                                            'count':0,
                                           }
                            subtotal['amount'] += ailine.price_subtotal * mult
                            subtotal['count'] += 1
                            dTotalByMonthAndSalesman[key_total] = subtotal
                    end_lineo = lineo
                    
                    smonths = sorted(months)
                    salesmen = ['Simon Micha','Thomas Schloesser']
                    lineo += 1
                    wso.write(lineo,0,_(u'Amounts Total'))
                    wso.write(lineo,4,_(u'Commissions'))
                    wso.write(lineo,8,_(u'Transfered to Dpt'))
                    lineo += 1
                    for col in (0,4,8):
                        wso.write(lineo,col,'Month')
                        for salesman in salesmen:
                            col += 1
                            wso.write(lineo,col,salesman)
                    for cmonth in smonths:
                        lineo += 1
                        col = 0
                        wso.write(lineo,col,cmonth)
                        for salesman in salesmen:
                            col += 1
                            wso.write(lineo,col,"SOMME.SI.ENS(I%i:I%i;J%i:J%i;\"=1\";K%i:K%i;\"=1\";L%i:L%i;\"=%s\";G%i:G%i;\"=%s\")" % (start_lineo,end_lineo,start_lineo,end_lineo,start_lineo,end_lineo,start_lineo,end_lineo,salesman,start_lineo,end_lineo,cmonth) )
                        col = 4
                        wso.write(lineo,col,cmonth)
                        for salesman in salesmen:
                            col += 1
                            if salesman == u'Thomas Schloesser':
                                wso.write(lineo,col,Formula("%s*0.1" % full_cell(lineo,col-4,'both') ),style_fuchsia)
                            else:
                                wso.write(lineo,col,Formula("%s*0.1" % full_cell(lineo,col-4,'both') ))
                        col = 8
                        wso.write(lineo,col,cmonth)
                        for salesman in salesmen:
                            col += 1
                            wso.write(lineo,col,Formula("%s*0.5" % full_cell(lineo,col-8,'both') ))

        period_ids = obj_period.search(cr,uid,[('date_start','>=',from_date),('date_stop','<=',to_date),('special','=',False)])
        others = [{'name':u'RdM','product_codes':['REPMEMB','ABTPREMREP','REPCOUPLE'],'special_user_ids':[80,],'months':-18,'new_pourcent':'0.04','renew_pourcent':'0.025','couple_product_codes':['REPCOUPLE',],'couple_pourcent':'0.1','couple_renew_pourcent':'0.01'}]
        for wsheet in others:
            key = wsheet['name']
            wso = wb1.add_sheet(key)
            wso.write(0,0,key + ' ' + french_date(from_date) + ' -> ' + french_date(to_date))
            wso.write(2,0,'Date : ' + datetime.datetime.now().strftime('%d/%m/%Y %H:%M:%S'))
            wso.write(3,0,_('Product codes : %s') % ','.join(wsheet['product_codes']))
            wso.write(5,0,'ailine_id')
            wso.write(5,1,'partner_id')
            wso.write(5,2,_('Partner Name'))
            wso.write(5,3,_('Partner Salesman'))
            wso.write(5,4,_('Date Invoice'))
            wso.write(5,5,_('Month Invoice'))
            wso.write(5,6,_('Invoice'))
            wso.write(5,7,_('Amount %i') % data['form']['year'])
            wso.write(5,8,_('New (18 months)'),style_yellow)
            wso.write(5,9,_('Assignment'),style_yellow)
            wso.write(5,10,_('Cotis associated'),style_yellow)
            wso.write(5,11,_('Details'))
            wso.write(5,12,_('Last Histo'))
            lineo = 6
            start_lineo = lineo+1
            prod_ids = obj_prod.search(cr,uid,[('default_code','in',wsheet['product_codes'])])
            if prod_ids:
                ailine_ids = obj_ailine.search(cr,uid,[('product_id','in',prod_ids),('invoice_id.period_id','in',period_ids)])
                if ailine_ids:
                    dTotalByMonthAndSalesman = {}
                    months = []
                    salesmen = []
                    ailines = obj_ailine.browse(cr,uid,ailine_ids)
                    for ailine in ailines:
                        if ailine.invoice_id and ailine.invoice_id.state not in ('draft','cancel'):
                            mult = 1
                            if ailine.invoice_id.journal_id.id in refund_sales_journal_ids:
                                mult = -1
                            month = _get_alpha_period(ailine.invoice_id.date_invoice)
                            salesman = (ailine.invoice_id.partner_id and ailine.invoice_id.partner_id.user_id) and ailine.invoice_id.partner_id.user_id.name or ''
                            last_histo = _get_last_histo(ailine.invoice_id.partner_id, commercial_ids + wsheet['special_user_ids'])
                            #
                            wso.write(lineo,0,ailine.id)
                            wso.write(lineo,1,ailine.invoice_id.partner_id and ailine.invoice_id.partner_id.id or '')
                            wso.write(lineo,2,ailine.invoice_id.partner_id and ailine.invoice_id.partner_id.name)
                            wso.write(lineo,3,salesman)
                            wso.write(lineo,4,_french_date(ailine.invoice_id.date_invoice))
                            wso.write(lineo,5,month) # add a space to treat period as a string in Excel
                            wso.write(lineo,6,ailine.invoice_id.number)
                            wso.write(lineo,7,ailine.price_subtotal * mult)
                            # search in past 18 months
                            # if same person billed for same product for CNN and AE
                            # if same partner billed for same product for RdM
                            current_date = datetime.datetime.strptime(ailine.invoice_id.period_id.date_start, "%Y-%m-%d")
                            months18_date = current_date + dateutil.relativedelta.relativedelta(months=wsheet['months'])
                            yesterday = current_date + dateutil.relativedelta.relativedelta(days=-1)
                            first_date = months18_date.strftime('%Y-%m-%d')
                            last_date = yesterday.strftime('%Y-%m-%d')
                            retro_period_ids = obj_period.search(cr,uid,[('date_start','>=',first_date),('date_stop','<=',last_date),('special','=',False)])
                            same_partner_ailine_ids = obj_ailine.search(cr,uid,[('product_id','in',prod_ids),('invoice_id.partner_id','=',ailine.invoice_id.partner_id.id),('invoice_id.period_id','in',retro_period_ids)])
                            if same_partner_ailine_ids:
                                wso.write(lineo,8,0,style_yellow) # 0 = doesn't count; 1 =count
                            else:
                                wso.write(lineo,8,1,style_yellow)
                            wso.write(lineo,9,salesman,style_yellow)
                            wso.write(lineo,10,(ailine.product_id and ailine.product_id.default_code in wsheet['couple_product_codes']) and 1 or 0,style_yellow)
                            wso.write(lineo,11,ailine.name or '')
                            wso.write(lineo,12,last_histo)
                            lineo += 1
                            #
                            if month not in months:
                                months.append(month)
                            if salesman not in salesmen:
                                salesmen.append(salesman)
                            key_total = (month,salesman)
                            if dTotalByMonthAndSalesman.has_key(key_total):
                                subtotal = dTotalByMonthAndSalesman[key_total]
                            else:
                                subtotal = {'amount':0.0,
                                            'count':0,
                                           }
                            subtotal['amount'] += ailine.price_subtotal * mult
                            subtotal['count'] += 1
                            dTotalByMonthAndSalesman[key_total] = subtotal
                    end_lineo = lineo
                    
                    smonths = sorted(months)
                    salesmen = ['Simon Micha','Thomas Schloesser']
                    lineo += 1
                    wso.write(lineo,0,_(u'Amounts Renew Alone'))
                    wso.write(lineo,4,_(u'Amounts New Alone'))
                    wso.write(lineo,8,_(u'Commissions'))
                    wso.write(lineo,12,_(u'Transfered to Dpt'))
                    lineo += 1
                    for col in (0,4,8,12):
                        wso.write(lineo,col,_(u'Month'))
                        for salesman in salesmen:
                            col += 1
                            wso.write(lineo,col,salesman)
                    for cmonth in smonths:
                        lineo += 1
                        col = 0
                        wso.write(lineo,col,cmonth)
                        for salesman in salesmen:
                            col += 1
                            wso.write(lineo,col,"SOMME.SI.ENS(H%i:H%i;I%i:I%i;\"=0\";J%i:J%i;\"=%s\";F%i:F%i;\"=%s\";K%i:K%i;\"=0\")" % (start_lineo,end_lineo,start_lineo,end_lineo,start_lineo,end_lineo,salesman,start_lineo,end_lineo,cmonth,start_lineo,end_lineo) )
                        col = 4
                        wso.write(lineo,col,cmonth)
                        for salesman in salesmen:
                            col += 1
                            wso.write(lineo,col,"SOMME.SI.ENS(H%i:H%i;I%i:I%i;\"=1\";J%i:J%i;\"=%s\";F%i:F%i;\"=%s\";K%i:K%i;\"=0\")" % (start_lineo,end_lineo,start_lineo,end_lineo,start_lineo,end_lineo,salesman,start_lineo,end_lineo,cmonth,start_lineo,end_lineo))

                        col = 8
                        wso.write(lineo,col,cmonth)
                        for salesman in salesmen:
                            col += 1
                            if salesman == u'Thomas Schloesser':
                                wso.write(lineo,col,Formula("%s*%s+%s*%s" % (full_cell(lineo,col-8,'both'),wsheet['renew_pourcent'],full_cell(lineo,col-4,'both'),wsheet['new_pourcent']) ),style_fuchsia)
                            else:
                                wso.write(lineo,col,Formula("%s*%s+%s*%s" % (full_cell(lineo,col-8,'both'),wsheet['renew_pourcent'],full_cell(lineo,col-4,'both'),wsheet['new_pourcent']) ))
                        col = 12
                        wso.write(lineo,col,cmonth)
                        for salesman in salesmen:
                            col += 1
                            wso.write(lineo,col,Formula("(%s+%s)*0.5" % (full_cell(lineo,col-8,'both'),full_cell(lineo+2+len(smonths),col-8,'both')) ))
                    lineo += 1
                    wso.write(lineo,0,_(u'Amounts Renew as Membership'))
                    wso.write(lineo,4,_(u'Amounts New as Membership'))
                    wso.write(lineo,8,_(u'Commissions'))
                    lineo += 1
                    for col in (0,4,8):
                        wso.write(lineo,col,_(u'Month'))
                        for salesman in salesmen:
                            col += 1
                            wso.write(lineo,col,salesman)
                    for cmonth in smonths:
                        lineo += 1
                        col = 0
                        wso.write(lineo,col,cmonth)
                        for salesman in salesmen:
                            col += 1
                            wso.write(lineo,col,"SOMME.SI.ENS(H%i:H%i;I%i:I%i;\"=0\";J%i:J%i;\"=%s\";F%i:F%i;\"=%s\";K%i:K%i;\"=1\")" % (start_lineo,end_lineo,start_lineo,end_lineo,start_lineo,end_lineo,salesman,start_lineo,end_lineo,cmonth,start_lineo,end_lineo) )
                        col = 4
                        wso.write(lineo,col,cmonth)
                        for salesman in salesmen:
                            col += 1
                            wso.write(lineo,col,"SOMME.SI.ENS(H%i:H%i;I%i:I%i;\"=1\";J%i:J%i;\"=%s\";F%i:F%i;\"=%s\";K%i:K%i;\"=1\")" % (start_lineo,end_lineo,start_lineo,end_lineo,start_lineo,end_lineo,salesman,start_lineo,end_lineo,cmonth,start_lineo,end_lineo) )

                        col = 8
                        wso.write(lineo,col,cmonth)
                        for salesman in salesmen:
                            col += 1
                            if salesman == u'Thomas Schloesser':
                                wso.write(lineo,col,Formula("%s*%s+%s*%s" % (full_cell(lineo,col-8,'both'),wsheet['renew_pourcent'],full_cell(lineo,col-4,'both'),wsheet['new_pourcent']) ),style_fuchsia)
                            else:
                                wso.write(lineo,col,Formula("%s*%s+%s*%s" % (full_cell(lineo,col-8,'both'),wsheet['renew_pourcent'],full_cell(lineo,col-4,'both'),wsheet['new_pourcent']) ))

        period_ids = obj_period.search(cr,uid,[('date_start','>=',from_date),('date_stop','<=',to_date),('special','=',False)])
        others = [{'name':u'PopUp','product_codes':['POPUPD','POPUPW','POPUPS'],'special_user_ids':[18,],'months':-30,'new_pourcent':'0.04','renew_pourcent':'0.025'}]
        for wsheet in others:
            key = wsheet['name']
            wso = wb1.add_sheet(key)
            wso.write(0,0,key + ' ' + french_date(from_date) + ' -> ' + french_date(to_date))
            wso.write(2,0,'Date : ' + datetime.datetime.now().strftime('%d/%m/%Y %H:%M:%S'))
            wso.write(3,0,_('Product codes : %s') % ','.join(wsheet['product_codes']))
            wso.write(5,0,'ailine_id')
            wso.write(5,1,'partner_id')
            wso.write(5,2,_('Partner Name'))
            wso.write(5,3,_('Partner Salesman'))
            wso.write(5,4,_('Date Invoice'))
            wso.write(5,5,_('Month Invoice'))
            wso.write(5,6,_('Invoice'))
            wso.write(5,7,_('Amount %i') % data['form']['year'])
            wso.write(5,8,_('New (18 months)'),style_yellow)
            wso.write(5,9,_('Assignment'),style_yellow)
            wso.write(5,10,_('Stand Origin'),style_yellow)
            wso.write(5,11,_('Details'))
            wso.write(5,12,_('Last Histo'))
            lineo = 6
            start_lineo = lineo+1
            prod_ids = obj_prod.search(cr,uid,[('default_code','in',wsheet['product_codes'])])
            if prod_ids:
                ailine_ids = obj_ailine.search(cr,uid,[('product_id','in',prod_ids),('invoice_id.period_id','in',period_ids)])
                if ailine_ids:
                    dTotalByMonthAndSalesman = {}
                    months = []
                    salesmen = []
                    ailines = obj_ailine.browse(cr,uid,ailine_ids)
                    for ailine in ailines:
                        if ailine.invoice_id and ailine.invoice_id.state not in ('draft','cancel'):
                            mult = 1
                            if ailine.invoice_id.journal_id.id in refund_sales_journal_ids:
                                mult = -1
                            month = _get_alpha_period(ailine.invoice_id.date_invoice)
                            salesman = (ailine.invoice_id.partner_id and ailine.invoice_id.partner_id.user_id) and ailine.invoice_id.partner_id.user_id.name or ''
                            last_histo = _get_last_histo(ailine.invoice_id.partner_id, commercial_ids + wsheet['special_user_ids'])
                            #
                            wso.write(lineo,0,ailine.id)
                            wso.write(lineo,1,ailine.invoice_id.partner_id and ailine.invoice_id.partner_id.id or '')
                            wso.write(lineo,2,ailine.invoice_id.partner_id and ailine.invoice_id.partner_id.name)
                            wso.write(lineo,3,salesman)
                            wso.write(lineo,4,_french_date(ailine.invoice_id.date_invoice))
                            wso.write(lineo,5,month) # add a space to treat period as a string in Excel
                            wso.write(lineo,6,ailine.invoice_id.number)
                            wso.write(lineo,7,ailine.price_subtotal * mult)
                            # search in past 18 months
                            # if same person billed for same product for CNN and AE
                            # if same partner billed for same product for RdM
                            current_date = datetime.datetime.strptime(ailine.invoice_id.period_id.date_start, "%Y-%m-%d")
                            months18_date = current_date + dateutil.relativedelta.relativedelta(months=wsheet['months'])
                            yesterday = current_date + dateutil.relativedelta.relativedelta(days=-1)
                            first_date = months18_date.strftime('%Y-%m-%d')
                            last_date = yesterday.strftime('%Y-%m-%d')
                            retro_period_ids = obj_period.search(cr,uid,[('date_start','>=',first_date),('date_stop','<=',last_date),('special','=',False)])
                            same_partner_ailine_ids = obj_ailine.search(cr,uid,[('product_id','in',prod_ids),('invoice_id.partner_id','=',ailine.invoice_id.partner_id.id),('invoice_id.period_id','in',retro_period_ids)])
                            if same_partner_ailine_ids:
                                wso.write(lineo,8,0,style_yellow) # 0 = doesn't count; 1 =count
                            else:
                                wso.write(lineo,8,1,style_yellow)
                            wso.write(lineo,9,salesman,style_yellow)
                            stand_invoice_id = ailine.invoice_id and ailine.invoice_id.id or -1
                            stand_ids = obj_stand.search(cr,uid,[('invoice_id','=',stand_invoice_id)])
                            origin_stand = ''
                            if stand_ids:
                                stand = obj_stand.browse(cr,uid,stand_ids[0])
                                origin_stand = stand.commissioned or ''
                            wso.write(lineo,10,origin_stand,style_yellow)
                            wso.write(lineo,11,ailine.name or '')
                            wso.write(lineo,12,last_histo)
                            lineo += 1
                            #
                            if month not in months:
                                months.append(month)
                            if salesman not in salesmen:
                                salesmen.append(salesman)
                            key_total = (month,salesman)
                            if dTotalByMonthAndSalesman.has_key(key_total):
                                subtotal = dTotalByMonthAndSalesman[key_total]
                            else:
                                subtotal = {'amount':0.0,
                                            'count':0,
                                           }
                            subtotal['amount'] += ailine.price_subtotal * mult
                            subtotal['count'] += 1
                            dTotalByMonthAndSalesman[key_total] = subtotal
                    end_lineo = lineo
                    
                    smonths = sorted(months)
                    salesmen = ['Simon Micha','Thomas Schloesser']
                    lineo += 1
                    wso.write(lineo,0,_(u'Amounts Renew Total'))
                    wso.write(lineo,4,_(u'Amounts New Total'))
                    wso.write(lineo,8,_(u'Commissions'))
                    wso.write(lineo,12,_(u'Transfered to Dpt'))
                    lineo += 1
                    for col in (0,4,8,12):
                        wso.write(lineo,col,_(u'Month'))
                        for salesman in salesmen:
                            col += 1
                            wso.write(lineo,col,salesman)
                    for cmonth in smonths:
                        lineo += 1
                        col = 0
                        wso.write(lineo,col,cmonth)
                        for salesman in salesmen:
                            col += 1
                            wso.write(lineo,col,"SOMME.SI.ENS(H%i:H%i;I%i:I%i;\"=0\";J%i:J%i;\"=%s\";F%i:F%i;\"=%s\";K%i:K%i;\"=%s\")" % (start_lineo,end_lineo,start_lineo,end_lineo,start_lineo,end_lineo,salesman,start_lineo,end_lineo,cmonth,start_lineo,end_lineo,salesman) )
                        col = 4
                        wso.write(lineo,col,cmonth)
                        for salesman in salesmen:
                            col += 1
                            wso.write(lineo,col,"SOMME.SI.ENS(H%i:H%i;I%i:I%i;\"=1\";J%i:J%i;\"=%s\";F%i:F%i;\"=%s\";K%i:K%i;\"=%s\")" % (start_lineo,end_lineo,start_lineo,end_lineo,start_lineo,end_lineo,salesman,start_lineo,end_lineo,cmonth,start_lineo,end_lineo,salesman) )

                        col = 8
                        wso.write(lineo,col,cmonth)
                        for salesman in salesmen:
                            col += 1
                            if salesman == u'Thomas Schloesser':
                                wso.write(lineo,col,Formula("%s*%s+%s*%s" % (full_cell(lineo,col-8,'both'),wsheet['renew_pourcent'],full_cell(lineo,col-4,'both'),wsheet['new_pourcent']) ),style_fuchsia)
                            else:
                                wso.write(lineo,col,Formula("%s*%s+%s*%s" % (full_cell(lineo,col-8,'both'),wsheet['renew_pourcent'],full_cell(lineo,col-4,'both'),wsheet['new_pourcent']) ))
                        col = 12
                        wso.write(lineo,col,cmonth)
                        for salesman in salesmen:
                            col += 1
                            #wso.write(lineo,col,Formula("(%s+%s)*0.5" % (full_cell(lineo,col-12,'both'),full_cell(lineo,col-8,'both')) ))
                            wso.write(lineo,col,Formula("%s*0.5" % full_cell(lineo,col-8,'both') ))
                    #wso.write(lineo,1,'Simon Micha')
                    #wso.write(lineo
                    #col = 1
                    #for salesman in ssalesmen:
                    #    wso.write(lineo,col,salesman)
                    #    wso.write(lineo+1,col,_('Monthly Amount'))
                    #    wso.write(lineo+1,col+1,_('Monthly Count'))
                    #    col += 2
                    #lineo += 2
                    #for month in smonths:
                    #    wso.write(lineo,0,_get_period(month))
                    #    col = 1
                    #    for salesman in ssalesmen:
                    #        if dTotalByMonthAndSalesman.has_key((month,salesman)):
                    #            wso.write(lineo,col,dTotalByMonthAndSalesman[(month,salesman)]['amount'])
                    #            wso.write(lineo,col+1,dTotalByMonthAndSalesman[(month,salesman)]['count'])
                    #        else:
                    #            wso.write(lineo,col,0.0)
                    #            wso.write(lineo,col+1,0)
                    #        col += 2
                    #    lineo += 1

        # factures de cotisation
        wso = wb1.add_sheet(u'Facturation de cotisation')
        wso.write(0,0,'Membership ' + str(data['form']['year']))
        wso.write(2,0,'Date : ' + datetime.datetime.now().strftime('%d/%m/%Y %H:%M:%S'))
        wso.write(5,0,'partner_id')
        wso.write(5,1,_('Partner Name'))
        wso.write(5,2,_('Partner Salesman'))
        wso.write(5,3,_('Date Invoice'))
        wso.write(5,4,_('Month Invoice'))
        wso.write(5,5,_('Amount %i') % data['form']['year'])
        wso.write(5,6,_('Document'))
        wso.write(5,7,_('Details'))
        wso.write(5,8,_('Last Histo'))
        wso.write(5,9,_(u'Zone Membre depuis'),style_orange)
        wso.write(5,10,_(u'Nouveau membre Comptage'),style_yellow) # +1 / 0 / -1 / NON
        wso.write(5,11,_(u'Assignment'),style_yellow)
        wso.write(5,12,_(u'Année Cotisation concernée'))
        wso.write(5,13,_(u'Année cotisation précédente'))
        wso.write(5,14,_(u'Montant cotis année précédente'))
        wso.write(5,15,_(u'Comptage documents cotis année précédente'))
        wso.write(5,16,_(u'Ligne fact membership antérieure même année'))
        wso.write(5,17,_(u'Facture à zéro antérieure'))
        wso.write(5,18,_(u'Ligne adhésion année précédente'))
        lineo = 6
        start_lineo = lineo+1
        prod_cotis_year_ids = obj_prod.search(cr,uid,[('membership_year','=',data['form']['year'])])
        prod_cotis_pyear_ids = obj_prod.search(cr,uid,[('membership_year','=',data['form']['year']-1)])
        prod_cotis_nyear_ids = obj_prod.search(cr,uid,[('membership_year','=',data['form']['year']+1)])
        ailine_ids = obj_ailine.search(cr,uid,[('invoice_id.period_id','in',period_ids),('product_id','in',prod_cotis_year_ids+prod_cotis_nyear_ids)])
        # some invoices have two lines : one for the current year of membership and one another for next year of membership.
        # in this case, both must be counted as 'new customer' if the first line is considered as 'new customer'
        this_year_invoices_states = {} ### dictionnay of invoice_id with final 'status' as calculated for this line with the current year
        next_year_invoices_states = {} #### dictionnary of invoice_id with status and line in excel of the status calculed as first evaluation
        months = []
        salesmen = []
        if ailine_ids:
            ailines = obj_ailine.browse(cr,uid,ailine_ids)
            for ailine in ailines:
                if ailine.invoice_id and ailine.invoice_id.state not in ('draft','cancel'):
                    mult = 1
                    if ailine.invoice_id.journal_id.id in refund_sales_journal_ids:
                        mult = -1
                    month = _get_alpha_period(ailine.invoice_id.date_invoice)
                    salesman = (ailine.invoice_id.partner_id and ailine.invoice_id.partner_id.user_id) and ailine.invoice_id.partner_id.user_id.name or ''
                    last_histo = _get_last_histo(ailine.invoice_id.partner_id, commercial_ids + wsheet['special_user_ids'])
                    wso.write(lineo,0,ailine.invoice_id.partner_id.id)
                    wso.write(lineo,1,ailine.invoice_id.partner_id.name)
                    wso.write(lineo,2,salesman)
                    wso.write(lineo,3,_french_date(ailine.invoice_id.date_invoice))
                    wso.write(lineo,4,month)
                    wso.write(lineo,5,ailine.price_subtotal * mult)
                    wso.write(lineo,6,ailine.invoice_id.number or '')
                    wso.write(lineo,7,ailine.name or '')
                    wso.write(lineo,8,last_histo)
                    wso.write(lineo,9,ailine.invoice_id.partner_id.membership_first_year or '',style_orange)
                    wso.write(lineo,11,salesman,style_yellow)
                    wso.write(lineo,12,ailine.product_id.membership_year)
                    wso.write(lineo,13,ailine.product_id.membership_year-1)
                    if ailine.product_id.id in prod_cotis_year_ids:
                        past_ids = [x for x in prod_cotis_pyear_ids]
                        current_ids = [x for x in prod_cotis_year_ids]
                    elif ailine.product_id.id in prod_cotis_nyear_ids:
                        past_ids = [x for x in prod_cotis_year_ids]
                        current_ids = [x for x in prod_cotis_nyear_ids]
                    else:
                        past_ids = []
                        current_ids = [x for x in prod_cotis_pyear_ids]
                    count = 0
                    total = 0
                    past_year_ailine_ids = obj_ailine.search(cr,uid,[('product_id','in',past_ids),('invoice_id.partner_id','=',ailine.invoice_id.partner_id.id)])
                    invoice_zero_past_year = False
                    if past_year_ailine_ids:
                        past_year_ailines = obj_ailine.browse(cr,uid,past_year_ailine_ids)
                        for py_ailine in past_year_ailines:
                            py_mult = 1
                            if py_ailine.invoice_id.journal_id.id in refund_sales_journal_ids:
                                py_mult = -1
                            count += 1
                            total += (py_mult * py_ailine.price_subtotal)
                            if py_ailine.price_subtotal >= -0.001 and py_ailine.price_subtotal <= 0.001:
                                invoice_zero_past_year = True
                                break
                    same_year_before = False
                    same_year_before_ailine_ids = obj_ailine.search(cr,uid,[('product_id','in',current_ids),('invoice_id.journal_id','in',sales_journal_ids),('invoice_id.partner_id','=',ailine.invoice_id.partner_id.id)])
                    if same_year_before_ailine_ids:
                        same_year_before_ailines = obj_ailine.browse(cr,uid,same_year_before_ailine_ids)
                        for syb_ailine in same_year_before_ailines:
                            if syb_ailine.invoice_id.date_invoice < ailine.invoice_id.date_invoice:
                                same_year_before = True
                                break
                    #
                    past_year_membership_line = False
                    special_pymline = False
                    mline_ids = obj_mline.search(cr,uid,[('partner','=',ailine.invoice_id.partner_id.id)])
                    if mline_ids:
                        mlines = obj_mline.browse(cr,uid,mline_ids)
                        for mline in mlines:
                            if mline.state != 'cancel' and mline.account_invoice_line:
                                if mline.account_invoice_line.product_id and mline.account_invoice_line.product_id.id in past_ids:
                                    past_year_membership_line = True
                                    if mline.account_invoice_line.invoice_id and mline.account_invoice_line.invoice_id.partner_id and mline.account_invoice_line.invoice_id.partner_id.id != mline.partner.id:
                                        special_pymline = True
                    wso.write(lineo,14,total)
                    wso.write(lineo,15,count)
                    wso.write(lineo,16,same_year_before and 'OUI' or '')
                    wso.write(lineo,17,invoice_zero_past_year and 'OUI' or '')
                    if special_pymline:
                        wso.write(lineo,18,past_year_membership_line and 'OUI' or '',style_red)
                    else:
                        wso.write(lineo,18,past_year_membership_line and 'OUI' or '')
                    # determination if this line must be considered as 'new member'
                    # +1 / 0 / -1 / NON = comptage en terme de nouveau membre
                    # for lines valid for the next year of membership, we determine the state as usual but it can be changed later
                    # if the invoice has an another line of membership for the current year
                    if (ailine.product_id.id in prod_cotis_nyear_ids) and (ailine.invoice_id.id in this_year_invoices_states.keys()):
                        # we take the state of the line of the current year from the same invoice, because the state is already calculted
                        current_state = this_year_invoices_states[ailine.invoice_id.id]
                        if current_state == 0:
                            wso.write(lineo,10,current_state)
                        elif current_state == 1:
                            wso.write(lineo,10,current_state,style_green)
                        elif current_state == -1:
                            wso.write(lineo,10,1,style_red)
                    else:
                        final_status = 0
                        if mult == -1:
                            if past_year_membership_line or total > 0.001 or ((total >= -0.001 and total <= 0.001) and invoice_zero_past_year):
                                wso.write(lineo,10,0)
                                final_status = 0
                            else:
                                wso.write(lineo,10,1,style_red)
                                final_status = -1
                        elif past_year_membership_line:
                            wso.write(lineo,10,0)
                            final_status = 0
                        elif total > 0.001:
                            wso.write(lineo,10,0)
                            final_status = 0
                        elif (total >= -0.001 and total <= 0.001) and invoice_zero_past_year:
                            wso.write(lineo,10,0)
                            final_status = 0
                        else:
                            if same_year_before:
                                wso.write(lineo,10,0)
                                final_status = 0
                            else:
                                wso.write(lineo,10,1,style_green)
                                final_status = 1
                        if ailine.product_id.id in prod_cotis_year_ids:
                            this_year_invoices_states[ailine.invoice_id.id] = final_status
                            # perhaps some lines of next year membership already injected in excel; and to be corrected from this line
                            if ailine.invoice_id.id in next_year_invoices_states.keys():
                                old_status_line = next_year_invoices_states[ailine.invoice_id.id]
                                if old_status_line['status'] != final_status:
                                    if final_status == 0:
                                        wso.write(old_status_line['line'],10,final_status)
                                    elif final_status == 1:
                                        wso.write(old_status_line['line'],10,final_status,style_green)
                                    elif final_status == -1:
                                        wso.write(old_status_line['line'],10,1,style_red)
                        elif ailine.product_id.id in prod_cotis_nyear_ids:
                            next_year_invoices_states[ailine.invoice_id.id] = {'line':lineo,'status':final_status}
                    if month not in months:
                        months.append(month)
                    if salesman not in salesmen:
                        salesmen.append(salesman)
                    lineo += 1
                    end_lineo = lineo

            smonths = sorted(months)
            salesmen = ['Simon Micha','Thomas Schloesser']
            lineo += 1
            wso.write(lineo,0,_(u'Amounts Renew Total'))
            wso.write(lineo,4,_(u'Amounts New Total'))
            wso.write(lineo,8,_(u'Commissions'))
            lineo += 1
            for col in (0,4,8):
                wso.write(lineo,col,_(u'Month'))
                for salesman in salesmen:
                    col += 1
                    wso.write(lineo,col,salesman)
            for cmonth in smonths:
                lineo += 1
                col = 0
                wso.write(lineo,col,cmonth)
                for salesman in salesmen:
                    col += 1
                    wso.write(lineo,col,"SOMME.SI.ENS(F%i:F%i;K%i:K%i;\"=0\";L%i:L%i;\"=%s\";E%i:E%i;\"=%s\")" % (start_lineo,end_lineo,start_lineo,end_lineo,start_lineo,end_lineo,salesman,start_lineo,end_lineo,cmonth) )
                col = 4
                wso.write(lineo,col,cmonth)
                for salesman in salesmen:
                    col += 1
                    wso.write(lineo,col,"SOMME.SI.ENS(F%i:F%i;K%i:K%i;\"=1\";L%i:L%i;\"=%s\";E%i:E%i;\"=%s\")" % (start_lineo,end_lineo,start_lineo,end_lineo,start_lineo,end_lineo,salesman,start_lineo,end_lineo,cmonth) )

                col = 8
                wso.write(lineo,col,cmonth)
                for salesman in salesmen:
                    col += 1
                    if salesman == u'Thomas Schloesser':
                        wso.write(lineo,col,Formula("%s*0.01+%s*0.1" % (full_cell(lineo,col-8,'both'),full_cell(lineo,col-4,'both')) ),style_fuchsia)
                    else:
                        wso.write(lineo,col,Formula("%s*0.01+%s*0.1" % (full_cell(lineo,col-8,'both'),full_cell(lineo,col-4,'both')) ))

        wb1.save('followup_yearly_products.xls')
        result_file = open('followup_yearly_products.xls','rb').read()

        # give the result to the user
        data['form']['msg']='Save the File with '".xls"' extension.'
        data['form']['followup_yearly_products.xls']=base64.encodestring(result_file)
        return data['form']

    states = {
        'init' : {
            'actions' : [_get_defaults],
            'result' : {'type':'form','arch':wizard_arch,'fields':wizard_fields,'state':[('end', 'Cancel'),('ok', 'Confirm') ]}
        },
        'ok' : {
            'actions': [_get_file],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]}
        },
    }

follow_yearly_products('follow_yearly_products')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

