# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2008 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
# FIXED : PrtID ->PrvtID
import pooler
import wizard
import base64
from osv import osv, orm
import datetime
from tools.translate import _
from pyExcelerator import *

form = """<?xml version="1.0"?>
<form string="Export Invoices to SEPA format">
   <field name="requested_collec_date"/>µ
   <field name="charge_bearer"/>
   <field name="last_invoice_date"/>
</form>"""

fields = {'requested_collec_date' : {'type':'date',
                                   'string':'Requested Collection Date',
                                   'help':'This is the date on which you would like the collection to be made by the bank. Please keep in mind that there are minimum delays for SEPA direct debits that depend on the type of mandate and the type of sequence.'
                                    },
          'charge_bearer':{'type':'selection',
                           'selection':[
                                ('SLEV', 'Following Service Level'),
                                ('SHAR', 'Shared'),
                                ('CRED', 'Borne by Creditor'),
                                ('DEBT', 'Borne by Debtor'),
                                ],
                           'string':'Charge Bearer', 
                           'required':True,
                           'default':'SLEV',
                           'help':'Following service level : transaction charges are to be applied following the rules agreed in the service level and/or scheme (SEPA Core messages must use this). Shared : transaction charges on the creditor side are to be borne by the creditor, transaction charges on the debtor side are to be borne by the debtor. Borne by creditor : all transaction charges are to be borne by the creditor. Borne by debtor : all transaction charges are to be borne by the debtor.'
                          },
          'state':{'type':'char','size':25,'default':'draft'},
          'last_invoice_date':{'type':'date','string':'Last Invoice Date','help':'Date of the last invoice to incorporate into theses export files','required':True},
}

save_form = """<?xml version="1.0"?>
<form string="Save File...">
   <field name="sepa_file.xml" colspan="4"/>
   <field name="sepa_file.xls" colspan="4"/>
   <separator string="Note" colspan="4" />
   <field name="note" colspan="4" nolabel="1" readonly="1"/>
   </form>"""

save_fields = {
    'sepa_file.xml' : {
        'string':'Export File', 
        'type':'binary', 
        'required': False, 
        'readonly':True, 
    }, 
    'sepa_file.xls' : {'string':'Excel File','type':'binary','required': False,'readonly':True,}, 
    'note' : {'string':'Log', 'type':'text'}, 
}

def _prepare_sepa_value(field_value, max_size=0):
    value = field_value
    value = value.replace('&','AND').replace(';',',')
    valid_caracters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789/-?:().,'+ "
    final_value = ''
    for carac in value:
        if carac in valid_caracters:
            final_value += carac
    if max_size and len(final_value) > max_size:
        final_value = final_value[0:max_size]
    return final_value

def _log_create(self, cr, uid, data, context):
    return {}

def _check(self, cr, uid, data, context):
    if data['form']['state'] == 'failed':
        return 'failed'
    return 'close'

def _create_file(self, cr, uid, data, context):
    user_note = 'XML File created'
    
    pool = pooler.get_pool(cr.dbname)
    #partner_obj = pool.get('res.partner')
    #bank_obj = pool.get('res.partner.bank')
    invoice_obj = pool.get('account.invoice')
    mandate_obj = pool.get('cci.sdd.mandate')
    obj_company = pooler.get_pool(cr.dbname).get('res.users').browse(cr, uid, uid).company_id
    
    #Header Record Start
    pain_flavor = 'pain.008.001.02'
    bic_xml_tag = 'BIC'
    name_maxsize = 70
    root_xml_tag = 'CstmrDrctDbtInitn'
    if data['form']['requested_collec_date']:
        my_requested_collec_date = data['form']['requested_collec_date']
    else:
        my_requested_collec_date = datetime.datetime.today().strftime('%Y-%m-%d')
    
    header_xml = """<?xml version="1.0" encoding="utf-8"?>
<Document xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns="urn:iso:std:iso:20022:tech:xsd:%s">
<CstmrDrctDbtInitn>
""" % pain_flavor

    my_company_name = _prepare_sepa_value(obj_company.name, name_maxsize )
    my_company_id = obj_company.partner_id.vat[2:]
    
    # A. Group header
    group_header_xml = """
\t<GrpHdr>
\t\t<MsgId>%s</MsgId>
\t\t<CreDtTm>%s</CreDtTm>
\t\t<NbOfTxs>%s</NbOfTxs>
\t\t<InitgPty>
\t\t\t<Nm>%s</Nm>
\t\t\t<Id>
\t\t\t\t<OrgId>
\t\t\t\t\t<Othr>
\t\t\t\t\t\t<Id>%s</Id>
\t\t\t\t\t\t<Issr>KBO-BCE</Issr>
\t\t\t\t\t</Othr>
\t\t\t\t</OrgId>
\t\t\t</Id>
\t\t</InitgPty>
\t</GrpHdr>"""

    # Transactions
    count_transactions = 0
    sum_total = 0.00
    
    # Group all invoices by type of mandates
    cr.execute("select id from account_invoice where domiciled=True and domiciled_send_date is null and type in ('out_invoice') and state ='open' and date_invoice <= '%s'" % data['form']['last_invoice_date'])
    inv_ids = map(lambda x:x[0], cr.fetchall())
    if not inv_ids:
        return {'note': _('There is no no invoices to export'), 'sepa_file.xml': False, 'sepa_file.xls': False, 'state':'failed' }
   
    ExcelFileName = 'sepa_file.xls'
    wb = Workbook()
    ws1 = wb.add_sheet(u'Domiciliations')

    collected_invoice_ids = []
    expired_sended_ids = []
    first_sended_ids = []
    sended_ids = []
    lines_per_seq_type = {}
    ws1.write(0,0,'Partner')
    ws1.write(0,1,'Invoice')
    ws1.write(0,2,'Amount')
    ws1.write(0,3,'Mandate')
    ws1.write(0,4,'Type')
    ws1.write(0,5,'LocalInstrument')
    excel_line = 1
    for inv in invoice_obj.browse(cr, uid, inv_ids):
        partner_id = inv.partner_id.id
        mandate_ids = mandate_obj.search(cr,uid,[('partner_id','=',partner_id),('state','=','valid')])
        if not mandate_ids:
            raise orm.except_orm(
                _('Error:'),
                _("Missing valid SEPA Direct Debit mandate with partner '%s' and Invoice ref '%s'.")
                % (inv.partner_id.name,
                    inv.number))
        mandate = mandate_obj.browse(cr,uid,mandate_ids[0])
        if mandate.type == 'oneoff':
            if not mandate.last_debit_date:
                line = {'amount':inv.residual or 0.0,'mandate':mandate,'inv_name':inv.number}
                collected_invoice_ids.append(inv.id)
                expired_sended_ids.append(mandate.id)
                sended_ids.append(mandate.id)
                if lines_per_seq_type.get('OOFF'):
                    lines_per_seq_type['OOFF'].append(line)
                else:
                    lines_per_seq_type['OOFF'] = [line]
            else:
                raise orm.except_orm(
                    _('Error:'),
                    _("The mandate with reference '%s' for partner '%s' has type set to 'One-Off' and it has a last debit date set to '%s', so we can't use it.")
                    % (mandate.unique_mandate_reference,
                       mandate.partner_id.name,
                       mandate.last_debit_date))
        elif mandate.type == 'recurrent':
            seq_type_map = {
                'recurring': 'RCUR',
                'first': 'FRST',
                'final': 'FNAL',
                }
            seq_type_label = mandate.recurrent_sequence_type
            assert seq_type_label is not False
            seq_type = seq_type_map[seq_type_label]
            line = {'amount':inv.residual or 0.0,'mandate':mandate,'inv_name':inv.number}
            collected_invoice_ids.append(inv.id)
            if mandate.recurrent_sequence_type == 'first' or not mandate.sepa_migrated:
                first_sended_ids.append(mandate.id)
            elif mandate.recurrent_sequence_type == 'final':
                expired_sended_ids.append(mandate.id)
            sended_ids.append(mandate.id)
            if lines_per_seq_type.get(seq_type):
                lines_per_seq_type[seq_type].append(line)
            else:
                lines_per_seq_type[seq_type] = [line]

    # Treat each sequence_type as a separate Payment Info 
    payinfos = ''
    paynum = 1
    message_unique_id = pool.get('ir.sequence').get(cr, uid, 'sdd.order.reference')
    
    for sequence_type,lines in lines_per_seq_type.items():
        payinfo_xml = """
\t<PmtInf>
\t\t<PmtInfId>%s</PmtInfId>
\t\t<PmtMtd>DD</PmtMtd>
\t\t<BtchBookg>%s</BtchBookg>
\t\t<NbOfTxs>%s</NbOfTxs>
\t\t<PmtTpInf>
\t\t\t<SvcLvl><Cd>SEPA</Cd></SvcLvl>
\t\t\t<LclInstrm><Cd>CORE</Cd></LclInstrm>
\t\t\t<SeqTp>%s</SeqTp>
\t\t</PmtTpInf>
\t\t<ReqdColltnDt>%s</ReqdColltnDt>
\t\t<Cdtr>
\t\t\t<Nm>%s</Nm>
\t\t</Cdtr>
\t\t<CdtrAcct>
\t\t\t<Id>
\t\t\t\t<IBAN>%s</IBAN>
\t\t\t</Id>
\t\t</CdtrAcct>
\t\t<CdtrAgt>
\t\t\t<FinInstnId>
\t\t\t\t<BIC>%s</BIC>
\t\t\t</FinInstnId>
\t\t</CdtrAgt>
\t\t<ChrgBr>%s</ChrgBr>
\t\t<CdtrSchmeId>
\t\t\t<Id>
\t\t\t\t<PrvtId>
\t\t\t\t\t<Othr>
\t\t\t\t\t\t<Id>%s</Id>
\t\t\t\t\t\t\t<SchmeNm>
\t\t\t\t\t\t\t\t<Prtry>SEPA</Prtry>
\t\t\t\t\t\t\t</SchmeNm>
\t\t\t\t\t</Othr>
\t\t\t\t</PrvtId>
\t\t\t</Id>
\t\t</CdtrSchmeId>
\t\t%s
\t</PmtInf>"""
        count_lines = 0
        total_lines = 0.0
        lines_xml = ''
        for line in lines:
            line_xml = """
\t\t<DrctDbtTxInf>
\t\t\t<PmtId>
\t\t\t\t<EndToEndId>%s</EndToEndId>
\t\t\t</PmtId>
\t\t\t<InstdAmt Ccy="EUR">%s</InstdAmt>
\t\t\t<DrctDbtTx>
\t\t\t\t<MndtRltdInf>
\t\t\t\t\t<MndtId>%s</MndtId>
\t\t\t\t\t<DtOfSgntr>%s</DtOfSgntr>
%s
\t\t\t\t</MndtRltdInf>
\t\t\t</DrctDbtTx>
\t\t\t<DbtrAgt>
\t\t\t\t<FinInstnId>
\t\t\t\t\t<BIC>%s</BIC>
\t\t\t\t</FinInstnId>
\t\t\t</DbtrAgt>
\t\t\t<Dbtr>
\t\t\t\t<Nm>%s</Nm>
\t\t\t</Dbtr>
\t\t\t<DbtrAcct>
\t\t\t\t<Id>
\t\t\t\t\t<IBAN>%s</IBAN>
\t\t\t\t</Id>
\t\t\t</DbtrAcct>
\t\t\t<RmtInf>
\t\t\t\t<Ustrd>%s</Ustrd>
\t\t\t</RmtInf>
\t\t</DrctDbtTxInf>
"""
            count_lines += 1
            total_lines += line['amount']
            if not line['mandate'].sepa_migrated:
                line_dom80_migration = """
\t\t\t\t\t<AmdmntInd>true</AmdmntInd>
\t\t\t\t\t<AmdmntInfDtls>
\t\t\t\t\t\t<OrgnlMndtId>%s</OrgnlMndtId>
\t\t\t\t\t\t<OrgnlCdtrSchmeId>
\t\t\t\t\t\t\t<Id>
\t\t\t\t\t\t\t\t<PrvtId>
\t\t\t\t\t\t\t\t\t<Othr>
\t\t\t\t\t\t\t\t\t\t<Id>%s</Id>
\t\t\t\t\t\t\t\t\t\t<SchmeNm>
\t\t\t\t\t\t\t\t\t\t\t<Prtry>SEPA</Prtry>
\t\t\t\t\t\t\t\t\t\t</SchmeNm>
\t\t\t\t\t\t\t\t\t</Othr>
\t\t\t\t\t\t\t\t</PrvtId>
\t\t\t\t\t\t\t</Id>
\t\t\t\t\t\t</OrgnlCdtrSchmeId>
\t\t\t\t\t</AmdmntInfDtls>
""" % (line['mandate'].original_mandate_identification,obj_company.original_creditor_identifier)
            else:
                line_dom80_migration = ''
            lines_xml += line_xml % (line['inv_name'],
                                     '%.2f' % line['amount'],
                                     line['mandate'].unique_mandate_reference,
                                     line['mandate'].signature_date,
                                     line_dom80_migration,
                                     line['mandate'].partner_bank_id.bank.bic,
                                     _prepare_sepa_value(line['mandate'].partner_id.name),
                                     line['mandate'].partner_bank_id.iban.upper(),
                                     line['inv_name']
                                    )
            # writing line in excel file
            ws1.write(excel_line,0,_prepare_sepa_value(line['mandate'].partner_id.name))
            ws1.write(excel_line,1,line['inv_name'])
            ws1.write(excel_line,2,'%.2f' % line['amount'])
            ws1.write(excel_line,3,line['mandate'].unique_mandate_reference)
            ws1.write(excel_line,4,sequence_type)
            ws1.write(excel_line,5,line['mandate'].local_instrument)
            excel_line += 1
            
        count_transactions += count_lines
        sum_total += total_lines
        payinfos += payinfo_xml % (message_unique_id + str(paynum).rjust(3,'0'),
                                   'true',
                                   str(count_lines),
                                   sequence_type,
                                   my_requested_collec_date,
                                   my_company_name,
                                   obj_company.sepa_iban,
                                   obj_company.sepa_bic,
                                   data['form']['charge_bearer'],
                                   obj_company.sepa_creditor_identifier,
                                   lines_xml,
                                  )
        paynum += 1
        
    # End A. Generating Group Header
    group_header_xml = group_header_xml % (message_unique_id,
                                           datetime.datetime.today().strftime('%Y-%m-%dT%H:%M:%S'),
                                           str( count_transactions ),
                                           my_company_name,
                                           my_company_id )
    trailer_xml = """
</CstmrDrctDbtInitn>
</Document>"""
    
    # File creation End
    sepa_data = header_xml + group_header_xml + payinfos + trailer_xml
    wb.save(ExcelFileName)

    pool.get('account.invoice').write(cr, uid, collected_invoice_ids,{'domiciled_send_date': datetime.datetime.today().strftime('%Y-%m-%d')},context=context)
    pool.get('cci.sdd.mandate').write(cr, uid, sended_ids,{'last_debit_date': datetime.datetime.today().strftime('%Y-%m-%d')},context=context)
    pool.get('cci.sdd.mandate').write(cr, uid, expired_sended_ids, {'state': 'expired'}, context=context)
    pool.get('cci.sdd.mandate').write(cr, uid, first_sended_ids, {'recurrent_sequence_type': 'recurring','sepa_migrated': True}, context=context)
    return {'note':user_note, 'sepa_file.xml': base64.encodestring(sepa_data),'sepa_file.xls':base64.encodestring(open(ExcelFileName,'rb').read()),'state':'created'}


def float2str(lst):
    return str(lst).rjust(16).replace('.', '')

def _log_create(self, cr, uid, data, context):
    return {}

def _check(self, cr, uid, data, context):
    return 'close'

class wizard_create_sepa_sdd(wizard.interface):
    
    states = {
        'init' : {
            'actions' : [],  
            'result' : {'type' : 'form', 
                        'arch' : form, 
                        'fields' : fields, 
                        'state' : [('end', 'Cancel'), ('export', 'Export') ]}
        }, 
        'export' : {
            'actions' : [_create_file], 
            'result' : {'type' : 'form', 
                        'arch' : save_form, 
                        'fields' : save_fields, 
                        'state' : [('next_state_check', 'Ok', 'gtk-ok') ]}
        },
       'next_state_check': {
            'actions': [_log_create],
            'result' : {'type': 'choice', 'next_state': _check }
        },
      'failed': {
                'actions': [],
                'result': {'type': 'state', 'state':'end'}
            },

      'close': {
                'actions': [],
                'result': {'type': 'state', 'state':'end'}
            }
      #'report': {
      #      'actions': [],
      #      'result': {'type':'print', 'report':'invoice.domiciliation.dom', 'state':'end'}
      #  }
    }
wizard_create_sepa_sdd('invoice.export.sepa.direct.debit')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:


