# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008 CCI Connect ASBL. (http://www.cciconnect.be) All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import datetime
import csv
from osv import fields, osv
from tools.translate import _
import codecs
import pymysql
          
def _check_email(email=''):
    result = ''
    if email:
        result = email.strip().lower().replace(' ','')
        if result.count('@') == 1:
            (part1,part2) = result.split('@')
            if part1 and part2 and len(part2) > 0 and len(part1) > 1:
                last_pos = part2.rfind('.')
                if last_pos > 0:
                    part3 = part2[last_pos+1:]
                    part2 = part2[:last_pos]
                    if len(part2)> 1: # forget the control of the length of the domain : too many exceptions
                        pass
                    else:
                        result = ''
                else:
                    result = ''
            else:
                result = ''
        else:
            result = ''
    return result
def _is_mail_personal_domain(email):
    #TODO in parameters or table
    parts = email.split('@')
    if len(parts) == 2:
        #print 'domain'
        #print parts[1]
        if parts[1].lower() in ['gmail.com','skynet.be','hotmail.com','scarlet.be','me.com','msn.com','outlook.com','yahoo.fr']:
            return True
        else:
            return False
    else:
        return False
def _convert_phone(string,PHONE_COUNTRY_PREFIX):
    def only_digits(string):
        cleaned = ''
        for carac in string:
            if carac in '0123456789':
                cleaned += carac
        return cleaned
    result = ''
    if not '(conf)' in string:
        string = only_digits(string)
        if len(string) > 0:
            if len(string) == 9:
                if string[0:2] in ['02','03','04','09']:
                    result = string[0:2] + "/" + string[2:5] + "." + string[5:7] + "." + string[7:]
                else:
                    result = string[0:3] + "/" + string[3:5] + "." + string[5:7] + "." + string[7:]
            elif len(string) == 10:
                result = string[0:4] + "/" + string[4:6] + "." + string[6:8] + "." + string[8:]
            else:
                # international number
                if string[0:2] == '00':
                    # search after a country with this prefix
                    prefix = string[2:4]
                    if prefix not in PHONE_COUNTRY_PREFIX:
                        prefix = string[2:5]
                        if prefix not in PHONE_COUNTRY_PREFIX:
                            prefix = string[2:6]
                            if prefix not in PHONE_COUNTRY_PREFIX:
                                prefix = ''
                    if prefix:
                        result = '+' + string[2:2+len(prefix)] + ' ' + string[2+len(prefix):4+len(prefix)]
                        rest = string[4+len(prefix):]
                        while len(rest) > 3:
                            result += '.' + rest[0:2]
                            rest = rest[2:]
                        result += '.' + rest
                    else:
                        result = 'International:'+string
    return result
def _convert_vat(vat):
    if vat and vat[0:2] == 'BE':
        result = vat[0:2]+'-'+vat[2:6]+'.'+vat[6:9]+'.'+vat[9:]
    else:
        result = vat
    return result
def _get_date_from_string(sdate):
    result = False
    if sdate[4] == '-' and sdate[7] ==  '-' and len(sdate) >= 10:
        result = datetime.date(int(sdate[0:4]),int(sdate[5:7]),int(sdate[8:10]))
    return result 
def _upper_accent(string):
    result = string.replace(u'à','A').replace(u'â','A').replace(u'ä','A').replace(u'ç','C').replace(u'é','E').replace(u'è','E').replace(u'ê','E').replace(u'ë','E')
    result = result.replace(u'î','I').replace(u'ï','I').replace(u'ô','O').replace(u'ö','O').replace(u'ù','U').replace(u'û','U').replace(u'ü','U')
    result = result.replace(u'À','A').replace(u'Â','A').replace(u'Ä','A').replace(u'É','E').replace(u'È','E').replace(u'Ê','E').replace(u'Ë','E')
    result = result.replace(u'Î','I').replace(u'Ï','I').replace(u'Ô','O').replace(u'Ö','O').replace(u'Ù','U').replace(u'Û','U').replace(u'Ü','U')
    result = result.upper()
    return result
def _title_accent(string):
    if string:
        index = 0
        first_letter = True
        result = u''
        while index < len(string):
            carac = string[index]
            if first_letter:
                carac = _upper_accent(carac)
            else:
                carac = carac.lower()
            first_letter = False
            if carac in ' -,.':
                first_letter = True
            result += carac
            index += 1
        if result.find(u' De ') > -1:
            result = result.replace(u' De ',u' de ')
        if result.find(u' DE ') > -1:
            result = result.replace(u' DE ',u' de ')
        if result.find(u' Du ') > -1:
            result = result.replace(u' Du ',u' du ')
        if result.find(u' DU ') > -1:
            result = result.replace(u' DU ',u' du ')
        if result.find(u' Des ') > -1:
            result = result.replace(u' Des ',u' des ')
        if result.find(u' DES ') > -1:
            result = result.replace(u' DES ',u' des ')
        if result.find(u' DeS ') > -1:
            result = result.replace(u' DeS ',u' des ')
        if result.find(u' DEs ') > -1:
            result = result.replace(u' DEs ',u' des ')
        if result.find(u' D\'') > -1:
            result = result.replace(u' D\'',u' d\'')
        if result.find(u' La ') > -1:
            result = result.replace(u' La ',u' la ')
        if result.find(u' LA ') > -1:
            result = result.replace(u' LA ',u' la ')
    else:
        result = ''
    return result
class cci_sync_mobile(osv.osv):
    _name = 'cci_sync_mobile'
    _description = "synchronisation from OpenERP to Mobile Application"
    _columns = {
        'datetime': fields.datetime('Datetime'),
        'type': fields.selection([('full','Full CSV'),('incremential','Just Modifications')],'Type'),
        'stats': fields.text('Stats'),
        'errors': fields.boolean('Errors'),
    }
    _defaults = {
        'datetime': lambda *a: datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
        'errors': lambda *a: True,
    }
    def convert_old_sector_ids_to_new_ids(self,cr,uid,dSectCode2ID):
        dResult = {}
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['ActivityCodeRootID'])
        if param_values.has_key('ActivityCodeRootID'):
            activityCodeRootID = param_values['ActivityCodeRootID']
            obj_categ = self.pool.get('res.partner.category')
            old_len = 0
            categ_ids = [ activityCodeRootID ]
            while len(categ_ids) > old_len:
                new_ids = categ_ids[old_len:] # ids of categories added last time
                old_len = len(categ_ids) # the new frontier ...
                new_categs = obj_categ.read(cr,uid,new_ids,['child_ids'])
                for categ in new_categs:
                    if categ['child_ids']:
                        categ_ids.extend(categ['child_ids'])
            categs = obj_categ.read(cr,uid,categ_ids,['name'],{'lang':'fr_FR'})
            dCategs = {}
            for categ in categs:
                formated_name = categ['name']
                posA = formated_name.rfind('[')
                posB = formated_name.rfind(']')
                if posA >= 0 and posB > 0 and posA < posB:
                    code = formated_name[posA+1:posB]
                    if dSectCode2ID.has_key(code):
                        dResult[categ['id']] = dSectCode2ID[code]
        return (dResult,categ_ids)
    def _get_phone_country_prefix(self,cr,uid):
        result = []
        obj_country = self.pool.get('cci.country')
        country_ids = obj_country.search(cr,uid,[('phoneprefix','!=',False),('phoneprefix','!=',0)])
        if country_ids:
            countries = obj_country.read(cr,uid,country_ids,['phoneprefix'])
            result = [str(x['phoneprefix']) for x in countries]
        return result
    def get_activity_sectors(self,partner,categ_ids,dOldSect2New):
        secteurs = [0,0,0]
        current_secteur = 0
        if partner.category_id:
            for categ in partner.category_id: ## category_id is, not like his name define, an array of category ids
                if categ.id in categ_ids and categ.id not in secteurs and dOldSect2New.has_key(categ.id):
                    secteurs[current_secteur] = dOldSect2New[categ.id]
                    current_secteur += 1
                    if current_secteur > 2:
                        break
        return (secteurs[0],secteurs[1],secteurs[2])
    def _export_full_csv(self,cr,uid,context={}):
        errors = []
        count_partners = 0
        count_addresses = 0
        count_contacts = 0
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['CompanyLinkedInURL','ContactLinkedInURL','CompanyFacebookURL','ProductFullPageIDs'])
        if param_values.has_key('CompanyLinkedInURL') and param_values.has_key('ContactLinkedInURL') and param_values.has_key('CompanyFacebookURL') and param_values.has_key('ProductFullPageIDs'):
            # Useful constants
            FIXED_DATE = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            PHONE_COUNTRY_PREFIX = self._get_phone_country_prefix(cr,uid)
            ONE_YEAR_AGO = datetime.date.today()-datetime.timedelta(days=366)
            PART_LINKED = param_values['CompanyLinkedInURL']
            PART_FB = param_values['CompanyFacebookURL']
            CONT_LINKED = param_values['ContactLinkedInURL']
            #MAX_CONTACTS = 10
            sectactiv_obj = self.pool.get('res.partner.activsector')
            sectactiv_ids = sectactiv_obj.search(cr,uid,[('directly','=',True)])
            hfFile3 = open('app_sects.csv', 'w')
            hfFile3.write(codecs.BOM_UTF8)
            hfCSV3 = csv.writer(hfFile3,delimiter=";",quotechar='"',quoting=csv.QUOTE_NONNUMERIC,lineterminator='\n')
            hfCSV3.writerow(['category_id','name','code','meaning'])
            sectors = sectactiv_obj.read(cr,uid,sectactiv_ids,['id','code','name'])
            dSectActivs = {}
            for sector in sectors:
                dSectActivs[sector['id']] = sector['name']
                record = []
                record.append(sector['id'])
                record.append(( (sector['name'] or '') + ' [' + sector['code'] + ']' ).encode('utf-8'))
                record.append((sector['code'] or '').encode('utf-8'))
                record.append((sector['name'] or '').encode('utf-8'))
                hfCSV3.writerow(record)
            partner_obj = self.pool.get('res.partner')
            partner_ids = partner_obj.search(cr,uid,[('state_id','=',1),('membership_state','in',['free','paid','invoiced'])])
            # add also Partners still with a valid full page
            # A valid page is a page invoiced maximum a year ago and not canceled by a credit nota
            #journal_ids = self.pool.get('account.journal').search(cr,uid,[('type','=','sale')])
            #mline_obj = self.pool.get('account.move.line')
            #mline_ids = mline_obj.search(cr,uid,[('product_id','in',param_values['ProductFullPageIDs']),('date','>=',ONE_YEAR_AGO),('journal_id','in',journal_ids)])
            #mlines = mline_obj.browse(cr,uid,mline_ids)
            supp_partner_ids = {}
            #for mline in mlines:
            #    if mline.partner_id:
            #        if mline.partner_id.id and mline.partner_id.id not in partner_ids:
            #            if mline.partner_id.id not in supp_partner_ids:
            #                data = {'id':mline.partner_id.id,
            #                        'amount':0.0,
            #                        'begindate':mline.date,
            #                        'enddate':(_get_date_from_string(mline.date)+datetime.timedelta(days=366)).strftime('%Y-%m-%d'),
            #                       }
            #            else:
            #                data = supp_partner_ids[mline.partner_id.id]
            #                if mline.date < data['begindate']:
            #                    data['begindate'] = mline.date
            #                new_enddate = (_get_date_from_string(mline.date)+datetime.timedelta(days=366)).strftime('%Y-%m-%d')
            #                if new_enddate > data['enddate']:
            #                    data['enddate'] = new_enddate
            #            data['amount'] += ((mline.credit or 0.0) - (mline.debit or 0.0))
            #            supp_partner_ids[mline.partner_id.id] = data
            #
            full_page_partner_ids = []
            #for (partner_id,data) in supp_partner_ids.items():
            #    full_page_partner_ids.append(partner_id)
            #    if data['amount'] > 0.001 and partner_id not in partner_ids:
            #        partner_ids.append(partner_id)
            in_five_days = (datetime.datetime.today()+datetime.timedelta(days=5)).strftime('%Y-%m-%d')
            thelastday = datetime.datetime.today().strftime('%Y-%m-%d')

            sub_type_obj = self.pool.get('partner_subscription.type')
            sub_type_ids = sub_type_obj.search(cr,uid,[('code','=','SOCREPELEC')])
            subscription_obj = self.pool.get('partner_subscription')
            subscription_ids = subscription_obj.search(cr,uid,[('type_id','in',sub_type_ids)])
            subscriptions = subscription_obj.browse(cr,uid,subscription_ids)
            for subs in subscriptions:
                if subs.state == 'current' and subs.begin and (subs.begin <= in_five_days) and subs.end and (subs.end >= thelastday) and subs.partner_id:
                    if not subs.partner_id.dir_exclude:
                        if subs.partner_id.id not in partner_ids:
                            partner_ids.append(subs.partner_id.id)
                        if subs.partner_id.id not in full_page_partner_ids:
                            full_page_partner_ids.append(subs.partner_id.id)
                        if subs.partner_id.id not in supp_partner_ids:
                            data = {'id':subs.partner_id.id,
                                    'amount':subs.price,
                                    'begindate':subs.begin,
                                    'enddate':subs.end,
                                   }
                            supp_partner_ids[subs.partner_id.id] = data
                        else:
                            data = supp_partner_ids[subs.partner_id.id]
                            data['amount'] += subs.price
                            data['begindate'] = min(data['begindate'],subs.begin)
                            data['enddate'] = max(data['enddate'],subs.end)
                            supp_partner_ids[subs.partner_id.id] = data
            all_address_partner_ids = list(partner_ids) # we record the ids of partners with all address published
            # search of all partners and addresses linked to premium; only the address can be 'full page' enabled by premium state
            #contact_obj = self.pool.get('res.partner.contact')
            #contact_ids = contact_obj.search(cr,uid,[('is_premium','=','OUI')])
            #full_page_address_ids = []
            supp_address_ids = {}
            #some_address_partner_address_ids = []
            #if contact_ids:
            #    contacts = contact_obj.browse(cr,uid,contact_ids)
            #    for contact in contacts:
            #        for job in contact.job_ids:
            #            if job.address_id and job.address_id.partner_id and not job.address_id.partner_id.dir_exclude:
            #                if job.address_id.partner_id.id not in partner_ids:
            #                    partner_ids.append(job.address_id.partner_id.id)
            #                if (job.address_id.partner_id.id,job.address_id.id) not in some_address_partner_address_ids:
            #                    some_address_partner_address_ids.append((job.address_id.partner_id.id,job.address_id.id))
            #                if contact.subscription_ids:
            #                    #full page subscription ?
            #                    for sub in contact.subscription_ids:
            #                        if sub.type_id and sub.type_id.code == 'REPMEMBRE' and sub.state  == 'current' and sub.begin <= aujourdhui and sub.end >= aujourdhui:
            #                            full_page_address_ids.append(job.address_id.id)
            #                            supp_address_ids[job.address_id.id] = {'id':job.address_id.id,
            #                                                                   'amount':sub.price,
            #                                                                   'begindate':sub.begin,
            #                                                                   'enddate':sub.end, 
            #                                                                  }
            # search of all partners linked to premium
            #contact_obj = self.pool.get('res.partner.contact')
            #contact_ids = contact_obj.search(cr,uid,[('is_premium','=','OUI')]) ## [('premium_begin','>','1980-01-01'),('premium_end','=',False)]
            #if contact_ids:
            #    contacts = contact_obj.browse(cr,uid,contact_ids)
            #    for contact in contacts:
            #        for job in contact.job_ids:
            #            if job.address_id and job.address_id.partner_id and job.address_id.partner_id.id not in partner_ids:
            #                partner_ids.append(job.address_id.partner_id.id)
            #
            partners = partner_obj.browse(cr,uid,partner_ids)
            hfFile1 = open('app_addrs.csv', 'w')
            hfFile1.write(codecs.BOM_UTF8)
            hfCSV1 = csv.writer(hfFile1,delimiter=";",quotechar='"',quoting=csv.QUOTE_NONNUMERIC,lineterminator='\n')
            hfCSV1.writerow(['partner_id','address_id','full_name','sort_name','num_ent','effectif_total','effectif_siege',
                             'sector1id','sector2id','sector3id','sector1name','sector2name','sector3name',
                             'activity_description','phone','fax','email',
                             'street','street2','zip_city','country','website','facebook','linkedin',
                             'creation','lastupdate','begin_fullpage','end_fullpage','has_premium'])
            hfFile2 = open('app_conts.csv', 'w')
            hfFile2.write(codecs.BOM_UTF8)
            hfCSV2 = csv.writer(hfFile2,delimiter=";",quotechar='"',quoting=csv.QUOTE_NONNUMERIC,lineterminator='\n')
            cont_fields = ['address_id','job_id','contact_id','sequence','name','first_name','courtesy','function_label',
                             'email','phone','mobile','birthdate','linkedin','creation','lastupdate','premium']
            hfCSV2.writerow(cont_fields)
            for partner in partners:
                if not partner.address:
                    errors.append({'id':1,'type':'partner without address','record_id':partner.id})
                else:
                    published_addresses = 0
                    for addr in partner.address:
                        if not addr.dir_exclude and partner.state_id.id == 1:
                            addr_record = []
                            addr_record.append(partner.id)
                            addr_record.append(addr.id)
                            if addr.dir_show_name:
                                addr_record.append(addr.dir_show_name.encode('utf-8'))
                            else:
                                if addr.name and addr.name.strip()[0:2] == '- ':
                                    addr_record.append((partner.name.strip() + ' ' + addr.name.strip()).encode('utf-8'))
                                elif addr.name:
                                    addr_record.append(addr.name.strip().encode('utf-8'))
                                else:
                                    addr_record.append(partner.name.strip().encode('utf-8'))
                            if addr.dir_sort_name:
                                addr_record.append(addr.dir_sort_name.strip().encode('utf-8'))
                            else:
                                addr_record.append( addr_record[-1:][0] )
                            addr_record.append(_convert_vat( partner.vat or '' ).encode('utf-8'))
                            addr_record.append(partner.employee_nbr_total or 0)
                            if addr.local_employee:
                                addr_record.append(addr.local_employee)
                            else:
                                addr_record.append(partner.employee_nbr or 0)
                            addr_record.append(addr.sector1 and addr.sector1.id or 0)
                            addr_record.append(addr.sector2 and addr.sector2.id or 0)
                            addr_record.append(addr.sector3 and addr.sector3.id or 0)
                            addr_record.append((addr.sector1 and dSectActivs[addr.sector1.id] or '').encode('utf-8'))
                            addr_record.append((addr.sector2 and dSectActivs[addr.sector2.id] or '').encode('utf-8'))
                            addr_record.append((addr.sector3 and dSectActivs[addr.sector3.id] or '').encode('utf-8'))
                            if addr.activity_description:
                                addr_record.append(addr.activity_description.encode('utf-8'))
                            else:
                                addr_record.append((partner.activity_description or '').encode('utf-8'))
                            addr_record.append(_convert_phone( addr.phone or '', PHONE_COUNTRY_PREFIX ).encode('utf-8'))
                            addr_record.append(_convert_phone( addr.fax or '', PHONE_COUNTRY_PREFIX ).encode('utf-8'))
                            addr_record.append((addr.email or '' ).encode('utf-8'))
                            addr_record.append((addr.street or '').encode('utf-8'))
                            addr_record.append((addr.street2 or '').encode('utf-8'))
                            if addr.zip_id:
                                addr_record.append(((addr.zip_id.name or '') + ' ' + ( addr.zip_id.city or '')).encode('utf-8'))
                            else:
                                addr_record.append('')
                            if not addr.country_id or addr.country_id.code == 'BE':
                                addr_record.append('')
                            else:
                                addr_record.append((addr.country_id.name or '').encode('utf-8'))
                            addr_record.append((partner.website or '').encode('utf-8'))
                            part_linkedin = ''
                            part_facebook = ''
                            for answer in partner.answers_ids:
                                if answer.question_id.id == PART_LINKED:
                                    part_linkedin = answer.text
                                if answer.question_id.id == PART_FB:
                                    part_facebook = answer.text
                            addr_record.append(part_facebook.encode('utf-8'))
                            addr_record.append(part_linkedin.encode('utf-8'))
                            addr_record.append(FIXED_DATE)
                            addr_record.append(FIXED_DATE)
                            if supp_partner_ids.has_key(partner.id) and supp_partner_ids[partner.id]['amount'] > 0.001:
                                addr_record.append(supp_partner_ids[partner.id]['begindate'])
                                addr_record.append(supp_partner_ids[partner.id]['enddate'])
                            else:
                                addr_record.append('')
                                addr_record.append('')
                            # extraction of contacts
                            # we extract the contacts to be published and if, no one, the first contacts
                            # we also add all premium contacts
                            selected_jobs = []
                            count_premium = 0
                            if addr.job_ids:
                                for job in addr.job_ids:
                                    #if job.is_premium == 'OUI':
                                    #    # we count only now
                                    #    count_premium += 1
                                    if job.contact_id and (job.sequence_directory and job.sequence_directory > 0 and job.sequence_directory < 99): # or (job.is_premium == 'OUI')):
                                        # search for linkedin url
                                        linkedin_url = ''
                                        if job.contact_id.answers_ids:
                                            for answer in job.contact_id.answers_ids:
                                                if answer.question_id.id == CONT_LINKED:
                                                    linkedin_url = answer.text or ''
                                                    break
                                        new_job = {'address_id': addr.id,
                                                   'job_id': job.id,
                                                   'contact_id': job.contact_id.id,
                                                   'sequence': job.sequence_directory,
                                                   'name': job.contact_id.name.encode('utf-8'),
                                                   'first_name': (job.contact_id.first_name or '').encode('utf-8'),
                                                   'courtesy': (job.contact_id.title or '').encode('utf-8'),
                                                   'email': (job.email or job.contact_id.email or '').encode('utf-8'),
                                                   'function_label': (job.function_label or '').encode('utf-8'),
                                                   'phone': _convert_phone(job.phone or '',PHONE_COUNTRY_PREFIX).encode('utf-8'),
                                                   'mobile': _convert_phone(job.contact_id.mobile or '',PHONE_COUNTRY_PREFIX).encode('utf-8'),
                                                   'birthdate': job.contact_id.birthdate or '',
                                                   'linkedin': linkedin_url.encode('utf-8'),
                                                   'creation': FIXED_DATE,
                                                   'lastupdate':  FIXED_DATE,
                                                   'premium': '', #job.contact_id.is_premium,
                                                  }
                                        selected_jobs.append(new_job)
                                        if selected_jobs:
                                            selected_jobs = sorted(selected_jobs,key= lambda x: x['sequence'])
                                #if not selected_jobs:
                                #    # there is no contacts with (sequence_directory) => we extract the premium
                                #    for job in addr.job_ids:
                                #        if job.contact_id and job.contact_id.is_premium == 'OUI':
                                #            # search for linkedin url
                                #            linkedin_url = ''
                                #            if job.contact_id.answers_ids:
                                #                for answer in job.contact_id.answers_ids:
                                #                    if answer.question_id.id == CONT_LINKED:
                                #                        linkedin_url = answer.text or ''
                                #                        break
                                #            new_job = {'address_id': addr.id,
                                #                       'job_id': job.id,
                                #                       'contact_id': job.contact_id.id,
                                #                       'sequence': job.sequence_directory,
                                #                       'name': job.contact_id.name.encode('utf-8'),
                                #                       'first_name': (job.contact_id.first_name or '').encode('utf-8'),
                                #                       'courtesy': (job.contact_id.title or '').encode('utf-8'),
                                #                       'email': (job.email or job.contact_id.email or '').encode('utf-8'),
                                #                       'function_label': (job.function_label or '').encode('utf-8'),
                                #                       'phone': _convert_phone(job.phone or job.contact_id.mobile or '',PHONE_COUNTRY_PREFIX).encode('utf-8'),
                                #                       'mobile': _convert_phone(job.contact_id.mobile or '',PHONE_COUNTRY_PREFIX).encode('utf-8'),
                                #                       'birthdate': job.contact_id.birthdate or '',
                                #                       'linkedin': linkedin_url.encode('utf-8'),
                                #                       'creation': FIXED_DATE,
                                #                       'lastupdate':  FIXED_DATE,
                                #                       'premium': job.contact_id.is_premium,
                                #                      }
                                #            selected_jobs.append(new_job)
                                if not selected_jobs:
                                    # there is still no contacts with (sequence_directory) => we extract the first 5 contacts in normal sequence
                                    for job in addr.job_ids:
                                        if job.contact_id:
                                            # search for linkedin url
                                            linkedin_url = ''
                                            if job.contact_id.answers_ids:
                                                for answer in job.contact_id.answers_ids:
                                                    if answer.question_id.id == CONT_LINKED:
                                                        linkedin_url = answer.text or ''
                                                        break
                                            new_job = {'address_id': addr.id,
                                                       'job_id': job.id,
                                                       'contact_id': job.contact_id.id,
                                                       'sequence': job.sequence_directory,
                                                       'name': job.contact_id.name.encode('utf-8'),
                                                       'first_name': (job.contact_id.first_name or '').encode('utf-8'),
                                                       'courtesy': (job.contact_id.title or '').encode('utf-8'),
                                                       'email': (job.email or job.contact_id.email or '').encode('utf-8'),
                                                       'function_label': (job.function_label or '').encode('utf-8'),
                                                       'phone': _convert_phone(job.phone or job.contact_id.mobile or '',PHONE_COUNTRY_PREFIX).encode('utf-8'),
                                                       'mobile': _convert_phone(job.contact_id.mobile or '',PHONE_COUNTRY_PREFIX).encode('utf-8'),
                                                       'birthdate': job.contact_id.birthdate or '',
                                                       'linkedin': linkedin_url.encode('utf-8'),
                                                       'creation': FIXED_DATE,
                                                       'lastupdate':  FIXED_DATE,
                                                       'premium': '', #job.contact_id.is_premium,
                                                      }
                                            selected_jobs.append(new_job)
                                    if selected_jobs:
                                        selected_jobs = sorted(selected_jobs,key= lambda x: x['sequence'])
                                if selected_jobs:
                                    new_sequence = 1
                                    for new_job in selected_jobs:
                                        if new_sequence <= MAX_CONTACTS or new_job['premium']:
                                            new_job = selected_jobs[new_sequence-1]
                                            new_job['sequence'] = new_sequence
                                            record = []
                                            for field in cont_fields:
                                                record.append(new_job[field])
                                            new_sequence += 1
                                            hfCSV2.writerow(record)
                                            count_contacts += 1
                            # end of address management : we indicate if this address has some premium and record this address only if member or full page or linked premium
                            if (partner.membership_state in ['free','invoiced','paid']) or (partner.id in full_page_partner_ids) or (count_premium > 0):
                                addr_record.append(count_premium and 'OUI' or '')
                                hfCSV1.writerow(addr_record)
                                published_addresses += 1
                    if published_addresses == 0:
                        errors.append({'id':2,'type':'partner without published address','record_id':partner.id})
                    else:
                        count_addresses += published_addresses
                        count_partners += 1
            hfFile1.close()
            hfFile2.close()
            hfFile3.close()
        else:
            errors.append({'id':999,'type':'missing parameters','record_id':0})
        return ('app_addrs.csv','app_conts.csv','app_sects.csv',count_partners,count_addresses,count_contacts,errors)
    def _incremential_update(self,cr,uid,context={}):
    
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['BDDeTest'])
        if param_values.has_key('BDDeTest'):
            TO_TEST = param_values['BDDeTest']
        else:
            TO_TEST = True
        
        # Start log
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'AppSyncIncremential')
        final_result = True
        #
        errors = []
        count_partners = 0
        count_addresses = 0
        count_contacts = 0
        aujourdhui = datetime.datetime.today().strftime('%Y-%m-%d')
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['CompanyLinkedInURL','ContactLinkedInURL','CompanyFacebookURL','ProductFullPageIDs','AppMobileConnectionDict'])
        if param_values.has_key('CompanyLinkedInURL') and param_values.has_key('ContactLinkedInURL') and \
                param_values.has_key('CompanyFacebookURL') and param_values.has_key('ProductFullPageIDs') and param_values.has_key('AppMobileConnectionDict'):
            # Useful constants
            FIXED_DATE = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            PHONE_COUNTRY_PREFIX = self._get_phone_country_prefix(cr,uid)
            ONE_YEAR_AGO = datetime.date.today()-datetime.timedelta(days=366)
            PART_LINKED = param_values['CompanyLinkedInURL']
            PART_FB = param_values['CompanyFacebookURL']
            CONT_LINKED = param_values['ContactLinkedInURL']
            #MAX_CONTACTS = 10
            cont_fields = ['address_id','job_id','contact_id','sequence','name','first_name','courtesy','function_label',
                             'email','phone','mobile','birthdate','linkedin','creation','lastupdate','premium']
            connection = param_values['AppMobileConnectionDict']
            
            # 1. Synchronise the secteur table and keep mysql-ids
            # Connection to the external db
            try:
                con = pymysql.connect(host=connection['host'], port=connection['port'], user=connection['user'], passwd=connection['pw'], db=connection['db'], charset='utf8');
                cur = con.cursor()
                sentence = "SELECT category_id, name, code, meaning FROM `app_sects`"
                cur.execute(sentence)
                data = cur.fetchall()
                dSectors = {}
                for line in data:
                    dSectors[line[2]] = {'category_id':line[0],'name':line[1],'meaning':line[3]}
                changes = False
                sectactiv_obj = self.pool.get('res.partner.activsector')
                sectactiv_ids = sectactiv_obj.search(cr,uid,[('directly','=',True)])
                sectors = sectactiv_obj.read(cr,uid,sectactiv_ids,['id','code','name'])
                dSectActivs = {}
                dSectCode2ID = {}
                lSendedCodes = []
                for sector in sectors:
                    lSendedCodes.append(sector['code'])
                    dSectActivs[sector['id']] = sector
                    dSectCode2ID[sector['code']]= sector['id']
                    if dSectors.has_key(sector['code']):
                        if dSectors[sector['code']]['name'] <> (u'%s [%s]' % (sector['name'],sector['code'])) or dSectors[sector['code']]['meaning'] <> sector['name']:
                            # update values
                            sentence = "UPDATE `app_sects` SET name = '%s', meaning = '%s' WHERE category_id = %s" % (('%s [%s]' % (sector['name'],sector['code'])).replace("'","''"),
                                                                                                                       (sector['name'] or '').replace("'","''"),
                                                                                                                       str(dSectors[sector['code']]['category_id']))
                            cur.execute(sentence)
                            changes = True
                            self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nMise à jour du secteur [%s] reussie -> %s' % (str(dSectors[sector['code']]['category_id']),sector['name']) )
                    else:
                        # insert new value
                        sentence = "INSERT INTO `app_sects` ( category_id, name, code, meaning ) VALUES ( %s, '%s', '%s', '%s')" % (sector['id'],
                                                                                                                                     ('%s [%s]' % (sector['name'],sector['code'])).replace("'","''"),
                                                                                                                                     sector['code'],
                                                                                                                                     (sector['name'] or '').replace("'","''"))
                        cur.execute(sentence)
                        changes = True
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nInsertion du secteur %s %s' % (str(sector['id']),sector['name']) )
                for (key,value) in dSectors.items():
                    if key not in lSendedCodes:
                        sentence = "DELETE from  `app_sects` WHERE code = %s" % key
                        cur.execute(sentence)
                        changes = True
                        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'\nEffacement secteur [%s] reussie' % (key) )
                if changes and final_result:
                    con.commit()
            except pymysql.err.Error, e:
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Connection Error %d: %s\nNo updating of app_sects' % (e.args[0],e.args[1]))
                final_result = False
                if cur:
                    cur.close()
                if con:
                    con.close()
            if final_result:
                try:
                    con = pymysql.connect(host=connection['host'], port=connection['port'], user=connection['user'], passwd=connection['pw'], db=connection['db'], charset='utf8');
                    cur = con.cursor()
                    sentence = """SELECT partner_id, address_id, full_name, sort_name, num_ent, effectif_total, effectif_siege, sector1id, sector2id, sector3id, 
                                         sector1name, sector2name, sector3name, activity_description, phone, fax, email, 
                                         street, street2, zip_city, country, website, facebook, linkedin, 
                                         creation, lastupdate, begin_fullpage, end_fullpage, has_premium FROM `app_addrs`"""
                    cur.execute(sentence)
                    data = cur.fetchall()
                    dAddrs = {}
                    for line in data:
                        dAddrs[line[1]] = {'partner_id':line[0],'address_id':line[1],'full_name':line[2],'sort_name':line[3],'num_ent':line[4],
                                           'effectif_total':line[5],'effectif_siege':line[6],'sector1id':line[7],'sector2id':line[8],'sector3id':line[9],
                                           'sector1name':line[10],'sector2name':line[11],'sector3name':line[12],'activity_description':line[13],
                                           'phone':line[14],'fax':line[15],'email':line[16],
                                           'street':line[17],'street2':line[18],'zip_city':line[19],'country':line[20],
                                           'website':line[21],'facebook':line[22],'linkedin':line[23],
                                           'creation':line[24],'lastupdate':line[25],'begin_fullpage':line[26],'end_fullpage':line[27],'has_premium':line[28]}
                    sentence = """SELECT address_id, job_id, contact_id, sequence, name, first_name, courtesy, function_label,
                                         email, phone, mobile, birthdate, linkedin, creation, lastupdate, premium FROM `app_conts`"""
                    cur.execute(sentence)
                    data = cur.fetchall()
                    dConts = {}
                    for line in data:
                        dConts[(line[0],line[2])] = {'address_id':line[0],'job_id':line[1],'contact_id':line[2],'sequence':line[3],
                                                     'name':line[4],'first_name':line[5],'courtesy':line[6],'function_label':line[7],
                                                     'email':line[8],'phone':line[9],'mobile':line[10],'birthdate':line[11],'linkedin':line[12],
                                                     'creation':line[13],'lastupdate':line[14],'premium':line[15]}
                except pymysql.err.Error, e:
                    self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,'Connection Error %d: %s\nNo updating of app_addrs and app_conts' % (e.args[0],e.args[1]))
                    final_result = False
                    if cur:
                        cur.close()
                    if con:
                        con.close()
                #
                (dOldSect2New,categ_ids) = self.convert_old_sector_ids_to_new_ids(cr,uid,dSectCode2ID) # table to convert id from res.partner_category to id in res.partner.activsector AND list of valid res.partner.category ids
                #
                partner_obj = self.pool.get('res.partner')
                #partner_ids = partner_obj.search(cr,uid,[('state_id','=',1),('membership_state','in',['free','paid','invoiced']),('dir_exclude','=',False)])
                partner_ids = partner_obj.search(cr,uid,[('state_id','=',1),('dir_exclude','=',False),'|',('dir_presence','=',True),('membership_state','in',['free','paid','invoiced'])])
                # add also Partners still with a valid full page
                # A valid page is a page invoiced maximum a year ago and not canceled by a credit nota
                # this is not valid today : we MUST have a current subsciption event free
                # cci_cleaning has a task that clean current subscriptions canceled by refund
                
                #journal_ids = self.pool.get('account.journal').search(cr,uid,[('type','=','sale')])
                #mline_obj = self.pool.get('account.move.line')
                #mline_ids = mline_obj.search(cr,uid,[('product_id','in',param_values['ProductFullPageIDs']),('date','>=',ONE_YEAR_AGO),('journal_id','in',journal_ids)])
                #mlines = mline_obj.browse(cr,uid,mline_ids)
                #for mline in mlines:
                #    if mline.partner_id:
                #        if mline.partner_id.id and mline.partner_id.id not in partner_ids and not mline.partner_id.dir_exclude:
                #            if mline.partner_id.id not in supp_partner_ids:
                #                data = {'id':mline.partner_id.id,
                #                        'amount':0.0,
                #                        'begindate':mline.date,
                #                        'enddate':(_get_date_from_string(mline.date)+datetime.timedelta(days=366)).strftime('%Y-%m-%d'),
                #                       }
                #            else:
                #                data = supp_partner_ids[mline.partner_id.id]
                #                if mline.date < data['begindate']:
                #                    data['begindate'] = mline.date
                #                new_enddate = (_get_date_from_string(mline.date)+datetime.timedelta(days=366)).strftime('%Y-%m-%d')
                #                if new_enddate > data['enddate']:
                #                    data['enddate'] = new_enddate
                #            data['amount'] += ((mline.credit or 0.0) - (mline.debit or 0.0))
                #            supp_partner_ids[mline.partner_id.id] = data
                #
                #for (partner_id,data) in supp_partner_ids.items():
                #    full_page_partner_ids.append(partner_id)
                #    if data['amount'] > 0.001 and partner_id not in partner_ids:
                #        partner_ids.append(partner_id)
                # add also Partners that have bought an annual subcription to this service
                supp_partner_ids = {}
                full_page_partner_ids = []
                in_five_days = (datetime.datetime.today()+datetime.timedelta(days=5)).strftime('%Y-%m-%d')
                thelastday = datetime.datetime.today().strftime('%Y-%m-%d')
                sub_type_obj = self.pool.get('partner_subscription.type')
                sub_type_ids = sub_type_obj.search(cr,uid,[('code','=','SOCREPELEC')])
                subscription_obj = self.pool.get('partner_subscription')
                subscription_ids = subscription_obj.search(cr,uid,[('type_id','in',sub_type_ids)])
                subscriptions = subscription_obj.browse(cr,uid,subscription_ids)
                for subs in subscriptions:
                    if subs.state == 'current' and subs.begin and (subs.begin <= in_five_days) and subs.end and (subs.end >= thelastday) and subs.partner_id:
                        if not subs.partner_id.dir_exclude:
                            if subs.partner_id.id not in partner_ids:
                                partner_ids.append(subs.partner_id.id)
                            if subs.partner_id.id not in full_page_partner_ids:
                                full_page_partner_ids.append(subs.partner_id.id)
                            if subs.partner_id.id not in supp_partner_ids:
                                data = {'id':subs.partner_id.id,
                                        'amount':subs.price,
                                        'begindate':subs.begin,
                                        'enddate':subs.end,
                                       }
                                supp_partner_ids[subs.partner_id.id] = data
                            else:
                                data = supp_partner_ids[subs.partner_id.id]
                                data['amount'] += subs.price
                                data['begindate'] = min(data['begindate'],subs.begin)
                                data['enddate'] = max(data['enddate'],subs.end)
                                supp_partner_ids[subs.partner_id.id] = data
                all_address_partner_ids = list(partner_ids) # we record the ids of partners with all address published
                # search of all partners and addresses linked to premium; only the address can be 'full page' enabled by premium state
                #contact_obj = self.pool.get('res.partner.contact')
                #contact_ids = contact_obj.search(cr,uid,[('is_premium','=','OUI')])
                #full_page_address_ids = []
                supp_address_ids = {}
                #some_address_partner_address_ids = []
                #if contact_ids:
                #    contacts = contact_obj.browse(cr,uid,contact_ids)
                #    for contact in contacts:
                #        for job in contact.job_ids:
                #            if job.address_id and job.address_id.partner_id and not job.address_id.partner_id.dir_exclude:
                #                if job.address_id.partner_id.id not in partner_ids:
                #                    partner_ids.append(job.address_id.partner_id.id)
                #                if (job.address_id.partner_id.id,job.address_id.id) not in some_address_partner_address_ids:
                #                    some_address_partner_address_ids.append((job.address_id.partner_id.id,job.address_id.id))
                #                if contact.subscription_ids:
                #                    #full page subscription ?
                #                    for sub in contact.subscription_ids:
                #                        if sub.type_id and sub.type_id.code == 'REPMEMBRE' and sub.state  == 'current' and sub.begin <= aujourdhui and sub.end >= aujourdhui:
                #                            full_page_address_ids.append(job.address_id.id)
                #                            supp_address_ids[job.address_id.id] = {'id':job.address_id.id,
                #                                                                   'amount':sub.price,
                #                                                                   'begindate':sub.begin,
                #                                                                   'enddate':sub.end, 
                #                                                                  }
                #
                lSendedAddrs = []
                lSendedConts = []
                updated_addrs = 0
                inserted_addrs = 0
                deleted_addrs = 0
                updated_conts = 0
                inserted_conts = 0
                deleted_conts = 0
                partners = partner_obj.browse(cr,uid,partner_ids)
                for partner in partners:
                    if not partner.address:
                        errors.append({'id':1,'type':'partner without address','record_id':partner.id})
                    else:
                        # extraction of partner activity sectors to use if no activity sectors on address
                        (partner_sector1,partner_sector2,partner_sector3) = self.get_activity_sectors(partner,categ_ids,dOldSect2New)
                        published_addresses = 0
                        for addr in partner.address:
                            if (not addr.dir_exclude) and partner.state_id.id == 1 and ((partner.id in all_address_partner_ids) or ((partner.id,addr.id) in some_address_partner_address_ids)) and (addr.type <> 'invoice' or addr.dir_presence):
                                addr_record = {}
                                addr_record['partner_id'] = partner.id
                                addr_record['address_id'] = addr.id
                                if addr.dir_show_name:
                                    addr_record['full_name'] = addr.dir_show_name.strip()
                                else:
                                    if addr.name and addr.name.strip()[0:2] == '- ':
                                        addr_record['full_name'] = (partner.name.strip() + ' ' + addr.name.strip())
                                    elif addr.name:
                                        addr_record['full_name'] = addr.name.strip()
                                    else:
                                        addr_record['full_name'] = partner.name.strip()
                                if addr.dir_sort_name:
                                    addr_record['sort_name'] = addr.dir_sort_name.strip()
                                else:
                                    addr_record['sort_name'] = addr_record['full_name']
                                addr_record['num_ent'] = _convert_vat( partner.vat or '' )
                                addr_record['effectif_total'] = partner.employee_nbr_total or 0
                                if addr.local_employee:
                                    addr_record['effectif_siege'] = addr.local_employee
                                else:
                                    addr_record['effectif_siege'] = partner.employee_nbr or 0
                                if addr.sector1:
                                    addr_record['sector1id'] = addr.sector1 and addr.sector1.id or 0
                                    addr_record['sector2id'] = addr.sector2 and addr.sector2.id or 0
                                    addr_record['sector3id'] = addr.sector3 and addr.sector3.id or 0
                                    addr_record['sector1name'] = addr.sector1 and dSectActivs[addr.sector1.id]['name'] or ''
                                    addr_record['sector2name'] = addr.sector2 and dSectActivs[addr.sector2.id]['name'] or ''
                                    addr_record['sector3name'] = addr.sector3 and dSectActivs[addr.sector3.id]['name'] or ''
                                else:
                                    addr_record['sector1id'] = partner_sector1
                                    addr_record['sector2id'] = partner_sector2
                                    addr_record['sector3id'] = partner_sector3
                                    addr_record['sector1name'] = partner_sector1 and dSectActivs[partner_sector1]['name'] or ''
                                    addr_record['sector2name'] = partner_sector2 and dSectActivs[partner_sector2]['name'] or ''
                                    addr_record['sector3name'] = partner_sector3 and dSectActivs[partner_sector3]['name'] or ''
                                if addr.activity_description:
                                    addr_record['activity_description'] = addr.activity_description
                                else:
                                    addr_record['activity_description'] = partner.activity_description or ''
                                addr_record['phone'] = _convert_phone( addr.phone or '', PHONE_COUNTRY_PREFIX )
                                addr_record['fax'] = _convert_phone( addr.fax or '', PHONE_COUNTRY_PREFIX )
                                addr_record['email'] = addr.email or ''
                                addr_record['street'] = addr.street or ''
                                addr_record['street2'] = addr.street2 or ''
                                if addr.zip_id:
                                    addr_record['zip_city'] = (addr.zip_id.name or '') + ' ' + ( addr.zip_id.city or '')
                                else:
                                    addr_record['zip_city'] = ''
                                if not addr.country_id or addr.country_id.code == 'BE':
                                    addr_record['country'] = ''
                                else:
                                    addr_record['country'] = addr.country_id.name or ''
                                addr_record['website'] = partner.website or ''
                                part_linkedin = ''
                                part_facebook = ''
                                for answer in partner.answers_ids:
                                    if answer.question_id.id == PART_LINKED:
                                        part_linkedin = answer.text
                                    if answer.question_id.id == PART_FB:
                                        part_facebook = answer.text
                                addr_record['facebook'] = part_facebook
                                addr_record['linkedin'] = part_linkedin
                                if (supp_partner_ids.has_key(partner.id) and supp_partner_ids[partner.id]['amount'] > 0.001) or (supp_address_ids.has_key(addr.id)):
                                    if (supp_partner_ids.has_key(partner.id) and supp_partner_ids[partner.id]['amount'] > 0.001) and (supp_address_ids.has_key(addr.id)):
                                        addr_record['begin_fullpage'] = min(supp_partner_ids[partner.id]['begindate'],supp_address_ids[addr.id]['begindate'])
                                        addr_record['end_fullpage'] = max(supp_partner_ids[partner.id]['enddate'],supp_address_ids[addr.id]['enddate'])
                                    elif (supp_partner_ids.has_key(partner.id) and supp_partner_ids[partner.id]['amount'] > 0.001) and not (supp_address_ids.has_key(addr.id)):
                                        addr_record['begin_fullpage'] = supp_partner_ids[partner.id]['begindate']
                                        addr_record['end_fullpage'] = supp_partner_ids[partner.id]['enddate']
                                    elif (supp_address_ids.has_key(addr.id)) and not (supp_partner_ids.has_key(partner.id) and supp_partner_ids[partner.id]['amount'] > 0.001):
                                        addr_record['begin_fullpage'] = supp_address_ids[addr.id]['begindate']
                                        addr_record['end_fullpage'] = supp_address_ids[addr.id]['enddate']
                                else:
                                    addr_record['begin_fullpage'] = ''
                                    addr_record['end_fullpage'] = ''
                                # extraction of contacts
                                # We publish all contacts, except 'mobile_exclude'
                                # For the address published because 'premium', we publish only premium names (decision of Vincnet Mausen 2015-04-15)
                                # except if the linked partner if member, in this case, the member state of the partner has priority on freemium state
                                selected_jobs = []
                                count_premium = 0
                                if addr.job_ids:
                                    #if (partner.membership_state not in ['free','invoiced','paid']) and ((partner.id,addr.id) in some_address_partner_address_ids):
                                    #    for job in addr.job_ids:
                                    #        if job.is_premium == 'OUI' and job.contact_id and not job.mobile_exclude:
                                    #            # we count only now
                                    #            count_premium += 1
                                    #            # search for linkedin url
                                    #            linkedin_url = ''
                                    #            if job.contact_id.answers_ids:
                                    #                for answer in job.contact_id.answers_ids:
                                    #                    if answer.question_id.id == CONT_LINKED:
                                    #                        linkedin_url = answer.text or ''
                                    #                        break
                                    #            new_job = {'address_id': addr.id,
                                    #                       'job_id': job.id,
                                    #                       'contact_id': job.contact_id.id,
                                    #                       'sequence': 1,
                                    #                       'name': job.contact_id.name,
                                    #                       'first_name': job.contact_id.first_name or '',
                                    #                       'courtesy': job.contact_id.title or '',
                                    #                       'email': job.email or job.contact_id.email or '',
                                    #                       'function_label': job.function_label or '',
                                    #                       'phone': _convert_phone(job.phone or '',PHONE_COUNTRY_PREFIX),
                                    #                       'mobile': _convert_phone(job.contact_id.mobile or '',PHONE_COUNTRY_PREFIX),
                                    #                       'birthdate': job.contact_id.birthdate or '',
                                    #                       'linkedin': linkedin_url,
                                    #                       'premium': job.contact_id.is_premium,
                                    #                      }
                                    #            selected_jobs.append(new_job)
                                    #else:
                                    for job in addr.job_ids:
                                        #if job.is_premium == 'OUI':
                                        #    # we count only now
                                        #    count_premium += 1
                                        if job.contact_id and not job.mobile_exclude:
                                            # search for linkedin url
                                            linkedin_url = ''
                                            if job.contact_id.answers_ids:
                                                for answer in job.contact_id.answers_ids:
                                                    if answer.question_id.id == CONT_LINKED:
                                                        linkedin_url = answer.text or ''
                                                        break
                                            new_job = {'address_id': addr.id,
                                                       'job_id': job.id,
                                                       'contact_id': job.contact_id.id,
                                                       'sequence': job.sequence_directory,
                                                       'name': job.contact_id.name,
                                                       'first_name': job.contact_id.first_name or '',
                                                       'courtesy': job.contact_id.title or '',
                                                       'email': job.email or job.contact_id.email or '',
                                                       'function_label': job.function_label or '',
                                                       'phone': _convert_phone(job.phone or '',PHONE_COUNTRY_PREFIX),
                                                       'mobile': _convert_phone(job.contact_id.mobile or '',PHONE_COUNTRY_PREFIX),
                                                       'birthdate': job.contact_id.birthdate or '',
                                                       'linkedin': linkedin_url,
                                                       'premium': '', #job.contact_id.is_premium,
                                                      }
                                            selected_jobs.append(new_job)
                                    if selected_jobs:
                                        selected_jobs = sorted(selected_jobs,key= lambda x: x['sequence'])
                                        new_sequence = 1
                                        for new_job in selected_jobs:
                                            #new_job = selected_jobs[new_sequence-1]
                                            if new_job['sequence'] < 90:
                                                new_job['sequence'] = new_sequence
                                                new_sequence += 1
                                            lSendedConts.append((new_job['address_id'],new_job['contact_id']))
                                            if dConts.has_key((new_job['address_id'],new_job['contact_id'])):
                                                # check to see if data have changed
                                                mysql_job_record = dConts[(new_job['address_id'],new_job['contact_id'])]
                                                pre_sentence = ''
                                                for field_name in ['job_id','sequence','name','first_name','courtesy','function_label','email','phone','mobile','birthdate','linkedin','premium']:
                                                    if new_job[field_name] <> mysql_job_record[field_name]:
                                                        if field_name in ['job_id','sequence']:
                                                            field_value = str(new_job[field_name])
                                                        else:
                                                            field_value = "'%s'" % new_job[field_name].replace("'","''").replace('\\','\\\\')
                                                        pre_sentence += ', `%s` = %s' % (field_name,field_value)
                                                if pre_sentence:
                                                    sentence = "UPDATE `app_conts` SET %s, lastupdate = '%s' WHERE address_id = %s AND contact_id = %s" % (pre_sentence[2:],FIXED_DATE,new_job['address_id'],new_job['contact_id'])
                                                    if TO_TEST:
                                                        print sentence
                                                    else:
                                                        cur.execute(sentence)
                                                    updated_conts += 1
                                            else:
                                                # inserting into app_conts
                                                pre_sentence = ''
                                                field_list = ''
                                                for field_name in ['address_id','job_id','contact_id','sequence','name','first_name','courtesy','function_label','email','phone','mobile','birthdate','linkedin','premium']:
                                                        field_list += ',`%s`' % field_name
                                                        if field_name in ['address_id','job_id','contact_id','sequence']:
                                                            field_value = str(new_job[field_name])
                                                        else:
                                                            field_value = "'%s'" % new_job[field_name].replace("'","''").replace('\\','\\\\')
                                                        pre_sentence += ',%s' % field_value
                                                field_list += ',`creation`,`lastupdate`'
                                                pre_sentence += ",'%s','%s'" % (FIXED_DATE,FIXED_DATE)
                                                sentence = "INSERT into `app_conts` (%s) VALUES (%s)" % (field_list[1:],pre_sentence[1:])
                                                if TO_TEST:
                                                    print sentence
                                                else:
                                                    cur.execute(sentence)
                                                inserted_conts += 1
                                            count_contacts += 1

                                # end of address management : we indicate if this address has some premium and record this address only if member or full page or linked premium
                                if (partner.membership_state in ['free','invoiced','paid']) or (partner.id in full_page_partner_ids) or (count_premium > 0):
                                    addr_record['has_premium'] = count_premium and 'OUI' or ''
                                    published_addresses += 1
                                else:
                                    addr_record['has_premium'] = ''
                                # Check if the address exists and has the same data
                                lSendedAddrs.append(addr_record['address_id'])
                                if dAddrs.has_key(addr_record['address_id']):
                                    mysql_record = dAddrs[addr_record['address_id']]
                                    pre_sentence = ''
                                    for field_name in ['partner_id','address_id','full_name','sort_name','num_ent','effectif_total','effectif_siege','sector1id','sector2id','sector3id',
                                                       'sector1name','sector2name','sector3name','activity_description','phone','fax','email','street','street2','zip_city','country',
                                                       'website','facebook','linkedin','begin_fullpage','end_fullpage','has_premium']:
                                        if addr_record[field_name] <> mysql_record[field_name]:
                                            if field_name in ['partner_id','address_id','effectif_total','effectif_siege','sector1id','sector2id','sector3id']:
                                                field_value = str(addr_record[field_name])
                                            else:
                                                field_value = "'%s'" % addr_record[field_name].replace("'","''").replace('\\','\\\\')
                                            pre_sentence += ', `%s` = %s' % (field_name,field_value)
                                    if pre_sentence:
                                        sentence = "UPDATE `app_addrs` SET %s, lastupdate = '%s' WHERE address_id = %s" % (pre_sentence[2:],FIXED_DATE,addr_record['address_id'])
                                        if TO_TEST:
                                            print sentence
                                        else:
                                            cur.execute(sentence)
                                        updated_addrs += 1
                                else:
                                    pre_sentence = ''
                                    field_list = ''
                                    for field_name in ['partner_id','address_id','full_name','sort_name','num_ent','effectif_total','effectif_siege','sector1id','sector2id','sector3id',
                                                       'sector1name','sector2name','sector3name','activity_description','phone','fax','email','street','street2','zip_city','country',
                                                       'website','facebook','linkedin','begin_fullpage','end_fullpage','has_premium']:
                                            field_list += ',`%s`' % field_name
                                            if field_name in ['partner_id','address_id','effectif_total','effectif_siege','sector1id','sector2id','sector3id']:
                                                field_value = str(addr_record[field_name])
                                            else:
                                                field_value = "'%s'" % addr_record[field_name].replace("'","''").replace('\\','\\\\')
                                            pre_sentence += ',%s' % field_value
                                    field_list += ',`creation`,`lastupdate`'
                                    pre_sentence += ",'%s','%s'" % (FIXED_DATE,FIXED_DATE)
                                    sentence = "INSERT into `app_addrs` (%s) VALUES (%s)" % (field_list[1:],pre_sentence[1:])
                                    if TO_TEST:
                                        print sentence
                                    else:
                                        cur.execute(sentence)
                                    inserted_addrs += 1
                        if published_addresses == 0:
                            errors.append({'id':2,'type':'partner without published address','record_id':partner.id})
                        else:
                            count_addresses += published_addresses
                            count_partners += 1
                to_delete_addr_ids = []
                for key in dAddrs.keys():
                    if key not in lSendedAddrs:
                        to_delete_addr_ids.append(key)
                if to_delete_addr_ids:
                    deleted_addrs = len(to_delete_addr_ids)
                    sentence = "DELETE FROM `app_conts` WHERE `address_id` in (%s)" % ','.join([str(x) for x in to_delete_addr_ids])
                    if TO_TEST:
                        print sentence
                    else:
                        cur.execute(sentence)
                    sentence = "DELETE FROM `app_addrs` WHERE `address_id` in (%s)" % ','.join([str(x) for x in to_delete_addr_ids])
                    if TO_TEST:
                        print sentence
                    else:
                        cur.execute(sentence)
                for key in dConts.keys():
                    if key not in lSendedConts and key[0] and key[1]:
                        deleted_conts += 1
                        sentence = "DELETE FROM `app_conts` WHERE `address_id` = %d AND `contact_id` = %d" % (key[0],key[1])
                        if TO_TEST:
                            print sentence
                        else:
                            cur.execute(sentence)
                if (updated_addrs or inserted_addrs or deleted_addrs or updated_conts or inserted_conts or deleted_conts) and final_result:
                    con.commit()
                self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u"""\nRésultats finaux :
                                                                                       \nAdresses mises à jour : %d
                                                                                       \nAdresses ajoutées : %d
                                                                                       \nAddresses effacées : %d
                                                                                       \nContacts mis à jour : %d
                                                                                       \nContacts ajoutés : %d
                                                                                       \nContacts effacés : %d
                                                                                    """ % (updated_addrs,inserted_addrs,deleted_addrs,updated_conts,inserted_conts,deleted_conts) )
        else:
            errors.append({'id':999,'type':'missing parameters','record_id':0})
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,u'\nErreurs :\n' + '\n - '.join(['[%s] %s sur partenaire (%s)' % (str(x['id']),x['type'],str(x['record_id'])) for x in errors]) )
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return (count_partners,count_addresses,count_contacts,errors)
cci_sync_mobile()

class cci_mobile_connection(osv.osv):
    _name = "cci_mobile.connection"
    _description = "Données de création de login-password pr application mobile CCI"
    _columns = {
        'detected_partner_id':fields.integer('Detected Partner'),
        'detected_address_id':fields.integer('Detected Address'),
        'detected_job_id':fields.integer('Detected Job'),
        'detected_contact_id':fields.integer('Detected Contact'),
        'vat':fields.char('VAT',size=40,required=True),
        'vat_subjected':fields.boolean('VAT Subjected',required=True),
        'partner_name':fields.char('Partner Name',size=120,required=True),
        'contact_name':fields.char('Contact Name',size=120,required=True),
        'contact_first_name':fields.char('Contact First Name',size=120,required=True),
        'function_label':fields.char('Function Label',size=200,required=True),
        'login':fields.char('Login',size=60,required=True),
        'motpasse':fields.char('Password',size=60,required=True),
        'courtesy':fields.char('Courtesy',size=30),
        'mobile':fields.char('Mobile',size=60,required=True),
        'email':fields.char('EMail',size=200,required=True),
        'street':fields.char('Street',size=200,required=True),
        'zip_code':fields.char('Zip Code',size=12,required=True),
        'city':fields.char('City',size=60,required=True),
        'company_phone':fields.char('Company Phone',size=60),
        'final_partner_id':fields.many2one('res.partner','Final Partner',ondelete='set null'),
        'final_address_id':fields.many2one('res.partner.address','Final Address',ondelete='set null'),
        'final_job_id':fields.many2one('res.partner.job','Final Job',ondelete='set null'),
        'final_contact_id':fields.many2one('res.partner.contact','Final Contact',ondelete='set null'),
        'state':fields.selection([('draft','Draft'),('injected','Injected'),('refused','Refused')],'State'),
        'state_desc':fields.text('Refuse Reason'),
        'active':fields.boolean('Active'),
    }
    _defaults = {
        'state': lambda *a: 'draft',
        'active': lambda *a: True,
    }
    def _valid_company_num(self,company_num):
        '''
        Check the VAT number depending of the country.
        http://sima-pc.com/nif.php
        '''
        vat_country, vat_number = company_num[:2].lower(), company_num[2:].replace(' ', '')
        partner_obj = self.pool.get('res.partner')
        check = getattr(partner_obj, 'check_vat_' + vat_country)
        if not check(vat_number):
            return False
        return True
    def _search_zip_id(self,cr,uid, zip_code, zip_loc):
        result = False
        zip_obj = self.pool.get('res.partner.zip')
        zip_ids = zip_obj.search(cr,uid,[('name','=',zip_code),('city','=',zip_loc)])
        if zip_ids:
            result = zip_ids[0]
        else:
            zip_ids = zip_obj.search(cr,uid,[('name','=',zip_code),('city','ilike',zip_loc)])
            if not zip_ids:
                # search in other names
                zip_ids = zip_obj.search(cr,uid,[('name','=',zip_code),('other_names','ilike',zip_loc)])
            if zip_ids and len(zip_ids) == 1:
                result = zip_ids[0]
            else:
                found = False
                zips = zip_obj.read(cr,uid,zip_ids,['name','city'])
                for zip_record in zips:
                    if len(zip_record['city']) == len(zip_loc):
                        result = zip_record['id']
                        found = True
                        break
        return result
    def create(self, cr, uid, vals, context=None):
        partner_id = 0
        address_id = 0
        job_id = 0
        contact_id = 0
        error = 0
        # TODO CHECK if all mandatory field are there and with values
        if self._valid_company_num(vals['vat']):
            detected_partner_id = vals['detected_partner_id'] or 0
            detected_address_id = vals['detected_address_id'] or 0
            detected_job_id = vals['detected_job_id'] or 0
            detected_contact_id = vals['detected_contact_id'] or 0
            if detected_partner_id > 0 and detected_address_id <= 0:
                # we record data on partner, create addresse and contact + job
                partner_id = detected_partner_id
                address_id = -1
                job_id = -1
                contact_id = -1
                self.pool.get('res.partner').write(cr,uid,[partner_id,],{'name':_upper_accent(vals['partner_name']), 'vat':vals['vat'], 'vat_subjected':vals['vat_subjected']})
                zip_ids = self.pool.get('res.partner').search(cr,uid,[('name','=',vals['zip_code']),('city','=',_upper_accent(vals['city']))])
                if zip_ids:
                    zip_id = zip_ids[0]
                else:
                    zip_id = False
                address_id = self.pool.get('res.partner.address').create(cr,uid,{'street':_upper_accent(vals['street']), 'zip':vals['zip_code'], 'city':_upper_accent(vals['city']), 'zip_id': zip_id, 'partner_id':partner_id, 'type':'other', 'phone':vals['company_phone']})
                if address_id:
                    if _upper_accent(vals['courtesy']) == 'MADAME':
                        gender = 'women'
                    else:
                        gender = 'man'
                    contact_id = self.pool.get('res.partner.contact').create(cr,uid,{'name':_upper_accent(vals['contact_name']),
                                                                                     'first_name':_title_accent(vals['contact_first_name']),
                                                                                     'title':vals['courtesy'],
                                                                                     'gender':gender,
                                                                                     'premium_login':vals['login'],
                                                                                     'premium_mp':vals['motpasse'],
                                                                                     'mobile':vals['mobile'],
                                                                                     'email':_is_mail_personal_domain(vals['email']) and _check_email(vals['email']) or False,
                                                                                    })
                    if contact_id:
                        job_id = self.pool.get('res.partner.job').create(cr,uid,{'function_label':_title_accent(vals['function_label']),
                                                                                 'email': (not _is_mail_personal_domain(vals['email'])) and _check_email(vals['email']) or False,
                                                                                 'contact_id':contact_id,
                                                                                 'address_id':address_id})
                vals['state'] = 'injected'
            elif detected_partner_id > 0 and detected_address_id > 0 and detected_contact_id <= 0:
                # we record data on partner and address, and create contact
                partner_id = detected_partner_id
                address_id = detected_address_id
                job_id = -1
                contact_id = -1
                self.pool.get('res.partner').write(cr,uid,[partner_id,],{'name':_upper_accent(vals['partner_name']), 'vat':vals['vat'], 'vat_subjected':vals['vat_subjected']})
                addr_data = {'street':_upper_accent(vals['street']), 'zip':vals['zip_code'], 'city':_upper_accent(vals['city']), 'partner_id':partner_id}
                zip_id = self._search_zip_id(cr,uid,vals['zip_code'] or '',_upper_accent(vals['city']))

                if zip_id:
                    addr_data['zip_id'] = zip_id
                self.pool.get('res.partner.address').write(cr,uid,[address_id,],addr_data)
                if _upper_accent(vals['courtesy']) == 'MADAME':
                    gender = 'women'
                else:
                    gender = 'man'
                contact_id = self.pool.get('res.partner.contact').create(cr,uid,{'name':_upper_accent(vals['contact_name']),
                                                                                 'first_name':_title_accent(vals['contact_first_name']),
                                                                                 'title':vals['courtesy'],
                                                                                 'gender':gender,
                                                                                 'premium_login':vals['login'],
                                                                                 'premium_mp':vals['motpasse'],
                                                                                 'mobile':vals['mobile'],
                                                                                 'email':_is_mail_personal_domain(vals['email']) and _check_email(vals['email']) or False,
                                                                                })
                if contact_id:
                    job_id = self.pool.get('res.partner.job').create(cr,uid,{'function_label':_title_accent(vals['function_label']),
                                                                             'email': (not _is_mail_personal_domain(vals['email'])) and _check_email(vals['email']) or False,
                                                                             'contact_id':contact_id,
                                                                             'address_id':address_id})
                vals['state'] = 'injected'
            elif detected_partner_id > 0 and detected_address_id > 0 and detected_contact_id > 0 and detected_job_id > 0:
                ## we just record the data
                partner_id = detected_partner_id
                address_id = detected_address_id
                job_id = detected_job_id
                contact_id = detected_contact_id
                self.pool.get('res.partner').write(cr,uid,[partner_id,],{'name':_upper_accent(vals['partner_name']), 'vat':vals['vat'], 'vat_subjected':vals['vat_subjected']})
                addr_data = {'street':_upper_accent(vals['street']), 'zip':vals['zip_code'], 'city':_upper_accent(vals['city']), 'partner_id':partner_id}
                zip_id = self._search_zip_id(cr,uid,vals['zip_code'] or '',_upper_accent(vals['city']))
                if zip_id:
                    addr_data['zip_id'] = zip_id
                self.pool.get('res.partner.address').write(cr,uid,[address_id,],addr_data)
                if _upper_accent(vals['courtesy']) == 'MADAME':
                    gender = 'women'
                else:
                    gender = 'man'
                self.pool.get('res.partner.contact').write(cr,uid,[contact_id,],{'name':_upper_accent(vals['contact_name']),
                                                                                 'first_name':_title_accent(vals['contact_first_name']),
                                                                                 'title':vals['courtesy'],
                                                                                 'gender':gender,
                                                                                 'premium_login':vals['login'],
                                                                                 'premium_mp':vals['motpasse'],
                                                                                 'mobile':vals['mobile'],
                                                                                 'email':_is_mail_personal_domain(vals['email']) and _check_email(vals['email']) or False,
                                                                                })
                self.pool.get('res.partner.job').write(cr,uid,[job_id,],{'function_label':_title_accent(vals['function_label']),
                                                                         'email': (not _is_mail_personal_domain(vals['email'])) and _check_email(vals['email']) or False,
                                                                         'contact_id':contact_id,
                                                                         'address_id':address_id})
                vals['state'] = 'injected'
            else:
                # we create the partner, so we create all data : address, contact, job...
                partner_ids = self.pool.get('res.partner').search(cr,uid,[('vat','=',vals['vat'])])
                if not partner_ids:
                    partner_id = self.pool.get('res.partner').create(cr,uid,{'name':_upper_accent(vals['partner_name']), 'vat':vals['vat'], 'vat_subjected':vals['vat_subjected']})
                    if partner_id > 0:
                        addr_data = {'street':_upper_accent(vals['street']), 'zip':vals['zip_code'], 'city':_upper_accent(vals['city']), 'partner_id':partner_id,'type':'other'}
                        zip_id = self._search_zip_id(cr,uid,vals['zip_code'] or '',_upper_accent(vals['city']))
                        if zip_id:
                            addr_data['zip_id'] = zip_id
                        address_id = self.pool.get('res.partner.address').create(cr,uid,addr_data)
                        if address_id:
                            if _upper_accent(vals['courtesy']) == 'MADAME':
                                gender = 'women'
                            else:
                                gender = 'man'
                            contact_id = self.pool.get('res.partner.contact').create(cr,uid,{'name':_upper_accent(vals['contact_name']),
                                                                                             'first_name':_title_accent(vals['contact_first_name']),
                                                                                             'title':vals['courtesy'],
                                                                                             'gender':gender,
                                                                                             'premium_login':vals['login'],
                                                                                             'premium_mp':vals['motpasse'],
                                                                                             'mobile':vals['mobile'],
                                                                                             'email':_is_mail_personal_domain(vals['email']) and _check_email(vals['email']) or False,
                                                                                            })
                            if contact_id:
                                job_id = self.pool.get('res.partner.job').create(cr,uid,{'function_label':_title_accent(vals['function_label']),
                                                                                         'email': (not _is_mail_personal_domain(vals['email'])) and _check_email(vals['email']) or False,
                                                                                         'contact_id':contact_id,
                                                                                         'address_id':address_id})
                    vals['state'] = 'injected'
                else:
                    vals['state'] = 'refused'
                    vals['state_desc'] = 'Already existing VAT Number'
                    error = -2
        else:
            vals['state'] = 'refused'
            vals['state_desc'] = 'Wrong VAT Number'
            error = -1
        vals['final_partner_id'] = partner_id
        vals['final_address_id'] = address_id
        vals['final_job_id'] = job_id
        vals['final_contact_id'] = contact_id
        result = super(osv.osv,self).create(cr,uid,vals,context)
        if error < 0:
            result = error
        return result
cci_mobile_connection()

class res_partner_job(osv.osv):
    _inherit = 'res.partner.job'
    _columns = {
        'mobile_exclude': fields.boolean('Mobile Directory Exclude'),
    }
    _defaults = {
        'mobile_exclude': lambda *a: False,
    }
    def name_get(self, cr, uid, ids, context={}):
        # the original function used function_id AND the variable name 'func' used in the original is prohibited !
        if not len(ids):
            return []
        res = []
        for r in self.browse(cr, uid, ids):
            function_label = ''
            try:
                function_label = r.function_label and (", " + r.function_label) or ""
                if r.contact_id:
                    function_label = self.pool.get('res.partner.contact').name_get(cr, uid, [r.contact_id.id])[0][1] + function_label
            except:
                pass
            res.append((r.id, function_label ))
        return res
res_partner_job()

