# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from osv import fields, osv
import datetime
import time
import tools
from tools.translate import _
from wordpress_xmlrpc import Client, WordPressPost
from wordpress_xmlrpc.methods.posts import GetPosts, NewPost, EditPost, DeletePost
import pooler

def date_french(date):
    return date[8:10]+'/'+date[5:7]+'/'+date[0:4]

def _get_phone_country_prefix(cr,uid):
    result = []
    obj_country = pooler.get_pool(cr.dbname).get('cci.country')
    country_ids = obj_country.search(cr,uid,[('phoneprefix','!=',False),('phoneprefix','!=',0)])
    if country_ids:
        countries = obj_country.read(cr,uid,country_ids,['phoneprefix'])
        result = [str(x['phoneprefix']) for x in countries]
    return result
def _convert_phone(string,PHONE_COUNTRY_PREFIX):
    def only_digits(string):
        cleaned = ''
        for carac in string:
            if carac in '0123456789':
                cleaned += carac
        return cleaned
    result = ''
    string = only_digits(string)
    if len(string) > 0:
        if len(string) == 9:
            if string[0:2] in ['02','03','04','09']:
                result = string[0:2] + "/" + string[2:5] + "." + string[5:7] + "." + string[7:]
            else:
                result = string[0:3] + "/" + string[3:5] + "." + string[5:7] + "." + string[7:]
        elif len(string) == 10:
            result = string[0:4] + "/" + string[4:6] + "." + string[6:8] + "." + string[8:]
        else:
            # international number
            if string[0:2] == '00':
                # search after a country with this prefix
                prefix = string[2:4]
                if prefix not in PHONE_COUNTRY_PREFIX:
                    prefix = string[2:5]
                    if prefix not in PHONE_COUNTRY_PREFIX:
                        prefix = string[2:6]
                        if prefix not in PHONE_COUNTRY_PREFIX:
                            prefix = ''
                if prefix:
                    result = '+' + string[2:2+len(prefix)] + ' ' + string[2+len(prefix):4+len(prefix)]
                    rest = string[4+len(prefix):]
                    while len(rest) > 3:
                        result += '.' + rest[0:2]
                        rest = rest[2:]
                    result += '.' + rest
                else:
                    result = 'International:'+string
    return result
    
class res_partner(osv.osv):
    _inherit = 'res.partner'
    _description = 'Partner'

    def _members_synchro(self, cr, uid, publish_type):
        cronline_id = self.pool.get('cci_logs.cron_line')._start(cr,uid,'UpdateMembersWebSite')
        final_result = True
        parameter_obj = self.pool.get('cci_parameters.cci_value')
        param_values = parameter_obj.get_value_from_names(cr,uid,['CCIHWebSiteURL','CCIHWebSiteL','CCIHWebSiteMP'])
        if param_values.has_key('CCIHWebSiteURL') and param_values.has_key('CCIHWebSiteL') and param_values.has_key('CCIHWebSiteMP'):
            url = param_values['CCIHWebSiteURL']
            login = param_values['CCIHWebSiteL']
            mp = param_values['CCIHWebSiteMP']
            #
            phone_prefixes = _get_phone_country_prefix(cr,uid)
            #
            obj_categ = self.pool.get('res.partner.category')
            old_len = 0
            categ_ids = [ 50 ]
            while len(categ_ids) > old_len:
                new_ids = categ_ids[old_len:] # ids of categories added last time
                old_len = len(categ_ids) # the new frontier ...
                new_categs = obj_categ.read(cr,uid,new_ids,['child_ids'])
                for categ in new_categs:
                    if categ['child_ids']:
                        categ_ids.extend(categ['child_ids'])
            categs = obj_categ.read(cr,uid,categ_ids,['name'],{'lang':'fr_FR'})
            dSects = {}
            for categ in categs:
                dSects[categ['id']] = categ['name']
            #
            connected = False
            text_result = u''
            key = u''
            try:
                wp = Client(url, login, mp)
                connected = True
            except Exception as e:
                text_result = u'Connection problem with www.ccih.be'
                final_result = False
            if connected:
                index = 0
                size = 50
                dWPPartners = {}
                dWPVATPartners = {}
                try:
                    all_keys = []
                    while index == 0 or posts:
                        posts = wp.call(GetPosts({'post_type':'member','number':size,'offset':index}))
                        for post in posts:
                            partner = {'id':post.id,'name':post.title,'member_category':'','member_city':'','member_contact':'','member_email':'','member_phone':'','member_street':'','member_website':'','member_zipcode':'','member_vat_number':'','initial_post':post}
                            for custom_field in post.custom_fields:
                                partner[custom_field['key']] = custom_field['value']
                            if partner['member_vat_number']:
                                dWPVATPartners[partner['member_vat_number']] = partner
                            else:
                                dWPPartners[post.title] = partner
                            if post.title not in all_keys:
                                all_keys.append(post.title)
                        index += size
                    result = True
                    #
                    dERPPartners = {}
                    dERPVATPartners =  {}
                    partner_obj = self.pool.get('res.partner')
                    partner_ids = partner_obj.search(cr,uid,[('membership_state','in',['free','invoiced','paid'])])
                    if partner_ids:
                        index = 0
                        size = 200
                        while partner_ids[index:index+size]:
                            partners = partner_obj.browse(cr,uid,partner_ids[index:index+size])
                            for partner in partners:
                                # search for default address
                                address = False
                                selected_job = False
                                for addr in partner.address:
                                    if addr.type == 'default':
                                        address = addr
                                if not address and partner.address:
                                    address = partner.address[0]
                                if address:
                                    sequence_max = 9999
                                    for job in address.job_ids:
                                        if job.function_code_label and 'G' in job.function_code_label and job.sequence_partner < sequence_max and job.contact_id:
                                            selected_job = job
                                            sequence_max = job.sequence_partner
                                pname = partner.name.strip()
                                if partner.title:
                                    pname += ( ' '+partner.title )
                                final_street = address.street and address.street or ''
                                if address.street2:
                                    final_street = final_street.strip() + ' ' + address.street2.strip()
                                # search for all categq to get activ sector
                                member_sect_activ = ''
                                sects = []
                                if partner.category_id:
                                    for categ in partner.category_id:
                                        if categ.id in dSects.keys():
                                            sects.append(dSects[categ.id])
                                    if sects:
                                        member_sect_activ = ', '.join(sects)
                                # search for the first G
                                member_contact = ''
                                if address.job_ids:
                                    current_sequence = 9999
                                    selected_job = False
                                    for job in address.job_ids:
                                        if job.function_code_label and ('G' in job.function_code_label) and (job.sequence_partner < current_sequence) and job.contact_id and job.contact_id.active:
                                            selected_job = job
                                            current_sequence = job.sequence_partner
                                    if selected_job:
                                        member_contact = (selected_job.contact_id.title and (selected_job.contact_id.title.strip()+' ') or '') + \
                                                         (selected_job.contact_id.first_name and (selected_job.contact_id.first_name.strip() + ' ') or '') + \
                                                         selected_job.contact_id.name.strip() + \
                                                         (selected_job.function_label and ( ' (' + selected_job.function_label.strip() + ')' ) or '')
                                vat_number = (partner.vat and len(partner.vat) >= 3) and partner.vat or ''
                                partner_data = {'id':partner.id,
                                                'name':pname,
                                                'member_category': member_sect_activ,
                                                'member_city': (address and address.zip_id) and address.zip_id.city.strip() or '',
                                                'member_contact':member_contact,
                                                'member_email': address.email and address.email.strip() or '',
                                                'member_phone': address.phone and _convert_phone(address.phone,phone_prefixes) or '',
                                                'member_street': final_street.strip(),
                                                'member_website': partner.website and partner.website.strip() or '',
                                                'member_zipcode': (address and address.zip_id) and address.zip_id.name.strip() or '',
                                                'member_vat_number': vat_number,}
                                dERPPartners[pname] = partner_data
                                if vat_number:
                                    dERPVATPartners[vat_number] = partner_data
                                if pname not in all_keys:
                                    all_keys.append(pname)
                            index += size
                    # 1. comparing vat numbers
                    for (wp_vat_number,wp_partner_data) in dWPVATPartners.items():
                        if dERPVATPartners.has_key(wp_vat_number):
                            # comparing both partner_data and EdidPost if difference
                            erp_partner_data = dERPVATPartners[wp_vat_number]
                            diff = []
                            new_post = dWPVATPartners[wp_vat_number]['initial_post']
                            new_custom_fields = []
                            for field in ['name','member_category','member_city','member_contact','member_email','member_phone','member_street','member_website','member_zipcode']:
                                if wp_partner_data[field].strip() != erp_partner_data[field]:
                                    diff.append('\t%s diff : %s <> %s\n' % (field, wp_partner_data[field], erp_partner_data[field]))
                                    new_custom_fields.append({'key':field,'value':erp_partner_data[field]})
                                else:
                                    new_custom_fields.append({'key':field,'value':wp_partner_data[field]})
                            if diff:
                                text_result += ( 'Comparing same VAT : %s has differences => modifying the site :\n' % wp_vat_number )
                                text_result += ''.join(diff)
                                if publish_type == 'publish':
                                    new_post.custom_fields = new_custom_fields
                                    print(new_post)
                                    #wp.call(EditPost(new_post.id,new_post))
                            # remove the partner with this VAT Number from dERPPartners
                        else:
                            dWPPartners[wp_partner_data['name']] = wp_partner_data
                            if wp_partner_data['name'] not in all_keys:
                                all_keys.append(wp_partner_data['name'])
                    # 2. comparing left sets
                    all_keys.sort()
                    for key in all_keys:
                        if dWPPartners.has_key(key) and dERPPartners.has_key(key):
                            # comparing data
                            diff = []
                            new_post = dWPPartners[key]['initial_post']
                            new_custom_fields = []
                            for field in ['member_category','member_city','member_contact','member_email','member_phone','member_street','member_website','member_zipcode','member_vat_number']:
                                if dWPPartners[key][field].strip() != dERPPartners[key][field]:
                                    diff.append('\t%s diff : %s <> %s\n' % (field, dWPPartners[key][field], dERPPartners[key][field]))
                                    new_custom_fields.append({'key':field,'value':dERPPartners[key][field]})
                                else:
                                    new_custom_fields.append({'key':field,'value':dWPPartners[key][field]})
                            if diff:
                                text_result += ( '%s has differences => modifying the site :\n' % key )
                                text_result += ''.join(diff)
                                if publish_type == 'publish':
                                    new_post.custom_fields = new_custom_fields
                                    print('edit')
                                    print(new_post)
                                    wp.call(EditPost(new_post.id,new_post))
                        elif dWPPartners.has_key(key) and not dERPPartners.has_key(key):
                            # deleting member
                            text_result += u'Membre à effacer du site : ' + key + u'\n'
                            if publish_type == 'publish':
                                print('deletepost')
                                wp.call(DeletePost(dWPPartners[key]['id']))
                                final_result = False
                        elif (not dWPPartners.has_key(key)) and dERPPartners.has_key(key):
                            if (dERPPartners[key]['member_vat_number'] and dERPPartners[key]['member_vat_number'] not in dWPVATPartners.keys()) or not dERPPartners[key]['member_vat_number']:
                                # adding member on wordpress
                                text_result += u'Membre à ajouter sur le site : ' + key + u'\n'
                                new_post = WordPressPost()
                                new_post.post_type = 'member'
                                new_post.title = key
                                new_post.custom_fields = []
                                for field in ['member_category','member_city','member_contact','member_email','member_phone','member_street','member_website','member_zipcode','member_vat_number']:
                                    new_post.custom_fields.append({'key':field,'value':dERPPartners[key][field]})
                                new_post.post_status = 'publish'
                                new_post.post_author = 1
                                new_post.comment_status = 'closed'
                                new_post.ping_status = 'closed'
                                if publish_type == 'publish':
                                    print('new_post')
                                    print(new_post)
                                    new_post_id = wp.call(NewPost(new_post))
                                    final_result = False
                except Exception as e:
                    final_result = False
                    print('erreur')
                    print(e)
                    text_result += 'ERREUR EN COURS DE ROUTE : ' + e
        self.pool.get('cci_logs.cron_line')._addComment(cr,uid,cronline_id,text_result)
        self.pool.get('cci_logs.cron_line')._stop(cr,uid,cronline_id,final_result,'\n---end---')
        return text_result
res_partner()

