# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
# 28/8/2012 Philmer : balise ns2: rajoutée devant Amount
import wizard
import time
import datetime
import pooler
import base64

from tools.translate import _

form_fyear = """<?xml version="1.0"?>
            <form string="Periodical VAT Declaration">
                <group>
                    <group colspan="4">
                        <separator string="Declare Periodical VAT" colspan="4"/><newline/>
                        <field name="period"/>
                        <field name="tax_code_id" string="Company"/>
                        <newline/>
                        <field name="ask_resitution"/>
                        <field name="ask_payment"/>
                        <field name="client_nihil" string="Is Last Declaration"/><label/>
                        <field name="close_period"/>
                        <field name="official"/>
                    </group>
                    <group colspan="4">
                        <separator string="Comments" colspan="4"/>
                        <field colspan="4" name="comments" nolabel="1"/>
                    </group>
                </group>
            </form>"""

fields_fyear = {
    'period': {'string': 'Period', 'type': 'many2one', 'relation': 'account.period', 'required': True,},
    'ask_resitution': {'type': 'boolean', 'string': 'Ask Restitution',},
    'ask_payment': {'type': 'boolean', 'string': 'Ask Payment',},
    'client_nihil': {'type': 'boolean', 'string': 'Last Declaration of Entreprise', 'help': 'Thick this case only if it concerns only the last statement on the civil or cessation of activity'},
    'tax_code_id': {'string': 'Company', 'type': 'many2one', 'relation': 'account.tax.code', 'required': True, 'domain': [('parent_id','=',False)]},
    'comments': {'string': 'Comments', 'type': 'text', 'size' : 64},
    'close_period': {'type':'boolean', 'string': 'Close Period', 'help':'Close the period after the declaration. Can always be reverted back later if needed...','default':True},
    'official': {'type':'boolean', 'string':'Consider this declaration as official', 'help':'If official, record the date and the final amount','default':True},
}

form = """<?xml version="1.0"?>
<form string="Notification">
     <separator string="XML File has been Created."  colspan="4"/>
     <field name="msg" colspan="4" nolabel="1"/>
     <field name="name"/><newline/>
     <field name="file_save" />
</form>"""

fields = {
    'name': {'string': 'File Name', 'type': 'char', 'size': 32, 'default': 'vat_declaration.xml'},
    'msg': {'string':'File created', 'type':'text', 'size':'100','readonly':True, 'default': 'Save the File with '".xml"' extension.'},
    'file_save':{'string': 'Save File', 'type': 'binary', 'readonly': True,},
}


class wizard_vat_declaration2012(wizard.interface):

    def _create_xml(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)
        obj_tax_code = pool.get('account.tax.code')
        obj_acc_period = pool.get('account.period')
        obj_user = pool.get('res.users')
        mod_obj = pool.get('ir.model.data')

        list_of_tags = ['00','01','02','03','44','45','46','47','48','49','54','55','56','57','59','61','62','63','64','71','72','81','82','83','84','85','86','87','88','91']

        tax_code_id = data['form']['tax_code_id']
        if tax_code_id:
            obj_company = obj_tax_code.browse(cr, uid, tax_code_id, context=context).company_id
        else:
            obj_company = obj_user.browse(cr, uid, uid).company_id

        vat_no = obj_company.partner_id.vat
        if not vat_no:
            raise wizard.except_wizard(_('Data Insufficient'),_('No VAT Number Associated with Main Company!'))
        vat_no = vat_no.replace(' ','').upper()
        vat = vat_no[2:]

        tax_code_ids = obj_tax_code.search(cr, uid, [('parent_id','child_of',[tax_code_id]), ('company_id','=',obj_company.id)], context=context)

        ctx = context.copy()
        ctx['period_id'] = data['form']['period'] #added context here
        tax_info = obj_tax_code.read(cr,uid,tax_code_ids,['code','sum_period'],context=ctx)

        name = email = phone = address = post_code = city = country_code = ''
        name, email, phone, city, post_code, address, country_code = pooler.get_pool(cr.dbname).get('res.company')._get_default_ad2012(obj_company.partner_id.address)

        account_period = obj_acc_period.browse(cr, uid, data['form']['period'], context=context)
        issued_by = vat_no[:2]
        comments = data['form']['comments'] or ''

        send_ref = str(obj_company.partner_id.id) + str(account_period.date_start[5:7]) + str(account_period.date_stop[:4])

        starting_month = account_period.date_start[5:7]
        ending_month = account_period.date_stop[5:7]
        quarter = str(((int(starting_month) - 1) / 3) + 1)

        if not email:
            raise wizard.except_wizard(_('Data Insufficient!'),_('No email address associated with the company.'))
        if not phone:
            raise wizard.except_wizard(_('Data Insufficient!'),_('No phone associated with the company.'))

        file_data = {
            'issued_by': issued_by,
            'vat_no': vat_no,
            'only_vat': vat_no[2:],
            'cmpny_name': obj_company.name,
            'address': address,
            'post_code': post_code,
            'city': city,
            'country_code': country_code,
            'email': email,
            'phone': phone.replace('.','').replace('/','').replace('(','').replace(')','').replace(' ',''),
            'send_ref': send_ref,
            'quarter': quarter,
            'month': starting_month,
            'year': str(account_period.date_stop[:4]),
            'client_nihil': (data['form']['client_nihil'] and 'YES' or 'NO'),
            'ask_restitution': (data['form']['ask_resitution'] and 'YES' or 'NO'),
            'ask_payment': (data['form']['ask_payment'] and 'YES' or 'NO'),
            'comments': comments,
        }

        data_of_file="""<?xml version="1.0"?>
<ns2:VATConsignment xmlns="http://www.minfin.fgov.be/InputCommon" xmlns:ns2="http://www.minfin.fgov.be/VATConsignment" VATDeclarationsNbr="1">
    <ns2:Representative>
        <RepresentativeID identificationType="NVAT" issuedBy="%(issued_by)s">%(only_vat)s</RepresentativeID>
        <Name>%(cmpny_name)s</Name>
        <Street>%(address)s</Street>
        <PostCode>%(post_code)s</PostCode>
        <City>%(city)s</City>
        <CountryCode>%(country_code)s</CountryCode>
        <EmailAddress>%(email)s</EmailAddress>
        <Phone>%(phone)s</Phone>
    </ns2:Representative>
    <ns2:VATDeclaration SequenceNumber="1" DeclarantReference="%(send_ref)s">
        <ns2:Declarant>
            <VATNumber xmlns="http://www.minfin.fgov.be/InputCommon">%(only_vat)s</VATNumber>
            <Name>%(cmpny_name)s</Name>
            <Street>%(address)s</Street>
            <PostCode>%(post_code)s</PostCode>
            <City>%(city)s</City>
            <CountryCode>%(country_code)s</CountryCode>
            <EmailAddress>%(email)s</EmailAddress>
            <Phone>%(phone)s</Phone>
        </ns2:Declarant>
        <ns2:Period>
    """ % (file_data)

        if starting_month != ending_month:
            #starting month and ending month of selected period are not the same 
            #it means that the accounting isn't based on periods of 1 month but on quarters
            data_of_file += '\t\t<ns2:Quarter>%(quarter)s</ns2:Quarter>\n\t\t' % (file_data)
        else:
            data_of_file += '\t\t<ns2:Month>%(month)s</ns2:Month>\n\t\t' % (file_data)
        data_of_file += '\t<ns2:Year>%(year)s</ns2:Year>' % (file_data)
        data_of_file += '\n\t\t</ns2:Period>\n'
        data_of_file += '\t\t<ns2:Data>\t'
        cases_list = []
        for item in tax_info:
            if item['code'] == '91' and ending_month != 12:
                #the tax code 91 can only be send for the declaration of December
                continue
            if item['code'] and item['sum_period']:
                if item['code'] == '71-72':
                    if item['sum_period'] >= 0:
                        item['code'] = '71'
                    else:
                        item['code'] = '72'
                if item['code'] in list_of_tags:
                    cases_list.append(item)
        cases_list.sort()
        for item in cases_list:
            grid_amount_data = {
                    'code': str(int(item['code'])),
                    'amount': str(abs(int(round(item['sum_period']*100)))),
                    }
            grid_amount_data['amount'] = grid_amount_data['amount'][0:len(grid_amount_data['amount'])-2] + '.' + grid_amount_data['amount'][len(grid_amount_data['amount'])-2:]
            data_of_file += '\n\t\t\t<ns2:Amount GridNumber="%(code)s">%(amount)s</ns2:Amount''>' % (grid_amount_data)

        data_of_file += '\n\t\t</ns2:Data>'
        data_of_file += '\n\t\t<ns2:ClientListingNihil>%(client_nihil)s</ns2:ClientListingNihil>' % (file_data)
        data_of_file += '\n\t\t<ns2:Ask Restitution="%(ask_restitution)s" Payment="%(ask_payment)s"/>' % (file_data)
        data_of_file += '\n\t\t<ns2:Comment>%(comments)s</ns2:Comment>' % (file_data)
        data_of_file += '\n\t</ns2:VATDeclaration> \n</ns2:VATConsignment>'
        data['form']['msg']='Save the File with '".xml"' extension.'
        data['form']['file_save'] = base64.encodestring(data_of_file.encode('utf8'))
        
        if data['form']['close_period']:
            mode = 'done'
            cr.execute('update account_journal_period set state=%s where period_id=%s', (mode, data['form']['period']))
            cr.execute('update account_period set state=%s where id=%s', (mode, data['form']['period']))
        if data['form']['official']:
            # check if date is in 25 days after the end of the period, either, it's not more considered as 'official'
            today = datetime.datetime.today()
            if account_period.date_stop >= (today+datetime.timedelta(days=-25)).strftime('%Y-%m-%d'):
                last_value = 0.0
                for item in tax_info:
                    if item['code'] in ['71','71-72']:
                        last_value = round(item['sum_period'],2) * -1
                    elif item['code'] == '72':
                        # *-1 too, because this float is negatiuve in 'item' array
                        last_value = round(item['sum_period'],2) * -1
                obj_acc_period.write(cr,uid,[data['form']['period']],{'vat_last_time':today.strftime('%Y-%m-%d %H:%M:%S'),'vat_last_value':last_value})
        return data['form']


    states = {
        'init': {
            'actions': [],
            'result': {'type':'form', 'arch':form_fyear, 'fields':fields_fyear, 'state':[('end','Cancel'),('go','Create XML')]},
        },
        'go': {
            'actions': [_create_xml],
            'result': {'type':'form', 'arch':form, 'fields':fields, 'state':[('end','Ok')]},
        }
    }

wizard_vat_declaration2012('wizard.account.xml.vat.declaration2012')
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
